/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/

package com.ibm.faces.bf.component.html;


import java.io.IOException;
import javax.faces.context.FacesContext;
import javax.faces.el.MethodBinding;
import javax.faces.el.ValueBinding;


/**
 * <p>By default, the <code>rendererType</code> property must be set to "<code>BfPanel</code>" This value can be changed by calling the <code>setRendererType()</code> method.</p>
*/
public class HtmlBfPanel extends com.ibm.faces.bf.component.UIBfPanel {


  public HtmlBfPanel() {
    super();
    setRendererType("BfPanel"); //$NON-NLS-1$
  }


  /*
   * <p>The standard component type for this component.</p>
   */
   public static final String COMPONENT_TYPE = "com.ibm.faces.bf.HtmlBfPanel"; //$NON-NLS-1$


  private boolean disableCancel = false;
  private boolean disableCancel_set = false;

  /**
   * <p>Return the value of the <code>disableCancel</code> property.  Contents:</p><p>
   * Boolean flag indicating whether the cancel button should
   * 				be disabled when this panel is active.
   * </p>
   */
  public boolean isDisableCancel() {
    if (this.disableCancel_set) {
      return this.disableCancel;
    }
    ValueBinding _vb = getValueBinding("disableCancel"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return false;
      } else {
        return ((Boolean) _result).booleanValue();
      }
    } else {
        return this.disableCancel;
    }
  }

  /**
   * <p>Set the value of the <code>disableCancel</code> property.</p>
   */
  public void setDisableCancel(boolean disableCancel) {
    this.disableCancel = disableCancel;
    this.disableCancel_set = true;
  }


  private boolean disableFinish = false;
  private boolean disableFinish_set = false;

  /**
   * <p>Return the value of the <code>disableFinish</code> property.  Contents:</p><p>
   * Boolean flag indicating whether the finish button should
   * 				be disabled when this panel is active.
   * </p>
   */
  public boolean isDisableFinish() {
    if (this.disableFinish_set) {
      return this.disableFinish;
    }
    ValueBinding _vb = getValueBinding("disableFinish"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return false;
      } else {
        return ((Boolean) _result).booleanValue();
      }
    } else {
        return this.disableFinish;
    }
  }

  /**
   * <p>Set the value of the <code>disableFinish</code> property.</p>
   */
  public void setDisableFinish(boolean disableFinish) {
    this.disableFinish = disableFinish;
    this.disableFinish_set = true;
  }


  private boolean disableTab = false;
  private boolean disableTab_set = false;

  /**
   * <p>Return the value of the <code>disableTab</code> property.  Contents:</p><p>
   * Boolean flag indicating whether the tab for this pane should 
   * 				be disabled initially on the page.
   * </p>
   */
  public boolean isDisableTab() {
    if (this.disableTab_set) {
      return this.disableTab;
    }
    ValueBinding _vb = getValueBinding("disableTab"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return false;
      } else {
        return ((Boolean) _result).booleanValue();
      }
    } else {
        return this.disableTab;
    }
  }

  /**
   * <p>Set the value of the <code>disableTab</code> property.</p>
   */
  public void setDisableTab(boolean disableTab) {
    this.disableTab = disableTab;
    this.disableTab_set = true;
  }


  private java.lang.String name;

  /**
   * <p>Return the value of the <code>name</code> property.  Contents:</p><p>
   * The title for this panel.
   * </p>
   */
  public java.lang.String getName() {
    if (null != this.name) {
      return this.name;
    }
    ValueBinding _vb = getValueBinding("name"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>name</code> property.</p>
   */
  public void setName(java.lang.String name) {
    this.name = name;
  }


  private boolean showFinishCancelButton = false;
  private boolean showFinishCancelButton_set = false;

  /**
   * <p>Return the value of the <code>showFinishCancelButton</code> property.  Contents:</p><p>
   * Boolean flag indicating whether the finish and cancel
   * 				button should be shown when this panel is active.
   * </p>
   */
  public boolean isShowFinishCancelButton() {
    if (this.showFinishCancelButton_set) {
      return this.showFinishCancelButton;
    }
    ValueBinding _vb = getValueBinding("showFinishCancelButton"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return false;
      } else {
        return ((Boolean) _result).booleanValue();
      }
    } else {
        return this.showFinishCancelButton;
    }
  }

  /**
   * <p>Set the value of the <code>showFinishCancelButton</code> property.</p>
   */
  public void setShowFinishCancelButton(boolean showFinishCancelButton) {
    this.showFinishCancelButton = showFinishCancelButton;
    this.showFinishCancelButton_set = true;
  }


  public Object saveState(FacesContext _context) {
    Object _values[] = new Object[10];
    _values[0] = super.saveState(_context);
    _values[1] = this.disableCancel ? Boolean.TRUE : Boolean.FALSE;
    _values[2] = this.disableCancel_set ? Boolean.TRUE : Boolean.FALSE;
    _values[3] = this.disableFinish ? Boolean.TRUE : Boolean.FALSE;
    _values[4] = this.disableFinish_set ? Boolean.TRUE : Boolean.FALSE;
    _values[5] = this.disableTab ? Boolean.TRUE : Boolean.FALSE;
    _values[6] = this.disableTab_set ? Boolean.TRUE : Boolean.FALSE;
    _values[7] = name;
    _values[8] = this.showFinishCancelButton ? Boolean.TRUE : Boolean.FALSE;
    _values[9] = this.showFinishCancelButton_set ? Boolean.TRUE : Boolean.FALSE;
    return _values;
  }


  public void restoreState(FacesContext _context, Object _state) {
    Object _values[] = (Object[]) _state;
    super.restoreState(_context, _values[0]);
    this.disableCancel = ((Boolean) _values[1]).booleanValue();
    this.disableCancel_set = ((Boolean) _values[2]).booleanValue();
    this.disableFinish = ((Boolean) _values[3]).booleanValue();
    this.disableFinish_set = ((Boolean) _values[4]).booleanValue();
    this.disableTab = ((Boolean) _values[5]).booleanValue();
    this.disableTab_set = ((Boolean) _values[6]).booleanValue();
    this.name = (java.lang.String) _values[7];
    this.showFinishCancelButton = ((Boolean) _values[8]).booleanValue();
    this.showFinishCancelButton_set = ((Boolean) _values[9]).booleanValue();
  }


}
