/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/

package com.ibm.faces.bf.component.html;


import java.io.IOException;
import javax.faces.context.FacesContext;
import javax.faces.el.MethodBinding;
import javax.faces.el.ValueBinding;


/**
 * <p>By default, the <code>rendererType</code> property must be set to "<code>DataGrid</code>" This value can be changed by calling the <code>setRendererType()</code> method.</p>
*/
public class HtmlDataGrid extends com.ibm.faces.bf.component.UIDataGrid {


  public HtmlDataGrid() {
    super();
    setRendererType("DataGrid"); //$NON-NLS-1$
  }


  /*
   * <p>The standard component type for this component.</p>
   */
   public static final String COMPONENT_TYPE = "com.ibm.faces.bf.HtmlDataGrid"; //$NON-NLS-1$


  private boolean allowColumnHide = false;
  private boolean allowColumnHide_set = false;

  /**
   * <p>Return the value of the <code>allowColumnHide</code> property.  Contents:</p><p>
   * Show the row of icons to hide a column
   * </p>
   */
  public boolean isAllowColumnHide() {
    if (this.allowColumnHide_set) {
      return this.allowColumnHide;
    }
    ValueBinding _vb = getValueBinding("allowColumnHide"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return false;
      } else {
        return ((Boolean) _result).booleanValue();
      }
    } else {
        return this.allowColumnHide;
    }
  }

  /**
   * <p>Set the value of the <code>allowColumnHide</code> property.</p>
   */
  public void setAllowColumnHide(boolean allowColumnHide) {
    this.allowColumnHide = allowColumnHide;
    this.allowColumnHide_set = true;
  }


  private boolean allowRowAddAndDelete = false;
  private boolean allowRowAddAndDelete_set = false;

  /**
   * <p>Return the value of the <code>allowRowAddAndDelete</code> property.  Contents:</p><p>
   * Are the fields readOnly or not
   * </p>
   */
  public boolean isAllowRowAddAndDelete() {
    if (this.allowRowAddAndDelete_set) {
      return this.allowRowAddAndDelete;
    }
    ValueBinding _vb = getValueBinding("allowRowAddAndDelete"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return false;
      } else {
        return ((Boolean) _result).booleanValue();
      }
    } else {
        return this.allowRowAddAndDelete;
    }
  }

  /**
   * <p>Set the value of the <code>allowRowAddAndDelete</code> property.</p>
   */
  public void setAllowRowAddAndDelete(boolean allowRowAddAndDelete) {
    this.allowRowAddAndDelete = allowRowAddAndDelete;
    this.allowRowAddAndDelete_set = true;
  }


  private boolean multiSelection = false;
  private boolean multiSelection_set = false;

  /**
   * <p>Return the value of the <code>multiSelection</code> property.  Contents:</p><p>
   * Multiple selection allowed or not. True-checkbox,
   * 				False-radio button.
   * </p>
   */
  public boolean isMultiSelection() {
    if (this.multiSelection_set) {
      return this.multiSelection;
    }
    ValueBinding _vb = getValueBinding("multiSelection"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return false;
      } else {
        return ((Boolean) _result).booleanValue();
      }
    } else {
        return this.multiSelection;
    }
  }

  /**
   * <p>Set the value of the <code>multiSelection</code> property.</p>
   */
  public void setMultiSelection(boolean multiSelection) {
    this.multiSelection = multiSelection;
    this.multiSelection_set = true;
  }


  private int navBarPosition = Integer.MIN_VALUE;
  private boolean navBarPosition_set = false;

  /**
   * <p>Return the value of the <code>navBarPosition</code> property.  Contents:</p><p>
   * Navigation bar position in datagrid-0-bottom,1-top, or
   * 				2-both
   * </p>
   */
  public int getNavBarPosition() {
    if (this.navBarPosition_set) {
      return this.navBarPosition;
    }
    ValueBinding _vb = getValueBinding("navBarPosition"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return Integer.MIN_VALUE;
      } else {
        return ((Integer) _result).intValue();
      }
    } else {
        return this.navBarPosition;
    }
  }

  /**
   * <p>Set the value of the <code>navBarPosition</code> property.</p>
   */
  public void setNavBarPosition(int navBarPosition) {
    this.navBarPosition = navBarPosition;
    this.navBarPosition_set = true;
  }


  private int pageSize = Integer.MIN_VALUE;
  private boolean pageSize_set = false;

  /**
   * <p>Return the value of the <code>pageSize</code> property.  Contents:</p><p>
   * Size of the page to display; Records greater pageSize
   * 				will be split into multiple pages
   * </p>
   */
  public int getPageSize() {
    if (this.pageSize_set) {
      return this.pageSize;
    }
    ValueBinding _vb = getValueBinding("pageSize"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return Integer.MIN_VALUE;
      } else {
        return ((Integer) _result).intValue();
      }
    } else {
        return this.pageSize;
    }
  }

  /**
   * <p>Set the value of the <code>pageSize</code> property.</p>
   */
  public void setPageSize(int pageSize) {
    this.pageSize = pageSize;
    this.pageSize_set = true;
  }


  private java.lang.String rowFilter;

  /**
   * <p>Return the value of the <code>rowFilter</code> property.  Contents:</p><p>
   * Expression to filter out some rows
   * </p>
   */
  public java.lang.String getRowFilter() {
    if (null != this.rowFilter) {
      return this.rowFilter;
    }
    ValueBinding _vb = getValueBinding("rowFilter"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>rowFilter</code> property.</p>
   */
  public void setRowFilter(java.lang.String rowFilter) {
    this.rowFilter = rowFilter;
  }


  private java.lang.String scrollSize;

  /**
   * <p>Return the value of the <code>scrollSize</code> property.  Contents:</p><p>
   * Number of pixels that are on display within the
   * 				scrolling section of the datagrid
   * </p>
   */
  public java.lang.String getScrollSize() {
    if (null != this.scrollSize) {
      return this.scrollSize;
    }
    ValueBinding _vb = getValueBinding("scrollSize"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>scrollSize</code> property.</p>
   */
  public void setScrollSize(java.lang.String scrollSize) {
    this.scrollSize = scrollSize;
  }


  private java.lang.String selectionColAlignment;

  /**
   * <p>Return the value of the <code>selectionColAlignment</code> property.  Contents:</p><p>
   * Alignment of selection column
   * </p>
   */
  public java.lang.String getSelectionColAlignment() {
    if (null != this.selectionColAlignment) {
      return this.selectionColAlignment;
    }
    ValueBinding _vb = getValueBinding("selectionColAlignment"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>selectionColAlignment</code> property.</p>
   */
  public void setSelectionColAlignment(java.lang.String selectionColAlignment) {
    this.selectionColAlignment = selectionColAlignment;
  }


  private java.lang.String selectionColName;

  /**
   * <p>Return the value of the <code>selectionColName</code> property.  Contents:</p><p>
   * Key for the name of the column for checkbox or radio
   * 				button
   * </p>
   */
  public java.lang.String getSelectionColName() {
    if (null != this.selectionColName) {
      return this.selectionColName;
    }
    ValueBinding _vb = getValueBinding("selectionColName"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>selectionColName</code> property.</p>
   */
  public void setSelectionColName(java.lang.String selectionColName) {
    this.selectionColName = selectionColName;
  }


  private java.lang.String selectionColWidth;

  /**
   * <p>Return the value of the <code>selectionColWidth</code> property.  Contents:</p><p>
   * Width of selection column
   * </p>
   */
  public java.lang.String getSelectionColWidth() {
    if (null != this.selectionColWidth) {
      return this.selectionColWidth;
    }
    ValueBinding _vb = getValueBinding("selectionColWidth"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>selectionColWidth</code> property.</p>
   */
  public void setSelectionColWidth(java.lang.String selectionColWidth) {
    this.selectionColWidth = selectionColWidth;
  }


  private boolean showRowIndex = false;
  private boolean showRowIndex_set = false;

  /**
   * <p>Return the value of the <code>showRowIndex</code> property.  Contents:</p><p>
   * Show row index or not; true-show, false-don't show
   * </p>
   */
  public boolean isShowRowIndex() {
    if (this.showRowIndex_set) {
      return this.showRowIndex;
    }
    ValueBinding _vb = getValueBinding("showRowIndex"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return false;
      } else {
        return ((Boolean) _result).booleanValue();
      }
    } else {
        return this.showRowIndex;
    }
  }

  /**
   * <p>Set the value of the <code>showRowIndex</code> property.</p>
   */
  public void setShowRowIndex(boolean showRowIndex) {
    this.showRowIndex = showRowIndex;
    this.showRowIndex_set = true;
  }


  private boolean showSelectionColumn = false;
  private boolean showSelectionColumn_set = false;

  /**
   * <p>Return the value of the <code>showSelectionColumn</code> property.  Contents:</p><p>
   * Flag to indicate whether selection column should be
   * 				shown.
   * </p>
   */
  public boolean isShowSelectionColumn() {
    if (this.showSelectionColumn_set) {
      return this.showSelectionColumn;
    }
    ValueBinding _vb = getValueBinding("showSelectionColumn"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return false;
      } else {
        return ((Boolean) _result).booleanValue();
      }
    } else {
        return this.showSelectionColumn;
    }
  }

  /**
   * <p>Set the value of the <code>showSelectionColumn</code> property.</p>
   */
  public void setShowSelectionColumn(boolean showSelectionColumn) {
    this.showSelectionColumn = showSelectionColumn;
    this.showSelectionColumn_set = true;
  }


  private java.lang.String styleClass;

  /**
   * <p>Return the value of the <code>styleClass</code> property.  Contents:</p><p>
   * Prefix specifying CSS for multiple data grids on page
   * </p>
   */
  public java.lang.String getStyleClass() {
    if (null != this.styleClass) {
      return this.styleClass;
    }
    ValueBinding _vb = getValueBinding("styleClass"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>styleClass</code> property.</p>
   */
  public void setStyleClass(java.lang.String styleClass) {
    this.styleClass = styleClass;
  }


  private java.lang.String width;

  /**
   * <p>Return the value of the <code>width</code> property.  Contents:</p><p>
   * Datagrid width--in pixels or percentage
   * </p>
   */
  public java.lang.String getWidth() {
    if (null != this.width) {
      return this.width;
    }
    ValueBinding _vb = getValueBinding("width"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>width</code> property.</p>
   */
  public void setWidth(java.lang.String width) {
    this.width = width;
  }


  public Object saveState(FacesContext _context) {
    Object _values[] = new Object[22];
    _values[0] = super.saveState(_context);
    _values[1] = this.allowColumnHide ? Boolean.TRUE : Boolean.FALSE;
    _values[2] = this.allowColumnHide_set ? Boolean.TRUE : Boolean.FALSE;
    _values[3] = this.allowRowAddAndDelete ? Boolean.TRUE : Boolean.FALSE;
    _values[4] = this.allowRowAddAndDelete_set ? Boolean.TRUE : Boolean.FALSE;
    _values[5] = this.multiSelection ? Boolean.TRUE : Boolean.FALSE;
    _values[6] = this.multiSelection_set ? Boolean.TRUE : Boolean.FALSE;
    _values[7] = new Integer(this.navBarPosition);
    _values[8] = this.navBarPosition_set ? Boolean.TRUE : Boolean.FALSE;
    _values[9] = new Integer(this.pageSize);
    _values[10] = this.pageSize_set ? Boolean.TRUE : Boolean.FALSE;
    _values[11] = rowFilter;
    _values[12] = scrollSize;
    _values[13] = selectionColAlignment;
    _values[14] = selectionColName;
    _values[15] = selectionColWidth;
    _values[16] = this.showRowIndex ? Boolean.TRUE : Boolean.FALSE;
    _values[17] = this.showRowIndex_set ? Boolean.TRUE : Boolean.FALSE;
    _values[18] = this.showSelectionColumn ? Boolean.TRUE : Boolean.FALSE;
    _values[19] = this.showSelectionColumn_set ? Boolean.TRUE : Boolean.FALSE;
    _values[20] = styleClass;
    _values[21] = width;
    return _values;
  }


  public void restoreState(FacesContext _context, Object _state) {
    Object _values[] = (Object[]) _state;
    super.restoreState(_context, _values[0]);
    this.allowColumnHide = ((Boolean) _values[1]).booleanValue();
    this.allowColumnHide_set = ((Boolean) _values[2]).booleanValue();
    this.allowRowAddAndDelete = ((Boolean) _values[3]).booleanValue();
    this.allowRowAddAndDelete_set = ((Boolean) _values[4]).booleanValue();
    this.multiSelection = ((Boolean) _values[5]).booleanValue();
    this.multiSelection_set = ((Boolean) _values[6]).booleanValue();
    this.navBarPosition = ((Integer) _values[7]).intValue();
    this.navBarPosition_set = ((Boolean) _values[8]).booleanValue();
    this.pageSize = ((Integer) _values[9]).intValue();
    this.pageSize_set = ((Boolean) _values[10]).booleanValue();
    this.rowFilter = (java.lang.String) _values[11];
    this.scrollSize = (java.lang.String) _values[12];
    this.selectionColAlignment = (java.lang.String) _values[13];
    this.selectionColName = (java.lang.String) _values[14];
    this.selectionColWidth = (java.lang.String) _values[15];
    this.showRowIndex = ((Boolean) _values[16]).booleanValue();
    this.showRowIndex_set = ((Boolean) _values[17]).booleanValue();
    this.showSelectionColumn = ((Boolean) _values[18]).booleanValue();
    this.showSelectionColumn_set = ((Boolean) _values[19]).booleanValue();
    this.styleClass = (java.lang.String) _values[20];
    this.width = (java.lang.String) _values[21];
  }


}
