/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/

package com.ibm.faces.bf.component.html;


import java.io.IOException;
import javax.faces.context.FacesContext;
import javax.faces.el.MethodBinding;
import javax.faces.el.ValueBinding;


/**
 * <p>By default, the <code>rendererType</code> property must be set to "<code>GraphDraw</code>" This value can be changed by calling the <code>setRendererType()</code> method.</p>
*/
public class HtmlGraphDraw extends com.ibm.faces.bf.component.UIGraphDraw {


  public HtmlGraphDraw() {
    super();
    setRendererType("GraphDraw"); //$NON-NLS-1$
  }


  /*
   * <p>The standard component type for this component.</p>
   */
   public static final String COMPONENT_TYPE = "com.ibm.faces.bf.HtmlGraphDraw"; //$NON-NLS-1$


  private boolean bar = false;
  private boolean bar_set = false;

  /**
   * <p>Return the value of the <code>bar</code> property.  Contents:</p><p>
   * Boolean flag controlling whether the bar chart should be
   * 				shown or not.
   * </p>
   */
  public boolean isBar() {
    if (this.bar_set) {
      return this.bar;
    }
    ValueBinding _vb = getValueBinding("bar"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return false;
      } else {
        return ((Boolean) _result).booleanValue();
      }
    } else {
        return this.bar;
    }
  }

  /**
   * <p>Set the value of the <code>bar</code> property.</p>
   */
  public void setBar(boolean bar) {
    this.bar = bar;
    this.bar_set = true;
  }


  private java.lang.String firstChart;

  /**
   * <p>Return the value of the <code>firstChart</code> property.  Contents:</p><p>
   * The first chart to be shown if more than one chart is
   * 				chosen. Valid values are pie, bar and line.
   * </p>
   */
  public java.lang.String getFirstChart() {
    if (null != this.firstChart) {
      return this.firstChart;
    }
    ValueBinding _vb = getValueBinding("firstChart"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>firstChart</code> property.</p>
   */
  public void setFirstChart(java.lang.String firstChart) {
    this.firstChart = firstChart;
  }


  private java.lang.String height;

  /**
   * <p>Return the value of the <code>height</code> property.  Contents:</p><p>
   * The height of the Flash control
   * </p>
   */
  public java.lang.String getHeight() {
    if (null != this.height) {
      return this.height;
    }
    ValueBinding _vb = getValueBinding("height"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>height</code> property.</p>
   */
  public void setHeight(java.lang.String height) {
    this.height = height;
  }


  private boolean line = false;
  private boolean line_set = false;

  /**
   * <p>Return the value of the <code>line</code> property.  Contents:</p><p>
   * Boolean flag controlling whether the line chart should
   * 				be shown or not.
   * </p>
   */
  public boolean isLine() {
    if (this.line_set) {
      return this.line;
    }
    ValueBinding _vb = getValueBinding("line"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return false;
      } else {
        return ((Boolean) _result).booleanValue();
      }
    } else {
        return this.line;
    }
  }

  /**
   * <p>Set the value of the <code>line</code> property.</p>
   */
  public void setLine(boolean line) {
    this.line = line;
    this.line_set = true;
  }


  private boolean pie = false;
  private boolean pie_set = false;

  /**
   * <p>Return the value of the <code>pie</code> property.  Contents:</p><p>
   * Boolean flag controlling whether the pie chart should be
   * 				shown or not.
   * </p>
   */
  public boolean isPie() {
    if (this.pie_set) {
      return this.pie;
    }
    ValueBinding _vb = getValueBinding("pie"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return false;
      } else {
        return ((Boolean) _result).booleanValue();
      }
    } else {
        return this.pie;
    }
  }

  /**
   * <p>Set the value of the <code>pie</code> property.</p>
   */
  public void setPie(boolean pie) {
    this.pie = pie;
    this.pie_set = true;
  }


  private boolean showHGridlinesBar = false;
  private boolean showHGridlinesBar_set = false;

  /**
   * <p>Return the value of the <code>showHGridlinesBar</code> property.  Contents:</p><p>
   * Boolean flag controlling whether horizontal gridlines
   * 				should be plotted for a bar chart.
   * </p>
   */
  public boolean isShowHGridlinesBar() {
    if (this.showHGridlinesBar_set) {
      return this.showHGridlinesBar;
    }
    ValueBinding _vb = getValueBinding("showHGridlinesBar"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return false;
      } else {
        return ((Boolean) _result).booleanValue();
      }
    } else {
        return this.showHGridlinesBar;
    }
  }

  /**
   * <p>Set the value of the <code>showHGridlinesBar</code> property.</p>
   */
  public void setShowHGridlinesBar(boolean showHGridlinesBar) {
    this.showHGridlinesBar = showHGridlinesBar;
    this.showHGridlinesBar_set = true;
  }


  private boolean showHGridlinesLine = false;
  private boolean showHGridlinesLine_set = false;

  /**
   * <p>Return the value of the <code>showHGridlinesLine</code> property.  Contents:</p><p>
   * Boolean flag controlling whether horizontal gridlines
   * 				should be plotted for a line chart.
   * </p>
   */
  public boolean isShowHGridlinesLine() {
    if (this.showHGridlinesLine_set) {
      return this.showHGridlinesLine;
    }
    ValueBinding _vb = getValueBinding("showHGridlinesLine"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return false;
      } else {
        return ((Boolean) _result).booleanValue();
      }
    } else {
        return this.showHGridlinesLine;
    }
  }

  /**
   * <p>Set the value of the <code>showHGridlinesLine</code> property.</p>
   */
  public void setShowHGridlinesLine(boolean showHGridlinesLine) {
    this.showHGridlinesLine = showHGridlinesLine;
    this.showHGridlinesLine_set = true;
  }


  private boolean showLabel = false;
  private boolean showLabel_set = false;

  /**
   * <p>Return the value of the <code>showLabel</code> property.  Contents:</p><p>
   * Boolean flag controlling whether the labels are
   * 				displayed in the graph.
   * </p>
   */
  public boolean isShowLabel() {
    if (this.showLabel_set) {
      return this.showLabel;
    }
    ValueBinding _vb = getValueBinding("showLabel"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return false;
      } else {
        return ((Boolean) _result).booleanValue();
      }
    } else {
        return this.showLabel;
    }
  }

  /**
   * <p>Set the value of the <code>showLabel</code> property.</p>
   */
  public void setShowLabel(boolean showLabel) {
    this.showLabel = showLabel;
    this.showLabel_set = true;
  }


  private boolean showLegend = false;
  private boolean showLegend_set = false;

  /**
   * <p>Return the value of the <code>showLegend</code> property.  Contents:</p><p>
   * Boolean flag controlling whether the legend is displayed
   * 				in the graph.
   * </p>
   */
  public boolean isShowLegend() {
    if (this.showLegend_set) {
      return this.showLegend;
    }
    ValueBinding _vb = getValueBinding("showLegend"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return false;
      } else {
        return ((Boolean) _result).booleanValue();
      }
    } else {
        return this.showLegend;
    }
  }

  /**
   * <p>Set the value of the <code>showLegend</code> property.</p>
   */
  public void setShowLegend(boolean showLegend) {
    this.showLegend = showLegend;
    this.showLegend_set = true;
  }


  private boolean showVGridlinesLine = false;
  private boolean showVGridlinesLine_set = false;

  /**
   * <p>Return the value of the <code>showVGridlinesLine</code> property.  Contents:</p><p>
   * Boolean flag controlling whether vertical gridlines
   * 				should be plotted for a line chart.
   * </p>
   */
  public boolean isShowVGridlinesLine() {
    if (this.showVGridlinesLine_set) {
      return this.showVGridlinesLine;
    }
    ValueBinding _vb = getValueBinding("showVGridlinesLine"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return false;
      } else {
        return ((Boolean) _result).booleanValue();
      }
    } else {
        return this.showVGridlinesLine;
    }
  }

  /**
   * <p>Set the value of the <code>showVGridlinesLine</code> property.</p>
   */
  public void setShowVGridlinesLine(boolean showVGridlinesLine) {
    this.showVGridlinesLine = showVGridlinesLine;
    this.showVGridlinesLine_set = true;
  }


  private boolean splitYAxis = false;
  private boolean splitYAxis_set = false;

  /**
   * <p>Return the value of the <code>splitYAxis</code> property.  Contents:</p><p>
   * Boolean flag controlling whether two Y-axes should be
   * 				plotted in a graph with exactly two series.
   * </p>
   */
  public boolean isSplitYAxis() {
    if (this.splitYAxis_set) {
      return this.splitYAxis;
    }
    ValueBinding _vb = getValueBinding("splitYAxis"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return false;
      } else {
        return ((Boolean) _result).booleanValue();
      }
    } else {
        return this.splitYAxis;
    }
  }

  /**
   * <p>Set the value of the <code>splitYAxis</code> property.</p>
   */
  public void setSplitYAxis(boolean splitYAxis) {
    this.splitYAxis = splitYAxis;
    this.splitYAxis_set = true;
  }


  private java.lang.String styleClass;

  /**
   * <p>Return the value of the <code>styleClass</code> property.  Contents:</p><p>
   * Prefix specifying CSS for multiple graphs on page
   * </p>
   */
  public java.lang.String getStyleClass() {
    if (null != this.styleClass) {
      return this.styleClass;
    }
    ValueBinding _vb = getValueBinding("styleClass"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>styleClass</code> property.</p>
   */
  public void setStyleClass(java.lang.String styleClass) {
    this.styleClass = styleClass;
  }


  private java.lang.String title;

  /**
   * <p>Return the value of the <code>title</code> property.  Contents:</p><p>
   * The title to be shown for the rendered graph.
   * </p>
   */
  public java.lang.String getTitle() {
    if (null != this.title) {
      return this.title;
    }
    ValueBinding _vb = getValueBinding("title"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>title</code> property.</p>
   */
  public void setTitle(java.lang.String title) {
    this.title = title;
  }


  private java.lang.String width;

  /**
   * <p>Return the value of the <code>width</code> property.  Contents:</p><p>
   * The width of the Flash control
   * </p>
   */
  public java.lang.String getWidth() {
    if (null != this.width) {
      return this.width;
    }
    ValueBinding _vb = getValueBinding("width"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>width</code> property.</p>
   */
  public void setWidth(java.lang.String width) {
    this.width = width;
  }


  private java.lang.String xaxisTitle;

  /**
   * <p>Return the value of the <code>xaxisTitle</code> property.  Contents:</p><p>
   * The title to be shown for the category (X axis title)
   * </p>
   */
  public java.lang.String getXaxisTitle() {
    if (null != this.xaxisTitle) {
      return this.xaxisTitle;
    }
    ValueBinding _vb = getValueBinding("xaxisTitle"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>xaxisTitle</code> property.</p>
   */
  public void setXaxisTitle(java.lang.String xaxisTitle) {
    this.xaxisTitle = xaxisTitle;
  }


  private java.lang.String yaxisDivisions;

  /**
   * <p>Return the value of the <code>yaxisDivisions</code> property.  Contents:</p><p>
   * The number of y axis divisions to be shown on the graph.
   * </p>
   */
  public java.lang.String getYaxisDivisions() {
    if (null != this.yaxisDivisions) {
      return this.yaxisDivisions;
    }
    ValueBinding _vb = getValueBinding("yaxisDivisions"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>yaxisDivisions</code> property.</p>
   */
  public void setYaxisDivisions(java.lang.String yaxisDivisions) {
    this.yaxisDivisions = yaxisDivisions;
  }


  private java.lang.String yaxisTitle;

  /**
   * <p>Return the value of the <code>yaxisTitle</code> property.  Contents:</p><p>
   * The title to be shown for the value (Y axis title)
   * </p>
   */
  public java.lang.String getYaxisTitle() {
    if (null != this.yaxisTitle) {
      return this.yaxisTitle;
    }
    ValueBinding _vb = getValueBinding("yaxisTitle"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>yaxisTitle</code> property.</p>
   */
  public void setYaxisTitle(java.lang.String yaxisTitle) {
    this.yaxisTitle = yaxisTitle;
  }


  public Object saveState(FacesContext _context) {
    Object _values[] = new Object[27];
    _values[0] = super.saveState(_context);
    _values[1] = this.bar ? Boolean.TRUE : Boolean.FALSE;
    _values[2] = this.bar_set ? Boolean.TRUE : Boolean.FALSE;
    _values[3] = firstChart;
    _values[4] = height;
    _values[5] = this.line ? Boolean.TRUE : Boolean.FALSE;
    _values[6] = this.line_set ? Boolean.TRUE : Boolean.FALSE;
    _values[7] = this.pie ? Boolean.TRUE : Boolean.FALSE;
    _values[8] = this.pie_set ? Boolean.TRUE : Boolean.FALSE;
    _values[9] = this.showHGridlinesBar ? Boolean.TRUE : Boolean.FALSE;
    _values[10] = this.showHGridlinesBar_set ? Boolean.TRUE : Boolean.FALSE;
    _values[11] = this.showHGridlinesLine ? Boolean.TRUE : Boolean.FALSE;
    _values[12] = this.showHGridlinesLine_set ? Boolean.TRUE : Boolean.FALSE;
    _values[13] = this.showLabel ? Boolean.TRUE : Boolean.FALSE;
    _values[14] = this.showLabel_set ? Boolean.TRUE : Boolean.FALSE;
    _values[15] = this.showLegend ? Boolean.TRUE : Boolean.FALSE;
    _values[16] = this.showLegend_set ? Boolean.TRUE : Boolean.FALSE;
    _values[17] = this.showVGridlinesLine ? Boolean.TRUE : Boolean.FALSE;
    _values[18] = this.showVGridlinesLine_set ? Boolean.TRUE : Boolean.FALSE;
    _values[19] = this.splitYAxis ? Boolean.TRUE : Boolean.FALSE;
    _values[20] = this.splitYAxis_set ? Boolean.TRUE : Boolean.FALSE;
    _values[21] = styleClass;
    _values[22] = title;
    _values[23] = width;
    _values[24] = xaxisTitle;
    _values[25] = yaxisDivisions;
    _values[26] = yaxisTitle;
    return _values;
  }


  public void restoreState(FacesContext _context, Object _state) {
    Object _values[] = (Object[]) _state;
    super.restoreState(_context, _values[0]);
    this.bar = ((Boolean) _values[1]).booleanValue();
    this.bar_set = ((Boolean) _values[2]).booleanValue();
    this.firstChart = (java.lang.String) _values[3];
    this.height = (java.lang.String) _values[4];
    this.line = ((Boolean) _values[5]).booleanValue();
    this.line_set = ((Boolean) _values[6]).booleanValue();
    this.pie = ((Boolean) _values[7]).booleanValue();
    this.pie_set = ((Boolean) _values[8]).booleanValue();
    this.showHGridlinesBar = ((Boolean) _values[9]).booleanValue();
    this.showHGridlinesBar_set = ((Boolean) _values[10]).booleanValue();
    this.showHGridlinesLine = ((Boolean) _values[11]).booleanValue();
    this.showHGridlinesLine_set = ((Boolean) _values[12]).booleanValue();
    this.showLabel = ((Boolean) _values[13]).booleanValue();
    this.showLabel_set = ((Boolean) _values[14]).booleanValue();
    this.showLegend = ((Boolean) _values[15]).booleanValue();
    this.showLegend_set = ((Boolean) _values[16]).booleanValue();
    this.showVGridlinesLine = ((Boolean) _values[17]).booleanValue();
    this.showVGridlinesLine_set = ((Boolean) _values[18]).booleanValue();
    this.splitYAxis = ((Boolean) _values[19]).booleanValue();
    this.splitYAxis_set = ((Boolean) _values[20]).booleanValue();
    this.styleClass = (java.lang.String) _values[21];
    this.title = (java.lang.String) _values[22];
    this.width = (java.lang.String) _values[23];
    this.xaxisTitle = (java.lang.String) _values[24];
    this.yaxisDivisions = (java.lang.String) _values[25];
    this.yaxisTitle = (java.lang.String) _values[26];
  }


}
