/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/

package com.ibm.faces.bf.component.html;


import java.io.IOException;
import javax.faces.context.FacesContext;
import javax.faces.el.MethodBinding;
import javax.faces.el.ValueBinding;


/**
 * <p>By default, the <code>rendererType</code> property must be set to "<code>GraphDrawData</code>" This value can be changed by calling the <code>setRendererType()</code> method.</p>
*/
public class HtmlGraphDrawData extends com.ibm.faces.bf.component.UIGraphDrawData {


  public HtmlGraphDrawData() {
    super();
    setRendererType("GraphDrawData"); //$NON-NLS-1$
  }


  /*
   * <p>The standard component type for this component.</p>
   */
   public static final String COMPONENT_TYPE = "com.ibm.faces.bf.HtmlGraphDrawData"; //$NON-NLS-1$


  private boolean grouped = false;
  private boolean grouped_set = false;

  /**
   * <p>Return the value of the <code>grouped</code> property.  Contents:</p><p>
   * Boolean flag controlling whether the data values should
   * 				be grouped by the label.
   * </p>
   */
  public boolean isGrouped() {
    if (this.grouped_set) {
      return this.grouped;
    }
    ValueBinding _vb = getValueBinding("grouped"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return false;
      } else {
        return ((Boolean) _result).booleanValue();
      }
    } else {
        return this.grouped;
    }
  }

  /**
   * <p>Set the value of the <code>grouped</code> property.</p>
   */
  public void setGrouped(boolean grouped) {
    this.grouped = grouped;
    this.grouped_set = true;
  }


  public Object saveState(FacesContext _context) {
    Object _values[] = new Object[3];
    _values[0] = super.saveState(_context);
    _values[1] = this.grouped ? Boolean.TRUE : Boolean.FALSE;
    _values[2] = this.grouped_set ? Boolean.TRUE : Boolean.FALSE;
    return _values;
  }


  public void restoreState(FacesContext _context, Object _state) {
    Object _values[] = (Object[]) _state;
    super.restoreState(_context, _values[0]);
    this.grouped = ((Boolean) _values[1]).booleanValue();
    this.grouped_set = ((Boolean) _values[2]).booleanValue();
  }


}
