/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/

package com.ibm.faces.bf.component.html;


import java.io.IOException;
import javax.faces.context.FacesContext;
import javax.faces.el.MethodBinding;
import javax.faces.el.ValueBinding;


/**
 * <p>By default, the <code>rendererType</code> property must be set to "<code>TabbedPanel</code>" This value can be changed by calling the <code>setRendererType()</code> method.</p>
*/
public class HtmlTabbedPanel extends com.ibm.faces.bf.component.UITabbedPanel {


  public HtmlTabbedPanel() {
    super();
    setRendererType("TabbedPanel"); //$NON-NLS-1$
  }


  /*
   * <p>The standard component type for this component.</p>
   */
   public static final String COMPONENT_TYPE = "com.ibm.faces.bf.HtmlTabbedPanel"; //$NON-NLS-1$


  private java.lang.String height;

  /**
   * <p>Return the value of the <code>height</code> property.  Contents:</p><p>
   * The height of the tabbed panel in pixels.
   * </p>
   */
  public java.lang.String getHeight() {
    if (null != this.height) {
      return this.height;
    }
    ValueBinding _vb = getValueBinding("height"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>height</code> property.</p>
   */
  public void setHeight(java.lang.String height) {
    this.height = height;
  }


  private java.lang.String initialPanelId;

  /**
   * <p>Return the value of the <code>initialPanelId</code> property.  Contents:</p><p>
   * The id of the first panel to be shown when the tabbed
   * 				panel is displayed
   * </p>
   */
  public java.lang.String getInitialPanelId() {
    if (null != this.initialPanelId) {
      return this.initialPanelId;
    }
    ValueBinding _vb = getValueBinding("initialPanelId"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>initialPanelId</code> property.</p>
   */
  public void setInitialPanelId(java.lang.String initialPanelId) {
    this.initialPanelId = initialPanelId;
  }


  private int numOfTabs = Integer.MIN_VALUE;
  private boolean numOfTabs_set = false;

  /**
   * <p>Return the value of the <code>numOfTabs</code> property.  Contents:</p><p>
   * The number of tabs to make visible when the tabbed panel
   * 				in rendered. Must be less than or equal to the total
   * 				number for panels in the tabbed panel.
   * </p>
   */
  public int getNumOfTabs() {
    if (this.numOfTabs_set) {
      return this.numOfTabs;
    }
    ValueBinding _vb = getValueBinding("numOfTabs"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return Integer.MIN_VALUE;
      } else {
        return ((Integer) _result).intValue();
      }
    } else {
        return this.numOfTabs;
    }
  }

  /**
   * <p>Set the value of the <code>numOfTabs</code> property.</p>
   */
  public void setNumOfTabs(int numOfTabs) {
    this.numOfTabs = numOfTabs;
    this.numOfTabs_set = true;
  }


  private boolean showBackNextButton = false;
  private boolean showBackNextButton_set = false;

  /**
   * <p>Return the value of the <code>showBackNextButton</code> property.  Contents:</p><p>
   * Boolean flag indicating whether the back and next
   * 				buttons should be shown for the panels in the tabbed
   * 				panel.
   * </p>
   */
  public boolean isShowBackNextButton() {
    if (this.showBackNextButton_set) {
      return this.showBackNextButton;
    }
    ValueBinding _vb = getValueBinding("showBackNextButton"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return false;
      } else {
        return ((Boolean) _result).booleanValue();
      }
    } else {
        return this.showBackNextButton;
    }
  }

  /**
   * <p>Set the value of the <code>showBackNextButton</code> property.</p>
   */
  public void setShowBackNextButton(boolean showBackNextButton) {
    this.showBackNextButton = showBackNextButton;
    this.showBackNextButton_set = true;
  }


  private boolean showTabs = false;
  private boolean showTabs_set = false;

  /**
   * <p>Return the value of the <code>showTabs</code> property.  Contents:</p><p>
   * Boolean flag indicating whether the panels should be
   * 				shown or hidden. The panels are hidden when using the
   * 				tabbed panel in the wizard mode.
   * </p>
   */
  public boolean isShowTabs() {
    if (this.showTabs_set) {
      return this.showTabs;
    }
    ValueBinding _vb = getValueBinding("showTabs"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return false;
      } else {
        return ((Boolean) _result).booleanValue();
      }
    } else {
        return this.showTabs;
    }
  }

  /**
   * <p>Set the value of the <code>showTabs</code> property.</p>
   */
  public void setShowTabs(boolean showTabs) {
    this.showTabs = showTabs;
    this.showTabs_set = true;
  }


  private int slantActiveLeft = Integer.MIN_VALUE;
  private boolean slantActiveLeft_set = false;

  /**
   * <p>Return the value of the <code>slantActiveLeft</code> property.  Contents:</p><p>
   * Number of pixels for "nip and tuck".
   * </p>
   */
  public int getSlantActiveLeft() {
    if (this.slantActiveLeft_set) {
      return this.slantActiveLeft;
    }
    ValueBinding _vb = getValueBinding("slantActiveLeft"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return Integer.MIN_VALUE;
      } else {
        return ((Integer) _result).intValue();
      }
    } else {
        return this.slantActiveLeft;
    }
  }

  /**
   * <p>Set the value of the <code>slantActiveLeft</code> property.</p>
   */
  public void setSlantActiveLeft(int slantActiveLeft) {
    this.slantActiveLeft = slantActiveLeft;
    this.slantActiveLeft_set = true;
  }


  private int slantActiveRight = Integer.MIN_VALUE;
  private boolean slantActiveRight_set = false;

  /**
   * <p>Return the value of the <code>slantActiveRight</code> property.  Contents:</p><p>
   * Number of pixels for "nip and tuck".
   * </p>
   */
  public int getSlantActiveRight() {
    if (this.slantActiveRight_set) {
      return this.slantActiveRight;
    }
    ValueBinding _vb = getValueBinding("slantActiveRight"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return Integer.MIN_VALUE;
      } else {
        return ((Integer) _result).intValue();
      }
    } else {
        return this.slantActiveRight;
    }
  }

  /**
   * <p>Set the value of the <code>slantActiveRight</code> property.</p>
   */
  public void setSlantActiveRight(int slantActiveRight) {
    this.slantActiveRight = slantActiveRight;
    this.slantActiveRight_set = true;
  }


  private int slantInactiveLeft = Integer.MIN_VALUE;
  private boolean slantInactiveLeft_set = false;

  /**
   * <p>Return the value of the <code>slantInactiveLeft</code> property.  Contents:</p><p>
   * Number of pixels for "nip and tuck".
   * </p>
   */
  public int getSlantInactiveLeft() {
    if (this.slantInactiveLeft_set) {
      return this.slantInactiveLeft;
    }
    ValueBinding _vb = getValueBinding("slantInactiveLeft"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return Integer.MIN_VALUE;
      } else {
        return ((Integer) _result).intValue();
      }
    } else {
        return this.slantInactiveLeft;
    }
  }

  /**
   * <p>Set the value of the <code>slantInactiveLeft</code> property.</p>
   */
  public void setSlantInactiveLeft(int slantInactiveLeft) {
    this.slantInactiveLeft = slantInactiveLeft;
    this.slantInactiveLeft_set = true;
  }


  private int slantInactiveRight = Integer.MIN_VALUE;
  private boolean slantInactiveRight_set = false;

  /**
   * <p>Return the value of the <code>slantInactiveRight</code> property.  Contents:</p><p>
   * Number of pixels for "nip and tuck".
   * </p>
   */
  public int getSlantInactiveRight() {
    if (this.slantInactiveRight_set) {
      return this.slantInactiveRight;
    }
    ValueBinding _vb = getValueBinding("slantInactiveRight"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return Integer.MIN_VALUE;
      } else {
        return ((Integer) _result).intValue();
      }
    } else {
        return this.slantInactiveRight;
    }
  }

  /**
   * <p>Set the value of the <code>slantInactiveRight</code> property.</p>
   */
  public void setSlantInactiveRight(int slantInactiveRight) {
    this.slantInactiveRight = slantInactiveRight;
    this.slantInactiveRight_set = true;
  }


  private java.lang.String styleClass;

  /**
   * <p>Return the value of the <code>styleClass</code> property.  Contents:</p><p>
   * Prefix specifying CSS for multiple tabbedpanels on the
   * 				same page
   * </p>
   */
  public java.lang.String getStyleClass() {
    if (null != this.styleClass) {
      return this.styleClass;
    }
    ValueBinding _vb = getValueBinding("styleClass"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>styleClass</code> property.</p>
   */
  public void setStyleClass(java.lang.String styleClass) {
    this.styleClass = styleClass;
  }


  private boolean variableTabLength = false;
  private boolean variableTabLength_set = false;

  /**
   * <p>Return the value of the <code>variableTabLength</code> property.  Contents:</p><p>
   * Boolean flag indicating whether individual tabs should
   * 				have a fixed length or variable length based on their
   * 				text or image.
   * </p>
   */
  public boolean isVariableTabLength() {
    if (this.variableTabLength_set) {
      return this.variableTabLength;
    }
    ValueBinding _vb = getValueBinding("variableTabLength"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return false;
      } else {
        return ((Boolean) _result).booleanValue();
      }
    } else {
        return this.variableTabLength;
    }
  }

  /**
   * <p>Set the value of the <code>variableTabLength</code> property.</p>
   */
  public void setVariableTabLength(boolean variableTabLength) {
    this.variableTabLength = variableTabLength;
    this.variableTabLength_set = true;
  }


  private java.lang.String width;

  /**
   * <p>Return the value of the <code>width</code> property.  Contents:</p><p>
   * The width of the tabbed panel in pixels.
   * </p>
   */
  public java.lang.String getWidth() {
    if (null != this.width) {
      return this.width;
    }
    ValueBinding _vb = getValueBinding("width"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>width</code> property.</p>
   */
  public void setWidth(java.lang.String width) {
    this.width = width;
  }


  public Object saveState(FacesContext _context) {
    Object _values[] = new Object[21];
    _values[0] = super.saveState(_context);
    _values[1] = height;
    _values[2] = initialPanelId;
    _values[3] = new Integer(this.numOfTabs);
    _values[4] = this.numOfTabs_set ? Boolean.TRUE : Boolean.FALSE;
    _values[5] = this.showBackNextButton ? Boolean.TRUE : Boolean.FALSE;
    _values[6] = this.showBackNextButton_set ? Boolean.TRUE : Boolean.FALSE;
    _values[7] = this.showTabs ? Boolean.TRUE : Boolean.FALSE;
    _values[8] = this.showTabs_set ? Boolean.TRUE : Boolean.FALSE;
    _values[9] = new Integer(this.slantActiveLeft);
    _values[10] = this.slantActiveLeft_set ? Boolean.TRUE : Boolean.FALSE;
    _values[11] = new Integer(this.slantActiveRight);
    _values[12] = this.slantActiveRight_set ? Boolean.TRUE : Boolean.FALSE;
    _values[13] = new Integer(this.slantInactiveLeft);
    _values[14] = this.slantInactiveLeft_set ? Boolean.TRUE : Boolean.FALSE;
    _values[15] = new Integer(this.slantInactiveRight);
    _values[16] = this.slantInactiveRight_set ? Boolean.TRUE : Boolean.FALSE;
    _values[17] = styleClass;
    _values[18] = this.variableTabLength ? Boolean.TRUE : Boolean.FALSE;
    _values[19] = this.variableTabLength_set ? Boolean.TRUE : Boolean.FALSE;
    _values[20] = width;
    return _values;
  }


  public void restoreState(FacesContext _context, Object _state) {
    Object _values[] = (Object[]) _state;
    super.restoreState(_context, _values[0]);
    this.height = (java.lang.String) _values[1];
    this.initialPanelId = (java.lang.String) _values[2];
    this.numOfTabs = ((Integer) _values[3]).intValue();
    this.numOfTabs_set = ((Boolean) _values[4]).booleanValue();
    this.showBackNextButton = ((Boolean) _values[5]).booleanValue();
    this.showBackNextButton_set = ((Boolean) _values[6]).booleanValue();
    this.showTabs = ((Boolean) _values[7]).booleanValue();
    this.showTabs_set = ((Boolean) _values[8]).booleanValue();
    this.slantActiveLeft = ((Integer) _values[9]).intValue();
    this.slantActiveLeft_set = ((Boolean) _values[10]).booleanValue();
    this.slantActiveRight = ((Integer) _values[11]).intValue();
    this.slantActiveRight_set = ((Boolean) _values[12]).booleanValue();
    this.slantInactiveLeft = ((Integer) _values[13]).intValue();
    this.slantInactiveLeft_set = ((Boolean) _values[14]).booleanValue();
    this.slantInactiveRight = ((Integer) _values[15]).intValue();
    this.slantInactiveRight_set = ((Boolean) _values[16]).booleanValue();
    this.styleClass = (java.lang.String) _values[17];
    this.variableTabLength = ((Boolean) _values[18]).booleanValue();
    this.variableTabLength_set = ((Boolean) _values[19]).booleanValue();
    this.width = (java.lang.String) _values[20];
  }


}
