/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//  
//OCO Source Materials
//  
//5724-E76 and 5724-E77
//  
// Copyright IBM Corp.  2003
//  
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////
package com.ibm.faces.bf.renderkit;
import java.io.IOException;
import java.util.StringTokenizer;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;

import com.ibm.faces.bf.component.UIGraphDraw;
import com.ibm.faces.bf.component.UITree;
import com.ibm.faces.bf.component.UITreeNodeAttr;
import com.ibm.faces.bf.component.UIBehavior;
import com.ibm.faces.bf.component.html.HtmlBehavior;
import com.ibm.faces.renderkit.html_extended.Utils;
import com.ibm.odcb.jrender.emitters.GraphDrawEmitter;
import com.ibm.odcb.jrender.emitters.NodeAttributeEmitterHelper;
import com.ibm.odcb.jrender.emitters.TreeViewEmitter;
import com.ibm.odcb.jrender.emitters.BehaviorEmitter;
/**
 * Renderer for {@link com.ibm.faces.bf.component.UIBehavior} component.
 * The renderer renders aggregates a behaviour as child control for ODC components on the page.
 * <p> 
 * <b>Tags:</b> {@link com.ibm.faces.bf.taglib.BehaviorTag}<br>
 * <b>Components:</b> {@link com.ibm.faces.bf.component.UIBehavior}
 * </p> 
 * @see BrowserFrameworkRenderer  
 * @author Jianren Li
 */
public class BehaviorRenderer extends BrowserFrameworkRenderer 
{
	
	public void encodeEnd(FacesContext context, UIComponent component) throws IOException 
	{
		if (context == null) {
			throw new NullPointerException();
		}
		
		super.encodeEnd(context, component);

		HtmlBehavior behaviorComponent = null;
		UIBehavior comp = null;
		String targetFacet, event, action, targetAction, function;
		
		if ( component instanceof HtmlBehavior )
		{
			behaviorComponent = (HtmlBehavior)component;
			event = behaviorComponent.getEvent();
			action = behaviorComponent.getBehaviorAction();
			targetAction = behaviorComponent.getTargetAction();
			function = behaviorComponent.getOnActionFunction();
		}
		else
		{
			comp = (UIBehavior) component;	
			event = (String)component.getAttributes().get("event"); //$NON-NLS-1$
			action = (String)component.getAttributes().get("behaviorAction"); //$NON-NLS-1$
			targetAction = (String)component.getAttributes().get("targetAction"); //$NON-NLS-1$
			function = (String)component.getAttributes().get("onActionFunction"); //$NON-NLS-1$
		}
	
		if(event == null || (action == null && function == null)){
			//no event or action has been defined, exit
			return;
		}
				
		if ( targetAction != null) 
		{
			UIComponent targetComp = null;
			
			//note : may be multiple targets
			StringTokenizer st = new StringTokenizer(targetAction,";");
 			StringBuffer targets = new StringBuffer();
 			String current;
			while(st.hasMoreTokens()){
				current = st.nextToken();
				//check component, this will search from components parent NamingContainer
				targetComp = component.findComponent(current);
				if(targetComp  == null){
					//check from view root if not found
					targetComp  = context.getViewRoot().findComponent(current);
					if(targetComp  == null){
						//method to resursively search through all components
						targetComp  = Utils.findComponent(context.getViewRoot(),current);
					}
				}
				if(targetComp  != null){
					targets.append(targetComp.getClientId(context));
				} else {
					targets.append(current);
				}
				if(st.hasMoreTokens()){
					targets.append(";");
				}
			}
			targetAction = targets.toString();
		}
		
		try 
		{
			BehaviorEmitter behavior = new BehaviorEmitter();
			behavior.addEvent(event);
			behavior.addBehaviorAction(action);
			behavior.addTargetAction(targetAction);
			behavior.addOnActionFunction(function);
			
			UIComponent parent=null;
			if(behaviorComponent!=null)
				parent = behaviorComponent.getParent();
			else
				parent = behaviorComponent.getParent();
		
			//Possible parents of behavior component are tree, treeNodeAttr and graph (datagrid not supported
			if(parent instanceof UITree)
			{
				UITree treeComp = (UITree)parent;
				TreeViewEmitter parentEmitter = (TreeViewEmitter)treeComp.getEmitter();
				parentEmitter.addBehavior(behavior);
			}
			else if(parent instanceof UITreeNodeAttr)
			{
				UITreeNodeAttr treeNodeComp = (UITreeNodeAttr)parent;
				NodeAttributeEmitterHelper parentEmitter = (NodeAttributeEmitterHelper)treeNodeComp.getEmitter();
				parentEmitter.addBehavior(behavior);
			}
			else if(parent instanceof UIGraphDraw)
			{
				UIGraphDraw graphComp = (UIGraphDraw)parent;
				GraphDrawEmitter parentEmitter = (GraphDrawEmitter)graphComp.getEmitter();
				parentEmitter.addBehavior(behavior);
			}
	
		}
	
		catch (Exception e) 
		{
				e.printStackTrace();
				throw new IOException();
		}
		
		
		
		

	}
	
}
