/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//  
//OCO Source Materials
//  
//5724-E76 and 5724-E77
//  
// Copyright IBM Corp.  2003
//  
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////
package com.ibm.faces.bf.renderkit;

import java.io.IOException;

import javax.faces.component.UIComponent;
import javax.faces.component.UIForm;
import javax.faces.context.FacesContext;
import javax.faces.el.ValueBinding;
import javax.faces.render.Renderer;

import com.ibm.faces.bf.component.UIBrowserFramework;
import com.ibm.faces.bf.component.html.HtmlClientBinder;
import com.ibm.faces.bf.component.html.HtmlClientData;

/**
 * <p>
 * Base class for all the Browser Framework renders. Initializes the <code>ELUtil</code> class
 * in its encodeEnd method which can be used directly by the subclasses
 * </p>
 * @see ELUtil
 * @author fenil
 */
public abstract class BrowserFrameworkRenderer extends Renderer {
	
	protected ELUtil elUtil;
	
	public ELUtil getELUtil(){
		return this.elUtil;
	}
	
	/* (non-Javadoc)
	 * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeEnd(FacesContext context, UIComponent component) throws IOException {
		super.encodeEnd(context, component);
		this.elUtil = new ELUtil(component);
	}	
	
	public static UIForm findParentForm(UIComponent component){
		UIComponent parent = component;
        while (null != (parent = parent.getParent())) {
            if (parent instanceof UIForm) {
            	return (UIForm)parent;
            }
        }
        return null;
	}
	
	protected void processPageDataBinding(FacesContext context, UIComponent component)throws IOException {
		
		if(!(component instanceof UIBrowserFramework)){
			return;
		}
		
		ValueBinding serversideVB = component.getValueBinding("value");
		if(serversideVB != null){
			//server side binding is set and we can use it
			//create, add and render a ClientData component
			HtmlClientData clientDataComponent = new HtmlClientData();
			clientDataComponent.setId(component.getId() + "_serverBindingClientData");
			clientDataComponent.setModelName(component.getId() + "_serverBindingDataModel");
			clientDataComponent.setValueBinding("value",serversideVB);
			component.getParent().getChildren().add(0,clientDataComponent);
			//render it
			clientDataComponent.encodeBegin(context);
			if(clientDataComponent.getRendersChildren()){
				clientDataComponent.encodeChildren(context);
			}
			clientDataComponent.encodeEnd(context);
			
			//create,add and render a ClientBinder component
			HtmlClientBinder clientBinderComponent = new HtmlClientBinder();
			clientBinderComponent.setValueBinding("value",serversideVB);
			component.getChildren().add(0,clientBinderComponent);
			//render it
			clientBinderComponent.encodeBegin(context);
			if(clientBinderComponent.getRendersChildren()){
				clientBinderComponent.encodeChildren(context);
			}
			clientBinderComponent.encodeEnd(context);
			
			//remove the valuebinding on the tree component as it has been passed to client components, if
			//left it will fail a validation call in clientbinder's renderer
			component.setValueBinding("value",null);
			//set the component serverDataBinding to true, this can be used to check 
			//if server data has been set for the component
			((UIBrowserFramework)component).setServerDataBinding(true);
		}
	}
}
