/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//
//OCO Source Materials
//
//5724-E76 and 5724-E77
//
// Copyright IBM Corp.  2005
//
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////
package com.ibm.faces.bf.renderkit;

import java.io.IOException;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;

import com.ibm.faces.bf.component.UITabbedPanel;
import com.ibm.odcb.jrender.emitters.ButtonPanelEmitter;
import com.ibm.odcb.jrender.emitters.TabbedPanelEmitter;
import com.ibm.odcb.jrender.misc.Streamer;

/**
* Renderer for {@link com.ibm.faces.bf.component.UIButtonPanel UIButtonPanel} component.
* The renderer basically outputs a &lt;div&gt; tag on the page with the id supplied to the <code>bfPanel</code> tag. 
* <p> 
* <b>Tags:</b> {@link com.ibm.faces.bf.taglib.ButtonPanelTag}<br>
* <b>Components:</b> {@link com.ibm.faces.bf.component.UIButtonPanel}
* </p> 
* @see BrowserFrameworkRenderer  
* @author Declan Fahey
*/
public class ButtonPanelRenderer extends BrowserFrameworkRenderer {

	/**
	  * This method retrieves all the information for a panel and initializes PanelEmitter
	  * for every panel.
	  * @param context javax.faces.context.FacesContext
	  * @exception IOException - An exception is thrown if encountered an error.
	  **/
	public void encodeBegin(FacesContext context, UIComponent component) throws IOException {
		
		Streamer.trace.Header().println("Entering encodeBegin() of ButtonPanelRenderer.java");
				
		UITabbedPanel tabbedPanel = (UITabbedPanel) component.getParent();
		String panelId = tabbedPanel.getClientId(context) + ":" + component.getId();
		String alignContent = (String) component.getAttributes().get("alignContent");
		ButtonPanelEmitter emitter = new ButtonPanelEmitter(panelId, alignContent);
		
		((TabbedPanelEmitter) (tabbedPanel.getEmitter())).setButtonPanel(emitter);
		ResponseWriter writer = context.getResponseWriter();
		
		writer.write("<div class=\"tabbedPanel_FooterDiv\" id=\"" + emitter.getPanelID() + "\"" + 
				" style=\"visibility: hidden\""+">");
		Streamer.trace.Header().println("Exiting encodeBegin() of ButtonPanelRenderer.java");

	}
	/**
	  * This method just pushes out </div>.
	  * @param context javax.faces.context.FacesContext
	  * @exception IOException - An exception is thrown if exporting fails.
	  **/
	public void encodeEnd(FacesContext context, UIComponent component) throws IOException {
		ResponseWriter writer = context.getResponseWriter();
		writer.write("</div>");
	}

}
