/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//  
//OCO Source Materials
//  
//5724-E76 and 5724-E77
//  
// Copyright IBM Corp.  2004
//  
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////
package com.ibm.faces.bf.renderkit;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.faces.component.UIComponent;
import javax.faces.component.UIGraphic;
import javax.faces.component.UIInput;
import javax.faces.component.UIOutput;
import javax.faces.component.UISelectBoolean;
import javax.faces.component.UISelectItem;
import javax.faces.component.UISelectItems;
import javax.faces.component.UISelectMany;
import javax.faces.component.UISelectOne;
import javax.faces.component.UIViewRoot;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.convert.Converter;
import javax.faces.convert.DateTimeConverter;
import javax.faces.convert.NumberConverter;
import javax.faces.el.PropertyNotFoundException;
import javax.faces.el.ValueBinding;
import javax.faces.model.SelectItem;

import com.ibm.faces.bf.component.UIBrowserFramework;
import com.ibm.faces.bf.component.UIClientBinder;
import com.ibm.faces.bf.util.InvalidKeyException;
import com.ibm.faces.bf.util.ODCContextHelper;
import com.ibm.faces.component.IScriptContributor;
import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.converter.MaskConverter;
import com.ibm.faces.converter.NumberConverterEx;
import com.ibm.faces.renderkit.html_extended.HxClientRenderUtil;
import com.ibm.odcb.jrender.emitters.PropertyBinderEmitter;
import com.ibm.odcb.jrender.emitters.WDO4JSEmitter;
import com.ibm.odcb.jrender.mediators.ExportException;
import com.ibm.odcb.jrender.misc.Streamer;
import com.ibm.odcb.jrender.misc.StringUtil;

/**
 * <p>
 * Renderer for {@link com.ibm.faces.bf.component.UIClientBinder UIClientBinder}
 * component, that creates the client-side bindings for the parent JSF
 * component. <br>
 * This class implements the <code>IScriptContributor</code> interface, so
 * that the JavaScript for the client-side binding is outputted on the page
 * after the control is rendered.
 * </p>
 * <p>
 * <b>Tags: </b> {@link com.ibm.faces.bf.taglib.ClientBinderTag}<br>
 * <b>Components: </b> {@link com.ibm.faces.bf.component.UIClientBinder}
 * </p>
 * 
 * @see IScriptContributor
 * @see BrowserFrameworkRenderer
 * @author fenil
 */
public class ClientBinderRenderer extends BrowserFrameworkRenderer implements IScriptContributor {
    private FacesContext context;

    private ResponseWriter out;

    private UIClientBinder binder;

    //	List of UIInput subclasses that can be bound
    private final static List inputClasses = Arrays.asList(new String[] { "javax.faces.component.html.HtmlInputHidden",
            "javax.faces.component.html.HtmlInputSecret", "javax.faces.component.html.HtmlInputText",
            "javax.faces.component.html.HtmlInputTextarea" });

    //List of UISelectOne subclasses that can be bound
    private final static List selectOneClasses = Arrays.asList(new String[] { "javax.faces.component.html.HtmlSelectOneListbox",
            "javax.faces.component.html.HtmlSelectOneMenu", "javax.faces.component.html.HtmlSelectOneRadio" });

    //List of UISelectMany subclasses that can be bound
    private final static List selectManyClasses = Arrays.asList(new String[] { "javax.faces.component.html.HtmlSelectManyListbox",
            "javax.faces.component.html.HtmlSelectManyMenu", "javax.faces.component.html.HtmlSelectManyCheckbox" });

    //List of UIGraphic subclasses that can be bound
    private final static List imageClasses = Arrays.asList(new String[] { "javax.faces.component.html.HtmlGraphicImage",
            "com.ibm.faces.component.html.HtmlGraphicImageEx" });
    
    //Boolean storing flag that governs encoding. If page has FacesContext encoding should occur.
    private final boolean _isFaces=StringUtil.checkEncoding();
    
    /*
     * (non-Javadoc)
     * 
     * @see javax.faces.render.Renderer#encodeBegin(javax.faces.context.FacesContext,
     *      javax.faces.component.UIComponent)
     */
    public void encodeBegin(FacesContext context, UIComponent component) throws IOException {
        Streamer.trace.Header().println("Starting encodeBegin of ClientBinderRenderer with id = " + component.getId());
        if (context == null || component == null) 
        {
            throw new NullPointerException();
        }
        //check if server data binding has already been applied to component
        //if so for now do not render, maybe later throw an exception as user should
        //not bind component to both server and client data
    	UIComponent parent = component.getParent();
        if(parent instanceof UIBrowserFramework)
        {
        	if(((UIBrowserFramework)parent).isServerDataBinding())
        	{
        		return;
        	}
        }
        //call super.encodeBegin to setup the elUtil
        super.encodeBegin(context, component);
        UIScriptCollector scriptCollector = UIScriptCollector.find(component);
        // if script collector is found, register with it to be able
        // to create JS Script at the end of the page
        if (scriptCollector != null) 
        {
            scriptCollector.register(this, component);
        }
        Streamer.trace.Header().println("Ending encodeBegin of ClientBinderRenderer with id = " + component.getId());
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.faces.bf.renderkit.BrowserFrameworkRenderer#encodeEnd(javax.faces.context.FacesContext,
     *      javax.faces.component.UIComponent)
     */
    public void encodeEnd(FacesContext context, UIComponent component) throws IOException {
        //check if server data binding has already been applied to component
        //if so for now do not render, maybe later throw an exception as user should
        //not bind component to both server and client data
    	UIComponent parent = component.getParent();
        if(parent instanceof UIBrowserFramework){
        	if(((UIBrowserFramework)parent).isServerDataBinding()){
        		return;
        	}
        }
        super.encodeEnd(context, component);
        if (parent instanceof UIBrowserFramework) {
            ((UIBrowserFramework) parent).setClientBinding(component.getValueBinding("value"));
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.faces.component.IScriptContributor#contributeScript(javax.faces.context.FacesContext,
     *      javax.faces.component.UIComponent)
     */
    public void contributeScript(FacesContext context, UIComponent component) throws IOException {
        Streamer.trace.Header().println("Starting contributeScript of ClientBinderRenderer with id = " + component.getId());
        if (context == null || component == null) {
            throw new NullPointerException();
        }
        //Add a runtime check against dual bindings
        UIComponent parent = component.getParent();
        this.checkComponent(parent);
        this.elUtil = new ELUtil(component);
        this.context = context;
        this.out = context.getResponseWriter();
        this.binder = (UIClientBinder) component;
        List attributes = null;
        String bindervar = null;
        if ((parent instanceof UISelectOne && selectOneClasses.contains(parent.getClass().getName()))
                || (parent instanceof UISelectMany && selectManyClasses.contains(parent.getClass().getName()))) 
        {
            bindervar = this.generateSelectScript((UIInput) parent);
        } 
        else if (parent instanceof UIOutput || parent instanceof UIGraphic) 
        {
            attributes = this.getBindingAttributes(parent);
            if (attributes.size() != 5) 
            {
                Streamer.warning.Header().println(
                        "binder: parent with id = " + parent.getId()
                                + " not being bound to the model as all required attributes are not available.");
                return;
            }
            //Ordering of the conditionals is very important because of the
            // class hierarchy. Check the class hierarchy before making
            //any changes to this

            if (parent instanceof UISelectBoolean
                    && parent.getClass().getName().equals("javax.faces.component.html.HtmlSelectBooleanCheckbox")) 
            {
                bindervar = this.writeBinderScript(parent, (String) attributes.get(0), (String) attributes.get(1), (String) attributes
                        .get(2), (String) attributes.get(3), (String) attributes.get(4), "onclick", "checked");
            } 
            else if (parent instanceof UIInput && inputClasses.contains(parent.getClass().getName())) 
            {
                bindervar = this.writeBinderScript(parent, (String) attributes.get(0), (String) attributes.get(1), (String) attributes
                        .get(2), (String) attributes.get(3), (String) attributes.get(4), "onblur", "value");
                this.setConverter((UIOutput) parent, bindervar);
            } 
            else if (parent instanceof UIOutput && parent.getClass().getName().equals("javax.faces.component.html.HtmlOutputText")) 
            {
                bindervar = this.writeBinderScript(parent, (String) attributes.get(0), (String) attributes.get(1), (String) attributes
                        .get(2), (String) attributes.get(3), (String) attributes.get(4), null, "innerHTML");
                this.setConverter((UIOutput) parent, bindervar);
            } 
            else if (parent instanceof UIGraphic && imageClasses.contains(parent.getClass().getName())) 
            {
                bindervar = this.writeBinderScript(parent, (String) attributes.get(0), (String) attributes.get(1), (String) attributes
                        .get(2), (String) attributes.get(3), (String) attributes.get(4), null, "src");
            }
        }
        if (bindervar != null) 
        {
        	//Encode bindervar for portal
        	bindervar = StringUtil.encodeString(bindervar,_isFaces);
        	//Write it out to the page
            this.out.write(bindervar + ".DataBind();");
            JSUtil.registerControl(context, parent, bindervar);
        }
        Streamer.trace.Header().println("Ending contributeScript of ClientBinderRenderer with id = " + component.getId());
    }

    /**
     * Utility method to extract all the attributes required to create the
     * client-side bindings.
     * 
     * @param component
     *            the component whose attributes are being retrieved.
     * @return the list containing the required attributes for client-side
     *         binding.
     */
    private List getBindingAttributes(UIComponent component) {
        List list = new ArrayList();
        Object root = null;
        try {
            root = this.elUtil.getRootObject();
        } catch (PropertyNotFoundException e2) {
            Streamer.error.Header().printStackTrace(e2);
            return list;
        }
        WDO4JSEmitter emitter = null;
        try {
            emitter = (WDO4JSEmitter) ODCContextHelper.getEmitter(this.context, this.elUtil.getModelName());
        } catch (InvalidKeyException e3) {
            Streamer.error.Header().println("No model found with name = " + this.elUtil.getModelName());
            Streamer.error.Header().printStackTrace(e3);
        }
        String lastObjectExportId = null;
        try {
            lastObjectExportId = emitter.getExportIDByObject(this.elUtil.getLastObject());
        } catch (ExportException e) {
            Streamer.error.Header().println("Export Exception caught while binding control" + component);
            Streamer.error.Header().printStackTrace(e);
            return list;
        } catch (PropertyNotFoundException e) {
            Streamer.error.Header().printStackTrace(e);
            return list;
        }
        //if the component does not have an id it cannot be bound
        if (!(component.getId() == null || component.getId().equals(""))) {
            list.add(lastObjectExportId);
            list.add(component.getClientId(context));
            list.add(this.elUtil.getModelName());
            list.add(this.elUtil.getLastToken());
            list.add(component.getId());
        }
        return list;
    }

    /**
     * Writes the client-side binding JavaScript to the output stream, by
     * delegating to the <code>Export</code> method of the
     * <code>PropertyBinderRenderer</code>.
     * 
     * @param component
     *            the <code>UIComponent</code> for which the client-side
     *            binding is being created.
     * @param exportId
     *            the export id for the object whose attribute is being bound to
     *            the control.
     * @param clientId
     *            the id for the control that is going to be put on the page.
     * @param modelName
     *            the name of the model to which the control is being bound.
     * @param attributeName
     *            the attribute on the object that being bound to the control.
     * @param componentId
     *            the id supplied to the component on the JSP.
     * @param controlChangeEvent
     *            the JS event which results in a change in the value in the
     *            control
     * @param controlProperty
     *            the property of the control that needs to be updated when the
     *            value that it is bound to changes in the model
     * @return the name of the JavaScript PropertyBinder object created
     * @throws IOException
     */
    private String writeBinderScript(UIComponent component, String exportId, String clientId, String modelName, String attributeName,
            String componentId, String controlChangeEvent, String controlProperty) throws IOException {
        Streamer.status.Header().println("BinderRenderer: Binding child with client id = " + clientId);
        Streamer.debug.Header().println(
                "binder: Binding control with id = " + clientId + " exportId = " + exportId + " attributeName = " + attributeName
                        + " change event = " + controlChangeEvent + " control property = " + controlProperty);
        PropertyBinderEmitter emitter = new PropertyBinderEmitter(exportId, clientId, true, modelName, attributeName, controlChangeEvent,
                controlProperty);
        ODCContextHelper.setEmitter(this.context, componentId, emitter);
        try {
            emitter.Export(this.out, ODCContextHelper.getODCPageContext(this.context));
        } catch (ExportException e) {
            Streamer.error.Header().println("Export exception caught while exporting contents of the property binder");
            Streamer.error.Header().printStackTrace(e);
            return null;
        } catch (IOException e) {
            Streamer.error.Header().println("IO exception caught while exporting contents of the property binder");
            Streamer.error.Header().printStackTrace(e);
            return null;
        }
        return emitter.ADAPTER_VAR + emitter.getExportID();
    }

    /**
     * Associates a client-side converter with a particular
     * <code>UIComponent</code>
     * 
     * @param component
     *            the <code>UIComponent</code> for which the converter is
     *            being set.
     * @param bindervar
     *            the name of the JavaScript PropertyBinder object associated
     *            with this component.
     * @throws IOException
     */

    private void setConverter(UIOutput component, String bindervar) throws IOException {
        if (bindervar == null) 
        {
            return;
        }
        Converter converter = component.getConverter();
        
        //Enocode variable for portal
        bindervar  = StringUtil.encodeString(bindervar,_isFaces);
         
        if (converter instanceof DateTimeConverter) 
        {
            this.out.write(bindervar + ".setConverter(" + HxClientRenderUtil.buildJsDateTimeConverter((DateTimeConverter) converter) + ");\n");
        } 
        else if (converter instanceof NumberConverter) 
        {
            this.out.write(bindervar + ".setConverter(" + HxClientRenderUtil.buildJsNumberConverter((NumberConverterEx) converter) + ");\n");
        } 
        else if (converter instanceof MaskConverter) 
        {
            MaskConverter mc = (MaskConverter) converter;
            if (!mc.isAsis()) 
            {
                this.out.write(bindervar + ".setConverter(new MaskConverter('" + mc.getMask() + "'));\n");
            }
        }

    }

    /**
     * Generates the binder JavaScript for the different kinds of SelectOne and
     * SelectMany components supported by JSF (list, combobox, checkbox and
     * radio button).
     * 
     * @param component
     *            the Select component for which the script is being generated.
     * @throws IOException
     */
    private String generateSelectScript(UIInput component) throws IOException {
        Streamer.status.Header().println("Generating select binding script for component with id = " + component.getId());
        Iterator selectItems = getSelectItems(this.context, component).iterator();
        Object root = null;
        try 
		{
            root = this.elUtil.getRootObject();
        } 
        catch (PropertyNotFoundException e2) 
		{
            Streamer.error.Header().println(
                    "Unable to find the root object for the binding expression = " + this.elUtil.getOriginalBindingExpression());
            Streamer.error.Header().printStackTrace(e2);
            return null;
        }
        WDO4JSEmitter emitter;
        try 
		{
            emitter = (WDO4JSEmitter) ODCContextHelper.getEmitter(this.context, this.elUtil.getModelName());
        } 
        catch (InvalidKeyException e3) 
		{
            Streamer.error.Header().println("No model found with name = " + this.elUtil.getModelName());
            Streamer.error.Header().printStackTrace(e3);
            return null;
        }
        String lastObjectExportId = null;
        String _lastObjectExportId=null;
        try 
		{
            //This is going to be the export id for the last eobject in the
            // valueRef expression
            lastObjectExportId = emitter.getExportIDByObject(this.elUtil.getLastObject());
            //Encode variable for portal
            _lastObjectExportId =StringUtil.encodeString(lastObjectExportId,_isFaces);
        } 
        catch (ExportException e) 
		{
            Streamer.error.Header().println("ExportException while getting the export id for the target object of the select control");
            Streamer.error.Header().printStackTrace(e);
            return null;
        } 
        catch (PropertyNotFoundException e) 
		{
            Streamer.error.Header().printStackTrace(e);
            return null;
        }
        //TODO Check to make sure that this works
        List exportIdList = new ArrayList();
        try 
		{
            while (selectItems.hasNext()) 
            {
                SelectItem item = (SelectItem) selectItems.next();
                exportIdList.add(emitter.getExportIDByObject(item));
            }
        } 
        catch (ExportException e) 
		{
            Streamer.error.Header().println("ExportException while getting the export ids for the select item list");
            Streamer.error.Header().printStackTrace(e);
            return null;
        }
        UIViewRoot viewRoot = this.context.getViewRoot();
        String arrayString = JSUtil.buildOriginalOptionStrings(exportIdList);
        String selectedValues = this.getValuesArrayForSelect();
        /*
         * String handlerName = "eventHandler" + viewRoot.createUniqueId();
         * String control = "control" + viewRoot.createUniqueId(); String
         * controlArray = "controlArray" + viewRoot.createUniqueId(); String
         * eventString = "var " + handlerName + " = new SelectEventHandler(" +
         * "WDO4JSModelRoot_" + this.elUtil.getModelName() + ".Root" + ", [" +
         * arrayString + "], '" + lastObjectExportId + "', '" +
         * this.elUtil.getLastToken() + "');\n"; this.out.write(eventString);
         * this.out.write("var " + control + " = document.getElementById('" +
         * component.getClientId(context) + "');\n"); this.out.write("if(" +
         * control + ".tagName.toLowerCase() == 'select'){\n");
         * this.out.write("selectValues(" + control + "," + selectedValues +
         * ");\n"); this.out.write(control + ".onchange = function(){" +
         * handlerName + ".handleSelect(this);}\n"); this.out.write("}\n");
         * this.out.write("else{\n"); this.out.write("var " + controlArray + " =
         * findControlsArray('" + component.getClientId(context) + "');\n");
         * this.out.write("if(" + controlArray + "[0].type == 'radio'){\n");
         * this.out.write("checkValues(" + controlArray + "," + selectedValues +
         * ");\n"); this.out.write("for(var k = 0; k < " + controlArray +
         * ".length; k++){\n"); this.out.write( controlArray + "[k].onclick =
         * function(){" + handlerName + ".handleRadio(" + controlArray +
         * ");}\n"); this.out.write("}\n"); this.out.write("}\n");
         * this.out.write("else if (" + controlArray + "[0].type ==
         * 'checkbox'){\n"); this.out.write("checkValues(" + controlArray + "," +
         * selectedValues + ");\n"); this.out.write("for(var n=0; n < " +
         * controlArray + ".length; n++){\n"); this.out.write( controlArray +
         * "[n].onclick = function(){" + handlerName + ".handleCheckbox(" +
         * controlArray + ");}\n"); this.out.write("}\n");
         * this.out.write("}\n"); this.out.write("}\n");
         */
        String selectControl = "control" + viewRoot.createUniqueId();
       
        //Encode variable for portal
        selectControl=StringUtil.encodeString(selectControl,_isFaces);
        String client_Id_Var=StringUtil.encodeString(component.getClientId(context),_isFaces);
        String root_Var=StringUtil.encodeString("WDO4JSModelRoot_" + this.elUtil.getModelName()
                 + ".Root",_isFaces );
        String last_token =StringUtil.encodeString(this.elUtil.getLastToken(),_isFaces);
        this.out.write("var " + selectControl + " = new Select(\"" + client_Id_Var+ "\", " + root_Var + ", [" + arrayString + "], '" + _lastObjectExportId + "', '" + last_token + "');\n");
      
        PropertyBinderEmitter binderEmitter = new PropertyBinderEmitter(lastObjectExportId, selectControl, false, this.elUtil
                .getModelName(), this.elUtil.getLastToken(), "onchange", "value");
        try {
            binderEmitter.Export(this.out, ODCContextHelper.getODCPageContext(this.context));

        } catch (ExportException e1) {
            Streamer.error.Header().println("Export exception caught while exporting contents of the property binder");
            Streamer.error.Header().printStackTrace(e1);
            return null;
        } catch (IOException e1) {
            Streamer.error.Header().println("IO exception caught while exporting contents of the property binder");
            Streamer.error.Header().printStackTrace(e1);
            return null;
        }
        return binderEmitter.ADAPTER_VAR + binderEmitter.getExportID();

    }

    private List getSelectItems(FacesContext context,UIComponent component) {
		ArrayList list = new ArrayList();
		Iterator children = component.getChildren().iterator();
		while (children.hasNext()) {
			UIComponent child = (UIComponent) children.next();
			if (child instanceof UISelectItem) {
				Object value = ((UISelectItem) child).getValue();
				if (value == null) {
					UISelectItem selectItem = (UISelectItem) child;
					list.add(new SelectItem(selectItem.getItemValue(), selectItem.getItemLabel(), selectItem.getItemDescription(), selectItem.isItemDisabled()));
				} else if (value instanceof SelectItem) {
					list.add(value);
				} else {
					throw new IllegalArgumentException();
				}
			} else if (child instanceof UISelectItems && null != context) {
				Object value = ((UISelectItems) child).getValue();
				if (value instanceof SelectItem) {
					list.add(value);
				} else if (value instanceof SelectItem[]) {
					SelectItem items[] = (SelectItem[]) value;
					for (int i = 0; i < items.length; i++) {
						list.add(items[i]);
					}
				} else if (value instanceof Collection) {
					Iterator elements = ((Collection) value).iterator();
					while (elements.hasNext()) {
						list.add(elements.next());
					}
				} else if (value instanceof Map) {
					Iterator keys = ((Map) value).keySet().iterator();
					while (keys.hasNext()) {
						Object key = keys.next();
						if (key == null) {
							continue;
						}
						Object val = ((Map) value).get(key);
						if (val == null) {
							continue;
						}
						list.add(new SelectItem(val.toString(), key.toString(),
								null));
					}
				} else {
					throw new IllegalArgumentException();
				}
			}
		}
		return (list);

	}
    
    /**
     * Checks is a <code>UIComponent</code> has a server-side
     * <code>ValueBinding</code>
     * 
     * @param component
     *            the component to be checked for the server-side binding.
     * @throws IOException
     */
    private void checkComponent(UIComponent component) throws IOException {
        ValueBinding vb = component.getValueBinding("value");
        if (vb != null) {
            throw new IOException("Component with id = " + component.getId() + " cannot have both client and server bindings.\n");
        }
    }

    /**
     * <p>
     * Creates a JavaScript array containing the which contains the currently
     * selected values in the <code>SelectOne</code> or
     * <code>SelectMany</code> control.
     * </p>
     * 
     * @return the JavaScript array string
     */
    private String getValuesArrayForSelect() {
        Object value = this.binder.getValue();
        if (value == null) {
            return "[]";
        } else {
            List list = new ArrayList();
            Object[] selectedValues = null;
            if (value.getClass().getName().charAt(0) == '[') {
                selectedValues = (Object[]) value;
            } else if (value instanceof Collection) {
                selectedValues = ((Collection) value).toArray();
            } else {
                list.add(value.toString());
            }
            if (selectedValues != null) {
                for (int i = 0; i < selectedValues.length; i++) {
                    list.add(selectedValues[i].toString());
                }
            }
            return "[" + JSUtil.buildOriginalOptionStrings(list) + "]";

        }
    }

}