/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//
//OCO Source Materials
//
//5724-E76 and 5724-E77
//
// Copyright IBM Corp.  2004
//
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////
package com.ibm.faces.bf.renderkit;

import java.io.IOException;
import java.util.Map;

import javax.faces.component.UIComponent;
import javax.faces.component.UIForm;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.el.PropertyNotFoundException;
import javax.faces.event.ActionEvent;

import com.ibm.faces.bf.component.UIClientData;
import com.ibm.faces.bf.util.InitializationUtil;
import com.ibm.faces.bf.util.ODCContextHelper;
import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.renderkit.html_extended.HxClientRenderUtil;
import com.ibm.odcb.jrender.emitters.WDO4JSEmitter;
import com.ibm.odcb.jrender.mediators.ExportException;
import com.ibm.odcb.jrender.mediators.PageContext;
import com.ibm.odcb.jrender.misc.Config;
import com.ibm.odcb.jrender.misc.Streamer;
import com.ibm.odcb.jrender.misc.StringUtil;
import com.ibm.odcb.jrender.utilities.ODCClientDataMarkupAggregator;


/**
 * Renderer for {@link com.ibm.faces.bf.component.UIClientData UIClientData}
 * component, that outputs the JavaScript for the client-side model and data.
 * This class also makes sure all the dependent JavaScript files from HxClient
 * are loaded on the page.
 * <p>
 * <b>Tags: </b> {@link com.ibm.faces.bf.taglib.ClientDataTag}<br>
 * <b>Components: </b> {@link com.ibm.faces.bf.component.UIClientData}
 * </p>
 *
 * @author fenil
 */
public class ClientDataRenderer extends BrowserFrameworkRenderer {

	
    /*
     * (non-Javadoc)
     *
     * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext,
     *      javax.faces.component.UIComponent)
     */
    public void encodeEnd(FacesContext context, UIComponent component) throws IOException {
        Streamer.trace.Header().println("Entering encodeEnd() of ClientDataRenderer.java");
        if (context == null) {
            throw new NullPointerException();
        }
        super.encodeEnd(context, component);
      
        String modelname = this.elUtil.getModelName();
        
        //Boolean storing flag that governs encoding. If page has FacesContext encoding should occur.
        boolean _isFaces=StringUtil.checkEncoding();
        
        ResponseWriter writer = context.getResponseWriter();
        //Get the PageContext from the current request. The PageContext object
        // is automatically initialized by
        //the managed bean creation facility in faces-config.xml
        PageContext pageContext = ODCContextHelper.getODCPageContext(context);
        Object rootObject = null;
    	        	
    	try {
    		rootObject = this.elUtil.resolveWholeExpressionToObject();
    	} catch (PropertyNotFoundException e) {
    		e.printStackTrace();
    		Streamer.error.Header().println("Exception occurred during rootBean retrieval in encodeEnd() of ClientDataRenderer.java");
    		throw new IOException();
    	}

		
        // -- Begin Kaushal's NL-suffix code addition 9/25/2003
        Streamer.trace.Header().println("Calling NLJSFilePrinter from ClientDataRenderer...");
        InitializationUtil.printIncludes(writer, context, pageContext);
        //		-- End Kaushal's NL-suffix code addition 9/25/2003

        //Logic for including Seoul hxclient_xx.js
        UIScriptCollector scriptCollector = UIScriptCollector.find(component);
        if (scriptCollector != null) {
            HxClientRenderUtil.initJSLibraries(scriptCollector, context, true);
        }
        //We have one WDO4JSEmitter per request per model..so we put it into
        // the request cache with the model name
        //All the other controls can retrieve the emitter using the model name
        WDO4JSEmitter modelEmitter = new WDO4JSEmitter();
        ODCContextHelper.setEmitter(context, this.elUtil.getModelName(), modelEmitter);
        modelEmitter.Init(rootObject, this.elUtil.getModelName());
        
        //Encoding Variables For Portal
        String _BindExpr = StringUtil.encodeString(this.elUtil.getOriginalBindingExpression(),_isFaces);
        String _ModelName = StringUtil.encodeString(this.elUtil.getModelName(),_isFaces);
        
        try {
        	ODCClientDataMarkupAggregator markupAggregator = new ODCClientDataMarkupAggregator(); //added by Thomas Delahunty to fix the writer memory Problem ODCClientDataMarkupAggregator coments for details
            
        	modelEmitter.Export(markupAggregator, pageContext);
            
            writer.write(markupAggregator.toString()); //added by Thomas Delahunty to fix the writer memory Problem see ODCClientDataMarkupAggregator coments for details
            
            writer.startElement("script type=\"text/javascript\"", component);
            writer.write("ODCRegistry.addModelName('" + _BindExpr + "','" + _ModelName
                    + "');\n");
            writer.endElement("script");
        } catch (ExportException e) {
            Streamer.error.Header().println("Export exception while exporting root object");
            e.printStackTrace();
            throw new IOException();
        }
        UIForm parentForm = BrowserFrameworkRenderer.findParentForm(component);
        if (parentForm != null ) {
            String formId = parentForm.getClientId(context);

            //Register the model agaist ODCRegistry
            writer.startElement("script type=\"text/javascript\"", component);
            writer.write("ODCRegistry.addModelInfo(\"" +_ModelName + "\"," + modelEmitter.CONTROL_VAR + ", \"" + formId + "\");");
            writer.endElement("script");
            JSUtil.registerDiffListener(component, writer, pageContext, formId);
        }
        
        Streamer.trace.Header().println("Exiting encodeEnd() of ClientDataRenderer.java");
    }

    /*
     * (non-Javadoc)
     *
     * @see javax.faces.render.Renderer#decode(javax.faces.context.FacesContext,
     *      javax.faces.component.UIComponent)
     */
    public void decode(FacesContext context, UIComponent component) {
        Streamer.trace.Header().println("Entering decode() of ClientDataRenderer.java component id: " + component.getId());
        if (context == null) {
            throw new NullPointerException();
        }
        
        //because client data is not encoded. We need to first try to see if the unencoded
        //key exists on the map. If it does, then ignore.
        //If that key is null, then try to find the encoded version of the key. 
        //If that returns something then add the diff string back onto the map
        //using the unencoded version of the key.
        Map requestParameterMap = context.getExternalContext().getRequestParameterMap();
        String diffName = new ELUtil(component).getModelName() + "Diff";
        String diffStr = (String) requestParameterMap.get(diffName);
        
        if(diffStr == null)
        {
        	String encodedDiffName = StringUtil.encodeString(diffName,StringUtil.checkEncoding());
        	String encodedDiffString = (String) requestParameterMap.get(encodedDiffName);
         
        	if(diffStr==null && encodedDiffString != null && !encodedDiffString.equals(""))
        	{
        		requestParameterMap.put(diffName,diffStr);
        		diffStr = encodedDiffString;
        	}
        }
        Streamer.debug.Header().println("in decode() of ClientDataRenderer.java diffStr before removing possibel portal encoding:" + diffStr);
        if(diffStr !=null && !diffStr.equals(""))
        {
           boolean isPortal = false;
           String httpReqClassName = Config.getString(Config.PROP_REQUEST_CLASS);
		   if(httpReqClassName.equals("com.ibm.faces.bf.util.HttpRequestRetrieverForServlet"))
		   {
		      //not a portal
		      isPortal = false;
		   }
		   else
		   {
              //it is a portal
		      isPortal = true;
		   }

           if(isPortal ==  true)
           {
              diffStr = StringUtil.restoreEncodedDiffToValidXML(diffStr);
              Streamer.debug.Header().println("in decode() of ClientDataRenderer.java diffStr after removing possibel portal encoding:" + diffStr);
           }

           ((UIClientData) component).setDiffStr(diffStr);
           component.queueEvent(new ActionEvent(component));
        }
        Streamer.debug.Header().println("in decode() of ClientDataRenderer.java");
        Streamer.trace.Header().println("Exiting decode() of ClientDataRenderer.java");
    }

}