/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//  
//OCO Source Materials
//  
//5724-E76 and 5724-E77
//  
// Copyright IBM Corp.  2004
//  
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////
package com.ibm.faces.bf.renderkit;
import java.io.IOException;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.faces.convert.DateTimeConverter;
import javax.faces.convert.NumberConverter;

import com.ibm.faces.bf.component.UIDataGrid;
import com.ibm.faces.bf.component.UIDataGridCol;
import com.ibm.faces.converter.MaskConverter;
import com.ibm.faces.converter.NumberConverterEx;
import com.ibm.faces.renderkit.html_extended.HxClientRenderUtil;
import com.ibm.odcb.jrender.emitters.DataGridEmitter;
import com.ibm.odcb.jrender.emitters.DataGridEmitterHelper;
import com.ibm.odcb.jrender.misc.Streamer;
/**
 * Renderer for {@link com.ibm.faces.bf.component.UIDataGridCol UIDataGridCol} component, 
 * that renders the columns for the client-side data table.
 * <p> 
 * <b>Tags:</b> {@link com.ibm.faces.bf.taglib.DataGridColTag}<br>
 * <b>Components:</b> {@link com.ibm.faces.bf.component.UIDataGridCol}
 * </p> 
 * @see BrowserFrameworkRenderer
 * @author Jianren Li
 */
public class DataGridColRenderer extends BrowserFrameworkRenderer
{
	
	/**
	 * @see javax.faces.component.UIComponent#encodeEnd(FacesContext)
	 */
	public void encodeEnd(FacesContext context, UIComponent component)
		throws IOException
	{
		Streamer.trace.Header().println(
			"enter encodeEnd() of DataGridColRenderer.java");
		if (context == null)
		{
			throw new NullPointerException();
		}
		super.encodeEnd(context, component);
		String attributeName = ((UIDataGridCol) component).getAttributeName();
		Streamer.debug.Header().println(
			"in encodeEnd() of DataGridColRenderer.java getting attributeName:"
				+ attributeName);
		//Fenil Change 01/14/04 -> Bundle use removed
		String colHead = (String) component.getAttributes().get("colHead");
		Streamer.debug.Header().println(
			"in encodeEnd() of DataGridColRenderer.java getting colHeadKey:"
				+ colHead);
		Boolean readOnly = (Boolean) component.getAttributes().get("readOnly");
		Boolean underline =
			(Boolean) component.getAttributes().get("underline");
		/*
		Boolean currencyPrefix =
			(Boolean) component.getAttributes().get("currencyPrefix");
		Boolean currencyPostfix =
			(Boolean) component.getAttributes().get("currencyPostfix");
		*/
		String alignment = (String) component.getAttributes().get("alignment");
		String width = (String) component.getAttributes().get("width");
		Streamer.debug.Header().println(
			"in encodeEnd() of DataGridColRenderer.java getting width:"
				+ width);
		/*
		Integer decimalPlaces =
			(Integer) component.getAttributes().get("decimalPlaces");
		Streamer.debug.Header().println(
			"in encodeEnd() of DataGridColRenderer.java getting decimalPlaces:"
				+ decimalPlaces);
		String additionalPrefix =
			(String) component.getAttributes().get("additionalPrefix");
		String additionalPostfix =
			(String) component.getAttributes().get("additionalPostfix");
		String dateFormat =
			(String) component.getAttributes().get("dateFormat");
		String timeFormat =
			(String) component.getAttributes().get("timeFormat");
		*/
		UIDataGrid myParent = (UIDataGrid) component.getParent();
		if (myParent == null)
		{
			Streamer.error.Header().println("The Parent DataGrid is null!");
			throw new IOException("The Parent DataGrid is null!");
		}
		String eAttributeName = attributeName;
		String converterCreationScript = null;
		//		Get converter child tag and set the data format type and parameters in the emitter
		Converter converter = ((UIDataGridCol) component).getConverter();
		Streamer.debug.Header().println("Converter = " + converter);
		if (converter != null)
		{
			if (converter instanceof DateTimeConverter)
			{
				DateTimeConverter dtc = (DateTimeConverter) converter;			
				converterCreationScript =
					HxClientRenderUtil.buildJsDateTimeConverter(dtc);
			}
			else if (converter instanceof NumberConverter)
			{
				NumberConverterEx nc = (NumberConverterEx) converter;
				converterCreationScript =
					HxClientRenderUtil.buildJsNumberConverter(nc);
			}
			else if (converter instanceof MaskConverter)
			{
				MaskConverter mc = (MaskConverter) converter;
				converterCreationScript = "new MaskConverter('" + mc.getMask() + "')";	
			}
		}
		DataGridEmitter dataGridEmitter = (DataGridEmitter)myParent.getEmitter();
		DataGridEmitterHelper col =	
			new DataGridEmitterHelper(
				eAttributeName,
				colHead == null ? eAttributeName : colHead,
				readOnly == null ? true : readOnly.booleanValue(),
				alignment == null ? "center" : alignment,
				width == null ? "-1" : width,
		        underline == null ? false : underline.booleanValue(),
		        converterCreationScript
		        );
				
		dataGridEmitter.addColumn(col);
		Streamer.trace.Header().println(
			"exit encodeEnd() of DataGridColRenderer.java");
	}
	
}
