/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//  
//OCO Source Materials
//  
//5724-E76 and 5724-E77
//
// Copyright IBM Corp.  2004
//  
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////
package com.ibm.faces.bf.renderkit;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Map;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.el.PropertyNotFoundException;

import com.ibm.faces.bf.component.UIBrowserFramework;
import com.ibm.faces.bf.component.UIDataGrid;
import com.ibm.faces.bf.util.ODCContextHelper;
import com.ibm.odcb.jrender.emitters.DataGridEmitter;
import com.ibm.odcb.jrender.emitters.WDO4JSEmitter;
import com.ibm.odcb.jrender.mediators.PageContext;
import com.ibm.odcb.jrender.misc.Streamer;
import com.ibm.odcb.jrender.misc.StringUtil;
import com.ibm.odcb.jrender.misc.Utils;
/**
 * Renderer for {@link com.ibm.faces.bf.component.UIDataGrid UIDataGrid} component, 
 * that renders the client-side data table.
 * <p> 
 * <b>Tags:</b> {@link com.ibm.faces.bf.taglib.DataGridTag}<br>
 * <b>Components:</b> {@link com.ibm.faces.bf.component.UIDataGrid}
 * </p> 
 * @see BrowserFrameworkRenderer
 * @author Jianren Li
 */
public class DataGridRenderer extends BrowserFrameworkRenderer {

	/**
	 * @see javax.faces.component.UIComponentBase#encodeBegin(FacesContext)
	 */
	public void encodeBegin(FacesContext context, UIComponent component) throws IOException {
		if (context == null) {
			throw new NullPointerException();
		}
		Streamer.trace.Header().println("begin encodeBegin() of UIDataGridRenderer.java");
		//call super.encodeBegin to set up the elUtl
		super.encodeBegin(context, component);
		String id = component.getId();
		Streamer.debug.println("in encodeBegin() of UIDataGridRenderer.java getting id:" + id);
		DataGridEmitter dataGridEmitter = new DataGridEmitter();
		((UIDataGrid) component).setEmitter(dataGridEmitter);
		Streamer.trace.Header().println("end encodeBegin() of UIDataGridRenderer.java");
	}

	/**
	 * @see javax.faces.component.UIComponentBase#encodeEnd(FacesContext)
	 */
	public void encodeEnd(FacesContext context, UIComponent component) throws IOException {
		Streamer.trace.Header().println("enter encodeEnd() of UIDataGridRenderer.java");
		if (context == null) {
			throw new NullPointerException();
		}
		super.encodeEnd(context, component);
		if (!this.elUtil.isListBinding()) {
			throw new IOException("The datagrid " + component.getId() + " must be bound to a list type");
		}
		ResponseWriter writer = context.getResponseWriter();
		Boolean allowRowAddAndDelete = (Boolean) component.getAttributes().get("allowRowAddAndDelete");
		//In order to be consistent with flag in emitter and JS (readOnly) semantically, we reverse the boolean here
		boolean readOnly = true;
		if (allowRowAddAndDelete.booleanValue() == true) {
			readOnly = false;
		} else {
			readOnly = true;
		}
		Boolean allowColumnHideWrapper = (Boolean) component.getAttributes().get("allowColumnHide");
		boolean allowColumnHide = allowColumnHideWrapper.booleanValue();
		String scrollSize = (String) component.getAttributes().get("scrollSize");
		String width = (String) component.getAttributes().get("width");
		Integer pageSize = (Integer) component.getAttributes().get("pageSize");
		String rowFilter = (String) component.getAttributes().get("rowFilter");
		String styleClass = (String) component.getAttributes().get("styleClass");
		Integer navBarPosition = (Integer) component.getAttributes().get("navBarPosition");
		if (navBarPosition == null)
			navBarPosition = new Integer(DataGridEmitter.NAV_BAR_BOTTOM);

		Boolean showRowIndex = (Boolean) component.getAttributes().get("showRowIndex");
		if (showRowIndex == null)
			showRowIndex = new Boolean(false);
		Boolean showSelectionColumn = (Boolean) component.getAttributes().get("showSelectionColumn");
		String selectionColName = null;
		Boolean multiSelection = null;
		String selectionColAlignment = null;
		String selectionColWidth = null;
		if (showSelectionColumn != null && showSelectionColumn.booleanValue()) {
			multiSelection = (Boolean) component.getAttributes().get("multiSelection");
			selectionColAlignment = (String) component.getAttributes().get("selectionColAlignment");
			selectionColWidth = (String) component.getAttributes().get("selectionColWidth");
			selectionColName = (String) component.getAttributes().get("selectionColName");
		}
		//Event handlers
		String onhighlight = JSUtil.trimEventHandler(((UIDataGrid) component).getOnhighlight());
		String onselect = JSUtil.trimEventHandler(((UIDataGrid) component).getOnselect());
		String onunselect = JSUtil.trimEventHandler(((UIDataGrid) component).getOnunselect());
		String onpage = JSUtil.trimEventHandler(((UIDataGrid) component).getOnpage());
		String onsort = JSUtil.trimEventHandler(((UIDataGrid) component).getOnsort());
		String onfilter = JSUtil.trimEventHandler(((UIDataGrid) component).getOnfilter());
		String onselectall = JSUtil.trimEventHandler(((UIDataGrid) component).getOnselectall());
		String onunselectall = JSUtil.trimEventHandler(((UIDataGrid) component).getOnunselectall());
		String onrowadd = JSUtil.trimEventHandler(((UIDataGrid) component).getOnrowadd());
		String onrowremove = JSUtil.trimEventHandler(((UIDataGrid) component).getOnrowremove());		
		
		Object obj = null;
		Object rootBean = null;
		try {
			rootBean = this.elUtil.getRootObject();
			obj = this.elUtil.getLastObject();
		} catch (PropertyNotFoundException e1) {
			Streamer.error.printStackTrace(e1);
			return;
		}
		//Streamer.debug.Header().println("in encodeBegin() of UIDataGridRenderer.java getting valueref object:" + obj);
		if (rootBean == null) {
			Streamer.error.Header().println("in encodeBegin() of UIDataGridRenderer.java--- rootBean is null");
			throw new NullPointerException();
		} else {
			Streamer.debug.Header().println("in encodeBegin() of UIDataGridRenderer.java--- rootBean is not null");
		}
		DataGridEmitter dataGridEmitter = (DataGridEmitter)((UIDataGrid)component).getEmitter();
		try {
			WDO4JSEmitter aWDO4JSEmitter =
				(WDO4JSEmitter) ODCContextHelper.getEmitter(context, this.elUtil.getModelName());
			//infer eObjectId and eReferenceName from Valueref
			String eObjectId = aWDO4JSEmitter.getExportIDByObject(obj);
			Streamer.debug.Header().println(
				"inside encodeBegin() of UIDataGridRenderer.java rootId from emitter:" + eObjectId);
			dataGridEmitter.Init(
				this.elUtil.getModelName(),
				eObjectId,
				aWDO4JSEmitter.getReferenceName(rootBean, this.elUtil.getLastToken()),
				readOnly,
				allowColumnHide,
				scrollSize,
				width,
				pageSize.intValue(),
				showSelectionColumn == null ? false : showSelectionColumn.booleanValue(),
				selectionColName,
				multiSelection == null ? false : multiSelection.booleanValue(),
				selectionColAlignment,
				selectionColWidth == null ? "1" : selectionColWidth,
				rowFilter,
				styleClass,
				navBarPosition.intValue(),
				showRowIndex.booleanValue(),
				onhighlight,
				onselect,
				onunselect,
				onpage,
				onfilter,
				onsort,
				onselectall,
				onunselectall,
				onrowadd,
				onrowremove);
			PageContext Ctx = ODCContextHelper.getODCPageContext(context);
			
			Utils.getURLRewriter(Ctx,context);
			
			dataGridEmitter.Export(new PrintWriter(writer), Ctx);
		} catch (Exception e) {
			e.printStackTrace();
			throw new IOException();
		}
		String datagridName = null;
		datagridName = StringUtil.encodeString(dataGridEmitter.CONTROL_VAR + dataGridEmitter.getExportID());
		writer.startElement("script type=\"text/javascript\"", component);
		JSUtil.registerControl(context, component, datagridName);
		writer.endElement("script");
		String stateString = ((UIBrowserFramework) component).getStateString();
		if (!(stateString == null || stateString.equals(""))) {
			writer.startElement("script type=\"text/javascript\"", component);
			writer.write(datagridName + ".restoreUIState(\"" + stateString + "\");");
			writer.endElement("script");
		}
		Streamer.debug.Header().println("exit encodeEnd() of UIDataGridRenderer.java stateString:" + stateString);
	}
	/**
	 * @see javax.faces.component.UIComponentBase#decode(FacesContext)
	 */
	public void decode(FacesContext context, UIComponent component) {
		Streamer.trace.Header().println("enter decode() of UIDataGridRenderer.java");
		if (context == null) {
			throw new NullPointerException();
		}
		String odcDataGridState1 = null;
		Map requestParameterMap = context.getExternalContext().getRequestParameterMap();
		odcDataGridState1 = (String) requestParameterMap.get(component.getClientId(context) + "UIS");
		((UIDataGrid) component).setStateString(odcDataGridState1);
		Streamer.debug.Header().println(
			"in decode() of UIDataGridRenderer.java odcDataGridState1:" + odcDataGridState1);
		Streamer.trace.Header().println("exit decode() of UIDataGridRenderer.java");
	}
}
