/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//  
//OCO Source Materials
//  
//5724-E76 and 5724-E77
//  
// Copyright IBM Corp.  2004
//  
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////
package com.ibm.faces.bf.renderkit;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.el.PropertyNotFoundException;
import javax.faces.el.ValueBinding;
import javax.faces.webapp.UIComponentTag;
import javax.servlet.ServletRequest;

import com.ibm.faces.bf.component.UIBrowserFramework;
import com.ibm.faces.bf.component.UIClientBinder;
import com.ibm.faces.bf.component.UIClientData;
import com.ibm.odcb.jrender.misc.PathTokenizer;
import com.ibm.odcb.jrender.misc.TypesUtil;


/**
 * <p>
 * This class manages the client-side value binding for the <code>UIBrowserFramework</code>
 * that it is associated to. It provides utility method to evaluate the value binding expressions
 * and to resolve it to an object.
 * </p>
 * @author fenil
 */
public class ELUtil {

	private UIComponent component;
	private String originalBindingExpression;
	//private String wdoBindingExpression;
	private PathTokenizer ot;
	//private PathTokenizer nt;
	private final static String ODC_MODEL_MAP = "ODCModels";
	//private final static String ODC_OBJECT_MAP = "ODCObjects";
	private String rootBindingExpression;
	private String modelName;
	private Object rootObject;
	private Object lastObject;
	private ValueBinding vb;
	
	/**
	 * Constructor
	 * @param component the <code>UIComponent with which this <code>ELUtil</code> is associated.
	 */
	public ELUtil(UIComponent component) {
		this.component = component;
		if (component instanceof UIClientBinder || component instanceof UIClientData) {
			this.createExpressionsAndTokenizers(component.getValueBinding("value"));
		} else if (component instanceof UIBrowserFramework) {
			if (((UIBrowserFramework) component).getClientBinding() != null) {
				this.createExpressionsAndTokenizers(((UIBrowserFramework) component).getClientBinding());
			}
		}
	}
	
	/**
	 * Extracts the binding expression from the <code>ValueBinding</code> for the component
	 * and initializes the local variables.
	 * @param vb
	 */
	private void createExpressionsAndTokenizers(ValueBinding vb) {
		if (vb == null) {
			return;
		}
		this.vb = vb;
		String expressionString = vb.getExpressionString();
		if (expressionString != null && !expressionString.equals("") && UIComponentTag.isValueReference(expressionString)) {
			expressionString = PathTokenizer.getStrippedValueString(expressionString);
		} else {
			//throw an exception if it not a VBL
		}
		this.originalBindingExpression = new String(expressionString);
		this.ot = new PathTokenizer(this.originalBindingExpression);
		//this.wdoBindingExpression = new String(this.stripCodeBehind(expressionString));
		//this.nt = new PathTokenizer(this.wdoBindingExpression);
	}

	/**
	 * Strips the codebehind prefix from the vbl
	 * @param vbl the vbl expression from which the prefix is to be removed
	 * @return the stripped string
	 */
	/*private String stripCodeBehind(String vbl) {
		if (vbl == null || vbl.equals("")) {
			return vbl;
		}
		if (vbl.startsWith(BrowserFrameworkConstants.SERVER_CODE_PREFIX)) {
			int index = vbl.indexOf('.');
			vbl = vbl.substring(index + 1, vbl.length());
		}
		return vbl;
	}*/
	
	/**
	 * Returns the root object for the client-side model that this component is associated to.
	 * @return Object the root object
	 */
	public Object getRootObject() throws PropertyNotFoundException {
		//return getObject(ot.getRoot());
		//need to make a check for [0] in the root binding expression because of the way WDO's are handled in Studio
		if (this.rootObject == null) {
			//this call sets everything up
			getModelName();
			this.rootObject = getObject(this.rootBindingExpression);
		}

		return this.rootObject;
	}
	
	
	/**
	 * Resolves the whole expression used to the create the client-side binding for the associated
	 * component to an object.
	 * @return the resolved object
	 * @throws PropertyNotFoundException
	 */
	public Object resolveWholeExpressionToObject() throws PropertyNotFoundException {
		return getObject(this.originalBindingExpression);
	}

	/**
	 * <p>
	 * Resolves and returns the last object pointed to by the binding expression for the associated component.
	 * For example if the associated component is has binding expression like #{x.y.z} then this method
	 * will resolve #{x.y} to an object and return that.<br>
	 * Some special conditions apply:-<br>
	 * <ul>
	 * <li>The path to the root object of the model must be a substring of the path to the last object, else the
	 * root object for the model will be returned.
	 * <li>If the path to the last object only has one token i.e. it is something like #{x}, then #{x} will be 
	 * resolved to an object and returned.
	 * </ul> 
	 * </p>
	 * @return
	 * @throws PropertyNotFoundException
	 */
	public Object getLastObject() throws PropertyNotFoundException {
		/*if (ot.getPathToLastObject().indexOf(".") == -1
			&& ot.getPathToLastObject().startsWith(BrowserFrameworkConstants.SERVER_CODE_PREFIX)
			&& ot.getTokens().length == 2) {
			return getObject(ot.getPath());
		}*/
		//TODO check references to this method to see if the logic above is needed in a modified manner
		//return getObject(ot.getPathToLastObject());
		//TODO Ask LDH if there is another way to handle this or maybe this code can be moved to WDO4JSEmitter
		//Logic here :- If the whole expression resolves to a Kevin WDO then the last object better be the root of the WDO, irrespective of what the user puts into the EL expression
		/*Object obj = this.resolveWholeExpressionToObject();
		if(obj instanceof com.ibm.websphere.sdo.internal.DataListAccessBeanImpl){
			Object tempObj = getObject(ot.getPathToLastObject());
			if(tempObj instanceof com.ibm.websphere.sdo.internal.DataObjectAccessBeanImpl){
				return tempObj;
			}
			return ((com.ibm.websphere.sdo.internal.DataListAccessBeanImpl)obj).getDataGraph().getDataObject().getDataObject("root");
		}
		else{
			return getObject(ot.getPathToLastObject());
		}*/

		if (this.lastObject == null) {
			//this call sets up everything...not the best piece of programming
			this.getModelName();
			String lastObjPath = ot.getPathToLastObject();
			String tempPath = null;
			if(lastObjPath.endsWith("]")){
				tempPath = lastObjPath.substring(0, lastObjPath.lastIndexOf('['));
			}
			//if the root binding expression is not a substring of the path to the last object then the last object == root object. This special condition is needed
			//for the WDO's in studio
			if(lastObjPath.indexOf(this.rootBindingExpression) == -1){
				this.lastObject = this.rootObject;
			}
			else{
				this.lastObject = getObject(lastObjPath);
			}
		}
		return this.lastObject;

	}
	
	/**
	 * Returns the last token in the binding expression.
	 * @return the last token
	 */
	public String getLastToken() {
		return this.ot.getLastToken();
	}
	
	/**
	 * Returns the entire binding expression of the associated component
	 * @return the entire binding expression
	 */
	public String getOriginalBindingExpression() {
		return this.originalBindingExpression;
	}
	
	
	/**
	 * Returns the binding expression for the root object of the model that this component is associated to.
	 * @return the binding expreessiong for the root of the model.
	 */
	public String getRootBindingExpression(){
		if(this.rootBindingExpression == null){
			//same logic as the call to getLastObject
			this.getModelName();
		}
		return this.rootBindingExpression;
	}

	/**
	 * Utility method to resolve a binding expression to an object.
	 * @param vbl the binding expression
	 * @return the resolved object
	 * @throws PropertyNotFoundException
	 */
	public static Object getObject(String vbl) throws PropertyNotFoundException {
        ValueBinding vb = FacesContext.getCurrentInstance().getApplication().createValueBinding("#{" + vbl + "}");
        Object obj = vb.getValue(FacesContext.getCurrentInstance());
        if (obj == null) {
            //if obj is null then there are 2 possibilities.
            //1) The property pointed to by the EL expression does not exist at all
            //2) The property does exist but it is has a null value. In this case we just return the null
            Class type = vb.getType(FacesContext.getCurrentInstance());
            if(type == null){
                throw new PropertyNotFoundException("The EL expression = " + vbl + " did not resolve to an object.");
            }
        }
        return obj;
    }
	
	/**
	 * Returns the name of the model that this component is associated to.
	 * @return the model name
	 */
	public String getModelName() {
		if (this.modelName == null) {
			if (this.component instanceof UIClientData) {
				this.modelName = ((UIClientData) this.component).getModelName();
				if (this.modelName == null || this.modelName.equals("")) {
					this.modelName = createModelName(this.originalBindingExpression);
				}
				this.rootBindingExpression = this.originalBindingExpression;
				addModel(this.originalBindingExpression, modelName);
				//addObject(modelName, this.resolveWholeExpressionToObject());
			} else {
				Map models = getRequestMap(ODC_MODEL_MAP);
				Iterator iterator = models.keySet().iterator();
				String matchedKey = null;
				int index = -1;
				while (iterator.hasNext()) {
					String key = (String) iterator.next();
					int tempIndex = -1;
					//check if it starts with the stored vbl
					if (this.originalBindingExpression.indexOf(key) == 0) {
						tempIndex = key.length() - 1;
					}
					if (tempIndex > index) {
						index = tempIndex;
						matchedKey = key;
					}
				}
				if (matchedKey != null) {
					//cache the binding expression for the root object locally
					this.rootBindingExpression = matchedKey;
					//add logic to check for [0] at end of the matched key in original binding expression
					if(this.originalBindingExpression.length() >= (matchedKey.length()+3) &&  this.originalBindingExpression.charAt(matchedKey.length()) == '[' && this.originalBindingExpression.charAt(matchedKey.length()+2) == ']'){
						this.rootBindingExpression = this.originalBindingExpression.substring(0, matchedKey.length()+3);
					}
					this.modelName = (String) models.get(matchedKey);
				}
			}
		}
		//TODO need to throw an exception if the model is not found
		return this.modelName;
	}
	
	/**
	 * Utility method to check if the component is bound to a list.
	 * @return true if the component is bound to a list.
	 */
	public boolean isListBinding() {
	    if(this.vb == null){
	        return false;
	    }
	    //Fix for defect in EMF. Remove when it is fixed.
	    if(vb.getType(FacesContext.getCurrentInstance()) == null){
	    	return true;
	    }
		boolean isList = false;
		try {
			isList = TypesUtil.isGeneralList(vb.getType(FacesContext.getCurrentInstance()).getName());
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
		return isList;
	}
	
	/**
	 * Makes an entry in the request cache associating a binding expression with a model name
	 * @param beanName the binding expression
	 * @param modelName the name of the model
	 */
	public static void addModel(String beanName, String modelName) {
		getRequestMap(ODC_MODEL_MAP).put(beanName, modelName);
	}

	//public void addObject(String modelName, Object rootObject) {
	//	getRequestMap(ODC_OBJECT_MAP).put(modelName, rootObject);
	//}

	private static Map getRequestMap(String mapName) {
		Map map = null;
		ServletRequest request = (ServletRequest) FacesContext.getCurrentInstance().getExternalContext().getRequest();
		if (request.getAttribute(mapName) == null) {
			map = new HashMap();
			request.setAttribute(mapName, map);
		} else {
			map = (Map) request.getAttribute(mapName);
		}
		return map;
	}
	
	/**
	 * Generates a model name from a binding expression. Basically all &quot;.&quot; are replaced
	 * with &quot;_&quot; and &quot;[&quot; and &quot;]&quot; are stripped out.
	 * @param vbExpression
	 * @return
	 */
	public static String createModelName(String vbExpression) {
		PathTokenizer pt = new PathTokenizer(vbExpression);
		String[] tokens = pt.getTokens();
		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < tokens.length; i++) {
			if (i != 0) {
				//append _ between each token i.e. replacing the . with _
				sb.append('_');
			}
			if (tokens[i].indexOf('[') != -1) {
				//remove any [0] in the EL expression
				sb.append(tokens[i].substring(0, tokens[i].indexOf('[')));
				//add the 0 as part of the modelName
				sb.append(tokens[i].substring(tokens[i].indexOf('[') + 1, tokens[i].indexOf(']')));
			} else {
				sb.append(tokens[i]);
			}
		}
		return sb.toString();
	}
}
