/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//  
//OCO Source Materials
//  
//5724-E76 and 5724-E77
//  
// Copyright IBM Corp.  2003
//  
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////
package com.ibm.faces.bf.renderkit;

import java.io.IOException;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;

import com.ibm.odcb.jrender.misc.Streamer;
import com.ibm.odcb.jrender.emitters.GraphDrawEmitter;
import com.ibm.faces.bf.component.UIGraphDraw;
import com.ibm.faces.bf.component.UIGraphDrawData;
import javax.faces.convert.Converter;
import javax.faces.convert.DateTimeConverter;
import javax.faces.convert.NumberConverter;
import com.ibm.faces.converter.MaskConverter;
import com.ibm.faces.converter.NumberConverterEx;
import com.ibm.faces.renderkit.html_extended.HxClientRenderUtil;


/**
 * Renderer for {@link com.ibm.faces.bf.component.UIGraphDrawData UIGraphDrawData} component. 
 * <p> 
 * <b>Tags:</b> {@link com.ibm.faces.bf.taglib.GraphDrawDataTag}<br>
 * <b>Components:</b> {@link com.ibm.faces.bf.component.UIGraphDrawData}
 * </p> 
 * @see BrowserFrameworkRenderer 
 * @author Vidya Rao
 */
public class GraphDrawDataRenderer extends BrowserFrameworkRenderer {
	/*
	 * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeEnd(FacesContext context, UIComponent component) throws IOException {
		Streamer.trace.Header().println("enter encodeEnd() of GraphDrawDataRenderer.java");
		if (context == null) {
			throw new NullPointerException();
		}
		Boolean grouped = (Boolean) component.getAttributes().get(GROUPED);
		Streamer.trace.Header().println("grouped = " + grouped);

		UIGraphDraw myParent = (UIGraphDraw)component.getParent();
		GraphDrawEmitter emitter = (GraphDrawEmitter)myParent.getEmitter();

		emitter.setGrouped(grouped == null ? false : grouped.booleanValue());

		//Create the metadata string here and set it in the emitter
		String format = GraphDrawEmitter.NO_FORMAT;
		String converterCreationScript = "";
		
		//Get converter child tag and set the data format type and parameters in the emitter
		Converter converter = ((UIGraphDrawData)component).getConverter();
		Streamer.debug.Header().println("Converter = " + converter);
		if (converter != null) {
			if (converter instanceof DateTimeConverter) {
				DateTimeConverter dtc = (DateTimeConverter)converter;
				format = GraphDrawEmitter.DATETIME_FORMAT;				
				//converterCreationScript = HxClientRenderUtil.buildJsDateConverter(dtc, dtc.getLocale());
				converterCreationScript = HxClientRenderUtil.buildJsDateTimeConverter(dtc);
			} else if (converter instanceof NumberConverter) {
				NumberConverterEx nc = (NumberConverterEx)converter;
				format = GraphDrawEmitter.NUMBER_FORMAT;
				converterCreationScript = HxClientRenderUtil.buildJsNumberConverter(nc);
			} else if (converter instanceof MaskConverter) {
				Streamer.warning.Header().println("Mask formatting cannot be done on graphdraw data series points. Only number, date or no formatting are allowed. Defaulting to no formatting.");
				format = GraphDrawEmitter.NO_FORMAT;
			}
		} else {
			format = GraphDrawEmitter.NO_FORMAT;
		}

		emitter.setDataFormat(format);
		emitter.setDataConverterScript(converterCreationScript);
	
		Streamer.trace.Header().println("exit encodeEnd() of GraphDrawDataRenderer.java");
	}

	private static final String GROUPED = "grouped";	
}
