/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//  
//OCO Source Materials
//  
//5724-E76 and 5724-E77
//  
// Copyright IBM Corp.  2003
//  
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////
package com.ibm.faces.bf.renderkit;

import java.io.IOException;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.faces.convert.DateTimeConverter;
import javax.faces.convert.NumberConverter;

import com.ibm.faces.bf.component.UIGraphDraw;
import com.ibm.faces.bf.component.UIGraphDrawLabels;
import com.ibm.faces.converter.MaskConverter;
import com.ibm.faces.converter.NumberConverterEx;
import com.ibm.faces.renderkit.html_extended.HxClientRenderUtil;
import com.ibm.odcb.jrender.emitters.GraphDrawEmitter;
import com.ibm.odcb.jrender.misc.Streamer;

/**
 * Renderer for {@link com.ibm.faces.bf.component.UIGraphDrawData UIGraphDrawLabels} component. 
 * <p> 
 * <b>Tags:</b> {@link com.ibm.faces.bf.taglib.GraphDrawLabelsTag}<br>
 * <b>Components:</b> {@link com.ibm.faces.bf.component.UIGraphDrawLabels}
 * </p> 
 * @see BrowserFrameworkRenderer 
 * @author Vidya Rao
 */
public class GraphDrawLabelsRenderer extends BrowserFrameworkRenderer
{
	/*
	 * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeEnd(FacesContext context, UIComponent component) throws IOException {
		Streamer.trace.Header().println("enter encodeEnd() of GraphDrawLabelsRenderer.java");
		if (context == null) {
			throw new NullPointerException();
		}		
		super.encodeEnd(context, component);
		//Boolean showLabel = (Boolean)component.getAttributes().get(SHOW_LABEL);
		String attributeName = (String)component.getAttributes().get(ATTRIBUTE_NAME);
		Streamer.trace.Header().println(" attributeName = " + attributeName);

		UIGraphDraw myParent = (UIGraphDraw)component.getParent();
		if (myParent == null)
		{
			Streamer.error.Header().println("The Parent GraphDraw is null!");
			throw new IOException("The Parent GraphDraw is null!");
		}
		
		String eAttributeName = attributeName;
		Streamer.trace.Header().println("eAttributeName = " + eAttributeName);
		
		GraphDrawEmitter emitter = (GraphDrawEmitter)myParent.getEmitter();
		//emitter.setShowLabel(showLabel == null ? true : showLabel.booleanValue());
		emitter.setLabelAttributeName(eAttributeName);
		
		//Get converter child tag and set the label format type and parameters in the emitter
		String format = GraphDrawEmitter.NO_FORMAT;
		//String formatParameters = null;
		String converterCreationScript = "";
		
		Converter converter = ((UIGraphDrawLabels)component).getConverter();
		Streamer.debug.Header().println("Converter = " + converter); 

		if (converter != null) {
			if (converter instanceof DateTimeConverter) {
				DateTimeConverter dtc = (DateTimeConverter)converter;
				format = GraphDrawEmitter.DATETIME_FORMAT;				
				converterCreationScript = HxClientRenderUtil.buildJsDateTimeConverter(dtc);
			} else if (converter instanceof NumberConverter) {
				NumberConverterEx nc = (NumberConverterEx)converter;
				format = GraphDrawEmitter.NUMBER_FORMAT;
				converterCreationScript = HxClientRenderUtil.buildJsNumberConverter(nc);
			} else if (converter instanceof MaskConverter) {
				MaskConverter mc = (MaskConverter)converter;
				format = GraphDrawEmitter.MASK_FORMAT;
				converterCreationScript = "new MaskConverter('"
					+ mc.getMask()
					+ "')";
			}
		} else {
			format = GraphDrawEmitter.NO_FORMAT;
		}

		emitter.setLabelFormat(format);
		emitter.setLabelConverterScript(converterCreationScript);
		
		Streamer.trace.Header().println("exit encodeEnd() of GraphDrawLabelsRenderer.java");
	}
	
	//private static final String SHOW_LABEL = "showLabel";
	private static final String ATTRIBUTE_NAME = "attributeName";
}
