/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//
//OCO Source Materials
//
//5724-E76 and 5724-E77
//
// Copyright IBM Corp.  2003
//
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////
package com.ibm.faces.bf.renderkit;

import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Locale;

import javax.faces.component.UIColumn;
import javax.faces.component.UIComponent;
import javax.faces.component.UIViewRoot;
import javax.faces.component.html.HtmlOutputText;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.el.PropertyNotFoundException;
import javax.faces.el.ValueBinding;

import com.ibm.faces.bf.component.UIGraphDraw;
import com.ibm.faces.bf.component.html.HtmlGraphDraw;
import com.ibm.faces.bf.component.html.HtmlGraphDrawData;
import com.ibm.faces.bf.component.html.HtmlGraphDrawDataSeries;
import com.ibm.faces.bf.component.html.HtmlGraphDrawLabels;
import com.ibm.faces.bf.portal.JSFURLRewriter;
import com.ibm.faces.bf.util.InitializationUtil;
import com.ibm.faces.bf.util.ODCContextHelper;
import com.ibm.faces.component.IScriptContributor;
import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.component.html.HtmlDataXml;
import com.ibm.faces.component.html.HtmlScriptCollector;
import com.ibm.odcb.jrender.emitters.GraphDrawEmitter;
import com.ibm.odcb.jrender.emitters.WDO4JSEmitter;
import com.ibm.odcb.jrender.mediators.ExportException;
import com.ibm.odcb.jrender.mediators.PageContext;
import com.ibm.odcb.jrender.misc.Config;
import com.ibm.odcb.jrender.misc.Streamer;
import com.ibm.odcb.jrender.misc.StringUtil;
import com.ibm.odcb.jrender.misc.URLRewriter;
import com.ibm.odcb.jrender.misc.Utils;



/**
* Renderer for {@link com.ibm.faces.bf.component.UIGraphDraw UIGraphDraw} component. 
* <p> 
* <b>Tags:</b> {@link com.ibm.faces.bf.taglib.GraphDrawTag}<br>
* <b>Components:</b> {@link com.ibm.faces.bf.component.UIGraphDraw}
* </p> 
* @see BrowserFrameworkRenderer 
* @author Fenil Shah
* @author Vidya Rao
*/
public class GraphDrawRenderer extends BrowserFrameworkRenderer implements IScriptContributor {
	/*
	 * @see javax.faces.render.Renderer#encodeBegin(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeBegin(FacesContext context, UIComponent component) throws IOException {
		Streamer.trace.Header().println("enter encodeBegin of GraphDrawRenderer.java");
		//call super.encodeBegin to setup the elutil
		super.encodeBegin(context, component);
		String id = component.getId();
		//create the emitter
		GraphDrawEmitter emitter = new GraphDrawEmitter();
		((UIGraphDraw) component).setEmitter(emitter);
		Streamer.trace.Header().println("exit encodeBegin of GraphDrawRenderer.java");
		
		
		if(component.getValueBinding("value") != null)
		{
			component.getAttributes().put("isPageData",new Boolean(true));	
		}
			
		
		UIScriptCollector scriptCollector = UIScriptCollector.find(component);
		scriptCollector.register(this, component);
		
		
	}

	/*
	 * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeEnd(FacesContext context, UIComponent component) throws IOException {
		
		
		Streamer.trace.Header().println("enter encodeEnd of GraphDrawRenderer.java");
		super.encodeEnd(context, component);
		
		boolean isPageData = false;
		HashMap xmlDataMap = null;
		if (component.getValueBinding("value") != null) {
			isPageData = true;
		}
		
		if (!isPageData) {

			if (!this.elUtil.isListBinding()) {
				throw new IOException("The graphDraw " + component.getId()
						+ " must be bound to a list type");
			}
		} else {
			xmlDataMap = exportXMLData(context, component);
		}	
		ResponseWriter writer = context.getResponseWriter();
		String id = component.getId();
		String width = (String) component.getAttributes().get(WIDTH);
		String height = (String) component.getAttributes().get(HEIGHT);
		String firstChart = (String) component.getAttributes().get(FIRST_CHART);
		Boolean showPie = (Boolean) component.getAttributes().get(SHOW_PIE);
		Boolean showBar = (Boolean) component.getAttributes().get(SHOW_BAR);
		Boolean showLine = (Boolean) component.getAttributes().get(SHOW_LINE);

		String yAxisDivisions = (String) component.getAttributes().get(Y_AXIS_DIVISIONS);
		//Integer yAxisDivisions = (Integer) component.getAttributes().get(Y_AXIS_DIVISIONS);
		//if (yAxisDivisions.intValue() < 0)
		//	yAxisDivisions = new Integer(DEFAULT_Y_AXIS_DIVISIONS);

		String title = (String) component.getAttributes().get(TITLE);
		String xAxisTitle = (String) component.getAttributes().get(X_AXIS_TITLE);
		String yAxisTitle = (String) component.getAttributes().get(Y_AXIS_TITLE);

		Boolean showLegend = (Boolean) component.getAttributes().get(SHOW_LEGEND);
		Boolean showLabel = (Boolean)component.getAttributes().get(SHOW_LABEL);		
		Boolean splitYAxis = (Boolean) component.getAttributes().get(SPLIT_Y_AXIS);

		Boolean showHBar = (Boolean) component.getAttributes().get(SHOW_HORIZONTAL_BAR);
		Boolean showHLine = (Boolean) component.getAttributes().get(SHOW_HORIZONTAL_LINE);
		Boolean showVLine = (Boolean) component.getAttributes().get(SHOW_VERTICAL_LINE);

		String styleClass = (String) component.getAttributes().get(STYLE_CLASS);

		
		String eObjectId = null;
		String eReferenceName = null;
		String modelName = null;
		
		if (!isPageData)
		{	
		
			Object obj = null;
			Object rootBean = null;
			try {
				rootBean = this.elUtil.getRootObject();
				obj = this.elUtil.getLastObject();
			} catch (PropertyNotFoundException e1) {
				Streamer.error.printStackTrace(e1);
				return;
			}
			if (rootBean == null) {
				Streamer.error.Header().println("in encodeBegin() of GraphDrawRenderer.java--- rootBean is null");
				throw new NullPointerException();
			} else {
				Streamer.debug.Header().println("in encodeBegin() of GraphDrawRenderer.java--- rootBean is not null");
			} 

			
			try {
				WDO4JSEmitter jsEmitter = (WDO4JSEmitter) ODCContextHelper.getEmitter(context, this.elUtil.getModelName());
				eObjectId = jsEmitter.getExportIDByObject(obj);
				eReferenceName = jsEmitter.getReferenceName(rootBean, this.elUtil.getLastToken());
			} catch (Exception e) {
				Streamer.error.printStackTrace(e);
				throw new IOException();
			}
			modelName = this.elUtil.getModelName();
		}	
		else 
		{	
			String tmpModelName = (String)xmlDataMap.get("modelName");
			
			modelName = tmpModelName;
			
			eReferenceName = (String)xmlDataMap.get("propertyName");
		}
		
		
		try {
			Streamer.debug.Header().println(
				"Initializing GraphDrawEmitter with these parameters \n"
					+ "id = "
					+ id
					+ " ,"
					+ "modelName = "
					+ modelName
					+ " ,"
					+ "eObjectId = "
					+ eObjectId
					+ " ,"
					+ "eReferenceName = "
					+ eReferenceName
					+ " ,"
					+ "width = "
					+ width
					+ " ,"
					+ "height = "
					+ height
					+ " ,"
					+ "firstChart = "
					+ firstChart
					+ " ,"
					+ "showPie = "
					+ showPie
					+ " ,"
					+ "showBar = "
					+ showBar
					+ " ,"
					+ "showLine = "
					+ showLine
					+ " ,"
					+ "yAxisDivisions = "
					+ yAxisDivisions
					+ " ,"
					+ "title = "
					+ title
					+ " ,"
					+ "xAxisTitle = "
					+ xAxisTitle
					+ " ,"
					+ "yAxisTitle = "
					+ yAxisTitle
					+ " ,"
					+ "showLegend = "
					+ showLegend
					+ " ,"
					+ "showLabel = "
					+ showLabel
					+ " ,"
					+ "splitYAxis = "
					+ splitYAxis
					+ " ,"
					+ "showHBar = "
					+ showHBar
					+ " ,"
					+ "showHLine = "
					+ showHLine
					+ " ,"
					+ "showVLine = "
					+ showVLine
					+ " ,"
					+ "styleClass = "
					+ styleClass
					);

			//set default values
			boolean pie = ((showPie == null) ? false : showPie.booleanValue());
			boolean bar = ((showBar == null) ? false : showBar.booleanValue());
			boolean line = ((showLine == null) ? false : showLine.booleanValue());
			if (!pie && !bar && !line) {
				bar = true;
				firstChart = BAR_CHART;
			}
			if (firstChart == null) {
				if (bar)
					firstChart = BAR_CHART;
				else if (pie)
					firstChart = PIE_CHART;
				else if (line)
					firstChart = LINE_CHART;
				else
					firstChart = BAR_CHART;
			}
			if (firstChart.toLowerCase().equals(BAR_CHART.toLowerCase()) && bar == false) {
				bar = true;
			}
			if (firstChart.toLowerCase().equals(PIE_CHART.toLowerCase()) && pie == false) {
				pie = true;
			}
			if (firstChart.toLowerCase().equals(LINE_CHART.toLowerCase()) && line == false) {
				line = true;
			}

			GraphDrawEmitter emitter = (GraphDrawEmitter)((UIGraphDraw)component).getEmitter();
			//initialize the emitter
			
			if (isPageData)
			{
				emitter.setIsPageData(true);
			}
			
			emitter.Init(
				id,
				modelName,
				eObjectId,
				eReferenceName,
				width == null ? DEFAULT_WIDTH : width,
				height == null ? DEFAULT_HEIGHT : height,
				firstChart,
				pie,
				bar,
				line,
				yAxisDivisions == null ? "-1" : yAxisDivisions,
				title == null ? "" : title,
				xAxisTitle == null ? "" : xAxisTitle,
				yAxisTitle == null ? "" : yAxisTitle,
				showLegend == null ? true : showLegend.booleanValue(),
				splitYAxis == null ? false : splitYAxis.booleanValue(),
				showLabel == null ? true : showLabel.booleanValue(),
				showHBar == null ? false : showHBar.booleanValue(),
				showHLine == null ? false : showHLine.booleanValue(),
				showVLine == null ? false : showVLine.booleanValue(),
				styleClass				
				);
			
						
			PageContext pageContext = ODCContextHelper.getODCPageContext(context);
			
			boolean reinit = false;
			URLRewriter urlRewriter = Utils.getURLRewriter(pageContext, context);
			if (urlRewriter == null)
				reinit = true;
						
			if (isPageData)
			{	
				// I don't think this gets called by the original tree as it assumed alot of 
				// the initialization was taken care of by the client data which it depended on
				// since we are removing this dependency include it here.
					//////////////////need to check if URLRewriter exists when not using client data 
				//URLRewriter urlRewriter = (URLRewriter) pageContext.getContextVariable("URL_REWRITER");
				if (reinit) {
					//URLRewriterFactory factory = URLRewriterFactoryCreator.getURLRewriterFactory();
					//urlRewriter = factory.createURLRewriter(context);
					//urlRewriter = new JSFURLRewriter(context.getExternalContext());
					//pageContext.addContextVariable("URL_REWRITER", urlRewriter);
					//pageContext.addContextVariable("URL_PREFIX", URLPrinter._prefix);
					//pageContext.addContextVariable("URL_POSTFIX", URLPrinter._postfix);
					pageContext.addContextVariable(
						"DEBUG_MODE",
						Config.getInteger(Config.PROP_CLIENT_DEBUG_MODE) == 1 ? "YES" : "NO");
					Locale localeObj = context.getViewRoot().getLocale();
					String localeStr = null;
					if (localeObj != null) {
						localeStr = localeObj.toString();
	
					} else {
						localeStr = Config.getString(Config.PROP_CLIENT_LOCALE);
	
					}
					//((UITabbedPanel)component).setLocale(localeStr);
					pageContext.addContextVariable("CLIENT_LOCALE", localeStr);
					final int logLevel = Config.getInteger(Config.PROP_CLIENT_LOG_LEVEL);
					String logLevelString = String.valueOf(logLevel);
					pageContext.addContextVariable("LOG_LEVEL", logLevelString);
				}
			}
			
			emitter.exportDIV(writer, ODCContextHelper.getODCPageContext(context));
		}
		catch (Exception ex)
		{
			Streamer.error.Header().println(
					"ExportException Error while exporting the contents in the graph draw emitter inside GraphDrawRenderer with id = "
						+ component.getId());
				Streamer.error.Header().printStackTrace(ex);
				return;
		}
			
		Streamer.trace.Header().println("exit encodeEnd of GraphDrawRenderer.java");
	}
	
	
	
	
	public HashMap exportXMLData(FacesContext context, UIComponent component)
	{
		Object value = ((HtmlGraphDraw)component).getValue();
		HtmlDataXml xmlComp = new HtmlDataXml();
			
		xmlComp.setTransient(true);	
		xmlComp.setValue(value);
		xmlComp.setVar("varroot");
			

		xmlComp.setValueBinding("value",component.getValueBinding("value"));
		
		for (int i=0; i<component.getChildren().size(); i++)
		{
			
			String parentVar = (String)xmlComp.getVar();
						
			if (component.getChildren().get(i) instanceof HtmlGraphDrawData) {
				
				HtmlGraphDrawData dataComponent = (HtmlGraphDrawData)component.getChildren().get(i);
				 
				for (int j=0; j<dataComponent.getChildren().size(); j++)
				{
					if (dataComponent.getChildren().get(j) instanceof HtmlGraphDrawDataSeries) {
						HtmlGraphDrawDataSeries seriesDataComponent = (HtmlGraphDrawDataSeries)dataComponent.getChildren().get(j);
						
						String attributeName = seriesDataComponent.getAttributeName();
						ValueBinding nodeValue = context.getApplication().createValueBinding("#{"+parentVar+"."+attributeName+"}");							
						
						UIColumn thecolumn = new UIColumn();
						HtmlOutputText theOutputText = new HtmlOutputText();
										
						theOutputText.setValueBinding("value",(ValueBinding)nodeValue);
						thecolumn.getChildren().add(theOutputText);
						xmlComp.getChildren().add(thecolumn);	
					}
				}
				
			}
			if (component.getChildren().get(i) instanceof HtmlGraphDrawLabels) {
				
				HtmlGraphDrawLabels labelComponent = (HtmlGraphDrawLabels)component.getChildren().get(i);
				
				String attributeName = labelComponent.getAttributeName();
				ValueBinding nodeValue = context.getApplication().createValueBinding("#{"+parentVar+"."+attributeName+"}");							
				
				UIColumn thecolumn = new UIColumn();
				HtmlOutputText theOutputText = new HtmlOutputText();
								
				theOutputText.setValueBinding("value",(ValueBinding)nodeValue);
				thecolumn.getChildren().add(theOutputText);
				xmlComp.getChildren().add(thecolumn);
			}
			
		}
		
		UIViewRoot viewRoot = context.getViewRoot();
		getScriptCollector(viewRoot.getChildren().iterator()).getChildren().add(xmlComp);
		

		String modelName = xmlComp.getClientId(context).replaceAll(":","_");
		
		String fullClassName = value.getClass().toString();
		
		String propertyName = fullClassName.charAt(fullClassName.length()-1) == ';'?fullClassName.substring(fullClassName.lastIndexOf(".")+1,fullClassName.length()-1):fullClassName.substring(fullClassName.lastIndexOf(".")+1);
		
		// '$' is a restricted character in xml
		// sdo returns its classname with this character, 
		// so we need to get rid of it.
		
		
		if (propertyName.indexOf("$") != -1)
		{
			propertyName = propertyName.replace('$','.');
		}
		//System.out.println("PropertyName is: "+propertyName);
		HashMap returnInfo = new HashMap();
		
		returnInfo.put("modelName",modelName);
		returnInfo.put("propertyName",propertyName);
		
		xmlComp.getAttributes().put("javaClassName",propertyName);
		
		
		try {
			xmlComp.encodeBegin(context);
			xmlComp.encodeChildren(context);
			xmlComp.encodeEnd(context);
		
			
		}
			
		
		
		catch (Exception e)
		{
			System.out.println(e.toString());
		}
		
		
		
		
		return returnInfo;
	}

	private HtmlScriptCollector getScriptCollector(Iterator components)
	{
		
		while(components.hasNext())
		{
			UIComponent comp = (UIComponent)components.next();
			
			if ( comp instanceof HtmlScriptCollector )
			{
				return (HtmlScriptCollector)comp;
				
			}
			if ( !comp.getChildren().isEmpty() )
			{	
				return getScriptCollector(comp.getChildren().iterator());
			}
		}
		
		return null;
	}
	
	
	public void contributeScript(FacesContext context, UIComponent component) throws IOException {
		
		

		ResponseWriter writer = context.getResponseWriter();
	
		GraphDrawEmitter emitter = (GraphDrawEmitter)((UIGraphDraw)component).getEmitter();
			
		try {	
			
	
			PageContext pageContext = ODCContextHelper.getODCPageContext(context);
		
			writer.endElement("script");
			InitializationUtil.printIncludes(writer, context, pageContext);
			writer.startElement("script type=\"text/javascript\"", component);
		
			emitter.Export(writer, ODCContextHelper.getODCPageContext(context));
			String graphControl = StringUtil.encodeString(emitter.CONTROL_VAR + emitter.getExportID());

			JSUtil.registerControl(context, component, graphControl);

			
		}
		catch (ExportException e1) {
			Streamer.error.Header().println(
				"ExportException Error while exporting the contents in the graph draw emitter inside GraphDrawRenderer with id = "
					+ component.getId());
			Streamer.error.Header().printStackTrace(e1);
			return;
		}
	}
	
	private static final String WIDTH = "width";
	private static final String HEIGHT = "height";

	private static final String FIRST_CHART = "firstChart";
	private static final String SHOW_PIE = "pie";
	private static final String SHOW_BAR = "bar";
	private static final String SHOW_LINE = "line";

	private static final String Y_AXIS_DIVISIONS = "yaxisDivisions";

	private static final String TITLE = "title";
	private static final String X_AXIS_TITLE = "xaxisTitle";
	private static final String Y_AXIS_TITLE = "yaxisTitle";

	private static final String SHOW_LEGEND = "showLegend";
	private static final String SHOW_LABEL = "showLabel";	
	private static final String SPLIT_Y_AXIS = "splitYAxis";

	private static final String PIE_CHART = "Pie";
	private static final String BAR_CHART = "Bar";
	private static final String LINE_CHART = "Line";
	private static final String DEFAULT_WIDTH = "600";
	private static final String DEFAULT_HEIGHT = "400";
	private static final String DEFAULT_Y_AXIS_DIVISIONS = "auto";
	
	private static final String SHOW_HORIZONTAL_BAR = "showHGridlinesBar";
	private static final String SHOW_HORIZONTAL_LINE = "showHGridlinesLine";	
	private static final String SHOW_VERTICAL_LINE = "showVGridlinesLine";
	private static final String STYLE_CLASS = "styleClass";	
}
