/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//
//OCO Source Materials
//
//5724-E76 and 5724-E77
//
// Copyright IBM Corp.  2004
//
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////
package com.ibm.faces.bf.renderkit;

import java.io.IOException;
import java.util.List;

import javax.faces.component.UIComponent;
import javax.faces.component.UIForm;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;

import com.ibm.faces.renderkit.html_extended.HxClientRenderUtil;
import com.ibm.odcb.jrender.emitters.BaseEmitter;
import com.ibm.odcb.jrender.mediators.PageContext;
import com.ibm.odcb.jrender.misc.StringUtil;

/**
* Contains a set of utility methods for rendering JavaScript on the page.
* @author fenil
*/
public class JSUtil {

	
	/**
	 * Method builds string array suited for JavaScript consumption with strings double quoted.
	 * @param strings a <code>List</code> containing strings which need to be added to the JavaScript array.
	 * @return the JavaScript array
	 */
	public static String buildOriginalOptionStrings(List strings) {
		String _originalValueString = "";
		for (int i = 0; i < strings.size(); i++) {
			if (i != 0) {
				_originalValueString = _originalValueString.concat(",");
			}
			_originalValueString = _originalValueString.concat(buildQuotedString((String) strings.get(i)));
		}
		return _originalValueString;
	}
	/**
	 * Convenience function to double quote a string and return.
	 * @param tempStr	Input is a regular string: a
	 * @return			Output is a quoted string: "a"
	 */
	public static String buildQuotedString(String tempStr) {
		String s1 = tempStr;
		s1 = StringUtil.QuoteDoubleAndEscape(s1, true);
		s1 = (s1.substring(1, s1.length() - 2)).concat(s1.substring(s1.length() - 1));
		return s1;
	}
	
	/**
	 * Registers a <code>UIComponent</code> with the client-side JavaScript registry. This registry contains a mapping from
	 * the component id to the the actually JavaScript object representing the component on the page.
	 * @param context the current FacesContext instance
	 * @param component the component being registered
	 * @param control the JavaScript object representing the component on the page
	 * @throws IOException
	 */
	public static void registerControl(FacesContext context, UIComponent component, String control) throws IOException {
		//String viewId = (context.getViewRoot().getId() != null) ? context.getViewRoot().getClientId(context) : null;
		String clientViewId = context.getViewRoot().getClientId(context);
		ResponseWriter writer = context.getResponseWriter();

		//writer.startElement("script", component);
		/*if (viewId == null) {
			writer.write(
				"ODCRegistry.addElementInfo(null,'"
					+ component.getId()
					+ "','"
					+ component.getClientId(context)
					+ "',"
					+ control
					+ ");");
		} else {*/
			writer.write(
				"ODCRegistry.addElementInfo('"
					+ clientViewId
					+ "','"
					+ component.getId()
					+ "','"
					+ component.getClientId(context)
					+ "',"
					+control
					+ ");"
					+ " ODCRegistry.addDefaultClientViewID('"+ clientViewId+"');");
			
			//if its a tree add it to array in registry to allow the detaching of events
			if(component.getRendererType().equals("Tree"))
			{
				//Needed to detach events 
				writer.write(
					"ODCRegistry.addTreeInfo('"+component.getId()+"',' "
					+ component.getClientId(context)+ "');");
			}
			else
			{
				//Add property binders to registry so they can be unbound
				writer.write(
					"ODCRegistry.addBinderInfo('"
					+ component.getClientId(context)
					+ "',"
					+control
					+ ");");
			}
						
	}
	
	/**
	 * The event handlers for event fired by the Faces Client components are associated at the tag level with something like
	 * onhighlight=&quot;return func_1(this, event)&quot;. This method trims the event handler &quot;return func_1(this, event)&quot; to
	 * &quot;func_1&quot; as only name of the function is required by the controls to delegate to an event handler when the event is
	 * fired. 
	 * @param eventHandler the event handler string from the tag
	 * @return the trimmed event handler string
	 */
	public static String trimEventHandler(String eventHandler) {
		if (eventHandler == null || eventHandler.equals("")) {
			return eventHandler;
		}
		eventHandler = eventHandler.trim();
		if (eventHandler.startsWith("return ")) {
			eventHandler = eventHandler.substring(7, eventHandler.length());
		}
		if (eventHandler.indexOf("(") != -1) {
			return eventHandler.substring(0, eventHandler.indexOf("("));
		}
		return eventHandler;
	}
	/**
	 * @param component
	 * @param writer
	 * @param pageContext
	 * @param formId
	 * @throws IOException
	 */
	public static void registerDiffListener(UIComponent component, ResponseWriter writer, PageContext pageContext, String formId) throws IOException {
		//We only register onsubmit event once for every form
		if(BaseEmitter.TestGuard(pageContext, "FORM_AUTODIFF_EVENT" + formId) == false)
		{
		   writer.startElement("script type=\"text/javascript\"", component);
		   writer.write("if(isIE()){\n");
		   writer.write("document.getElementById('" + formId
		           + "').attachEvent('onsubmit', function(){ODCRegistry.saveAllToForm(document.getElementById('" + formId + "'));});\n");
		   writer.write("}\n");
		   writer.write("else{\n");
		   writer.write("document.getElementById('" + formId
		           + "').addEventListener('submit', function(){ODCRegistry.saveAllToForm(document.getElementById('" + formId
		           + "'));}, false);\n");
		   writer.write("}\n");
		   writer.endElement("script");
		}
	}

	
	/**
	 *  Gets the parent form for this component and generate an onsumbit fucntion
	 *  that will cause the ODCRegistry to get all the controls in teh form to 
	 *  generate the UIStateString when asubmit is done
	 * 
	 *  if the component isn't in a form, it will return a blank string 
	 * */
	public static String registerUIStateListenerJS(UIComponent component, FacesContext context) throws IOException {

		StringBuffer genString = null;
		
		UIComponent parent = component.getParent();
		
		while(parent != null && !(parent instanceof UIForm))
		{
			parent = parent.getParent();
		}
		
		if(parent != null)
		{
			String formid = parent.getClientId(context);
			
			genString = new StringBuffer();
			genString.append("\n"); //$NON-NLS-1$
			genString.append(HxClientRenderUtil.HX_VAR_NAME);
			genString.append(".addBehavior(\"");		 
			genString.append(formid);
			genString.append("\", \"onsubmit\", new ");
			genString.append(HxClientRenderUtil.HX_VAR_NAME);
			genString.append(".JSFBehaviorSubmitListener());");		
		}
		else
			return null; 
		
		return genString.toString(); 
	}
	
}
