/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//  
//OCO Source Materials
//  
//5724-E76 and 5724-E77
//  
// Copyright IBM Corp.  2003
//  
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////
package com.ibm.faces.bf.renderkit;

import java.io.IOException;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;

import com.ibm.faces.bf.component.UIBfPanel;
import com.ibm.faces.bf.component.UITabbedPanel;
import com.ibm.odcb.jrender.emitters.PanelEmitter;
import com.ibm.odcb.jrender.emitters.TabbedPanelEmitter;
import com.ibm.odcb.jrender.misc.Streamer;

/**
 * Renderer for {@link com.ibm.faces.bf.component.UIBfPanel UIBfPanel} component.
 * The renderer basically outputs a &lt;div&gt; tag on the page with the id supplied to the <code>bfPanel</code> tag. 
 * <p> 
 * <b>Tags:</b> {@link com.ibm.faces.bf.taglib.BfPanelTag}<br>
 * <b>Components:</b> {@link com.ibm.faces.bf.component.UIBfPanel}
 * </p> 
 * @see BrowserFrameworkRenderer  
 * @author Jianren Li
 */
public class PanelRenderer extends BrowserFrameworkRenderer {

	/**
	  * This method retrieves all the information for a panel and initializes PanelEmitter
	  * for every panel.
	  * @param context javax.faces.context.FacesContext
	  * @exception IOException - An exception is thrown if encountered an error.
	  **/
	public void encodeBegin(FacesContext context, UIComponent component) throws IOException {
		//Fenil Change 01/14/04 -> Bundle use removed
		Streamer.trace.Header().println("Entering encodeEnd() of PanelRenderer.java");
		String name = (String) component.getAttributes().get("name");
		
		String tabIcon = (String) component.getAttributes().get("tabIcon");
		String parentPane = (String) component.getAttributes().get("parentPane");
		
		String onEnter = JSUtil.trimEventHandler(((UIBfPanel) component).getOnenter());
		String onExit = JSUtil.trimEventHandler(((UIBfPanel) component).getOnexit());
		boolean showFinishCancelButton =
			((Boolean) component.getAttributes().get("showFinishCancelButton")).booleanValue();
		
		//RATLC01082051 - added properties, used to create PanelEmitter below...
		Object disable;
		boolean disableCancel = false;
		boolean disableFinish = false;
		
		boolean disableTab = false;
		
		if(null != (disable = component.getAttributes().get("disableCancel")))
			disableCancel = ((Boolean) disable).booleanValue();
		if(null != (disable = component.getAttributes().get("disableFinish"))) 
			disableFinish = ((Boolean) disable).booleanValue();
		if(null != (disable = component.getAttributes().get("disableTab"))) 
			disableTab = ((Boolean) disable).booleanValue();
		
		
		//Fenil Change...client id supplied to the panel emitter instead of component id
		UITabbedPanel tabbedPanel = (UITabbedPanel) component.getParent();
		String panelId = tabbedPanel.getClientId(context) + ":" + component.getId();
		PanelEmitter emitter =
			new PanelEmitter(name, panelId, onEnter, onExit, showFinishCancelButton, disableFinish, disableCancel, 
					tabIcon, parentPane, disableTab);
		((TabbedPanelEmitter) (tabbedPanel.getEmitter())).addPanel(emitter);
		ResponseWriter writer = context.getResponseWriter();
		// decfahey: added class=tabbedPanel_DIV to fix style problems with components within the panel
		writer.write("<div class=\"tabbedPanel_DIV\" id=\"" + emitter.getPanelID() + "\"" + " " + "style=\"visibility: hidden\"" + ">");
		Streamer.trace.Header().println("Exiting encodeEnd() of PanelRenderer.java");

	}
	/**
	  * This method just pushes out </div>.
	  * @param context javax.faces.context.FacesContext
	  * @exception IOException - An exception is thrown if exporting fails.
	  **/
	public void encodeEnd(FacesContext context, UIComponent component) throws IOException {
		ResponseWriter writer = context.getResponseWriter();
		writer.write("</div>");
	}

}
