/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//  
//OCO Source Materials
//  
//5724-E76 and 5724-E77
//  
// Copyright IBM Corp.  2003
//  
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////
package com.ibm.faces.bf.renderkit;

import java.io.IOException;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;

import javax.faces.component.UICommand;
import javax.faces.component.UIComponent;
import javax.faces.component.UIForm;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;

import com.ibm.faces.bf.component.UIBrowserFramework;
import com.ibm.faces.bf.component.UITabbedPanel;
import com.ibm.faces.bf.portal.JSFURLRewriter;
import com.ibm.faces.bf.portal.URLPrinter;
import com.ibm.faces.bf.util.InitializationUtil;
import com.ibm.faces.bf.util.ODCContextHelper;
import com.ibm.odcb.jrender.emitters.TabbedPanelEmitter;
import com.ibm.odcb.jrender.mediators.ExportException;
import com.ibm.odcb.jrender.mediators.PageContext;
import com.ibm.odcb.jrender.misc.Config;
import com.ibm.odcb.jrender.misc.Streamer;
import com.ibm.odcb.jrender.misc.StringUtil;
import com.ibm.odcb.jrender.misc.URLRewriter;

/**
 * Renderer for {@link com.ibm.faces.bf.component.UITabbedPanel UITabbedPanel} component.
 * The renderer aggregates all the {@link com.ibm.faces.bf.component.UIBfPanel UIBfPanel} child components
 * of the <code>UITabbedPanel</code> component and renders the <strong>Tabbed Panel</strong> as one control
 * on the page. 
 * <p> 
 * <b>Tags:</b> {@link com.ibm.faces.bf.taglib.TabbedPanelTag}<br>
 * <b>Components:</b> {@link com.ibm.faces.bf.component.UITabbedPanel}
 * </p> 
 * @see BrowserFrameworkRenderer  
 * @author Jianren Li
 */
public class TabbedPanelRenderer extends BrowserFrameworkRenderer {

	private static final String BACK_BUTTON = "back";
	private static final String NEXT_BUTTON = "next";
	private static final String FINISH_BUTTON = "finish";
	private static final String CANCEL_BUTTON = "cancel";
	private static final String[] FACETS = { BACK_BUTTON, NEXT_BUTTON, FINISH_BUTTON, CANCEL_BUTTON };

	/**
	  * This method retrieves all the information for a tabbedpanel and initializes TabbedPanelEmitter. 
	  * @param context javax.faces.context.FacesContext
	  * @exception IOException - An exception is thrown if encountered an error.
	  **/
	public void encodeBegin(FacesContext context, UIComponent component) throws IOException {
		Streamer.trace.Header().println("Entering encodeBegin() of TabbedPanelRenderer.java");
		TabbedPanelEmitter emitter = new TabbedPanelEmitter();
		emitter.setOverriddingId(component.getClientId(context));
		((UITabbedPanel) component).setEmitter(emitter);
		//Width and height are treated as Strings to supports specifying them in pixels or percentage
		String width = (String) component.getAttributes().get("width");
	
		String height = (String) component.getAttributes().get("height");
				
		String styleClass = (String) component.getAttributes().get("styleClass");
		String layout = (String) component.getAttributes().get("layout");
		String onFinish = JSUtil.trimEventHandler(((UITabbedPanel) component).getOnfinish());
		String onCancel = JSUtil.trimEventHandler(((UITabbedPanel) component).getOncancel());
		String onpanelenter = JSUtil.trimEventHandler(((UITabbedPanel) component).getOnpanelenter());
		String onpanelexit = JSUtil.trimEventHandler(((UITabbedPanel) component).getOnpanelexit());
		String oninitialpanelshow = JSUtil.trimEventHandler(((UITabbedPanel) component).getOninitialpanelshow());
		String initialPanelId = ((UITabbedPanel)component).getInitialPanelId();
		int slantActiveLeft = ((Integer) component.getAttributes().get("slantActiveLeft")).intValue();
		int slantActiveRight = ((Integer) component.getAttributes().get("slantActiveRight")).intValue();
		int slantInactiveLeft = ((Integer) component.getAttributes().get("slantInactiveLeft")).intValue();
		int slantInactiveRight = ((Integer) component.getAttributes().get("slantInactiveRight")).intValue();

		boolean showBackNextButton = ((Boolean) component.getAttributes().get("showBackNextButton")).booleanValue();
		boolean showTabs = ((Boolean) component.getAttributes().get("showTabs")).booleanValue();
		int numOfTabs = ((Integer) component.getAttributes().get("numOfTabs")).intValue();
		//boolean onFinishSubmit = ((Boolean) component.getAttributes().get("onFinishSubmit")).booleanValue();
		boolean variableTabLength = ((Boolean) component.getAttributes().get("variableTabLength")).booleanValue();
		emitter.Init(
			height,
			width,
			numOfTabs,
			onFinish,
			onCancel,
			onpanelenter,
			onpanelexit,
			oninitialpanelshow,
			initialPanelId,
			layout,
			showBackNextButton,
			showTabs,
			styleClass,
			variableTabLength,
			slantActiveLeft,
			slantActiveRight,
			slantInactiveLeft,
			slantInactiveRight);
		Streamer.trace.Header().println("Exiting encodeBegin() of TabbedPanelRenderer.java");

	}
	/**
	  * This method pushes out all JavaScript code for Browser Framework initialization if TabbedPanel is used
	  * standalone, the TabbesPanel itself, and also checks for the existence of two buttons (Finish and cancel) 
	  * needed for this tabbed panel.
	  * @param context javax.faces.context.FacesContext
	  * @exception IOException - An exception is thrown if exporting fails.
	  **/
	public void encodeEnd(FacesContext context, UIComponent component) throws IOException {
		Streamer.trace.Header().println("Entering encodeEnd() of TabbedPanelRenderer.java");
		ResponseWriter writer = context.getResponseWriter();
		PageContext pageContext = ODCContextHelper.getODCPageContext(context);
		//String finishButtonClientId = null;
		//String cancelButtonClientId = null;
		//String backButtonClientId = null;
		//String nextButtonClientId = null;
		//int buttonCount = 0;

		/*Iterator children = component.getChildren().iterator();
		while (children.hasNext()) {
			UIComponent child = (UIComponent) children.next();
			if (child instanceof UICommand
				&& (child.getClass().getName().equals("com.ibm.faces.component.html.HtmlCommandExButton"))) {
				//buttonCount++;
				if (child.getId().endsWith("_finish")) {
					finishButtonClientId = child.getClientId(context);
				} else if (child.getId().endsWith("_cancel")) {
					cancelButtonClientId = child.getClientId(context);
				}
				else if (child.getId().endsWith("_back")) {
					backButtonClientId = child.getClientId(context);
				}
				else if (child.getId().endsWith("_next")) {
					nextButtonClientId = child.getClientId(context);
				} else {
					Streamer.error.Header().println(
						"Buttons other than Finish or Cancel are found in TabbedPanelRenderer.java");
					//throw an exception
					throw new IOException();
				}
			}
		}*/
		//Fenil Change -> Buttons are now facets of the tabbed panel
		for (int i = 0; i < FACETS.length; i++) {
			UIComponent btn = component.getFacet(FACETS[i]);
			if (btn != null) {
				encodeRecursive(context, btn);
			}
		}
		UICommand finishButton = (UICommand) component.getFacet(FINISH_BUTTON);
		UICommand cancelButton = (UICommand) component.getFacet(CANCEL_BUTTON);
		UICommand nextButton = (UICommand) component.getFacet(NEXT_BUTTON);
		UICommand backButton = (UICommand) component.getFacet(BACK_BUTTON);
		//throw an excpetion if any of the facets are missing
		if (finishButton == null || cancelButton == null || nextButton == null || backButton == null) {
			Streamer.error.Header().println("Facet:finish, cancel, next or back missing in the tabbed panel.");
			throw new IOException();
		}
		TabbedPanelEmitter emitter = (TabbedPanelEmitter) ((UITabbedPanel) component).getEmitter();
		emitter.setFinishButtonId(finishButton.getClientId(context));
		emitter.setCancelButtonId(cancelButton.getClientId(context));
		emitter.setBackButtonId(backButton.getClientId(context));
		emitter.setNextButtonId(nextButton.getClientId(context));
		// Code inserted 9/5/03 by Kaushal Kurapati.
		// Code to check whether URLRewriter exists already in the PageContext.
		// if not, create URLRewriter through URLRewriterFactoryCreator.
		// Stick URLRewriter into PageContext. Now it will be available in emitter's Export method.
		// This code makes the TabPanel emitter indenependent of the InitializationEmitter.
		//	---begin code insert by Kaushal 9/5/03---
		URLRewriter urlRewriter = (URLRewriter) pageContext.getContextVariable("URL_REWRITER");
		if (urlRewriter == null) {
			//URLRewriterFactory factory = URLRewriterFactoryCreator.getURLRewriterFactory();
			//urlRewriter = factory.createURLRewriter(context);
			urlRewriter = new JSFURLRewriter(context.getExternalContext());
			pageContext.addContextVariable("URL_REWRITER", urlRewriter);
			//pageContext.addContextVariable("URL_PREFIX", URLPrinter._prefix);
			//pageContext.addContextVariable("URL_POSTFIX", URLPrinter._postfix);
			pageContext.addContextVariable(
				"DEBUG_MODE",
				Config.getInteger(Config.PROP_CLIENT_DEBUG_MODE) == 1 ? "YES" : "NO");
			Locale localeObj = context.getViewRoot().getLocale();
			String localeStr = null;
			if (localeObj != null) {
				localeStr = localeObj.toString();

			} else {
				localeStr = Config.getString(Config.PROP_CLIENT_LOCALE);

			}
			//((UITabbedPanel)component).setLocale(localeStr);
			pageContext.addContextVariable("CLIENT_LOCALE", localeStr);
			final int logLevel = Config.getInteger(Config.PROP_CLIENT_LOG_LEVEL);
			String logLevelString = String.valueOf(logLevel);
			pageContext.addContextVariable("LOG_LEVEL", logLevelString);
		}
		// ---end code insert by Kaushal 9/5/03---
		Streamer.trace.Header().println("Calling NLJSFilePrinter from UITabbedPanel...");
		InitializationUtil.printIncludes(writer, context, pageContext);
		URLPrinter.exportMiscellaneous(writer, urlRewriter);
		try {
			emitter.Export(writer, pageContext);
		} catch (ExportException e) {
			Streamer.error.Header().println("ExportException in exporting the tabbed panel controls");
			e.printStackTrace();
		} catch (IOException e) {
			Streamer.error.Header().println("IOException in exporting the tabbed panel controls");
			e.printStackTrace();
		}
		String tabId = StringUtil.encodeString( emitter.CONTROL_VAR + emitter.getExportID());
		//		writer.write("<SCRIPT language=\"JavaScript\">");
		//		writer.write("ODCPageControl.registerControl(\"" + component.getClientId(context) + "\"," + tabId + ");");
		//		writer.write("</SCRIPT>");
		writer.startElement("script type=\"text/javascript\"", component);
		JSUtil.registerControl(context, component, tabId);
		writer.endElement("script");
		String stateString = ((UIBrowserFramework) component).getStateString();
		if (!(stateString == null || stateString.equals(""))) {
			writer.startElement("script type=\"text/javascript\"", component);
			writer.write(tabId + ".restoreUIState(\"" + stateString + "\");");
			writer.endElement("script");
		}
		UIForm parentForm = BrowserFrameworkRenderer.findParentForm(component);
		if(parentForm != null){
			JSUtil.registerDiffListener(component, writer, pageContext, parentForm.getClientId(context));
		}
		
		Streamer.trace.Header().println("Exiting encodeEnd() of TabbedPanelRenderer.java");
	}

	/**
	  * This method receives a state string from a browser and store it inside
	  * the stateBean, which will be used when next time the TabbedPanel is rendered to restore
	  * its UI state.
	  * @param context javax.faces.context.FacesContext
	  * @exception IOException - An exception is thrown if encountered an error.
	  **/
	public void decode(FacesContext context, UIComponent component) {
		Streamer.trace.Header().println("Entering decode() of TabbedPanelRenderer.java");
		String tabControlState = null;
		Map requestParameterMap = context.getExternalContext().getRequestParameterMap();
		tabControlState = (String) requestParameterMap.get(component.getClientId(context) + "UIS");
		((UITabbedPanel) component).setStateString(tabControlState);
		Streamer.trace.Header().println("Exiting decode() of TabbedPanelRenderer.java");
	}

	private void encodeRecursive(FacesContext context, UIComponent component) throws IOException {
		component.encodeBegin(context);
		if (component.getRendersChildren()) {
			component.encodeChildren(context);
		} else {
			Iterator kids = component.getChildren().iterator();
			while (kids.hasNext()) {
				UIComponent kid = (UIComponent) kids.next();
				encodeRecursive(context, kid);
			}
		}
		component.encodeEnd(context);
	}
}
