/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//
//OCO Source Materials
//
//5724-E76 and 5724-E77
//
// Copyright IBM Corp.  2003
//
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////
package com.ibm.faces.bf.renderkit;

import java.io.IOException;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import com.ibm.faces.bf.component.UITreeColumnData;
import com.ibm.faces.bf.component.UITreeNodeAttr;
import com.ibm.odcb.jrender.emitters.ColDataEmitterHelper;
import com.ibm.odcb.jrender.emitters.NodeAttributeEmitterHelper;

/**
* Renderer for {@link com.ibm.faces.bf.component.UITreeTable UITreeColumnData} component.
* The renderer renders the individual column data for each node of the tree and adds them to the
* parent {@link com.ibm.faces.bf.component.UITree UITreeNodeAttr} component.
* <p> 
* <b>Tags:</b> {@link com.ibm.faces.bf.taglib.TreeColumnDataTag}<br>
* <b>Components:</b> {@link com.ibm.faces.bf.component.UITreeColumnData}
* </p> 
* @see BrowserFrameworkRenderer  
* @author Jianren Li
*/
public class TreeColumnDataRenderer extends BrowserFrameworkRenderer
{
		/**
		  * This method retrieves all the information for a tree nodes column data and initializes the 
		  * treeColumnDataEmitter for every column
		  * @param context javax.faces.context.FacesContext
		  * @exception IOException - An exception is thrown if encountered an error.
		  **/
		public void encodeEnd(FacesContext context, UIComponent component) throws IOException 
		{
			if (context == null) 
			{
				throw new NullPointerException();
			}
			super.encodeEnd(context, component);
			//Get attributeName for columnData
			UITreeColumnData comp = (UITreeColumnData) component;
			String attr = comp.getAttributeName();
						
			UITreeNodeAttr myParent = (UITreeNodeAttr) comp.getParent();
			//If this node has no UITree parent, fails it.
			if (myParent == null) 
			{
				throw new IOException();
			}
			
			NodeAttributeEmitterHelper nodeAttrEmitter = null;
			nodeAttrEmitter = (NodeAttributeEmitterHelper)myParent.getEmitter();
			
			try 
			{
				//Attach attributes to columnData and add columnData to node
				ColDataEmitterHelper col = new ColDataEmitterHelper();
				col.addAttributeName(attr);
				nodeAttrEmitter.addColumnData(col);
			} 
			catch (Exception e) 
			{
				e.printStackTrace();
				throw new IOException();
			}
		
		}
}
	


