/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//  
//OCO Source Materials
//  
//5724-E76 and 5724-E77
//  
// Copyright IBM Corp.  2003
//  
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////
package com.ibm.faces.bf.renderkit;

import java.io.IOException;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;

import com.ibm.faces.bf.component.UITreeColumn;
import com.ibm.faces.bf.component.UITreeTable;
import com.ibm.odcb.jrender.emitters.ColAttrEmitterHelper;
import com.ibm.odcb.jrender.emitters.TreeTableEmitter;

/**
 * Renderer for {@link com.ibm.faces.bf.component.UITreeTable UITreeColumn} component.
 * The renderer renders the individual columns of the Tree Table control and adds them to the
 * parent {@link com.ibm.faces.bf.component.UITree UITreeTable} component.
 * <p> 
 * <b>Tags:</b> {@link com.ibm.faces.bf.taglib.TreeColumnTag}<br>
 * <b>Components:</b> {@link com.ibm.faces.bf.component.UITreeColumn}
 * </p> 
 * @see BrowserFrameworkRenderer  
 * @author Jianren Li
 */
public class TreeColumnRenderer extends BrowserFrameworkRenderer{
		
	
		/**
		  * This method retrieves all the information for a tree column and initializes the treeColumnEmitter
		  * for every column
		  * @param context javax.faces.context.FacesContext
		  * @exception IOException - An exception is thrown if encountered an error.
		  **/
		public void encodeEnd(FacesContext context, UIComponent component) throws IOException 
		{
			if (context == null) {
				throw new NullPointerException();
			}
			super.encodeEnd(context, component);
			//Get attributes for the column
			String id = component.getId();
			UITreeColumn comp = (UITreeColumn) component;
			String width = comp.getWidth();
			String label= comp.getLabel();
			UITreeTable myParent = (UITreeTable) comp.getParent();
			//If this node has no UITree parent, fails it.
			if (myParent == null) {
				throw new IOException();
			}
			
			TreeTableEmitter treeTableEmitter = null;
			treeTableEmitter = (TreeTableEmitter)myParent.getEmitter();
			
			try {
				//Attach attributes to columnEmitter and then add the column to the tree table
				ColAttrEmitterHelper node = new ColAttrEmitterHelper();
				node.addId(id);
				node.addWidth(width);
				node.addLabel(label);
				treeTableEmitter.addColumn(node);
				
			} catch (Exception e) {
				e.printStackTrace();
				throw new IOException();
			}
		}

	}


