/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//  
//OCO Source Materials
//  
//5724-E76 and 5724-E77
//  
// Copyright IBM Corp.  2003
//  
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////
package com.ibm.faces.bf.renderkit;

import java.io.IOException;
import java.util.StringTokenizer;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;

import com.ibm.faces.bf.component.UITree;
import com.ibm.faces.bf.component.UITreeNodeAttr;
import com.ibm.odcb.jrender.emitters.NodeAttributeEmitterHelper;
import com.ibm.odcb.jrender.emitters.TreeViewEmitter;
import com.ibm.odcb.jrender.emitters.WDO4JSEmitter;
import com.ibm.odcb.jrender.mediators.gen.ecore.EClassMap;
import com.ibm.odcb.jrender.misc.Streamer;


/**
 * Renderer for {@link com.ibm.faces.bf.component.UITreeNodeAttr UITreeNodeAttr} component.
 * The renderer renders the individual nodes of the Tree View control and adds them to the
 * parent {@link com.ibm.faces.bf.component.UITree UITree} component.
 * <p> 
 * <b>Tags:</b> {@link com.ibm.faces.bf.taglib.TreeNodeAttrTag}<br>
 * <b>Components:</b> {@link com.ibm.faces.bf.component.UITreeNodeAttr}
 * </p> 
 * @see BrowserFrameworkRenderer  
 * @author Jianren Li
 */
public class TreeNodeAttrRenderer extends BrowserFrameworkRenderer 
{
	
	/**
	  * This method retrieves the data bean for the treeNode and initializes nodeAtrrEmitterHelper.
	  * @param context javax.faces.context.FacesContext
	  * @exception IOException - An exception is thrown if encountered an error.
	  **/
	public void encodeBegin(FacesContext context, UIComponent component) throws IOException 
	{
		if (context == null) {
			throw new NullPointerException();
		}
		
		//check for page data binding and process it if set
		processPageDataBinding(context,component);
		
		//call super.encodeBegin to setup elutil
		super.encodeBegin(context, component);
		String id = component.getId();
		UITreeNodeAttr comp = (UITreeNodeAttr) component;
		String className = comp.getClassName();
		try {
			boolean isClientData = true;			
			if (((Boolean)comp.getParent().getAttributes().get("isPageData")) != null) {	
				isClientData = false;
			}			
		
			NodeAttributeEmitterHelper nodeAttrHelper;
			
			if(isClientData)
			{
				EClassMap map = WDO4JSEmitter.getEClassMapByName(className, null);
				String eClassName = map.getExport();
				nodeAttrHelper = new NodeAttributeEmitterHelper(eClassName);
			}
			else
			{
				
				String tmpClassName = null;
				////#IF# SDO
//				if (className.indexOf("commonj.sdo.DataObject") != -1)
				////#END#
				////#IF# WDO
                                if (className.indexOf("com.ibm.etools.wdo") != -1)
                                ////#END#
				{
					
					tmpClassName = className.substring(className.lastIndexOf("`")+1,className.length()-1);
					if (tmpClassName.indexOf("DataGraphRoot") != -1)
					{	
					
					//	tmpClassName = HxClientRenderUtil.HX_VAR_NAME;
						
					}	 
				}	
				else {
					tmpClassName = (className.indexOf(".")==-1)?className:className.substring(className.lastIndexOf(".")+1);
				}
				nodeAttrHelper = new NodeAttributeEmitterHelper(tmpClassName);
			}
			
			((UITreeNodeAttr) component).setEmitter(nodeAttrHelper);
		}
		catch (Exception e) 
		{
			e.printStackTrace();
			throw new IOException();
		}
		
		Streamer.trace.Header().println("Exiting encodeBegin() of TreeNodeAttrRenderer.java");
		
	}

	
	/**
	  * This method retrieves all the information for a tree node and initializes NodeAttributeEmitterHelper
	  * for every node.
	  * @param context javax.faces.context.FacesContext
	  * @exception IOException - An exception is thrown if encountered an error.
	  **/
	public void encodeEnd(FacesContext context, UIComponent component) throws IOException {
		Streamer.trace.Header().println("Entering encodeEnd() of TreeNodeAttrRenderer.java");
		if (context == null) {
			throw new NullPointerException();
		}
		super.encodeEnd(context, component);
		String id = component.getId();
		
		UITreeNodeAttr comp = (UITreeNodeAttr) component;
		String attributeName = comp.getAttributeName();
		String nodeLabel = comp.getNodeLabel();
		String referenceName = comp.getReferenceName();
		String closeIcon = (String) comp.getAttributes().get("closeIcon");
		String openIcon = (String) comp.getAttributes().get("openIcon");
		//breff: Adding handler for showSystemIconFlag. Default value is true
		String showSystemIconFlag = (String) comp.getAttributes().get("showSystemIcon");
		Boolean showSystemIcon=Boolean.TRUE;
		if(showSystemIconFlag != null && showSystemIconFlag.equalsIgnoreCase("false"))
			showSystemIcon=Boolean.FALSE;
		
		String onhighlight  = JSUtil.trimEventHandler(comp.getOnhighlight());
		String onselect = JSUtil.trimEventHandler(comp.getOnselect());
		String onunselect = JSUtil.trimEventHandler(comp.getOnunselect());
		String onexpand = JSUtil.trimEventHandler(comp.getOnexpand());
		String oncollapse = JSUtil.trimEventHandler(comp.getOncollapse());
		String ondragenterover = JSUtil.trimEventHandler(comp.getOndragenterover());
		
		UITree myParent = (UITree) comp.getParent();
		//If this node has no UITree parent, fails it.
		if (myParent == null) {
			throw new IOException();
		}
		
		TreeViewEmitter treeViewEmitter = null;
		treeViewEmitter = (TreeViewEmitter)myParent.getEmitter();
		
		try {
			boolean isClientData = true;			
			if (((Boolean)comp.getParent().getAttributes().get("isPageData")) !=null ) {	
				isClientData = false;
			}			
			
			NodeAttributeEmitterHelper nodeAttrHelper = (NodeAttributeEmitterHelper)comp.getEmitter();
			
			if ( isClientData ) {
				
				String eAttributeName = attributeName;
				
				//We expect a referenceName with a format: "users,stocks"
				String eReferenceName = null;
				if (referenceName != null && !referenceName.equals("")) {
	
					StringTokenizer st = new StringTokenizer(referenceName, ",");
					//loop thru all individual references
					while (st.hasMoreTokens()) {
						String tok = st.nextToken();
	
						eReferenceName = tok;
						nodeAttrHelper.addStyleMap(eAttributeName, nodeLabel, eReferenceName);
					}
				}
				else{
					nodeAttrHelper.addStyleMap(eAttributeName, nodeLabel,  null);
				}
				// NOTE: Kaushal, Feb. 18'04; addIconMap gets an additional 5th parameter--boolean flag
				// to indicate whether to show system icon or not. 
				// For now its TRUE; with tooling support, it can be specified by users.
				// Runtime tooling side added by Breffni June 15 '05
				nodeAttrHelper.addIconMap(closeIcon, openIcon, null, null,showSystemIcon);
					
			}	
			else
			{
				//ReferenceName is expected in the format: "users,stocks"
				
			/*	if (referenceName != null && !referenceName.equals("")) {
					
					StringTokenizer st = new StringTokenizer(referenceName, ",");
					//loop thru all individual references
					while (st.hasMoreTokens()) {
						String tok = st.nextToken();
						nodeAttrHelper.addStyleMap(attributeName, nodeLabel, tok);
					}	
				}
		
				else{
					nodeAttrHelper.addStyleMap(attributeName, nodeLabel,  null);
				} */
			 
				nodeAttrHelper.addIconMap(closeIcon, openIcon, null, null,showSystemIcon);
			
			}
			
			
			if (onhighlight != null && !onhighlight.equals("")) {
				nodeAttrHelper.addEventHandler("onhighlight", onhighlight);
			}
			if (onselect != null && !onselect.equals("")) {
				nodeAttrHelper.addEventHandler("onselect", onselect);
			}
			if (onunselect != null && !onunselect.equals("")) {
				nodeAttrHelper.addEventHandler("onunselect", onunselect);
			}
			if (onexpand != null && !onexpand.equals("")) {
				nodeAttrHelper.addEventHandler("onexpand", onexpand);
			}
			if (oncollapse != null && !oncollapse.equals("")) {
				nodeAttrHelper.addEventHandler("oncollapse", oncollapse);
			}
			if (ondragenterover != null && !ondragenterover.equals("")) {
				nodeAttrHelper.addEventHandler("ondragenterover", ondragenterover);
			}
			nodeAttrHelper.addBehaviorMap();
			treeViewEmitter.addNode(nodeAttrHelper);   
	
		} catch (Exception e) {
			e.printStackTrace();
			throw new IOException();
		}
		Streamer.trace.Header().println("Exiting encodeEnd() of TreeNodeAttrRenderer.java");
	}

	
}
