/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//
//OCO Source Materials
//
//5724-E76 and 5724-E77
//
// Copyright IBM Corp.  2003
//
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////
package com.ibm.faces.bf.renderkit;

import java.io.IOException;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import com.ibm.faces.bf.component.UITree;
import com.ibm.faces.bf.component.UITreeTable;
import com.ibm.odcb.jrender.emitters.TreeTableEmitter;
import com.ibm.odcb.jrender.emitters.TreeViewEmitter;
import com.ibm.odcb.jrender.misc.Streamer;


/**
* Renderer for {@link com.ibm.faces.bf.component.UITreeTable UITreeColumn} component.
* The renderer renders 
* <p> 
* <b>Tags:</b> {@link com.ibm.faces.bf.taglib.TreeColumnTag}<br>
* <b>Components:</b> {@link com.ibm.faces.bf.component.UITreeColumn}
* </p> 
* @see BrowserFrameworkRenderer  
* @author Breffni Coffey
*/

public class TreeTableRenderer extends BrowserFrameworkRenderer
{
	/**
	  * This method initializes the TreeTableEmitter
	  * @param context javax.faces.context.FacesContext
	  * @param component javax.faces.component.UIComponent;
	  **/
	public void encodeBegin(FacesContext context, UIComponent component) throws IOException 
	{
		if (context == null) 
		{
			throw new NullPointerException();
		}
			
		//check for page data binding and process it if set
		processPageDataBinding(context,component);
		
		//call super.encodeBegin to setup elutil
		super.encodeBegin(context, component);
		String id = component.getId();
		TreeTableEmitter treeTableEmitter = new TreeTableEmitter();
		treeTableEmitter.setId(id);
		((UITreeTable) component).setEmitter(treeTableEmitter);
						
	}
	
	/**
	  * This method adds a treeTable to the tree
	  * @param context javax.faces.context.FacesContext
	  * @param component javax.faces.component.UIComponent;
	  * @exception IOException - An exception is thrown if encountered an error.
	  **/
	public void encodeEnd(FacesContext context, UIComponent component) throws IOException 
	{
		if (context == null) 
		{
			throw new NullPointerException();
		}
		super.encodeEnd(context, component);
		String id = component.getId();
		Streamer.debug.Header().println("in encodeEnd() of TreeTableRenderer.java getting id:" + id);
		UITreeTable comp = (UITreeTable) component;
	
		UITree myParent = (UITree) comp.getParent();
		//If this node has no UITree parent, fails it.
		if (myParent == null) 
		{
			throw new IOException();
		}
		
		TreeViewEmitter treeViewEmitter = null;
		treeViewEmitter = (TreeViewEmitter)myParent.getEmitter();
		
		try 
		{
			TreeTableEmitter treeTableEmitter = (TreeTableEmitter)comp.getEmitter();
			treeViewEmitter.addTable(treeTableEmitter);
		} 
		catch (Exception e) 
		{
			e.printStackTrace();
			throw new IOException();
		}
		Streamer.trace.Header().println("Exiting encodeEnd() of TreeNodeAttrRenderer.java");
	}
	
}
