/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//  
//OCO Source Materials
//  
//5724-E76 and 5724-E77
//  
// Copyright IBM Corp.  2003
//  
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////
package com.ibm.faces.bf.renderkit;

import java.io.IOException;
import java.io.PrintWriter;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;

import com.ibm.faces.bf.component.UIWebService;
import com.ibm.faces.bf.util.ODCContextHelper;
import com.ibm.odcb.jrender.emitters.WebServicesEmitter;
import com.ibm.odcb.jrender.mediators.PageContext;
import com.ibm.odcb.jrender.misc.Streamer;
import com.ibm.odcb.jrender.misc.StringUtil;

/**
 * Renderer for {@link com.ibm.faces.bf.component.UIWebService UIWebService} component.
 * The renderer aggregates the input paramters for the WebService call from the child {@link com.ibm.faces.bf.component.UIWsInput UIWsInput} components.
 * It also creates the bindings from the result of the WebService to the model on the page based on the child {@link com.ibm.faces.bf.component.UIWsOutput UIWsOuput} components.
 * <p> 
 * <b>Tags:</b> {@link com.ibm.faces.bf.taglib.WebServiceTag}<br>
 * <b>Components:</b> {@link com.ibm.faces.bf.component.UIWebService}
 * </p> 
 * @see BrowserFrameworkRenderer
 * @author Fenil Shah
 * @author Rajesh V Ramachandran
 */
public class WebServiceRenderer extends BrowserFrameworkRenderer {
	/* 
	 * @see javax.faces.render.Renderer#encodeBegin(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeBegin(FacesContext context, UIComponent component) throws IOException {
		if (context == null) {
			throw new NullPointerException();
		}
		super.encodeBegin(context, component);
		UIWebService ws = (UIWebService) component;
		WebServicesEmitter wsEmitter = new WebServicesEmitter();
		ws.setEmitter(wsEmitter);
		Streamer.trace.Header().println("exit encodeBegin() of WebServiceRenderer.java");

	}

	/* 
	 * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeEnd(FacesContext context, UIComponent component) throws IOException {
		Streamer.trace.Header().println("enter encodeEnd() of WebServiceRenderer.java");
		if (context == null) {
			throw new NullPointerException();
		}
		super.encodeEnd(context, component);
		UIWebService ws = (UIWebService) component;
		WebServicesEmitter wsEmitter = (WebServicesEmitter)ws.getEmitter();
		Streamer.trace.Header().println("enter encodeBegin() of WebServiceRenderer.java");

		String wsdlURL = ws.getWsdlURL();
		Streamer.debug.Header().println("in encodeBegin() of WebServiceRenderer.java getting wsdlURL: " + wsdlURL);

		String friendlyName = ws.getWsAlias();
		if(friendlyName == null || friendlyName.equals("")){
			friendlyName = component.getClientId(context);
		}
		Streamer.debug.Header().println("in encodeBegin() of WebServiceRenderer.java getting wsAlias: " + friendlyName);

		String webServiceOperation = ws.getWebServiceOperation();
		Streamer.debug.Header().println(
			"in encodeBegin() of WebServiceRenderer.java getting wsOperation: " + webServiceOperation);
		wsEmitter.setWsdlURL(wsdlURL);
		wsEmitter.setFriendlyName(friendlyName);
		wsEmitter.setWebServiceOperation(webServiceOperation);
		ResponseWriter writer = context.getResponseWriter();
		try {
			String id = component.getId();
			Streamer.debug.Header().println("in encodeEnd() of WebServiceRenderer.java getting component id: " + id);

			PageContext Ctx = ODCContextHelper.getODCPageContext(context);
			PrintWriter out = new PrintWriter(writer);

			wsEmitter.Export(out, Ctx);

			String wsName = null;

			wsName = StringUtil.encodeString(wsEmitter.ADAPTER_VAR + wsEmitter.getExportID());
			writer.startElement("script type=\"text/javascript\"", component);
			JSUtil.registerControl(context, component, wsName);
			writer.endElement("script");
		} catch (Exception e) {
			e.printStackTrace();
			throw new IOException();
		}
		Streamer.trace.Header().println("exit encodeBegin() of WebServiceRenderer.java");
	}
}
