/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//  
//OCO Source Materials
//  
//5724-E76 and 5724-E77
//  
// Copyright IBM Corp.  2003
//  
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////
package com.ibm.faces.bf.renderkit;

import java.io.IOException;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.el.PropertyNotFoundException;

import com.ibm.faces.bf.component.UIWebService;
import com.ibm.faces.bf.component.UIWsInput;
import com.ibm.faces.bf.util.InvalidKeyException;
import com.ibm.faces.bf.util.ODCContextHelper;
import com.ibm.odcb.jrender.emitters.WDO4JSEmitter;
import com.ibm.odcb.jrender.emitters.WebServicesEmitter;
import com.ibm.odcb.jrender.mediators.PageContext;
import com.ibm.odcb.jrender.misc.Streamer;

/**
 * Renderer for {@link com.ibm.faces.bf.component.UIWsInput UIWsInput} component.
 * The renderer renders one {@link com.ibm.faces.bf.component.UIWsInput} component, which is basically one
 * input parameter for the WebService call.
 * <p> 
 * <b>Tags:</b> {@link com.ibm.faces.bf.taglib.WsInputTag}<br>
 * <b>Components:</b> {@link com.ibm.faces.bf.component.UIWsInput}
 * </p> 
 * @see BrowserFrameworkRenderer
 * @author Fenil Shah
 * @author Rajesh V Ramachandran
 */
public class WsInputRenderer extends BrowserFrameworkRenderer {
	/* 
	 * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeEnd(FacesContext context, UIComponent component) throws IOException {
		Streamer.trace.Header().println("enter encodeEnd() of WsInputRenderer.java");
		if (context == null) {
			throw new NullPointerException();
		}
		super.encodeEnd(context, component);
		String id = component.getId();
		Streamer.debug.Header().println("in encodeEnd() of WsInputRenderer.java getting component id: " + id);
		String wsRespName = ((UIWsInput) component).getRequestElementName();
		String wsAttrName = ((UIWsInput) component).getAttributeName();

		WebServicesEmitter wsEmitter = null;
		UIWebService wsParent = (UIWebService) component.getParent();
		if (wsParent == null) {
			throw new IOException();
		}
		wsEmitter = (WebServicesEmitter)wsParent.getEmitter();
		if (this.elUtil.getOriginalBindingExpression() == null) {
			UIWsInput input = (UIWsInput) component;
			String literal = (String) input.getValue();
			wsEmitter.AddInputParameter(literal, wsRespName);
		} else {
			try {
				Object parentObject = this.elUtil.getLastObject();

				if (parentObject == null) {
					Streamer.debug.Header().println("in encodeEnd() of WsInputRenderer.java--- rootBean is null");
					throw new NullPointerException();
				}

				PageContext Ctx = ODCContextHelper.getODCPageContext(context);

				String modelName = (String) this.elUtil.getModelName();
				//String modelAttrName = this.elUtil.getLastToken();
				

				WDO4JSEmitter aWDO4JSEmitter = (WDO4JSEmitter) ODCContextHelper.getEmitter(context, modelName);
				String modelId = aWDO4JSEmitter.getExportIDByObject(parentObject);
				String modelAttrName = aWDO4JSEmitter.getReferenceName(this.elUtil.getRootObject(), this.elUtil.getLastToken());
				
				boolean useAsList = this.elUtil.isListBinding();				
				Streamer.debug.Header().println(
					"in encodeEnd() of WsInputRenderer.java getting rootId from emitter: " + modelId);
				wsEmitter.AddInputParameter(modelName,modelId, modelAttrName, wsRespName, wsAttrName,useAsList);

			} catch (PropertyNotFoundException ex) {
				Streamer.error.printStackTrace(ex);
				return;
			} catch (InvalidKeyException exx) {
				Streamer.error.printStackTrace(exx);
				return;
			} catch (Exception e) {
				Streamer.error.printStackTrace(e);
				throw new IOException();
			}
		}
		Streamer.trace.Header().println("exit encodeEnd() of WsInputRenderer.java");
	}
}
