/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//  
//OCO Source Materials
//  
//5724-E76 and 5724-E77
//  
// Copyright IBM Corp.  2003
//  
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////
package com.ibm.faces.bf.renderkit;

import java.io.IOException;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.el.PropertyNotFoundException;

import com.ibm.faces.bf.component.UIWebService;
import com.ibm.faces.bf.component.UIWsOutput;
import com.ibm.faces.bf.util.InvalidKeyException;
import com.ibm.faces.bf.util.ODCContextHelper;
import com.ibm.odcb.jrender.emitters.WDO4JSEmitter;
import com.ibm.odcb.jrender.emitters.WebServicesEmitter;
import com.ibm.odcb.jrender.mediators.PageContext;
import com.ibm.odcb.jrender.misc.Streamer;

/**
 * Renderer for {@link com.ibm.faces.bf.component.UIWsInput UIWsOutput} component.
 * The renderer renders one {@link com.ibm.faces.bf.component.UIWsOutput} component, which is basically one
 * mapping from the result of the WebService to some attribute or reference in the client-side model on the page.
 * <p> 
 * <b>Tags:</b> {@link com.ibm.faces.bf.taglib.WsOutputTag}<br>
 * <b>Components:</b> {@link com.ibm.faces.bf.component.UIWsOutput}
 * </p> 
 * @see BrowserFrameworkRenderer
 * @author Fenil Shah
 * @author Rajesh V Ramachandran
 */
public class WsOutputRenderer extends BrowserFrameworkRenderer {
	/*
	 * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeEnd(FacesContext context, UIComponent component) throws IOException {
		Streamer.trace.Header().println("enter encodeEnd() of WsOutputRenderer.java");

		if (context == null) {
			throw new NullPointerException();
		}
		super.encodeEnd(context, component);
		String id = component.getId();
		Streamer.debug.Header().println("in encodeEnd() of UIWsInput.java getting component id: " + id);
		String responseElementName = ((UIWsOutput) component).getResponseElementName();
		String wsAttrName = ((UIWsOutput) component).getAttributeName();
		Streamer.debug.Header().println(
			"in encodeEnd() of WsOutputRenderer.java getting responseElementName: " + wsAttrName);

		try {

			Object parentObject = this.elUtil.getLastObject();
			if (parentObject == null) {
				Streamer.debug.Header().println("in encodeEnd() of WsOutputRenderer.java--- rootBean is null");
				throw new NullPointerException();
			}

			PageContext Ctx = ODCContextHelper.getODCPageContext(context);

			WebServicesEmitter wsEmitter = null;
			UIWebService wsParent = (UIWebService) component.getParent();
			if (wsParent == null) {
				throw new IOException();
			}
			wsEmitter = (WebServicesEmitter)wsParent.getEmitter();

			String modelName = (String) this.elUtil.getModelName();
			Object rootBean = this.elUtil.getRootObject();

			//String modelAttrName = this.elUtil.getLastToken();
			WDO4JSEmitter aWDO4JSEmitter = (WDO4JSEmitter) ODCContextHelper.getEmitter(context, modelName);
			String modelAttrName = aWDO4JSEmitter.getReferenceName(rootBean, this.elUtil.getLastToken());

			String modelId = aWDO4JSEmitter.getExportIDByObject(parentObject);
			Streamer.debug.Header().println("in encodeEnd() of WsOutputRenderer.java getting modelId " + modelId);
			wsEmitter.AddOutputParameter(modelName,modelId, modelAttrName, responseElementName,wsAttrName);
		} catch (PropertyNotFoundException ex) {
			Streamer.error.printStackTrace(ex);
			throw new IOException();
		} catch (InvalidKeyException exx) {
			Streamer.error.printStackTrace(exx);
			return;
		} catch (Exception e) {
			Streamer.error.printStackTrace(e);
			throw new IOException();

		}
		Streamer.trace.Header().println("exit encodeEnd() of WsOutputRenderer.java");
	}
}
