/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/

package com.ibm.faces.bf.taglib;


import com.ibm.faces.util.StaticMethodBinding;
import com.ibm.faces.util.TagUtil;
import java.io.IOException;
import javax.faces.component.UIComponent;
import com.ibm.faces.bf.component.UIBehavior;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import javax.faces.event.ValueChangeEvent;
import javax.faces.convert.Converter;
import javax.faces.el.ValueBinding;
import javax.faces.el.MethodBinding;
import javax.faces.webapp.UIComponentTag;
import javax.faces.webapp.UIComponentBodyTag;
import javax.servlet.jsp.JspException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


public class BehaviorTag extends UIComponentTag {


    public static Log log = LogFactory.getLog(BehaviorTag.class);

    //
    // Instance Variables
    //

    private java.lang.String behaviorAction;
    private java.lang.String onActionFunction;
    private java.lang.String targetAction;

    private java.lang.String event;

    //
    // Setter Methods
    //

    public void setBehaviorAction(java.lang.String behaviorAction) {
        this.behaviorAction = behaviorAction;
    }

    public void setOnActionFunction(java.lang.String onActionFunction) {
        this.onActionFunction = onActionFunction;
    }

    public void setTargetAction(java.lang.String targetAction) {
        this.targetAction = targetAction;
    }

    public void setEvent(java.lang.String event) {
        this.event = event;
    }


    //
    // General Methods
    //

    public String getRendererType() { return "Behavior";  } //$NON-NLS-1$
    public String getComponentType() { return "com.ibm.faces.bf.HtmlBehavior"; } //$NON-NLS-1$

    protected void setProperties(UIComponent component) {
        super.setProperties(component);
        UIBehavior behaviorComponent = (UIBehavior)component;

        if (behaviorAction != null) {
            if (isValueReference(behaviorAction)) {
                ValueBinding vb = TagUtil.getValueBinding(behaviorAction);
                behaviorComponent.setValueBinding("behaviorAction", vb); //$NON-NLS-1$
            } else {
                behaviorComponent.setBehaviorAction(behaviorAction);
            }
        }
        if (onActionFunction != null) {
            if (isValueReference(onActionFunction)) {
                ValueBinding vb = TagUtil.getValueBinding(onActionFunction);
                behaviorComponent.setValueBinding("onActionFunction", vb); //$NON-NLS-1$
            } else {
                behaviorComponent.setOnActionFunction(onActionFunction);
            }
        }
        if (targetAction != null) {
            if (isValueReference(targetAction)) {
                ValueBinding vb = TagUtil.getValueBinding(targetAction);
                behaviorComponent.setValueBinding("targetAction", vb); //$NON-NLS-1$
            } else {
                behaviorComponent.setTargetAction(targetAction);
            }
        }
        if (event != null) {
            if (isValueReference(event)) {
                ValueBinding vb = TagUtil.getValueBinding(event);
                behaviorComponent.setValueBinding("event", vb); //$NON-NLS-1$
            } else {
                behaviorComponent.getAttributes().put("event", event); //$NON-NLS-1$
            }
        }
    }

    //
    // Methods From TagSupport
    //

    public int doStartTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doStartTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public int doEndTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doEndTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public String getDebugString() {
        String result = "id: "+this.getId()+" class: "+this.getClass().getName(); //$NON-NLS-1$ $NON-NLS-2$
        return result;
    }

}
