/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/

package com.ibm.faces.bf.taglib;


import com.ibm.faces.util.StaticMethodBinding;
import com.ibm.faces.util.TagUtil;
import java.io.IOException;
import javax.faces.component.UIComponent;
import com.ibm.faces.bf.component.UIBfPanel;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import javax.faces.event.ValueChangeEvent;
import javax.faces.convert.Converter;
import javax.faces.el.ValueBinding;
import javax.faces.el.MethodBinding;
import javax.faces.webapp.UIComponentTag;
import javax.faces.webapp.UIComponentBodyTag;
import javax.servlet.jsp.JspException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


public class BfPanelTag extends UIComponentTag {


    public static Log log = LogFactory.getLog(BfPanelTag.class);

    //
    // Instance Variables
    //

    private java.lang.String onenter;
    private java.lang.String onexit;

    private java.lang.String disableCancel;
    private java.lang.String disableFinish;
    private java.lang.String disableTab;
    private java.lang.String name;
    private java.lang.String parentPane;
    private java.lang.String showFinishCancelButton;
    private java.lang.String tabIcon;

    //
    // Setter Methods
    //

    public void setOnenter(java.lang.String onenter) {
        this.onenter = onenter;
    }

    public void setOnexit(java.lang.String onexit) {
        this.onexit = onexit;
    }

    public void setDisableCancel(java.lang.String disableCancel) {
        this.disableCancel = disableCancel;
    }

    public void setDisableFinish(java.lang.String disableFinish) {
        this.disableFinish = disableFinish;
    }

    public void setDisableTab(java.lang.String disableTab) {
        this.disableTab = disableTab;
    }

    public void setName(java.lang.String name) {
        this.name = name;
    }

    public void setParentPane(java.lang.String parentPane) {
        this.parentPane = parentPane;
    }

    public void setShowFinishCancelButton(java.lang.String showFinishCancelButton) {
        this.showFinishCancelButton = showFinishCancelButton;
    }

    public void setTabIcon(java.lang.String tabIcon) {
        this.tabIcon = tabIcon;
    }


    //
    // General Methods
    //

    public String getRendererType() { return "BfPanel";  } //$NON-NLS-1$
    public String getComponentType() { return "com.ibm.faces.bf.HtmlBfPanel"; } //$NON-NLS-1$

    protected void setProperties(UIComponent component) {
        super.setProperties(component);
        UIBfPanel bfpanelComponent = (UIBfPanel)component;

        if (onenter != null) {
            if (isValueReference(onenter)) {
                ValueBinding vb = TagUtil.getValueBinding(onenter);
                bfpanelComponent.setValueBinding("onenter", vb); //$NON-NLS-1$
            } else {
                bfpanelComponent.setOnenter(onenter);
            }
        }
        if (onexit != null) {
            if (isValueReference(onexit)) {
                ValueBinding vb = TagUtil.getValueBinding(onexit);
                bfpanelComponent.setValueBinding("onexit", vb); //$NON-NLS-1$
            } else {
                bfpanelComponent.setOnexit(onexit);
            }
        }
        if (disableCancel != null) {
            if (isValueReference(disableCancel)) {
                ValueBinding vb = TagUtil.getValueBinding(disableCancel);
                bfpanelComponent.setValueBinding("disableCancel", vb); //$NON-NLS-1$
            } else {
                boolean _disableCancel = new Boolean(disableCancel).booleanValue();
                bfpanelComponent.getAttributes().put("disableCancel", _disableCancel ? Boolean.TRUE : Boolean.FALSE); //$NON-NLS-1$
            }
        }
        if (disableFinish != null) {
            if (isValueReference(disableFinish)) {
                ValueBinding vb = TagUtil.getValueBinding(disableFinish);
                bfpanelComponent.setValueBinding("disableFinish", vb); //$NON-NLS-1$
            } else {
                boolean _disableFinish = new Boolean(disableFinish).booleanValue();
                bfpanelComponent.getAttributes().put("disableFinish", _disableFinish ? Boolean.TRUE : Boolean.FALSE); //$NON-NLS-1$
            }
        }
        if (disableTab != null) {
            if (isValueReference(disableTab)) {
                ValueBinding vb = TagUtil.getValueBinding(disableTab);
                bfpanelComponent.setValueBinding("disableTab", vb); //$NON-NLS-1$
            } else {
                boolean _disableTab = new Boolean(disableTab).booleanValue();
                bfpanelComponent.getAttributes().put("disableTab", _disableTab ? Boolean.TRUE : Boolean.FALSE); //$NON-NLS-1$
            }
        }
        if (name != null) {
            if (isValueReference(name)) {
                ValueBinding vb = TagUtil.getValueBinding(name);
                bfpanelComponent.setValueBinding("name", vb); //$NON-NLS-1$
            } else {
                bfpanelComponent.getAttributes().put("name", name); //$NON-NLS-1$
            }
        }
        if (parentPane != null) {
            if (isValueReference(parentPane)) {
                ValueBinding vb = TagUtil.getValueBinding(parentPane);
                bfpanelComponent.setValueBinding("parentPane", vb); //$NON-NLS-1$
            } else {
                bfpanelComponent.getAttributes().put("parentPane", parentPane); //$NON-NLS-1$
            }
        }
        if (showFinishCancelButton != null) {
            if (isValueReference(showFinishCancelButton)) {
                ValueBinding vb = TagUtil.getValueBinding(showFinishCancelButton);
                bfpanelComponent.setValueBinding("showFinishCancelButton", vb); //$NON-NLS-1$
            } else {
                boolean _showFinishCancelButton = new Boolean(showFinishCancelButton).booleanValue();
                bfpanelComponent.getAttributes().put("showFinishCancelButton", _showFinishCancelButton ? Boolean.TRUE : Boolean.FALSE); //$NON-NLS-1$
            }
        }
        if (tabIcon != null) {
            if (isValueReference(tabIcon)) {
                ValueBinding vb = TagUtil.getValueBinding(tabIcon);
                bfpanelComponent.setValueBinding("tabIcon", vb); //$NON-NLS-1$
            } else {
                bfpanelComponent.getAttributes().put("tabIcon", tabIcon); //$NON-NLS-1$
            }
        }
    }

    //
    // Methods From TagSupport
    //

    public int doStartTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doStartTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public int doEndTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doEndTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public String getDebugString() {
        String result = "id: "+this.getId()+" class: "+this.getClass().getName(); //$NON-NLS-1$ $NON-NLS-2$
        return result;
    }

}
