/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/

package com.ibm.faces.bf.taglib;


import com.ibm.faces.util.StaticMethodBinding;
import com.ibm.faces.util.TagUtil;
import java.io.IOException;
import javax.faces.component.UIComponent;
import com.ibm.faces.bf.component.UIClientData;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import javax.faces.event.ValueChangeEvent;
import javax.faces.convert.Converter;
import javax.faces.el.ValueBinding;
import javax.faces.el.MethodBinding;
import javax.faces.webapp.UIComponentTag;
import javax.faces.webapp.UIComponentBodyTag;
import javax.servlet.jsp.JspException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


public class ClientDataTag extends UIComponentTag {


    public static Log log = LogFactory.getLog(ClientDataTag.class);

    //
    // Instance Variables
    //

    private java.lang.String actionListener;
    private java.lang.String modelName;
    private java.lang.String value;


    //
    // Setter Methods
    //

    public void setActionListener(java.lang.String actionListener) {
        this.actionListener = actionListener;
    }

    public void setModelName(java.lang.String modelName) {
        this.modelName = modelName;
    }

    public void setValue(java.lang.String value) {
        this.value = value;
    }


    //
    // General Methods
    //

    public String getRendererType() { return "ClientData";  } //$NON-NLS-1$
    public String getComponentType() { return "com.ibm.faces.bf.HtmlClientData"; } //$NON-NLS-1$

    protected void setProperties(UIComponent component) {
        super.setProperties(component);
        UIClientData clientdataComponent = (UIClientData)component;

        if (actionListener != null) {
            if (isValueReference(actionListener)) {
                Class args[] = { ActionEvent.class };
                MethodBinding vb = FacesContext.getCurrentInstance().getApplication().createMethodBinding(actionListener, args);
                clientdataComponent.setActionListener(vb);
            } else {
              Object params [] = {actionListener};
              throw new javax.faces.FacesException();
            }
            }
        if (modelName != null) {
            if (isValueReference(modelName)) {
                ValueBinding vb = TagUtil.getValueBinding(modelName);
                clientdataComponent.setValueBinding("modelName", vb); //$NON-NLS-1$
            } else {
                clientdataComponent.setModelName(modelName);
            }
        }
        if (value != null) {
            if (isValueReference(value)) {
                ValueBinding vb = TagUtil.getValueBinding(value);
                clientdataComponent.setValueBinding("value", vb); //$NON-NLS-1$
            } else {
                clientdataComponent.setValue(value);
            }
        }
    }

    //
    // Methods From TagSupport
    //

    public int doStartTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doStartTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public int doEndTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doEndTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public String getDebugString() {
        String result = "id: "+this.getId()+" class: "+this.getClass().getName(); //$NON-NLS-1$ $NON-NLS-2$
        return result;
    }

}
