/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/

package com.ibm.faces.bf.taglib;


import com.ibm.faces.util.StaticMethodBinding;
import com.ibm.faces.util.TagUtil;
import java.io.IOException;
import javax.faces.component.UIComponent;
import com.ibm.faces.bf.component.UIGraphDrawData;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import javax.faces.event.ValueChangeEvent;
import javax.faces.convert.Converter;
import javax.faces.el.ValueBinding;
import javax.faces.el.MethodBinding;
import javax.faces.webapp.UIComponentTag;
import javax.faces.webapp.UIComponentBodyTag;
import javax.servlet.jsp.JspException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


public class GraphDrawDataTag extends UIComponentTag {


    public static Log log = LogFactory.getLog(GraphDrawDataTag.class);

    //
    // Instance Variables
    //


    private java.lang.String grouped;

    //
    // Setter Methods
    //

    public void setGrouped(java.lang.String grouped) {
        this.grouped = grouped;
    }


    //
    // General Methods
    //

    public String getRendererType() { return "GraphDrawData";  } //$NON-NLS-1$
    public String getComponentType() { return "com.ibm.faces.bf.HtmlGraphDrawData"; } //$NON-NLS-1$

    protected void setProperties(UIComponent component) {
        super.setProperties(component);
        UIGraphDrawData graphdrawdataComponent = (UIGraphDrawData)component;

        if (grouped != null) {
            if (isValueReference(grouped)) {
                ValueBinding vb = TagUtil.getValueBinding(grouped);
                graphdrawdataComponent.setValueBinding("grouped", vb); //$NON-NLS-1$
            } else {
                boolean _grouped = new Boolean(grouped).booleanValue();
                graphdrawdataComponent.getAttributes().put("grouped", _grouped ? Boolean.TRUE : Boolean.FALSE); //$NON-NLS-1$
            }
        }
    }

    //
    // Methods From TagSupport
    //

    public int doStartTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doStartTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public int doEndTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doEndTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public String getDebugString() {
        String result = "id: "+this.getId()+" class: "+this.getClass().getName(); //$NON-NLS-1$ $NON-NLS-2$
        return result;
    }

}
