/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/

package com.ibm.faces.bf.taglib;


import com.ibm.faces.util.StaticMethodBinding;
import com.ibm.faces.util.TagUtil;
import java.io.IOException;
import javax.faces.component.UIComponent;
import com.ibm.faces.bf.component.UIWebService;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import javax.faces.event.ValueChangeEvent;
import javax.faces.convert.Converter;
import javax.faces.el.ValueBinding;
import javax.faces.el.MethodBinding;
import javax.faces.webapp.UIComponentTag;
import javax.faces.webapp.UIComponentBodyTag;
import javax.servlet.jsp.JspException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


public class WebServiceTag extends UIComponentTag {


    public static Log log = LogFactory.getLog(WebServiceTag.class);

    //
    // Instance Variables
    //

    private java.lang.String webServiceOperation;
    private java.lang.String wsAlias;
    private java.lang.String wsdlURL;


    //
    // Setter Methods
    //

    public void setWebServiceOperation(java.lang.String webServiceOperation) {
        this.webServiceOperation = webServiceOperation;
    }

    public void setWsAlias(java.lang.String wsAlias) {
        this.wsAlias = wsAlias;
    }

    public void setWsdlURL(java.lang.String wsdlURL) {
        this.wsdlURL = wsdlURL;
    }


    //
    // General Methods
    //

    public String getRendererType() { return "WebService";  } //$NON-NLS-1$
    public String getComponentType() { return "com.ibm.faces.bf.HtmlWebService"; } //$NON-NLS-1$

    protected void setProperties(UIComponent component) {
        super.setProperties(component);
        UIWebService webserviceComponent = (UIWebService)component;

        if (webServiceOperation != null) {
            if (isValueReference(webServiceOperation)) {
                ValueBinding vb = TagUtil.getValueBinding(webServiceOperation);
                webserviceComponent.setValueBinding("webServiceOperation", vb); //$NON-NLS-1$
            } else {
                webserviceComponent.setWebServiceOperation(webServiceOperation);
            }
        }
        if (wsAlias != null) {
            if (isValueReference(wsAlias)) {
                ValueBinding vb = TagUtil.getValueBinding(wsAlias);
                webserviceComponent.setValueBinding("wsAlias", vb); //$NON-NLS-1$
            } else {
                webserviceComponent.setWsAlias(wsAlias);
            }
        }
        if (wsdlURL != null) {
            if (isValueReference(wsdlURL)) {
                ValueBinding vb = TagUtil.getValueBinding(wsdlURL);
                webserviceComponent.setValueBinding("wsdlURL", vb); //$NON-NLS-1$
            } else {
                webserviceComponent.setWsdlURL(wsdlURL);
            }
        }
    }

    //
    // Methods From TagSupport
    //

    public int doStartTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doStartTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public int doEndTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doEndTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public String getDebugString() {
        String result = "id: "+this.getId()+" class: "+this.getClass().getName(); //$NON-NLS-1$ $NON-NLS-2$
        return result;
    }

}
