/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//
//OCO Source Materials
//
//5724-E76 and 5724-E77
//
// Copyright IBM Corp.  2003
//
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////
package com.ibm.faces.bf.util;
import java.util.HashMap;

import javax.faces.context.FacesContext;
import javax.faces.el.PropertyNotFoundException;
import javax.servlet.http.HttpServletRequest;

import com.ibm.odcb.jrender.emitters.BaseEmitter;
import com.ibm.odcb.jrender.mediators.PageContext;
import com.ibm.odcb.jrender.misc.Config;
import com.ibm.odcb.jrender.misc.Streamer;
import com.ibm.odcb.jrender.misc.StringUtil;
/**
 * A utility class used to load up beans which have been created from the faces-config.xml
 * and also to retrieve objects cached into the ServletRequest. 
 * @author: Jianren Li
 *
 **/
public class ODCContextHelper
{
	private static String emitterHashMapKey = "odcEmitterHashMap";
	private static String odcPageContextKey = "odcPageContext";
	/** This static method will help retrieve an Browser Framework emitter object which is stored
	 * inside a Hashmap object thru HttpServletRequest, so this object can be shared by multiple tag renderers
	 * within a single HttpServletRequest.
	 * @param aFacesContext javax.faces.context.FacesContext - The FacesContext object from a JSF tag renderer.
	 * @param crtlName java.lang.String - The control name passed in through JSF tag as the key to retrieve the emitter.
	 * @return BaseEmitter com.ibm.odcb.jrender.emitters.BaseEmitter - The emitter instance
	 * @exception  InvalidKeyException If no object is found.
	 **/
	public static BaseEmitter getEmitter(
		FacesContext aFacesContext,
		String crtlName)
		throws InvalidKeyException
	{
		BaseEmitter anEmitter = null;
		HashMap aHashMap = getEmitterHashmap(aFacesContext);
		Object anEmitterObj = aHashMap.get(crtlName);
		//We are supposed to have this emitter object.  If null, nullpointerexception
		if (anEmitterObj == null)
		{
			throw new InvalidKeyException(
				"No emitter found with key = " + crtlName);
		}
		else
		{
			anEmitter = (BaseEmitter) anEmitterObj;
		}
		return anEmitter;
	}
	/** This static method will help set a Browser Framework emitter object which is stored
	 * inside a Hashmap object thru HttpServletRequest, so this object can be shared by multiple tag renderers
	 * within a single HttpServletRequest.
	 * @param aFacesContext javax.faces.context.FacesContext - The FacesContext object from a JSF tag renderer.
	 * @param crtlName java.lang.String - The control name passed in through JSF tag as the key to retrieve the emitter.
	 * @param newEmitter com.ibm.odcb.jrender.emitters.BaseEmitter - The emitter instance to be stored and used later.
	 **/
	public static void setEmitter(
		FacesContext aFacesContext,
		String crtlName,
		BaseEmitter newEmitter)
	{
		HashMap aHashMap = getEmitterHashmap(aFacesContext);
		aHashMap.put(crtlName, newEmitter);
	}
	/** This private static method will help get a HashMap object which is used to store
	 * emitter objects, so we can retrieve an emitter object referenced by a later tag renderer
	 * within a single request.
	 * @param aFacesContext javax.faces.context.FacesContext - a FacesContext object from a JSF tag renderer
	 * @return HashMap
	 **/
	private static HashMap getEmitterHashmap(FacesContext aFacesContext)
	{
		HashMap aHashMap = null;
		HttpServletRequest request =
			(HttpServletRequest) aFacesContext
				.getExternalContext()
				.getRequest();
		Object aHashMapObj = request.getAttribute(emitterHashMapKey);
		if (aHashMapObj != null)
		{
			aHashMap = (HashMap) aHashMapObj;
		}
		else
		{
			aHashMap = new HashMap(10);
			request.setAttribute(emitterHashMapKey, aHashMap);
		}
		return aHashMap;
	}
	/** This static method will help get a Browser Framework PageContext object which is stored
	 * inside HttpServletRequest, so this object can be shared by multiple tag renderers
	 * within a single HttpServletRequest.  In portal environment, portal internal API is used to
	 * get the HttpServletRequest object which is shared by all portlets on a page.
	 * @param aFacesContext javax.faces.context.FacesContext - a FacesContext object from a JSF tag renderer
	 * @return PageContext
	 **/
	public static PageContext getODCPageContext(FacesContext aFacesContext)
	{
		
		HttpServletRequest servReq = null;
		String httpReqClassName = Config.getString(Config.PROP_REQUEST_CLASS);
		try
		{
			HttpRequestRetrieverInterface servReqI =
				(HttpRequestRetrieverInterface) Class
					.forName(httpReqClassName)
					.newInstance();
			servReq = servReqI.getHttpServletRequest(aFacesContext);
		}
		catch (InstantiationException e1)
		{
			Streamer.error.Header().println(
				"Instantiation Exception while creating dynamic HttpRequestRetrieverInterface");
			e1.printStackTrace();
		}
		catch (IllegalAccessException e1)
		{
			Streamer.error.Header().println(
				"IllegalAccessException while creating dynamic HttpRequestRetrieverInterface");
			e1.printStackTrace();
		}
		catch (ClassNotFoundException e1)
		{
			Streamer.error.Header().println(
				"ClassNotFoundException while creating dynamic HttpRequestRetrieverInterface");
			e1.printStackTrace();
		}
		String key = StringUtil.encodeString(odcPageContextKey);
		PageContext pc = (PageContext) servReq.getAttribute(key);
		if (pc != null)
		{
			return pc;
		}
		else
		{
			PageContext newPC = new PageContext();
			servReq.setAttribute(key, newPC);
			return newPC;
		}
		// ---- end code addition by Kaushal 10/7/2003 --------
	}
	/**
	 * This static method is used to get hold of a managed bean created using faces-config.xml using the valueRef from
	 * the JSF tags.
	 * @param valueRef java.lang.String - the id used to identify the object to be retrieved
	 * @param context the FacesContext javax.faces.context.FacesContext - from which the object is to be retrieved
	 * @return Object - The object resolved by using valueRef
	 * @exception InvalidValueRefException - An exception will be thrown if the valueRef can not be resolved.
	 **/
	public static Object getObjectByReference(
		String value,
		FacesContext context)
		throws PropertyNotFoundException
	{
		//Object obj = (Util.getValueBinding(value)).getValue(context);
		//Fenil Change.. dependency on jsf-ri Util class removed
		Object obj = FacesContext.getCurrentInstance().getApplication().createValueBinding("#{"+value+"}").getValue(context);
		if (obj == null)
		{
			throw new PropertyNotFoundException(
				"The EL expression = "
					+ value
					+ " did not resolve to an object.");
		}
		return obj;
	}
}
