/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2003, 2006 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.odc.jsf.components.renderer.rte;

import java.io.IOException;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import javax.faces.FacesException;
import javax.faces.component.UIComponent;
import javax.faces.component.UIInput;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.render.Renderer;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.PageContext;

import com.ibm.faces.util.ContextParams;
import com.ibm.faces.util.LifecycleUtil;
import com.ibm.faces.util.StringUtil;
import com.ibm.odc.jsf.components.components.rte.UIRichTextEditor;


/**
 * @author wangs
 */
public class RichTextRenderer extends Renderer {
	
	//resource bundle
    
    private static final String DEFAULT_BUNDLE_NAME = "com.ibm.odc.jsf.components.renderer.rte.settings"; //$NON-NLS-1$
    private ResourceBundle resourceBundle = null; //set up the first time its referenced (from getresourceBundle)
	
	
	// rte default properties
	private String m_name;
	private String m_jspContext;
	private String m_jsp;
	private String m_imageDirectory = null;
	//private String m_locale = "en_US";
	private String m_width;
	private String m_height;
	private String m_cssPath;
    // starter jsp
    private String m_starterJsp;
	
	private static final String MODE_EDIT = "edit"; //$NON-NLS-1$
	private static final String MODE_VIEW = "view"; //$NON-NLS-1$

	private final String readonly ="readonly"; //$NON-NLS-1$ // for use by designer
	private final String readOnly ="readOnly"; //$NON-NLS-1$ // this is the one used by the tag

	private static final String RENDER_TYPE = "RichTextEditor"; //$NON-NLS-1$
	private static final String COMPONENT = com.ibm.odc.jsf.components.components.rte.UIRichTextEditor.TYPE;

	private String content_field = null;
	private static boolean debug = false;
	private String result=null; //not needed, referenced only once in one method.
	
	//TODO this doesn't seem to be ever referenced, can it be removed?
	private String JAVASCRIPT_VAR = null;
	//TODO this doesn't seem to be ever referenced, can it be removed?
	private String EDITOR_ELEMENT = null;
    
    private String starterJsp = null; //TODO this is not needed
    private String renderType = null; //TODO this is not needed
    private String componentType = null; //TODO this is not needed

	
	
	public void decode(FacesContext context, UIComponent component) {
		if (debug)
			System.out.println("rteRender: in decode");  //$NON-NLS-1$

		if (context == null || component == null) {
			//throw new NullPointerException(Util.getExceptionMessage(Util.NULL_PARAMETERS_ERROR_MESSAGE_ID));
			throw new NullPointerException();
		}

		content_field = getContentFieldName(context, component);

		/**
		 * read the raw text from the hidden contents field
		 */
		String sEditorContents = (String) context.getExternalContext().getRequestParameterMap().get(content_field);

		if (sEditorContents != null) {
			if (component instanceof UIRichTextEditor) {
				((UIRichTextEditor) component).setSubmittedValue(sEditorContents);
			}
		}

		//Fenil Change...this is not needed now
		//((UIRichTextEditor) component).setValid(true);		

		/* debug STRING */
		// System.out.println("decode: " + sEditorContents);
		if (debug)
			System.out.println("rteRender: sEditorContents: " + sEditorContents);  //$NON-NLS-1$
	}

	/**
	 * @see javax.faces.render.Renderer#encodeBegin(FacesContext, UIComponent)
	 */

	public void encodeBegin(FacesContext context, UIComponent component) throws IOException {
		if (debug)
			System.out.println("rteRender: in encodeBegin");  //$NON-NLS-1$

		if (context == null || component == null) {
			//throw new NullPointerException(Util.getExceptionMessage(Util.NULL_PARAMETERS_ERROR_MESSAGE_ID));
			throw new NullPointerException();
		}
	}

	/**
	 * @see javax.faces.render.Renderer#encodeChildren(FacesContext, UIComponent)
	 */

	public void encodeChildren(FacesContext context, UIComponent component) throws IOException {
		if (debug)
			System.out.println("rteRender: in encodeChildren");  //$NON-NLS-1$

		if (context == null || component == null) {
			//throw new NullPointerException(Util.getExceptionMessage(Util.NULL_PARAMETERS_ERROR_MESSAGE_ID));
			throw new NullPointerException();
		}
	}
    
    protected void initResources(){
        // rte default properties
        m_name           = getResourceString("xstr_name"); //$NON-NLS-1$
        m_jspContext     = getResourceString("xstr_jspContext"); //$NON-NLS-1$
        m_jsp            = getResourceString("xstr_jsp"); //$NON-NLS-1$

        m_width          = getResourceString("xstr_width"); //$NON-NLS-1$
        m_height         = getResourceString("xstr_height"); //$NON-NLS-1$
        m_cssPath        = getResourceString("xstr_rteStyle"); //$NON-NLS-1$

        // starter jsp
        m_starterJsp     = getResourceString("xstr_starterJsp"); //$NON-NLS-1$
    }
    
    protected String getResourceString(String key){
        try {
        	if (resourceBundle == null){
        		resourceBundle = ResourceBundle.getBundle(DEFAULT_BUNDLE_NAME);
        	}
            return resourceBundle.getString(key);
        } catch (MissingResourceException e) {
            return '!' + key + '!';
        }
    }
    
    protected Object getPageContext(FacesContext context){
		ServletRequest request = (ServletRequest) context.getExternalContext().getRequest();
		return request.getAttribute("rte_pageContext");
    }
    
    protected void includeResource(Object contextHandler, String resource) throws ServletException, IOException{
    	((PageContext)contextHandler).include(resource);
    }
    
    protected String getComponentsValue(UIComponent component)
    {
    	String currentValue = (String)((UIInput)component).getValue();
    	
    	return currentValue;
    }
    
	public void encodeEnd(FacesContext context, UIComponent component) {
		
		String currentValue = getComponentsValue(component);
		
		if (debug)
			System.out.println("rteRender: in getEndTextToRender");  //$NON-NLS-1$
		if (debug)
			System.out.println("GetEditor::doStartTag:Entered");  //$NON-NLS-1$
		//if (debug) System.out.println("buffer: " + buffer);
		if (debug)
			System.out.println("currentValue: " + currentValue);  //$NON-NLS-1$
        
        Object pageContext = getPageContext(context);
        if (pageContext == null){
            throw new FacesException("Unable to load Page Context");
        }
        //This initialises all the renderers fields from a resource bundle set up in the contextHandler.
        initResources();

		String starter = (String) component.getAttributes().get("starterJsp");
		
		setStarterJsp((null != starter && starter.length() > 0) ? starter : m_starterJsp);
		setRenderType(RENDER_TYPE);
		setComponentType(COMPONENT);

		String resource = starterJsp.startsWith("/") ? starterJsp : "/" + starterJsp; //$NON-NLS-1$ //$NON-NLS-2$
		if (debug)
			System.out.println("GetEditor::doStartTag:Dispatching to resource = " + resource); //$NON-NLS-1$
		try {
			//			Fenil Change -> Do the script include here an remove the script include from the tooling
			ResponseWriter writer = context.getResponseWriter();
			//TODO I don't think this script markup is needed, since its been added by the editor.jsp
			

			//pass attributes to the starter jsp 
			//Fenil Change..id should be obtained from the component, not the attribute map
			//String name = (String)component.getAttributes().get("id");
			
			//TODO Warning, shouldn't this name be the clientID? id may not me unique enough (two rte's could potentially exist on the one page within different containers, with the same id - would be fine until the generated functions are called.
			//String name = component.getId();
			String name = getEditorElementName(context, component);
			
			//RATLC01083006
			if(m_imageDirectory == null){
				   m_imageDirectory = LifecycleUtil.getJSResourceURLPattern(context.getExternalContext().getInitParameter(ContextParams.JS_RESOURCE_SERVLET_URL_PATTERN)) + getResourceString("xstr_imageDir"); //$NON-NLS-1$
			}
			
			setRequestAttr(pageContext, getResourceString("xstr_rteName"), (name == null) ? m_name : name); //$NON-NLS-1$
			setRequestAttr(pageContext, getResourceString("xstr_rte_jspContext"), m_jspContext); //$NON-NLS-1$
			setRequestAttr(pageContext, getResourceString("xstr_rte_jsp"), m_jsp); //$NON-NLS-1$
			setRequestAttr(pageContext, getResourceString("xstr_rte_imageDirectory"), m_imageDirectory); //$NON-NLS-1$
			setRequestAttr(pageContext, getResourceString("xstr_rte_cssPath"), m_cssPath); //$NON-NLS-1$

			
            //process the comma separated values set on toolbars property.
            String toolbarkeys = (String)component.getAttributes().get("toolbars");
            if (toolbarkeys == null){
                //use the default which is defined in the reource bundle.
                toolbarkeys = getResourceString("xstr_LWPBridge_controls");
            }else{
                if (toolbarkeys.equals("none")){
                    //this is the only exception to the allowed values, if set to none, we want to pass in a blank string so no jsp files are added.
                    toolbarkeys="";
                }else{
                    //process the comma separated list and convert it into a comma separated list of jsp files names.
                    StringBuffer sbuf = new StringBuffer();
                    String[] toolbars = StringUtil.splitToArray(toolbarkeys, ',');

                    for (int i = 0; i < toolbars.length; i++) {
                        String key = toolbars[i].trim();
                        sbuf.append("toolbar_")
                        .append(key)
                        .append(".jsp");
                        if (i < toolbars.length - 1){
                            sbuf.append(", ");
                        }
                    }
                    toolbarkeys = sbuf.toString();
                }
            }
            //set the toobar keys on the attrbute map.
            if(toolbarkeys != null)
            	setRequestAttr(pageContext, getResourceString("xstr_rte_controls"), toolbarkeys); //$NON-NLS-1$
						
			/*this is to fix the readonly */
			
			//String mode = (String) component.getAttributes().get("readOnly");
			Boolean mode  = null;/*= (Boolean) component.getAttributes().get("readOnly");*/
			Object mode_obj = component.getAttributes().get(readOnly); //$NON-NLS-1$
			
			if(mode_obj == null)
				mode_obj = component.getAttributes().get(readonly); // get the readonly for designer 
			
			if(mode_obj instanceof Boolean)
				mode = (Boolean) mode_obj;

			if(mode_obj instanceof String)
				mode = Boolean.valueOf(((String)mode_obj));
			
			/*setRequestAttr(
				pageContext,
				"rte_mode",
				(mode == null) || (mode.equalsIgnoreCase("false")) ? MODE_EDIT : MODE_VIEW);*/
			setRequestAttr(
					pageContext,
					getResourceString("xstr_rte_mode"), //$NON-NLS-1$
					(mode == null) || (mode.booleanValue() == false) ? MODE_EDIT : MODE_VIEW);

			/*this is to fix the readonly */
			
			String locale =
				context.getViewRoot().getLocale().getLanguage() + "_" + context.getViewRoot().getLocale().getCountry(); //$NON-NLS-1$
			setRequestAttr(pageContext, getResourceString("xstr_rte_locale"), locale); //$NON-NLS-1$

			String width = (String) component.getAttributes().get("width"); //$NON-NLS-1$
			setRequestAttr(pageContext, getResourceString("xstr_rte_width"), (width == null) ? m_width : width); //$NON-NLS-1$

			String height = (String) component.getAttributes().get("height"); //$NON-NLS-1$
			setRequestAttr(pageContext, getResourceString("xstr_rte_height"), (height == null) ? m_height : height); //$NON-NLS-1$

			//			String listener = (String)component.getAttributes().get("value");
			//			if (listener==null) {
			//				listener = "";
			//			} 

			if (debug) {
				setRequestAttr(pageContext, getResourceString("xstr_rte_debug"), "true"); //$NON-NLS-1$ //$NON-NLS-2$
			} else {
				setRequestAttr(pageContext, getResourceString("xstr_rte_debug"), "false"); //$NON-NLS-1$ //$NON-NLS-2$
			}

			String tabindex = (String) component.getAttributes().get("tabindex");
			setRequestAttr(pageContext,getResourceString("xstr_rte_tabindex"),tabindex);
			
			//String rteContent = htmlEncode(getRichTextBody((listener!=null)&&(listener.length()>0)? listener: currentValue));
			String rteContent = htmlEncode(getRichTextBody((currentValue != null) ? currentValue : "")); //$NON-NLS-1$
			setRequestAttr(pageContext, getResourceString("xstr_rte_listener"), getRteListener((name == null) ? m_name : name, rteContent)); //$NON-NLS-1$

			//set up hidden value
			content_field = getContentFieldName(context, component);
			//	Fenil Change...parent need not always be the form..this has been changed so that we directly look for the hidden field without bothering
			//about the form
			//UIComponent comp = component.getParent();

			//if (comp instanceof UIForm) {
			//	setRequestAttr (pageContext, "rte_formName", ((UIForm)comp).getId());
			setRequestAttr(pageContext, getResourceString("xstr_rte_formItem"), content_field); //$NON-NLS-1$
			//}

			//RATLC01135964 - emit all our output prior to including the RTE to avoid buffer problems
			writer.write(
				"\r\t<input type=\"hidden\" id=\"" //$NON-NLS-1$
					+ content_field
					+ "\" name=\"" //$NON-NLS-1$
					+ content_field
					+ "\" value = \"" //$NON-NLS-1$
					+ rteContent
					+ "\">\n"); //$NON-NLS-1$				
			
			includeResource(pageContext, resource);
			if (debug)
				System.out.println("GetEditor::doStartTag:Included JSP " + resource); //$NON-NLS-1$
		} catch (Exception t) {
			System.out.println("GetEditor::doEndTag:ERROR THROWABLE: " + t.getMessage());
			t.printStackTrace();
			//throw new ServletException (t);
		}
	}


	//private String jsp = ".rteres/DocEditor/jsf/RteStarter.jsp";

	/*
	 * Preserving Odyssey debug flag used for simple logging
	 */
	//private final static boolean debug = true;

	/**
	 * Set the starter jsp fullpath. 
	 * 
	 * @param starterJsp The fullpath for starter jsp page
	 */
	protected void setStarterJsp(String starterJsp) {
		this.starterJsp = starterJsp;
	}

	/**
	 * Set the render type
	 * 
	 * @param renderType The render type
	 */
	protected void setRenderType(String renderType) {
		this.renderType = renderType;
	}

	/**
	 * Set the component type
	 * 
	 * @param componentType The UI component type
	 */
	protected void setComponentType(String componentType) {
		this.componentType = componentType;
	}

	/**
	 * Set the attributes to be passed to the ServletRequest to starter jsp. This 
	 * method must be called within the overided doStartTag() method
	 * 
	 * @param name The attribute name
	 * @param val  The attribute value
	 */
	protected void setRequestAttr(Object parameterMap, String name, Object val) {
		//assume at this level its of type PageContext, overriding methods will cast to something else
		((PageContext)parameterMap).getRequest().setAttribute(name, val);
	}

	/**
	 * @see com.sun.faces.taglib.FacesTag#getLocalRendererType()
	 */
	public String getLocalRendererType() {
		return renderType; // @see faces-config.xml
	}

	/**
	 * @see javax.faces.webapp.FacesTag#createComponent()
	 */
	public String getComponentType() {
		return componentType; // @see faces-config.xml
	}

    //TODO can we remove this? its not being used
	private String getEditorElementName(FacesContext context, UIComponent component) {
		/**
		 * generate the id for the rich text control itself (use the id we're given)
		 */
		return encodeClientId(component.getClientId(context));
	}
	/*
	private String getJavaScriptVarName(FacesContext context, UIComponent component) {
		/**
		 * generate the JavaScript variable name for the rich text editor
		 */
	//return JavaScriptUtil.makeVariableName(component.getClientId(context));
	/*}
	*/

	private String getContentFieldName(FacesContext context, UIComponent component) {
		/**
		 * Build the hidden content field based on the name of the rte
		 * id as there may be more than one rte on a given page.
		 *
		 */

		return encodeClientId(component.getClientId(context) + "_contents"); //$NON-NLS-1$
	}

	/*
	 * pre IE5.5
	 * return everything contained between <body> ... </body>
	 * post IE5.5
	 * just remove LFs, CRs and process single quotes so that the JS
	 * code we generate cannot have malformed strings
	 *
	 */
	private String getRichTextBody(String sEditorContents) {
		// 
		// Always check for these buggers!
		//
		if (sEditorContents != null) {
			//sEditorContents = sEditorContents.replace('\'', '\"');			
			sEditorContents = StringUtil.replaceAll(sEditorContents, "\'", "&amp;#39;"); // replace ' with it unicode character code
			if(sEditorContents.indexOf("&#39;") > 0) // if it exists
				sEditorContents = StringUtil.replaceAll(sEditorContents, "&#39;", "&amp;#39;"); // replace it 
			     // this is here cause sometimes the ' is coming in as &#39; which is getting translated back into '
							// which is causing javascript errors.
			sEditorContents = sEditorContents.replace('\n', ' ');
			sEditorContents = sEditorContents.replace('\r', ' ');
		}
		return sEditorContents;
	}
    
 	private String getRteListener(String editorName, String content) {
		return "IBM_RTE_setEditorHTML('" + editorName + "', '" + content + "')"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	}

	public String htmlEncode(String str) {
		for (int i = 0; i < str.length(); i++) {

			if (str.charAt(i) == '\n') {
				//str = str.substring(0, i) + "&crarr;" + str.substring(i + 1);
				str = str.substring(0, i) + "" + str.substring(i + 1); //$NON-NLS-1$
			}
			if (str.charAt(i) == '\r') {
				str = str.substring(0, i) + "&crarr;" + str.substring(i + 1); //$NON-NLS-1$
			}
			if (str.charAt(i) == '\"') {
				str = str.substring(0, i) + "&quot;" + str.substring(i + 1); //$NON-NLS-1$
			}
		}
		return str;
	}
	
	//this guy is used to replace the : with _, the reason for this is the id of the rte is used
	// increating the names of the JavaScript Functions - : aren't allowed chars for this
	public String encodeClientId(String clientId){

		clientId = StringUtil.replaceAll(clientId,":", "_");
		return clientId;
	}
}
