/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2003, 2006 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.odc.jsf.components.taglib.rte;

import javax.faces.component.UIComponent;
import javax.faces.component.UIInput;
import javax.faces.component.ValueHolder;
import javax.faces.context.FacesContext;
import javax.faces.el.ValueBinding;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.PageContext;

import com.ibm.odc.jsf.components.components.rte.UIRichTextEditor;

public class RichTextEditorTag extends BxFacesTag {
	private static final String RENDER_TYPE = "RichTextEditor";
	private static final String COMPONENT =
		com.ibm.odc.jsf.components.components.rte.UIRichTextEditor.TYPE;
	private UIRichTextEditor editor = null;
	private String width = null;
	private String height = null;
	private String readOnly = null;
	private String value = null;
	private PageContext pageContext = null;

	private String tabindex = null;
	
	public RichTextEditorTag() {
		//setRenderType(RENDER_TYPE);
		setComponentType(COMPONENT);
	}
	public UIComponent createComponent() {
		if (this.editor == null)
			this.editor = new UIRichTextEditor();
		return this.editor;
	}
	/**
	  * @see javax.faces.webapp.UIComponentTag#getRendererType()
	  */
	public String getRendererType() {
		//The UIComponent does the rendering by itself so we return a null for the renderer type
		return RENDER_TYPE;
	}
	/**
	  * @see javax.faces.webapp.UIComponentTag#getComponentType()
	  */
	public String getComponentType() {
		return COMPONENT;
	}
	public int doStartTag() throws JspException {
		/*if (pageContext == null) {
			FacesContext facesContext = FacesContext.getCurrentInstance();
			pageContext =
				(javax.servlet.jsp.PageContext) facesContext
					.getExternalContext()
					.getContext();
		}*/
		pageContext.getRequest().setAttribute("rte_pageContext", pageContext);
		return super.doStartTag();
	}
	protected void setProperties(UIComponent component) {
		super.setProperties(component);
		//Fenil Change...add logic to get literal value or a VBL
		if (component instanceof ValueHolder) {
			if (value != null) {
				if (isValueReference(value)) {
					ValueBinding vb =
						FacesContext
							.getCurrentInstance()
							.getApplication()
							.createValueBinding(
							value);
					((UIInput) component).setValueBinding("value", vb);
				} else {
					((ValueHolder) component).setValue(value);
				}
			}
		}
		//this.setAttribute(component, "value", getValue());
		
		/* this is to fix the readonly*/
		
		//this.setAttribute(component, "readOnly", getReadOnly());

		if (readOnly != null) {
			if (isValueReference(readOnly)) {
				ValueBinding vb_r =
					FacesContext
						.getCurrentInstance()
						.getApplication()
						.createValueBinding(
								readOnly);
				((UIInput) component).setValueBinding("readOnly", vb_r);
			} else 
			{
				this.setAttribute(component, "readOnly", getReadOnly());
			}
		}
		
		/* this is to fix the readonly*/
		
		//this.setAttribute(component, "id", getId());
		this.setAttribute(component, "height", getHeight());
		this.setAttribute(component, "width", getWidth());
		this.setAttribute(component, "starterJsp", getStarterJsp());
		this.setAttribute(component,"tabindex",getTabindex());
		}
	public String getValue() {
		return value;
	}
	public void setValue(String value) {
		this.value = value;
	}
	public String getReadOnly() {
		return readOnly;
	}
	public void setReadOnly(String readOnly) {
		this.readOnly = readOnly;
	}
	//	public String getId() {
	//		return id;
	//	}
	//	public void setId(String id) {
	//		this.id = id;
	//	}
	public String getHeight() {
		return height;
	}
	public void setHeight(String height) {
		this.height = height;
	}
	public String getWidth() {
		return width;
	}
	public void setWidth(String width) {
		this.width = width;
	}
	
	public String getStarterJsp () {
		return super.getStarterJsp();
	}

	public void setStarterJsp (String starterJsp) {
		super.setStarterJsp(starterJsp);
	}
	
	public void setTabindex(String tabindex){
		this.tabindex = tabindex;
	}
	
	public String getTabindex(){
		return this.tabindex;
	}
	
	/**
	  * Sets the value to be set on the UIComponent using a key.  If value is null,
	  * value will not be set.
	  * @param value UIComponent - The UIComponent on which the value to be set
	  * @param key Object - The key for the value
	  * @param value Object - The value to be set
	  */
	protected void setAttribute(
		UIComponent component,
		Object key,
		Object value) {
		if (component != null
			&& key != null
			&& component.getAttributes().get(key) == null
			&& value != null) {
			component.getAttributes().put(key, value);
		}
	}

	  /**
	   * <p>Set the <code>PageContext</code> of the page containing this
	   * tag instance.</p>
	   *
	   * @param pageContext The enclosing <code>PageContext</code>
	   */
   public void setPageContext(PageContext pageContext) {

			super.setPageContext(pageContext);
			this.pageContext = pageContext;
		}
	
}
