/************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corp. 2003.  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following [enclosed] code is
 * sample code created by IBM Corporation.  This sample code is
 * not part of any standard or IBM product and is provided to you
 * solely for the purpose of assisting you in the development of
 * your applications.  The code is provided "AS IS", without
 * warranty of any kind.  IBM shall not be liable for any damages
 * arising out of your use of the sample code, even if they have
 * been advised of the possibility of such damages.
 *************************************************************************/
package com.ibm.odcb.jrender.diff;

import org.eclipse.emf.ecore.*;
import com.ibm.odcb.jrender.misc.EMFHelper;
import com.ibm.odcb.jrender.mediators.gen.ecore.EFeatureMap;
import java.util.Iterator;
import java.util.ArrayList;

/**
 * @author Titania Gupta
 * <p>
 * A helper class used for Diff Navigation.
 */
public class DiffHelper
{
  /**
   * Constructor for DiffHelper
   */
  public DiffHelper() {}
  
  /**
   * The name of the 'Diff' EClassifier in the generated XX_client.ecore file
   */
  public static final String DIFF_ECLASS_NAME = "Diff";
  /**
   * The name of the 'Client_Root' EClassifier in the generated XX_client.ecore file
   */
  public static final String CLIENT_ROOT_ECLASS_NAME = "ClientRoot";
  /**
   * The name of the 'Current' EClassifier in the generated XX_client.ecore file
   */
  public static final String CURRENT_EFEATURE_NAME = "Current";
  /**
   * The name of the 'Original' EClassifier in the generated XX_client.ecore file
   */
  public static final String ORIGINAL_EFEATURE_NAME = "Original";

  /**
   * Prints the org.eclipse.emf.ecore.EStructuralFeature values of the Diff EObject
   * in XML format to System.out.  Can be used for verification purposes.
   * <p>
   * @param Diff The org.eclipse.emf.ecore.EObject representing the Diff
   */
  public static void printDiff(EObject diff)
  {
    try 
    {
      System.out.println();
      System.out.println();
      System.out.println("---- PRINTING DIFF EOBJECT --------------------------------------");
      System.out.println("<"+DIFF_ECLASS_NAME+">");      
      Object current = EMFHelper.getEFeatureValue(diff, CURRENT_EFEATURE_NAME);
      EMFHelper.printEObject((EObject)current, CURRENT_EFEATURE_NAME, "  ", new java.util.ArrayList());
      Object original = EMFHelper.getEFeatureValue(diff, ORIGINAL_EFEATURE_NAME);
      EMFHelper.printEObject((EObject)original, ORIGINAL_EFEATURE_NAME, "  ", new java.util.ArrayList());
      System.out.println("</"+DIFF_ECLASS_NAME+">");
      System.out.println("---- END OF DIFF EOBJECT --------------------------------------");
    }
    catch (Exception e) {e.printStackTrace();}
  }

  /**
   * Compares the xmi:id of the given EObject and the given xmiId.
   * <p>
   * @param  o     An emf.ecore.EObject from the Diff (must be part of the Diff EObject).
   * @param  xmiId The xmi:Id to compare.
   * @return true if the xmi:ids match; false otherwise; 
   *          false if the EObject was not part of a Diff
   */  
  public static boolean compareIds(EObject o, String xmiId)
  {
    try
    {
      //get odc:id feature.  odc:id is same as xmi:id.  xmi:id cannot be obtained from EObject.  EMF does not include this in the list of eattributes
      String id = (String) EMFHelper.getEFeatureValue( o, EFeatureMap.ODYSSEY_ID_FEATURE );
      if (id != null)
      {
        if ( (id).equals(xmiId) )
          return true;
      }
    }
    catch (Exception e) {}
    return false;
  }
    
  /**
   * Traverses the given list and gets the EObject with the given xmi:id.
   * This method may be called in a loop as you iterate through the EReferences with multiplicity >1 of a Diff EObject.
   * <p>
   * @param  list          The list containing other lists or EObjects from the Diff.
   *                        This is usually an org.eclipse.emf.ecore.EReference object of multiplicity >1.
   * @param  xmiId         The xmi:id.
   * @param  traversedList The list of EObjects already traversed.
   *                        Empty the first time when called from within a loop, 
   *                        the same list is passed on subsequent calls to this method.
   *                        This is needed to avoid infinite loops caused by two way references.
   * @return The org.eclipse.emf.ecore.EObject with the given xmi:id.
   */
  public static EObject getEObjectByXmiId(org.eclipse.emf.ecore.util.EcoreEList list, String xmiId, ArrayList traversedList) throws Exception
  {
    EObject o = null;
    Iterator it = list.iterator();
    while (it.hasNext())
    {
      Object eobj = it.next();
      if (eobj instanceof org.eclipse.emf.ecore.util.EcoreEList)
      {
        return getEObjectByXmiId((org.eclipse.emf.ecore.util.EcoreEList) eobj, xmiId, traversedList); 
      }
      else if (eobj instanceof EObject)
      {
        if ( compareIds( (EObject) eobj, xmiId ) )
          return (EObject) eobj;
        o = getEObjectByXmiId( (EObject) eobj, xmiId, traversedList );
        if (o != null)
          return o;
      }
    }
    return o;
  }  
  
  /**
   * Traverses the EReferences of the given EObject and gets the EObject with the given xmi:id.
   * This method may be called in a loop as you iterate through the EReferences of a Diff EObject.
   * <p>
   * @param  eobj          The EObject from the Diff.
   *                        This is usually an org.eclipse.emf.ecore.EReference object.
   * @param  xmiId         The xmi:id.
   * @param  traversedList The list of EObjects already traversed.
   *                        Empty the first time when called from within a loop, 
   *                        the same list is passed on subsequent calls to this method.
   *                        This is needed to avoid infinite loops caused by two way references.
   * @return The org.eclipse.emf.ecore.EObject with the given xmi:id.
   */
  public static EObject getEObjectByXmiId(EObject eobj, String xmiId, ArrayList traversedList) throws Exception
  {
   EObject o = null;
   if (traversedList == null)
   {
      traversedList = new ArrayList();
   }    
      //if this EObject has already been compared then skip.  This is needed to avoid infinite loops.
   boolean compared = traversedList.contains(eobj);
   if (!compared)  
   {         
    traversedList.add(eobj);
    EClass c = eobj.eClass();
    Iterator refs = c.getEReferences().iterator();
    while (refs.hasNext())
    {
      try
      {
        EReference ref = (EReference) refs.next();
        Object refobj = eobj.eGet(ref, true);
        if (refobj != null)
        {
          if (refobj instanceof org.eclipse.emf.ecore.util.EcoreEList)
          {
            o = getEObjectByXmiId( (org.eclipse.emf.ecore.util.EcoreEList) refobj, xmiId, traversedList);
          }
          else if (refobj instanceof EObject)
          {
            if ( compareIds( (EObject) refobj, xmiId ) )
              return (EObject) refobj;
            o = getEObjectByXmiId( (EObject) refobj, xmiId, traversedList );
          }
          if (o != null)
            return o;
        }
      }
      catch (Exception e) {}
    }
   }
   return o;
  }
  
  /**
   * Gets the EObject with the given xmi:id from the given Diff in its original state.
   * <p>
   * @param  Diff The Diff EObject.
   * @param  xmiId    The xmi:id.
   * @return The org.eclipse.emf.ecore.EObject with the xmi:id in it's original state.
   */
  public static EObject getOriginalEObjectByXmiId(EObject diff, String xmiId)
  {
    EObject o = null;
    try
    {
      EObject original = (EObject) EMFHelper.getEFeatureValue(diff, ORIGINAL_EFEATURE_NAME);
      o = getEObjectByXmiId(original, xmiId, new ArrayList());
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
    return o;
  }
  
}
