/************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corp. 2003.  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following [enclosed] code is
 * sample code created by IBM Corporation.  This sample code is
 * not part of any standard or IBM product and is provided to you
 * solely for the purpose of assisting you in the development of
 * your applications.  The code is provided "AS IS", without
 * warranty of any kind.  IBM shall not be liable for any damages
 * arising out of your use of the sample code, even if they have
 * been advised of the possibility of such damages.
 *************************************************************************/
package com.ibm.odcb.jrender.diff;

////#IF# SDO
//import commonj.sdo.DataObject;
////#END#

////#IF# WDO
import com.ibm.etools.wdo.DataObject;
////#END#

/**
 * @author Titania Gupta
 * <p>
 * Information about the <code>Diff<code> object.
 */
public class DiffInfo
{
  public DiffInfo(){}
  
  /**
   * Constructor for <code>DiffInfo<code>.
   * <p>
   * @param theEClassName Name of the EClass in the generated XX_client.ecore.
   *                       This is the same as the EClassMap 'export' name in the XX.emap file.
   * @param theCrud       The CRUD action (<u>C<u>reate, <u>R<u>ead, <u>U<u>padate or <u>D<u>elete) performed on the current object of the Diff.
   * @param theCurrent    The org.eclipse.emf.ecore.EObject representing the current state of the object.
   * @param theOriginal   The org.eclipse.emf.ecore.EObject representing the original state of the object.
   * @param theAncestors  List of ancestors (org.eclipse.emf.ecore.EObjects) of the object.
   */
  public DiffInfo(String theEClassName, char theCrud, DataObject theCurrent, DataObject theOriginal, DiffInfo theParent, String propertyName)
  {
    this.eClassName = theEClassName;
    this.crud = theCrud;
    this.current = theCurrent;
    this.original = theOriginal;
    this.parent = theParent;
    this.propertyName = propertyName;
  }

  /**
   * Name of the EClass in the generated XX_client.ecore.
   * This is the same as the EClassMap 'export' name in the XX.emap file.
   */
  protected String eClassName;
  
  /**
   * The CRUD action (<u>C<u>reate, <u>R<u>ead, <u>U<u>padate or <u>D<u>elete) performed on the current object of the Diff.
   */
  protected char crud;
  
  /**
   * The org.eclipse.emf.ecore.EObject representing the current state of the object.
   */
  protected DataObject current;
  
  /**
   * The org.eclipse.emf.ecore.EObject representing the original state of the object.
   */
  protected DataObject original;
  
  /**
   * The class property name this instance is associated with.
   */
  protected String propertyName;
  
  /**
   * The DiffInfo of the parent object.
   */
  protected DiffInfo parent;
  
/**
 * @return
 */
public char getCrud()
{
	return crud;
}

/**
 * @return
 */
public DataObject getCurrent()
{
	return current;
}

/**
 * @return
 */
public String getEClassName()
{
	return eClassName;
}

/**
 * @return
 */
public DataObject getOriginal()
{
	return original;
}

/**
 * @param c
 */
public void setCrud(char c)
{
	crud = c;
}

/**
 * @param object
 */
public void setCurrent(DataObject object)
{
	current = object;
}

/**
 * @param string
 */
public void setEClassName(String string)
{
	eClassName = string;
}

/**
 * @param object
 */
public void setOriginal(DataObject object)
{
	original = object;
}

/**
 * @return
 */
public String getPropertyName() {
	return propertyName;
}

/**
 * @param string
 */
public void setPropertyName(String string) {
	propertyName = string;
}

/**
 * @return
 */
public DiffInfo getParent() {
	return parent;
}

/**
 * @param info
 */
public void setParent(DiffInfo info) {
	parent = info;
}

}
