/************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corp. 2003.  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following [enclosed] code is
 * sample code created by IBM Corporation.  This sample code is
 * not part of any standard or IBM product and is provided to you
 * solely for the purpose of assisting you in the development of
 * your applications.  The code is provided "AS IS", without
 * warranty of any kind.  IBM shall not be liable for any damages
 * arising out of your use of the sample code, even if they have
 * been advised of the possibility of such damages.
 *************************************************************************/
package com.ibm.odcb.jrender.diff;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import com.ibm.odcb.jrender.diff.generators.DiffHandlerGenerator;


public abstract class DiffProcessor
 {
   public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;

   public abstract DiffedObjectsLists handle(Object RootObject, String DiffStr) throws DiffException;
   
   public static DiffedObjectsLists Import(Object RootObject, String DiffStr, String JavaComplexName)
   throws DiffException
    {
      DiffProcessor DP = DiffProcessorFactory.getInstance().getDiffProcessor(JavaComplexName);
      return DP.handle(RootObject, DiffStr);
    }
    
   public static class DiffedObjectsLists
    {
      protected List createdObjects = new ArrayList();
      protected List deletedObjects = new ArrayList();
	/**
	 * @return Returns the createdObjects.
	 */
	public List getCreatedObjects() {
		return createdObjects;
	}
	/**
	 * @return Returns the deletedObjects.
	 */
	public List getDeletedObjects() {
		return deletedObjects;
	}
    }

   public static DiffedObjectsLists getDiffedObjects(Map Env)
    {
      DiffedObjectsLists OL = new DiffedObjectsLists();
      Iterator I = Env.entrySet().iterator();
      while (I.hasNext() == true)
       {
         Map.Entry E = (Map.Entry) I.next();
         String Str = (String) E.getKey();
         if (Str.startsWith(DiffHandlerGenerator.MARKER_CLASSCREATE) == true)
          {
            OL.createdObjects.add(E.getValue());
          }
         else if (Str.startsWith(DiffHandlerGenerator.MARKER_CLASSDELETE) == true)
          {
            OL.deletedObjects.add(E.getValue());
          }
       }
      return OL;
    }
 }
