/************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corp. 2003.  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following [enclosed] code is
 * sample code created by IBM Corporation.  This sample code is
 * not part of any standard or IBM product and is provided to you
 * solely for the purpose of assisting you in the development of
 * your applications.  The code is provided "AS IS", without
 * warranty of any kind.  IBM shall not be liable for any damages
 * arising out of your use of the sample code, even if they have
 * been advised of the possibility of such damages.
 *************************************************************************/
package com.ibm.odcb.jrender.diff;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import com.ibm.odcb.jrender.mediators.ExportException;
import com.ibm.odcb.jrender.mediators.MediatorFactory;
import com.ibm.odcb.jrender.mediators.gen.ecore.EClassMap;
import com.ibm.odcb.jrender.mediators.gen.ecore.EPackageMap;
import com.ibm.odcb.jrender.misc.ODCClassLoader;
import com.ibm.odcb.jrender.misc.Streamer;


public class DiffProcessorFactory {
    public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;

    public static DiffProcessorFactory getInstance() {
        if (_Instance == null) {
            synchronized (_Mutex) {
                if (_Instance == null) {
                    _Instance = new DiffProcessorFactory(GetDiffProcessors());
                }
            }
        }
        return _Instance;
    }

    protected static DiffProcessorFactory _Instance;

    protected static Object _Mutex = new Object();

    public DiffProcessorFactory(Map DiffProcessors) {
        _DiffProcessors = DiffProcessors;
    }

    protected Map _DiffProcessors;

    ////////////////////////////////////////////////////////////////////////////////////////////////
    // Methods to get a Mediator based on various combinations of parameters
    protected DiffProcessor getDiffProcessor(String BaseType, String ExtraName) {
        return (DiffProcessor) _DiffProcessors.get(EClassMap.MakeComplexName(BaseType, ExtraName));
    }

    protected DiffProcessor getDiffProcessor(String JavaComplexName) {
        return (DiffProcessor) _DiffProcessors.get(JavaComplexName);
    }

    public static HashMap GetDiffProcessors() {
        Streamer.status.println().Header().println("Instanciating the DiffProcessors");
        HashMap DiffProcessors = new HashMap(57);
        try {
            Iterator I = MediatorFactory.getInstance().getUniverse().getEPackageMapsIterator();
            while (I.hasNext() == true) {
                EPackageMap EPM = (EPackageMap) I.next();
                EClassMap RootECM = EPM.getRootEClassMap();
                String ClassName = null;
                DiffProcessor DP = null;

                String pkg = RootECM.getClientEcoreName();
                pkg = (pkg.substring(0, pkg.lastIndexOf("/"))).replace('/', '.').replace('\\', '.');
                ClassName = pkg + "." + EPM.getName() + "_DiffProcessor";
                try {
                    DP = (DiffProcessor) ODCClassLoader.getClassInstance(ClassName);
                } catch (Exception e) {
                    Streamer.warning.Header().println("Cannot create an instance of the DiffProcessor '"+ClassName+"'");
                    Streamer.error.Header().printStackTrace(e);
                } 
                Iterator I2 = EPM.getEClassMapsByJavaComplexName().values().iterator();
                while (I2.hasNext() == true) {
                    EClassMap ECM = (EClassMap) I2.next();
                    String Key = ECM.getJavaComplexName();
                    Streamer.debug.Header().println("Adding a DiffProcessor '" + ClassName + "' for Key '" + Key + "'.");
                    DiffProcessors.put(Key, DP);
                }

            }
        } catch (ExportException E) {
            Streamer.error.Header().println("Cannot get the EPackageMaps for this universe.");
            Streamer.error.Header().printStackTrace(E);
        }
        return DiffProcessors;
    }

}