/////////////////////////////////////////////////////////////////////////
//  IBM Confidential
//  
//  OCO Source Materials
//  
//  PID xxxx-xxx
//  
//   Copyright IBM Corp.  2003
//  
//  The source code for this program is not published or other-
//  wise divested of its trade secrets, irrespective of what has
//  been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////


package com.ibm.odcb.jrender.diff.generators;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Writer;
import java.util.Iterator;

import com.ibm.odcb.jrender.diff.DiffException;
import com.ibm.odcb.jrender.mediators.ExportException;
import com.ibm.odcb.jrender.mediators.gen.ecore.EClassMap;
import com.ibm.odcb.jrender.mediators.gen.ecore.EFeatureMap;
import com.ibm.odcb.jrender.misc.Config;
import com.ibm.odcb.jrender.misc.EMFHelper;
import com.ibm.odcb.jrender.misc.Streamer;
import com.ibm.odcb.jrender.misc.TypesUtil;
import com.ibm.odcb.jrender.utilities.EMapGen;

/**
 * @author ldh
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class DiffHandlerGenerator
 {
   public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;
   
   protected static final void GenWarning(Writer Out)
    {
      try
      {
        Out.write("//     *********************************************************\n");
        Out.write("//     ** THIS CODE IS GENERATED AND **MUST NOT** BE MODIFIED **\n");
        Out.write("//     *********************************************************\n");
      }
      catch (IOException e)
      {
        e.printStackTrace();
      }
    }

   protected final void GenVersionHistory(Writer Out)
    {
      try
      {
        Out.write("\n");
        Out.write("\n");
        Out.write("////////////////////////////////////////////////////////////////////////////////////////////////////\n");
        Out.write("////////////////////////////////////////////////////////////////////////////////////////////////////\n");
        Out.write("//\n");
        GenWarning(Out);
        Out.write("//\n");

        //////// -- ABSTACT -- ////////////////////
        PrintVersionHistory(Out);
        //////// -- ABSTACT -- ////////////////////

        Out.write("//\n");
        Out.write("////////////////////////////////////////////////////////////////////////////////////////////////////\n");
        Out.write("////////////////////////////////////////////////////////////////////////////////////////////////////\n");
        Out.write("\n");
        Out.write("\n");
        GenWarning(Out);
        Out.write("\n");
        Out.write("\n");
      }
      catch (IOException e)
      {
        e.printStackTrace();
      }    
   }
    
   public static String getDiffHandlerClassName(EClassMap ECM)
    {
      return ECM.getEPackageName()+"_"+ECM.getExport()+"_DiffHandler";
    }
    
   public static final String MARKER_CLASSUPDATE     = "CLASS+UPDATE==>"    ;
   public static final String MARKER_CLASSCREATE     = "CLASS+CREATE==>"    ;
   public static final String MARKER_CLASSDELETE     = "CLASS+DELETE==>"    ;
   public static final String MARKER_CLIENTSIGNATURE = "CLIENT+SIGNATURE==>";
   
   
   protected static final int MEDIATOR_DEBUG_MODE = Config.getInteger(Config.PROP_MEDIATOR_DEBUG_MODE);

   public void gen(EClassMap RootECM, EClassMap ECM, String Dir, String Package) 
   throws FileNotFoundException, DiffException, ExportException, ClassNotFoundException
    {
       // The extra name could have our special separator character. In case it does, we replace it with a '_'
       String ClassName = getDiffHandlerClassName(ECM);
       String FileName  = Dir+File.separator+ClassName+".java";
       Streamer.debug.Header().println("FileName ="+FileName+", and ClassName ="+ClassName);
       //TGupta Note (defect# 274304): delete the existing file.  This is to fix a Windows file system bug:
       //if a file exists with the same name but different case,
       //ex: Stock.java vs STOCK.java, they are considered the same file.
       //but, in Java, they are different files and you get compile errors.
       //so, delete the existing file so that a new one created.
       File file = new File(FileName);
       if (file.exists())
         file.delete();
       PrintWriter Out = new PrintWriter(new FileOutputStream(FileName));
       GenVersionHistory(Out);
       Out.write("package "+Package+";\n"
                +"\n"
//                +"import java.util.Arrays;\n"
//                +"import java.util.Iterator;\n"
                +"import java.util.Map;\n"
                +"import java.util.List;\n"
                +"\n"
                +"import "+EMFHelper.DataObject_CLASS.getName()+";\n"
                +"\n"
                +"import com.ibm.odcb.jrender.diff.DiffException;\n"
                +"import com.ibm.odcb.jrender.diff.DiffHandler;\n"
                +"import com.ibm.odcb.jrender.diff.DiffInfo;\n"
                +"import com.ibm.odcb.jrender.diff.DiffProcessor;\n"
                +"import com.ibm.odcb.jrender.mediators.Mediator;\n"
                +"import com.ibm.odcb.jrender.mediators.gen.ecore.EClassMap;\n"
                +"import com.ibm.odcb.jrender.misc.EMFHelper;\n"
//                +"import com.ibm.odcb.jrender.misc.ListUtil;\n"
                +"import com.ibm.odcb.jrender.misc.Streamer;\n"
                +"\n");
       GenWarning(Out);
       
       Out.write("\n"
                +"public class "+ClassName+" implements DiffHandler\n"
                +" {\n");
// LDH-NOTE: Gen code does not need the copyright
//       Out.println("   public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;");

       String RootClassName = RootECM.getBaseType().replace('$','.');
       String ObjBaseType   = ECM    .getBaseType().replace('$','.');
       
//       String MediatorTypeName = Package+"."+ECM.getEPackageName()+"_"+ECM.getExport()+com.ibm.odcb.jrender.mediators.gen.generators.Object_to_WDO4JS_as_JS_compact._KEY;

       Out.write("   public "+ClassName+"(Object Root, Map Env, EClassMap ECM, Mediator M)\n"
                +"    {\n"
                +"      _Root = Root;\n"
                +"      _Env  = Env ;\n"
                +"      _ECM  = ECM ;\n"
                +"      _M    = M   ;\n"
                +"    }\n"
                +"   protected Object    _Root;\n"
                +"   protected Mediator  _M   ;\n"
                +"   protected Map       _Env ;\n"
                +"   protected EClassMap _ECM ;\n"
                +"\n"
                +"\n"
                +"   public void handle(DiffInfo Diff) throws DiffException, Exception\n"
                +"    {\n");
       if (MEDIATOR_DEBUG_MODE == 1)
        {
          Out.write("      Streamer.debug.Header().println(this.getClass().getName()+\".handle().\");\n");
        }             
       Out.write("      "+ObjBaseType+" ServerObject = ("+ObjBaseType+") find(Diff);\n"
                +"      DataObject ClientObject = Diff.getCurrent();\n"
                +"      String ODCId = ClientObject.getString(\"ODCid\");\n"
                +"\n"
                +"      char CRUDFlag = Diff.getCrud();\n"
                +"      if (CRUDFlag == 'U')\n"
                +"       {\n"
                +"         if (ServerObject == null)  // this is an update, so we must find the stock object, otherwise, exception\n"
                +"          {\n"
                +"            throw new DiffException(\"Trying to update a '\"+EMFHelper.getEClassName(ClientObject)\n"
                +"                                   +\"' object we could not find: \"+getId(Diff, true));\n"
                +"          }\n"
                +"         if (_Env.get(\""+MARKER_CLASSUPDATE+ObjBaseType+"#\"+ODCId) == null) // not already processed\n"
                +"          {\n"
				+"            EMFHelper.validateDataObject(ClientObject);\n"
                +"            copy(ClientObject, ServerObject, false);\n"
                +"            SynchOriginal(Diff);\n"
                +"            _Env.put(\""+MARKER_CLASSUPDATE+ObjBaseType+"#\"+ODCId, ServerObject);\n"
                +"          }\n"
                +"        }\n"
                +"       else if (CRUDFlag == 'C')\n"
                +"        {\n"
                +"          // This is a creation, so we beter not have found that object, unless it has been\n"
                +"          // processed already\n"
                +OutputCreationCode(ECM, RootECM, "ServerObject", ObjBaseType)
                +"        }\n"
                +"       else if (CRUDFlag == 'D')\n"
                +"        { // this is a deletion, so we better have found the object, or have processed it before already\n"
                +"          if (ServerObject == null || _Env.get(\""+MARKER_CLASSDELETE+ObjBaseType+"#\"+ODCId) != null)\n"
                +"           {\n"
                +"             throw new DiffException(\"Trying to delete a '\"+EMFHelper.getEClassName(ClientObject)\n"
                +"                                    +\"' object we could not find: \"+getId(Diff, true));\n"
                +"           }\n"
                +"          _Env.put(\""+MARKER_CLASSDELETE+ObjBaseType+"#\"+ODCId, ServerObject);\n"
                +"          CRUDFlag = 'R';\n"
                +"        }\n"
                +"\n"
                +"       if (CRUDFlag == 'A')\n"
                +"        { // this is an Add operation\n"
                +"          AddRemove(Diff, ServerObject, true);\n"
                +"        }\n"
                +"       else if (CRUDFlag == 'R')\n"
                +"        { // this is a Remove operation\n"
                +"          AddRemove(Diff, ServerObject, false);\n"
                +"        }\n"
                +"    }\n"
                +"\n"
                +"\n"
                +"   public Object find(DiffInfo Diff)\n"
                +"   throws DiffException, Exception\n"
                +"    {\n");
       if (RootECM.getJavaComplexName().equals(ECM.getJavaComplexName()) == true)
        {
          Out.write("     return _Root;\n");
        }
       else
        {
          Out.write("      // Compute the signature for the client object\n"
                   +"      String ClientObjectSignature;\n"
                   +"      if (Diff.getOriginal() == null)\n"
                   +"       {\n"
                   +"         if (Diff.getCrud() == 'U')\n"
                   +"          {\n"
                   +"            throw new DiffException(\"CURRENT Object '\"+EMFHelper.getEClassName(Diff.getCurrent())\n"
                   +"                                   +\" : \"+getId(Diff, false)+\"' does not have a matching ORIGINAL object. This is expected for Updates.\");\n"
                   +"          }\n"
                   +"         ClientObjectSignature = getClientSignature(Diff.getCurrent());\n"
                   +"       }\n"
                   +"      else\n"
                   +"       {\n"
                   +"         ClientObjectSignature = getClientSignature(Diff.getOriginal());\n"
                   +"       }\n"
                   +"      // A common pattern is to add the same object to multiple places, or add\n"
                   +"      // multiple objects to the same parent. So caching objects on signatures is\n"
                   +"      // important. We check if the object has already been found before.\n"
                   +"      "+ObjBaseType+" ServerObject = ("+ObjBaseType+") _Env.get(\""+MARKER_CLIENTSIGNATURE+"\"+ClientObjectSignature);\n"
                   +"      if (ServerObject != null)\n"
                   +"       return ServerObject;\n"
                   +"      // At this point, we are facing a new object, so we need to search for it. Let's get the parent\n"
                   +"      // object, find the DiffHandler for its class, and call the find method on it.\n"
                   +OutputParentFinderCode("      ", "Object", ECM, RootECM, true));
          if (MEDIATOR_DEBUG_MODE == 1)
           {
             Out.write("      Streamer.debug.Header().println(\"Searching for object with signature '\"+ClientObjectSignature+\"'.\");\n");
           }
          Out.write(OutputFindCode(ECM, "ServerObject", ObjBaseType)
                   +"      if (ServerObject != null)\n"
                   +"       {\n");
          if (MEDIATOR_DEBUG_MODE == 1)
           {
             Out.write("      Streamer.debug.Header().println(\"        Found object with signature '\"+ClientObjectSignature+\"'.\");\n");
           }
          Out.write("         _Env.put(\"CLIENT+SIGNATURE==>\"+ClientObjectSignature, ServerObject);\n"
                   +"       }\n"
                   +"      return ServerObject;\n");
        }
       Out.write("    }\n"
                +"\n"
                +"\n"
                +"   public static void copy(DataObject ClientObject, "+ObjBaseType+" ServerObject, boolean Create)\n"
                +"    throws DiffException, Exception\n"
                +"     {\n"
//LDH-Note: Is this the right thing to do? Can Primary fields be changed? Normally, i'd say no, but for many
//         users, they'll just take the clientData as generated by the tools, and all single cardinality
//         fields are automatically made primary, so it seems unrealistic to force that PKs cannot be changed.
//         if So though, we need to update Signatures.
                +OutputSetters(ECM)
                +"     }\n"
                +"\n"
                +"\n"
                +"   // After synching the server object, we need to synch the Original object in the Diff. Why? The server copy has been updated.\n"
                +"   // If this object is part of the primary key for another object in the Diff, then suddenly, the Original copy will have outdated\n"
                +"   // values. So we need to make sure that Original and Server objects are always equivalent\n" 
                +"   public void SynchOriginal(DiffInfo Diff)\n"
                +"    throws DiffException\n"
                +"     {\n"
                +"       DataObject Current  = Diff.getCurrent ();\n"
                +"       DataObject Original = Diff.getOriginal();\n"
                +OutputSynch(ECM)
                +"     }\n"
                +"\n"
                +"\n"
                +"   public String getClientSignature(DataObject Obj)\n"
                +"    {\n"
                +"      if (Obj == null) return \"\";\n"
                +"      if (EMFHelper.getEClassName(Obj).equals(\""+ECM.getExport()+"\") == false)\n"
                +"          {\n"
                +"            Streamer.error.Header().println(\"DiffHandler.getSignature for '"+ECM.getEClass().getName()+"' called for DataObject of type '\"+EMFHelper.getEClassName(Obj)+\"'\")\n"
                +"                                   .println(\"Please make sure that the ecore and emap are defined correctly.\"); \n"
                +"          }\n"
                +"      StringBuffer SigStr = new StringBuffer();\n"
                +"      _ECM.getKeySignature(SigStr, Obj, 0, null);\n"
                +"      return \"CLIENT_MODEL`"+ECM.getJavaComplexName()+"\"+SigStr.toString();\n"
                +"    }\n"
                +"\n"
                +"   public String getSignature(Object ServerObject)\n"
                +"    {\n"
                +"      return _M.getSignature(ServerObject, \"CLIENT_MODEL\", true, 0);\n"
                +"    }\n"
                +"\n"
                +"   public String getId(DiffInfo Diff, boolean Original)\n"
                +"    {\n"
                +"      StringBuffer Str = new StringBuffer();\n"
                +"      Str.append(Diff.getEClassName()).append(\": \");\n"
                +"      DataObject ClientObject = Original ? Diff.getOriginal() : Diff.getCurrent();\n"
                +"      if (ClientObject == null)\n"
                +"       {\n"
                +"         Str.append(\"null\");\n"
                +"       }\n"
                +"      else\n"
                +"       {\n"
                +OutputGetIdCode("         ", ECM)
                +"       }\n"
                +"      return Str.toString();\n"
                +"    }\n"
                +"\n"
                +"   public void AddRemove(DiffInfo Diff, "+ObjBaseType+" ServerObject, boolean Add)\n"
                +"   throws DiffException, Exception\n"
                +"    {\n"
                +OutputParentFinderCode("      ", "Object", ECM, RootECM, false)
                +OutputAddRemoveCode(ECM)
                +"    }\n"
                +" }\n");
       Out.close();
    }
   
   public static String OutputCreationCode(EClassMap ECM, EClassMap rootEcm, String ObjectName, String ObjBaseType)
   throws DiffException
    {
       String code = "";
       if(TypesUtil.isInstantiable(ECM.getBaseTypeClass()) == false && ECM.getBaseTypeClass() != EMFHelper.DataObject_CLASS){
           code += "          throw new DiffException(\"Cannot create a new instance of a non concrete class '"+ECM.getBaseTypeClass().getName()+"'.\");\n";
       }
       else{
           code = "          if (ServerObject != null && _Env.get(\""+MARKER_CLASSCREATE+ObjBaseType+"#\"+ODCId) == null)\n"
           +"           {\n"
           +"             throw new DiffException(\"Trying to create a new '\"+EMFHelper.getEClassName(ClientObject)\n"
           +"                                    +\"' object, but it already exists: \"+getId(Diff, true));\n"
           +"           }\n";
           
           if (ECM.getBaseTypeClass() == EMFHelper.DataObject_CLASS)
           {
           	// We are in WDO/SDO territory!
           	
           	// This is fix for 6.0.0.1 RATLC00979703 
           	// This fix ensures that no child object is created where the parent reference's containment
           	// property is false. If the parent object has containment = true then we create the child from there
           	// Else, if the object has a parent reference in the root of the datagraph with containment = true
           	// then create the child from there. Otherwise there is no containing references so throw Exception.
           	// -Adam (adamberm)
           	
           	
           	// Check for containment value in parent:
            Iterator I = ECM.getUniverse().getEClassMapsIterator();
            boolean containOK = false,	// remembers whether we have found a parent reference with containment true
					useroot = false;
             
            while ((I.hasNext() == true) && (containOK == false))
            {
	           EClassMap ParentECM = (EClassMap) I.next();
	           Iterator I2 = ParentECM.getEFeatureMapIterator();
	           
	           // Need to make sure that potential parent is not root -> roots contain all.
	           // In the case where root==parent, the containing reference used is the root's reference
	           // perhaps not optimal
	           while ((I2.hasNext() == true) && (containOK == false) 
	           		&& (!rootEcm.getECoreComplexName().equals(ParentECM.getECoreComplexName())))
	           {
	           		{
	           			//Check for containment value in parent:
	           			EFeatureMap ParentEFM = (EFeatureMap) I2.next();
	           			if (   ParentEFM.getReferenceMap() != null 
	           					&& ParentEFM.getReferenceMap().getEPackageName().equals(ECM.getEPackageName()) 
								&& ParentEFM.getReferenceMap().getECoreComplexName().equals(ECM.getECoreComplexName())
								&& (ParentEFM.isContainment() == true))
	           			{
	           				// create child with parent as parent
	           				containOK = true;
	           				break;
	           			}
					}
                }
            }
            
            //no non-root parent found so we see if there is a containing reference in 'root'
            if(containOK == false)
            {
               	// see if it is a child of the root node with containment=true
               	Iterator I3 = rootEcm.getEFeatureMapIterator();
	            while ((I3.hasNext()) == true && (containOK == false))
	            {
	            	EFeatureMap RootEFM = (EFeatureMap) I3.next();
	            	if (   RootEFM.getReferenceMap() != null 
	            		&& RootEFM.getReferenceMap().getEPackageName().equals(ECM.getEPackageName()) 
	                	&& RootEFM.getReferenceMap().getECoreComplexName().equals(ECM.getECoreComplexName())
						&& RootEFM.isContainment() == true)
	            	{
	                  	// create child with root as parent
	                	useroot = true;
	                	containOK = true;
	                	break;
                   }
           		}
             }
            

            
            if(containOK == false){
            		code += "          //'Create' is not possible for this object as there is no containing parent or root reference\n"
            			+	"          throw new DiffException(\"Could not find suitable parent for new object. There must exist a containing parent or root reference.\");\n";
            		return code;
            }
            else
            {
            	code += OutputParentFinderCode("          ", "DataObject", ECM, rootEcm, false);
            	if(useroot == true){
            		//we know _Root is DataObject (SDO)
            		
            		code += "          //Create new project as child of root; Either parent is root or root is container\n"
            			+	"          "+ObjectName+" = ((DataObject)_Root).createDataObject(_ECM.getEClassName());\n";
            	}
            	else
            	{
            		//Parent has containing reference
            		//This is the pre-fix line that was used in 6.0/1.1
            		code += "          //Create new project as child of parent; Parent is container"
            			+	"          "+ObjectName+" = ParentServerObject.createDataObject(Diff.getPropertyName());\n";
            	}
            }
            // END fix RATLC00979703
            
           }
           else
           {
               // We are in regular bean territory
           		// ADAM NOTE: There is no containment check for beans as they do not use SDO on update/create
           		// This may need further investigation...
               code += "          "+ObjectName+" = new "+ObjBaseType+"();\n";
           }
           
		 	code +=	"          EMFHelper.validateDataObject(ClientObject);\n"
		   		+"          copy(ClientObject, ServerObject, true);\n"
				+"          _Env.put(\""+MARKER_CLASSCREATE+ObjBaseType+"#\"+ODCId, ServerObject);\n"
				+"          CRUDFlag = 'A';\n";
       }
       return code;
    }

   public static String OutputSetters(EClassMap ECM)
   throws DiffException, ClassNotFoundException
    {
      Iterator I = ECM.getEFeatureMapIterator();
      StringBuffer Str = new StringBuffer();
      while (I.hasNext() == true)
       {
         EFeatureMap EFM = (EFeatureMap) I.next();
         if (EFM.getSetter() != null)
          {
            String ClientGetter = EMapGen.FindProperty(EFM.getEFeature(), EMFHelper.DataObject_CLASS, null, true);
            ClientGetter = "ClientObject."+EFeatureMap.CleanupGetterSetter(ClientGetter);
            if (EFM.isReference() == false)  /// Simple type attribute
             {
               if (EFM.isMany() == true)  // array or list or iterator
                {
                  // It's an array of int, boolean, char etc... We need to convert
                  if (TypesUtil.isArrayOfBasicJavaType(EFM.getGetterType(), true) == true)
                   {
                     ClientGetter = "com.ibm.odcb.jrender.misc.ListUtil.to_"+TypesUtil.getShortJavaTypeFromArrayType(EFM.getGetterType())
                                                       +"Array("+ClientGetter+")";
                   }
                  // It's an array of Integer, Boolean, Character etc... We need to get the Array
                  else if (TypesUtil.isArrayOfBasicJavaType(EFM.getGetterType(), false) == true)
                   {
                     ClientGetter = "("+TypesUtil.getShortJavaTypeFromArrayType(EFM.getGetterType())
                                       +"[])"+ClientGetter+".toArray()";
                   }
                  else if (TypesUtil.isGeneralListNoArrays(EFM.getGetterType()) == true)
                   {
                     // Everything is fine. This is the leap of faith part. The ECore said it was a list of some basic type
                     // and internally, we got List, or Iterator, or Collection, so we trust that inside, there are the right
                     // things
                   }
                  // Huh? What??? This should never happen as we have verified that everything was fine in the model verification phase
                  else
                   {
                     String Msg = "Unexpected type '"+EFM.getGetterType()
                                 +"' for property '"+EFM.getParentMapping().getEClassName()+"."+EFM.getEFeatureName()
                                 +"'. Expected a general list of basic Java type (int, Boolean, String...).";
                     Streamer.error.Header().println(Msg);
                     throw new DiffException(Msg);  
                   }
                }
               else // This is a single cardinality attribute
                {
                  if (TypesUtil.isPrimitiveJavaType(EFM.getGetterType()) == true)
                   {
                     // Let it be. The code for the client getter is perfect. 
                   }
                  else
                   {
                     // Here, we need to make some changes. The getter gives us an int, boolean, char, but the
                     // setter in the server object requires Integer, Boolean, Character etc.... So we need to change
                     // the ClientGetter.
                     ClientGetter = "("+EFM.getGetterType()+")"
                                       +"ClientObject.get(\""+EFM.getEFeatureName()+"\")";
                   }
                }
               // need code to check whether type is boolean primitive type.
               // if the SDO is a boolean primitive type, the method isSet returns the
               // value of the boolean field. This means a field which stores a boolean field false
               // will return false from isSet. The original (V<7) code interperets this as unset 
               // and hence will not update boolean true to false... Adam (cs defect: #44848)
               if ( TypesUtil.isPrimitiveJavaType(EFM.getGetterType()) == true
               		&& EFM.getGetterType() == boolean.class.getName() )
               {
               		// primitive boolean type (** not Boolean Object)
               		// do not print isSet check
               		// assumes primitive boolean value is always "set"
					Str.append(EFM.isID() == true?"  // Primary attribute\n":"\n")
						.append("       ").append("ServerObject.").append(EFM.MergeSetterCode(ClientGetter)).append(";\n");
               }
               else
               {
               		Str.append("       if (ClientObject.isSet(\""+EFM.getEFeatureName()+"\") == true)")
               			.append(EFM.isID() == true?"  // Primary attribute\n":"\n")
						.append("        ").append("ServerObject.").append(EFM.MergeSetterCode(ClientGetter)).append(";\n");
               }
               
               
             }
            else
             {
               // If an object's reference is being modified, then the sub object must be in the Diff string
               // with either a 'C', 'D', 'A' or 'R'. So we really do not have to copy anything here.
               // This case is really easy: we do nothing
             }
          }
       }
      return Str.toString();
    }


   public static String OutputSynch(EClassMap ECM)
    {
      Iterator I = ECM.getEFeatureMapIterator();
      StringBuffer Str = new StringBuffer();
      while (I.hasNext() == true)
       {
         EFeatureMap EFM = (EFeatureMap) I.next();
         // Only synch attributes that are iD fields and have a setter
         if (EFM.getSetter() != null && EFM.isReference() == false && EFM.isID() == true)
          {
            Str.append("       if (Current.isSet(\""+EFM.getEFeatureName()+"\") == true)\n")
               .append("        ").append("Original.set(\""+EFM.getEFeatureName()+"\", Current.get(\""+EFM.getEFeatureName()+"\"));\n");
          }
       }
      return Str.toString();
    }


   public void PrintVersionHistory (Writer Out)
   throws IOException
    {
      Out.write("//  JWL 1.1.1, Auto-generated DiffHandlers");
    }

   public static String OutputFindCode(EClassMap ECM, String ServerObjectName, String ObjBaseType)
   throws ClassNotFoundException, DiffException
    {
      StringBuffer Str = new StringBuffer();
      Iterator I = ECM.getUniverse().getEClassMapsIterator();
      boolean FirstEClass = true;
      while (I.hasNext() == true)
       {
         EClassMap ParentECM = (EClassMap) I.next();
         Iterator I2 = ParentECM.getEFeatureMapIterator();
         boolean InEClass = false;
         boolean FirstEFeature = true;
         while (I2.hasNext() == true)
          {
            EFeatureMap ParentEFM = (EFeatureMap) I2.next();
            // We are only looking for References that match the type we are looking for
            if (   ParentEFM.getReferenceMap() != null 
                && ParentEFM.getReferenceMap().getEPackageName().equals(ECM.getEPackageName()) && ParentEFM.getReferenceMap().getECoreComplexName().equals(ECM.getECoreComplexName()))
             {
               // BINGO !!!!  We found a parent reference for that class
               if (InEClass == false) // This is the first time for the current EClass, so we need the if statement
                { // this code will kick in at runtime only if we have the right class
                  String ParentType = ParentECM.getBaseType().replace('$','.');
                  Str.append(FirstEClass == true?"      " : "      else\n")
                     .append(      "if (EMFHelper.getEClassName(ParentClientObject).equals(\"").append(ParentEFM.getParentMapping().getExport()).append("\") == true)\n")
                     .append("       {\n")
                     .append("         ").append(ParentType).append(" PSO = (").append(ParentType).append(") ParentServerObject;\n");
                }
               // Now we need to check which property we are looking at.
               Str.append(FirstEFeature == true?"         " : "         else ")
                  .append(         "if (Diff.getPropertyName().equals(\"").append(ParentEFM.getEFeatureName()).append("\") == true)\n")
                  .append("          {\n");
               if (ParentEFM.isMany() == false)
                {
                  // This is a simple Get. We check that we have the right object, and then return it.
                  Str.append("            if (ClientObjectSignature.equals(getSignature(PSO.").append(EFeatureMap.CleanupGetterSetter(ParentEFM.getGetter())).append(")) == true)\n")
                     .append("             ServerObject = (" + ObjBaseType + ")PSO.").append(EFeatureMap.CleanupGetterSetter(ParentEFM.getGetter())).append(";\n");
                }
               else
                {
                  // Here, we can have arrays, lists, iterators, collections.
                  String GetterType = ParentEFM.getGetterType();
                  if (TypesUtil.isArray(GetterType) == true)
                   {
                     Str.append("            ").append(ObjBaseType).append("[] SOs = (").append(ObjBaseType).append("[]) PSO.").append(EFeatureMap.CleanupGetterSetter(ParentEFM.getGetter())).append(";\n")
                        .append("            for (int i = 0; i < SOs.length; ++i)\n")
                        .append("             {\n")
                        .append("               "+ObjBaseType+" SO = SOs[i];\n")
                        .append("               Sig = getSignature(SO);\n");
                     if (MEDIATOR_DEBUG_MODE == 1)
                      {
                        Str.append("               Streamer.debug.Header().println(\"     Checking object with signature '\"+Sig+\"'.\");\n");
                      }
                     Str.append("               if (ClientObjectSignature.equals(Sig) == true)\n")
                        .append("                {\n")
                        .append("                  ServerObject = SO;\n")
                        .append("                  break;\n")
                        .append("                }\n")
                        .append("             }\n");
                   }
                  else if (TypesUtil.isGeneralListNoArrays(GetterType) == true)
                   {
                     Str.append("            java.util.Iterator I = PSO.").append(EFeatureMap.CleanupGetterSetter(ParentEFM.getGetter()));
                     if (GetterType.equals(Iterator.class.getName()) == false)
                      Str.append(".iterator()");
                     Str.append(";\n")
                        .append("            while (I.hasNext() == true)\n")
                        .append("             {\n")
                        .append("               "+ObjBaseType+" SO = ("+ObjBaseType+") I.next();\n")
                        .append("               Sig = getSignature(SO);\n");
                     if (MEDIATOR_DEBUG_MODE == 1)
                      {
                        Str.append("               Streamer.debug.Header().println(\"     Checking object with signature '\"+Sig+\"'.\");");
                      }
                     Str.append("               if (ClientObjectSignature.equals(Sig) == true)\n")
                        .append("                ServerObject = SO;\n")
                        .append("             }\n");
                   }
                  else
                   {
                     String Msg = "The Reference '"+ParentEFM.getParentMapping().getEClassName()+"."+ParentEFM.getEFeatureName()
                                 +"' of type '"+ParentEFM.getGetterType()+"' is neither an array of a supported collection (Collection, List, Iterator).";
                     Streamer.error.Header().println(Msg);
                     throw new DiffException(Msg);  
                   }
                }
               Str.append("          }\n");
                
               FirstEClass = false;
               FirstEFeature = false;
               InEClass=true;
             }
          }
         if (InEClass == true)
          {      
            Str.append("       } // EClassName == "+ECM.getExport()+"\n");
          }
       }
      return Str.toString();
    }
    
   public static String OutputParentFinderCode(String Header, String Type, EClassMap ecm, EClassMap rootEcm, boolean forfind)
    {
   	//TG Note: we need to check for null parent diff here.  If you're supplied the root, parent diff is null
   	//at that point, do a simple check to see if the current object is the root and return root if yes
	  String rootif = Header+Type+" ParentServerObject = null;\n"
	        +Header+"String Sig = null;\n"
	        +Header+"DataObject ParentClientObject = null;\n"
	  	    +Header+"if (ParentDiff == null)\n"
	        +Header+"{\n";
	  if (forfind)
	  {
  		if (ecm.getBaseType().equals(EMFHelper.DataObject_CLASS.getName()))
  		{
  			rootif += Header+"  if ((((org.eclipse.emf.ecore.EObject)_Root).eClass().getEPackage().getName()+\"_\"+EMFHelper.getEClassName(("+EMFHelper.DataObject_CLASS.getName()+")_Root)).equals(\""+ecm.getExport()+"\"))\n"
	                 +Header+"  {\n"
					 +Header+"    Sig = getSignature(_Root);\n"
					 +Header+"    if (ClientObjectSignature.equals(Sig))\n"
		             +Header+"	    return _Root;\n"
					 +Header+"    else\n"
					 +Header+"      return null;\n"
					 +Header+"  }\n"
					 +Header+"  else\n"
					 +Header+"    throw new DiffException(\"Could not match the runtime root type '\"+EMFHelper.getEClassName(("+EMFHelper.DataObject_CLASS.getName()+")_Root)+\"' with the expected type '"+ecm.getExport()+"'\");\n";
  		}
  		else
  		{
  			rootif += Header+"  if (_Root instanceof "+ecm.getBaseType().replace('$','.')+")\n"
			         +Header+"  {\n"
					 +Header+"    Sig = getSignature(_Root);\n"
					 +Header+"    if (ClientObjectSignature.equals(Sig))\n"
		             +Header+"	    return _Root;\n"
					 +Header+"    else\n"
					 +Header+"      return null;\n"
					 +Header+"  }\n"
					 +Header+"  else\n"
					 +Header+"    throw new DiffException(\"Could not match the runtime root type '\"+_Root.getClass().getName()+\"' with the expected type '"+ecm.getBaseType()+"'\");\n";
  		}
	  }						
	  else
	  {
	  	rootif += Header+"  throw new DiffException(\"Trying to create/add a root Object which is not supported\");\n";
	  }
      rootif += Header+"}\n"
               +Header+"else\n"
		       +Header+"{\n";
	  
     String finder =  Header+"DiffInfo ParentDiff = Diff.getParent();\n"
		     +rootif
             +Header+"  ParentClientObject = ParentDiff.getCurrent();\n"
             +Header+"  DiffHandler DH = (DiffHandler) _Env.get(\""+DiffProcessorGenerator.MARKER_DIFFHANDLER+"\"+ParentDiff.getEClassName());\n"
             +Header+"  if (DH == null)\n"
             +Header+"   {\n"
             +Header+"     throw new DiffException(\"Cannot locate the DiffHandler for client EClass '\"+ParentDiff.getEClassName()+\"'.\");\n"
             +Header+"   }\n"
             +Header+"  ParentServerObject = ("+Type+") DH.find(ParentDiff);\n"
             +Header+"  if (ParentServerObject == null)\n"
             +Header+"   {\n"
             +Header+"     throw new DiffException(\"Cannot find the parent server object '\"+DH.getId(ParentDiff, true)+\"' for '\"+getId(Diff, true)+\"'.\");\n"
             +Header+"   }\n"
             +Header+"}\n";
	  
      return finder;
    }
    
   public static String OutputAddRemoveCode(EClassMap ECM)
   throws DiffException, ClassNotFoundException
    {
      StringBuffer Str = new StringBuffer();
      Iterator I = ECM.getUniverse().getEClassMapsIterator();
      boolean FirstEClass = true;
      Str.append("      String ParentClientObjectEClassName = EMFHelper.getEClassName(ParentClientObject);\n")
         .append("      String PropertyName = Diff.getPropertyName();\n");
      while (I.hasNext() == true)
       {
         EClassMap ParentECM = (EClassMap) I.next();
         Iterator I2 = ParentECM.getEFeatureMapIterator();
         boolean InEClass = false;
         boolean FirstEFeature = true;
         while (I2.hasNext() == true)
          {
            EFeatureMap ParentEFM = (EFeatureMap) I2.next();
            // We are only looking for References that match the type we are looking for
            if (   ParentEFM.getReferenceMap() != null 
                && ParentEFM.getReferenceMap().getECoreComplexName().equals(ECM.getECoreComplexName()) == true
                && ParentEFM.getSetter() != null
               )
             {
               // BINGO !!!!  We found a parent reference for that class
               if (InEClass == false) // This is the first time for the current EClass, so we need the if statement
                { // this code will kick in at runtime only if we have the right class
                  String ParentType = ParentECM.getBaseType().replace('$','.');
                  Str.append(FirstEClass == true?"      " : "      else ")
                     .append(      "if (ParentClientObjectEClassName.equals(\"").append(ParentEFM.getParentMapping().getExport()).append("\") == true)\n")
                     .append("       {\n")
                     .append("         ").append(ParentType).append(" PSO = (").append(ParentType).append(") ParentServerObject;\n");
                }
               // Now we need to check which property we are looking at.
               Str.append(FirstEFeature == true?"         " : "         else ")
                  .append(         "if (PropertyName.equals(\"").append(ParentEFM.getEFeatureName()).append("\") == true)\n")
                  .append("          {\n");
               if (ParentEFM.isMany() == false)
                {
                  // This is a simple Get. We check that we have the right object, and then return it.
                  Str.append("            if (Add == true)\n")
                     .append("             PSO.").append(ParentEFM.MergeSetterCode("ServerObject")).append(";\n")
                     .append("            else\n")
                     .append("             PSO.").append(ParentEFM.MergeSetterCode("null")).append(";\n");
                }
               else
                {
                  // Here, we can have arrays, lists, iterators, collections.
                  String GetterType = ParentEFM.getGetterType();
                  if (TypesUtil.isArray(GetterType) == true)
                   {
                     Str.append("            "+TypesUtil.getComplexJavaTypeFromArrayType(ParentEFM.getGetterType())+"\n")
                        .append("                Old = PSO."+EFeatureMap.CleanupGetterSetter(ParentEFM.getGetter())+",\n")
                        .append("                New = new "+TypesUtil.getBaseJavaTypeFromArrayType(ParentEFM.getGetterType())+"[Old.length+(Add==true?1:-1)];\n")
                        .append("            int j = 0;\n")
                        .append("            for (int i = 0; i < Old.length; ++i)\n")
                        .append("             {\n")
                        .append("               "+ECM.getBaseType()+" O = Old[i];\n")
                        .append("               if (Add == true || ServerObject != O)\n")
                        .append("                {\n")
                        .append("                  New[j] = O;\n")
                        .append("                  ++j;\n")
                        .append("                }\n")
                        .append("             }\n")
                        .append("            if (Add == true)\n")
                        .append("             New[j] = ServerObject;\n")
                        .append("            PSO.").append(ParentEFM.MergeSetterCode("New")).append(";\n");
                   }
                  else if (GetterType.equals(Iterator.class.getName()) == true)
                   {
                     Str.append("            List L = com.ibm.odcb.jrender.misc.ListUtil.toList(PSO."+EFeatureMap.CleanupGetterSetter(ParentEFM.getGetter())+");\n")
                        .append("            if (Add == true)\n")
                        .append("             L.add(ServerObject);\n")
                        .append("            else\n")
                        .append("             L.remove(ServerObject);\n")
                        .append("            PSO.").append(ParentEFM.MergeSetterCode("L.iterator()")).append(";\n");
                   }
                  else if (TypesUtil.isGeneralListNoArrays(GetterType) == true)
                   {
                     Str.append("            "+GetterType+" L = PSO."+EFeatureMap.CleanupGetterSetter(ParentEFM.getGetter())+";\n")
                        .append("            if (Add == true)\n")
                        .append("             L.add(ServerObject);\n")
                        .append("            else\n");
                     if (ECM.getBaseTypeClass() != EMFHelper.DataObject_CLASS)
                     {
                         Str.append("             L.remove(ServerObject);\n")
                         .append("            PSO.").append(ParentEFM.MergeSetterCode("L")).append(";\n");
                     }
                     else{
                         Str.append("			ServerObject.delete();\n");
                     }
                   }
                  else
                   {
                     String Msg = "The Reference '"+ParentEFM.getParentMapping().getEClassName()+"."+ParentEFM.getEFeatureName()
                                 +"' of type '"+ParentEFM.getGetterType()+"' is neither an array of a supported collection (Collection, List, Iterator).";
                     Streamer.error.Header().println(Msg);
                     throw new DiffException(Msg);  
                   }
                }
               Str.append("          }\n");
                
               FirstEClass = false;
               FirstEFeature = false;
               InEClass=true;
             }
          }
         if (InEClass == true)
          {      
            Str.append("       } // EClassName == "+ECM.getExport()+"\n");
          }
       }
      if (FirstEClass == false)
       {
         Str.append("      else\n")
            .append("       {\n")
            .append("         throw new DiffException(\"Failed adding/removing an object '"+ECM.getEClassName()+"' to/from '\"+ParentClientObjectEClassName+\".\"+PropertyName+\"': the parent class or property do not exist.\");\n")
            .append("       }\n");
       }
      return Str.toString();
    }
    
   public static String OutputGetIdCode(String Header, EClassMap ECM)
    {
      StringBuffer Str = new StringBuffer();
      Iterator I = ECM.getEFeatureMapIterator();
      while (I.hasNext() == true)
       {
         EFeatureMap EFM = (EFeatureMap) I.next();
         if (EFM.isID() == true && EFM.isReference() == false && EFM.isMany() == false)
          {
            Str.append(Header).append("Str.append(\""+EFM.getEFeatureName()+"\").append(\"=\").append(ClientObject.get(\""+EFM.getEFeatureName()+"\")).append(\";\");\n");
          }
       }
      return Str.toString();
    }
 }
