/************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corp. 2003.  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following [enclosed] code is
 * sample code created by IBM Corporation.  This sample code is
 * not part of any standard or IBM product and is provided to you
 * solely for the purpose of assisting you in the development of
 * your applications.  The code is provided "AS IS", without
 * warranty of any kind.  IBM shall not be liable for any damages
 * arising out of your use of the sample code, even if they have
 * been advised of the possibility of such damages.
 *************************************************************************/
package com.ibm.odcb.jrender.diff.generators;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Writer;
import java.util.Iterator;

import com.ibm.odcb.jrender.diff.DiffException;
import com.ibm.odcb.jrender.mediators.ExportException;
import com.ibm.odcb.jrender.mediators.gen.ecore.EClassMap;
import com.ibm.odcb.jrender.mediators.gen.ecore.EPackageMap;
import com.ibm.odcb.jrender.misc.EMFHelper;
import com.ibm.odcb.jrender.misc.Streamer;


public class DiffProcessorGenerator
 {
   public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;
   
   protected static final void GenWarning(Writer Out)
    {
      try
      {
        Out.write("//     *********************************************************\n");
        Out.write("//     ** THIS CODE IS GENERATED AND **MUST NOT** BE MODIFIED **\n");
        Out.write("//     *********************************************************\n");
      }
      catch (IOException e)
      {
        e.printStackTrace();
      }
    }

   protected final void GenVersionHistory(Writer Out)
    {
      try
      {
        Out.write("\n");
        Out.write("\n");
        Out.write("////////////////////////////////////////////////////////////////////////////////////////////////////\n");
        Out.write("////////////////////////////////////////////////////////////////////////////////////////////////////\n");
        Out.write("//\n");
        GenWarning(Out);
        Out.write("//\n");

        //////// -- ABSTACT -- ////////////////////
        PrintVersionHistory(Out);
        //////// -- ABSTACT -- ////////////////////

        Out.write("//\n");
        Out.write("////////////////////////////////////////////////////////////////////////////////////////////////////\n");
        Out.write("////////////////////////////////////////////////////////////////////////////////////////////////////\n");
        Out.write("\n");
        Out.write("\n");
        GenWarning(Out);
        Out.write("\n");
        Out.write("\n");
      }
      catch (IOException e)
      {
        e.printStackTrace();
      }    
   }
    
   public static final String MARKER_DIFFHANDLER     = "DIFF+HANDLER==>"    ;

   public void gen(EPackageMap EPM, String Dir, String Package) 
   throws FileNotFoundException, DiffException, ExportException
    {
       // The extra name could have our special separator character. In case it does, we replace it with a '_'
       String ClassName = EPM.getName()+"_DiffProcessor";
       String FileName  = Dir+File.separator+ClassName+".java";
       Streamer.debug.Header().println("FileName ="+FileName+", and ClassName ="+ClassName);
       //TGupta Note (defect# 274304): delete the existing file.  This is to fix a Windows file system bug:
       //if a file exists with the same name but different case,
       //ex: Stock.java vs STOCK.java, they are considered the same file.
       //but, in Java, they are different files and you get compile errors.
       //so, delete the existing file so that a new one created.
       File file = new File(FileName);
       if (file.exists())
         file.delete();
       PrintWriter Out = new PrintWriter(new FileOutputStream(FileName));
       GenVersionHistory(Out);
       Out.write("package "+Package+";\n"
                +"\n"
                +"import java.util.HashMap;\n"
                +"import java.util.Map;\n"
                +"import java.util.List;\n"
                +"\n"
                +"import com.ibm.odcb.jrender.diff.DiffProcessor;\n"
                +"import com.ibm.odcb.jrender.diff.DiffException;\n"
                +"import com.ibm.odcb.jrender.diff.DiffHandler;\n"
//                +"import com.ibm.odcb.jrender.diff.DiffInfo;\n"
                +"import com.ibm.odcb.jrender.diff.DiffTraverser;\n"
                +"import com.ibm.odcb.jrender.mediators.ExportException;\n"
                +"import com.ibm.odcb.jrender.mediators.Mediator;\n"
                +"import com.ibm.odcb.jrender.mediators.MediatorFactory;\n"
//                +"import com.ibm.odcb.jrender.mediators.gen.ecore.EClassMap;\n"
                +"import com.ibm.odcb.jrender.mediators.gen.generators.Object_to_WDO4JS_as_JS_compact;\n"
//                +"import com.ibm.odcb.jrender.misc.EMFHelper;\n"
                +"import com.ibm.odcb.jrender.misc.Streamer;\n"
                +"\n");

       String RootJavaClassName = EPM.getRootEClassMap().getBaseType().replace('$','.');
       if (RootJavaClassName.equals(EMFHelper.DataObject_CLASS.getName()) == true)
        {
          Out.write("import "+RootJavaClassName+";\n\n");
        }

       GenWarning(Out);
       
       Out.write("\n"
                +"public class "+ClassName+" extends DiffProcessor\n"
                +" {\n"
                +"   public "+ClassName+"()\n"
                +"    {\n"
                +"    }\n"
                +"\n"
                +"   protected Map _DiffHandlers = new HashMap(7);"
                +"\n"
                +"\n"
                +"   public DiffedObjectsLists handle(Object RootObject, String DiffStr)\n"
                +"   throws DiffException\n"
                +"    {\n"
                +"      try\n"
                +"       {\n"
                +"         Streamer.debug.Header().println(\" = = = = = = = = =  Diffing '"+EPM.getName()+"'  = = = = = = = = = \");\n"
                +"         Streamer.trace.Header().println(DiffStr);\n"
                +"         HashMap Env = new HashMap(101);\n"
                +"         DiffTraverser DT = new DiffTraverser(\""+EPM.getName()+"_"+EPM.getRootEClassMapName()+"\", DiffStr);\n"
                +"         DiffHandler DH;\n");

       Iterator I = EPM.getEClassMapsByJavaComplexName().values().iterator();
       while (I.hasNext() == true)
        {
          EClassMap ECM = (EClassMap) I.next();
          String DiffHandlerKey = EPM.getName()+"_"+ECM.getEClassName();
          Out.write("         DH = new "+DiffHandlerGenerator.getDiffHandlerClassName(ECM)+"\n"
                   +"                     ( ("+RootJavaClassName+")RootObject, Env,\n"
                   +"                        Mediator.getEClassMapForJavaComplexName(\""+ECM.getJavaComplexName()+"\"),\n"
                   +"                        MediatorFactory.getMediator_byTypeKey(\""+ECM.getJavaComplexName()+"\", Object_to_WDO4JS_as_JS_compact._KEY)\n"
                   +"                       );\n"
                   +"         Env.put(\""+MARKER_DIFFHANDLER+DiffHandlerKey+"\", DH);\n" 
                   +"         DT.addHandler(\""+DiffHandlerKey+"\", DH);\n");
        }

       Out.print("         DT.traverse();\n"
                +"         return getDiffedObjects(Env);\n"
                +"       }\n"
                +"      catch (ExportException e)\n"
                +"       {\n"
                +"         throw new DiffException(e.getMessage());\n"
                +"       }\n"
                +"      catch (Exception e)\n"
                +"       {\n"
                +"         Streamer.error.Header().printStackTrace(e);\n"
                +"       }\n"
                +"     return null;\n"
                +"    }\n"
                +" }\n");
       Out.close();
    }

   public void   PrintVersionHistory (Writer Out)
    {
    }
 }
