/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//  
//OCO Source Materials
//  
//5724-E76 and 5724-E77
//  
// Copyright IBM Corp.  2003
//  
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////

/*
 * Created on Mar 24, 2003
 *
 */
package com.ibm.odcb.jrender.emitters;

import java.io.Writer;
import java.io.IOException;
import java.util.ArrayList;

import com.ibm.odcb.jrender.mediators.ExportException;
import com.ibm.odcb.jrender.mediators.PageContext;
import com.ibm.odcb.jrender.misc.Streamer;
/**
 * @author Kaushal <br>
 * 
 * BaseEmitter class serves as base class for all emitters. Common functionality
 * such as control and adapter variable names, storage of varIndex, etc. are pooled here.
 */
public abstract class BaseEmitter
 {
	public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;
	
	public String CONTROL_VAR;
	public String ADAPTER_VAR;
	public String BINDER_VAR;
	public String COLUMN_VAR;
	public String NODE_ATTR_VAR;
	public String EVENT_HANDLER_VAR;
	public String HTML_ELEMENT;
	public String PAGE_CONTROL_VAR;
	protected ArrayList _behaviorList = new ArrayList();
	//If this id is set up, it will be used as the id of the anchoring div
	//of a control.
	private String overriddingId = null;

  /**
 	* Default Constructor which is empty.
 	*
    */	
	public BaseEmitter()
	{
		// empty base constructor is needed for initialization emitter, etc., so that
		// compiler calls this by default in cases when there is no need for a component and
		// adapter var in certain emitters.
		PAGE_CONTROL_VAR = "ODCPageControl";
	}
	/**
	 * This is the consturctor that is called from all the emitters first before doing their initializations.
	 * This constructor sets the standardized variable names for the emitter to use.
	 * <p>
	 * @param componentName String that specifies the name of the component being instantiated.
	 */
	public BaseEmitter(String componentName)
	{
		CONTROL_VAR = componentName + "ControlVar";
		ADAPTER_VAR = componentName + "AdapterVar";
		BINDER_VAR  = componentName + "BinderVar";
		COLUMN_VAR	= componentName + "Var";
		NODE_ATTR_VAR= componentName+ "Var";
		EVENT_HANDLER_VAR = componentName + "Var";
		HTML_ELEMENT = componentName;
		PAGE_CONTROL_VAR = "ODCPageControl";
		
		Streamer.trace.Header().println("Standard JS variables initialized in BaseEmitter constructor.");
	}

	/**
	 * Abstract Export() method--does not do much in the base class. The derived classes implement
	 * the appropriate functionality.
	 * <p>
	 * @param out	Writer to output the HTML or JS to.
	 * @param PC	PageContext that contains the information about the page's context.
	 * @throws ExportException
	 * @throws IOException
	 */	
	public abstract void Export(Writer out, PageContext PC) throws ExportException,IOException;

	// This _varIndex and getExportID() are common to all emitters, so we are pushing them
	// into the base emitter (June 10, 2003).
	protected int	 _varIndex = -666; // evil value!
	/**
	 * Method returns the variable Index (a counter) that is used by this
	 * class to uniquely instantiate JavaScript variables. For instance,
	 * var myTree would be var myTree18. The number 18 is stored in varIndex and
	 * returned by this method.
	 * <p>
	 * @return	The nextInstance count from PageContext is stored in varIndex, which is returned.	
	 */
	public int getExportID() { return _varIndex;	}

	// this function will be filled in later...based on our needs
	public void Export_Base(Writer out, PageContext PC)
	{
	}
	
	// Added 9/23/2003 - Kaushal
	public static boolean TestGuard(PageContext PC, String Guard)
	{
		if(PC.getContextVariable(Guard) == null)
		{
			synchronized(PC) // to avoid race conditions
			{
				if(PC.getContextVariable(Guard) == null)
				{
					PC.addContextVariable(Guard,"YES");
					return false;
				}
			}
		}
		return true;
	}
	/**
	 * @return
	 */
	public String getOverriddingId() {
		return overriddingId;
	}

   /**If this id is set up, it will be used as the id of the anchoring div
	 *of a control.
	 * @param string
	 */
	public void setOverriddingId(String string) {
		overriddingId = string;
	}
	
	/**
	 *  Adds behavior emitter to parent ODC component emitter
	 */
	public void addBehavior(BehaviorEmitter behavior)
	{
		_behaviorList.add(behavior);
	}

}
