/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//  
//OCO Source Materials
//  
//5724-E76 and 5724-E77
//  
// Copyright IBM Corp.  2003
//  
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////

/*
 * Created on July 8, 2003
 *
 */
package com.ibm.odcb.jrender.emitters;

import java.io.IOException;
import java.io.Writer;

import com.ibm.odcb.jrender.mediators.ExportException;
import com.ibm.odcb.jrender.mediators.PageContext;
import com.ibm.odcb.jrender.misc.Streamer;

/**
 * @author Kaushal <br>
 * Binder emitter emits JS code for the PropertyBinder, to bind
 * a given control with a specified change/click event and the
 * corresponding 'value' (or 'innerHTML') to be changed.
 */
public class BinderEmitter extends BaseEmitter {

	public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;
	
	/**
	 * Constructor: does not do much except call the BaseEmitter with the appropriate
	 * component name so that the BINDER_VAR variable is initialized properly.
	 * Init() method does all the class variable initializations.	
	 */
	public BinderEmitter()
	{
		super("ODCPropertyBinder");
	}

	// Class variables	
	protected String _modelName;
    protected String _eObjectID;
    protected String _eAttributeName;// eAttributeName in model class that is bound to text element
    protected String _htmlProperty;
    protected String _controlChangeEventName;
	//protected ArrayList _targetControlEmitterArray = new ArrayList();
	protected BaseEmitter _targetControlEmitter;
	protected String _targetControlID;
	
	/**
	 * Init method to initialize class variables. Use this in the case when only ONE control is being
	 * associated with a property binder.
	 * <p>
	 * @param modelName			Name of the data model on the page to bind this control to
	 * @param eObjectID			ID of the eObject in the data model to bind this control to
	 * @param eAttributeName	EAttribute name in the eObject that is bound to the control
	 * @param targetControlEmitter		The property binder is being generated for this control.
	 * @param htmlProperty				Param takes two values: 'value' or 'innerHTML' depending on the HTML control
	 * @param controlChangeEventName	Name of JavaScript trigger event --> onChange, etc.
	 */
	public void Init(String modelName, String eObjectID, String eAttributeName, BaseEmitter targetControlEmitter,
					 String htmlProperty, String controlChangeEventName)
	{
		_modelName = modelName;
		_eObjectID = eObjectID;
		_eAttributeName = eAttributeName;
		_htmlProperty = htmlProperty;
		_controlChangeEventName = controlChangeEventName;
		this.setTargetControlEmitter(targetControlEmitter);

		Streamer.debug.Header().println("ModelName: "+_modelName+
										"eObjectID: "+_eObjectID+
										"eAttributeName: "+_eAttributeName+
										"htmlProperty: "+_htmlProperty+
										"controlChangeEventName: "+_controlChangeEventName+
										"targetControlID: "+_targetControlID);
	}

	/**
	 * Init method to initialize class variables. Use this in the case when only ONE control is being
	 * associated with a property binder.
	 * <p>
	 * @param modelName			Name of the data model on the page to bind this control to
	 * @param eObjectID			ID of the eObject in the data model to bind this control to
	 * @param eAttributeName	EAttribute name in the eObject that is bound to the control
	 * @param targetControlID			The ID of target control for which the property binder is being generated.
	 * @param htmlProperty				Param takes two values: 'value' or 'innerHTML' depending on the HTML control
	 * @param controlChangeEventName	Name of JavaScript trigger event --> onChange, etc.
	 */
	public void Init(String modelName, String eObjectID, String eAttributeName, String targetControlID,
					 String htmlProperty, String controlChangeEventName)
	{
		_modelName = modelName;
		_eObjectID = eObjectID;
		_eAttributeName = eAttributeName;
		_htmlProperty = htmlProperty;
		_controlChangeEventName = controlChangeEventName;
		this.setTargetControlID(targetControlID);
		
		Streamer.debug.Header().println("ModelName: "+_modelName+
										"eObjectID: "+_eObjectID+
										"eAttributeName: "+_eAttributeName+
										"htmlProperty: "+_htmlProperty+
										"controlChangeEventName: "+_controlChangeEventName+
										"targetControlID: "+_targetControlID);
	}

	// All set functions to initialize class instance variables.
	public void setModelName     (String  modelName    )  { _modelName      = modelName     ; }
	public void setEObjectID     (String  eObjectID    )  { _eObjectID      = eObjectID     ; }
	public void setEAttributeName(String eAttributeName)  { _eAttributeName = eAttributeName; }
	public void setHTMLProperty	 (String htmlProperty  )  { _htmlProperty	= htmlProperty  ; }
	public void setControlChangeEvent(String changeEventName) { _controlChangeEventName	= changeEventName; }
	public void setTargetControlEmitter(BaseEmitter emitter)
	{ 
		_targetControlEmitter = emitter;
		_targetControlID = _targetControlEmitter.CONTROL_VAR + _targetControlEmitter.getExportID();
	}
	public void setTargetControlID(String targetControlID){ _targetControlID = targetControlID;	}

	/**
	 * Export method emits the JS, HTML code for the text field or text area HTML elements.
	 * The elements are bound to a data model on the page via a PropertyBinder.
	 * <p>
	 * @param out	Writer output stream to write the generated code to
	 * @param PC	Page Context
	 */
	public void Export(Writer out, PageContext PC) throws ExportException, IOException
	{
		// varIndex is set to the same index as the one in the control.
		// Do NOT increment varIndex here using PageContext.
		// Having the same number for the binder as the control will ease understanding of the code.

		//_varIndex = PC.NextInstanceCount();
		
		Streamer.trace.Header().println("Entering BinderEmitter export...");
		
		_varIndex = _targetControlEmitter.getExportID();
		out.write("<script type=\"text/javascript\">");out.write("\n");
		out.write("var "+BINDER_VAR + _varIndex+" = new PropertyBinder(findEObjectByXMIID(WDO4JSModelRoot_"+_modelName+".Root,'"+_eObjectID+"')"			
																	+",'"+_eAttributeName+"'"
																	+",document.getElementById('"+_targetControlID+"')"
																	+",'"+_htmlProperty+"', '"+_controlChangeEventName+"');");
		out.write("\n");
		out.write(BINDER_VAR + _varIndex+".DataBind();");out.write("\n");
		out.write("</script>");out.write("\n");

		Streamer.trace.Header().println("Exiting BinderEmitter export...");
	}
}