/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//  
//OCO Source Materials
//  
//5724-E76 and 5724-E77
//  
// Copyright IBM Corp.  2003
//  
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////

/*
 * Created on Apr 30, 2003
 *
 */
package com.ibm.odcb.jrender.emitters;

import java.io.IOException;
import java.io.Writer;

import com.ibm.odcb.jrender.mediators.ExportException;
import com.ibm.odcb.jrender.mediators.PageContext;
import com.ibm.odcb.jrender.misc.Streamer;


/**
 * @author Kaushal <br>
 * 
 * Java interface to generate code for a standard HTML Button control.
 * This button is NOT tied to any WDO; it did not make sense for a button control
 * to be tied to a WDO.
 */
public class ButtonEmitter extends BaseEmitter {

	public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;

	protected final String _defaultType = "SUBMIT";
	public static final String SUBMIT = "SUBMIT";
	public static final String RESET  = "RESET";
	/**
	 * Default Constructor just calls the base constructor with the component name to
	 * set up the CONTROL and ADAPTER Variables appropriately. If object is instantiated
	 * with the default constructor, developer has to call set() methods to set class variables or
	 * use the Init() method to initialize class vars.
	 */
	public ButtonEmitter()
	{
		super("ODCButton");
	}
	/**
	 * Constructor that takes all the parameter values and calls Init() to initialize class variables.
	 * <p>
	 * @param type				Type of Button (Submit by default, or Reset).
	 * @param value				Value that is sent to the form
	 * @param buttonText		Text displayed on the button itself
	 * @param callBackFunction	Callback function name that handles button events
	 */
	public ButtonEmitter(	String type,
							String value,
							String buttonText,
							String callBackFunction)
	{
		super("ODCButton");
		this.Init(	type,
					value,
					buttonText,
					callBackFunction);
	}

	// Class variables	
	protected String _type = ButtonEmitter.SUBMIT;	// Type is SUBMIT by default
	protected String _value;						// value sent to the form
	protected String _buttonText;					// text displayed on the button itself
	protected String _callBackFunction;
	
	// init() initializes all class variables.
	public void Init(	String type,
					 	String value,
					 	String buttonText,
						String callBackFunction)
	{
		_type		 	= type;
		_value		 	= value;
		_buttonText  	= buttonText;
		_callBackFunction= callBackFunction;
	}

	// All set functions to initialize class instance variables.
	public void setType(String type)					{	_type 			= type;				}
	public void setValue(String value)					{	_value 			= value;			}
	public void setButtonText(String buttonText)		{	_buttonText 	= buttonText;		}
	public void setCallBackFunction(String callBackFunction){_callBackFunction = callBackFunction;	}

	/**
	 * Export method emits the HTML code for rendering the button with the specified button text.
	 * Remember that this button control is NOT tied to the WDO.
	 * <p>
	 * @param out	Writer to output the JS, HTML to
	 * @param PC	Page Context
	 */
	public void Export(Writer out, PageContext PC) throws ExportException, IOException
	{
		Streamer.trace.Header().println("Entering ButtonEmitter export...");
		_varIndex = PC.NextInstanceCount();
		out.write("<button type="+_type);

		out.write(" name=\""+CONTROL_VAR+_varIndex+"\"");
		if(_value != null)
		{
			out.write(" value=\""+_value+"\"");
		}
		if(_callBackFunction != null)
		{
			out.write(" onclick=\""+_callBackFunction+"()\"");
		}
		out.write(">"); out.write("\n");
		out.write(_buttonText); out.write("\n");
		out.write("</button>"); out.write("\n");
		out.write("\n");
		//out.write("<script type=\"text/javascript\">"); out.write("\n");

		// Registering control not needed: being done in JSF (July 30, 2003)
		//out.write(PAGE_CONTROL_VAR+".registerControl(\""+CONTROL_VAR+_varIndex+"\", "+CONTROL_VAR+_varIndex+");"); out.write("\n");

		//out.write("</script>"); out.write("\n");
		Streamer.trace.Header().println("Exiting ButtonEmitter export...");
	}
}