/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//
//OCO Source Materials
//
//5724-E76 and 5724-E77
//
// Copyright IBM Corp.  2003,2004
//
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////
package com.ibm.odcb.jrender.emitters;
import java.io.IOException;
import java.io.Writer;
import java.lang.reflect.Method;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.Locale;
import java.util.ResourceBundle;

import com.ibm.faces.renderkit.html_extended.HxClientRenderUtil;
import com.ibm.odcb.jrender.mediators.PageContext;
import com.ibm.odcb.jrender.misc.NLResourceManager;
import com.ibm.odcb.jrender.misc.Streamer;
import com.ibm.odcb.jrender.misc.URLRewriter;

/**
 * @author tgupta
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class ClientConverterHelper
{
	public static final String DATESTYLE_FULL = "full"; //$NON-NLS-1$
	public static final String DATESTYLE_LONG = "long"; //$NON-NLS-1$
	public static final String DATESTYLE_MEDIUM = "medium"; //$NON-NLS-1$
	public static final String DATESTYLE_SHORT = "short"; //$NON-NLS-1$
	public static final String DATETYPE_DATE = "date"; //$NON-NLS-1$
	public static final String DATETYPE_TIME = "time"; //$NON-NLS-1$
	public static final String DATETYPE_BOTH = "both"; //$NON-NLS-1$
	public static final String NUMBERSTYLE_CURRENCY = "currency"; //$NON-NLS-1$
	public static final String NUMBERSTYLE_NUMBER = "number"; //$NON-NLS-1$
	public static final String NUMBERSTYLE_PERCENT = "percent"; //$NON-NLS-1$
	private final static String PROPERTIES_FILE = "runtimeibm"; //$NON-NLS-1$

	/**
	 * Builds the javascript statement(s) to instanciate a converter and register it.
	 * It returns the id of the converter so it can be looked up in the converter map.
	 *
	 * @param pattern
	 * @param type - one of: "date", "time", or "both"
	 * @param dateStyle - one of: "short", "medium", "long", "full"
	 * @param timeStyle - one of: "short", "medium", "long", "full"
	 * @param locale
	 * @return the javascript new converter statement
	 */
	public static final String getJsDateConverterString(
		String pattern,
		String type,
		String dateStyle,
		String timeStyle,
		Locale locale)
	{
		String format = null;
		if (pattern != null && !pattern.equals("")) //$NON-NLS-1$
			format = pattern;
		else
		{
			DateFormat df = null;
			if (type.equals(DATETYPE_BOTH))
			{ //$NON-NLS-1$
				df =
					DateFormat.getDateTimeInstance(
						getStyleCode(dateStyle),
						getStyleCode(timeStyle),
						locale);
			}
			else
				if (type.equals(DATETYPE_DATE))
				{ //$NON-NLS-1$
					df =
						DateFormat.getDateInstance(
							getStyleCode(dateStyle),
							locale);
				}
				else
					if (type.equals(DATETYPE_TIME))
					{ //$NON-NLS-1$
						df =
							DateFormat.getTimeInstance(
								getStyleCode(timeStyle),
								locale);
					}
					else
					{
						new IllegalArgumentException("Invalid type: " + type); //$NON-NLS-1$
					}
			format = ((SimpleDateFormat) df).toPattern();
		}
		if (null != format)
		{
			StringBuffer javascript = new StringBuffer();
			javascript.append("new ");
			javascript.append(HxClientRenderUtil.HX_VAR_NAME);
			javascript.append(".DateTimeConverter(\"strict:1\", \"format:"); //$NON-NLS-1$
			javascript.append(format);
			javascript.append("\")"); //$NON-NLS-1$
			return javascript.toString();
		}
		return null;
	}
	private static int getStyleCode(String s)
	{
		int dateCode = DateFormat.DEFAULT;
		if (s != null && s != "")
		{ //$NON-NLS-1$
			if (DATESTYLE_SHORT.equalsIgnoreCase(s))
			{
				dateCode = DateFormat.SHORT;
			}
			else
				if (DATESTYLE_MEDIUM.equalsIgnoreCase(s))
				{
					dateCode = DateFormat.MEDIUM;
				}
				else
					if (DATESTYLE_LONG.equalsIgnoreCase(s))
					{
						dateCode = DateFormat.LONG;
					}
					else
						if (DATESTYLE_FULL.equalsIgnoreCase(s))
						{
							dateCode = DateFormat.FULL;
						}
		}
		return dateCode;
	}
	private static String getJsMaskConverterString(String mask)
	{
		return "new MaskConverter('" + mask + "')";
	}
	/**
	 * Builds the javascript statement(s) to instanciate a converter and register it.
	 * It returns the id of the converter so it can be looked up in the converter map.
	 *
	 * @param pattern
	 * @paran type - one of: "currency", "number"
	 * @param currencyCode
	 * @param currencySymbol
	 * @param maxIntegerDigits - nullable
	 * @param minIntegerDigits - nullable
	 * @param maxFractionDigits
	 * @param minFractionDigits
	 * @param groupingUsed - nullable (defaults to true)
	 * @param locale
	 * @return the javascript new converter statement
	 */
	public static final String getJsNumberConverter(
		String pattern,
		String type,
		String currencyCode,
		String currencySymbol,
		Integer maxIntegerDigits,
		Integer minIntegerDigits,
		int maxFractionDigits,
		int minFractionDigits,
		Boolean groupingUsed,
		Locale locale)
	{
		String converterPattern = null;
		try
		{
			// Create and configure the formatter to be used
			NumberFormat formatter = getNumberFormat(pattern, type, locale);
			if (((pattern != null) && !pattern.equals(""))
				|| NUMBERSTYLE_CURRENCY.equals(type))
			{ //$NON-NLS-1$
				configureCurrency(currencyCode, currencySymbol, formatter);
			}
			configureFormatter(
				maxIntegerDigits,
				minIntegerDigits,
				maxFractionDigits,
				minFractionDigits,
				groupingUsed,
				formatter);
			if (formatter instanceof DecimalFormat)
			{
				converterPattern = ((DecimalFormat) formatter).toPattern();
			}
			else
			{
				String formattedResult =
					formatter.format(
						new Double(111111111111111111.111111111111111111));
				converterPattern = formattedResult.replace('1', '#');
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
			return null;
		}
		if (null != converterPattern && converterPattern.length() > 0)
		{
			StringBuffer javascript = new StringBuffer();
			javascript.append("new "); //$NON-NLS-1$
			javascript.append(HxClientRenderUtil.HX_VAR_NAME);
			javascript.append(".NumberConverter(\"strict:1\", \"pattern:");
			javascript.append(converterPattern);
			javascript.append("\", \"locale:"); //$NON-NLS-1$
			DecimalFormatSymbols symbols = new DecimalFormatSymbols(locale);
			javascript.append(symbols.getGroupingSeparator());
			javascript.append(symbols.getDecimalSeparator());
			javascript.append(symbols.getPercent());
			javascript.append(symbols.getPerMill());
			javascript.append(symbols.getMinusSign());
			javascript.append(symbols.getCurrencySymbol());
			javascript.append("\")"); //$NON-NLS-1$
			return javascript.toString();
		}
		return null;
	}
	/**
	 * <p>Return a <code>NumberFormat</code> instance to use for formatting
	 * and parsing in this {@link Converter}.</p>
	 *
	 * @param locale The <code>Locale</code> used to select formatting
	 *  and parsing conventions
	 *
	 * @exception ConverterException if no instance can be created
	 */
	static private NumberFormat getNumberFormat(
		String pattern,
		String type,
		Locale locale)
	{
		if (pattern == null && type == null)
		{
			throw new IllegalArgumentException("Either pattern or type must" + " be specified."); //$NON-NLS-1$ //$NON-NLS-2$
		}
		// If pattern is specified, type is ignored
		if (pattern != null)
		{
			DecimalFormatSymbols symbols = new DecimalFormatSymbols(locale);
			return (new DecimalFormat(pattern, symbols));
		}
		// Create an instance based on the specified type
		else
			if (type.equals(NUMBERSTYLE_CURRENCY))
			{
				return (NumberFormat.getCurrencyInstance(locale));
			}
			else
				if (type.equals(NUMBERSTYLE_NUMBER))
				{
					return (NumberFormat.getNumberInstance(locale));
				}
				else
					if (type.equals(NUMBERSTYLE_PERCENT))
					{
						return (NumberFormat.getPercentInstance(locale));
					}
					else
					{

						throw new IllegalArgumentException(type);
					}
	}
	
	private static Boolean curClassAvailable = null;
	private static Class curClass = null;
	private static Class[] stringArg = new Class[]{String.class};
	
	private static void configureCurrency(String currencyCode, String currencySymbol, NumberFormat formatter) throws Exception {
		//if code or symbol not set do nothing
		if(currencyCode == null && currencySymbol == null){
			return;
		}
		if(curClassAvailable == null) {
				try {
					curClass = Class.forName("java.util.Currency");
					curClassAvailable = Boolean.TRUE;
				} catch (ClassNotFoundException e) {
					//jre version < 1.4
					curClassAvailable = Boolean.FALSE;
				}
		}
		
		//if currencycode is set and currency class is available
		if(currencyCode != null && curClassAvailable.booleanValue()){
			
			Method method = curClass.getMethod("getInstance",stringArg);
			Object cur = method.invoke(null, new Object[]{currencyCode});
			
			Class numFormatClass = Class.forName("java.text.NumberFormat");
			method = numFormatClass.getMethod("setCurrency",new Class[]{curClass});
			method.invoke(formatter, new Object[]{cur});
			return;
		}
		String curSymbol = null;
		
		if (currencySymbol != null){
			curSymbol = currencySymbol;
		} else if (currencyCode != null){
			curSymbol = currencyCode;
			
		}

		if (curSymbol != null){
			DecimalFormat df = (DecimalFormat) formatter;
			DecimalFormatSymbols dfs = df.getDecimalFormatSymbols();
			dfs.setCurrencySymbol(curSymbol);
			df.setDecimalFormatSymbols(dfs);
		}
	}
	
	
	/**
	 * <p>Configure the specified <code>NumberFormat</code> based on the
	 * formatting properties that have been set.</p>
	 *
	 * @param formatter The <code>NumberFormat</code> instance to configure
	 */
	private static void configureFormatter(
		Integer maxIntegerDigits,
		Integer minIntegerDigits,
		int maxFractionDigits,
		int minFractionDigits,
		Boolean groupingUsed,
		NumberFormat formatter)
	{
		if (groupingUsed != null)
		{
			formatter.setGroupingUsed(groupingUsed.booleanValue());
		}
		else
		{
			formatter.setGroupingUsed(true);
		}
		if (maxIntegerDigits != null)
		{
			formatter.setMaximumIntegerDigits(maxIntegerDigits.intValue());
		}
		if (minIntegerDigits != null)
		{
			formatter.setMinimumIntegerDigits(minIntegerDigits.intValue());
		}
		formatter.setMaximumFractionDigits(maxFractionDigits);
		formatter.setMinimumFractionDigits(minFractionDigits);
	}
	public static void main(String[] args)
	{
		String dateConverterString =
			ClientConverterHelper.getJsDateConverterString(
				"MM/dd/yyyy HH:mm:ss z",
				"date",
				"long",
				null,
				Locale.getDefault());
		System.out.println("DateConverterString: " + dateConverterString);
		String currencyConverterString =
			ClientConverterHelper.getJsNumberConverter(
				"#,##0.00;(#,##0.00)",
				"currency",
				"USD",
				"US$",
				null,
				null,
				2,
				2,
				null,
				Locale.getDefault());
		System.out.println(
			"CurrncyConverterString: " + currencyConverterString);
	}
	public static void initJSLibraries(
		Locale locale,
		Writer out,
		PageContext pc)
		throws IOException
	{
		URLRewriter R =
			(URLRewriter) pc.getContextVariable(
				InitializationEmitter._ID_URL_REWRITER);
		initJSLibraries(locale, out, pc, R);
	}
	public static void initJSLibraries(
		Locale locale,
		Writer out,
		PageContext pc,
		URLRewriter R)
		throws IOException
	{
		Streamer.trace.Header().println(
			"Entering ClientConverterHelper.initJSLibraries()");

		StringBuffer sbuffer = new StringBuffer();
		if (BaseEmitter.TestGuard(pc, "CLIENT_CONVERTER_HELPER_JS_FLAG")
			== false)
		{
			sbuffer
				.append("\n<script type=\"text/javascript\" src=\"")
				.append(R.Rewrite(HxClientRenderUtil.HX_LIB_FILENAME))
				.append("\"></script>\n");
			Streamer.debug.Header().print(
				"Output from ClientConverterHelper.initJSLibraries():"
					+ sbuffer.toString());
			out.write(sbuffer.toString());
			try
			{
				String hxclient_SFileFullPath =
					NLResourceManager.getFile(
						"baseurl",
						"jsl/hxclient_S_BF",
						"jsl/hxclient_S_BF",
						locale.toString(),
						R);
				Streamer.debug.Header().println(
					"hxclient_S file path --> " + hxclient_SFileFullPath);
				out.write(
					"<script type=\"text/javascript\" src=\"" + hxclient_SFileFullPath + "\"></script>");
				out.write("\n");
			}
			catch (Exception e1)
			{
				e1.printStackTrace();
				Streamer.error.Header().printStackTrace(e1);
			}
			sbuffer = new StringBuffer();
						sbuffer.append("<script type=\"text/javascript\">\n")
								.append(HxClientRenderUtil.HX_VAR_NAME)
							   .append(".setResourceServer(\'"+ R.Rewrite("") + "\');")
							   .append("</script>\n");

						out.write(sbuffer.toString());

		}
		Streamer.trace.Header().println(
			"Exiting ClientConverterHelper.initJSLibraries()");
	}

	public static String buildLocalizedLibName(
		ResourceBundle bundle,
		String lib)
	{
		if (bundle == null)
		{
			bundle = ResourceBundle.getBundle(PROPERTIES_FILE);
		}
		if (bundle == null)
		{
			return lib;
		}
		String suffix = bundle.getString("Locale.suffix"); //$NON-NLS-1$
		if (suffix == null
			|| suffix.trim().length() == 0
			|| suffix.equals("!Locale.suffix!"))
		{ //$NON-NLS-1$
			return lib;
		}
		int n = lib.lastIndexOf(".js"); //$NON-NLS-1$
		return lib.substring(0, n) + "_" + suffix + ".js"; //$NON-NLS-1$ //$NON-NLS-2$
	}
	/*
	 * This method will add one line of JavaScript code (hX_*.onPageLoad();)
	 * once to a html page, and this method should be called at the end of a page.
	 */
	public static void printOnPageLoad(Writer writer, PageContext pc)
	{
		try
		{
			//We only want to export this code once at the end of a page
			if (BaseEmitter
				.TestGuard(pc, "CLIENT_CONVERTER_HELPER_ONLOAD_FLAG")
				== false)
			{
				writer.write("<script type=\"text/javascript\">");
				writer.write(HxClientRenderUtil.HX_VAR_NAME + ".onPageLoad();\n"); //$NON-NLS-1$
				writer.write("</script>");
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
			Streamer.error.Header().print(
				"Output from ClientConverterHelper.printOnPageLoad()");
		}
	}
}
