/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//  
//OCO Source Materials
//  
//5724-E76 and 5724-E77
//  
// Copyright IBM Corp.  2003
//  
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////

/*
 * Created on July 2, 2003
 *
 */
package com.ibm.odcb.jrender.emitters;

import com.ibm.odcb.jrender.misc.StringUtil;
/**
 * @author Kaushal <br>
 * ControlMedataHelper is a standalone helper class that builds a metadata string 
 * of the form:  ["source EFeature Name", "Target EObject ID", "Target EFeature Name"]
 * from its class vars and helps the EventSelectAndSetEmitter to pass this metadata
 * to the JavaScript SelectAndSet event handler. EventSelectAndSetEmitter holds a
 * list of objects of the type ControlMetadataHelper.
 */
public class ControlMetadataHelper {
	public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;
	
	/**
	 * Constructor
	 * @param sourceEFeatureName the name of the efeature on the source object to be used for the event. Null if the source object is to be used directly.
	 * @param targetEObjectId the id of the target eobject where the data needs to be copied.
	 * @param targetEFeatureName the name of the target efeature to which the data needs to be copied. Null if the data needs to be copied directly to the target eobject.
	 */
	public ControlMetadataHelper(String sourceEFeatureName, String targetEObjectId, String targetEFeatureName) {
		this.Init(sourceEFeatureName, targetEObjectId, targetEFeatureName);
	}
	
	/**
	 * Initializes the <code>ControlMetaDataHelper</code> class. Same does the same job as the constructor.
	 * @param sourceEFeatureName
	 * @param targetEObjectId
	 * @param targetEFeatureName
	 */
	public void Init(String sourceEFeatureName, String targetEObjectId, String targetEFeatureName) {
		this.sourceEFeatureName = sourceEFeatureName;
		this.targetEObjectId = targetEObjectId;
		this.targetEFeatureName = targetEFeatureName;
	}

	protected String sourceEFeatureName, targetEObjectId, targetEFeatureName;

	/**
	 * Convenience function to construct a metaData String as shown in the return field explanation.
	 * The string is constructed from all the paramters supplied when the class is initialized. This metadata string is used by the
	 * SelectAndSet Emitter to pass data to the JavaScript event handler.
	 * 
	 * @return		Returns constructed string like this: ["source EFeature Name", "Target EObject ID", "Target EFeature Name"]
	 */
	public String constructMetadataString() {
		String metadataString = "[";
		metadataString = metadataString.concat(this.buildQuotedString(this.sourceEFeatureName));
		metadataString = metadataString.concat(", ");
		metadataString = metadataString.concat(this.buildQuotedString(this.targetEObjectId));
		metadataString = metadataString.concat(", ");
		metadataString = metadataString.concat(this.buildQuotedString(this.targetEFeatureName));
		metadataString = metadataString.concat("]");
		return metadataString;
	}

	/**
	 * Convenience function to double quote a string and return.
	 * @param tempStr	Input is a regular string: a
	 * @return			Output is a quoted string: "a"
	 */
	protected String buildQuotedString(String tempStr) {
		if (tempStr == null || tempStr.equals("")) {
			return "null";
		} else {
			String s1 = tempStr;
			s1 = StringUtil.QuoteDoubleAndEscape(s1, true);
			s1 = (s1.substring(1, s1.length() - 2)).concat(s1.substring(s1.length() - 1));
			return s1;
		}
	}
}