/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//  
//OCO Source Materials
//  
//5724-E76 and 5724-E77
//  
// Copyright IBM Corp.  2003
//  
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////

/*
 * Created on May 7, 2003
 *
 */
package com.ibm.odcb.jrender.emitters;

import java.io.Writer;
import java.io.IOException;
import com.ibm.odcb.jrender.mediators.ExportException;
import java.util.ArrayList;
import com.ibm.odcb.jrender.mediators.PageContext;
import com.ibm.odcb.jrender.misc.Config;
import com.ibm.odcb.jrender.misc.Streamer;
import com.ibm.odcb.jrender.misc.URLRewriter;
import com.ibm.odcb.jrender.misc.StringUtil;

//import com.ibm.odcb.jrender.misc.Config;

/**
 * @author Kaushal <br>
 *         Java interface to generate code for DataGrid control. Takes the help
 *         of DataGridEmitterHelper which holds a column of data.
 */
public class DataGridEmitter extends BaseEmitter {

	public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;

	public static final int NAV_BAR_BOTTOM = 0;

	public static final int NAV_BAR_TOP = 1;

	public static final int NAV_BAR_BOTH = 2;
	
	 
	private static final String BUTTONS_GO = Config.versionizeFilename("Buttons_Go.gif");
	private static final String BUTTONS_LEFT = Config.versionizeFilename("Buttons_Left.gif");
	private static final String BUTTONS_RIGHT = Config.versionizeFilename("Buttons_Right.gif");
	private static final String BUTTONS_FIRST = Config.versionizeFilename("Buttons_First.gif");
	private static final String BUTTONS_LAST = Config.versionizeFilename("Buttons_Last.gif");	    
	private static final String N_SORT_A_Z = Config.versionizeFilename("n-sort-a-z.gif");
    private static final String N_SORT_Z_A = Config.versionizeFilename("n-sort-z-a.gif");	    
    private static final String CLEAR = Config.versionizeFilename("clear.gif");
    private static final String SELECT_ALL = Config.versionizeFilename("select_all.gif");
    private static final String SELECT_NONE = Config.versionizeFilename("select_none.gif");
    private static final String ADD_ROW = Config.versionizeFilename("add_row.gif");
    private static final String REMOVE_ROW = Config.versionizeFilename("remove_row.gif");
    private static final String OK = Config.versionizeFilename("ok.gif");
    private static final String CANCEL = Config.versionizeFilename("cancel.gif");
    private static final String SHOW_COLS = Config.versionizeFilename("Show_Cols.gif");
    private static final String HIDE_COLUMN = Config.versionizeFilename("Hide_Column.gif");
    private static final String PAGING_TABLE_TOP_RIGHT = Config.versionizeFilename("PagingTableTopRight.gif");
    private static final String PAGING_TABLE_BOTTOM_RIGHT = Config.versionizeFilename("PagingTableBottomRight.gif");
    private static final String TABLE_LEFT = Config.versionizeFilename("tableleft.gif");
    
    
    private static final String DATAGRIDCONTROL = Config.versionizeFilename("datagridcontrol.js");
    private static final String DATAGRIDADAPTER = Config.versionizeFilename("datagridadapter.js");
    private static final String JSL_DATAGRID_C = Config.versionizeFilename("jsl_datagrid_c.js");
	/**
	 * Default constructor: just calls the base emitter constructor with the
	 * name of the component.
	 */
	public DataGridEmitter() {
		super("ODCDataGrid");
	}

	// Class variables
	protected String _eObjectID;

	protected String _eReferenceName;

	protected String _modelName;

	protected String _metaDataArrayString;

	protected String _width;

	protected boolean _readOnlyFlag = false; // by default you can read/write

	protected boolean _allowColumnHideFlag = false; // by default you cannot show/hide columns
	
	protected String _scrollSize = ""; // -1 implies no scrolling allowed. > 0 implies scrolling allowed
	
	protected int _pageSize = -1; // -1 implies no paging allowed. > 0 implies paging allowed

	// paging allowed.

	protected String _Selection = null;

	protected String _rowFilter = null;

	protected String _styleClass = null;

	protected ArrayList _colArray = new ArrayList(7);

	protected int _navBarPosition = NAV_BAR_BOTTOM; // default -> 0 : Bottom.

	protected boolean _showRowIndex = false; // initialize to default: false.

	protected String _onhighlight, _onselect, _onunselect, _onpage, _onsort,
			_onfilter, _onselectall, _onunselectall, _onrowadd, _onrowremove;

	/**
	 * Base Init method for DataGridEmitter class. <br>
	 * 
	 * @param modelName
	 *            Name of the data model this control is bound to
	 * @param eObjectID
	 *            The eObject in the data model this control talks to
	 * @param eReferenceName
	 *            Reference name in the eObject this control talks to
	 *            specifically
	 * @param readOnlyFlag
	 *            Are the fields readOnly or not
	 * @param allowColumnHideFlag
	 *            Are the show hide column controls displayed 
	 * @param scrollSize
	 *            Size of the scrolling panel 
	 * @param pageSize
	 *            Size of the page to display; Records > pageSize will be split
	 *            into multiple pages
	 */
	public void Init(String modelName, String eObjectID, String eReferenceName,
			boolean readOnlyFlag, boolean allowColumnHideFlag, String scrollSize, int pageSize) {
		Init(modelName, eObjectID, eReferenceName, readOnlyFlag, allowColumnHideFlag, scrollSize, null, pageSize,
				false, null, false, null, "-1", null, null, NAV_BAR_BOTTOM,
				false, null, null, null, null, null, null, null, null, null, null);
	}

	/**
	 * Base Init method PLUS initialization of rowFilter expression and
	 * styleClass string. <br>
	 * 
	 * @param modelName
	 *            Name of the data model this control is bound to
	 * @param eObjectID
	 *            The eObject in the data model this control talks to
	 * @param eReferenceName
	 *            Reference name in the eObject this control talks to
	 *            specifically
	 * @param readOnlyFlag
	 *            Are the fields readOnly or not
	 * @param allowColumnHideFlag
	 *            Are the show hide column controls displayed
	 * @param scrollSize
	 *            Size of the scrolling panel 
	 * @param pageSize
	 *            Size of the page to display; Records > pageSize will be split
	 *            into multiple pages
	 * @param rowFilter
	 *            Expression specifying row filter
	 * @param styleClass
	 *            Prefix specifying CSS for multiple data grids on page
	 */
	public void Init(String modelName, String eObjectID, String eReferenceName,
			boolean readOnlyFlag, boolean allowColumnHideFlag, String scrollSize, int pageSize, String rowFilter,
			String styleClass) {
		Init(modelName, eObjectID, eReferenceName, readOnlyFlag, allowColumnHideFlag, scrollSize, null, pageSize,
				false, null, false, null, "-1", rowFilter, styleClass,
				NAV_BAR_BOTTOM, false, null, null, null, null, null, null,
				null, null, null, null);
	}

	/**
	 * Init method that initializes ALL the class variables. This Init adds
	 * selection params to the interface as well. <br>
	 * 
	 * @param modelName
	 *            Name of the data model this control is bound to
	 * @param eObjectID
	 *            The eObject in the data model this control talks to
	 * @param eReferenceName
	 *            Reference name in the eObject this control talks to
	 *            specifically
	 * @param readOnlyFlag
	 *            Are the fields readOnly or not
	 * @param allowColumnHideFlag
	 * 				Are the show hide column controls displayed 
	 * @param scrollSize
	 *            Size of the scrolling panel 
	 * @param pageSize
	 *            Size of the page to display; Records > pageSize will be split
	 *            into multiple pages
	 * @param SelectionColumnName
	 *            Name of the column that is selected
	 * @param MultiSelection
	 *            Multiple selection allowed or not
	 * @param SelectionAlignment
	 *            Alignment of selection
	 * @param SelectionWidth
	 *            Width of selection
	 * @param rowFilter
	 *            Expression specifying row filter
	 * @param styleClass
	 *            Prefix specifying CSS for multiple data grids on page
	 * @param navBarPosition
	 *            Position of navigation bar in Datagrid; 0-bottom, 1-top, and
	 *            2-both
	 * @param showRowIndex
	 *            Show the row index or not
	 * @param onhighlight
	 *            Event Handler for the &quot;onhightlight&quot; event
	 * @param onselect
	 *            Event Handler for the &quot;onselect&quot; event
	 * @param onunselect
	 *            Event Handler for the &quot;onunselect&quot; event
	 */
	public void Init(String modelName, String eObjectID, String eReferenceName,
			boolean readOnlyFlag, boolean allowColumnHideFlag, String scrollSize, String width,  int pageSize, boolean showSelectionColumn,
			String SelectionColumnName, boolean MultiSelection,
			String SelectionAlignment, String SelectionWidth, String rowFilter,
			String styleClass, int navBarPosition, boolean showRowIndex,
			String onhighlight, String onselect, String onunselect,
			String onpage, String onfilter, String onsort, String onselectall,
			String onunselectall, String onrowadd, String onrowremove) {
		_eObjectID = eObjectID;
		_eReferenceName = eReferenceName;
		_modelName = modelName;
		_readOnlyFlag = readOnlyFlag;
		_allowColumnHideFlag = allowColumnHideFlag;
		_scrollSize = scrollSize;
		_pageSize = pageSize;
		_width = width;
		//_Selection = SelectionColumnName == null ? null :
		// "['"+SelectionColumnName+"\','"+(MultiSelection?"checkbox":"radio")+"','"+SelectionAlignment+"',"+SelectionWidth+"]";
		if (showSelectionColumn) {
			_Selection = "["
					+ (SelectionColumnName == null ? null : "'"
							+ SelectionColumnName + "'")
					+ ",'"
					+ (MultiSelection ? "checkbox" : "radio")
					+ "',"
					+ (SelectionAlignment == null ? null : "'"
							+ SelectionAlignment + "'") + "," + SelectionWidth
					+ "]";
		} else {
			_Selection = null;
		}
		_rowFilter = rowFilter;
		_styleClass = styleClass;

		//----Begin additions by Kaushal Dec. 18,2003.
		if ((navBarPosition >= 0) && (navBarPosition <= 2))
			_navBarPosition = navBarPosition;
		else
			_navBarPosition = NAV_BAR_BOTTOM;

		_showRowIndex = showRowIndex;
		//----End additions Dec.18,2003

		_onhighlight = onhighlight;
		_onselect = onselect;
		_onunselect = onunselect;
		_onpage = onpage;
		_onfilter = onfilter;
		_onsort = onsort;
		_onselectall = onselectall;
		_onunselectall = onunselectall;
		_onrowadd = onrowadd;
		_onrowremove = onrowremove;

		Streamer.debug.Header().println(
				"eObjectID: " + _eObjectID + "\n" + "eReferenceName: "
						+ this._eReferenceName + "\n" + "modelName: "
						+ this._modelName + "\n" + "readOnlyFlag: "
						+ this._readOnlyFlag  + "\n" + "allowColumnHideFlag: "
						+ this._allowColumnHideFlag  + "\n" + "scrollSize: "
						+ this._scrollSize + "\n" + "pageSize: "
						+ this._pageSize + "\n" + "selection: "
						+ this._Selection + "\n" + "rowFilter: "
						+ this._rowFilter + "\n" + "styleClass: "
						+ this._styleClass + "\n" + "navBarPosition "
						+ this._navBarPosition + "\n" + "showRowIndex: "
						+ this._showRowIndex);
	}

	// All set functions to initialize class instance variables.
	public void setEObjectID(String eObjectID) {
		_eObjectID = eObjectID;
	}

	public void setEReferenceName(String eReferenceName) {
		_eReferenceName = eReferenceName;
	}

	public void setModelName(String modelName) {
		_modelName = modelName;
	}

	public void setReadOnlyFlag(boolean readOnlyFlag) {
		_readOnlyFlag = readOnlyFlag;
	}

	public void setAllowColumnHideFlag(boolean allowColumnHideFlag) {
		_allowColumnHideFlag = allowColumnHideFlag;
	}
	
	public void setScrollSize(String scrollSize) {
		_scrollSize = scrollSize;
	}	
	
	public void setPageSize(int pageSize) {
		_pageSize = pageSize;
	}

	public void setRowFilter(String rowFilter) {
		_rowFilter = rowFilter;
	}

	public void setStyleClass(String styleClass) {
		_styleClass = styleClass;
	}

	public void addColumn(DataGridEmitterHelper c) {
		_colArray.add(c);
	}

	public void setNavBarPosition(int pos) {
		_navBarPosition = pos;
	}

	public void setShowRowIndex(boolean showRowIndex) {
		_showRowIndex = showRowIndex;
	}

	public void setOnhighlight(String onhighlight) {
		_onhighlight = onhighlight;
	}

	public void setOnselect(String onselect) {
		_onselect = onselect;
	}

	public void setOnunselect(String onunselect) {
		_onunselect = onunselect;
	}

	public void setOnpage(String onpage) {
		_onpage = onpage;
	}

	public void setOnsort(String onsort) {
		_onsort = onsort;
	}

	public void setOnfilter(String onfilter) {
		_onfilter = onfilter;
	}

	public void setOnselectall(String onselectall) {
		_onselectall = onselectall;
	}

	public void setOnunselectall(String onunselectall) {
		_onunselectall = onunselectall;
	}

	public void setOnrowadd(String onrowadd) {
		_onrowadd = onrowadd;
	}
	
	public void setOnrowremove(String onrowremove) {
		_onrowremove = onrowremove;
	}

	public void setWidth(String width){
		_width = width;
	}
	

	/**
	 * Method cycles through all columns in datagrid and constructs a metadata
	 * array string as needed by the DataGrid JS control. The string tells the
	 * JS control the attributes of each column--whether it is readOnly or not,
	 * right justified or center justified, etc. etc. Each column
	 * (DatagridEmitterHelper) holds its own data.
	 * <p>
	 */
	public void constructMetaDataArrayString() {
		String ms = "[";
		for (int i = 0; i < _colArray.size(); ++i) {
			DataGridEmitterHelper cTemp = (DataGridEmitterHelper) _colArray
					.get(i);
			if (i == 0) {
				ms = ms + cTemp.getMetaDataArrayString();
			} else {
				ms = ms + ", " + cTemp.getMetaDataArrayString();
			}
		}
		ms = ms + "]";
		this._metaDataArrayString = ms;
		Streamer.debug.Header().println(
				"Column Metadata String: " + this._metaDataArrayString);
	}

	/**
	 * Export method emits the JS code for the data grid control and its
	 * adapter. This is the code for binding the model to the control so that
	 * any changes in model or the control data are reflected immediately at the
	 * other end.
	 * <p>
	 * 
	 * @param out
	 *            Writer output stream to write the generated code to
	 * @param PC
	 *            Page Context that holds the context of the page
	 */
	public void Export(Writer out, PageContext PC) throws ExportException,
			IOException 
	{
		//Check if encoding should occur
		boolean isFaces=StringUtil.checkEncoding();
		
		Streamer.trace.Header().println("Entering DataGridEmitter export...");
		_varIndex = PC.NextInstanceCount();
		
		out.write("<script type=\"text/javascript\">ODCProgressBar.startRenderControl(datagrid);</script>\n");
		URLRewriter R = (URLRewriter) PC
				.getContextVariable(InitializationEmitter._ID_URL_REWRITER);
		boolean debugMode = PC
				.getContextVariable(InitializationEmitter._ID_DEBUG_MODE) == InitializationEmitter.DEBUG_YES ? true
				: false;

		if (TestGuard(PC, "DATAGRID_JS_FLAG") == false) {
			if (debugMode == true) {
				out.write("\n<script type=\"text/javascript\" src=\""
						+ R.Rewrite("jsl/datagrid/" + DATAGRIDCONTROL)
						+ "\"></script>" + "\n<script type=\"text/javascript\" src=\""
						+ R.Rewrite("jsl/datagrid/" + DATAGRIDADAPTER)
						+ "\"></script>");
				Streamer.trace
						.Header()
						.println(
								"Included individual datagrid JS control and adapter files...");
			} else // no need to TestGuard for DATAGRID LIB FLAG
			{
				out
						.write("<script type=\"text/javascript\" src=\""
								+ R.Rewrite("jsl/" + JSL_DATAGRID_C)
								+ "\"></script>\n");
				Streamer.trace.Header().println(
						"Included compressed datagrid JS file...");
			}
		}

		//String _StyleSheetDir =
		// Config.getString(Config.PROP_CLIENT_STYLESHEETDIR);
		//out.write("<link rel=\"stylesheet\" type=\"text/css\" href=\"" +
		// R.Rewrite(_StyleSheetDir + "/datagrid.css") + "\">\n"
		out.write("<span id=\"" 
				+ StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces)
				+ "\"></span>\n"
				+ "<script type=\"text/javascript\">\n" + "var td = document.getElementById(\""
				+ StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces)
				+ "\");\n");

		out.write("var " 
				+ StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces)
				+ " = new DataGrid(td, new Array(");
		for (int i = 0; i < this._colArray.size(); ++i) {
			DataGridEmitterHelper dgeh = (DataGridEmitterHelper) _colArray
					.get(i);
			if (i != 0) {
				out.write(", ");
			}
			out.write("\"" + dgeh.getDisplayLabel() + "\"");
		}
		out.write("), URL_REWRITER_PREFIX);\n");

		this.constructMetaDataArrayString();

		//Encoding variables for Portal
		_modelName=StringUtil.encodeString(_modelName,isFaces);
		String _modelRoot=StringUtil.encodeString("WDO4JSModelRoot_"
				+ _modelName + ".Root",isFaces);
		_eObjectID = StringUtil.encodeString(_eObjectID,isFaces) ;
		
		out.write(
				StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces)
				+ ".metaData = "
				+ _metaDataArrayString + ";\n" + "var " 
				+ StringUtil.encodeString(ADAPTER_VAR + _varIndex,isFaces)
				+ " = new DatagridAdapter(" 
				+ StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces)
				+ ");\n"

				// Set the model root
				+ StringUtil.encodeString(ADAPTER_VAR + _varIndex,isFaces)
				+ ".setModel("+_modelRoot+");\n"

				// Set the 'type of the object' to propertyName. You are
				// associating this object class to datagrid.
				+ StringUtil.encodeString(ADAPTER_VAR + _varIndex,isFaces)
				+ ".propertyName = \""
				+ _eReferenceName + "\";\n"

				// get handle to object given eObjectID-->XMI ID (e.g. "DIRE1");
				// set parentEObject in datagrid to object handle.
				+ "var rootObject = findEObjectByXMIID("+_modelRoot+", \"" + _eObjectID + "\");\n"
				+ StringUtil.encodeString(ADAPTER_VAR + _varIndex,isFaces)
				+ ".parentEObject = rootObject;\n"

				+ StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces)
				+ ".ReadOnly = " + _readOnlyFlag
				+ ";\n"
				+ StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces)
				+ ".AllowColumnHide = " + _allowColumnHideFlag
				+ ";\n");

		if(_scrollSize != null && !_scrollSize.equals("")){			
			out.write(
					StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces)
					+ ".AllowScrolling = true;\n"
					+ StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces)
					+ ".ScrollSize = \"" + _scrollSize + "\";\n");
		}

		if (_rowFilter != null) {
			out.write(
					StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces)
					+ ".RowFilter = \"" + _rowFilter
					+ "\";\n");
		}
		if (_styleClass != null) {
			out.write(
					StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces)
					+ ".setStyleClass(\""
					+ _styleClass + "\");\n");
		}
		//remove the condition so that when pageSize is <=0, display all the
		// rows.
		if (_pageSize > 0) {
			out.write(
					StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces)
					+ ".AllowPaging = true;\n"
					+ StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces)
					+ ".PageSize = " + _pageSize + ";\n");
		}
		else {
			out.write(
					StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces)
					+ ".AllowPaging = false;\n"
					+ StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces)
					+ ".PageSize = " + _pageSize + ";\n");
		}
			
		if (_Selection != null) {
			out.write(
					StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces)
					+ ".selectCol = " + _Selection
					+ ";\n");
		}
		if(_width != null && !_width.equals("")){
			out.write(
					StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces)
					+ ".setWidth(\"" + _width + "\");\n");
		}

		// Added by Kaushal Dec. 18. 2003
		out.write(
				StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces)
				+ ".NavBarPosition = "
				+ _navBarPosition + ";\n" 
				+ StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces)
				+ ".showRowIndex = " + _showRowIndex + ";\n");
			
		//encode URLs for icons in portal

		out.write(StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces) + ".datagridIcons[\"go\"]=\""+R.Rewrite("jsl/datagrid/icons/" + BUTTONS_GO)+"\";\n");
		out.write(StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces) + ".datagridIcons[\"previous\"]=\""+R.Rewrite("jsl/datagrid/icons/" + BUTTONS_LEFT)+"\";\n");
	    out.write(StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces) + ".datagridIcons[\"next\"]=\""+R.Rewrite("jsl/datagrid/icons/" + BUTTONS_RIGHT)+"\";\n");
	    out.write(StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces) + ".datagridIcons[\"start\"]=\""+R.Rewrite("jsl/datagrid/icons/" + BUTTONS_FIRST)+"\";\n");
	    out.write(StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces) + ".datagridIcons[\"end\"]=\""+R.Rewrite("jsl/datagrid/icons/" + BUTTONS_LAST)+"\";\n");	    
	    out.write(StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces) + ".datagridIcons[\"sortup\"]=\""+R.Rewrite("jsl/datagrid/icons/" + N_SORT_A_Z)+"\";\n");
	    out.write(StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces) + ".datagridIcons[\"sortdown\"]=\""+R.Rewrite("jsl/datagrid/icons/" + N_SORT_Z_A)+"\";\n");	    
	    out.write(StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces) + ".datagridIcons[\"clear\"]=\""+R.Rewrite("jsl/datagrid/icons/" + CLEAR)+"\";\n");
	    out.write(StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces) + ".datagridIcons[\"selectall\"]=\""+R.Rewrite("jsl/datagrid/icons/" + SELECT_ALL)+"\";\n");
	    out.write(StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces) + ".datagridIcons[\"unselectall\"]=\""+R.Rewrite("jsl/datagrid/icons/" + SELECT_NONE)+"\";\n");
	    out.write(StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces) + ".datagridIcons[\"addrow\"]=\""+R.Rewrite("jsl/datagrid/icons/" + ADD_ROW)+"\";\n");
	    out.write(StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces) + ".datagridIcons[\"delrow\"]=\""+R.Rewrite("jsl/datagrid/icons/" + REMOVE_ROW)+"\";\n");
	    out.write(StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces) + ".datagridIcons[\"acceptrow\"]=\""+R.Rewrite("jsl/datagrid/icons/" + OK)+"\";\n");
	    out.write(StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces) + ".datagridIcons[\"cancelrow\"]=\""+R.Rewrite("jsl/datagrid/icons/" + CANCEL)+"\";\n");
	    out.write(StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces) + ".datagridIcons[\"showallcols\"]=\""+R.Rewrite("jsl/datagrid/icons/" + SHOW_COLS)+"\";\n");
	    out.write(StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces) + ".datagridIcons[\"hidecol\"]=\""+R.Rewrite("jsl/datagrid/icons/" + HIDE_COLUMN)+"\";\n");
	    out.write(StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces) + ".datagridIcons[\"wpsPagingTableHeaderEnd\"]=\""+R.Rewrite("jsl/datagrid/icons/" + PAGING_TABLE_TOP_RIGHT)+"\";\n");
	    out.write(StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces) + ".datagridIcons[\"wpsPagingTableFooterEnd\"]=\""+R.Rewrite("jsl/datagrid/icons/" + PAGING_TABLE_BOTTOM_RIGHT)+"\";\n");
	    out.write(StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces) + ".datagridIcons[\"wpsPagingTableHeaderStart\"]=\""+R.Rewrite("jsl/datagrid/icons/" + TABLE_LEFT)+"\";\n");

	    
	    
		//add events
		if (_onhighlight != null && !_onhighlight.equals("")) {
			out.write(
					StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces)
					+ ".addHandler(\"OnHighlight\",\"" + _onhighlight
					+ "\");\n");
		}
		if (_onselect != null && !_onselect.equals("")) {
			out.write(
					StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces)
					+ ".addHandler(\"OnSelect\",\""
					+ _onselect + "\");\n");
		}
		if (_onunselect != null && !_onunselect.equals("")) {
			out.write(
					StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces)
					+ ".addHandler(\"OnUnSelect\",\""
					+ _onunselect + "\");\n");
		}
		if (_onselectall != null && !_onselectall.equals("")) {
			out.write(
					StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces)
					+ ".addHandler(\"OnSelectAll\",\"" + _onselectall
					+ "\");\n");
		}
		if (_onunselectall != null && !_onunselectall.equals("")) {
			out.write(
					StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces)
					+ ".addHandler(\"OnUnSelectAll\",\"" + _onunselectall
					+ "\");\n");
		}
		if (_onpage != null && !_onpage.equals("")) {
			out.write(
					StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces)
					+ ".addHandler(\"OnPage\",\""
					+ _onpage + "\");\n");
		}
		if (_onfilter != null && !_onfilter.equals("")) {
			out.write(
					StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces)
					+ ".addHandler(\"OnFilter\",\""
					+ _onfilter + "\");\n");
		}
		if (_onsort != null && !_onsort.equals("")) {
			out.write(
					StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces)
					+ ".addHandler(\"OnSort\",\""
					+ _onsort + "\");\n");
		}
		if (_onrowadd != null && !_onrowadd.equals("")) {
			out.write(
					StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces)
					+ ".addHandler(\"OnRowAdd\",\""
					+ _onrowadd + "\");\n");
		}
		
		if (_onrowremove != null && !_onrowremove.equals("")) {
			out.write(
					StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces)
					+ ".addHandler(\"OnRowRemove\",\""
					+ _onrowremove + "\");\n");
		}

		if(_scrollSize != null && !_scrollSize.equals("")){
			out.write(
			"if (window.addEventListener){window.addEventListener(\"resize\", function() {" 
			+ StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces)
			+ ".redisplayGrid();}, true);} else if (window.attachEvent){window.attachEvent(\"onresize\", function() {" 
			+ StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces)
			+ ".redisplayGrid();}, true);}\n"
			);
		}
		
		out.write(
				StringUtil.encodeString(ADAPTER_VAR + _varIndex,isFaces)
				+ ".bind();\n" 
				+ StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces)
				+ ".ShowGrid();\n"
				+ "ODCProgressBar.endRenderControl(datagrid);\n"
				+ "</script>\n");
		Streamer.trace.Header().println("Exiting DataGridEmitter export...");
	}
}