/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//  
//OCO Source Materials
//  
//5724-E76 and 5724-E77
//  
// Copyright IBM Corp.  2003
//  
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////
/*
 * Created on June 9, 2003
 *
 */
package com.ibm.odcb.jrender.emitters;
//import java.util.ArrayList;
import com.ibm.odcb.jrender.misc.Streamer;
//import com.ibm.odcb.jrender.misc.StringUtil;
//import com.ibm.odcb.jrender.mediators.ExportException;
//import com.ibm.odcb.jrender.mediators.PageContext;
/**
 * @author Kaushal <br>
 * 
 * DataGridEmitterHelper is a Java interface to create a data grid column with
 * the relevant attributes.
 */
public class DataGridEmitterHelper
{
	public static final String COPYRIGHT =
		com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;
	/**
	 * Constructor is initialized with parameters, which in turn calls the Init method
	 * to initialize class variables.
	 * <p>		
	 * @param eAttributeName	Attribute name in eClass that will be displayed
	 * @param displayLabel		Label to display in the column header
	 * @param readOnly			Is this column readOnly or not
	 * @param alignment			Alignment of this column (Left, Right, or Center)
	 * @param width				Width of the column
	 * @param underline			Should the column be underlined or not
	 * @param converterCreationScript	JavaScript code to represent a JS object of a converter.
	 */
	public DataGridEmitterHelper(
		String eAttributeName,
		String displayLabel,
		boolean readOnly,
		String alignment,
		String width,
		boolean underline,
		String converterCreationScript)
	{
		Init(
			eAttributeName,
			displayLabel,
			readOnly,
			alignment,
			width,
			underline,
			converterCreationScript);
	}
	
	/**
	 * Intializes the <code>DataGridEmitterHelper</code> class. Same as the constructor.
	 * @param eAttributeName
	 * @param displayLabel
	 * @param readOnly
	 * @param alignment
	 * @param width
	 * @param underline
	 * @param converterCreationScript
	 */
	public void Init(
		String eAttributeName,
		String displayLabel,
		boolean readOnly,
		String alignment,
		String width,
		boolean underline,
		String converterCreationScript)
	{
		_eAttributeName = eAttributeName;
		_displayLabel = displayLabel;
		_readOnly = readOnly;
		_alignment = alignment;
		_width = width;
		_underline = underline;
		_converterCreationScript = converterCreationScript;
	}
	// class instance variables
	protected String _eAttributeName;
	protected String _displayLabel;
	protected boolean _readOnly;
	protected String _alignment;
	protected String _width;
	protected boolean _underline;
	protected String _converterCreationScript = null;
	protected String	_metaDataString = "";

	/**
	 * Sets the alignment.
	 * @param alignment The alignment to set
	 */
	public void setAlignment(String alignment)
	{
		this._alignment = alignment;
	}
	
	/**
	 * Sets the displayLabel.
	 * @param displayLabel The displayLabel to set
	 */
	public void setDisplayLabel(String displayLabel)
	{
		this._displayLabel = displayLabel;
	}
	/**
	 * Sets the eAttributeName.
	 * @param eAttributeName The eAttributeName to set
	 */
	public void setEAttributeName(String eAttributeName)
	{
		this._eAttributeName = eAttributeName;
	}
	/**
	 * Sets the readOnly.
	 * @param readOnly The readOnly to set
	 */
	public void setReadOnly(boolean readOnly)
	{
		this._readOnly = readOnly;
	}
	
	/**
	 * Sets the width.
	 * @param width The width to set
	 */
	public void setWidth(String width)
	{
		this._width = width;
	}
	/**
	 * Sets underline parameter to true or false
	 * @param underline
	 */
	public void setUnderline(boolean underline)
	{
		this._underline = underline;
	}
	/**
	 * Gets the display Label string used to set the label of the column in data grid.
	 * @return Returns display label string
	 */
	public String getDisplayLabel()
	{
		return this._displayLabel;
	}
	/**
	 * Constructs the metaDataString from the individual params provided to this class. This
	 * metaDataString for each column is in turn used by Datagrid Emitter to construct an array
	 * of meta Data strings for specifying the format for the entire data grid's columns.
	 *
	 * @return Returns the constructed meta data string from the individual parameters.
	 */
	public String getMetaDataArrayString()
	{
		
		if(_converterCreationScript == null)
		{
			_converterCreationScript ="";
		}
		
		// Each string has to be checked if it is null or not; if not, we need to output it with surrounding
		// quotes in the string...so we use the convenience function in StringUtil, which puts a backslash
		// and double quotes to each string. Then we have to remove the backslash...we use substring etc. to
		// remove the backslashes.
		// E.g.: input is USD, after StringUtil's convenience function, it becomes \"USD\"
		// Final output is "USD"
		
		String tempString = "";
		tempString =
			"[\""
				+ this._eAttributeName
				+ "\", "
				+ "["
				+ this._readOnly
				+ ", \""
				+ this._alignment
				+ "\", "
				+ this._width
				+ ", "
				+ this._underline
				+ "], "
				+ _converterCreationScript
				+ "]";  
		this._metaDataString = tempString;
		Streamer.debug.Header().println(
			"Datagrid meta data string: " + this._metaDataString);
		return _metaDataString;
	}
}