/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//  
//OCO Source Materials
//  
//5724-E76 and 5724-E77
//  
// Copyright IBM Corp.  2003
//  
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////
/*
 * Created on Apr 30, 2003
 *
 */
package com.ibm.odcb.jrender.emitters;
import java.io.Writer;
import java.io.IOException;
import java.util.Calendar;
import java.util.Locale;

import com.ibm.faces.renderkit.html_extended.HxClientRenderUtil;
import com.ibm.odcb.jrender.mediators.ExportException;
import com.ibm.odcb.jrender.mediators.InvalidParameterException;
import com.ibm.odcb.jrender.mediators.PageContext;
import com.ibm.odcb.jrender.misc.Streamer;
/**
 * @author Jianren Li <br>
 * 
 * Java interface to generate JS code for Date Picker
 */
public class DatePickerEmitter extends BaseEmitter
{
	public static final String COPYRIGHT =
		com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;
	private static final String STYLE_SHORT = "short"; //$NON-NLS-1$
	private static final String STYLE_MEDIUM = "medium"; //$NON-NLS-1$
	private static final String STYLE_LONG = "long"; //$NON-NLS-1$
	private static final String STYLE_FULL = "full"; //$NON-NLS-1$
	static private final String DEFAULT_DATE_FORMAT = "yyyy/MM/dd"; //$NON-NLS-1$
	static private final String DEFAULT_BUTTON_BORDER = "-1"; //$NON-NLS-1$
	static private final String DEFAULT_BASE_YEAR_OFFSET = "80"; //$NON-NLS-1$
	private static final String DATETYPE_DATE = "date"; //$NON-NLS-1$
	//static private final String DEFAULT_STRICTNESS = "1"; //$NON-NLS-1$
	/* 
	 * Tmp flag indicating whether the date picker should be rendered purely as
	 * a client side JSFDatePicker(...) instantiation or whether to use the alternative
	 * route, i.e. using JS JSFAddConverter(...) JSFAddValidator(...) etc.
	 */
	static private final boolean RENDER_AS_JSFDATEPICKER_ONLY = false;
	Locale locale = Locale.US;
	String inputTextId = null;
	String pattern, dateStyle, timeStyle = null;
	String type = DATETYPE_DATE;
	String firstDay, buttonBorder;
	boolean required = false;
	String cSSPrefix = null;
	String btnColor = null;
	//String strict = null;
	// Constructors; default one is empty; second one does all the work.
	// If object is instantiated with default one, developer has to call
	// set() methods to set class variables.
	public DatePickerEmitter()
	{
		super("ODCDatePicker");
	}
	/* init() initializes all class variables.
		 * @param inputTextId The id of InputText field on HTML page, mandatory
		 */
	public void Init(String inputTextId) throws InvalidParameterException
	{
		Init(
			inputTextId,
			null,
			null,
			null,
			null,
			null,
			null,
			null,
			null,
			null,
			false);
	}
	/* init() initializes all class variables.
	 * @param inputTextId The id of InputText field on HTML page, mandatory
	 * @param locale The locale of the page, optional, default will be en_US.
	 * @param all other parameters are optional
	 */
	public void Init(
		String inputTextId,
		Locale locale,
		String pattern,
		String type,
		String dateStyle,
		String timeStyle,
		String firstDay,
		String buttonBorder,
		String cSSPrefix,
		String btnColor,
		boolean required) throws InvalidParameterException
	{
		
		Streamer.trace.Header().println(
							"Entering DatePickeremitter.Init()");
		if (inputTextId != null && !inputTextId.equals(""))
		{
			this.inputTextId = inputTextId;
		}
		else
		{
			Streamer.error.Header().print("The id for inputtext field is either null or empty.");
			//throw an exception
			throw new InvalidParameterException("The id for inputtext field is either null or empty."); 
		}
		if (locale != null)
		{
			this.locale = locale;
		}
		this.pattern = pattern;
		if (type != null && !type.equals(""))
		{
			this.type = type;
		}
		this.dateStyle = dateStyle;
		this.timeStyle = timeStyle;
		this.firstDay = firstDay;
		this.buttonBorder = buttonBorder;
		this.cSSPrefix = cSSPrefix;
		this.btnColor = btnColor;
		this.required = required;
		Streamer.trace.Header().println(
									"Exiting DatePickeremitter.Init()");
	}
	/**
	 * Export method generates the JS code for displaying the date picker control.
	 * It is not bound to any model yet.
	 * <p>
	 * @param out	Writer output stream to write the code to
	 * @param PC	Page Context
	 */
	public void Export(Writer out, PageContext PC)
		throws ExportException, IOException
	{
		Streamer.trace.Header().println(
					"Entering DatePickeremitter.Export()");
		
		out.write("<script type=\"text/javascript\">"); out.write("\n");
		
		Writer writer = out;
		String sRequired = null;
		//UIInput input = null;
		/* Get a default value of the first day of week - locale sensitive */
		//Locale locale = context.getViewRoot().getLocale();
		Calendar cal = Calendar.getInstance(locale);
		int DefaultFirstDay = cal.getFirstDayOfWeek();
		// TODO MWD: Our day of week is 0 based, java.util.Calendar is 1 based
		// We must make ours consistent with java.util.Calendar - meanwhile do this:
		DefaultFirstDay--;
		// 1st find the UIInput field we're assisting and read whatever
		// relevant attributes that my be set on it. 
		StringBuffer jsBuffer = new StringBuffer();
		String converterId = inputTextId;
		String converterScript =
			ClientConverterHelper.getJsDateConverterString(
				pattern,
				type,
				dateStyle,
				timeStyle,
				locale);
		jsBuffer.append(
				HxClientRenderUtil.HX_VAR_NAME
				+ ".addConverter(\""
				+ converterId
				+ "\", "
				+ converterScript
				+ ");");
		//String validatorId = null;
		//????
		if (null != converterId)
		{
			writer.write("\n");
			writer.write(jsBuffer.toString());
			/*
			 * We can't have a validator without a converter so only get the 
			 * validator (as follows) if we've found a converter. 
			 */
			jsBuffer.setLength(0); // reset buffer 
			/*validatorId = HxClientRenderUtil.buildJsRangeValidator(jsBuffer, context, input);
			if (null != validatorId) {
				writer.write("\n");
				writer.write(jsBuffer.toString());
			}*/
		}
		if (required == true)
			sRequired = "true";
		String sFirstDay =
			getNumericAttribute(
				"first-day-of-week:",
				firstDay,
				new Integer(DefaultFirstDay).toString(),
				new Integer(0),
				new Integer(6));
		String sBtnBorder =
			getNumericAttribute(
				"button-border:",
				buttonBorder,
				DEFAULT_BUTTON_BORDER,
				new Integer(-1),
				new Integer(3));
		writer.write(
				"\n"
				+ HxClientRenderUtil.HX_VAR_NAME
				+ ".addComponent(\""
				+ inputTextId
				+ "\", new "
				+ HxClientRenderUtil.HX_VAR_NAME
				+ ".JSFDatePicker(");
		if (null != converterId)
			writer.write("\"converter:" + converterId + "\""); //$NON-NLS-1$ //$NON-NLS-2$
		//if (null != validatorId)
		//	writer.write(", \"" + "validator:" + validatorId + "\""); //$NON-NLS-1$ //$NON-NLS-2$
		if (cSSPrefix != null && cSSPrefix != "")
			writer.write(", \"CSS-prefix:" + cSSPrefix + "\""); //$NON-NLS-1$ //$NON-NLS-2$
		if (btnColor != null && btnColor != "")
			writer.write(", \"button-color:" + btnColor + "\""); //$NON-NLS-1$ //$NON-NLS-2$
		if (sRequired != null && sRequired != "")
			writer.write(", \"required:" + sRequired + "\""); //$NON-NLS-1$ //$NON-NLS-2$
		//	no need to check if following attributes have values as defaults are automatically set if not on page
		writer.write(", \"" + sFirstDay + "\""); //$NON-NLS-1$ //$NON-NLS-2$
		writer.write(", \"" + sBtnBorder + "\""); //$NON-NLS-1$ //$NON-NLS-2$
		writer.write("));\n"); //$NON-NLS-1$
		writer.write("</script>"); out.write("\n");
		Streamer.trace.Header().println(
							"Exiting DatePickeremitter.Export()");
	}
	/**
	 * Fetch, validate and transform a numeric component attribute embedded in a string
	 * @param component		- JSF component holding the requested attribute
	 * @param sAttInName	- name of JSF attribute, e.g. ShowTabs
	 * @param sAttOutName	- "show-tabs:"
	 * @param DefaultValue	- value to be assigned if nothing is found
	 * @param MinValue		- smallest valid value, if any
	 * @param MaxValue		- largest valid value, if any
	 * @return 				- e.g. "show-tabs:1"
	 */
	// TODO MWD: abstract to utility class so any renderer can use this
	private String getNumericAttribute(
		String sAttOutName,
		String value,
		String DefaultValue,
		Integer MinValue,
		Integer MaxValue)
	{
		String sValue = value;
		if (sValue == null || sValue == "")
			sValue = DefaultValue;
		else
		{
			try
			{
				int iValue = Integer.parseInt(sValue);
				if (MinValue != null && iValue < MinValue.intValue())
					sValue = DefaultValue;
				else
					if (MaxValue != null && iValue > MaxValue.intValue())
						sValue = DefaultValue;
			}
			catch (NumberFormatException e)
			{
				sValue = DefaultValue;
			}
		}
		sValue = sAttOutName + sValue;
		return sValue;
	}
}