/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//  
//OCO Source Materials
//  
//5724-E76 and 5724-E77
//  
// Copyright IBM Corp.  2003
//  
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////

/*
 * Created on June 20, 2003
 *
 */
package com.ibm.odcb.jrender.emitters;

import java.io.IOException;
import java.io.Writer;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;

import com.ibm.odcb.jrender.mediators.ExportException;
import com.ibm.odcb.jrender.mediators.PageContext;
import com.ibm.odcb.jrender.misc.Streamer;
import com.ibm.odcb.jrender.misc.StringUtil;

/**
 * @author Kaushal <br>
 * Event emitter is a generic emitter to connect one source control with one or more
 * other target adapters with specified events.
 */
public class EventSelectAndActivateEmitter extends BaseEmitter {
	public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;

	// Class variables
	protected String _sourceControl;
	protected String _eventName;
	protected String _eClassName;
	//protected ArrayList 	_targetEmitterList = new ArrayList(7);
	protected Map _targetEmitterMap = new Hashtable();
	protected BaseEmitter _sourceEmitter;
	//protected String _propertyName;

	/**
	 * Empty constructor. Just a placeholder to create an object of the type, SelectEmitter.
	 * Use Init() methods to initialize class vars, in conjunction with this empty constructor.
	 *
	 */
	public EventSelectAndActivateEmitter() {
		super("ODCEventSelectAndActivateHandler");
	}

	/**
	 * Constructor that calls the Init function to initialize class vars in EventEmitter
	 * <p>
	 * @param sourceEmitter		Of type BaseEmitter, this is used to extract the name of the Source Control.
	 * @param eventName			Name of the event in the source control the event handler is being tied to. 
	 * @param eClassName		The name of the eclass in the source control for which the event needs to be fired.
	 * @param targetEmitter		Name of target emitter (adapter) that is bound to the source control.
	 * @param propertyString	The reference or attribute in the source eclass to be used as the source. Null if the eclass needs to be used directly.
	 */
	public EventSelectAndActivateEmitter(
		BaseEmitter sourceEmitter,
		String eventName,
		String eClassName,
		BaseEmitter targetEmitter,
		String propertyString) {
		super("ODCEventSelectAndActivateHandler");
		this.Init(sourceEmitter, eventName, eClassName, targetEmitter, propertyString);
	}

	/**
	 * Constructor that calls the Init function to initialize class vars in EventEmitter
	 * <p>
	 * @param sourceEmitter		Of type BaseEmitter, this is used to extract the name of the Source Control.
	 * @param eventName			Name of the event in the source control the event handler is being tied to. 
	 * @param eClassName		EClassName is the first param to the JS function GenericEventHandler.
	 */
	public EventSelectAndActivateEmitter(BaseEmitter sourceEmitter, String eventName, String eClassName) {
		super("ODCEventSelectAndActivateHandler");
		this.Init(sourceEmitter, eventName, eClassName);
	}

	/**
	 * Init() without any targetEmitters; Use this when there are more than 1 adapters being passed in
	 * After this Init, use addTargetEmitter() to add to the targetEmitter list.
	 * 
	 * @param sourceEmitter		Of type BaseEmitter, this is used to extract the name of the Source Control.
	 * @param eventName			Name of the event in the source control the event handler is being tied to. 
	 * @param eClassName		EClassName is the first param to the JS function GenericEventHandler.
	 */
	public void Init(BaseEmitter sourceEmitter, String eventName, String eClassName) {
		this._sourceEmitter = sourceEmitter;
		this._sourceControl = sourceEmitter.CONTROL_VAR + sourceEmitter.getExportID();
		this._eventName = eventName;
		this._eClassName = eClassName;
	}

	/**
	 *  Use this version of Init when there is only one targetEmitter to add...a convenience function.
	 * 
	 * @param sourceEmitter		Of type BaseEmitter, this is used to extract the name of the Source Control.
	 * @param eventName			Name of the event in the source control the event handler is being tied to. 
	 * @param eClassName		EClassName is the first param to the JS function GenericEventHandler.
	 * @param targetEmitter		Name of target emitter (adapter) that is bound to the source control.
	 */
	public void Init(
		BaseEmitter sourceEmitter,
		String eventName,
		String eClassName,
		BaseEmitter targetEmitter,
		String propertyString) {
		this._sourceEmitter = sourceEmitter;
		this._sourceControl = sourceEmitter.CONTROL_VAR + sourceEmitter.getExportID();
		this._eventName = eventName;
		this._eClassName = eClassName;
		this.addTargetEmitter(targetEmitter, propertyString);
	}

	/**
	 * AddTargetEmitter() adds to the list of target adapters this event handler is hooking up the control to
	 * @param emitter	TargetEmitter that is being added to the list of target emitters for this event handler
	 * @param propertyString	The reference or attribute in the source eclass to be used as the source. Null if the eclass needs to be used directly.
	 */
	public void addTargetEmitter(BaseEmitter emitter, String propertyString) {
		String adapterVar = emitter.ADAPTER_VAR + emitter.getExportID();
		//_targetEmitterList.add(adapterVar);
		if (propertyString == null) {
			propertyString = "";
		}
		_targetEmitterMap.put(adapterVar, propertyString);
	}
	public void setSourceEmitter(BaseEmitter sourceEmitter) {
		this._sourceEmitter = sourceEmitter;
		this._sourceControl = sourceEmitter.CONTROL_VAR + sourceEmitter.getExportID();
	}
	public void setSourceControl(String sourceControl) {
		this._sourceControl = sourceControl;
	}
	public void setEventName(String eventName) {
		this._eventName = eventName;
	}
	public void setEClassName(String eClassName) {
		this._eClassName = eClassName;
	}
	
	/**
	 * Export emits the JS code to wire the event handler to the event.
	 * @param out	Writer to output the javascript to
	 * @param PC	PageContext
	 */
	public void Export(Writer out, PageContext PC) throws ExportException, IOException {
		Streamer.trace.Header().println("Entering EventSelectAndActivate Emitter...");
		//_varIndex = PC.NextInstanceCount();
		//String eventHandlerStr = super.EVENT_HANDLER_VAR + super.getExportID();
		String adapterArrayStr = this.constructTargetAdapterArrayString();
		//Fenil Change -> Event handlers now need to be wrapped around by a function which takes the event as a parameter
		out.write("<script type=\"text/javascript\">");
		out.write("\n");
		
		 //Boolean storing flag that governs encoding. If page has FacesContext encoding should occur.
		boolean isFaces=StringUtil.checkEncoding();
		
		String funcName = "func" + PC.NextInstanceCount();
		//encode function name
		funcName=StringUtil.encodeString(funcName,isFaces);
		out.write("function " + funcName + "(thisObj, thisEvent){\n");
		out.write("SelectAndActivateEventHandler(thisEvent," + adapterArrayStr + ");");
		out.write("\n");
		out.write("}\n");
		out.write(this._sourceControl + ".addHandler(\"" + this._eventName + "\",\"" + funcName + "\",\"" + _eClassName + "\");");
		out.write("\n");
		out.write("</script>");
		Streamer.trace.Header().println("Exiting EventSelectAndActivate Emitter...");
	}

	/**
	 * This is a convenience function to construct the target adapter array string as required by
	 * the JavaScript function. Input: ODCSelectAdapterVar22; Output: ["ODCSelectAdapterVar22"]
	 * If input is an array, output will be comma separated quotes enclosed JS array.
	 * 
	 * @return String Method returns adapter array string, constructed properly in the right format
	 */
	protected String constructTargetAdapterArrayString() {
		String adapterArrayString = "[";
		Iterator it = this._targetEmitterMap.keySet().iterator();

		//for(int i = 0; i < _targetEmitterList.size(); i++)
		int i = 0;
		while (it.hasNext()) {
			String adapterVar = (String) it.next();
			String propertyString = (String) this._targetEmitterMap.get(adapterVar);
			String tempStr =
				!propertyString.equals("") ? "[" + adapterVar + ",\"" + propertyString + "\"]" : adapterVar;
			//String s1 = tempStr;
			//s1 = StringUtil.QuoteDoubleAndEscape(s1,true);			
			//s1 = (s1.substring(1,s1.length()-2)).concat(s1.substring(s1.length()-1));
			if (i != 0)
				adapterArrayString = adapterArrayString.concat(", " + tempStr);
			else
				adapterArrayString = adapterArrayString.concat(tempStr);
			i++;
		}
		adapterArrayString = adapterArrayString.concat("]");
		Streamer.debug.Header().println("EventSelectAndActivateEmitter --> adapterArrayString: " + adapterArrayString);
		return adapterArrayString;
	}
}