/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//  
//OCO Source Materials
//  
//5724-E76 and 5724-E77
//  
// Copyright IBM Corp.  2003
//  
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////

/*
 * Created on July 2, 2003
 *
 */
package com.ibm.odcb.jrender.emitters;

import java.io.IOException;
import java.io.Writer;
import java.util.ArrayList;

import com.ibm.odcb.jrender.mediators.ExportException;
import com.ibm.odcb.jrender.mediators.PageContext;
import com.ibm.odcb.jrender.misc.Streamer;
import com.ibm.odcb.jrender.misc.StringUtil;
/**
 * @author Kaushal <br>
 * Event emitter is a generic emitter to copy an eobject or attribute from one source
 * control to a another compatible eobject or attrbute in the model. Basically it performs
 * an attribute to attribute or object to object copy. The object to object copy will be a deep copy.
 */
public class EventSelectAndSetEmitter extends BaseEmitter {
	public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;

	// Class variables
	protected String _modelName;
	protected String _sourceControl;
	protected String _eventName;
	protected String _eClassName;
	protected ArrayList _controlMetadataHelperList = new ArrayList(8);
	protected BaseEmitter _sourceEmitter;

	/**
	 * Empty constructor. Just a placeholder to create an object of the type, ODCEventSelectAndSetHandler.
	 * Use Init() methods to initialize class vars, in conjunction with this empty constructor.
	 *
	 */
	public EventSelectAndSetEmitter() {
		super("ODCEventSelectAndSetHandler");
	}

	/**
	 * Constructor that calls the Init function to initialize class vars in EventEmitter
	 * <br>
	 * @param sourceEmitter		Of type BaseEmitter, this is used to extract the name of the Source Control.
	 * @param modelName			The model that the source control is contained in
	 * @param eventName			Name of the event in the source control the event handler is being tied to. 
	 * @param eClassName		EClassName is the first param to the JS function GenericEventHandler.
	 */
	public EventSelectAndSetEmitter(BaseEmitter sourceEmitter, String modelName, String eventName, String eClassName) {
		super("ODCEventSelectAndSetHandler");
		this.Init(sourceEmitter, modelName, eventName, eClassName);
	}

	/**
	 * Init method initializes the key class vars: sourceEmitter, eventName and eClassName.
	 * @param sourceEmitter		Of type BaseEmitter, this is used to extract the name of the Source Control.
	 * @param modelName			The model that the source control is contained in
	 * @param eventName			Name of the event in the source control the event handler is being tied to. 
	 * @param eClassName		EClassName is the first param to the JS function GenericEventHandler.
	 */
	public void Init(BaseEmitter sourceEmitter, String modelName, String eventName, String eClassName) {
		this._sourceEmitter = sourceEmitter;
		this._sourceControl = sourceEmitter.CONTROL_VAR + sourceEmitter.getExportID();
		this._eventName = eventName;
		this._eClassName = eClassName;
		this._modelName = modelName;

		Streamer.debug.Header().println(
			"sourceControl: "
				+ this._sourceControl
				+ "\n"
				+ "eventName: "
				+ this._eventName
				+ "\n"
				+ "eClassName: "
				+ this._eClassName
				+ "\n"
				+ "modelName: "
				+ this._modelName);
	}

	public void addControlMetadataHelper(ControlMetadataHelper cmh) {
		_controlMetadataHelperList.add(cmh);
	}
	public void setSourceControl(String sourceControl) {
		this._sourceControl = sourceControl;
	}
	public void setEventName(String eventName) {
		this._eventName = eventName;
	}
	public void setEClassName(String eClassName) {
		this._eClassName = eClassName;
	}
	public void setModelName(String modelName) {
		this._modelName = modelName;
	}
	public void setSourceEmitter(BaseEmitter sourceEmitter) {
		this._sourceEmitter = sourceEmitter;
		this._sourceControl = sourceEmitter.CONTROL_VAR + sourceEmitter.getExportID();
	}

	/**
	 * Export emits the JS code to call the SelectAndSetEventHandler and also emits the code
	 * related to the specific source control: ODCTree1.OnHighlight = ODCEventHandler2;
	 * The controlMetadataArray string is got from the convenience function constructControlMetadataArrayString()
	 * <br>
	 * @param out	Writer to output the javascript to
	 * @param PC	PageContext
	 */
	public void Export(Writer out, PageContext PC) throws ExportException, IOException {
		Streamer.trace.Header().println("Entering EventSelectAndSetEmitter...");
		//_varIndex = PC.NextInstanceCount();
		//String eventHandlerStr = super.EVENT_HANDLER_VAR + super.getExportID();
		//Boolean storing flag that governs encoding. If page has FacesContext encoding should occur.
		boolean isFaces = StringUtil.checkEncoding();
		String modelString = "WDO4JSModelRoot_" + _modelName + ".Root";
		String controlMetadataArrayStr = this.constructControlMetadataArrayString();

		out.write("<script type=\"text/javascript\">");
		out.write("\n");
		
		String funcName = "func" + PC.NextInstanceCount();
		//encode function name
		funcName=StringUtil.encodeString(funcName,isFaces);
		out.write("function " + funcName + "(thisObj, thisEvent){\n");
		out.write(
			"SelectAndSetEventHandler(thisEvent,"
				+ controlMetadataArrayStr
				+ ");");
		out.write("\n");
		out.write("}\n");
		out.write(this._sourceControl + ".addHandler(\"" + this._eventName + "\",\"" + funcName + "\",\"" + _eClassName + "\");");
		out.write("\n");
		out.write("</script>");
		Streamer.trace.Header().println("Exiting EventSelectAndSetEmitter...");
	}

	/**
	 * This is a convenience function to construct the control metadata array string as required by
	 * the JavaScript function. Output: [["source EFeature Name", "Target EObject ID", "Target EFeature Name"],
	 * 									 ["source EFeature Name", "Target EObject ID", "Target EFeature Name"],
	 * 									 ["source EFeature Name", "Target EObject ID", "Target EFeature Name"]]
	 * The number of array entries depend on the size of the control Metadata Helper List.
	 * Each object in the control metadata helper list holds one array element of the above array.
	 * <br>
	 * @return String Method returns adapter array string, constructed properly in the right format
	 */
	protected String constructControlMetadataArrayString() {
		String controlMetadataHelperArrayString = "[";

		for (int i = 0; i < _controlMetadataHelperList.size(); i++) {
			ControlMetadataHelper tempCMH = (ControlMetadataHelper) _controlMetadataHelperList.get(i);
			if (i != 0)
				controlMetadataHelperArrayString =
					controlMetadataHelperArrayString.concat(", " + tempCMH.constructMetadataString());
			else
				controlMetadataHelperArrayString =
					controlMetadataHelperArrayString.concat(tempCMH.constructMetadataString());
		}
		controlMetadataHelperArrayString = controlMetadataHelperArrayString.concat("]");
		Streamer.debug.Header().println(
			"EventSelectAndSetEmitter --> control Metadata helper string: " + controlMetadataHelperArrayString);
		return controlMetadataHelperArrayString;
	}
}