/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//  
//OCO Source Materials
//  
//5724-E76 and 5724-E77
//  
// Copyright IBM Corp.  2003
//  
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////
/*
 * Created on Mar 24, 2003
 *
 */
package com.ibm.odcb.jrender.emitters;
import java.io.IOException;
import java.io.Writer;
import com.ibm.odcb.jrender.mediators.ExportException;
import com.ibm.odcb.jrender.mediators.PageContext;
import com.ibm.odcb.jrender.misc.Config;
import com.ibm.odcb.jrender.misc.Streamer;
import com.ibm.odcb.jrender.misc.URLRewriter;
//import com.ibm.odcb.jrender.misc.NLWebResourceManager;
import com.ibm.odcb.jrender.misc.NLResourceManager;

//import com.ibm.odcb.jrender.misc.NumberFormattingInfo;
/**
 * @author Kaushal <br>
 * Java interface to generate all appropriate include files for ODC-enabled web pages
 * Appropriate path generation is done via the URLRewriter interface in this class.
 */
public class InitializationEmitter extends BaseEmitter {
	public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;
	
	
	/**
	 * Minimal Constructor
	 * @param Rewriter the rewriter to be used to rewrite resource URL's
	 * @param serverName the name of the server where the application is deployed
	 * @param portNumber the port number that the Web server is running on
	 */
	public InitializationEmitter(URLRewriter Rewriter, String serverName, int portNumber) {
		this.Init(Rewriter, serverName, portNumber);
		
		//If we have not figured out whether it is a portal
		if(isPortal ==-1)
		{
		   String httpReqClassName = Config.getString(Config.PROP_REQUEST_CLASS);
		   if(httpReqClassName.equals("com.ibm.faces.bf.util.HttpRequestRetrieverForServlet"))
		   {
		   	  //not a portal
		      isPortal = 0;  
		   }
		   else
		   {
              //it is a portal
			  isPortal = 1;
		   }
	    }
	}
	
	/**
	 * @deprecated
	 * @param DebugMode
	 * @param LogLevel
	 * @param Locale
	 * @param Rewriter
	 * @param serverName
	 * @param portNumber
	 */
	public InitializationEmitter(
		boolean DebugMode,
		int LogLevel,
		String Locale,
		URLRewriter Rewriter,
		String serverName,
		int portNumber) {
		this.Init(DebugMode, LogLevel, Locale, Rewriter, serverName, portNumber);
	}
	
	/**
	 * @deprecated
	 * @param Rewriter
	 * @param serverName
	 * @param portNumber
	 */
	public void Init(URLRewriter Rewriter, String serverName, int portNumber) {
		this.Init(
			_DefaultClientDebugMode,
			_DefaultClientLogLevel,
			_DefaultClientLocale,
			Rewriter,
			serverName,
			portNumber);
	}
	
	/**
	 * @deprecated
	 * @param DebugMode
	 * @param LogLevel
	 * @param Locale
	 * @param Rewriter
	 * @param serverName
	 * @param portNumber
	 */
	public void Init(
		boolean DebugMode,
		int LogLevel,
		String Locale,
		URLRewriter Rewriter,
		String serverName,
		int portNumber) {
		_DebugMode = DebugMode;
		this.setLocale(Locale);
		_LogLevel = LogLevel;
		_Rewriter = Rewriter;
		_baseURL = serverName == null ? null : "http://" + serverName + ":" + portNumber;
		Streamer.debug.Header().println("DebugMode: " + _DebugMode);
		Streamer.debug.Header().println("LOG_LEVEL: " + _LogLevel);
		Streamer.debug.Header().println("LOCALE: " + _Locale);
		Streamer.debug.Header().println("Base URL: " + _baseURL);
	}
	// LogLevel values
	public static int LOG_LEVEL_NONE = -1;
	public static int LOG_LEVEL_ERROR = 0;
	public static int LOG_LEVEL_WARNING = 1;
	public static int LOG_LEVEL_INFO = 2;
	public static int LOG_LEVEL_DEBUG = 3;
	// Dafault value holders from the properties file
	protected final boolean _DefaultClientDebugMode     = Config.getInteger(Config.PROP_CLIENT_DEBUG_MODE   ) == 1 ? true : false;
	protected final String  _DefaultClientLocale        = Config.getString (Config.PROP_CLIENT_LOCALE       );
	protected final int     _DefaultClientLogLevel      = Config.getInteger(Config.PROP_CLIENT_LOG_LEVEL    );
	protected final boolean _ShowProgressBar            = Config.getInteger(Config.PROP_PROGRESSBAR         ) == 1 ? true : false;
	// Main member variables
	protected String _Locale;
	protected boolean _DebugMode;
	protected int _LogLevel;
	protected URLRewriter _Rewriter;
	protected String _baseURL;
	
	//This is the variable to keep track whether it is a portal app
	private static int isPortal = -1;
	 
	// ContextVariable IDs
	public static final String _ID_ODCB_INIT_FLAG = "ODCB_INIT_FLAG";
	public static final String _ID_DEBUG_MODE = "DEBUG_MODE";
	public static final String _ID_URL_REWRITER = "URL_REWRITER";
	public static final String _ID_URL_PREFIX = "URL_PREFIX";
	public static final String _ID_URL_POSTFIX = "URL_POSTFIX";
	public static final String DEBUG_YES = "YES";
	public static final String _ID_CLIENT_LOCALE = "CLIENT_LOCALE";
	public static final String _ID_LOG_LEVEL = "LOG_LEVEL";
	public void setLocale(String locale) {
		_Locale = locale;
	}

	private static final String JSLUTIL = Config.versionizeFilename("jslUtil.js");
	private static final String LOGGER = Config.versionizeFilename("logger.js");
	private static final String DETECTBROWSER = Config.versionizeFilename("detectBrowser.js");
	private static final String MASKCONVERTER = Config.versionizeFilename("maskconverter.js");
	private static final String ODCREGISTRY = Config.versionizeFilename("ODCRegistry.js");
	private static final String GENERICEVENTHANDLER = Config.versionizeFilename("genericEventHandler.js");
	private static final String ODCSESSIONCONTROLLER = Config.versionizeFilename("odcSessionController.js");
	private static final String PROGRESSBAR = Config.versionizeFilename("progressbar.js");
	private static final String JSL_CORE_C = Config.versionizeFilename("jsl_core_c.js");

	
	/**
	 * Includes all the core Faces Client JS files on the page, making sure the files not included
	 * more than once on the same page.
	 * @param out The output writer to write to
	 * @param PC the ODC <code>PageContext</code> object.
	 */
	public void Export(Writer out, PageContext PC) throws ExportException, IOException {
		Streamer.trace.Header().println("Entering InitializationEmitter export...");
		//if (PC.getContextVariable(_ID_ODCB_INIT_FLAG) != null)
		// return;
		//PC.addContextVariable(_ID_ODCB_INIT_FLAG, "YES");
		if (TestGuard(PC, _ID_ODCB_INIT_FLAG) != false)
			return;
		Streamer.trace.Header().println("Executing initialization emitter...");
		
		// Locale specific message file includes
		if (BaseEmitter.TestGuard(PC, "NL_JS_FLAG") == false) {
			try {
				String odcMessageFileFullPath =
				   NLResourceManager.getFile(_baseURL, "jsl/OdysseyMessage","jsl/OdysseyMessage", _Locale, _Rewriter);
				Streamer.debug.Header().println("odc Message file path --> " + odcMessageFileFullPath);
				out.write("<script type=\"text/javascript\" src=\"" + odcMessageFileFullPath + "\"></script>");
				out.write("\n");
			} catch (Exception e1) {
				e1.printStackTrace();
				Streamer.error.Header().printStackTrace(e1);
			}
			try {
				String odcSessionControllerFileFullPath =
				   NLResourceManager.getFile(
						_baseURL,
						"jsl/odcSessionControllerMessage",
				        "jsl/odcSessionControllerMessage",
						_Locale, 
						_Rewriter);
				Streamer.debug.Header().println("odc Controller file path --> " + odcSessionControllerFileFullPath);
				out.write("<script type=\"text/javascript\" src=\"" + odcSessionControllerFileFullPath + "\"></script>");
				out.write("\n");
			} catch (Exception e2) {
				e2.printStackTrace();
				Streamer.error.Header().printStackTrace(e2);
			}

		}
		if (TestGuard(PC, "CORE_JS_FLAG") == false) {
			if (_DebugMode == true) {
				PC.addContextVariable(_ID_DEBUG_MODE, InitializationEmitter.DEBUG_YES);
				out.write(
					"\n<script type=\"text/javascript\" src=\""
						+ _Rewriter.Rewrite("jsl/" + JSLUTIL)
						+ "\"></script>"
						+ "\n<script type=\"text/javascript\" src=\""
						+ _Rewriter.Rewrite("jsl/" + LOGGER)
						+ "\"></script>"
						+ "\n<script type=\"text/javascript\" src=\""
						+ _Rewriter.Rewrite("jsl/" + DETECTBROWSER)
						+ "\"></script>"
						+ "\n<script type=\"text/javascript\" src=\""
						+ _Rewriter.Rewrite("jsl/converter/" + MASKCONVERTER)
						+ "\"></script>"
						+ "\n<script type=\"text/javascript\" src=\""
						+ _Rewriter.Rewrite("jsl/" + ODCREGISTRY)
						+ "\"></script>"
						+ "\n<script type=\"text/javascript\" src=\""
						+ _Rewriter.Rewrite("jsl/" + GENERICEVENTHANDLER)
						+ "\"></script>"
						+ "\n<script type=\"text/javascript\" src=\""
						+ _Rewriter.Rewrite("jsl/" + ODCSESSIONCONTROLLER)
						+ "\"></script>"
						+ "\n<script type=\"text/javascript\" src=\""
						+ _Rewriter.Rewrite("jsl/" + PROGRESSBAR)
						+ "\"></script>");
				out.write("\n");
				Streamer.trace.Header().println("Included individual core JS files...");
			} else {
				// jsl_core_c.js encompasses all NLS related JS files + jslUtil, logger files
				//if (TestGuard(PC, "CORE_LIB_FLAG") == false) 
				out.write(
					"<script type=\"text/javascript\" src=\"" + _Rewriter.Rewrite("jsl/" + JSL_CORE_C) + "\"></script>");
				out.write("\n");
				Streamer.trace.Header().println("Included compressed core JS file...");
			}
			// Number formatting is done via the Java class NumberFormattingInfo (jrender.misc package)
			// The JS file formatInfo.js has been split into 3: currency, number, and datetime format info
			// numberFormatInfo.js is not really used; instead the following Java code is used.
			// Added by Kaushal, Dec.16, 2003.
			//NumberFormattingInfo.printNumberFormattingInfo(out, _Locale);
			this.printProgressBar(out, PC);
		}
		/////////////////////////////////////////////////////////////////////////////////////////////////////
		// Need to send the prefix and postfix substrings to allow the JSL in the browser to rewrite URLs.
		final String SAMPLE_URL = "jsl_css/x.css";
		String Rewritten = _Rewriter.Rewrite(SAMPLE_URL);
		// Let's extract those prefix and postfix strings
		//int i = Rewritten.lastIndexOf(SAMPLE_URL);
		/*if (i == -1) {
			throw new ExportException(
				"The supplied URLRewriter rewrote '"
					+ Rewritten
					+ "' as '"
					+ Rewritten
					+ "', which is improper. Proper Rewriters cannot mangle the URL passed in.");
		}*/
		//String Prefix = Rewritten.substring(0, i);
		//String Postfix = Rewritten.substring(i + SAMPLE_URL.length(), Rewritten.length());
		out.write("<script type=\"text/javascript\">");
		//out.write("\n");
		//out.write("var ODCCommonLocale=\"" + _Locale + "\";");
		//out.write("\n");
		String URLPrefix = "getURLPrefix(\"" + Rewritten + "\",\"" + SAMPLE_URL + "\")";
		String URLPostfix = "getURLPostfix(\"" + Rewritten + "\",\"" + SAMPLE_URL + "\")";
		out.write(
			"Log.setLogLevel("
				+ _LogLevel
				+ "); Log.setURLRewriter("
				+ URLPrefix
				+ ", "
				+ URLPostfix
				+ "); \n"
				+ " var URL_REWRITER_PREFIX = "
				+ URLPrefix
				+ "; var URL_REWRITER_POSTFIX= "
				+URLPostfix
		        + "; var ODCPORTAL="
						+ isPortal
				+ "; var PAGE_LOCALE = \""
				+ _Locale
				+ "\"; \n"
				+ "</script>");
		out.write("\n");
		// Let's add the substrings in the context so that other emitters could use them
		PC.addContextVariable(_ID_URL_REWRITER, _Rewriter);
		//PC.addContextVariable(_ID_URL_PREFIX, Prefix);
		//PC.addContextVariable(_ID_URL_POSTFIX, Postfix);
		PC.addContextVariable(_ID_CLIENT_LOCALE, _Locale);
		PC.addContextVariable(_ID_LOG_LEVEL, String.valueOf(_LogLevel));
		Streamer.trace.Header().println("Exiting InitializationEmitter export...");
	}

	private void printProgressBar(Writer out, PageContext PC) throws IOException {
		if (this._ShowProgressBar) {
			out.write("<script type=\"text/javascript\">\n");
			out.write("ODCProgressBar.setVisible();\n");
			String varName = "holder" + PC.NextInstanceCount();
			out.write("var " + varName + " = new ColorHolder(ODCProgressBar.getRow());\n");
			out.write("setInterval(\"" + varName + ".changeColor()\", 350);\n");
			out.write("if(window.addEventListener)\n");
			out.write("window.addEventListener('load',function(){ODCProgressBar.hide();} ,false);\n");
			out.write("else\n");
			out.write("window.attachEvent(\"onload\", function(){ODCProgressBar.hide();});\n");
			out.write("</script>\n");
		}
	}
}
