/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//  
//OCO Source Materials
//  
//5724-E76 and 5724-E77
//  
// Copyright IBM Corp.  2003
//  
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////

/*
 * Created on Apr 17, 2003
 *
 */
package com.ibm.odcb.jrender.emitters;

import java.io.Writer;
import java.io.IOException;
import com.ibm.odcb.jrender.mediators.ExportException;
import com.ibm.odcb.jrender.mediators.PageContext;
import com.ibm.odcb.jrender.misc.Streamer;
import com.ibm.odcb.jrender.misc.StringUtil;
/**
 * @author Kaushal <br>
 * InputText Emitter is a Java interface for generating JS, HTML code for TextArea and TextField HTML elements
 */
public class InputTextEmitter extends BaseEmitter {

	public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;
	protected final int _defaultRows = 10;
	protected final int _defaultCols = 20;
	
	public InputTextEmitter()
	{
		super("ODCInputText");
	}

	// Class variables	
	protected boolean _TextArea; // false="textfield", true="textarea"
	protected String _modelName;
    protected String _eObjectID;
    protected String _eAttributeName;// eAttributeName in model class that is bound to text element
	protected int    _rows = _defaultRows;
	protected int	 _cols = _defaultCols;
	
	/**
	 * Init method to initialize class variables. This constructor does not have the rows and columns
	 * set; in this case it is assumed that the Text FIELD is desired.
	 * <br>
	 * @param modelName			Name of the data model on the page to bind this control to
	 * @param eObjectID			ID of the eObject in the data model to bind this control to
	 * @param eAttributeName	EAttribute name in the eObject that is bound to the control
	 */
	public void Init(String modelName, String eObjectID, String eAttributeName)
     {
       _TextArea = false;
       _modelName = modelName;
       _eObjectID = eObjectID;
       _eAttributeName = eAttributeName;
     }
	/**
	 * Init method to initialize class variables. This constructor HAS the rows and columns
	 * to set; in this case it is assumed that the Text AREA is desired.
	 * <br>
	 * @param modelName			Name of the data model on the page to bind this control to
	 * @param eObjectID			ID of the eObject in the data model to bind this control to
	 * @param eAttributeName	EAttribute name in the eObject that is bound to the control
	 * @param rows				Number of rows desired for the text area
	 * @param cols				Number of columns desired for the text area
	 */
   public void Init(String modelName, String eObjectID, String eAttributeName, int rows, int cols)
    {
      _TextArea = true;
      _modelName = modelName;
      _eObjectID = eObjectID;
      _eAttributeName = eAttributeName;
      _rows = rows; 
      _cols = cols;
	}

	// All set functions to initialize class instance variables.
    public void setTextArea      (boolean TextArea     )  { _TextArea       = TextArea      ; }
	public void setModelName     (String  modelName    )  { _modelName      = modelName     ; }
	public void setEObjectID     (String  eObjectID    )  { _eObjectID      = eObjectID     ; }
	public void setEAttributeName(String eAttributeName)  { _eAttributeName = eAttributeName; }
	public void setRows(int rows){	_rows = rows;	}
	public void setCols(int cols){	_cols = cols;	}

	/**
	 * Export method emits the JS, HTML code for the text field or text area HTML elements.
	 * The elements are bound to a data model on the page via a PropertyBinder.
	 * <br>
	 * @param out	Writer output stream to write the generated code to
	 * @param PC	Page Context
	 */
	public void Export(Writer out, PageContext PC) throws ExportException, IOException
	{
		Streamer.trace.Header().println("Entering InputTextEmitter export...");
		_varIndex = PC.NextInstanceCount();
		
	    //Boolean storing flag that governs encoding. If page has FacesContext encoding should occur.
		boolean isFaces=StringUtil.checkEncoding();
		
		//Encoding variables for portal
		 _modelName=StringUtil.encodeString(_modelName,isFaces);
	     String modelName = StringUtil.encodeString("WDO4JSModelRoot_"+_modelName+".Root",isFaces);
	     _eObjectID = StringUtil.encodeString(_eObjectID,isFaces);
	     CONTROL_VAR= StringUtil.encodeString(CONTROL_VAR,isFaces);
	     ADAPTER_VAR= StringUtil.encodeString(ADAPTER_VAR,isFaces);
		
		if(_TextArea == true)
		{
          out.write("<textarea id=\""+CONTROL_VAR+_varIndex+"\" cols=\""+_cols+"\"  rows=\""+_rows+"\" /></textarea>");
          out.write("\n");
		}
		else
		{
          out.write("<input type=\"text\" id=\""+CONTROL_VAR+_varIndex+"\" />");
		  out.write("\n");
		}
		out.write("<script type=\"text/javascript\">");out.write("\n");
		
		out.write("var "+ADAPTER_VAR+_varIndex+" = new PropertyBinder(findEObjectByXMIID("+modelName+",'"+_eObjectID+"')"
                                                                                +",'"+_eAttributeName+"'"
                                                                                +",document.getElementById('"+CONTROL_VAR+_varIndex+"')"
                                                                                +",'value', 'onchange');");
		out.write("\n");
		out.write(ADAPTER_VAR+_varIndex+".DataBind();");out.write("\n");

		// Registering control not needed: being done in JSF (July 30, 2003)
		//out.write(PAGE_CONTROL_VAR+".registerControl(\""+CONTROL_VAR+_varIndex+"\", "+CONTROL_VAR+_varIndex+");"); out.write("\n");

 		out.write("</script>");out.write("\n");
		Streamer.trace.Header().println("Exiting InputTextEmitter export...");
	}
}