/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//
//OCO Source Materials
//
//5724-E76 and 5724-E77
//
// Copyright IBM Corp.  2003
//
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////
/*
 * Created on June 5, 2003
 *
 */
package com.ibm.odcb.jrender.emitters;
import java.io.IOException;
import java.io.Writer;
import java.util.ArrayList;

import com.ibm.faces.bf.renderkit.JSUtil;
import com.ibm.odcb.jrender.mediators.ExportException;
import com.ibm.odcb.jrender.mediators.PageContext;
import com.ibm.odcb.jrender.misc.SpecialSymbolHandler;
import com.ibm.odcb.jrender.misc.StringUtil;

/**
 * @author Kaushal <br>
 * <code>NodeAttributeEmitterHelper</code> is a helper class for the <code>TreeViewEmitter</code>.
 * This class supplies the metadata to the <code>TreeViewEmitter</code> to construct a tree node.
 * This class was updated to be extend base emitter for tree table implementation
 */
public class NodeAttributeEmitterHelper extends BaseEmitter 
{
	public static final String COPYRIGHT =
		com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;
	public static final String TREE_NODE_VAR = "ODCNodeAttr";
	// class instance variables
	protected String _eClassType;
	protected String _eAttributeName;
	protected ArrayList _eReferenceArray = new ArrayList();
	protected String _openIcon;
	protected String _closedIcon;
	protected String _nodeIconAttribute = null;
	protected String _nodeIconAttributeValue = null;
	protected String _styleMap;
	protected String _colDataMap;
	protected ArrayList _behaviorMap= new ArrayList();
	protected ArrayList _iconMap = new ArrayList();
	protected ArrayList _handlerMap = new ArrayList();
	protected boolean _isFaces;
	protected ArrayList _colDataList = new ArrayList();
	
	/**
	 * @param index The _varIndex to set.
	 */
	public void setVarIndex(int index) {
		_varIndex = index;
	}
	/**
	 * Constructor takes eClassType as parameter
	 * @param eClassType		eClass type of this node object
	 */
	public NodeAttributeEmitterHelper(String eClassType)
	{
		super("ODCTree");
		_eClassType = eClassType;
		_colDataMap ="";
		_styleMap="";
		_isFaces=StringUtil.checkEncoding();
	}
	
	public NodeAttributeEmitterHelper()
	{
		super("ODCTree");
		_colDataMap ="";
		_styleMap="";
		_isFaces=StringUtil.checkEncoding();
		
	}
	
	/**
	 * Specifies the text styling information for this tree node.
	 * @param eAttributeName the eAttribute for the eClass,with which this node is configured, which is used as the label for the tree node.
	 * @param nodeLabel If the the eClass with which this node is configured does not have an eAttribute which can be used as a label for the 
	 * node the user can specify the <code>nodeLabel</code>, which is a static string to be used as the label for the tree node.
	 * @param eReferenceName this is name of the reference in the eClass, with which this node is configured, to be used for the next level of nodes.
	 * @throws IOException
	 */
	public void addStyleMap(
		String eAttributeName,
		String nodeLabel,
		String eReferenceName)
		throws IOException
	{
		//If we have both an attribute and a nodeLabel specified, we will use attribute as the label for tree node.
		//If both are null or empty, we will use class name as the label for tree node.
		if (eAttributeName == null && nodeLabel == null)
		{
			nodeLabel = _eClassType;		
		}
		
		this.addEReference(eReferenceName);
		if (eAttributeName != null)
		{
			_styleMap += _eClassType + ":" + eAttributeName;
	    }
		else if(nodeLabel != null)
		{
			_styleMap += _eClassType + ":" + nodeLabel;
		}
		if (eReferenceName != null)
			_styleMap += ":" + eReferenceName + ";";
		else
			_styleMap += ";";
	}
	
	/**
	 * Specifies the behaviors for this tree node.
	 */
	public void addBehaviorMap()
	{
		if(_behaviorList!=null)
		{
			for(int ind=0;ind<_behaviorList.size();ind++)
			{
				BehaviorEmitter behavior =(BehaviorEmitter)_behaviorList.get(ind);
				StringBuffer tempSB=new StringBuffer(128);
				
				//Add actionFuntion if it does not exist null will be added
				tempSB.append(",\""+JSUtil.trimEventHandler(behavior.getOnActionFunction())+"\"");
				
			
				if(behavior.getBehaviorAction()!=null)
				{
					//If preceded by function needs ; to seperate
					tempSB.append(",\""+behavior.getBehaviorAction()+"\"");
					
					//If has target action include as parameter
					if(behavior.getTargetAction()!=null)
						tempSB.append(",\""+behavior.getTargetAction()+"\"");
					
				}
				String tempStr = tempSB.length() == 0 ? null : "\""+_eClassType+":"+behavior.getEvent()+"\""+tempSB.toString(); 
				this._behaviorMap.add(tempStr);
				tempSB=null;
			}
		}
		
	}
	
	/**
	 * Specifies the icon styling information for this tree node.
	 * @param closedIcon the path to the icon to be used for this node when it is collapsed
	 * @param openIcon the path to the icon to be used for this node when it is expanded
	 * @param nodeIconAttribute the <code>nodeIconAttribute</code> and <code>nodeIconValue</code> act as filters for icon to be shown for this node.
	 * The <code>nodeIconAttribute</code> is the name of an attribute on the eClass with which this node is configured. The <code>nodeIconValue</code>
	 * is the value of the <code>nodeIconAttribute</code>. Both <code>nodeIconAttribute</code> and <code>nodeIconValue</code> must be specified for filter.
	 * When both are specified, it means that for a particular attribute on the eclass with which this node is configured, specified by <code>nodeIconAttribute</code>
	 * has a value specified by <code>nodeIconValue</code>, then use the <code>closedIcon</code> for the close icon and use <code>openIcon</code> for the open icon.
	 * @param nodeIconAttributeValue the value of the <code>nodeIconAttribute</code> specified.
	 * @param showSystemIconFlag Boolean to indicate if the system icon needs to shown for the this node. 
	 */
	public void addIconMap(
			String closedIcon,
			String openIcon,
			String nodeIconAttribute,
			String nodeIconAttributeValue,
			Boolean showSystemIconFlag
			)
		{
			addIconMap(closedIcon,openIcon,nodeIconAttribute,nodeIconAttributeValue,showSystemIconFlag,null,null);
		}
	/**
	 * Overloaded version of the <code>addIconMap</code> method. This method additionally allows the user to specify alternate text for this tree node.
	 * @param closedIcon
	 * @param openIcon
	 * @param nodeIconAttribute
	 * @param nodeIconAttributeValue
	 * @param showSystemIconFlag
	 * @param closedIconAlt The alternate text for the closed icon.
	 * @param openIconAlt The alternate text for the open icon.
	 */
	public void addIconMap(
		String closedIcon,
		String openIcon,
		String nodeIconAttribute,
		String nodeIconAttributeValue,
		Boolean showSystemIconFlag,
		String closedIconAlt,
		String openIconAlt)
	{
		if (closedIcon == null && openIcon == null)
			return;
		if(closedIconAlt!=null){
			SpecialSymbolHandler.replaceSpecialSymbol(closedIconAlt);
			SpecialSymbolHandler.replaceLtGtAposQuot(closedIconAlt);
		}
			
		if(openIconAlt!=null){
			SpecialSymbolHandler.replaceSpecialSymbol(openIconAlt);
			SpecialSymbolHandler.replaceLtGtAposQuot(openIconAlt);
		}
			
		
		if (nodeIconAttribute == null || nodeIconAttributeValue == null)
			_iconMap.add(
				_eClassType
					+ ","
					+ (closedIcon == null ? "" : closedIcon)
					+ " , "
					+ (openIcon == null ? "" : openIcon)
					+ ","
					+ (showSystemIconFlag.booleanValue() == true
						? "true"
						: "false")
					+ ","
					+ (closedIconAlt ==null?"":closedIconAlt)
					+","
					+ (openIconAlt ==null?"":openIconAlt)			
						);
		else
			_iconMap.add(
				_eClassType
					+ ","
					+ nodeIconAttribute
					+ " = "
					+ nodeIconAttributeValue
					+ " , "
					+ (closedIcon == null ? "" : closedIcon)
					+ " , "
					+ (openIcon == null ? "" : openIcon)
					+ ","
					+ (showSystemIconFlag.booleanValue() == true
						? "true"
						: "false")
					+ ","
					+ (closedIconAlt ==null?"":closedIconAlt)
					+","
					+ (openIconAlt ==null?"":openIconAlt)							
															
						);
	}
	
	/**
	 * Used to add an event handler for this node.
	 * @param action the name of the event for which the event handler is being added. The current events for which event handlers can be
	 * added are &quot;onhighlight&quot;, &quot;onselect&quot;, &quot;onunselect&quot; and &quot;ondragenterover&quot;.
	 * @param functionName the name of JavaScript event handling function for the specified event.
	 */
	public void addEventHandler(String action, String functionName)
	{
	   _handlerMap.add(_eClassType + ":" + action + "," + functionName);
	}
	

	/**
	 * Used to add column data for this node if it is a tree table.
	 * @param action the name of the event for which the event handler is being added. The current events for which event handlers can be
	 * added are &quot;onhighlight&quot;, &quot;onselect&quot;, &quot;onunselect&quot; and &quot;ondragenterover&quot;.
	 * @param functionName the name of JavaScript event handling function for the specified event.
	 */
	public void addColumnData(ColDataEmitterHelper colData)
	{
		_colDataList.add(colData);		
	}

	
	/**
	 * Returns the sytle map for this tree node.
	 * @return the style map string
	 */
	public String getStyleMap()
	{
		return _styleMap;
	}
	
	/**
	 * Returns the icon maps for this tree node.
	 * @return a list of strings, where each string is one icon map for this node.
	 */
	public ArrayList getIconMap()
	{
		return _iconMap;
	}
	
	
	/**
	 * Returns the event handlers for this tree node.
	 * @return a list of strings, where each string specifies one event handler for this tree node.
	 */
	public ArrayList getHandlerMap()
	{
		return _handlerMap;
	}
	
	/**
	 * Returns the event handlers for this tree node.
	 * @return Arraylist that specifies one event handler for this tree node per element.
	 */
	public ArrayList getBehaviorMap()
	{
		return _behaviorMap;
	}
	/**
	 * For a particular tree node, multiple eReferences can be added for the next level of child nodes. The <code>addIconMap</code>
	 * lets you add one child reference. Use this method to add more child references.
	 * @param v
	 */
	public void addEReference(String v)
	{
		_eReferenceArray.add(v);
	}
	
	/**
	 * Uses the colDataEmitterHelper to gather information for individual columns to be emitted
	 * in the Export method of the nodeAttrEmitterHelper 
	 */
	public void constructTreeColumnData()
	{
		StringBuffer colString = new StringBuffer(128);
		ColDataEmitterHelper treeColData;
	
		//Create data map containing all data for the node
		for (int i = 0; i <  _colDataList.size(); ++i) 
		{
			treeColData = (ColDataEmitterHelper)  _colDataList.get(i);
			String dataRef=treeColData.getAttributeName();
			colString.append(dataRef+";");
		}
		
		//Data map contains a reference to the node and the tabular data associated with it
		_colDataMap = colString.length() == 0 ? null : _eClassType + ":"+colString.toString();
		return;
	}
	
	
	/** 
	 * Export method does all the emitting of JavaScript code on to a page for the Tree Table's columne data.
	 * @param out 	Writer used to write the JavaScript to
	 * @param PC	PageContext that supplies the instance count to uniquely instantiate tree variables.
	 */
	public void Export(Writer out, PageContext PC) throws ExportException, IOException 
	{
		//Encode for portal
		String adapterVar = StringUtil.encodeString(ADAPTER_VAR + _varIndex,_isFaces);
		

		//If it is a treetable add columns to the node		
		if(_colDataList.size()!=0)
		{
			//Call the constructTreeColumnData() method which in turn cycles through all the columns for each node 
			// and gets their individual attributes and concatenates them.
			this.constructTreeColumnData();
			//Emit column data to the page
			if (_colDataMap != null) 
			{
				out.write(adapterVar + ".addNodeColumnData(\"" + _colDataMap + "\");\n");
			}
		}
		//If the node has behaviors associated with them emit them
		for(int i=0;i<_behaviorMap.size();i++)
		{
		
			out.write(adapterVar + ".addNodeBehavior(" + _behaviorMap.get(i) + ");\n");
		}
	


		
	}
	
	/*
	public void setType			    (String v)	{ _eClassType      = v;	}
	public void setEAttributeName	(String v)	{ _eAttributeName  = v;	}
	public void setOpenIcon			(String v)  { _openIcon = v;	    }
	public void setClosedIcon		(String v)  { _closedIcon = v;      }
	
	
	**
	 * Constructor is initialized with parameters, which in turn calls the Init method
	 * to initialize class variables.
	 * <p>
	 * @param eClassType		eClass type of the root eObject
	 * @param eAttributeName	Attribute name in eClass that will be displayed
	 * @param openIcon			Name of the image file used for open state of the tree folder(s)
	 * @param closedIcon	 	Name of the image file used for closed state of the tree folder(s)
	 *
	public NodeAttributeEmitterHelper(String eClassType, String eAttributeName, String eReferenceName,
	                                  String closedIcon, String openIcon)
	{
		Init(eClassType, eAttributeName, eReferenceName, closedIcon, openIcon);
	 }
	public NodeAttributeEmitterHelper(String eClassType, String eAttributeName, String eReferenceName)
	 {
	   Init(eClassType, eAttributeName, eReferenceName, null,null);
	 }
	public NodeAttributeEmitterHelper(String eClassType, String eAttributeName)
	 {
	   Init(eClassType, eAttributeName, null, null, null);
	 }
	
	public NodeAttributeEmitterHelper(String eClassType,
									  String eAttributeName,
									  String eReferenceName,
									  String closedIcon, String openIcon,
									  String nodeIconAttribute,
									  String nodeIconAttributeValue)
	{
		Init(eClassType, eAttributeName, eReferenceName, closedIcon, openIcon, nodeIconAttribute, nodeIconAttributeValue);
	 }
	
	public void Init(String eClassType, String eAttributeName, String eReferenceName, String closedIcon, String openIcon)
	 {
		_eClassType		= eClassType;
		_eAttributeName = eAttributeName;
	    setClosedIcon(closedIcon);
	    setOpenIcon(openIcon);
	    if (eReferenceName != null)
	     _eReferenceArray.add(eReferenceName);
	 }
	public void Init(String eClassType, String eAttributeName, String eReferenceName)
	 {
	   Init(eClassType, eAttributeName, eReferenceName, null, null);
	 }
	public void Init(String eClassType, String eAttributeName)
	 {
	   Init(eClassType, eAttributeName, null, null, null);
	 }
	public void Init(String eClassType,
					 String eAttributeName,
					 String eReferenceName,
					 String closedIcon,
					 String openIcon,
					 String nodeIconAttribute,
					 String nodeIconAttributeValue)
	 {
		this.Init(eClassType, eAttributeName, eReferenceName, closedIcon, openIcon);
		this._nodeIconAttribute = nodeIconAttribute;
		this._nodeIconAttributeValue = nodeIconAttributeValue;
	
		Streamer.debug.Header().println("eClassType: "		+this._eClassType+"\n"+
										"eAttribute: "		+this._eAttributeName+"\n"+
										"eReference: "		+eReferenceName+"\n"+
										"closedIcon: "		+this._closedIcon+"\n"+
										"openIcon  : "		+this._openIcon+"\n"+
										"nodeIconAttribute: "+this._nodeIconAttribute+"\n"+
										"nodeIconAttributeValue: "+this._nodeIconAttributeValue);
	 }
	
	
	**
	 * getStyleMapString constructs a string that the ODCTreeAdapter.setStyleMap() JS method
	 * will require. Example String: DIRE: name : childrendirectories; DIRE :name : childrenfiles ;
	 * String is constructed as follows: eClassType : eAttributeName : eReferenceName;
	 * Each node has its own style map string; the tree emitter concatenates the style map strings of individual
	 * nodes and passes the composite string on to setStyleMap()
	 * <p>
	 * @return String	constructed string for setting style map in tree adapter.
	 *
	public String getStyleMapString()
	{
		StringBuffer S = new StringBuffer(128);
		if(this._eReferenceArray.size()!=0)
		{
		  for(int i = 0; i < _eReferenceArray.size(); ++i)
		   {
	         S.append(this._eClassType).append(" : ").append(this._eAttributeName)
	                                   .append(" : ").append(this._eReferenceArray.get(i)).append("; ");
	       }
		}
		else
		{
			S.append(this._eClassType).append(" : ").append(this._eAttributeName).append("; ");
		}
		Streamer.debug.Header().println("StyleMapString: "+S.toString());
		return	S.toString();
	}
	
	**
	 * getNodeIconMapString constructs a string that the ODCTreeAdapter.setNodeIconMap() JS method
	 * will require. Example String: DIRE : jsl\\tree\\icons\\folder.gif : jsl\\tree\\icons\\folderopen.gif ;
	 * String is constructed as follows: eClassType : jsl\\\\tree\\\\icons\\\\closedIcon;
	 * Each node has its own node icon map string; the tree emitter concatenates the node icon map strings of individual
	 * nodes and passes the composite string on to setNodeIconMap()
	 * <p>
	 * @return String	constructed string for setting node icon map in tree adapter.
	 *
	public String getNodeIconMapString()
	{
	    if (_closedIcon == null && _openIcon == null)
	     return null;
	    if(_nodeIconAttribute == null || _nodeIconAttributeValue == null)
			return 	_eClassType+" : "+(_closedIcon==null?"":_closedIcon)+" : "+(_openIcon==null?"":_openIcon);
		else
			return 	_eClassType+" : "+ _nodeIconAttribute + " = " + _nodeIconAttributeValue + " : " + (_closedIcon==null?"":_closedIcon)+" : "+(_openIcon==null?"":_openIcon);
	}*/
	public static void main(String[] args)
	{
		//NodeAttributeEmitterHelper X = new NodeAttributeEmitterHelper("User", "Name", "Portfolios", "jsl\\icons\\closed.gif", "jsl\\icons\\open.gif");
		//System.out.println(X.getNodeIconMapString());
		NodeAttributeEmitterHelper X = new NodeAttributeEmitterHelper("Dir");
		try
		{
		
		   X.addStyleMap("Name",null, "SubDirectories");
		   X.addStyleMap("Name2",null, "SubDir2");
	    }
	    catch(Exception e)
	    {
	    	e.printStackTrace();
	    }
		X.addIconMap(
			"jsl\\icons\\closed.gif",
			"jsl\\icons\\open.gif",
			null,
			null,
			Boolean.TRUE,
			"closeiconalt",
			"openiconalt");
			
		X.addIconMap(
			"jsl\\icons\\closed.gif",
			"jsl\\icons\\open.gif",
			"Name",
			"Pending Drafts",
			Boolean.FALSE,
			"close&#34;iconalt1",
			"openi&#34;conalt1");
			
		X.addIconMap(
			"jsl\\icons\\closed.gif",
			"jsl\\icons\\open.gif",
			"Name",
			"Updates",
			Boolean.TRUE
			);
		
		String styleMap = X.getStyleMap();
		ArrayList iconMap = X.getIconMap();
		ArrayList handlerMap = X.getHandlerMap();
		System.out.println("Style Map: " + styleMap);
		for (int i = 0; i < iconMap.size(); i++)
			System.out.println("Icon Map--I: " + i + " " + iconMap.get(i));
		for (int j = 0; j < handlerMap.size(); j++)
			System.out.println(
				"Handler Map--J: " + j + " " + handlerMap.get(j));
	}


}