/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//
//OCO Source Materials
//
//5724-E76 and 5724-E77
//
// Copyright IBM Corp.  2003
//
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////

package com.ibm.odcb.jrender.emitters;

import java.io.IOException;
import java.io.Writer;

import com.ibm.odcb.jrender.mediators.ExportException;
import com.ibm.odcb.jrender.mediators.PageContext;
import com.ibm.odcb.jrender.misc.Streamer;
import com.ibm.odcb.jrender.misc.StringUtil;

/**
 * A generic emitter to bind an attribute of an eobject to data change
 * event on a particular HTML control.
 * @author fenil
 */
public class PropertyBinderEmitter extends BaseEmitter {

	protected String exportId, controlId, modelName, attributeName, controlChangeEvent, controlProperty;
	protected boolean isHTMLControl;
	
	/**
	 * Minimal Constructor.
	 *
	 */
	public PropertyBinderEmitter() {
		super("PropertyBinder");
	}

	/**
	 * Specialized constructor.
	 * @param exportId the export id of the eobject, whose attribute is being bound.
	 * @param controlId the id of the HTML control, being connected to this eobject.
	 * @param modelName the name of the model, which contains the eobject.
	 * @param attributeName the attribute of the eobject whose value is bound to the HTML control.
	 * @param controlChangeEvent the name of the event which is fired when the value of the HTML control changes
	 * @param controlProperty the property of the HTML control that is bound to attribute of the eobject.
	 */
	public PropertyBinderEmitter(
		String exportId,
		String controlId,
		boolean isHTMLControl,
		String modelName,
		String attributeName,
		String controlChangeEvent,
		String controlProperty) {
		this();
		this.Init(exportId, controlId, isHTMLControl, modelName, attributeName, controlChangeEvent, controlProperty);
	}
	

	/**
	 * Initializer. Same as the constructor.
	 * @param exportId
	 * @param controlId
	 * @param modelName
	 * @param attributeName
	 * @param controlChangeEvent
	 * @param controlProperty
	 */
	public void Init(
		String exportId,
		String controlId,
		boolean isHTMLControl,
		String modelName,
		String attributeName,
		String controlChangeEvent,
		String controlProperty) {
		this.exportId = exportId;
		this.controlId = controlId;
		this.isHTMLControl = isHTMLControl;
		this.modelName = modelName;
		this.attributeName = attributeName;
		this.controlChangeEvent = controlChangeEvent;
		this.controlProperty = controlProperty;
		
		Streamer.debug.Header().println("exportID: "		+this.exportId+ "\n"+
										"controlID: "		+this.controlId+ "\n"+
										"modelName: "		+this.modelName+ "\n"+
										"attributeName: "	+this.attributeName+"\n"+
										"controlChangeEvent: "+this.controlChangeEvent+"\n"+
										"controlProperty: "	 +this.controlProperty 
										);
	}
	
	/**
	 * Export the JavaScript which bind the eobject to the HTML control.
	 * @param out the writer which writes the JS to the output stream.
	 * @param PC the ODC <code>PageContext</code> object.
	 */
	public void Export(Writer out, PageContext PC) throws ExportException, IOException {
		Streamer.trace.Header().println("Entering PropertyBinderEmitter export...");
		this._varIndex = PC.NextInstanceCount();
		String varName = ADAPTER_VAR+_varIndex;
		String control = null;
		
		//Boolean storing flag that governs encoding. If page has FacesContext encoding should occur.
		boolean isFaces=StringUtil.checkEncoding();
		
		if(this.isHTMLControl)
		{
		    control = "document.getElementById('"+ controlId+ "')";
		}
		else
		{
		    control = this.controlId;
		}
		
		//Encoding Variables For Portal
		String _varName = StringUtil.encodeString(varName,isFaces);
        modelName=StringUtil.encodeString(modelName,isFaces);
        String _modelName = StringUtil.encodeString("WDO4JSModelRoot_"+modelName+".Root",isFaces);
        String _exportId = StringUtil.encodeString(exportId,isFaces);
        
		out.write(
			"var "
				+ _varName
				+ " = new PropertyBinder(findEObjectByXMIID("
				+ _modelName
				+ ",'"
				+ _exportId
				+ "')"
				+ ",'"
				+ attributeName
				+ "',"
				+ control);
		if (controlChangeEvent != null) {
			out.write(",'" + controlProperty + "', '" + controlChangeEvent + "');");
		} else {
			out.write(",'" + controlProperty + "', null);");
		}
		//out.write("\n");
		//out.write(varName + ".DataBind();");
		//out.write("\n");
		//out.write("</SCRIPT>");
		out.write("\n");
		Streamer.trace.Header().println("Exiting PropertyBinderEmitter export...");
	}

	/**
	 * @param string
	 */
	public void setAttributeName(String string) {
		attributeName = string;
	}

	/**
	 * @param string
	 */
	public void setControlChangeEvent(String string) {
		controlChangeEvent = string;
	}

	/**
	 * @param string
	 */
	public void setControlId(String string) {
		controlId = string;
	}

	/**
	 * @param string
	 */
	public void setControlProperty(String string) {
		controlProperty = string;
	}

	/**
	 * @param string
	 */
	public void setExportId(String string) {
		exportId = string;
	}

	/**
	 * @param string
	 */
	public void setModelName(String string) {
		modelName = string;
	}

}
