/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//  
//OCO Source Materials
//  
//5724-E76 and 5724-E77
//  
// Copyright IBM Corp.  2003
//  
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////

/*
 * Created on Apr 30, 2003
 *
 */
package com.ibm.odcb.jrender.emitters;

import java.io.Writer;
import java.io.IOException;
import com.ibm.odcb.jrender.mediators.ExportException;
import java.util.ArrayList;
import com.ibm.odcb.jrender.mediators.PageContext;
import com.ibm.odcb.jrender.misc.Streamer;


/**
 * @author Kaushal <br>
 *
 * Java interface to generate Radio Button HTML element
 */
public class RadioButtonEmitter extends BaseEmitter {

	public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;
	//public static final String RADIO_BUTTON_VAR = "ODCRadioButton";
	//public static final String RADIO_BUTTON_BINDER_VAR = "ODCRadioButtonBinder";

	protected final int _defaultCheckedIndex = -1;
	// Constructors; default one is empty; second one does all the work.
	// If object is instantiated with default one, developer has to call
	// set() methods to set class variables.
	public RadioButtonEmitter()
	{
		super("ODCRadioButton");
	}
	/**
	 * Constructor that initializes all class variables by calling an Init() function 
	 * <p>
	 * @param modelName						Name of the data model this control is bound to
	 * @param eObjectID						ID of the eObject in the data model that the control is bound to
	 * @param eAttributeName				Attribute name in the eObject that the control specifically talks to
	 * @param radioButtonValueList			Values of the radio button constituents
	 * @param radioButtonTextDisplayList	Text to display along side the radio buttons
	 * @param checkedIndex					Which of the radio buttons is to be checked by default
	 */
	public RadioButtonEmitter(String modelName,
							  String eObjectID,
							  String eAttributeName,
							  ArrayList radioButtonValueList,
							  ArrayList radioButtonTextDisplayList,
							  int checkedIndex)
	{
		super("ODCRadioButton");
		this.Init(modelName, eObjectID, eAttributeName,
				  radioButtonValueList, radioButtonTextDisplayList, checkedIndex);
	}

	// Class variables	
	protected String _modelName;
	protected String _eObjectID;
	protected String _eAttributeName;
	protected int _checkedIndex = _defaultCheckedIndex; // checkedIndex ranges between 0 to radio Button ArraySize - 1
	protected ArrayList _radioButtonValueList = new ArrayList(10);
	protected ArrayList _radioButtonTextDisplayList = new ArrayList(10);
	
	// init() initializes all class variables.
	public void Init(String modelName,
					 String eObjectID,
					 String eAttributeName,
					 ArrayList radioButtonValueList,
					 ArrayList radioButtonTextDisplayList,
					 int checkedIndex)
	{
		// Making sure the specified checked Index is not out of bounds
		if(checkedIndex < radioButtonValueList.size())
		{	_checkedIndex = checkedIndex;} 
		else
		{	_checkedIndex = _defaultCheckedIndex; } 

		_radioButtonValueList= radioButtonValueList;
		_radioButtonTextDisplayList = radioButtonTextDisplayList;
		_modelName = modelName;
		_eObjectID = eObjectID;
		_eAttributeName = eAttributeName;
		
		Streamer.debug.Header().println("In RadioButtonEmitter --> " + "\n"+
										"modelName: "		+this._modelName+ "\n"+
										"eObjectId: "		+this._eObjectID+"\n"+
										"eAttributeName: "	+this._eAttributeName+"\n"
										);
	}

	// All set functions to initialize class instance variables.

	public void setModelName(String modelName)		{ _modelName = modelName; 	}
	public void setEObjectID(String eObjectID)		{ _eObjectID = eObjectID;	}
	public void setEAttributeName(String eAttributeName){_eAttributeName = eAttributeName;	}
	public void setCheckedIndex(int checkedIndex)	{	_checkedIndex = checkedIndex;	}
	public void addRadioButton(String radioButtonValue, String radioButtonTextDisplayString)
	{
		_radioButtonValueList.add(radioButtonValue);
		_radioButtonTextDisplayList.add(radioButtonTextDisplayString);
	}
	
	/**
	 * Export method emits the JS code to render the Radio Button HTML elements and binds
	 * it to a data model via a Property Binder.
	 * <p>
	 * @param out	Writer output stream that the generated code is written to
	 * @param PC	Page Context
	 */
	public void Export(Writer out, PageContext PC) throws ExportException,IOException
	{
		Streamer.trace.Header().println("Entering RadioButtonEmitter export...");
		_varIndex = PC.NextInstanceCount();
		int numRadioButtons = java.lang.Math.min(_radioButtonValueList.size(),_radioButtonTextDisplayList.size());
		for(int i=0; i<numRadioButtons; i++)
		{
			if(_checkedIndex == i)
			{
				out.write("<input type=radio name=\""+CONTROL_VAR+_varIndex+"\" value=\""+_radioButtonValueList.get(i)+"\" checked=\"checked\" />");
				out.write("\n");
			}
			else
			{
				out.write("<input type=radio name=\""+CONTROL_VAR+_varIndex+"\" value=\""+_radioButtonValueList.get(i)+"\" />");
				out.write("\n");
			}
			String tempStr = (String)_radioButtonTextDisplayList.get(i);
			out.write(tempStr);
			out.write("\n");
		}
		
		out.write("<script type=\"text/javascript\">");out.write("\n");
		out.write("\n");out.write("var "+ADAPTER_VAR+_varIndex+" = new PropertyBinder(" +
											"findEObjectByXMIID(WDO4JSModelRoot_"+_modelName+".Root, \""+_eObjectID+"\")," +
											"\n\t\t\""+_eAttributeName+"\","+
											"\n\t\t document.getElementById(\""+CONTROL_VAR+_varIndex+"\"),"+
											"\n\t\t\"value\","+
											"\n\t\t\"onclick\");");
		out.write("\n");
		out.write(ADAPTER_VAR+_varIndex+".DataBind();");out.write("\n");

		// Registering control not needed: being done in JSF (July 30, 2003)
		//out.write(PAGE_CONTROL_VAR+".registerControl(\""+CONTROL_VAR+_varIndex+"\", "+CONTROL_VAR+_varIndex+");"); out.write("\n");

		out.write("</script>");out.write("\n");
		out.write("\n");
		Streamer.trace.Header().println("Exiting RadioButtonEmitter export...");
	}
}