/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//  
//OCO Source Materials
//  
//5724-E76 and 5724-E77
//  
// Copyright IBM Corp.  2003
//  
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////

/*
 * Created on Apr 30, 2003
 *
 */
package com.ibm.odcb.jrender.emitters;

import java.io.Writer;
import java.io.IOException;

import com.ibm.odcb.jrender.misc.Config;
import com.ibm.odcb.jrender.misc.Streamer;
import com.ibm.odcb.jrender.misc.StringUtil;
import com.ibm.odcb.jrender.mediators.ExportException;
import java.util.ArrayList;
import com.ibm.odcb.jrender.mediators.PageContext;
import com.ibm.odcb.jrender.misc.URLRewriter;


/**
 * @author Kaushal <br>
 * 
 * Java interface for a select / pull-down menu / HTML element
 */
public class SelectEmitter extends BaseEmitter {

	public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;

	private static final String SELECTCONTROL = Config.versionizeFilename("selectcontrol.js");
	private static final String SELECTADAPTER = Config.versionizeFilename("selectadapter.js");
	private static final String JSL_SELECT_C = Config.versionizeFilename("jsl_select_c.js");

	protected final int _defaultSize = 1;  		  // Number of things that show up in select window.
	protected final int _defaultMultipleFlag = 0; // Zero implies no multiple selections allowed.

	// Constructors; default one is empty; second one does all the work.
	// If object is instantiated with default one, developer has to call
	// set() methods to set class variables.
	public SelectEmitter()
	{
		super("ODCSelect");
	}

	/**
	 * Constructor to initialize all class variables. 
	 * <p>
	 * @param modelName				Name of the data model the control is bound to	
	 * @param eObjectID				ID of the eObject in the data model the control is bound to
	 * @param eAttributeName		Attribute name within the eObject the control talks to
	 * @param name					Name of the html select element
	 * @param size					Size is the number of options in the select element
	 * @param callBackFunction		Callback function name that process events from the select actions
	 * @param multipleFlag			Is multiple selection allowed or not
	 * @param optionValueList		List of the values to be shown as options
	 * @param optionTextList		Text to be displayed for each of the options in the Select
	 */
	public SelectEmitter(	String 	modelName,
							String 	eObjectID,
							String	eReferenceName,
							String 	eAttributeName,
							int 	size,
							String 	callBackFunction,
							int 	multipleFlag,
							ArrayList optionValueList,
							ArrayList optionTextList)
	{
		super("ODCSelect");
		this.Init(	modelName,
					eObjectID,
					eReferenceName,
					eAttributeName,
					size,
				  	multipleFlag,
				  	callBackFunction,
				  	optionValueList,
				  	optionTextList);
	}
	public SelectEmitter(	String modelName,
							String eObjectID,
							String eReferenceName,
							String eAttributeName,
							String callBackFunction,
							ArrayList optionValueList,
							ArrayList optionTextList)
	{
		super("ODCSelect");
		this.Init(modelName,
				  eObjectID,
				  eReferenceName,
				  eAttributeName,
				  _defaultSize,
				  _defaultMultipleFlag,
				  callBackFunction,
				  optionValueList,
				  optionTextList);
	}

	// Class variables	
	protected String _modelName;
	protected String _eObjectID;
	protected String _eReferenceName;
	protected String _eAttributeName;
	protected String _callBackFunction;
	protected int	 _size = _defaultSize;
	protected int	 _multipleFlag = _defaultMultipleFlag;
	protected ArrayList _optionValueList = new ArrayList();
	protected ArrayList _optionTextList  = new ArrayList();
	protected String _originalValueString = "";
	protected String _originalTextString  = "";
	
	// init() initializes all class variables.
	public void Init(String modelName,
				  	 String eObjectID,
					 String eReferenceName,
					 String eAttributeName,
					 int	size,
					 int	multipleFlag,
					 String callBackFunction,
					 ArrayList optionValueList,
					 ArrayList optionTextList)
	{
		_modelName 		  = modelName;
		_eObjectID		  = eObjectID;
		_eReferenceName	  = eReferenceName;
		_eAttributeName	  = eAttributeName;
		_size	 	 	  = size;
		_multipleFlag= multipleFlag;
		_callBackFunction = callBackFunction;
		_optionValueList  = optionValueList;
		_optionTextList   = optionTextList;
		
		Streamer.debug.Header().println("In SelectEmitter --> " + "\n"+
										"modelName: "		+this._modelName+ "\n"+
										"eObjectId: "		+this._eObjectID+"\n"+
										"eReferenceName: "	+this._eReferenceName+"\n"+
										"eAttributeName: "	+this._eAttributeName+"\n"+
										"size: "			+this._size+"\n"+
										"multipleFlag: "	+this._multipleFlag+"\n"+
										"callBackFunction: "+this._callBackFunction+"\n"
										);
	}

	// All set functions to initialize class instance variables.
	public void setModelName(String modelName)
	{
		_modelName = modelName;
	}
	public void setEObjectID(String eObjectID)
	{
		_eObjectID = eObjectID;
	}
	public void setEAttributeName(String eAttributeName)
	{
		_eAttributeName = eAttributeName;
	}
	public void setEReferenceName(String eReferenceName)
	{
		_eReferenceName = eReferenceName;
	}
	public void setCallBackFunction(String callBackFunction)
	{
		_callBackFunction = callBackFunction;
	}
	public void setSize(int size)
	{
		_size = size;
	}
	public void setMultipleFlag(int multipleFlag)
	{
		_multipleFlag = multipleFlag;
	}
	public void addSelectOption(String optionValue, String optionText)
	{
		_optionValueList.add(optionValue);
		_optionTextList.add(optionText);
	}

	/**
	 * Export method that emits the JS code for rendering a select HTML element. The export method
	 * also emits code for the Select JS Control and Adapter and binds a WDO to the Select HTML element.
	 * 
	 * <p>
	 * @param out	Writer output stream the generated code is output to
	 * @param PC	Page Context
	 */
	public void Export(Writer out, PageContext PC) throws ExportException, IOException
	{
		Streamer.trace.Header().println("Entering SelectEmitter export...");
		_varIndex = PC.NextInstanceCount();
		
		URLRewriter R = (URLRewriter) PC.getContextVariable(InitializationEmitter._ID_URL_REWRITER);
		boolean debugMode = PC.getContextVariable(InitializationEmitter._ID_DEBUG_MODE) == InitializationEmitter.DEBUG_YES ? true:false;

		if(TestGuard(PC,"SELECT_JS_FLAG") == false)
		{
			if(debugMode == true)
			{		
				out.write("\n<script type=\"text/javascript\" src=\""+R.Rewrite("jsl/select/" + SELECTCONTROL)+"\"></script>"
						 +"\n<script type=\"text/javascript\" src=\""+R.Rewrite("jsl/select/" + SELECTADAPTER)+"\"></script>");
				Streamer.trace.Header().println("Included select JS control and adapter files...");
			}
			else
			{
				out.write("<script type=\"text/javascript\" src=\""+R.Rewrite("jsl/" + JSL_SELECT_C)+"\"></script>");
				out.write("\n");
				Streamer.trace.Header().println("Included compressed select JS file...");
			}
		}

		if(_multipleFlag == 0)
		{
				out.write("<select size=\""+_size+"\" id=\""+HTML_ELEMENT+_varIndex+"\" name=\""+HTML_ELEMENT+_varIndex+"\"");
		}
		else
		{
			out.write("<select multiple=\"multiple\" size=\""+_size+" id=\""+HTML_ELEMENT+_varIndex+"\" name=\""+HTML_ELEMENT+_varIndex+"\"");
		}
		if(_callBackFunction != null)
		{
			out.write(" onclick=\""+_callBackFunction+"(this)\"");
		}
		out.write(" >");out.write("\n");
		
		// If optionValues or textValues are not set by developer, the numOptions would be zero.
		// In that case, the emitter does not emit HTML code for OPTIONS.
		int numOptions = _optionValueList.size();
		if(numOptions != 0)
			this.buildOriginalOptionStrings(numOptions);
		
		out.write("</select>");out.write("\n");
		out.write("<script type=\"text/javascript\">");out.write("\n");
		out.write("\n");
		
		out.write("var "+CONTROL_VAR+_varIndex+" = new SelectControl();");out.write("\n");
		out.write(CONTROL_VAR+_varIndex+".selectObj = window.document.getElementById(\""+HTML_ELEMENT+_varIndex+"\");");out.write("\n");
		
		out.write(CONTROL_VAR+_varIndex+".originalValueArray = ["+_originalValueString+"];");out.write("\n");
		out.write(CONTROL_VAR+_varIndex+".originalTextArray = ["+_originalTextString+"];");out.write("\n");
		out.write(CONTROL_VAR+_varIndex+".setOriginalOptions();");out.write("\n");
		
		out.write("var "+ADAPTER_VAR+_varIndex+" = new SelectAdapter("+CONTROL_VAR+_varIndex+");");out.write("\n");
		out.write(ADAPTER_VAR+_varIndex+".attributeName = \""+_eAttributeName+"\";");out.write("\n");
		out.write(ADAPTER_VAR+_varIndex+".parentEObject = findEObjectByXMIID(WDO4JSModelRoot_"+_modelName+".Root, \""+_eObjectID+"\");");out.write("\n");
		out.write(ADAPTER_VAR+_varIndex+".propertyName = \""+_eReferenceName+"\";");out.write("\n");
		out.write(ADAPTER_VAR+_varIndex+".bind();");out.write("\n");
		out.write(CONTROL_VAR+_varIndex+".updateControl();");out.write("\n");
		
		// Registering control not needed: being done in JSF (July 30, 2003)
		//out.write(PAGE_CONTROL_VAR+".registerControl(\""+CONTROL_VAR+_varIndex+"\", "+CONTROL_VAR+_varIndex+");"); out.write("\n");
		//out.write(PAGE_CONTROL_VAR+".registerAdapter(\""+ADAPTER_VAR+_varIndex+"\", "+ADAPTER_VAR+_varIndex+");"); out.write("\n");

		out.write("</script>");out.write("\n");
		Streamer.trace.Header().println("Exiting SelectEmitter export...");
	}
	
	/**
	 * Method builds string array suited for JavaScript consumption with strings double quoted.
	 * @param numOptions Number of options in select control, as specified by user (not WDO input)
	 */
	public void buildOriginalOptionStrings(int numOptions)
	{
		for	(int i = 0; i < numOptions; i++)
		{
			if(i != 0)
			{
				_originalValueString = _originalValueString.concat(",");
				_originalTextString  = _originalTextString.concat(",");
			}
			_originalValueString = _originalValueString.concat(this.buildQuotedString((String)this._optionValueList.get(i)));			
			_originalTextString  = _originalTextString.concat(this.buildQuotedString((String)this._optionTextList.get(i)));			
		}
	}
	
	/**
	 * Convenience function to double quote a string and return.
	 * @param tempStr	Input is a regular string: a
	 * @return			Output is a quoted string: "a"
	 */
	protected String buildQuotedString(String tempStr)
	{	
		String s1 = tempStr;
		s1 = StringUtil.QuoteDoubleAndEscape(s1,true);			
		s1 = (s1.substring(1,s1.length()-2)).concat(s1.substring(s1.length()-1));
		return s1;
	}
}