/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//
//OCO Source Materials
//
//5724-E76 and 5724-E77
//
// Copyright IBM Corp.  2003
//
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////

/*
 * Created on Apr 7, 2003
 *
 */
package com.ibm.odcb.jrender.emitters;

import java.io.IOException;
import java.io.Writer;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import com.ibm.odcb.jrender.mediators.ExportException;
import com.ibm.odcb.jrender.mediators.PageContext;
import com.ibm.odcb.jrender.misc.Config;
import com.ibm.odcb.jrender.misc.Streamer;
import com.ibm.odcb.jrender.misc.URLRewriter;
import com.ibm.odcb.jrender.misc.StringUtil;


/**
 * Generates the JavaScript code for the TabbedPanel.
 * @author Kaushal
 */
public class TabbedPanelEmitter extends BaseEmitter {

	public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;

	private static final String TABBEDPANELCONTROL = Config.versionizeFilename("tabbedpanelcontrol.js");
	private static final String JSL_TABBEDPANEL_C = Config.versionizeFilename("jsl_tabbedpanel_c.js");

	protected String _tabSwitchHeight = null;
	protected String _tabSwitchWidth = null;
	protected String _OnFinish;
	protected String _OnCancel;
	protected String _onpanelenter;
	protected String _onpanelexit;
	protected String _oninitialpanelshow;
	protected String _initialPanelId;
	protected String _layout;
	protected String _FinishButtonId = null;
	protected String _CancelButtonId = null;
	protected String _BackButtonId = null;
	protected String _NextButtonId = null;
	protected boolean _ShowTabs;
	protected boolean _ShowBackNextButtons;
	protected ArrayList _panelList = new ArrayList(7);
	protected HashMap _panelMap = new HashMap();
	// new flag to indicate there is a custom button panel present.
	protected ButtonPanelEmitter _ButtonPanel = null;
	
	protected String _styleClass;
	protected int _tabSize;
	//protected boolean _OnFinishSubmit;
	protected boolean _VariableTabLength;
	//protected boolean _percentageExists = false;
	protected int _slantActiveLeft = -1;
	protected int _slantActiveRight = -1;
	protected int _slantInactiveLeft = -1;
	protected int _slantInactiveRight = -1;
	
	private final String _WIZARD ="wizard";
	private final String _NOTEBOOK ="notebook";
	
	/**
	 * Minimal Constructor.
	 *
	 */
	public TabbedPanelEmitter() {
		super("ODCTabPanel");
	}

	/**
	 * This constructor initializes all class variables needed by tab panel.
	 * <p>
	 * @param tabSwitchHeight		Height of the tab panel in pixels
	 * @param tabSwitchWidth		Width of the tab panel in pixels
	 * @param tabSize				How many tabs to be displayed in one page
	 * @param onFinish		        Call back function that handles Finish button push event
	 * @param onCancel				Call back function that handles the Cancel button push event
	 * @param onpanelenter			Call back function that handles the &quot;onpanelenter&quot; event. This event is fired when
	 * entering any panel in the tabbed panel.
	 * @param onpanelexit			Call back function that handles the &quot;onpanelexit&quot; event. This event is fired when
	 * exitting any panel in the tabbed panel.
	 * @param ShowBackNextButtons	Show next and previous buttons or not
	 * @param ShowTabs				Show tabs on top of the tab panel or not
	 * @param styleClass			the CSS style class to be used for the tabbed panel
	 * @param OnFinishSubmit		Specfies whether the finish button is to be used to the submit the page back to the server.
	 * @param variableTabLength		Specifies whether the length of the tabs should vary based on the size of the labels of
	 * the individual panels.
	 * @param slantActiveLeft 		the number of pixels to be trimmed from the left corner of the currently active panel
	 * @param slantActiveRight		the number of pixels to be trimmed from the right corner of the currently active panel
	 * @param slantInactiveLeft		the number of pixels to be trimmed from the left corner of an inactive panel
	 * @param slantInctiveRight		the number of pixels to be trimmed from the right corner of an inactive panel
	 */
	public TabbedPanelEmitter(
		String tabSwitchHeight,
		String tabSwitchWidth,
		int tabSize,
		String onFinish,
		String onCancel,
		String onpanelenter,
		String onpanelexit,
		boolean ShowBackNextButtons,
		boolean ShowTabs,
		String styleClass,
		boolean variableTabLength,
		int slantActiveLeft,
		int slantActiveRight,
		int slantInactiveLeft,
		int slantInactiveRight,
		String layout) {
		//_tabSwitchHeight = tabSwitchHeight;
		//_tabSwitchWidth = tabSwitchWidth;
		
		
		if (tabSwitchHeight != null) {
			this.setTabPanelHeight(tabSwitchHeight);
		}
		if (tabSwitchWidth != null)
		{	
			this.setTabPanelWidth(tabSwitchWidth);
		}
		
		
		_tabSize = tabSize;
		_ShowTabs = ShowTabs;
		_ShowBackNextButtons = ShowBackNextButtons;
		_styleClass = styleClass;
		_OnFinish = onFinish;
		_OnCancel = onCancel;
		_onpanelenter = onpanelenter;
		_onpanelexit = onpanelexit;
		_VariableTabLength = variableTabLength;
		_slantActiveLeft = slantActiveLeft;
		_slantActiveRight = slantActiveRight;
		_slantInactiveLeft = slantInactiveLeft;
		_slantInactiveRight = slantInactiveRight;
		_layout = layout;
	}

	/**
	 *  Init method that initializes all class variables needed by tab panel. Same as the constructor.
	 * @param tabSwitchHeight
	 * @param tabSwitchWidth
	 * @param tabSize
	 * @param onFinish
	 * @param onCancel
	 * @param onpanelenter
	 * @param onpanelexit
	 * @param ShowBackNextButtons
	 * @param ShowTabs
	 * @param styleClass
	 * @param OnFinishSubmit
	 * @param variableTabLength
	 * @param slantActiveLeft
	 * @param slantActiveRight
	 * @param slantInactiveLeft
	 * @param slantInactiveRight
	 */
	public void Init(
		String tabSwitchHeight,
		String tabSwitchWidth,
		int tabSize,
		String onFinish,
		String onCancel,
		String onpanelenter,
		String onpanelexit,
		String onpanelshow,
		String initialPanelId,
		String layout,
		boolean ShowBackNextButtons,
		boolean ShowTabs,
		String styleClass,
		boolean variableTabLength,
		int slantActiveLeft,
		int slantActiveRight,
		int slantInactiveLeft,
		int slantInactiveRight) {
		//_tabSwitchHeight = tabSwitchHeight;
		//_tabSwitchWidth = tabSwitchWidth;
		if (tabSwitchHeight != null) {
			this.setTabPanelHeight(tabSwitchHeight);
		}
		if (tabSwitchWidth != null)
		{	
			this.setTabPanelWidth(tabSwitchWidth);
		}
		_tabSize = tabSize;
		_ShowTabs = ShowTabs;
		_ShowBackNextButtons = ShowBackNextButtons;
		_styleClass = styleClass;
		_OnFinish = onFinish;
		_OnCancel = onCancel;
		_onpanelenter = onpanelenter;
		_onpanelexit = onpanelexit;
		_oninitialpanelshow = onpanelshow;
		_initialPanelId = initialPanelId;
		_VariableTabLength = variableTabLength;
		_layout = layout;

		_slantActiveLeft = slantActiveLeft;
		_slantActiveRight = slantActiveRight;
		_slantInactiveLeft = slantInactiveLeft;
		_slantInactiveRight = slantInactiveRight;

		Streamer.debug.Header().println(
			"In TabbedPanelEmitter --> "
				+ "\n"
				+ "tabSwitchHeight: "
				+ this._tabSwitchHeight
				+ "\n"
				+ "tabSwitchWidth: "
				+ this._tabSwitchWidth
				+ "\n"
				+ "tabSize: "
				+ this._tabSize
				+ "\n"
				+ "showBackNextButtons?: "
				+ this._ShowBackNextButtons
				+ "\n"
				+ "showTabs?: "
				+ this._ShowTabs
				+ "\n"
				+ "cssPrefix: "
				+ this._styleClass
				+ "\n"
				+ "variableTabLength: "
				+ this._VariableTabLength
				+ "\n"
				+ "slantActiveLeft: "
				+ this._slantActiveLeft
				+ "\n"
				+ "slantActiveRight: "
				+ this._slantActiveRight
				+ "\n"
				+ "slantInactiveLeft: "
				+ this._slantInactiveLeft
				+ "\n"
				+ "slantInactiveRight: "
				+ this._slantInactiveRight);
	}

	// All set methods
	public void setTabPanelHeight(String height) {
		char ch = height.charAt(height.length() - 1);
		String lHeight;
		if (ch == '%') {
			lHeight = height.substring(0, height.length() - 1); // stripping out the % sign at the end
		//	_percentageExists = true;
		} else {
			lHeight = height;
		}
		Integer heightInt = Integer.valueOf(lHeight);
		// Only assign value if height specified was greater than zero.
		if (heightInt.intValue() > 0)
			_tabSwitchHeight = height;
	}
	public void setTabPanelWidth(String width) {
		char cw = width.charAt(width.length() - 1);
		String lWidth;
		if (cw == '%') {
			lWidth = width.substring(0, width.length() - 1); // stripping out the % sign at the end
		//	_percentageExists = true;
		} else {
			lWidth = width;
		}
		Integer widthInt = Integer.valueOf(lWidth);
		// Only assign value if width specified was greater than zero.
		if (widthInt.intValue() > 0)
			_tabSwitchWidth = width;
	}
	public void setStyleClass(String styleClass) {
		_styleClass = styleClass;
	}
	public void setTabSize(int tabSize) {
		_tabSize = tabSize;
	}

	public void setFinishButtonId(String anId) {
		_FinishButtonId = anId;
	}
	public void setCancelButtonId(String anId) {
		_CancelButtonId = anId;
	}

	public void setBackButtonId(String anId) {
		_BackButtonId = anId;
	}
	public void setNextButtonId(String anId) {
		_NextButtonId = anId;
	}

	public void setOnFinish(String onFinish) {
		_OnFinish = onFinish;
	}

	public void setOnCancel(String onCancel) {
		_OnCancel = onCancel;
	}

	public void setOnpanelenter(String onpanelenter) {
		_onpanelenter = onpanelenter;
	}

	public void setOnpanelexit(String onpanelexit) {
		_onpanelexit = onpanelexit;
	}
	
	public void setOninitialpanelshow(String oninitialpanelshow){
		_oninitialpanelshow = oninitialpanelshow;
	}
	
	public void setInitialPanelId(String initialPanelId){
		_initialPanelId = initialPanelId;
	}
	
	public void addPanel(PanelEmitter P) {
		
		
		// Need to wrap the original ArrayList with a hash Map
		// This will allow child tabs to have their own arrayList
		// associated in the hashmap with their parents object.
		
		
		if (P.getParentPane() == null) {
			this._panelList.add(P);
		}
		else {
			ArrayList tmpList;
			tmpList = (ArrayList)_panelMap.get(P.getParentPane());
			if (tmpList == null) {
				tmpList = new ArrayList();
			}
			tmpList.add(P);
			_panelMap.put(P.getParentPane(),tmpList); 	
		}
		
		
	}
	
	public void setButtonPanel(ButtonPanelEmitter Bp)
	{
		this._ButtonPanel = Bp;
	}

	/**
	 * Export emits the code to render the tab panel and give it its attribute values.
	 * Tab panel does not really associate itself with a data model, it merely serves as a visual
	 * container for other controls--text fields, text areas, or tree control.
	 * <p>
	 * @param out	Writer output stream to write the generated code to
	 * @param PC	Page Context
	 */
	public void Export(Writer out, PageContext PC) throws ExportException, IOException {
		Streamer.trace.Header().println("Entering TabbedPanelEmitter export...");
		out.write("<script type=\"text/javascript\">ODCProgressBar.startRenderControl(tabbed_panel);</script>\n");
		_varIndex = PC.NextInstanceCount();
		URLRewriter R = (URLRewriter) PC.getContextVariable(InitializationEmitter._ID_URL_REWRITER);
		boolean debugMode =
			PC.getContextVariable(InitializationEmitter._ID_DEBUG_MODE) == InitializationEmitter.DEBUG_YES
				? true
				: false;

		String locale = "";
		String logLevelString = "";
		String Prefix = "";
		String Postfix = "";
		Integer logLevel;
		String id = null;
		
		// Check if encoding should occur
		boolean isFaces = StringUtil.checkEncoding();
		//Encode variable
		String encodedControlVar =  StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces);
		
		if(this.getOverriddingId()!=null)
		{
		   id = this.getOverriddingId();
		}
		else
		{
		   id = encodedControlVar + "DIV";
		}

		if (TestGuard(PC, "TAB_JS_FLAG") == false) {
			if (debugMode == true) {
				out.write("\n<script type=\"text/javascript\" src=\"" + R.Rewrite("jsl/tab/" + TABBEDPANELCONTROL) + "\"></script>\n");
				Streamer.trace.Header().println("Included individual tabbedpanel JS file...");
			} else {
				out.write("<script type=\"text/javascript\" src=\"" + R.Rewrite("jsl/" + JSL_TABBEDPANEL_C) + "\"></script>\n");
				Streamer.trace.Header().println("Included compressed tabbedpanel JS file...");
			}
		}
		//String _StyleSheetDir = Config.getString(Config.PROP_CLIENT_STYLESHEETDIR);
		//out.write("<link rel=\"stylesheet\" type=\"text/css\" href=\"" + R.Rewrite(_StyleSheetDir + "/tabpanel.css") + "\">\n"

		// Create tab conatiner header; set Tab Panel properties: name, height, width, theme.
		out.write("<div id=\"" + id + "\"></div>\n"
		         +"<script type=\"text/javascript\">\n");

		// Added a new parameter Aug. 19. 2003 --> URL_REWRITER_PREFIX, to match Qun's changes in tab panel JS code
		out.write(encodedControlVar+ " = new TabPanel(document.getElementById(\""+ id + "\"), "
				                                           + _ShowBackNextButtons + ", "
				                                           + _ShowTabs + ", URL_REWRITER_PREFIX);\n"
				 +encodedControlVar + ".Name = \"" + encodedControlVar + "\";\n");

		if (_tabSwitchHeight != null)
			out.write(encodedControlVar + ".setHeight(\"" + _tabSwitchHeight + "\");\n");
		if (_tabSwitchWidth != null)
			out.write(encodedControlVar + ".setWidth(\"" + _tabSwitchWidth + "\");\n");
		if (_layout != null)
		{
			_layout = _layout.toLowerCase();
			//allowable values are "", "wizard", "notebook"
			if(!(_layout.trim()).equalsIgnoreCase("")) // cause there is no need to set it as normal
			{
				if( _layout.equalsIgnoreCase(_WIZARD) || _layout.equalsIgnoreCase(_NOTEBOOK))
					out.write(encodedControlVar + ".setLayout(\"" + _layout + "\");\n");
				else
					System.err.println("+++++++++++++++ allowable tabpanel layout values are - 'wizard', 'notebook' or blank. '"+ _layout +"' is not. you will get a normal tabpanel. ++++++++++");
			}
		}
		out.write(encodedControlVar + ".setVariableTabLength(\"" + _VariableTabLength + "\");\n");
/*
		if (_percentageExists)
		 {
			out.write("function " + encodedControlVar + "resize() { " + CONTROL_VAR + _varIndex + ".reSize(); }\n"
			         +"\n"
			         +"if (window.addEventListener)\n"
			         +" window.addEventListener('resize'," + encodedControlVar + "resize,false);\n"
			         +"else\n"
			         +" window.attachEvent('onresize'," + encodedControlVar + "resize);\n");
		}
*/

		out.write(encodedControlVar + ".tabSize = " + _tabSize + ";\n");
		if (_FinishButtonId != null)
		 {
			out.write(encodedControlVar + ".setFinishButtonId(\"" + _FinishButtonId + "\");" + "\n");
		 }
		if (_CancelButtonId != null)
		 {
			out.write(encodedControlVar + ".setCancelButtonId(\"" + _CancelButtonId + "\");" + "\n");
		 }
		if (_BackButtonId != null)
		 {
			out.write(encodedControlVar + ".setBackButtonId(\"" + _BackButtonId + "\");" + "\n");
		 }
		if (_NextButtonId != null)
		 {
			out.write(encodedControlVar + ".setNextButtonId(\"" + _NextButtonId + "\");" + "\n");
		 }

		// Added Aug. 19, in response to Qun's tab panel JS code changes.
		// If CssPrefix is NOT null then call the JS function setCSSPrefix(); else do nothing
		// pass string into setCSSPrefix, instead of object -- fixed Oct. 27, 2003: Kaushal
		if (_styleClass != null)
		 {
		   out.write(encodedControlVar + ".setStyleClass(\"" + _styleClass + "\");" + "\n");
		 }

		out.write(encodedControlVar+".setNipAndTuckStyle("+_slantActiveLeft
		                                                      + " ," + _slantActiveRight
				                                              + " ," + _slantInactiveLeft
				                                              + " ," + _slantInactiveRight
				                                         + ");\n");

		// cycle through panelList and call add() for each member
		StringBuffer X = new StringBuffer(64);
		//RATLC01082051 - StringBuffers for disabled cancel/finish lists
		StringBuffer disableCancelList = new StringBuffer();
		StringBuffer disableFinishList = new StringBuffer();
		
		Iterator mapIterator = _panelMap.keySet().iterator();
		
		
		//System.out.println(mapIterator.toString());
		
		//System.out.println(_panelList.toString());
		
		boolean rootpass = false;
		
		// Changing to loop through hash map. Then loop through lists in map.
		while (mapIterator.hasNext() || !rootpass)
		{
			if (rootpass) {
				_panelList = (ArrayList) _panelMap.get(mapIterator.next());
			}
			rootpass = true;
			//System.out.println(_panelList.toString());
		
		for (int i = 0; i < _panelList.size(); i++) {
			PanelEmitter tempPanel = (PanelEmitter)_panelList.get(i);
			//System.out.println(tempPanel.toString());
			PanelEmitter prevPanel = null;
			PanelEmitter nextPanel = null;
			String nextPanelID = null;
			String prevPanelID = null;
			if (i == 0) {
				// prevPanel will be null here;
				// nextPanel ID will also be null if there is only 1 panel (then it is the 1st & last)
				if (_panelList.size() == 1) {
					nextPanel = null;
					nextPanelID = null;
				} else {
					nextPanel = (PanelEmitter) _panelList.get(i + 1);
					nextPanelID = nextPanel.getPanelID();
				}
				this.ExportAddStatements(i, tempPanel.getPanelDisplayName(), tempPanel.getPanelID(), nextPanelID,
				// next panel ID
				null, // prev panel ID
				tempPanel.getOnEnter(), tempPanel.getOnExit(), out, PC, tempPanel.getTabIcon(), 
				tempPanel.getParentPane(), tempPanel.getDisableTab());
			} else if (i == _panelList.size() - 1) {
				// nextPanel will be null here;
				// we'll be a little paranoid here and double check for size = 1 (single panel case)
				// this check is really not needed since this case was handled in the "if" branch above.
				if (_panelList.size() == 1) {
					prevPanel = null;
					prevPanelID = null;
				} else {
					prevPanel = (PanelEmitter) _panelList.get(i - 1);
					prevPanelID = prevPanel.getPanelID();
				}

				this.ExportAddStatements(i, tempPanel.getPanelDisplayName(), tempPanel.getPanelID(), null,
				// next panel ID
				prevPanelID, // prev panel ID
				tempPanel.getOnEnter(), tempPanel.getOnExit(), out, PC, tempPanel.getTabIcon(), 
				tempPanel.getParentPane(), tempPanel.getDisableTab());
			} else {
				prevPanel = (PanelEmitter) _panelList.get(i - 1);
				nextPanel = (PanelEmitter) _panelList.get(i + 1);
				this.ExportAddStatements(
					i,
					tempPanel.getPanelDisplayName(),
					tempPanel.getPanelID(),
					nextPanel.getPanelID(),
					prevPanel.getPanelID(),
					tempPanel.getOnEnter(),
					tempPanel.getOnExit(),
					out,
					PC,
					tempPanel.getTabIcon(),
					tempPanel.getParentPane(),
					tempPanel.getDisableTab());
			}
			// Creating the list of panels with the Finish button
			if (tempPanel._FinishCancelButton == true) {
				if (X.length() == 0) {
					X.append("'").append(tempPanel._panelID).append("'");
				} else {
					X.append(",'").append(tempPanel._panelID).append("'");
				}
			}
			//RATLC01082051 - put disabled lists together
			//disabled cancel button list
			if(tempPanel._disableCancel == true){
				if(disableCancelList.length() == 0){
					disableCancelList.append("'").append(tempPanel._panelID).append("'");
				} else {
					disableCancelList.append(",'").append(tempPanel._panelID).append("'");
				}
			}
			//disabled finish button list
			if(tempPanel._disbaleFinish == true){
				if(disableFinishList.length() == 0){
					disableFinishList.append("'").append(tempPanel._panelID).append("'");
				} else {
					disableFinishList.append(",'").append(tempPanel._panelID).append("'");
				}
			}
		}
		
		}
		//X.setCharAt(0,'[');
		if (X.length() > 0)
         {
			out.write(encodedControlVar + ".addFinishCancelButton([" + X.toString() + "]);\n");
		 }
		
		//RATLC01082051 - emit disable cancel/finish script
		if(disableCancelList.length() > 0){
			out.write(encodedControlVar + ".disableCancelButton([" + disableCancelList.toString() + "]);\n");
		}
		if(disableFinishList.length() > 0){
			out.write(encodedControlVar + ".disableFinishButton([" + disableFinishList.toString() + "]);\n");
		}
		
		
		
		if (this._OnFinish != null && !this._OnFinish.equals(""))
         {
			out.write(encodedControlVar + ".addHandler(\"onFinish\"," + "\"" + this._OnFinish + "\"" + ");\n");
		 }
		if (this._OnCancel != null && !this._OnCancel.equals(""))
         {
			out.write(encodedControlVar + ".addHandler(\"onCancel\"," + "\"" + this._OnCancel + "\"" + ");\n");
		 }
		if(this._onpanelenter != null && !this._onpanelenter.equals(""))
         {
			out.write(encodedControlVar + ".addHandler(\"onpanelenter\"," + "\"" + this._onpanelenter + "\"" + ");\n");
		 }
		if(this._onpanelexit != null && !this._onpanelexit.equals(""))
         {
			out.write(encodedControlVar + ".addHandler(\"onpanelexit\"," + "\"" + this._onpanelexit + "\"" + ");\n");
		 }
		if(this._oninitialpanelshow != null && !this._oninitialpanelshow.equals(""))
        {
			out.write(encodedControlVar + ".addHandler(\"oninitialpanelshow\"," + "\"" + this._oninitialpanelshow + "\"" + ");\n");
		 }
		if(this._initialPanelId != null && !this._initialPanelId.equals(""))
        {
			out.write(encodedControlVar + ".initialPanelId=\"" + _initialPanelId + "\";\n");
		}
		// This is for the buttonPanel at the end.
		if(this._ButtonPanel != null)
		{
			out.write(encodedControlVar + ".setCustomButtonPanel(\""+this._ButtonPanel.getPanelID()+
					"\",\""+this._ButtonPanel.getAlignContent()+"\");\n");			
		}
		out.write(encodedControlVar + ".updateControl();\n"
		          + "ODCProgressBar.endRenderControl(tabbed_panel);\n"
						 +"</script>\n");
		Streamer.trace.Header().println("Exiting TabbedPanelEmitter export...");
	}

	protected void ExportAddStatements(int ID, String panelDisplayName,
		                               String panelID, String nextPanelID, String prevPanelID,
		                               String OnEnter, String OnExit,
		                               Writer out, PageContext PC, String tabIcon, String parentPane, boolean disableTab)
	throws IOException
     {
		//Check if encoding should occur
		boolean isFaces = StringUtil.checkEncoding();
		//Encode variable
		String encodedControlVar =  StringUtil.encodeString(CONTROL_VAR + _varIndex,isFaces);
        out.write( encodedControlVar+".addCaseFromDIV("+ID+",\""+panelDisplayName
       													 +"\",\""+panelID+"\""
                                                         +(nextPanelID == null ? ",null" : ",\"" + nextPanelID + "\"")
                                                         +(prevPanelID == null ? ",null" : ",\"" + prevPanelID + "\"")
                                                         +(OnEnter     == null ? ",null" : ",\"" + OnEnter + "\"")
                                                         +(OnExit      == null ? ",null" : ",\"" + OnExit + "\"")
														 +(tabIcon == null?",null" : ",\"" + tabIcon + "\"")
														 +(parentPane == null?",null" : ",\""+ parentPane + "\"")
														 +(disableTab == false?",false" : ",true")
				                                      +");\n");
	 }
}