/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//  
//OCO Source Materials
//  
//5724-E76 and 5724-E77
//  
// Copyright IBM Corp.  2004
//  
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////
package com.ibm.odcb.jrender.emitters;

import java.io.IOException;
import java.io.Writer;
import java.util.Locale;

import com.ibm.faces.renderkit.html_extended.HxClientRenderUtil;
import com.ibm.odcb.jrender.mediators.ExportException;
import com.ibm.odcb.jrender.mediators.InvalidParameterException;
import com.ibm.odcb.jrender.mediators.PageContext;
import com.ibm.odcb.jrender.misc.Streamer;

/**
 * @author fenil
 */
public class TimePickerEmitter extends BaseEmitter {

	private Locale locale = Locale.US;
	private String pattern, timeStyle, inputTextId;
	private int hourIncrement, minuteIncrement, secondIncrement;

	public TimePickerEmitter() {
		super("ODCTimePicker");
	}

	public void Init(String inputTextId) throws InvalidParameterException {
		this.Init(inputTextId, 0, 0, 0);
	}

	public void Init(String inputTextId, int hourIncrement, int minuteIncrement, int secondIncrement) throws InvalidParameterException{
		this.Init(inputTextId, null, null, ClientConverterHelper.DATESTYLE_SHORT, hourIncrement, minuteIncrement, secondIncrement);
	}

	public void Init(
		String inputTextId,
		Locale locale,
		String pattern,
		String timeStyle,
		int hourIncrement,
		int minuteIncrement,
		int secondIncrement)
		throws InvalidParameterException {
		Streamer.trace.Header().println("Entering TimePickerEmitter.Init()");
		if (inputTextId != null && !inputTextId.equals("")) {
			this.inputTextId = inputTextId;
		} else {
			Streamer.error.Header().print("The id for inputtext field is either null or empty.");
			//throw an exception
			throw new InvalidParameterException("The id for inputtext field is either null or empty.");
		}
		if (locale != null) {
			this.locale = locale;
		}
		this.pattern = pattern;
		this.timeStyle = timeStyle;
		this.hourIncrement = hourIncrement;
		this.minuteIncrement = minuteIncrement;
		this.secondIncrement = secondIncrement;
		Streamer.trace.Header().println("Exiting TimePickerEmitter.Init()");
	}

	/* (non-Javadoc)
	 * @see com.ibm.odcb.jrender.emitters.BaseEmitter#Export(java.io.Writer, com.ibm.odcb.jrender.mediators.PageContext)
	 */
	public void Export(Writer out, PageContext PC) throws ExportException, IOException {
		Streamer.trace.Header().println("Entering TimePickerEmitter.Export()");
		out.write("<script type=\"text/javascript\">\n");
		Writer writer = out;
		StringBuffer jsBuffer = new StringBuffer();
		String converterId = inputTextId;
		String converterScript =
			ClientConverterHelper.getJsDateConverterString(
				pattern,
				ClientConverterHelper.DATETYPE_TIME,
				null,
				timeStyle,
				locale);
		jsBuffer.append(HxClientRenderUtil.HX_VAR_NAME + ".addConverter(\"" + converterId + "\", " + converterScript + ");");
		if (null != converterId) {
			writer.write("\n");
			writer.write(jsBuffer.toString());
			jsBuffer.setLength(0); // reset buffer 
		}
		writer.write(HxClientRenderUtil.HX_VAR_NAME + ".addComponent(\"" + this.inputTextId + "\", new " + HxClientRenderUtil.HX_VAR_NAME + ".JSFSpinner(\"type:datetime\""); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		if (null != converterId)
			writer.write(", \"converter:" + converterId + "\""); //$NON-NLS-1$ //$NON-NLS-2$
		//if all increments are null provide a default increment of 1 for all
		if ((this.hourIncrement == 0) && (this.minuteIncrement == 0) && (this.secondIncrement == 0)) {
			writer.write(", \"increment:3600\", \"increment1:60\", \"increment2:1\""); //$NON-NLS-1$
		} else {
			if (this.hourIncrement != 0) {
				if (this.hourIncrement > 0) {
					writer.write(", \"increment:" + this.hourIncrement * 3600 + "\""); //$NON-NLS-1$ //$NON-NLS-2$
				} else {
					writer.write(", \"increment:" + Math.abs(this.hourIncrement) * 3600 + "\""); //$NON-NLS-1$ //$NON-NLS-2$
				}
			}
			if (this.minuteIncrement != 0) {
				if (this.minuteIncrement > 0) {
					writer.write(", \"increment1:" + this.minuteIncrement * 60 + "\""); //$NON-NLS-1$ //$NON-NLS-2$
				} else {
					writer.write(", \"increment1:" + Math.abs(this.minuteIncrement) * 60 + "\""); //$NON-NLS-1$ //$NON-NLS-2$
				}
			}
			if (this.secondIncrement != 0) {
				if (this.secondIncrement > 0) {
					writer.write(", \"increment2:" + this.secondIncrement + "\""); //$NON-NLS-1$ //$NON-NLS-2$
				} else {
					writer.write(", \"increment2:" + Math.abs(this.secondIncrement) + "\""); //$NON-NLS-1$ //$NON-NLS-2$
				}
			}
		}
		writer.write("));\n"); //$NON-NLS-1$
		writer.write("</script>\n");
		Streamer.trace.Header().println("Exiting TimePickerEmitter.Export()");
	}

	/**
	 * @return
	 */
	public String getInputTextId() {
		return inputTextId;
	}

	/**
	 * @return
	 */
	public Locale getLocale() {
		return locale;
	}

	/**
	 * @return
	 */
	public String getPattern() {
		return pattern;
	}

	/**
	 * @return
	 */
	public String getTimeStyle() {
		return timeStyle;
	}

	/**
	 * @param string
	 */
	public void setInputTextId(String string) {
		inputTextId = string;
	}

	/**
	 * @param locale
	 */
	public void setLocale(Locale locale) {
		this.locale = locale;
	}

	/**
	 * @param string
	 */
	public void setPattern(String string) {
		pattern = string;
	}

	/**
	 * @param string
	 */
	public void setTimeStyle(String string) {
		timeStyle = string;
	}

}
