/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//
//OCO Source Materials
//
//5724-E76 and 5724-E77
//
// Copyright IBM Corp.  2003
//
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////
package com.ibm.odcb.jrender.emitters;
import java.io.IOException;
import java.io.Writer;
import java.util.ArrayList;

import com.ibm.odcb.jrender.mediators.ExportException;
import com.ibm.odcb.jrender.mediators.PageContext;
import com.ibm.odcb.jrender.misc.StringUtil;

/**
 * @author Breffni Coffey <br>
 * <code>TreeTableEmitter</code> is a helper class for the <code>TreeViewEmitter</code>.
 * This class supplies the metadata to the <code>TreeViewEmitter</code> to construct a tree table.
 * It also acts as a parent emitter for the <code>ColAttrEmitterHelper</code> and as a result emits
 * column style attributes onto the page
 */

public class TreeTableEmitter extends BaseEmitter 
{
	// class instance variables
	protected boolean _isFaces;
	protected String _treeColMap;
	private String id;
	protected ArrayList _colList = new ArrayList();
	
	public TreeTableEmitter() {
		super("ODCTree");
		_isFaces=StringUtil.checkEncoding();
	}

	/**
	 * Constructor is initialized with parameters.
	 * @param modelName 	the name of the model, whose eobject this tree is going to be connected to
	 * @param eobjectID		The ID of the eobject to be used as the root of the tree.
	 */
	public TreeTableEmitter(String modelName, String eObjectID) 
	{
		super("ODCTree");
		this.Init(modelName, eObjectID);
	}
	
	public void Init(String modelName, String eObjectID) 
	{
		_isFaces=StringUtil.checkEncoding();
		_treeColMap="";
	}
	
	/**
	 * Used to set the var index for accessing the treeControl on the page
	 * @param vIndex index passed in from parent emitter to be used in conjunction with CONTROL_VAR
	 */
	public void setVarIndex(int vIndex)
	{
		_varIndex=vIndex;
	}
	
	/**
	 * Adds a column to the treeTable.
	 * @param v the node to be added.
	 */
	public void addColumn(ColAttrEmitterHelper v) 
	{
		//System.out.println(v.id);
		_colList.add(v);
	}

	/**
	 * Constructs map of tree column style attributes passed in via the treeColumn tag.
	 */
	protected void constructTreeColumnMap() 
	{
		StringBuffer colString = new StringBuffer(128);
		ColAttrEmitterHelper treeCol;
		for (int i = 0; i <  _colList.size(); ++i) 
		{
			treeCol = (ColAttrEmitterHelper)  _colList.get(i);
			String attrName=treeCol.getLabel();
			String width =treeCol.getWidth();
			colString.append(attrName+","+width+";");
		}

		_treeColMap = colString.length() == 0 ? null : colString.toString();
	}
	
	/** 
	 * Export method does all the emitting of JavaScript code on to a page.
	 * In the case of a tree table it only emits treeTableColumn data
	 * @param out 	Writer used to write the JavaScript to
	 * @param PC	PageContext that supplies the instance count to uniquely instantiate tree variables.
	 */
	public void Export(Writer out, PageContext PC) throws ExportException, IOException 
	{
		//Encoding variables for portal
		String controlVar = StringUtil.encodeString(CONTROL_VAR + _varIndex,_isFaces);
		
		// Call the constructTreeColumns() method which in turn cycles through all the tree columns
		// and gets their individual attributes and concatenates them.
		this.constructTreeColumnMap();

		//Write to page
		if (_treeColMap != null) {
			out.write(controlVar + ".setupColumnAttributes(\"" + _treeColMap + "\");\n");
		}
		
	}
	/**
	 * @return Returns the id.
	 */
	public String getId() {
		return id;
	}
	/**
	 * @param id The id to set.
	 */
	public void setId(String id) {
		this.id = id;
	}
}
