/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//  
//OCO Source Materials
//  
//5724-E76 and 5724-E77
//  
// Copyright IBM Corp.  2003
//  
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////

package com.ibm.odcb.jrender.emitters;

import java.util.ArrayList;
import java.io.Writer;
import java.io.IOException;
import com.ibm.faces.bf.renderkit.JSUtil;
import com.ibm.faces.renderkit.html_extended.HxClientRenderUtil;
import com.ibm.faces.renderkit.html_extended.Utils;
import com.ibm.odcb.jrender.mediators.ExportException;
import com.ibm.odcb.jrender.mediators.PageContext;
import com.ibm.odcb.jrender.misc.Config;
import com.ibm.odcb.jrender.misc.Streamer;
import com.ibm.odcb.jrender.misc.URLRewriter;
import com.ibm.odcb.jrender.misc.StringUtil;

//import com.ibm.odcb.jrender.misc.Config;

/**
 * @author Kaushal <br>
 * TreeView Emitter is a Java interface for generating an Odyssey
 * tree control in a web page.
 */
public class TreeViewEmitter extends BaseEmitter {
	public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;

	private static final String MINUSTOP = Config.versionizeFilename("minustop.gif");
	private static final String MINUS = Config.versionizeFilename("minus.gif");
	private static final String MINUSBOTTOM = Config.versionizeFilename("minusbottom.gif");
	private static final String PLUSTOP = Config.versionizeFilename("plustop.gif");
	private static final String PLUS = Config.versionizeFilename("plus.gif");
	private static final String PLUSBOTTOM = Config.versionizeFilename("plusbottom.gif");
	private static final String JOINTOP = Config.versionizeFilename("jointop.gif");
	private static final String JOIN = Config.versionizeFilename("join.gif");
	private static final String JOINBOTTOM = Config.versionizeFilename("joinbottom.gif");
	private static final String OPEN_FOLDER = Config.versionizeFilename("open_folder.gif");
	private static final String FOLDER = Config.versionizeFilename("folder.gif");
	private static final String PAGE = Config.versionizeFilename("page.gif");
	private static final String ROOT = Config.versionizeFilename("root.gif");
	private static final String EMPTY = Config.versionizeFilename("empty.gif");
	private static final String LINE = Config.versionizeFilename("line.gif");
	private static final String PLAINMINUS = Config.versionizeFilename("plainminus.gif");
	private static final String PLAINPLUS = Config.versionizeFilename("plainplus.gif");
	
	private static final String TREECONTROL = Config.versionizeFilename("treecontrol.js");
	private static final String TREEADAPTER = Config.versionizeFilename("treeadapter.js");
	private static final String BEHAVIORS = Config.versionizeFilename("hxclient_be.js");
	private static final String JSL_TREE_C = Config.versionizeFilename("jsl_tree_c.js");
	
	private static final String ECLASS = Config.versionizeFilename("eclass.js");
	private static final String ECREATOR = Config.versionizeFilename("ecreator.js");
	private static final String EOBJECT = Config.versionizeFilename("eobject.js");
	private static final String XMIHANDLER = Config.versionizeFilename("xmiHandler.js");
	private static final String BINDER = Config.versionizeFilename("binder.js");
	private static final String XML2EMF = Config.versionizeFilename("xml2emf.js");
	private static final String JSL_WDO4JS_C = Config.versionizeFilename("jsl_wdo4js_c.js");
	
	
	/**
	 * Minimal Constructor
	 *
	 */
	public TreeViewEmitter() {
		super("ODCTree");
		_isFaces=StringUtil.checkEncoding();
	}

	/**
	 * Constructor is initialized with parameters.
	 * @param modelName 	the name of the model, whose eobject this tree is going to be connected to
	 * @param eobjectID		The ID of the eobject to be used as the root of the tree.
	 */
	public TreeViewEmitter(String modelName, String eObjectID) {
		super("ODCTree");
		this.Init(modelName, eObjectID);
	}

	/**
	 * Initializer. Same as the constructor.
	 * @param modelName
	 * @param eObjectID
	 */
	public void Init(String modelName, String eObjectID) {
		_isFaces=StringUtil.checkEncoding();
		_modelName = modelName;
		_eObjectID = eObjectID;
	}

	// class instance variables
	protected boolean _isFaces;
	protected boolean _isPageData;
	protected String _modelName;
	protected String _eObjectID;
	protected String _styleMap;
	protected String _sysIconStyle;
	protected String _rootVisibleFlagString = "true"; // default is true
	protected String _workPlaceFlagString = "false"; // default is false
	protected String _dynamicFlagString = "false"; // default is false
	protected String _enableSelectString = "false";
	protected String _userHighlightString = "false"; // default is false
	protected String _treeWidth = null;
	protected String _treeHeight = null;
	protected String _treeTableCols = null; //default is false
	protected String _treeTableColWidths = null; //default is false
	protected boolean _percentageExists = false;
	protected String _styleClass;
	//global tree events
	protected String _onnodehighlight, _onnodeselect, _onnodeunselect, _onnodedrop, _onnodedragenterover, _onnodeexpand, _onnodecollapse;
	protected ArrayList _treeTable= new ArrayList(1);
	protected ArrayList _nodeList = new ArrayList(7);
	protected ArrayList _iconMapArr = new ArrayList();
	protected ArrayList _handlerMapArr = new ArrayList();
	protected ArrayList _behaviorMapArr = new ArrayList();
	protected ArrayList _colDataMapArr = new ArrayList();
	
	//
	protected String _dir = null;
	protected String _lang = null;
	
	public void setIsPageData(boolean b)
	{
		_isPageData = b;
	}
	
	
	/**
	 * Sets the system icon style string for the tree
	 * @param v
	 */
	public void setSystemIconStyle(String v) {
		_sysIconStyle = v;
	}

	/**
	 * Sets the id for eobject to be used for the root of the tree.
	 * @param v
	 */
	public void setRootId(String v) {
		_eObjectID = v;
	}

	/**
	 * Adds a node to the tree.
	 * @param v the node to be added.
	 */
	public void addNode(NodeAttributeEmitterHelper v) {
		_nodeList.add(v);
	}
	
	/**
	 * Adds a treeTable to the tree.
	 * @param v the treeTable to be added.
	 */
	public void addTable(TreeTableEmitter v) 
	{
		_treeTable.add(v);
	}

	/**
	 * The tree renderer needs access to this when using page data.
	 * @return
	 */
	public ArrayList getNodes()
	{
		return this._nodeList;
	}
	
	
	/**
	 * Sets the width of the tree. If specifying the width in pixels, use just a number. If specifying in percentage use a string
	 * like &quot;number%&quot;
	 * @param width
	 */
	public void setTreeWidth(String width) {
		char cw = width.charAt(width.length() - 1);
		String lWidth;
		if (cw == '%') {
			lWidth = width.substring(0, width.length() - 1); // stripping out the % sign at the end
			_percentageExists = true;
		} else {
			lWidth = width;
		}
		Integer widthInt = Integer.valueOf(lWidth);
		// Only assign value if width specified was greater than zero.
		if (widthInt.intValue() > 0)
			_treeWidth = width;
	}

	/**
	 * Sets the height of the tree. If specifying the height in pixels, use just a number. If specifying in percentage use a string
	 * like &quot;number%&quot;
	 * @param width
	 */
	public void setTreeHeight(String height) {
		char ch = height.charAt(height.length() - 1);
		String lHeight;
		if (ch == '%') {
			lHeight = height.substring(0, height.length() - 1); // stripping out the % sign at the end
			_percentageExists = true;
		} else {
			lHeight = height;
		}
		Integer heightInt = Integer.valueOf(lHeight);
		// Only assign value if height specified was greater than zero.
		if (heightInt.intValue() > 0)
			_treeHeight = height;
	}

	/**
	 * Specifies whether the tree should be shown with tabular data
	 * @param treeTableCols


	 */
	public void setTreeTableCols(String treeTableCols) {
		
		if (treeTableCols != null)
			_treeTableCols = treeTableCols; // else, default value, i.e., true


	}
	
	/**
	 * Specifies widths for columns if the tree is to be shown with tabular data
	 * @param treeTableColWidths
	 */
	public void setTreeTableColWidths(String treeTableColWidths) {
		if (treeTableColWidths != null)
			_treeTableColWidths = treeTableColWidths; // else, default value, i.e., true
	}
	
	/**
	 * Specifies whether the root of the tree should be shown.
	 * @param rootVisibleFlag
	 */
	public void setRootVisibleFlag(boolean rootVisibleFlag) {
		if (rootVisibleFlag == false)
			_rootVisibleFlagString = "false"; // else, default value, i.e., true
	}
	
	/**
	 * Specifies whether the tree should display in the WorkPlace style
	 * @param rootVisibleFlag
	 */
	public void setWorkPlaceFlag(boolean workPlaceFlag) {
		if (workPlaceFlag == true)
			_workPlaceFlagString = "true";
		// else, default value, i.e., true
	}

	/**
	 * Specifies whether tree requires nodes to be added dynamically
	 * @param dynamicFlag
	 */
	public void setDynamicFlag(boolean dynamicFlag) {
		if (dynamicFlag == true)
			_dynamicFlagString = "true";
		// else, default value, i.e., true
	}
	
	/**
	 * Specifies whether the user controls highlighting of the tree nodes for drag and drop
	 * @param userHighlight
	 */
	public void setUserHighlight(boolean userHighlight ) {
		if (userHighlight == true)
			_userHighlightString = "true";
		// else, default value, i.e., true
	}
	
	/**
	 * Specifies whether the nodes in the tree should be selectable.
	 * @param enableSelect
	 */
	public void setEnableSelect(boolean enableSelect) {
		if (enableSelect == true)
			_enableSelectString = "true"; // else, default value, i.e., false
	}

	/**
	 * Sets the event handler for the global &quot;onselect&quot; event on the tree.
	 * This event is fired when any node in the tree is selected.
	 * @param onnodeselect
	 */
	public void setOnnodeexpand(String onnodeexpand) 
	{
		_onnodeexpand=onnodeexpand;
	}
	
	/**
	 * Sets the event handler for the global &quot;onselect&quot; event on the tree.
	 * This event is fired when any node in the tree is selected.
	 * @param onnodeselect
	 */
	public void setOnnodecollapse(String onnodecollapse) 
	{
		_onnodecollapse=onnodecollapse;
	}
	
	/**
	 * Sets the event handler for the global &quot;onhighlight&quot; event on the tree.
	 * This event is fired when any node in the tree is highlighted.
	 * @param onnodehighlight
	 */
	public void setOnnodehighlight(String onnodehighlight) 
	{
		_onnodehighlight = onnodehighlight;
	}

	/**
	 * Sets the event handler for the global &quot;onselect&quot; event on the tree.
	 * This event is fired when any node in the tree is selected.
	 * @param onnodeselect
	 */
	public void setOnnodeselect(String onnodeselect) 
	{
		_onnodeselect=onnodeselect;
	}

	/**
	 * Sets the event handler for the global &quot;onunselect&quot; event on the tree.
	 * This event is fired when any node in the tree is deselected.
	 * @param onnodeunselect
	 */
	public void setOnnodeunselect(String onnodeunselect) 
	{
		_onnodeunselect=onnodeunselect;
	}
	
	/**
	 * Sets the event handler for the global &quot;onnodedrop&quot; event on the tree.
	 * This event is fired an item is dropped to any node in the tree.
	 * @param onnodedrop
	 */
	public void setOnnodedrop(String onnodedrop) 
	{
		_onnodedrop=onnodedrop;
	}

	/**
	 * Sets the event handler for the global &quot;onnodedragenterover&quot; event on the tree.
	 * This event is fired an item is dropped to any node in the tree.
	 * @param onnodedragenterover
	 */
	public void setOnnodedragenterover(String onnodedragenterover) 
	{
		_onnodedragenterover=onnodedragenterover;
	}
	
	/**
	 * Sets the CSS style class for the tree.
	 * @param styleClass
	 */
	public void setStyleClass(String styleClass) 
	{
		_styleClass = styleClass;
	}

	/**
	 * Sets the dir for the tree.
	 * @param dir
	 */
	public void setDir(String dir) 
	{
		_dir = dir;
	}
	
	/**
	 * Sets the lang for the tree.
	 * @param lang
	 */
	public void setLang(String lang) 
	{
		_lang = lang;
	}
	
	/**
	 * Uses the nodeAttributeEmitterHelper to gather information for individual nodes to be emitted
	 * in the Export method of the treeViewEmitter 
	 */
	protected void constructStyleMapIconMapHandlerMap() {
		StringBuffer S1 = new StringBuffer(128);
		NodeAttributeEmitterHelper t;
		//Encoding variable
		String encodedStyle="";
		
		for (int i = 0; i < _nodeList.size(); ++i) 
		{
			t = (NodeAttributeEmitterHelper) _nodeList.get(i);
			
			//encode string and append it to style list.
			// the encoding for client data requires the style map to be encoded
			// we do not require this for page data binding.
			if (this._isPageData)
			{	
				encodedStyle = t.getStyleMap();
			}
			else {
				encodedStyle=StringUtil.encodeString(t.getStyleMap(),_isFaces);
			}
				
			S1.append(encodedStyle);
			
			ArrayList nodeIconMapArray = t.getIconMap();
			if (nodeIconMapArray.size() != 0)
				_iconMapArr.addAll(nodeIconMapArray);

			ArrayList nodeHandlerMapArray = t.getHandlerMap();
			if (nodeHandlerMapArray.size() != 0)
				_handlerMapArr.addAll(nodeHandlerMapArray);
			
		}
		_styleMap = S1.length() == 0 ? null : S1.toString();
		Streamer.debug.Header().println("MapStyle: " + this._styleMap);
	}

	
	public void exportSpan(Writer out, PageContext PC) throws ExportException, IOException {

		_varIndex = PC.NextInstanceCount();
		
		String controlVar = StringUtil.encodeString(CONTROL_VAR + _varIndex,_isFaces);
		
		String dirlang = "";
		
		if(_dir != null && _dir.trim().length()>0)
			dirlang =" dir=\""+_dir+"\"";
		if(_lang != null && _lang.trim().length()>0)
			dirlang +=" lang=\""+_lang+"\"";

		
		if(dirlang != null && dirlang.trim().length() >0)
			out.write("<span id=\""	+ controlVar+ "\""+ dirlang +"></span>\n");			
		else
			out.write("<span id=\""	+ controlVar+ "\"></span>\n");
	}
	
	
	
	/** 
	 * Export method does all the emitting of JavaScript code on to a page.
	 * In the case of a tree control, first a tree control is created and bound to an
	 * HTML element (parentElem). Next a tree adapter is created, initializing with
	 * the tree control. Finally the tree node styles are set and the tree is rendered.
	 * <p>
	 * @param out 	Writer used to write the JavaScript to
	 * @param PC	PageContext that supplies the instance count to uniquely instantiate tree variables.
	 */
	public void Export(Writer out, PageContext PC) throws ExportException, IOException {
		Streamer.trace.Header().println("Entering TreeViewEmitter export...");
	//	out.write("<script type=\"text/javascript\">ODCProgressBar.startRenderControl(treeview);</script>\n");
		out.write("\nODCProgressBar.startRenderControl(treeview);</script>\n");
	//	_varIndex = PC.NextInstanceCount();
		URLRewriter R = (URLRewriter) PC.getContextVariable(InitializationEmitter._ID_URL_REWRITER);
	
			
		boolean debugMode =
			PC.getContextVariable(InitializationEmitter._ID_DEBUG_MODE) == InitializationEmitter.DEBUG_YES
				? true
				: false;

		if (TestGuard(PC, "TREE_JS_FLAG") == false) {
			if (debugMode == true) {
				out.write(
						"\n<script type=\"text/javascript\" src=\"" +
						R.Rewrite(Utils.versionizeFilename("hxclient_base64.js")) + "\"></script>" +
						"\n<script type=\"text/javascript\" src=\""
							+ R.Rewrite("jsl/tree/" + TREECONTROL)
							+ "\"></script>"
							+ "\n<script type=\"text/javascript\" src=\""
							+ R.Rewrite("jsl/tree/" + TREEADAPTER)
							+ "\"></script>"
							+ "\n<script type=\"text/javascript\" src=\""
							+ R.Rewrite(BEHAVIORS )
							+ "\"></script>");
				if (this._isPageData)
				{
					out.write("<script type=\"text/javascript\" src=\"" + R.Rewrite("jsl/" + XML2EMF) + "\"></script>\n");
					out.write("\n<script type=\"text/javascript\" src=\"" + R.Rewrite("jsl/" +  ECLASS) + "\"></script>\n<script type=\"text/javascript\" src=\""
			                + R.Rewrite("jsl/" + ECREATOR) + "\"></script>\n<script type=\"text/javascript\" src=\"" + R.Rewrite("jsl/" + EOBJECT)
			                + "\"></script>\n<script type=\"text/javascript\" src=\"" + R.Rewrite("jsl/" + XMIHANDLER)
			                + "\"></script>\n<script type=\"text/javascript\" src=\"" + R.Rewrite("jsl/" + BINDER) + "\"></script>\n");
				}
			} else // we don't need to test again for LIB_FLAG here;
			{
				out.write("<script type=\"text/javascript\" src=\"" + R.Rewrite(Utils.versionizeFilename("hxclient_base64.js")) + "\"></script>\n");
				out.write("<script type=\"text/javascript\" src=\"" + R.Rewrite("jsl/" + JSL_TREE_C) + "\"></script>\n");
				out.write("<script type=\"text/javascript\" src=\"" + R.Rewrite(BEHAVIORS) + "\"></script>\n");
				if (this._isPageData)
				{
					out.write("<script type=\"text/javascript\" src=\"" + R.Rewrite("jsl/" + JSL_WDO4JS_C) + "\"></script>");
					out.write("\n");
				}
			}
		}
	
		
		//String _StyleSheetDir = Config.getString(Config.PROP_CLIENT_STYLESHEETDIR);
		//out.write("<link rel=\"stylesheet\" type=\"text/css\" href=\"" + R.Rewrite(_StyleSheetDir + "/tree.css") + "\">\n");

		// Call the constructStyleStrings() method which in turn cycles through all the tree nodes
		// and gets their individual style-map and node-icon map strings and concatenates them.
		this.constructStyleMapIconMapHandlerMap();
		
		//Encoding variables for portal
		String controlVar = StringUtil.encodeString(CONTROL_VAR + _varIndex,_isFaces);
		String adaptorVar = StringUtil.encodeString(ADAPTER_VAR + _varIndex,_isFaces);
	
		String nameVar=StringUtil.encodeString(_modelName,_isFaces);
	
		String modelVar;
	
		String eObjectVar;
		

/*		out.write(
				"<span id=\""
				+ controlVar
				+ "\"></span>\n"
				+ "<script type=\"text/javascript\">\n"); */
		
		
		out.write("<script type=\"text/javascript\">\n");
		
		if (this._isPageData) 
		{
			eObjectVar = _eObjectID;
			
			modelVar = nameVar+"xml2emf";
			
			out.write("var "
					+ modelVar
					+ "= new XML2EMF("
					+ nameVar	
					+ ");\n"				
						);
		}
		else {
			
			modelVar=StringUtil.encodeString("WDO4JSModelRoot_"+nameVar+".Root",_isFaces);
			eObjectVar=StringUtil.encodeString(_eObjectID,_isFaces);
						
		}
				
		out.write("var "
				+ controlVar
				+ " = new Tree(window.document.getElementById(\""
				+ controlVar
				+ "\"), URL_REWRITER_PREFIX);\n"
				+ "var "
				+ adaptorVar
				+ " = new TreeAdapter("
				+ controlVar
				+ ", "
				+ modelVar
				+ ");\n"

		// Below we set the exact ID of the root object in the directory tree
		+ adaptorVar + ".setRootByID(\"" + eObjectVar + "\");\n");

		if (_styleMap != null) {
			out.write(adaptorVar + ".setStyleMap(\"" + _styleMap + "\");\n");
		}

		
		//Breff: bug RATLC00977868 
		//Custom icons were not appearing in portal - a call to URLRewriter is required to obtain correct URL 
		/*if (_iconMapArr.size() != 0) {
			for (int i = 0; i < _iconMapArr.size(); i++) {
				String tempStr = (String) _iconMapArr.get(i);
				if (tempStr != null) {
					out.write(ADAPTER_VAR + _varIndex + ".addNodeIconMap(\"" + tempStr + "\");\n");
				}
			}
		}*/
		//Set encoded default icons for the tree
		out.write(
					controlVar + ".treeIcons = {"
					+ "\n minustop: \""+R.Rewrite("jsl/tree/icons/" + MINUSTOP)+"\","
					+ "\n minus:\""+R.Rewrite("jsl/tree/icons/" + MINUS)+"\","
					+ "\n minusbottom:\""+R.Rewrite("jsl/tree/icons/" + MINUSBOTTOM)+"\","
					+ "\n plustop:\""+R.Rewrite("jsl/tree/icons/" + PLUSTOP)+"\","
					+ "\n plus:\""+R.Rewrite("jsl/tree/icons/" + PLUS)+"\","
					+ "\n plusbottom:\""+R.Rewrite("jsl/tree/icons/" + PLUSBOTTOM)+"\","
					+ "\n jointop:\""+R.Rewrite("jsl/tree/icons/" + JOINTOP)+"\","
					+ "\n joint:\""+R.Rewrite("jsl/tree/icons/" + JOIN)+"\","
					+ "\n jointbottom:\""+R.Rewrite("jsl/tree/icons/" + JOINBOTTOM)+"\","
					+ "\n nodeopen:\""+R.Rewrite("jsl/tree/icons/" + OPEN_FOLDER)+"\","
					+ "\n nodeclose:\""+R.Rewrite("jsl/tree/icons/" + FOLDER)+"\","
					+ "\n leaf:\""+R.Rewrite("jsl/tree/icons/" + PAGE)+"\","
					+ "\n baseopen:\""+R.Rewrite("jsl/tree/icons/" + ROOT)+"\","
					+ "\n empty:\""+R.Rewrite("jsl/tree/icons/" + EMPTY)+"\","
					+ "\n line:\""+R.Rewrite("jsl/tree/icons/" + LINE)+"\","
					+ "\n plainminus:\""+R.Rewrite("jsl/tree/icons/" + PLAINMINUS)+"\","
					+ "\n plainplus:\""+R.Rewrite("jsl/tree/icons/" + PLAINPLUS)+"\"}; \n");
		
		if (_iconMapArr.size() != 0) 
		{
			for (int i = 0; i < _iconMapArr.size(); i++) 
			{
			
					String tempStr = (String) _iconMapArr.get(i);
					//count number of commas in tempstr
					int commaCount=0;
					for(int count=0;count<tempStr.length();count++)
					{
						if(tempStr.charAt(count)==',')
							commaCount++;
					}
					
					//increment commaCOunt by 1 to give total elements in tempStr
					int numElems=commaCount+1;
					
					if (tempStr != null) 
					{
						//parse line to seperate first entry specifying the node and file names that immediatly follow
						String[] parsedItem = StringUtil.splitToArray(tempStr,',');
						if(parsedItem[0]!=null)
							parsedItem[0]=StringUtil.encodeString(parsedItem[0],_isFaces);
						///if there is a custom open icon (2nd element in array)
						if(parsedItem[1]!=null)
						{
							if(parsedItem[1].indexOf("ibmjsfres")>-1)
							{
								//trim off ibmjsfres its gonna be added on by r.rewrite()
								parsedItem[1]=parsedItem[1].substring(parsedItem[1].indexOf('/')+1,parsedItem[1].length());
								//now call Rewrite so that for portal it returns /.projname/.ibmjsfres/ etc.
								parsedItem[1]=R.Rewrite(parsedItem[1]);
							}
						}
						//if there is a custom close icon (3rd icon in array)
						if(parsedItem[2]!=null)
						{
							if(parsedItem[2].indexOf("ibmjsfres")>-1)
							{
								//trim off ibmjsfres its gonna be added on by r.rewrite()
								parsedItem[2]=parsedItem[2].substring(parsedItem[2].indexOf('/')+1,parsedItem[2].length());
								//now call Rewrite so that for portal it returns /.projname/.ibmjsfres/ etc.
								parsedItem[2]=R.Rewrite(parsedItem[2]);
							}
						}	
						//reconstruct tempstr with new variables
						for(int j=0;j<parsedItem.length;j++)
						{
							if(j==0)
								tempStr=parsedItem[j];
							else
								tempStr=tempStr+","+parsedItem[j];
						}
						
						//if there are trailing commas at the end we have to reappend them
						if(numElems>parsedItem.length)
						{
							//tempStr may be missing some info so it is shown as a series of empty commas
							int noInfo=numElems-parsedItem.length;
							//for each item missing in tempStr a comma is added
							for(int k=0;k<noInfo;k++)
								tempStr=tempStr+",";
						}		
						out.write(adaptorVar + ".addNodeIconMap(\"" + tempStr + "\");\n");
					}
			}
			
		}
		//End breff
		if (_handlerMapArr.size() != 0) {
			for (int i = 0; i < _handlerMapArr.size(); i++) 
			{
				String tempStr = (String) _handlerMapArr.get(i);
				if (tempStr != null) 
				{
					if (!this._isPageData) {
						tempStr=StringUtil.encodeString(tempStr,_isFaces);
					}	
					out.write(adaptorVar + ".addNodeEventHandler(\"" + tempStr + "\");\n");
				}
			}
		}

		//Call treeTable emitter to set up and emit treeTable columns
		if (_treeTable.size() != 0) 
		{
			for (int i = 0; i < _treeTable.size(); i++) 
			{
				TreeTableEmitter TTEmitter = (TreeTableEmitter)_treeTable.get(i);
				TTEmitter.setVarIndex(_varIndex);
				String tempStr=TTEmitter.getId();
				
				if (tempStr != null) 
				{
					//We want tempStr to be some sort of id
					tempStr=StringUtil.encodeString(tempStr,_isFaces);
					out.write(controlVar + ".addTreeTable(\"" + tempStr + "\");\n");
					
					//call treeTableEmitter to set up cols
					TTEmitter.Export(out,PC);
				}
			}			
		}
		
		//For each column call treeNodeAttr emitter to set up and emit treeTable column data
		if(_nodeList.size()!=0)
		{
			for (int j = 0; j < _nodeList.size(); j ++)
			{
				NodeAttributeEmitterHelper NAEmitter = (NodeAttributeEmitterHelper)_nodeList.get(j);
				NAEmitter.setVarIndex(_varIndex);
				NAEmitter.Export(out, PC);


			}

		}
		
		if (_sysIconStyle != null) {
			out.write(adaptorVar + ".setSystemIconStyle(\"" + _sysIconStyle + "\");\n");
		}
		if (_treeWidth != null) {
			out.write(controlVar + ".setWidth(\"" + _treeWidth + "\");\n");
		}
		if (_treeHeight != null) {
			out.write(controlVar + ".setHeight(\"" + _treeHeight + "\");\n");
		}
		if (_treeTableCols != null) {
			if(_treeTableColWidths!=null)
			{
				out.write(controlVar + ".setTreeTableCols(\"" + _treeTableCols +"\",\""+_treeTableColWidths+"\");\n");
			}
			else
				out.write(controlVar + ".setTreeTableCols(\"" + _treeTableCols +"\");\n");
		}
		if (_styleClass != null)
			out.write(controlVar + ".setStyleClass(\"" + _styleClass + "\");" + "\n");

		/*if (_percentageExists) {
			out.write(
				"function "
					+ controlVar
					+ "resize() { "
					+ controlVar
					+ ".reSize(); }\n"
					+ "\n"
					+ "if (window.addEventListener)\n"
					+ " window.addEventListener('resize',"
					+ controlVar
					+ "resize,false);\n"
					+ "else\n"
					+ " window.attachEvent('onresize',"
					+ controlVar
					+ "resize);\n");
		}*/

		out.write(
				controlVar
				+ ".rootVisibleFlag = "
				+ _rootVisibleFlagString
				+ ";\n"
				+ controlVar
				+ ".enableSelect = "
				+ _enableSelectString
				+ ";\n");
		
		//if workPlaceFlag is true then emmit it
		if(_workPlaceFlagString=="true")
		{
		out.write(
				controlVar
					+ ".workPlaceFlag = "
					+ _workPlaceFlagString
					+ ";\n");
		}
		
//		if dynamicFlag is true then emmit it
		if(_dynamicFlagString=="true")
		{
		out.write(
				controlVar
					+ ".dynamicFlag = "
					+ _dynamicFlagString
					+ ";\n");
		}
		
		//if userHighlight flag is true then emmit it
		if(_userHighlightString=="true")
		{
		out.write(
				controlVar
					+ ".userHighlight = "
					+ _userHighlightString
					+ ";\n");
		}
		
		//add global tree behaviors
		if(_behaviorList!=null)
		{
			for(int ind=0;ind<_behaviorList.size();ind++)
			{
				BehaviorEmitter behavior =(BehaviorEmitter)_behaviorList.get(ind);
				//hX_4.addBehavior("form1:text1", "onclick", new hX_4.JSFBehaviorGeneric("action:alert", "target:you clicked!"));
				if(behavior.getTargetAction()!=null)
				{
					out.write(controlVar + ".addBehavior(\""+behavior.getEvent()+"\",\""+ JSUtil.trimEventHandler(behavior.getOnActionFunction())+"\",new " + HxClientRenderUtil.HX_VAR_NAME + ".JSFBehaviorGeneric(\"action:"+behavior.getBehaviorAction()+"\",\" target:"+behavior.getTargetAction()+"\"));\n");
				}
				else
				{
						out.write(controlVar + ".addBehavior(\""+behavior.getEvent()+"\",\""+ JSUtil.trimEventHandler(behavior.getOnActionFunction())+"\",new " + HxClientRenderUtil.HX_VAR_NAME + ".JSFBehaviorGeneric(\"action:"+behavior.getBehaviorAction()+"\"));\n");
				}
				
			}
		}
		
		//add global tree events
		if (_onnodeexpand != null && !_onnodeexpand.equals("")) {
			out.write(controlVar + ".addHandler(\"onnodeexpand\",\"" + _onnodeexpand + "\");\n");
		}
		if (_onnodecollapse != null && !_onnodecollapse.equals("")) {
			out.write(controlVar + ".addHandler(\"onnodecollapse\",\"" + _onnodecollapse + "\");\n");
		}
		if (_onnodehighlight != null && !_onnodehighlight.equals("")) {
			out.write(controlVar + ".addHandler(\"onnodehighlight\",\"" + _onnodehighlight + "\");\n");
		}
		if (_onnodeselect != null && !_onnodeselect.equals("")) {
			out.write(controlVar + ".addHandler(\"onnodeselect\",\"" + _onnodeselect + "\");\n");
		}
		if (_onnodeunselect != null && !_onnodeunselect.equals("")) {
			out.write(controlVar + ".addHandler(\"onnodeunselect\",\"" + _onnodeunselect + "\");\n");
		}	
		if (_onnodedrop != null && !_onnodedrop.equals("")) {
			out.write(controlVar + ".addHandler(\"onnodedrop\",\"" + _onnodedrop + "\");\n");
		}
		if (_onnodedragenterover != null && !_onnodedragenterover.equals("")) {
			out.write(controlVar + ".addHandler(\"onnodedragenterover\",\"" + _onnodedragenterover + "\");\n");
		}
		out.write(
				adaptorVar
				+ ".bind();\n"
				+ controlVar
				+ ".updateControl();\n"
				+ "ODCProgressBar.endRenderControl(treeview);\n");
			//	+ "</script>\n");
		Streamer.trace.Header().println("Exiting TreeViewEmitter export...");
	}
}