/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//  
//OCO Source Materials
//  
//5724-E76 and 5724-E77
//  
// Copyright IBM Corp.  2003
//  
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////

package com.ibm.odcb.jrender.emitters;

import java.io.IOException;
import java.io.Writer;
import java.util.List;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;

////#IF# SDO
//import org.eclipse.emf.ecore.sdo.EDataObject;
//import org.eclipse.emf.ecore.util.EcoreEList;
////#END#

import com.ibm.odcb.jrender.diff.DiffException;
import com.ibm.odcb.jrender.diff.DiffProcessor;
import com.ibm.odcb.jrender.mediators.DynamicMediator;
import com.ibm.odcb.jrender.mediators.ExportException;
import com.ibm.odcb.jrender.mediators.Mediator;
import com.ibm.odcb.jrender.mediators.MediatorFactory;
import com.ibm.odcb.jrender.mediators.PageContext;
import com.ibm.odcb.jrender.mediators.gen.ecore.EClassMap;
import com.ibm.odcb.jrender.misc.Config;
import com.ibm.odcb.jrender.misc.EMFHelper;
import com.ibm.odcb.jrender.misc.Streamer;
import com.ibm.odcb.jrender.misc.URLRewriter;
import com.ibm.odcb.jrender.misc.StringUtil;

////#IF# SDO
//import commonj.sdo.DataGraph;
//import commonj.sdo.DataObject;
////#END#

////#IF# WDO
import com.ibm.etools.wdo.DataObject;
import com.ibm.etools.wdo.datagraph.DataGraph;
////#END#

/**
 * @author ldh
 * 
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates. To enable and disable the creation of type
 * comments go to Window>Preferences>Java>Code Generation.
 */
public class WDO4JSEmitter extends BaseEmitter {
    public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;

    public static final String VERSION = EMFHelper.VERSION;

    protected String modelTypeKey;

    protected String dataTypeKey;

    // We can accept objects of all kinds, but in the end, we either need real
    // objects with properties and all,
    // objects that represent business objects, objects that have been
    // explicitely mapped, or DataObject instances.
    // This method makes a few checkes for classes we need to support, and if a
    // match is found, transforms the Object
    // into the appropriate DataObject instance. This is particularely important
    // for DynamicWDO support (which requires
    // us to understand the ECore type of a DataObject rather than its Java
    // type), and JSF WDO tooling (which supplies
    // us with wrapped WDO objects using the classes DataObjectAccessBeanImpl
    // and DataListAccessBeanImpl, so we need
    // to fish for the DataObjects in them).
    // The transformer class, given an object, will transform that object into
    // the right instance/type, and set the BaseType
    // and ExtraName variables necessary for a lot of features, to the right
    // values.

    public static class Transformer {
        public String _BaseType;

        public String _ExtraName;

        public Object _O;

        public String _ReferenceName;

        protected static final void CheckO(Object O) {
            if (O == null) {
                throw new Error("WDO4JSEmitter.Init(Object, ModelName) called with a null Object.");
            }

            ////#IF# WDO
            if (O instanceof List && !(O instanceof
             com.ibm.websphere.sdo.internal.DataListAccessBeanImpl))
            {
               throw new Error("WDO4JSEmitter called with a generic List (type)"+ O.getClass().getName()+ "'). A Root object must be supplied.");
            }
            ////#END#
            
            ////#IF# SDO
//            
//            if (O instanceof List)
//            {
//            	if (!((O instanceof EcoreEList) || 
//            		  (O instanceof com.ibm.websphere.sdo.internal.DataListAccessBeanImpl) ) ) 
//                {
//                  throw new Error("WDO4JSEmitter called with a generic List (type '" + O.getClass().getName()
//                        + "'). A Root object must be supplied.");
//                }
//            }
            ////#END#
        }

        public Transformer(Object Obj) {
            CheckO(Obj);
            Streamer.debug.Header().println(
                    "Transformer called with Object of type '" + Obj.getClass().getName() + "'.");
            _O = Obj;
            	////#IF# WDO
            	try {
              	  if (_O instanceof com.ibm.websphere.sdo.internal.DataObjectAccessBeanImpl) {
                    Streamer.debug.Header().println("The object passed is a DataObjectAccessBeanImpl");
                    _O = ((com.ibm.websphere.sdo.internal.DataObjectAccessBeanImpl) _O).getEDataObject();
                } else if (_O instanceof com.ibm.websphere.sdo.internal.DataListAccessBeanImpl) {
            	////#END#
            
                //    // The structure of a dynamically created data list is
                //   // always to have the Root class have a "root"
                //    // EReference, and from there, a single multi-cardinality
                //    // reference. so let's check for this structure,
                //    // advance the received Object to the "root" node, and
                //    // figure out the name of the single EReference
                //    // dangling from it.
            
            	////#IF# WDO
                    Streamer.debug.Header().println("The object passed is a DataListAccessBeanImpl");
                   _O = ((com.ibm.websphere.sdo.internal.DataListAccessBeanImpl) _O).getDataGraph();

                    _O = EMFHelper.getDataObject(_O, true);
                   _ReferenceName = getReferenceName(((EObject) _O).eClass(),
                           ((com.ibm.websphere.sdo.internal.DataListAccessBeanImpl) Obj).getSchema());
                }
            	} catch (Throwable E) 
             	 { 
             	   Streamer.warning
              	          .Header()
              	          .println(
              	                  "Tried to test for AccessBeans, but failed. This is perfectly OK if you are not dealing with AccessBeans");
               	 Streamer.debug.Header().printStackTrace(E);
            	}
            	////#END#
            
            ////#IF# SDO
//
//            if(_O instanceof EcoreEList){
//                _ReferenceName = ((EcoreEList)_O).getEObject().eClass().getEStructuralFeature(((EcoreEList)_O).getFeatureID()).getEType().getName();
//                _O = ((EcoreEList)_O).getEObject();
//                if(_O instanceof EDataObject){            
//                    _O = EMFHelper.getDataObject(((EDataObject)_O).getDataGraph(), true);                   
//                }
//            }
//            else if (_O instanceof com.ibm.websphere.sdo.internal.DataObjectAccessBeanImpl) {
//                Streamer.debug.Header().println("The object passed is a DataObjectAccessBeanImpl");
//                _O = ((com.ibm.websphere.sdo.internal.DataObjectAccessBeanImpl) _O).getEDataObject();
//            } 
//            else if (_O instanceof com.ibm.websphere.sdo.internal.DataListAccessBeanImpl) {
//                    Streamer.debug.Header().println("The object passed is a DataListAccessBeanImpl");
//                   _O = ((com.ibm.websphere.sdo.internal.DataListAccessBeanImpl) _O).getDataGraph();
//                   _O = EMFHelper.getDataObject(_O, true);
//                   _ReferenceName = getReferenceName(((EObject) _O).eClass(),
//                           ((com.ibm.websphere.sdo.internal.DataListAccessBeanImpl) Obj).getSchema());
//            }
            ////#END#

            if (_O instanceof DataGraph) {
                _O = EMFHelper.getDataObject(_O, false);
            }
            if (_O instanceof DataObject) {
                _BaseType = EMFHelper.DataObject_CLASS.getName();
                _ExtraName = DynamicMediator.MakeExtraName((EObject) _O);
            } 
            else {
                _BaseType = _O.getClass().getName();
                _ExtraName = null;
            }
            Streamer.debug.Header().println(
                    "Transformer results: BaseType='" + _BaseType + "'; ExtraName='" + _ExtraName + "'.");
        }

        public static String getReferenceName(EClass RootObject, EClass Reference) {
            Streamer.debug.Header().println(
                    "Finding in '" + RootObject.getName() + "' the first reference of type '" + Reference.getName()
                            + "'.");
            List ERefs = RootObject.getEAllReferences();
            for (int i = 0; i < ERefs.size(); ++i) {
                EReference R = (EReference) ERefs.get(i);
                Streamer.debug.Header().println(
                        "Comparing EClass '" + R.getEType().getName() + "' with '" + Reference.getName() + "'.");
                if (R.getEType().equals(Reference) == true) {
                    // Check for extra references of the same name. We only
                    // return the first one found, so we need a warning
                    // if we find others which are going to be ignored.
                    while (++i < ERefs.size()) {
                        EReference R2 = (EReference) ERefs.get(i);
                        if (R2.getEType().equals(Reference) == true) {
                            Streamer.warning.Header().println(
                                    "We found more than one EReference of type '" + Reference.getName()
                                            + "' under EClass '" + RootObject.getName()
                                            + "'. Returning the first one found '" + R2.getName() + "'.");
                            break;
                        }
                    }
                    Streamer.debug.Header().println(
                            "Returning reference '" + R.getName() + "' of type '" + R.getEType().getName()
                                    + "' off Root object of type '" + RootObject.getName() + "'.");
                    return R.getName();
                }
            }
            return null;
        }
    }

    public WDO4JSEmitter() {
    }

    protected Object _O;

    protected String _BaseType;

    protected String _ExtraName;

    protected String _ModelExportMode;

    protected String _DataExportMode;

    protected String _ModelName;
    
    protected boolean _isFaces;

    protected PageContext _PageCtx;

    protected int _ExportCount;

    private static final String ECLASS = Config.versionizeFilename("eclass.js");
    private static final String ECREATOR = Config.versionizeFilename("ecreator.js");
    private static final String EOBJECT = Config.versionizeFilename("eobject.js");
    private static final String XMIHANDLER = Config.versionizeFilename("xmiHandler.js");
    private static final String BINDER = Config.versionizeFilename("binder.js");
    private static final String JSL_WDO4JS_C = Config.versionizeFilename("jsl_wdo4js_c.js");
    // Full Init
    public void Init(Object O, String BaseType, String ExtraName, boolean JSModelExport, boolean JSDataExport,
            String ModelName) {
        Transformer.CheckO(O);
        _O = O;
        _BaseType = BaseType;
        _ExtraName = ExtraName;
        _ModelExportMode = (JSModelExport == true) ? "_wdo4js_js" : "_wdo4js_xmi";
        _DataExportMode = (JSDataExport == true) ? "_wdo4js_js" : "_wdo4js_xmi";
        _ModelName = ModelName;
        
        //Boolean storing flag that governs encoding. If page has FacesContext encoding should occur.
        _isFaces=StringUtil.checkEncoding();
        
        //Encoding variables for portal
        String model_var = StringUtil.encodeString(_ModelName,_isFaces);
        CONTROL_VAR = StringUtil.encodeString("WDO4JSModelRoot_" + model_var,_isFaces);
        
        try {
            modelTypeKey = MediatorFactory.GenTypeKey(_BaseType, _ExtraName, _ModelExportMode);
            dataTypeKey = MediatorFactory.GenTypeKey(_BaseType, _ExtraName, _DataExportMode);
            MediatorFactory.initGenericMediator(modelTypeKey, dataTypeKey, _O);
        } catch (ExportException e) {
            Streamer.error.Header().printStackTrace(e);
        }
    }

    // Init assuming JS Export for both model and data
    public void Init(Object O, String BaseType, String ExtraType, String ModelName) {
        Init(O, BaseType, ExtraType, true, true, ModelName);
    }

    // Init assuming JS Export for both model and data, and basic type
    // information
    public void Init(Object O, String BaseType, String ModelName) {
        Init(O, BaseType, null, true, true, ModelName);
    }

    // Init assuming JS Export for both model and data, basic type information,
    // and core type to be derived
    // automatically from the type of the Object passed.
    public void Init(Object O, String ModelName) {
        // It is possible to get generic WDO objects. If so, we need to make a
        // few renamings in order to find the right
        // mediator, which might have been generated dynamically.
        Transformer T = new Transformer(O);
        Init(T._O, T._BaseType, T._ExtraName, true, true, ModelName);
    }

    public void Export(Writer Out, PageContext Ctx) throws IOException, ExportException {
        this.exportIncludes(Out, Ctx);
        ExportModel(Out, Ctx);
        ExportData(Out, Ctx);
        //Fenil Change..move the JS call to register the model with the page
        // control from JSF UIDataModel to the emitter
        //Out.write("<SCRIPT language=\"JavaScript\">");
        //Out.write("ODCRegistry.addModelInfo(\"" + _ModelName + "\"," + this.CONTROL_VAR + ");");
        //Out.write("</SCRIPT>");
    }

    public void ExportModel(Writer Out, PageContext Ctx) throws IOException, ExportException {
        // Note: Kaushal -- 10/10/2003
        // this exportIncludes() is being called from here because some one can
        // call ExportModel()
        // and ExportData() independently of calling Export() itself. So the
        // includes, which were
        // only being pushed out from Export() earlier, would not have been
        // pushed out in that case.
        // To guard from that, we are calling exportIncludes() even here. This
        // is a repititive funcion
        // call, but the includes are clearly guarded from duplicate inclusion.
        this.exportIncludes(Out, Ctx);
        
        _ModelName = StringUtil.encodeString(_ModelName,_isFaces);
        
        try {
            Out.write("<script type=\"text/javascript\"> ODCProgressBar.startLoadModel(\"" + _ModelName + "\");</script> \n");

            Mediator.ExportSchema(Out, Ctx, modelTypeKey, _ModelName, _O);

            Out.write("<script type=\"text/javascript\"> ODCProgressBar.endLoadModel(\"" + _ModelName + "\");</script> \n");
        } catch (IOException e) {
            Streamer.error.Header().printStackTrace(e);
            throw new ExportException(e.toString());
        }
    }

    public void ExportData(Writer Out, PageContext Ctx) throws ExportException {
        _PageCtx = Ctx;
        _ExportCount = Ctx.NextInstanceCount() + 1;
        boolean DebugFlag = Ctx.getContextVariable(InitializationEmitter._ID_DEBUG_MODE) != null;
        try {
            Out.write("<script type=\"text/javascript\"> ODCProgressBar.startLoadDataInstance(\"" + _ModelName + "\");</script> \n");            
            Mediator.ExportData(Out, Ctx, _O, dataTypeKey, DebugFlag);
            _ExportCount = Ctx.NextInstanceCount() - _ExportCount;
            Out.write("<script type=\"text/javascript\"> ODCProgressBar.endLoadDataInstance(\"" + _ModelName + "\");</script> \n");
        } catch (IOException e) {
            Streamer.error.Header().printStackTrace(e);
            throw new ExportException(e.toString());
        }
    }

    
    public int getExportCount() {
        return _ExportCount;
    }

    public String getExportIDByObject(Object O) throws ExportException {
        Transformer T = new Transformer(O);
        return Mediator.getExportID(_PageCtx, T._O, MediatorFactory.GenTypeKey(T._BaseType, T._ExtraName,
                _DataExportMode), _ModelName);
    }

    public String getExportIDByName(Object O, String BaseType, String ExtraName) throws ExportException {
        return Mediator.getExportID(_PageCtx, O, MediatorFactory.GenTypeKey(BaseType, ExtraName, _DataExportMode),
                _ModelName);
    }

    public String getReferenceName(Object O, String ReferenceName) throws ExportException {
        Transformer T = new Transformer(O);
        return T._ReferenceName == null ? ReferenceName : T._ReferenceName;
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////////////////
    // Get an EMap based on a class name, and the extra name. Id no extra name
    // is available, pass in null.
    // This method returns the EClassMap that corresponds to the object passed
    // in, in which casem BaseType and
    // ExtraName are automatically derived, or by explicitely passing in those
    // parameters.
    public static EClassMap getEClassMapByObject(Object O) throws ExportException {
        Transformer T = new Transformer(O);
        return Mediator.getEClassMapForJavaComplexName(T._BaseType, T._ExtraName);
    }

    public static EClassMap getEClassMapByName(String BaseType, String ExtraName) throws ExportException {
        //call the internal APIs to get the reverse mapping;
        return Mediator.getEClassMapForJavaComplexName(BaseType, ExtraName);
    }

    private void exportIncludes(Writer Out, PageContext Ctx) throws IOException, ExportException {
        if (TestGuard(Ctx, "WDO4JS_JS_FLAG") == false) {
            URLRewriter R = (URLRewriter) Ctx.getContextVariable(InitializationEmitter._ID_URL_REWRITER);
            if (Ctx.getContextVariable(InitializationEmitter._ID_DEBUG_MODE) == InitializationEmitter.DEBUG_YES) {
                Out.write("\n<script type=\"text/javascript\" src=\"" + R.Rewrite("jsl/" +  ECLASS) + "\"></script>\n<script type=\"text/javascript\" src=\""
                        + R.Rewrite("jsl/" + ECREATOR) + "\"></script>\n<script type=\"text/javascript\" src=\"" + R.Rewrite("jsl/" + EOBJECT)
                        + "\"></script>\n<script type=\"text/javascript\" src=\"" + R.Rewrite("jsl/" + XMIHANDLER)
                        + "\"></script>\n<script type=\"text/javascript\" src=\"" + R.Rewrite("jsl/" + BINDER) + "\"></script>\n");
            } else {
                Out.write("<script type=\"text/javascript\" src=\"" + R.Rewrite("jsl/" + JSL_WDO4JS_C)
                        + "\"></script>\n");
            }
        }
    }

    public DiffProcessor.DiffedObjectsLists Import(String DiffStr) throws DiffException {
        return DiffProcessor.Import(_O, DiffStr, EClassMap.MakeComplexName(_BaseType, _ExtraName));
    }
}
