/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//
//OCO Source Materials
//
//5724-E76 and 5724-E77
//
// Copyright IBM Corp.  2003
//
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////

package com.ibm.odcb.jrender.emitters;

import java.io.Writer;
import java.io.IOException;
import com.ibm.odcb.jrender.mediators.ExportException;
import java.util.ArrayList;
import com.ibm.odcb.jrender.mediators.PageContext;
import com.ibm.odcb.jrender.misc.Config;
import com.ibm.odcb.jrender.misc.Streamer;
import com.ibm.odcb.jrender.misc.URLRewriter;
import com.ibm.odcb.jrender.misc.StringUtil;
/**
 * The <strong>WebServicesEmitter</strong> generates code for creating a web service and associating it with 
 * a data model.
 * 
 * @author Kaushal Kurapati
 * @author Rajesh V Ramachandran
 */
public class WebServicesEmitter extends BaseEmitter {

	public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;
	private static final String WSSAMPLE_ASCRIPT_SWF =  Config.versionizeFilename("wssample_ascript.swf");

	private static final String WEBSERVICE = Config.versionizeFilename("webservice.js");
	private static final String WEBSERVICEADAPTER = Config.versionizeFilename("webserviceadapter.js");
	private static final String JSL_WEBSERVICE_C = Config.versionizeFilename("jsl_webservice_c.js");
	
	// Constructors; default one is empty; second one does all the work.
	// If object is instantiated with default one, developer has to call
	// set() methods to set class variables.
	public WebServicesEmitter() {
		super("ODCWebService");
	}
	/**
	 * Constructor that initializes all class variables.
	 * <p>
	 * @param modelName				Name of the data model that this web service is bound to
	 * @param wsdlURL				WSDL URL that provides the web service
	 * @param friendlyName			An alias for referring to the web service
	 * @param webServiceOperation	The name of the actual operation that is called in the web service
	 */
	public WebServicesEmitter(String wsdlURL, String friendlyName, String webServiceOperation) {
		super("ODCWebService");
		this.Init(wsdlURL, friendlyName, webServiceOperation);
	}

	// Class variables
	protected String _wsdlURL;
	protected String _friendlyName;
	protected String _webServiceOperation;
	protected ArrayList _wsInput = new ArrayList(7);
	protected ArrayList _wsOutput = new ArrayList(7);

	/**
	 * Static class WSInput holds the eObjectID and eAttributeName that serve as inputs
	 * to the web service operation.
	 */
	protected static class WSInput {
		public WSInput(
			String eObjectID,
			String eAttributeName,
			String requestElementName,
			String attributeName,
			String modelName,boolean useAsList) {
			_eObjectID = eObjectID;
			_eAttributeName = eAttributeName;
			_RequestElementName = requestElementName;
			_attribute = attributeName;
			_modelName = modelName;
			_useAsList = useAsList;
		}
		public WSInput(String value, String requestElementName) {
			_RequestElementName = requestElementName;
			_constantValue = value;
		}
		public String _eObjectID;
		public String _eAttributeName;
		public String _RequestElementName;
		public String _attribute;
		public String _constantValue;
		public String _modelName;
		public boolean _useAsList;
	}

	/**
	 * Static class WSOutput carries the target eObjectID, the eAttributeName to update
	 * in the eObejct and the name of the ResponseElement in the WebService that maps to the eAttribute
	 * in the eObject
	 */
	protected static class WSOutput {
		public WSOutput(String eObjectID, String eAttributeName, String ResponseElementName,String modelName,String attributeName) {
			_eObjectID = eObjectID;
			_eAttributeName = eAttributeName;
			_ResponseElementName = ResponseElementName;
			_modelName = modelName;
			_attributeName = attributeName;
		}

		public String _eObjectID;
		public String _eAttributeName;
		public String _ResponseElementName;
		public String _modelName;
		public String _attributeName;

	}

	// init() initializes all class variables.
	public void Init(String wsdlURL, String friendlyName, String webServiceOperation) {
		_wsdlURL = wsdlURL;
		_friendlyName = friendlyName;
		_webServiceOperation = webServiceOperation;
	
		Streamer.debug.Header().println(
			"In WebServices emitter --> "
				+ "\n"
				+ "\n"
				+ "wsdl URL: "
				+ this._wsdlURL
				+ "\n"
				+ "friendlyName: "
				+ this._friendlyName
				+ "\n"
				+ "Webservice Operation: "
				+ this._webServiceOperation);
	}

	public void AddInputParameter(
		String modelName,
		String eObjectID,
		String eAttributeName,
		String requestElementName,
		String attributeName,boolean useAsList) {
		_wsInput.add(new WSInput(eObjectID, eAttributeName, requestElementName, attributeName,modelName,useAsList));
	}
	public void AddInputParameter(String value, String requestElement) {
		_wsInput.add(new WSInput(value, requestElement));
	}
	public void AddOutputParameter(String modelName,String eObjectID, String eAttributeName, String ResponseElementName,String attributeName) {
		_wsOutput.add(new WSOutput(eObjectID, eAttributeName, ResponseElementName,modelName,attributeName));
	}

	// All set functions to initialize class instance variables.
	/**
	 * Sets the _wsdlURL.
	 * @param _wsdlURL The _wsdlURL to set
	 */
	public void setWsdlURL(String wsdlURL) {
		_wsdlURL = wsdlURL;
	}
	/**
	 * Sets the _friendlyName.
	 * @param _friendlyName The _friendlyName to set
	 */
	public void setFriendlyName(String friendlyName) {
		_friendlyName = friendlyName;
	}
	/**
	 * Sets the _webServiceOperation.
	 * @param _webServiceOperation The _webServiceOperation to set
	 */
	public void setWebServiceOperation(String webServiceOperation) {
		_webServiceOperation = webServiceOperation;
	}

	/**
	 * Export method emits the code for creating a web service control and adapter, and setting up the
	 * input and output parameters in the web service.
	 * <p>
	 * @param out	Writer output stream that the generated code is written to
	 * @param PC	Page Context
	 */
	public void Export(Writer out, PageContext PC) throws ExportException, IOException {

		Streamer.trace.Header().println("Entering WebServiceEmitter export...");
		out.write("<script type=\"text/javascript\">\n");
		out.write("ODCProgressBar.startRenderControl(webservice);\n");
		out.write("</script>\n");
		//Fenil Change
		//remove all leading and trailing spaces from the friendly name and replace internal spaces with an _
		this._friendlyName = this._friendlyName.trim().replace(' ', '_');
		//if JSF client id is used as the friendly name then it will have : in it
		this._friendlyName = this._friendlyName.replace(':','_');
		_varIndex = PC.NextInstanceCount();
		
		//Check if encoding should occur
		boolean isFaces = StringUtil.checkEncoding();
		
		final String lControlVarStr = StringUtil.encodeString(CONTROL_VAR + _varIndex, isFaces);
		final String lAdapterVarStr = StringUtil.encodeString(ADAPTER_VAR + _varIndex, isFaces);
		URLRewriter R = (URLRewriter) PC.getContextVariable(InitializationEmitter._ID_URL_REWRITER);
		boolean debugMode =
			PC.getContextVariable(InitializationEmitter._ID_DEBUG_MODE) == InitializationEmitter.DEBUG_YES
				? true
				: false;

		if (TestGuard(PC, "WEBSERVICES_JS_FLAG") == false) {
			if (debugMode == true) {
				out.write("\n<script type=\"text/javascript\" src=\"" + R.Rewrite("jsl/webservice/" + WEBSERVICE) + "\"></</script>>");
				out.write("\n<script type=\"text/javascript\" src=\"" + R.Rewrite("jsl/webservice/" + WEBSERVICEADAPTER) + "\"></script>");
				Streamer.trace.Header().println("Included individual webservice JS files...");
			} else {
				out.write(
					"<script type=\"text/javascript\" src=\"" + R.Rewrite("jsl/" + JSL_WEBSERVICE_C) + "\"></script>");
				out.write("\n");
				Streamer.trace.Header().println("Included compressed webservice JS file...");
			}
			out.write(
				"<object classid=\"clsid:d27cdb6e-ae6d-11cf-96b8-444553540000\" codebase=\"http://fpdownload.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=7,0,0,0\" width=\"10\" height=\"10\" id=\"wscomp\" align=\"middle\">");
			out.write("\n");
			out.write("<param name=\"allowScriptAccess\" value=\"sameDomain\"/>");
			out.write("\n");
			out.write("<param name=\"movie\" value=\"" + R.Rewrite("jsl/webservice/" + WSSAMPLE_ASCRIPT_SWF) + "\">");
			out.write("\n");
			out.write(" <param name=\"quality\" value=\"high\"/>");
			out.write("\n");
			out.write("<param name=\"bgcolor\" value=\"#ffffff\"/>");
			out.write("\n");
			out.write(
				" <embed src=\""
					+ R.Rewrite("jsl/webservice/" + WSSAMPLE_ASCRIPT_SWF)
					+ "\"  quality=\"high\" bgcolor=\"#ffffff\" width=\"10\" height=\"10\" name=\"wscomp\" align=\"middle\" allowScriptAccess=\"sameDomain\" type=\"application/x-shockwave-flash\" pluginspage=\"http://www.macromedia.com/go/getflashplayer\" />");
			out.write("\n");
			out.write("</object>");
		}
		out.write(
			"<script type=\"text/javascript\">\n"
				+ "var "
				+ lControlVarStr
				+ " = new WebServiceControl(window.document.wscomp);\n"
				+ "var "
				+ lAdapterVarStr
				+ " = new WebServiceAdapter("
				+ lControlVarStr
				+ ", '"
				+ lAdapterVarStr
				+ ".resultHandler', '"
				+ _wsdlURL
				+ "', '"
				+ _friendlyName
				+ "', '"
				+ _webServiceOperation
				+ "');");
		out.write("\n");
		StringBuffer XXX = new StringBuffer(128);
		if (_wsInput.size() > 0) {

			for (int i = 0; i < _wsInput.size(); ++i) {
				WSInput I = (WSInput) _wsInput.get(i);
				if (I._constantValue != null) {
					XXX
						.append(",['")
						.append("null")
						.append("','")
						.append(I._constantValue)
						.append("','")
						.append("null")
						.append("','")
						.append(I._RequestElementName)
						.append("','")
						.append("null")
						.append("']");
				} else {
					XXX
						.append(",['")
						.append(I._eObjectID)
						.append("','")
						.append(I._eAttributeName)
						.append("',")
						.append("WDO4JSModelRoot_"+I._modelName+".Root")
						.append(",'")
						.append(I._RequestElementName)
						.append("','")
						.append(I._attribute)	
					    .append("','")		
						.append(I._useAsList)		  
						.append("']");
				}
			}
			XXX.append("]");
			XXX.setCharAt(0, '[');
		} else {
			XXX.append("[]");
		}
		out.write(lAdapterVarStr);
		out.write(".setupParams(");
		out.write(XXX.toString());
		out.write(",");
		XXX.setLength(0);
		if (_wsOutput.size() > 0) {
			for (int i = 0; i < _wsOutput.size(); ++i) {
				WSOutput O = (WSOutput) _wsOutput.get(i);
				XXX
					.append(",['")
					.append(O._eObjectID)
					.append("','")
					.append(O._eAttributeName)
					.append("',")
					.append("WDO4JSModelRoot_"+O._modelName+".Root")
					.append(",'")
					.append(O._ResponseElementName)
					.append("','")
					.append(O._attributeName)
					.append("']");
			}
			XXX.append("]");
			XXX.setCharAt(0, '[');
		} else {
			XXX.append("[]");
		}
		out.write(XXX.toString());
		XXX.setLength(0);		
		out.write(");");		
		out.write("</script>");
		out.write("\n");
      // Brendan: #43099
		//out.write("<script type=\"text/javascript\">");
		//out.write("\n");
		//out.write("function " + lControlVarStr + "Callback(){");
		//out.write("\n");
		//out.write(lAdapterVarStr);
		//out.write(".resultHandler();");
		//out.write("\n");
		//out.write("}");
		//out.write("</script>");
		out.write("<script type=\"text/javascript\">\n");
		out.write("ODCProgressBar.endRenderControl(webservice);\n");
		out.write("</script>\n");
		Streamer.trace.Header().println("Exiting WebServiceEmitter export...");
	}
}
