/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//
//OCO Source Materials
//
//5724-E76 and 5724-E77
//
// Copyright IBM Corp.  2003
//
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////

package com.ibm.odcb.jrender.mediators;

import java.io.File;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;

import com.ibm.odcb.jrender.mediators.gen.MappingsCompiler;
import com.ibm.odcb.jrender.mediators.gen.WDO4JSMappings;
import com.ibm.odcb.jrender.misc.Config;
import com.ibm.odcb.jrender.misc.EMFHelper;
import com.ibm.odcb.jrender.misc.FileUtil;
import com.ibm.odcb.jrender.misc.PropertiesUtil;
import com.ibm.odcb.jrender.misc.Streamer;
import com.ibm.odcb.jrender.utilities.Bean2ECoreGen;
import com.ibm.odcb.jrender.utilities.EMapGen;

/**
 * @author ldh
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class DynamicMediator
 {
	   // There needs to be more error handling here. This will be called at runtime, or by the tooling,
	   // and so, saving the ECore to file needs a return code, generating the EMap does too etc....
	   public static boolean HandleDynamicWDOMapping(EClass C, String PackagePath
	                                               , String PropertiesPath, String BaseSourcePath 
	                                               , String BaseBuildPath, boolean RegenerateAll)
	    {
	      ///////////////////////////////////////////////////////////////////////////////////////////////////
	      // Figuring out the EPackage name
	      EPackage Pkg = C.getEPackage();
	      String EPackageName = Pkg.getName();
	
	      DynamicHelper DH = new DynamicHelper();
	      if (DH.Init(PropertiesPath, BaseSourcePath, BaseBuildPath, PackagePath, EPackageName) == false)
	       return false;
	
	      ///////////////////////////////////////////////////////////////////////////////////////////////////
	      // Generate the ECore
	      EMFHelper.SaveEPackageToECore(Pkg, DH._BaseSourcePath+"/"+PackagePath, "/"+EPackageName+".ecore");
	      if (FileUtil.FileCopy(DH._BaseSourcePath+"/"+PackagePath+"/"+EPackageName+".ecore", 
	                            DH._BaseBuildPath+"/"+PackagePath+"/"+EPackageName+".ecore", true) == false)
	       return false;
	
	      DH.EMapGen(null, C.getName());
	
	      if (DH.RegisterResources(true) == false)
	       return false;
	
	      return RegenerateAll == true? DH.MediatorGen(null, true)
	                                  : DH.MediatorGen(EPackageName, false);
	    }
   public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;

   protected static final String DYN_WDO_BASE_TYPE  = EMFHelper.DataObject_CLASS.getName();
   protected static final String DYN_WDO_EXTRA_NAME = "DynWDO`";
   protected static final String DYN_WDO_COMPLEX_NAME_START  = EMFHelper.DataObject_CLASS.getName()+"(";

   //////////////////////////////////////////////////////////////////////////////////////////////////
   // ExtraName is basically a String: DynWDO`<package_name>`<eclass_name>
   public static String MakeExtraName(EClass C)
    {
		String RootEClassName = C.getName();
		EPackage P = C.getEPackage();
		String EPackageName = P.getName();
		return DYN_WDO_EXTRA_NAME + EPackageName+"`"+RootEClassName;
	 }
   public static String MakeExtraName(EObject O)
    {
      return MakeExtraName(O.eClass());
    }
   public static String MakeExtraName(String EPackageName, String RootEClassName)
    {
      return DYN_WDO_EXTRA_NAME + EPackageName+"`"+RootEClassName;
    }
   public static boolean isExtraNameDynamic(String ExtraName)
    {
      return ExtraName.startsWith(DYN_WDO_EXTRA_NAME);
    }

	//////////////////////////////////////////////////////////////////////////////////////////////////
	// ComplexName is a String: com.ibm.etools.wdo.dataObject(<extra_name>)
   public static String MakeComplexName(EClass C)
    {
	  return DYN_WDO_COMPLEX_NAME_START+MakeExtraName(C)+")";
	}
   public static String MakeComplexName(EObject O)
    {
	  return DYN_WDO_COMPLEX_NAME_START+MakeExtraName(O.eClass())+")";
	}
   public static String MakeComplexName(String EPackageName, String RootEClassName)
	{
	  return DYN_WDO_COMPLEX_NAME_START+MakeExtraName(EPackageName,RootEClassName)+")";
	}
   public static boolean isComplexNameDynamic(String ComplexName)
    {
      return ComplexName.startsWith(DYN_WDO_COMPLEX_NAME_START+DYN_WDO_EXTRA_NAME);
    }

   public static String getBasePartFromComplexName(String ComplexName)
    {
      int i = ComplexName.indexOf('(');
      return i == -1 ? ComplexName : ComplexName.substring(0, i);
    }


   protected static final boolean TOTO = PrintVersion();
   protected static boolean PrintVersion()
    {
      Streamer.status.Header().println("JWL Version is '"+EMFHelper.VERSION+"' at static initialization of DynamicMediator.");
      return true;
    }
   
   protected static class DynamicHelper
    {
      public DynamicHelper() 
       {
         Streamer.status.Header().println("JWL Version is '"+EMFHelper.VERSION+"' at dynamic usage of DynamicMediator.");
       }
      protected String _PropertiesPath;
      protected String _PropertiesFileName;
      protected String _BaseSourcePath;
      protected String _BaseBuildPath;
      protected String _PackagePath;
      protected String _DataName;
      
      ///////////////////////////////////////////////////////////////////////////////////////////////////
      // Normalizes the 
      public static String CleanPath(String Path)
       {
         // Make sure that the path does not have leading or trailing white-space characters
         Path = Path.trim();

         // Make sure there is no mix of '/' and '\\' separators
         Path = Path.replace('/', File.separatorChar);

         // Make sure that the path ends with a '\\' character
         if (Path.endsWith(File.separator) == false)
          Path=Path+File.separator;
          
         // Make sure that the drive letter, if any, is upper case.
         if (Path.length() > 2 && Path.charAt(1)==':' && Path.charAt(2)==File.separatorChar && Character.isUpperCase(Path.charAt(0)) == false)
          Path = Character.toUpperCase(Path.charAt(0)) + Path.substring(1);

         return Path; 
       }

      ///////////////////////////////////////////////////////////////////////////////////////////////////
      // Get some Properties first for base source and base build paths
      public boolean Init(String PropertiesPath, String BaseSourcePath, String BaseBuildPath, 
                          String PackagePath, String DataName)
       {
         // Make sure we get the latest properties
         Config.ReInit();
      
         _PropertiesPath= CleanPath(PropertiesPath);
         _BaseSourcePath= CleanPath(BaseSourcePath);
         _BaseBuildPath = BaseBuildPath ;
         _PackagePath   = PackagePath   ;
         _DataName      = DataName      ;
      
         if (_PropertiesPath.startsWith(_BaseSourcePath) == false)
          {
            Streamer.error.Header().println("The PropertiesPath passed '"+_PropertiesPath
                                         +"' does not match with the BaseSourcePath '"+_BaseSourcePath+"' passed in.");
            return false;
          }
         _PropertiesFileName = _PropertiesPath.substring(_BaseSourcePath.length());
         
         return true;
       }

     public boolean EMapGen(Map Mappings, String RootEClassName)
      {
        EMapGen.Params P = new EMapGen.Params();
        P._SourcePath    = _BaseSourcePath;
        P._BuildPath     = _BaseBuildPath ;
        P._EcoreFileName = _BaseSourcePath+"/"+_PackagePath+"/"+_DataName+".ecore";
        P._RootEClassName= RootEClassName;
        P._Mappings      = Mappings;
        P.MakeDefaultEMapMerge();
        
        try
         {
           EMapGen.Process(P);
         }
        catch (Exception E)
         {
           Streamer.error.Header().println("Generating the EMap failed.");
           Streamer.error.Header().printStackTrace(E);
           return false;
         }

        if (FileUtil.FileCopy(_BaseSourcePath+"/"+_PackagePath+"/"+_DataName+".emap", 
                              _BaseBuildPath+"/"+_PackagePath+"/"+_DataName+".emap", true) == false)
         return false;

        return true;
      }

       
      public boolean RegisterResources(boolean Register)
       {
         ///////////////////////////////////////////////////////////////////////////////////////////////////
         // Register ECore and EMap in the Properties file
         try
          {
            PropertiesUtil Prop = new PropertiesUtil();
            Prop.Open(_BaseSourcePath+"/"+_PropertiesFileName);
            if (Register == true)
             {
               Prop.AddProperty(Config.PROP_ECORE_FILES.toString(), _PackagePath+"/"+_DataName+".ecore");
               Prop.AddProperty(Config.PROP_EMAP_FILES .toString(), _PackagePath+"/"+_DataName+".emap" );
             }
            else
             {
               Prop.DeleteProperty(Config.PROP_ECORE_FILES.toString(), _PackagePath+"/"+_DataName+".ecore");
               Prop.DeleteProperty(Config.PROP_EMAP_FILES .toString(), _PackagePath+"/"+_DataName+".emap" );
             }
            Prop.Save();

            if (FileUtil.FileCopy(_BaseSourcePath+"/"+_PropertiesFileName, 
                                  _BaseBuildPath+"/"+_PropertiesFileName, true) == false)
             return false;

            Config.ReInit();
            return true;
          }
         catch (Exception E)
          {
            Streamer.error.Header().println("Cannot update the properties file resources");
            Streamer.error.Header().printStackTrace(E);
            return false;
          }
       }

      public boolean MediatorGen(String EPackageName, boolean ForceRegeneration)
       {
         ///////////////////////////////////////////////////////////////////////////////////////////////////
         // Generate and do not compile the Mediators, update the factory.
         MappingsCompiler MC = MediatorFactory.BuildMediators(true, false, _BaseSourcePath, null,
                                                              null, EPackageName, ForceRegeneration);

         if (MC.getVerificationStatus() == false || MC.getGenerationStatus() == false)
          {
            Streamer.error.Header().println("Could not create the SDO4JS mappings for "+_DataName);
            return false;
          }
         Streamer.status.Header().println("Successfully created the SDO4JS mappings for "+_DataName);

         if (FileUtil.FileCopy(_BaseSourcePath+"/"+_PackagePath+"/"+_DataName+"/"+_DataName+"_client.ecore", 
                               _BaseBuildPath +"/"+_PackagePath+"/"+_DataName+"/"+_DataName+"_client.ecore", true) == false)
          return false;
          
         return true;
       }       
    }




   // There needs to be more error handling here. This will be called at runtime, or by the tooling,
   // and so, saving the ECore to file needs a return code, generating the EMap does too etc....
   public static boolean HandleDynamicBeanMapping(Class C, String BeanName, Map Mappings, boolean Include, Map IncludesExcludes,
                                                  String PackagePath, String PropertiesPath, String BaseSourcePath, String BaseBuildPath,
                                                  boolean RegenerateAll)
    {
      DynamicHelper DH = new DynamicHelper();
      if (DH.Init(PropertiesPath, BaseSourcePath, BaseBuildPath, PackagePath, BeanName) == false)
       return false;

      ///////////////////////////////////////////////////////////////////////////////////////////////////
      // Generate the ECore
      {
        com.ibm.odcb.jrender.utilities.Bean2ECoreGen.Params P = new com.ibm.odcb.jrender.utilities.Bean2ECoreGen.Params();
        P._BasePath = new File(DH._BaseSourcePath);
        P._ECoreFile= new File(DH._BaseSourcePath+"/"+PackagePath+"/"+BeanName+".ecore");
        P._RootJavaClassName = C.getName();
        P._Mappings = Mappings;
        P._Include = Include;
        P._IncludesExcludes = IncludesExcludes;
      
        File Dir = new File(P._ECoreFile.getParent());
        if (Dir.isDirectory() == false && Dir.mkdirs() == false)
         {
           Streamer.error.Header().println("The second parameter '"+P._ECoreFile+"' does not point to a valid path, or a path which cannot be created off the project root path");
           return false;
         }
        try
         {
           Bean2ECoreGen.Process(P);
         }
        catch (Exception E)
         {
           Streamer.error.Header().println("Generating the ECore failed.");
           Streamer.error.Header().printStackTrace(E);
           return false;
         }
        Mappings = Bean2ECoreGen.GetKnownTypesAndClear();
        if (FileUtil.FileCopy(DH._BaseSourcePath+"/"+PackagePath+"/"+BeanName+".ecore", 
                              DH._BaseBuildPath+"/"+PackagePath+"/"+BeanName+".ecore", true) == false)
         return false;
      }

      ///////////////////////////////////////////////////////////////////////////////////////////////////
      // Generate the EMap
      // Finding the Root EClass name based on the known Java Class name
      String RootEClassName = null;
      Iterator I = Mappings.entrySet().iterator();
      while (I.hasNext() == true)
       {
         Map.Entry E = (Map.Entry) I.next();
         String S = (String) E.getValue();
         if (S.equals(C.getName()) == true)
          {
            RootEClassName = (String) E.getKey();
            break;
          }
       }
      if (RootEClassName == null)
       {
         Streamer.error.Header().println("Could not find an EClass Name for Java Class '"+RootEClassName+"'.");
         return false;
       }
      DH.EMapGen(Mappings, RootEClassName);

      ///////////////////////////////////////////////////////////////////////////////////////////////////
      // And do the rest      
      if (DH.RegisterResources(true) == false)
       return false;
       
      return RegenerateAll == true? DH.MediatorGen(null, true)
                                  : DH.MediatorGen(BeanName, false);
    }

   // There needs to be more error handling here. This will be called at runtime, or by the tooling,
   // and so, saving the ECore to file needs a return code, generating the EMap does too etc....
   public static boolean handleDynamicSDOMapping(String ePackageName, String rootEClassName, String packagePath
                                               , String propertiesPath, String baseSourcePath 
                                               , String baseBuildPath, boolean regenerateAll)
    {
      ///////////////////////////////////////////////////////////////////////////////////////////////////
      // Figuring out the EPackage name
      
      //EPackage Pkg = C.getEPackage();
      //String EPackageName = Pkg.getName();

      DynamicHelper DH = new DynamicHelper();
      if (DH.Init(propertiesPath, baseSourcePath, baseBuildPath, packagePath, ePackageName) == false)
       return false;

      ///////////////////////////////////////////////////////////////////////////////////////////////////
      // Generate the ECore
      //commenting out this call.  Tooling should call the overloaded method with boolean before calling DynMediator
      //EMFHelper.SaveEPackageToECore(Pkg, DH._BaseSourcePath+"/"+PackagePath, "/"+EPackageName+".ecore");
      if (FileUtil.FileCopy(DH._BaseSourcePath+"/"+packagePath+"/"+ePackageName+".ecore", 
                            DH._BaseBuildPath+"/"+packagePath+"/"+ePackageName+".ecore", true) == false)
       return false;

      DH.EMapGen(null, rootEClassName);

      if (DH.RegisterResources(true) == false)
       return false;

      return regenerateAll == true? DH.MediatorGen(null, true)
                                  : DH.MediatorGen(ePackageName, false);
    }

   public static boolean RemoveDynamicMapping(String Name, String PackagePath
                                            , String PropertiesPath, String BaseSourcePath, String BaseBuildPath)
    {
      DynamicHelper DH = new DynamicHelper();
      if (DH.Init(PropertiesPath, BaseSourcePath, BaseBuildPath, PackagePath, Name) == false)
       return false;

      // Let's delete the ECore and EMap files
      FileUtil.FileDelete(DH._BaseSourcePath+"/"+PackagePath+"/"+Name+".ecore");
      FileUtil.FileDelete(DH._BaseSourcePath+"/"+PackagePath+"/"+Name+".emap");
      // Let's delete the mediators
      FileUtil.DirDelete (DH._BaseSourcePath+"/"+PackagePath+"/"+Name);

      if (DH.RegisterResources(false) == false)
       {
         Streamer.error.Header().println("Could not clean up the SDO4JS mapping resources for "+Name);
         return false;
       }

      Streamer.status.Header().println("Successfully cleaned up the SDO4JS mapping resources for "+Name);
      return true;
    }
    
   public static EMapWriter getEMapWriter(String ePackageName, String BaseSourcePath, String BaseBuildPath)
	{
		MappingsCompiler mc = MediatorFactory.BuildMediators(false, false, BaseSourcePath, BaseBuildPath,
                                                           null, ePackageName, false);
		WDO4JSMappings universe = mc.getUniverse();
		EMapWriter writer = new EMapWriter(universe, ePackageName, BaseSourcePath, BaseBuildPath);
		return writer;
	}
    
 }
 
