/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//
//OCO Source Materials
//
//5724-E76 and 5724-E77
//
// Copyright IBM Corp.  2003
//
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////

package com.ibm.odcb.jrender.mediators;

import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.io.FileNotFoundException;
import java.io.PrintWriter;

import java.io.FileOutputStream;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;

import com.ibm.odcb.jrender.mediators.gen.MappingsCompiler;
import com.ibm.odcb.jrender.mediators.gen.WDO4JSMappings;
import com.ibm.odcb.jrender.mediators.gen.ecore.CalculateAttribute;
import com.ibm.odcb.jrender.mediators.gen.ecore.EClassMap;
import com.ibm.odcb.jrender.mediators.gen.ecore.EFeatureMap;
import com.ibm.odcb.jrender.mediators.gen.ecore.EPackageMap;
import com.ibm.odcb.jrender.utilities.EMapGen;

import com.ibm.odcb.jrender.misc.ResourceHandler;
import com.ibm.odcb.jrender.misc.Streamer;


public class EMapWriter
 {
   public EMapWriter(WDO4JSMappings universe, String ePackageName, String baseSourcePath, String baseBuildPath)
    {
      this.universe         = universe;
//      this.ePackageName     = ePackageName;
      this._EPM             = universe.getEPackageMap(ePackageName);
//      this.eMapFilePath     = this.ePackageName.getEMapXMLFileName();
//      this.emapResourceName = universe.getEMapResourceNameForEPackage(ePackageName);
      this.baseSourcePath   = baseSourcePath;
      this.baseBuildPath    = baseBuildPath;
    }
   
//   protected String         eMapFilePath;
//   protected String         ePackage;
//   protected String         emapResourceName;
   protected WDO4JSMappings universe;
   protected EPackageMap    _EPM;
   protected String         baseSourcePath;
   protected String         baseBuildPath;
 
   public static final String ECLASSMAP_NOT_FOUND_MESSAGE   = "ECLASSMAP_NOT_FOUND_MESSAGE";
   public static final String EFEATUREMAP_NOT_FOUND_MESSAGE = "EFEATUREMAP_NOT_FOUND_MESSAGE";
   public static final String FILE_NOT_FOUND_MESSAGE        = "FILE_NOT_FOUND_MESSAGE";
   public static final String GETTER_NOT_FOUND_MESSAGE      = "GETTER_NOT_FOUND_MESSAGE";
   public static final String CANNOT_REMOVE_ECLASSMAP_DUE_TO_DANGLING_REFERENCES_MESSAGE = "CANNOT_REMOVE_ECLASSMAP_DUE_TO_DANGLING_REFERENCES_MESSAGE";
   public static final String CLASS_NOT_FOUND = "CLASS_NOT_FOUND";
   public static final String SETTER_NOT_FOUND_MESSAGE = "SETTER_NOT_FOUND_MESSAGE";
   
   
   public static final String TYPE_BOOLEAN = "boolean";
   public static final String TYPE_INTEGER = "int"    ;
   public static final String TYPE_FLOAT   = "float"  ;
   public static final String TYPE_STRING  = "string" ;
   public static final String TYPE_CHAR    = "char"   ;
   public static final String TYPE_DATE    = "date"   ;
   
    /**
     * @deprecated 
     */
   public String addFeatureMap(String classExportName, String featureName, String getter, boolean id)
    {
      return addFeatureMap(classExportName, featureName, getter, null, id);
    }
   /**
    * Adds an EFeatureMap with the given properties to the specified parentEClassMap.
    * If the parent EClassMap is not found, an error message is returned.
    * If the EFeatureMap already exist, it is replaced.
    * @param classExportName - The export name of the parent EClassMap
    * @param featureName     - name of the EFeature
    * @param get             - the get method or field
    * @param set             - the set method or field with "__SET_OBJ__" as a marker, 
    *                         for example "setName(__SET_OBJ__)", or "setStirng('someProperty', __SET_OBJ__)". 
    *                         Note the single quotes in place of double quotes. 
    * @param export          - export name of this feature
    * @param id              - true if this is part of a primary key; false otherwise
    * @return an error message or null
    */
   public String addFeatureMap(String classExportName, String featureName, String getter, String setter, boolean id)
    {
      EClassMap ECM = universe.getEClassMapByExport(classExportName);
      String str = null;
      if (ECM == null)
      {
      	//ERROR
        str =  ResourceHandler.getString(ECLASSMAP_NOT_FOUND_MESSAGE, new String[]{classExportName});
      	Streamer.error.Header().println(str);
      	return str;
      }
      
      EStructuralFeature ESF = ECM.getEStructuralFeatureByName(featureName);
      PropertyDescriptor[] PDs = null;
      try
       {
         PDs = Introspector.getBeanInfo(ECM.getBaseTypeClass()).getPropertyDescriptors();
       }
      catch (java.beans.IntrospectionException E)
       {
         //new message CLASS_NOT_FOUND
         str =  ResourceHandler.getString(CLASS_NOT_FOUND, new String[]{ECM.getBaseTypeClass().getName()});
       	 Streamer.error.Header().println(str);
       	 return str;
         //return ResourceHandler.getString(GETTER_NOT_FOUND_MESSAGE, new String[]{featureName});
       }
      
      if (getter == null)
       {
         getter = EMapGen.FindProperty(ESF, ECM.getBaseTypeClass(), PDs, true);
         if (getter.equals(EMapGen.ACCESS_METHOD_NEEDED))
         {
           //return ResourceHandler.getString(GETTER_NOT_FOUND_MESSAGE, new String[]{featureName});
           str =  ResourceHandler.getString(GETTER_NOT_FOUND_MESSAGE, new String[]{featureName});
       	   Streamer.error.Header().println(str);
       	   return str;
         }
       }
      if (setter == null)
       {
         setter = EMapGen.FindProperty(ESF, ECM.getBaseTypeClass(), PDs, false);
         if (setter.equals(EMapGen.ACCESS_METHOD_NEEDED))
         {
            str =  ResourceHandler.getString(SETTER_NOT_FOUND_MESSAGE, new String[]{featureName});
        	Streamer.warning.Header().println(str);
        	setter = null;
          }
       }
      //EFeatureMap(String eAttributeName, boolean isID, String Get, String Export)
      EFeatureMap EFM = ECM.getEFeatureMapByEFeatureName(featureName);
      boolean DynamicWDO = DynamicMediator.isComplexNameDynamic(ECM.getJavaComplexName());
      if (DynamicWDO)
      {
         featureName = EMapGen.getWDOFeatureName(ESF, ECM.getEPackageName());
      }
      if (EFM == null)
      {
         EFM = new EFeatureMap(featureName, id, getter, setter, true);       
      }
      else
      {
         ECM.removeEFeatureMap(EFM);
         EFM.setID(id);
         EFM.setGet(getter);
      }
      ECM.addEFeatureMap(EFM);
      if (id)
        ECM.addEIdFeatureMap(EFM);
      return null;
   }

    /**
     * Adds a Client EFeatureMap with the given properties to the specified parentEClassMap.
     * If the parent EClassMap is not found, an error message is returned.
     * If an EFeatureMap with the same featurename already exists, it is replaced.
     * @param classExportName - The export name of the parent EClassMap
     * @param featureName     - name of the Client EFeature
     * @param jsExpression    - the java script expression to resolve
     * @param type            - the type that the java script expression resolves to
     * @param export          - export name of this feature
     * @param id              - true if this is part of a primary key; false otherwise
     * @return an error message or null
     */
   public String addClientFeatureMap(String classExportName, String featureName, String jsExpression, String type, boolean id)
   {          
      EClassMap m = universe.getEClassMapByExport(classExportName);
      String str = null;
      if (m == null)
      {
      	//error
      	str =  ResourceHandler.getString(ECLASSMAP_NOT_FOUND_MESSAGE, new String[]{classExportName});
      	Streamer.error.Header().println(str);
        return  str;
      }
      EFeatureMap f = m.getEFeatureMapByEFeatureName(featureName);
      if (f != null)
      {
         m.removeEFeatureMap(f);
      }
      /*CalculateAttribute(
            String eAttributeName,
            boolean isID,
            String Export,
            String calculateExpression,
            String calculatedType)*/      
      f = new CalculateAttribute(featureName,id,jsExpression,type);
      m.addEFeatureMap(f);
      if (id)
        m.addEIdFeatureMap(f);
      return null;
   }
    /**
     * @deprecated 
     */
    public final String addClientFeatureMap(String classExportName, String featureName, String jsExpression, String type, String export, boolean id)
     {
       return addClientFeatureMap(classExportName, featureName, jsExpression, type, id);
     }            




    /**
     * Removes the EFeatureMap with the specified properties from the given parent EClassMap.
     * If the parent EClassMap is not found, an error message is returned.
     * If the EFeatureMap was not found in the parent EClassMap, an error message is returned.
     * @param classExportName   - export name of the parent EClassMap
     * @param featureExportName - export name of the EFeatureMap to remove
     * @return an error message or null
     */
   public String removeFeatureMap(String classExportName, String featureExportName)
   {
      EClassMap m = universe.getEClassMapByExport(classExportName);
      if (m == null)
      {
      	String Str = ResourceHandler.getString(ECLASSMAP_NOT_FOUND_MESSAGE, new String[]{classExportName});
      	Streamer.error.Header().println(Str);
      	return Str;
      }
      EFeatureMap f = m.getEFeatureMapByExport(featureExportName);
      if (f == null)
       {
      	 String Str = ResourceHandler.getString(EFEATUREMAP_NOT_FOUND_MESSAGE, new String[]{featureExportName, classExportName});
         Streamer.warning.Header().println(Str);
         return null;
       }
      else
      {
         m.removeEFeatureMap(f);
         return null;
      }
   }

    /**
     * Adds an EClassMap to the instance's universe.  If an EClassMap with
     * the same java application class name (appClassPath) already exist,
     * the existing EClassMap is updated with the new EClassName (and EClass) and export.
     * All the eFeatures of the existing EClassMap is retained.  You will neeed
     * to specifically call <code>removeFeatureMap<code> for each one you want removed.
     * @param appClassPath - The java class name with the full package info
     * @param eClassName   - Name of the eClass 
     * @param export       - Export name
     * @return an error message or null
     */
   public String addClassMap(String appClassPath, String eClassName, String export)
   {
      //EClassMap(String ePackageName, String eClassName, 
          //      String ComplexAppClassName,
      //          String Export, String XMLFileSource, 
       //         String theSourceEmapName, boolean diffOnRefresh)
   	
      Streamer.status.Header().println("Tracing in addClassMap() of EMapwriter.java Package name:"+_EPM.getName()+" appClassPath: " + appClassPath + " eClassName:" + eClassName + "export: " + export);

      EClassMap m = universe.getEClassMapByJavaComplexName(appClassPath);
      if (m == null)
       {
         m = new EClassMap(_EPM.getName(), eClassName, appClassPath, export, _EPM.getEMapXMLFileName(), _EPM.getEMapResourceName(), true);
      }
      else
      {
         //remove existing EClassMap from the universe
         universe.removeEClassMap(m);
      }
      //update values of existing EClassMap with the incoming parameters
      if ( m.getEClass() == null || !eClassName.equals(m.getEClassName()) )
      {
         m.setEClassName(eClassName);
         EClass ec = (EClass) universe.getEClassifier(_EPM.getName(), eClassName);
         m.setEClass(ec);
      }      
      m.setExport(export);                           
      universe.addEClassMap(_EPM, m, true);
      return null;
   }
      
   /**
    * Removes the EClassMap with the specified properties from the instance's universe.
    * If the EClassMap is not found, an error message is returned.
    * @param classExportName   - export name of the parent EClassMap
    * @return an error message or null
    */
   public String removeClassMap(String classExportName)
    {
      String  str = null;
   	  EClassMap m = universe.getEClassMapByExport(classExportName);
      if (m != null)
       {
         universe.removeEClassMap(m);
         return null;
       }
      else
      {
        
      	str =  ResourceHandler.getString(ECLASSMAP_NOT_FOUND_MESSAGE, new String[]{classExportName});
      	Streamer.error.Header().println(str);
      	return str;
      }
   }

    /**
     * Writes the EClassMaps in the instance's universe and it's children (EFeatureMaps)
     * into a ".emap" file.  
     * If the file already exists, it overwrites the contents.
     * No backups of existing files are taken.
     * @param genMediatorsToo
     * @return an error message or null
     */
   public String write(boolean genMediatorsToo)
    {
       String str = null; 
   	   try
       {
         PrintWriter out = new PrintWriter(new FileOutputStream(_EPM.getEMapXMLFileName()));
         EMapGen.writeEMap(out, _EPM, _EPM.getRootEClassMap().getEClassName(),
                           universe.getEClassMapsForEPackageIterator(_EPM.getName()));
         out.close();      
         if (genMediatorsToo)
          {
            MappingsCompiler MC = MediatorFactory.BuildMediators(true, false, baseSourcePath, null, null, 
                                                                 _EPM.getName(), true);
            if (MC.getVerificationStatus() == false)
             {
               return "Error creating the ClientData: mappings are inconsistent";
             }
            if (MC.getGenerationStatus() == false)
             {
               return "Error creating the ClientData: could not generate the code artifacts";
             }
          }
       }
      catch (FileNotFoundException e)
       {
         //return ResourceHandler.getString(FILE_NOT_FOUND_MESSAGE,new String[]{_EPM.getEMapXMLFileName()});
         str =  ResourceHandler.getString(FILE_NOT_FOUND_MESSAGE,new String[]{_EPM.getEMapXMLFileName()});
       	 Streamer.error.Header().println(str);
       	 return str;
       }
      return null;
    }
   /**
    * @return
    */
//   public String getEPackageName() {
//      return ePackageName;
//   }

   /**
    * @return
    */
//   public WDO4JSMappings getUniverse() {
//      return universe;
//   }

   /**
    * @param string
    */
//   public void setEPackageName(String string) {
//      ePackageName = string;
//   }

   /**
    * @param mappings
    */
//   public void setUniverse(WDO4JSMappings mappings) {
//      universe = mappings;
//   }

   /**
    * @return
    */
//   public String getEMapFilePath() {
//      return eMapFilePath;
//   }

   /**
    * @param string
    */
//   public void setEMapFilePath(String string) {
//      eMapFilePath = string;
//   }
   
   public String removeClassMap(String classExportName, boolean deleteReferences)
   {
      String str = null;
   	  EClassMap m = universe.getEClassMapByExport(classExportName);
      if (m == null) //Asking to remove something we cannot find... so we are ok.
      {
        str =  ResourceHandler.getString(ECLASSMAP_NOT_FOUND_MESSAGE, new String[]{classExportName});
      	Streamer.warning.Header().println(str);
      	return null;
      }
     universe.removeEClassMap(m);
      return null;
   }
   
   public String setDiffOnRefresh(String classExportName, boolean isDiffOnRefresh) {
	EClassMap classMap = universe.getEClassMapByExport(classExportName);
	if (classMap == null)
		return ResourceHandler.getString(ECLASSMAP_NOT_FOUND_MESSAGE, new String[] {classExportName});
	classMap.setDiffOnRefresh(isDiffOnRefresh);
	return null;
}

}
