/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//
//OCO Source Materials
//
//5724-E76 and 5724-E77
//
// Copyright IBM Corp.  2003
//
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////

package com.ibm.odcb.jrender.mediators;

import java.io.Writer;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import com.ibm.odcb.jrender.mediators.gen.ecore.EClassMap;
import com.ibm.odcb.jrender.mediators.gen.ecore.EFeatureMap;
import com.ibm.odcb.jrender.mediators.gen.generators.FeatureExporter;
import com.ibm.odcb.jrender.misc.Config;
import com.ibm.odcb.jrender.misc.StringUtil;
import com.ibm.odcb.jrender.misc.TypesUtil;
////#IF# SDO
//import commonj.sdo.DataObject;
////#END#

////#IF# WDO
import com.ibm.etools.wdo.DataObject;
////#END#


public class GenericSDO4JSMediator extends Mediator {
	
	private EClassMap eClassMap = null;
	public static final String KEY = "_wdo4js_js";
	protected final int     MEDIATOR_DEBUG_MODE = Config.getInteger(Config.PROP_MEDIATOR_DEBUG_MODE);
	protected final boolean _isFaces;
	public GenericSDO4JSMediator(EClassMap ecm)
	{
		super("GenericSDO4JSMediator", ","); 
		eClassMap = ecm;
		_isFaces = StringUtil.checkEncoding();
	}

	/* (non-Javadoc)
	 * @see com.ibm.odcb.jrender.mediators.Mediator#getSignature(java.lang.Object, java.lang.String, boolean, int)
	 */
	public String getSignature(
		Object o,
		String modelName,
		boolean first,
		int complexPropertyCount) {
			if (o == null) return "";
			try
			 {
			   StringBuffer sigStr = new StringBuffer(128);
			   //findEClassMap(o);
			   eClassMap.getKeySignature(sigStr,(DataObject)o,1,null);			   		   
			   return first == true ? modelName+"`"+eClassMap.getJavaComplexName()+sigStr.toString() : sigStr.toString();
			 }
			catch (Exception e)
			 {
			   if (e instanceof java.lang.ClassCastException)
				{
				  System.out.println("ClassCastException: Expecting type 'commonj.sdo.DataObject' but found object of type '"+o.getClass().getName()+"'"); 
				}
			   e.printStackTrace();
			   return null; 
			 }
	}

	private DataObject getDataObject(Object o) throws ExportException
	{
		if (o instanceof DataObject)
		{
		  return (DataObject) o;
		}
		else if (o instanceof org.eclipse.emf.ecore.util.EcoreEList)
		{
		  Iterator it = ((org.eclipse.emf.ecore.util.EcoreEList)o).iterator();
		  if (it.hasNext())
		  {
		  	Object lo = it.next();
			return getDataObject(lo);
		  }
		}
		else
		{
		  throw new ExportException("GenericSDO4JSMediator does not support the type '"+o.getClass().getName()+"'.  Only DataObjects and EcoreELists containing DataObjects are supported.");
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see com.ibm.odcb.jrender.mediators.Mediator#Export(java.io.Writer, com.ibm.odcb.jrender.mediators.PageContext)
	 */
	public void Export(Writer out, PageContext ctx, Object o) throws ExportException {
		try
		{
		 if (o == null || !(o instanceof DataObject))
		   throw new ExportException("Generically exporting schema without supplying an instance of DataObject in not supported!  DataObject can not be null!");
		   
		 //Encoding Variables for Portal
	     String client_var = StringUtil.encodeString("WDO4JSModel_"+ctx.getSchemaExportName(),_isFaces);
		 String model_var = StringUtil.encodeString("WDO4JSMR_"+ctx.getLastModelName(),_isFaces); 
		 String export_class_var = StringUtil.encodeString(eClassMap.getExport()+"Class",_isFaces);
		 String export_var = StringUtil.encodeString(eClassMap.getExport(),_isFaces);
		 String client_ecore_var = StringUtil.encodeString(eClassMap.getClientEcoreName(),_isFaces);
		 String schema_var=StringUtil.encodeString("WDO4JSModel_"+ctx.getSchemaExportName(),_isFaces);
		 String encoded_a=StringUtil.encodeString("A",_isFaces);
		 String encoded_r=StringUtil.encodeString("R",_isFaces);
		 String encoded_XT1=StringUtil.encodeString("XTOTOX1",_isFaces);
		 String encoded_XT2=StringUtil.encodeString("XTOTOX2",_isFaces);
		 
		
		 //findEClassMap(o);
		 boolean FirstTime = ctx.isFirstSchemaExport();
		 if (FirstTime == true)
		  {
		 	out.write("<script type=\"text/javascript\"> var "+encoded_a+"; var " + encoded_XT1+"="+encoded_a+";"+encoded_a+" =new ECreator().AddEAs; var "+encoded_r+"; var "+encoded_XT2+"="+encoded_r+"; "+encoded_r+"=new ECreator().AddERs;");
			out.write("var ");
			out.write(client_var); 
			out.write("=new EClass("); 
			out.write(model_var); 
			out.write("\");\n");
		  }

		 out.write("var "+export_class_var+"=new EClass(\""+export_var+"\","+(eClassMap.isDiffOnRefresh()?1:0)+");"); 
		 out.write(encoded_r+"("+client_var+")"); out.write(",[[\""+export_var+"\","+export_class_var+",0,-1,0,0]]);\n");
		 
		 FeatureExporter exporter = new FeatureExporter(eClassMap);
		 exporter.createSchemaString();		 
		 HashMap referenceTypes = exporter.getReferenceTypes();
		 Iterator it = referenceTypes.keySet().iterator();
		 while (it.hasNext())
		  {
		  	EClassMap ref = (EClassMap) it.next();
		  	EFeatureMap f = (EFeatureMap) referenceTypes.get(ref);
		  	Object refobj = ((DataObject)o).get(f.getEFeatureName());
		  	DataObject dobj = getDataObject(refobj);
		  	if (dobj != null)		  	  
			  super.ExportSchema(out, ctx, MediatorFactory.GenTypeKey(ref.getJavaComplexName(),KEY),dobj);
		  }
		  
		  out.write(exporter.getEAttributesSchema());
		  out.write(exporter.getEReferencesSchema());

		 if (FirstTime == true)
		  { 
		 	out.write("var "); 
		 	out.write(model_var); 
		 	out.write("=new XMILoader(");
		 	out.write(schema_var);
			out.write("); \n"); 
		 	out.write(", '"+export_var+")', '"+client_ecore_var+"');\n");
			out.write(encoded_a+"="+encoded_XT1+";"+encoded_r+"="+encoded_XT2+";</script>\n");
		  }
		}
		catch (Exception e)
		{
		  throw new ExportException(e.toString());
		}
	}

	/* (non-Javadoc)
	 * @see com.ibm.odcb.jrender.mediators.Mediator#Export(java.io.Writer, com.ibm.odcb.jrender.mediators.PageContext, java.lang.Object, java.lang.String, boolean)
	 */
	public void Export(
		Writer out,
		PageContext ctx,
		Object o,
		String var,
		boolean debugMode)
		throws ExportException {
		try
		{
         //Encoding Variables for Portal
		 String model_var = StringUtil.encodeString("WDO4JSMR_"+ctx.getLastModelName(),_isFaces); 
		 String export_class_var = StringUtil.encodeString(eClassMap.getExport()+"Class",_isFaces);
		 String export_var = StringUtil.encodeString(eClassMap.getExport(),_isFaces);
		 String _var = StringUtil.encodeString(var,_isFaces);
		 String encoded_c=StringUtil.encodeString("C",_isFaces);
		 String encoded_i=StringUtil.encodeString("I",_isFaces);
		 String encoded_XT1=StringUtil.encodeString("XTOTOX1",_isFaces);
		 String encoded_XT2=StringUtil.encodeString("XTOTOX2",_isFaces);
		
		 boolean firstTime = ctx.isFirstDataExport();
		 if (firstTime)
		 {
		 	out.write("<script type=\"text/javascript\"> var"+ encoded_c +";");
		 	out.write("var "+ encoded_XT1 +" = "+ encoded_c+";");
		 	out.write(encoded_c+"=new EFactory().create;");
		 	out.write("var" +encoded_i+ ";");
		 	out.write("var "+ encoded_XT2+"="+encoded_i+";"); 
		 	out.write(encoded_i+"=new ECreator().Init;\n");
		 }

		 if (o == null)
		  {
			out.write("var "+_var+" = null;");
			return;
		 }
		 DataObject obj = (DataObject) o;
		 //findEClassMap(o);
		 out.write("var "+var+" = "+encoded_c+"("+export_class_var+";");
		 Iterator it = eClassMap.getEFeatureMapIterator();
		 int i = -1;
		 StringBuffer eAttributesParams = new StringBuffer(128);
		 StringBuffer eReferencesParams = new StringBuffer(128);
		 while (it.hasNext() == true)
		  {
		   ++i;
		   EFeatureMap f = (EFeatureMap) it.next();
		   if ( !f.isCalculateAttribute() )
		   {
			if (f.getReferenceMap() != null)
			 {
			   if (!f.isMany() && !TypesUtil.isGeneralList(f.getGetterType()) )
				{
				  String refVar = Mediator.ExportData(out, ctx, obj.get(f.getEFeatureName()), MediatorFactory.GenTypeKey(f.getReferenceMap().getJavaComplexName(),KEY), debugMode);
				  eReferencesParams.append(","+  ((refVar == null || (refVar.length() == 0))  ? "null" : refVar) + (MEDIATOR_DEBUG_MODE == 1 ? " /*"+f.getExport()+"*/":""));
				  //EReferencesParams.append(HEADER+"\"+"+(I.hasNext() ? VarStr : "("+VarStr+".length() == 0 ? \"null\": "+VarStr+")"+(MEDIATOR_DEBUG_MODE == 1 ? " +\"/*"+ME.getExport()+"*/\"":"")));

				}
			   else
				{
 				  List l = obj.getList(f.getEFeatureName());
				  Mediator.ExportData(out, ctx, l, MediatorFactory.GenTypeKey(f.getReferenceMap().getJavaComplexName(),KEY), debugMode);
				  String refVar = ctx.PopArrayVar(MediatorFactory.GenTypeKey(f.getReferenceMap().getJavaComplexName(),KEY));
				  eReferencesParams.append(","+ refVar == null? "null" : "["+refVar+"]"+(MEDIATOR_DEBUG_MODE == 1 ? " /*"+f.getExport()+"*/":""));
				  //EReferencesParams.append(HEADER+"\"+("+VarStr+"==null?\"null\":\"[\"+"+VarStr+"+\"]\")"+(MEDIATOR_DEBUG_MODE == 1 ? " +\"/*"+ME.getExport()+"*/\"":""));

				}
			 }
			 else
			 {
				String GetType = f.getGetterType();
				char OutputType = TypesUtil.getOutputTypeFromJavaType(GetType);
				if (TypesUtil.isGeneralList(GetType) == true)  // Arrays, Collections, Lists, ArrayLists
				 {
				   List l = obj.getList(f.getEFeatureName());
				   //DataObject does not support Object[].  Only Lists?
				   //so, don't check for it
				   /*if (TypesUtil.isArrayOfBasicJavaType(GetType, true) == true) 
				   {					
					eAttributesParams.append(","+ Mediator.constructArrayString(l)+(MEDIATOR_DEBUG_MODE == 1 ? " /*"+f.getExport()+"":""));
				   }
				   else       */             
					eAttributesParams.append(","+ Mediator.constructArrayString(l,OutputType)+(MEDIATOR_DEBUG_MODE == 1 ? " /*"+f.getExport()+"*/":""));                      
				 }
				else  // This is not a list or array, so we are in single basic types, primitive or boxed
				 {
				   //!!!!! ################################################## !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!11
				   // LDH-NOTE: There is similar, but very different code in Mediator.FormatBasicTypeOutput. Keep in synch.
				   //          Need to look at a way to consolidate
				   //!!!!! ################################################## !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!11
				   if (OutputType == TypesUtil.TypeInfo._DATE_OUT)
					{
					  Date date = obj.getDate(f.getEFeatureName());
					  eAttributesParams.append(","+(date == null?-1:date.getTime())+(MEDIATOR_DEBUG_MODE == 1 ? " /*"+f.getExport()+"*/":""));
					}
				   else if (OutputType == TypesUtil.TypeInfo._CHAR_OUT)
					{ 
					  char c = obj.getChar(f.getEFeatureName());
					  eAttributesParams.append(","+c+(MEDIATOR_DEBUG_MODE == 1 ? " /*"+f.getExport()+"*/":""));
					}
				   else if (OutputType == TypesUtil.TypeInfo._STRING_OUT || OutputType == TypesUtil.TypeInfo._DEFAULT_OUT)
					{
					  Object s = obj.get(f.getEFeatureName());
					  // LDH-NOTE: when writing any object in Java, toString() is automatically involved when necesssary. 
					  //          We don't need to push it out, and most importantly, that allows us to deal with NULL 
					  //          pointers automatically as well. For this reason, DEFAULT and STRING cases have been
					  //          combined.
					  eAttributesParams.append(","+(s == null?"\"\"":com.ibm.odcb.jrender.misc.StringUtil.QuoteDoubleAndEscape(s,false)+(MEDIATOR_DEBUG_MODE == 1 ? " /*"+f.getExport()+"*/":"")));
					}
				   else
					{
					  Object s = obj.get(f.getEFeatureName());
					  eAttributesParams.append(","+s+(MEDIATOR_DEBUG_MODE == 1 ? " /*"+f.getExport()+"*/":""));
					}
				 }
			 }
		   }
		  }
		  
		 /*if (eAttributesParams.length() > 0)
		    eAttributesParams.setCharAt(0, ' ');*/
		  out.write(encoded_i+"("+_var+",[\""+_var+"\""+eAttributesParams.toString()+"],[");
		  if (eReferencesParams.length() > 0)
		   {
			 eReferencesParams.setCharAt(0, '[');
			 out.write(" "+eReferencesParams.toString());
		   }
		  out.write("]);\n");
		  	 		
		  if (firstTime == true)
		   {  
		  	 out.write(model_var); 
		   	 out.write(".Root.eAdd('"+export_var+"', "+_var+");"); 
			 out.write(encoded_c+"="+encoded_XT1+";"+encoded_i+"="+encoded_XT2+"; </script>\n"); 
		   }
		}
		catch (Exception e)
		{
			throw new ExportException(e.toString());
		}

	}

	/* (non-Javadoc)
	 * @see com.ibm.odcb.jrender.mediators.Mediator#ExportXMILoader(java.io.Writer, com.ibm.odcb.jrender.mediators.PageContext)
	 */
	public void ExportXMILoader(Writer out, PageContext ctx)
		throws ExportException {
		 
		//Encoding Variables for portal
		String model_var = StringUtil.encodeString("WDO4JSMR_"+ctx.getLastModelName(),_isFaces); 
		String export_class_var = StringUtil.encodeString(eClassMap.getExport()+"Class",_isFaces);
		String export_var = StringUtil.encodeString(eClassMap.getExport(),_isFaces);
		String schema_var = StringUtil.encodeString("WDO4JSModel_"+ctx.getSchemaExportName(),_isFaces);
		String client_ecore_var = StringUtil.encodeString(eClassMap.getClientEcoreName(),_isFaces);
		 		 
		try
		{
			try
			{
			  boolean FirstTime = ctx.isFirstSchemaExport();
			  StringBuffer sb = new StringBuffer();
			  if (FirstTime == true)
			  {
				sb.append("<script type=\"text/javascript\">")
				.append(model_var)
				.append("=new XMILoader(")
				.append(schema_var)
				.append(", ")
				.append(export_var)
				.append(", ")
				.append(client_ecore_var)
				.append("\n")
				.append("</script>\n");
				out.write(sb.toString());
			   }
			}
			catch (Exception e)
			{
			  e.printStackTrace();
			}		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

	}

/*/    private void findEClassMap(Object o) throws ExportException
    {
		DataObject obj = (DataObject) o;
		EClass ec = ((EObject) obj).eClass();
		EPackage	pkg = ec.getEPackage();
		WDO4JSMappings universe = MediatorFactory.getInstance().getUniverse();
		this.eClassMap = universe.getEClassMapByEPackageAndECoreComplexName(pkg.getName(), ec.getName());
    }*/
	public static void main(String[] args) {
	}


}
