/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//
//OCO Source Materials
//
//5724-E76 and 5724-E77
//
// Copyright IBM Corp.  2003
//
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////
package com.ibm.odcb.jrender.mediators;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;

import com.ibm.odcb.jrender.mediators.gen.MappingsCompiler;
import com.ibm.odcb.jrender.mediators.gen.WDO4JSMappings;
import com.ibm.odcb.jrender.mediators.gen.ecore.EClassMap;
import com.ibm.odcb.jrender.mediators.gen.ecore.EPackageMap;
import com.ibm.odcb.jrender.mediators.gen.generators.Generator;
import com.ibm.odcb.jrender.misc.EMFHelper;
import com.ibm.odcb.jrender.misc.Streamer;
import com.ibm.odcb.jrender.utilities.EMapGen;
////#IF# SDO
import commonj.sdo.DataObject;
////#END#

////#IF# WDO
//import com.ibm.etools.wdo.DataObject;
////#END#


public class GenericSDO4JSMediatorGen
 {
   private String         clientEcorename;
   private DataObject     obj;
   private WDO4JSMappings universe;
   private EPackage       pkg;
   private EPackageMap    epm;
   private static final String RELATIVE_EMAP_FILEPATH = "com/ibm/dynwdo4jsmediators/";
  
  public GenericSDO4JSMediatorGen(DataObject dataObj)
  {
     this.obj = dataObj;
  }

   public void gen()
    {
      Streamer.trace.Header().println("Enter GenericMediator.gen()");
      try
       {
         Streamer.status.Header().println("GM.gen(): Getting current Universe");
         universe = MediatorFactory.getInstance().getUniverse();
         Streamer.status.Header().println("GM.gen(): Loading Ecore for the DataObject into the current Universe");
         loadEcore();
         Streamer.status.Header().println("GM.gen(): Creating and Loading an identity EMap for the DataObject into the current Universe");
         loadEMap();
         Streamer.status.Header().println("GM.gen(): Verifying the Universe");
         boolean ok = universe.Verify();
         if (ok)
          {
            Streamer.status.Header().println("GM.gen(): Creating and Loading Client Ecore for the DataObject into the current Universe");
            loadClientEcore();
            Streamer.status.Header().println("GM.gen(): Exporting Schema for the DataObject");     
            registerMediators();
          }
       }
      catch (Exception e)
       {
         Streamer.error.Header().printStackTrace(e);
       }    
      Streamer.trace.Header().println("Exit GenericMediator.gen()");
    }

	private void loadClientEcore() 
	  {
		String s = MappingsCompiler.createClientEcore(epm);
		InputStream is = new ByteArrayInputStream(s.getBytes());
		MappingsCompiler.cacheClientEcore(epm, clientEcorename, is, universe);
	  }

	  private void loadEMap() 
	  {
		Map params = EMapGen.initParams(null, pkg);
		HashMap ecMaps = EMapGen.createEClassMaps(pkg,params,RELATIVE_EMAP_FILEPATH+pkg.getName()+".emap",null);
		//get client ecore name & instantialte EPackageMap
		if (ecMaps.values().iterator().hasNext())
		{
		  EClassMap m = (EClassMap) ecMaps.values().iterator().next();
		  clientEcorename = m.getClientEcoreName();
		  epm = new EPackageMap(pkg.getName());
		  epm.setRootEClassMapName(EMFHelper.getEClassName(this.obj));
		  epm.setEMapResourceName (m.getSourceEmapName());
		  epm.setEMapXMLFileName  (m.getXMLFileSource());
		  universe.addEPackageMap(epm, false);
		  epm.setUniverse(universe);
		}
		//add EClassMaps to universe
		Iterator I = ecMaps.values().iterator();
		while (I.hasNext() == true)
		 {
		   EClassMap m = (EClassMap) I.next();
		   universe.addEClassMap(epm, m, true);
		 }
	  }

	  private void loadEcore() 
	  {
		EClass ec = ((EObject) obj).eClass();
		pkg = ec.getEPackage();
		universe.addEPackage(pkg, true);
	  }

   private void registerMediators() 
    {
      Iterator it = universe.getEClassMapsForEPackageIterator(epm.getName());
      while (it.hasNext() == true)
       {
         EClassMap m = (EClassMap) it.next();
         try
          {
            for (int i = 0; i < Generator.MEDIATOR_KEYS.length; ++i)
             {
               String typeKey = MediatorFactory.GenTypeKey(m.getJavaComplexName(), Generator.GENS[i].getIdentifier());
               Streamer.debug.Header().println("Adding GenericSDO4JSMediator for Key '"+typeKey+"'.");
               MediatorFactory.getInstance().addMediator(typeKey, new GenericSDO4JSMediator(m));
             }
          }
         catch (Exception e)
          {
            Streamer.error.Header().printStackTrace(e);
          }
       }
    }

   public static boolean isGenericSDO4JSMediatorNeeded(String typeKey, Object o) throws ExportException
    {
      boolean ok = false;
      if (o instanceof DataObject)
       {
         MediatorFactory factory = MediatorFactory.getInstance();
         Mediator mediator = factory.getMediator(typeKey);
         if (mediator == null)
          ok = true;
       }
      return ok;
    }
  
  public static void main(String[] args) 
  {
  }
  
}