/////////////////////////////////////////////////////////////////////////
//  IBM Confidential
//  
//  OCO Source Materials
//  
//  PID xxxx-xxx
//  
//   Copyright IBM Corp.  2003
//  
//  The source code for this program is not published or other-
//  wise divested of its trade secrets, irrespective of what has
//  been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////


package com.ibm.odcb.jrender.mediators;

import java.io.ByteArrayInputStream;
import java.io.Writer;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;

import javax.faces.context.FacesContext;

import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.Resource;

import com.ibm.faces.renderkit.html_extended.HxClientRenderUtil;
import com.ibm.odcb.jrender.mediators.gen.ecore.EClassMap;
import com.ibm.odcb.jrender.mediators.gen.ecore.EPackageMap;
import com.ibm.odcb.jrender.misc.EMFHelper;
import com.ibm.odcb.jrender.misc.StringUtil;
import com.ibm.odcb.jrender.misc.TypesUtil;

////#IF# SDO
import commonj.sdo.DataObject;
////#END#

////#IF# WDO
//import com.ibm.etools.wdo.DataObject;
////#END#

/**
 * @author ldh
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public abstract class Mediator
 {
   public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;

   public static final String _MEDIATOR_SEPERATOR = "`";

   public Mediator(String VarBaseName, String ArrayVarDelimiter)
    {
      _VarBaseName = VarBaseName;
      _ArrayVarDelimiter = ArrayVarDelimiter;
    }
   protected String _VarBaseName;
   protected String _ArrayVarDelimiter;
   
   // Returns null to indicate that there is no unicity
   // returns "" to indicate that this is a singleton
   // returns a unique string that identifies this object logically (not necessarily physically).
   public abstract String getSignature(Object O, String modelName, boolean First, int ComplexPropertyCount);

   public abstract void   Export(Writer Out, PageContext Ctx, Object O)                                throws ExportException;
   public abstract void   Export(Writer Out, PageContext Ctx, Object O, String Var, boolean DebugMode) throws ExportException;
   public abstract void   ExportXMILoader(Writer Out, PageContext Ctx) throws ExportException;

   
   
   
   /////////////////////////////////////////////////////////////////////////////////////////////
   // Schema export
   /////////////////////////////////////////////////////////////////////////////////////////////   
   // This is the public entry point into the WDO4JS subsystem to export a Model/Schema out to a browser
   // Because this method can be called from multiple threads on the same context, we must make sure that 
   // the operation is properly synchronized.
   public static void ExportSchema(Writer Out, PageContext Ctx, String TypeKey, String ModelName, Object o)
   throws ExportException
    {
      synchronized(Ctx)
      {
        Ctx.setModelName(ModelName);         
        ExportSchema(Out, Ctx, TypeKey, o);
      }
    }

   // This method is kept for backward compatibility with previous Mediators generated.
//   protected static void ExportSchema(Writer Out, PageContext Ctx, String TypeKey)
//   throws ExportException
//    {
//      ExportSchema(Out,Ctx,TypeKey,null);
//    }

   protected static void ExportSchema(Writer Out, PageContext Ctx, String TypeKey, Object o)
   throws ExportException
    {
      MediatorFactory As = MediatorFactory.getInstance();
      Mediator A = As.getMediator(TypeKey);
      if (A == null)
       {
           throw new ExportException("WDO4JS Mediator for '"+TypeKey+"' not found");
       }
      if (Ctx.AddSchemaExport(TypeKey) == true)
       {
         //cache the model name exported per TypeKey, to be reused in subsequent
         //export of the same model
         Ctx.addExportedModelName(TypeKey, Ctx.getLastModelName());
         Ctx.ExportSchema(Out, A, o);
       }
      else
       {
         //export the XMILoaderInit statement here
         //since this model has already been exported, reuse it
         
         //if model name is the same, then dont even export XMILoader
      	 //Encoding: previosly this contained a call to the mediators ExportXMILoader method
     	 //This was to reduce duplication of data on the page if a model was being written out twice.
     	 //However individual models are required for two porlets to work on the page so now both models
     	 //are written out in their entirety with encoded variables.
         if ( !(Ctx.getExportedModelName(TypeKey).equals(Ctx.getLastModelName())) )
         {
        	Ctx.addExportedModelName(TypeKey, Ctx.getLastModelName());
            Ctx.ExportSchema(Out, A, o);
         }
       }      
    }

   /////////////////////////////////////////////////////////////////////////////////////////////
   // Data export
   /////////////////////////////////////////////////////////////////////////////////////////////
   // This is the public entry point into the WDO4JS subsystem to export a data instance graph out to a browser
   // Because this method can be called from multiple threads on the same context, we must make sure that 
   // the operation is properly synchronized.
   public static String ExportData(Writer Out, PageContext Ctx, Object O, String TypeKey, boolean DebugFlag)
   throws ExportException
    {
      if (O == null)
       return "null";
       
      // passing null is OK here because this method cannot be called unless some schema has been exported already
      MediatorFactory As = MediatorFactory.getInstance();
      Mediator A = As.getMediator(TypeKey);
      if (A == null)
       {
           throw new ExportException("WDO4JS Mediator for  '"+TypeKey+"' not found");
       }
      
      String Var;
      try
       { 
           synchronized(Ctx)
          {
            Var = Ctx.AddInstanceExport(O, A, DebugFlag, Ctx.getLastModelName());
            Ctx.ExportData(Out, O, Var, A, DebugFlag);
        }
       }
      catch (AlreadyOutputException X)
       {
         Var = X.getMessage();
       }
      return Var;
    }
  
   
   public static EClassMap getEClassMapForJavaComplexName(String BaseType, String ExtraName)
   throws ExportException
    {
      return getEClassMapForJavaComplexName(EClassMap.MakeComplexName(BaseType, ExtraName));
    }
   public static EClassMap getEClassMapForJavaComplexName(String JavaComplexName)
   throws ExportException
    {
      //call the internal APIs to get the reverse mapping;
      EClassMap m = MediatorFactory.getInstance().getUniverse().getEClassMapByJavaComplexName(JavaComplexName);
      if (m == null)
      {
         throw new ExportException("EClassMap for '"+JavaComplexName+"' not found");
      }
      return m;
    }
  
   public static String getExportID(PageContext Ctx, Object O, String TypeKey, String modelName)
   throws ExportException
    {
      // passing null is OK here because this method cannot be called unless some stuff has been exported already
      MediatorFactory As = MediatorFactory.getInstance();
      Mediator A = As.getMediator(TypeKey);
      if (A == null)
       {
        throw new ExportException("WDO4JS Mediator for  '"+TypeKey+"' not found");
       }
      return Ctx.getInstanceExport(O, A, modelName);
    }
   
   protected static void ExportData(Writer Out, PageContext Ctx, Collection ObjCollection, String TypeKey, boolean DebugData)
   throws ExportException
    {
      ExportData(Out, Ctx, (ObjCollection == null) ? null : ObjCollection.iterator(), TypeKey, DebugData);
    }

    protected static void ExportData(Writer Out, PageContext Ctx, Object[] ObjArray, String TypeKey, boolean DebugData)
    throws ExportException
     {
       ExportData(Out, Ctx, (ObjArray == null) ? null : Arrays.asList(ObjArray).iterator(), TypeKey, DebugData);
     }

   protected static void ExportData(Writer Out, PageContext Ctx, Iterator ObjList, String TypeKey, boolean DebugFlag)
   throws ExportException
    {
      Ctx.PushArrayType(TypeKey);

      if (ObjList == null)
       return;

      // passing null is OK here because this method cannot be called unless some schema has been exported already
      MediatorFactory As = MediatorFactory.getInstance();
      Mediator A = As.getMediator(TypeKey);
      if (A == null)
       {
           throw new ExportException("WDO4JS Mediator for class '"+TypeKey+"' not found");
       }

      while (ObjList.hasNext() == true)
       {
         String Var;
         try
          { 
            Object O = ObjList.next();
            Var = Ctx.AddInstanceExport(O, A, DebugFlag, Ctx.getLastModelName());
            Ctx.ExportData(Out, O, Var, A, DebugFlag);
          }
         catch (AlreadyOutputException X)
          {
            Var = X.getMessage();
          }
         Ctx.PushArrayVar(TypeKey, Var, A._ArrayVarDelimiter);
       }
     }

   public static String FormatBasicTypeOutput(Object Obj, char OutputType)
    {   
      //!!!!! ################################################## !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!11
      // LDH-NOTE: There is similar, but very different code in Object_to_WDO4JS_as_JS_compact.GenData. Keep in synch.
      //          Need to look at a way to consolidate
      //!!!!! ################################################## !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!11

      if (Obj == null)
       return "null";
       
      if (OutputType == TypesUtil.TypeInfo._DATE_OUT)
       {
         return ""+((Date)Obj).getTime();
       }
      if (OutputType == TypesUtil.TypeInfo._CHAR_OUT)
       {
         return "'"+Obj+"'";
       }
      if (OutputType == TypesUtil.TypeInfo._STRING_OUT || OutputType == TypesUtil.TypeInfo._DEFAULT_OUT)
       {
         return StringUtil.QuoteDoubleAndEscape(Obj.toString(), false);
       }

      return Obj.toString();
    }
   
   /**
   * This is a method to generate a JavaScript array of basic types based on a Java collection containing
    * Java basic types (Integer, Boolean etc...)
   * 
   * @return String Method returns array string, constructed properly in the right format
   */
   public static String constructArrayString(Iterator I, char OutputType)
    {
       if (I == null) return "null";   
       StringBuffer Str = new StringBuffer(128);
       while (I.hasNext() == true) 
        {
          Str.append(",").append(FormatBasicTypeOutput(I.next(), OutputType));
        }
       Str.append("]");
       Str.setCharAt(0, '[');
       return Str.toString();
    }
   public static String constructArrayString(Collection C, char OutputType)
    {
      return (C == null) ? "null" : constructArrayString(C.iterator(), OutputType);
    }
   
   /**
   * This is a convenience function to construct the target adapter array string as required by
   * the JavaScript function. Input: ODCSelectAdapterVar22; Output: ["ODCSelectAdapterVar22"]
   * If input is an array, output will be comma separated quotes enclosed JS array.
   * 
   * @return String Method returns array string, constructed properly in the right format
   */
   public static String constructArrayString(Object[] A, char OutputType)
    {
      if (A == null) return "null";     
       if (A.length == 0) return "[]";     
      StringBuffer Str = new StringBuffer(128);
      for (int i=0; i<A.length; i++)
       {
          Str.append(",").append(FormatBasicTypeOutput(A[i], OutputType));
       }
       Str.append("]");
       Str.setCharAt(0, '[');
       return Str.toString();
   }
   
   public static String constructArrayString(char[] A)
    {
       if (A == null) return "null";      
       if (A.length == 0) return "[]";     
       StringBuffer Str = new StringBuffer(A.length*2+2);
       for (int i=0; i<A.length; i++)    
        Str.append(",'").append(A[i]).append("'");
       Str.append("]");
       Str.setCharAt(0, '[');
       return Str.toString();
    }
   public static String constructArrayString(boolean[] A)
    {
       if (A == null) return "null";      
       if (A.length == 0) return "[]";     
       StringBuffer Str = new StringBuffer(A.length*2+2);
       for (int i=0; i<A.length; i++)    
        Str.append(",").append(A[i]);
       Str.append("]");
       Str.setCharAt(0, '[');
       return Str.toString();
    }
   public static String constructArrayString(double[] A)
    {
       if (A == null) return "null";      
       if (A.length == 0) return "[]";     
       StringBuffer Str = new StringBuffer(A.length*2+2);
       for (int i=0; i<A.length; i++)    
        Str.append(",").append(A[i]);
       Str.append("]");
       Str.setCharAt(0, '[');
       return Str.toString();
    }
   public static String constructArrayString(float[] A)
    {
       if (A == null) return "null";      
       if (A.length == 0) return "[]";     
       StringBuffer Str = new StringBuffer(A.length*2+2);
       for (int i=0; i<A.length; i++)    
        Str.append(",").append(A[i]);
       Str.append("]");
       Str.setCharAt(0, '[');
       return Str.toString();
    }
   public static String constructArrayString(int[] A)
    {
       if (A == null) return "null";      
       if (A.length == 0) return "[]";     
       StringBuffer Str = new StringBuffer(A.length*2+2);
       for (int i=0; i<A.length; i++)    
        Str.append(",").append(A[i]);
       Str.append("]");
       Str.setCharAt(0, '[');
       return Str.toString();
    }
   public static String constructArrayString(long[] A)
    {
       if (A == null) return "null";      
       if (A.length == 0) return "[]";     
       StringBuffer Str = new StringBuffer(A.length*2+2);
       for (int i=0; i<A.length; i++)    
        Str.append(",").append(A[i]);
       Str.append("]");
       Str.setCharAt(0, '[');
       return Str.toString();
    }
   public static String constructArrayString(short[] A)
    {
       if (A == null) return "null";      
       if (A.length == 0) return "[]";     
       StringBuffer Str = new StringBuffer(A.length*2+2);
       for (int i=0; i<A.length; i++)    
        Str.append(",").append(A[i]);
       Str.append("]");
       Str.setCharAt(0, '[');
       return Str.toString();
    }
   public static String constructArrayString(byte[] A)
    {
       if (A == null) return "null";      
       if (A.length == 0) return "[]";     
       StringBuffer Str = new StringBuffer(A.length*2+2);
       for (int i=0; i<A.length; i++)    
        Str.append(",").append(A[i]);
       Str.append("]");
       Str.setCharAt(0, '[');
       return Str.toString();
    }


   /**
   * This is a convenience function to construct the target adapter array string as required by
   * the JavaScript function. Input: ODCSelectAdapterVar22; Output: ["ODCSelectAdapterVar22"]
   * If input is an array, output will be comma separated quotes enclosed JS array.
   * 
   * @return String Method returns array string, constructed properly in the right format
   */
   public static String constructArrayStringAsXMI(String eAttributeName, Iterator it)
   {
      String arrayString = "";
      while (it.hasNext())   
      {
      Object obj = it.next();
      String s1 = ""+obj;
      if (obj instanceof java.lang.String)
      {
         s1 = StringUtil.QuoteDoubleAndEscape(s1,true);
         if (s1.length()>3)//strip out the quotes on the ends
           s1 = s1.substring(2, s1.length()-2);
      }      
      arrayString = arrayString.concat("\n<"+eAttributeName+">"+s1+"</"+eAttributeName+">");         
      }
      return arrayString;
   }
   
   /**
   * This is a convenience function to construct the target adapter array string as required by
   * the JavaScript function. Input: ODCSelectAdapterVar22; Output: ["ODCSelectAdapterVar22"]
   * If input is an array, output will be comma separated quotes enclosed JS array.
   * 
   * @return String Method returns array string, constructed properly in the right format
   */
   public static String constructArrayStringAsXMI(String eAttributeName, Object[] objArray)
   {
         if (objArray == null) return "";
      String arrayString = "";
      for (int i=0; i<objArray.length; i++)   
      {
      Object obj = objArray[i];
      String s1 = ""+obj;
      if (obj instanceof java.lang.String)
      {
         s1 = StringUtil.QuoteDoubleAndEscape(s1,true);
         if (s1.length()>3)//strip out the quotes on the ends
           s1 = s1.substring(2, s1.length()-2);
      }      
      arrayString = arrayString.concat("\n<"+eAttributeName+">"+s1+"</"+eAttributeName+">");         
      }
      return arrayString;
   }

   public static DataObject getDiffDataObject(String exportName, String xmlDiff)
   throws ExportException
   {
	DataObject o = null;
	try
	{
	  //If in portal environment, then try to get the view Id from the application map.
	  //If such a key exists, then remove all occurances of the encoding from the diff gram. 
	  FacesContext facesContext = FacesContext.getCurrentInstance();
	  boolean inPortal = HxClientRenderUtil.isPortal(facesContext.getExternalContext().getRequest());
	  if(inPortal)
	  {
			String viewId = facesContext.getExternalContext().getApplicationMap().get("PortalEncoding").toString();
			if(viewId != null && !viewId.equals(""))
			{
				xmlDiff = StringUtil.replaceAll(xmlDiff,viewId,"");
			}
	  }
	  EClassMap m = MediatorFactory.getInstance().getUniverse().getEClassMapByExport(exportName);
	  if (m == null)
	  {
		throw new ExportException("EClassMap for exported class '"+exportName+"' not found.");
	  }
       
	  EPackageMap epm = MediatorFactory.getInstance().getUniverse().getEPackageMap(m.getEPackageName());
	  if (epm == null)
	   {
		 throw new ExportException("Client Resource for EClassMap with export '"+exportName+"' in EPackage '"+m.getEPackageName()+"' not found.");
	   }
	  Resource pres = epm.getClientECoreResource();
	  ByteArrayInputStream is = new ByteArrayInputStream(xmlDiff.getBytes());
	  o = EMFHelper.loadDataObject(is, (EPackage) pres.getContents().get(0));
	}
	catch (Exception e)
	{
	  throw new ExportException(e.toString());
	}       
	 return o;
	}

 }
 
 
 
 
 
 
 
