/////////////////////////////////////////////////////////////////////////
//  IBM Confidential
//  
//  OCO Source Materials
//  
//  PID xxxx-xxx
//  
//   Copyright IBM Corp.  2003
//  
//  The source code for this program is not published or other-
//  wise divested of its trade secrets, irrespective of what has
//  been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////


package com.ibm.odcb.jrender.mediators;

import java.util.HashMap;
import java.util.Iterator;

import com.ibm.odcb.jrender.mediators.gen.MappingsCompiler;
import com.ibm.odcb.jrender.mediators.gen.WDO4JSMappings;
import com.ibm.odcb.jrender.mediators.gen.ecore.EClassMap;
import com.ibm.odcb.jrender.misc.Streamer;

////#IF# SDO
import commonj.sdo.DataObject;
////#END#

////#IF# WDO
//import com.ibm.etools.wdo.DataObject;
////#END#


/**
 * @author ldh
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class MediatorFactory
 {
   public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;
   
   protected static MediatorFactory _Instance;
   protected static Object  _Mutex = new Object();
   protected HashMap        _Mediators;
   protected WDO4JSMappings _Universe ;

   public static MediatorFactory getInstance() throws ExportException
    { 
      if (_Instance == null)
       {
       	 synchronized(_Mutex)
       	  {
            if ( _Instance == null)
             {
               // this method is called at runtime. As such, it must bypass the compilation step
               // Code that needs to generate and compile the artifacts should call GenMediators.
               MappingsCompiler MC = BuildMediators(false, false, null, null, null, null, false);
               if (MC.getUniverse().getError() != 0)
                throw new ExportException("Cannot create MediatorFactory Instance.  The universe failed verification.");
             }
          }
       }
      return _Instance;
    }

   protected MediatorFactory(WDO4JSMappings Universe, HashMap Mediators)
    {
      Init(Universe, Mediators);
    }
   protected void Init(WDO4JSMappings Universe, HashMap Mediators)
    {
      _Universe = Universe;
      _Mediators = Mediators == null ? new HashMap() : Mediators;
    }

   public static void GenMediators(boolean Compile, String BaseSourcePath, String BaseBuildPath, 
                                   String CompilerCommandLine, String EPackageName, 
                                   boolean ForceRegeneration)
    {
      BuildMediators(true, Compile, BaseSourcePath, BaseBuildPath, CompilerCommandLine,
                     EPackageName, ForceRegeneration);
    }

   protected static synchronized MappingsCompiler BuildMediators(boolean Gen, boolean Compile, 
                                                                 String BaseSourcePath, String BaseBuildPath, 
                                                                 String CompilerCommandLine,
                                                                 String EPackageName, boolean ForceRegeneration)
    {
      MappingsCompiler MC = null;
	   boolean OK = true;
      try
       {
         //String XXX = getRootPath();
         MC = new MappingsCompiler(BaseSourcePath, BaseBuildPath, CompilerCommandLine);
         //MC.Init(XXX+MEDIATOR_DIR, MEDIATOR_PACKAGE);
         MC.init();
         //we need to tell PrepareTheUniverse that it's for buildtime
         //so that it can save the XMLFileSource Path in EClassMaps.  We should not need this during runtime.
         OK = MC.PrepareTheUniverse(/*Compile*/);
         if (OK == true)
          OK = MC.VerifyTheUniverse();
         // Don't generate anything unless asked for it At runtime, this is called through the getInstance method with
         // compile == false.
         if (OK == true && Gen == true)
          { 
			OK = MC.createClientEcores();
			if (OK == true)
             OK = MC.CompileTheUniverse(Compile, EPackageName, ForceRegeneration);
		  }

		 if (OK == true && (Gen==false && Compile==false))
		  {
		    // This method could be called at "run-time", and invalidate the MediatorFactory that could have been cached by some 
		    // code. Here, we check if _Instance has already been set, in which case, we just re-init it.
		    if (_Instance == null)
		     _Instance = new MediatorFactory(MC.getUniverse(), MC.GetMediators());
		    else
			 _Instance.Init(MC.getUniverse(), MC.GetMediators());
			
			OK = MC.cacheClientEcores();
		  }
       }
      catch (Exception E)
       {
         Streamer.error.Header().println("Cannot create some or all mediators for this system");
         Streamer.error.Header().printStackTrace(E);
         OK=false;
       }
      
      if (OK)
       { 
         Streamer.status.Header().println("Mediator generation completed Successfully.");
       }
      else
       {
	     Streamer.error.Header().println("Mediator generation completed with errors.");
       }
      return MC;
    }

   ////////////////////////////////////////////////////////////////////////////////////////////////    
   // Methods to generate a TypeKey to find Mediators
   public static String GenTypeKey(String BaseType, String ExtraName, String ExportMode)
    {
      return EClassMap.MakeComplexName(BaseType, ExtraName)+Mediator._MEDIATOR_SEPERATOR+ExportMode;
    }
   public static String GenTypeKey(String ComplexName, String ExportMode)
    {
      return ComplexName+Mediator._MEDIATOR_SEPERATOR+ExportMode;
    }

   ////////////////////////////////////////////////////////////////////////////////////////////////
   // Methods to get a Mediator based on various combinations of parameters    
   protected Mediator getMediator(String BaseType, String ExtraName, String ExportMode)
    {
      return (Mediator) _Mediators.get(GenTypeKey(BaseType, ExtraName, ExportMode));
    }    
   protected Mediator getMediator(String ComplexName, String ExportMode)
    {
      return (Mediator) _Mediators.get(GenTypeKey(ComplexName, ExportMode));
    }
   protected Mediator getMediator(String TypeKey)
    {
      return (Mediator) _Mediators.get(TypeKey);
    }

   ////////////////////////////////////////////////////////////////////////////////////////////////
   
   //deprecated - replaced by getMediator_byTypeKey
   public static Mediator getMediator_OnlyUseIfYouKnowWhatYouAreDoing(String ComplexName, String ExportMode)
   throws ExportException
    {
      String TypeKey = MediatorFactory.GenTypeKey(ComplexName, ExportMode);
      Mediator M = (Mediator) getInstance()._Mediators.get(TypeKey);
      if (M == null)
       {
         Streamer.error.Header().println("Cannot find the mediator for key "+TypeKey);
       }
      return M;
    }
    
   public static Mediator getMediator_byTypeKey(String ComplexName, String ExportMode)
   throws ExportException
    {
      String TypeKey = MediatorFactory.GenTypeKey(ComplexName, ExportMode);
      Mediator M = (Mediator) getInstance()._Mediators.get(TypeKey);
      if (M == null)
       {
         Streamer.error.Header().println("Cannot find the mediator for key "+TypeKey);
       }
      return M;
    }
   
    protected void addMediator(String typeKey, Mediator mediator)
    {
    	_Mediators.put(typeKey, mediator);
    }

   protected void printMediatorList()
    {
      Streamer.debug.Header().println("Printing out the list of Mediators in the factory");
      Iterator I = _Mediators.keySet().iterator();
      while (I.hasNext() == true)
       {
         String Key = (String) I.next();
         Streamer.debug.Header().println("   "+Key);
       }
    } 
   
   public WDO4JSMappings getUniverse()
    {
      return _Universe;
    }
    
    public static void initGenericMediator(String modelTypeKey, String dataTypeKey, Object o) throws ExportException
    {
		boolean generic = GenericSDO4JSMediatorGen.isGenericSDO4JSMediatorNeeded(modelTypeKey, o);
		if (!generic)
		  generic = GenericSDO4JSMediatorGen.isGenericSDO4JSMediatorNeeded(dataTypeKey, o);
		if (generic)
		{
			GenericSDO4JSMediatorGen mGen = new GenericSDO4JSMediatorGen((DataObject)o);
			mGen.gen();
		}
    }
    
/*   protected static String getRootPath()
   {
     String XXX = ResourceLocator.GetRootPathFromMarkerClassName(MEDIATOR_PACKAGE+".MARKER_ODCB");
     String PossibleEnding1 = MEDIATOR_DIR  +"MARKER_ODCB"+".class";
     String PossibleEnding2 = MEDIATOR_DIR_2+"MARKER_ODCB"+".class";
     if (XXX.endsWith(PossibleEnding1) == true || XXX.endsWith(PossibleEnding2) == true)
     {
       XXX = XXX.substring(0, XXX.length()-PossibleEnding1.length());
     }
     return XXX;    
   }*/

 }
