/////////////////////////////////////////////////////////////////////////
//  IBM Confidential
//  
//  OCO Source Materials
//  
//  PID xxxx-xxx
//  
//   Copyright IBM Corp.  2003
//  
//  The source code for this program is not published or other-
//  wise divested of its trade secrets, irrespective of what has
//  been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////


package com.ibm.odcb.jrender.mediators;

import java.io.Writer;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import com.ibm.odcb.jrender.misc.IntWrapper;

/**
 * @author ldh
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class PageContext
 {
   public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;
   
   	public PageContext()
   	 {
   	 }

   protected String _ModelName;
   protected String _SchemaExportName;
   protected HashMap _ExportedModelNames = new HashMap();
   protected boolean exportDateEDataType = false;
   public boolean isExportDateEDataType()
   {
   	return exportDateEDataType;
   }
   public void setExportDateEDataType(boolean dateEDataType)
   {
	exportDateEDataType = dateEDataType;
   }
   public String getExportedModelName(String typeKey)
   {
   	return (String) _ExportedModelNames.get(typeKey);
   }
   public void addExportedModelName(String typeKey, String modelName)
   {
	 _ExportedModelNames.put(typeKey, modelName);
   }
   public String getSchemaExportName()
   {
   	return _SchemaExportName;
   }
   public void setSchemaExportName(String schemaName)
   {
	_SchemaExportName = schemaName;
   }
   public void setModelName(String ModelName)
   throws ExportException
    {
      if (_SchemaExportCount != 0)
       {
         throw new ExportException("Trying to set a model name '"+ModelName+"' in the middle of a model export '"+_ModelName+"'. Model names can only be set before a model is exported.");
       }
      _ModelName = ModelName;
    }
   public String getLastModelName()
    {
      return _ModelName;
    }
    
   protected int _SchemaExportCount = 0;
   public boolean isFirstSchemaExport() { return _SchemaExportCount == 1; }

   public void ExportSchema(Writer out, Mediator a, Object o) throws ExportException
    {
      if (_ModelName == null)
       {
         throw new ExportException("Trying to export a model without a name. PageContext.setModel() was not called properly.");
       }
      ++_SchemaExportCount;
	  //set the name of the schema exported to be shared for subsequent export of the same model
      if (isFirstSchemaExport())
      {
		_SchemaExportName = _ModelName;
      }
      a.Export(out, this, o);
      --_SchemaExportCount;
    }

   protected int _DataExportCount = -1;
   public boolean isFirstDataExport() { return _DataExportCount == 0; }
   
   public void ExportData(Writer Out, Object O, String Var, Mediator A, boolean DebugMode) throws ExportException
    {
      ++_DataExportCount;
      if (A == null)
       {
         throw new ExportException("Export called with a null Mediator !!!");
       }
      A.Export(Out, this, O, Var, DebugMode);
      --_DataExportCount;
    }
    
   public void ExportXMILoader(Writer Out, Mediator A) throws ExportException
   {
	if (_ModelName == null)
	 {
	   throw new ExportException("Trying to export a model without a name. PageContext.setModel() was not called properly.");
	 }
	++_SchemaExportCount;
	A.ExportXMILoader(Out, this);
	--_SchemaExportCount;   	
   }

   /////////////////////////////////////////////////////////////////////////////
   // Keep track of the exported schemas for the page 	 
   	protected ArrayList _ExportedSchemas = new ArrayList(10);
 	
   	public boolean AddSchemaExport(String X)
   	 {
   	   synchronized(_ExportedSchemas)
   	    {
      	  for (int i = 0; i < _ExportedSchemas.size(); ++i)
   	       {
   	         String Name = (String) _ExportedSchemas.get(i);
   	         if (X.equals(Name)==true)
 	            return false;
 	         }
 	        _ExportedSchemas.add(X);
   	      return true;
 	      }
   	 }

   /////////////////////////////////////////////////////////////////////////////
   // Keep track of the variable IDs exported    
   protected IntWrapper _InstanceCount = new IntWrapper(0);
   
   public int NextInstanceCount()
    {
      synchronized(_InstanceCount)
       {
         _InstanceCount.increment();
         return _InstanceCount.getValue();
       }
    }
   
   ///////////////////////////////////////////////////////////////////////////////////////////
   // Keeps track of the objects exported with their uniquely identifying signature,
   // and the variable they were exported under
   protected HashMap _ExportedInstances = new HashMap(500);
   
   public String AddInstanceExport(Object O, Mediator A, boolean DebugFlag, String modelName)
   throws AlreadyOutputException
    {
      String Signature = A.getSignature(O, modelName, true, 0);
//      Streamer.debug.Header().println("Signature is: "+Signature);
      String Var = getInstanceExport(Signature);
      if (Var != null)
       {
//         Streamer.status.Header().println("Signature has been found. Object already output");
       	 throw new AlreadyOutputException(Var);
       }
//      Streamer.status.Header().println("Signature has not been found. Object has not been output already");
      Var = (DebugFlag ? A._VarBaseName : "_")+NextInstanceCount();
      _ExportedInstances.put(Signature, Var);
      return Var;
    }
   public String getInstanceExport(Object O, Mediator A, String modelName)
    {
      return (String) _ExportedInstances.get(A.getSignature(O, modelName, true, 0));
    }
   public String getInstanceExport(String Signature)
    {
      return (String) _ExportedInstances.get(Signature);
    }

   ///////////////////////////////////////////////////////////////////////////////////////////
   // Keeps track of instance data exported in an arrat context
   protected ArrayList _ArrayInstances = new ArrayList(20);
   protected int _ArrayInstanceIndex = -1;
   static protected class ArrayInstanceSpec
    {
      public ArrayInstanceSpec(String ElementType)
       {
       	 Init(ElementType);
       }
      public void Init(String ElementType)
       {
       	 _ElementType = ElementType;
       	 _ElementVar = null;
       }
      public String _ElementType;
      public StringBuffer _ElementVar; // = new StringBuffer(128);
    }
    
   public void PushArrayType(String ElementType)
    {
      ++_ArrayInstanceIndex;
      if (_ArrayInstanceIndex >= _ArrayInstances.size())
       {
       	 _ArrayInstances.add(null);
       }
      ArrayInstanceSpec A = (ArrayInstanceSpec) _ArrayInstances.get(_ArrayInstanceIndex);
      if (A == null)
       {
         A  = new ArrayInstanceSpec(ElementType);
         _ArrayInstances.add(_ArrayInstanceIndex, A);
       }
      else
       {
         A.Init(ElementType);
       }
    }
   public void PushArrayVar(String ElementType, String ElementVar, String VarSeparator)
   throws ExportException
    {
      for (int i = _ArrayInstanceIndex; i >= 0; --i)
       {
       	 ArrayInstanceSpec A = (ArrayInstanceSpec) _ArrayInstances.get(i);
       	 if (A._ElementType.equals(ElementType) == true)
       	  {
            if (A._ElementVar == null) A._ElementVar = new StringBuffer(127);
       	    A._ElementVar.append(VarSeparator).append(ElementVar);
       	    return;
       	  }
       }
      throw new ExportException("Cannot find an array definition of type '"+ElementType+"' to add the variable '"+ElementVar+"' to.");
    }
   public String PopArrayVar(String ElementType)
    {
      for (int i = _ArrayInstanceIndex; i >= 0; --i)
       {
       	 ArrayInstanceSpec A = (ArrayInstanceSpec) _ArrayInstances.get(i);
       	 if (A._ElementType.equals(ElementType) == true)
       	  {
       	  	--_ArrayInstanceIndex;
            if (A._ElementVar != null && A._ElementVar.length() != 0)
             {
       	       return A._ElementVar.substring(1);
             }
            return "";
       	  }
       }
      return null;
    }
    
   // Additional "context" storage for applications that use the context
   protected HashMap _ContextVariables = new HashMap();

   // Returns null is successful, otherwise, returns back the Value object
   public Object addContextVariable(String Name, Object Value)
    {
      Object O = _ContextVariables.put(Name, Value);
      return (O != null) ? _ContextVariables.put(Name, O) : null;
    }
   public Object getContextVariable(String Name)
    {
      return _ContextVariables.get(Name);
    }
   public Object RemoveContextVariable(String Name)
    {
      return _ContextVariables.remove(Name);
    }

   /**
    * To store the EClass export names already output as schema.  Used to make sure each EClass schema is output only once.
    */  
   protected ArrayList _ExportedSchemaList = new ArrayList(10);
   public void addToExportedSchemaList(String exportName)
   {
     _ExportedSchemaList.add(exportName);
     _TotalSchemasToExport--;
   }
   
   public boolean isSchemaExported(String exportName)
   {
     return _ExportedSchemaList.contains(exportName);
   }   
   
   public Iterator getExportedSchemaListIterator()
   {
     return _ExportedSchemaList.iterator();
   }
   
   public int getExportedSchemaCount()
   {
     return _ExportedSchemaList.size();
   }
   
   //to keep tract of the total number of schemas to export for an ecore file
  protected int _TotalSchemasToExport = 0;
  
  public int getTotalSchemasToExport()
  {
    return _TotalSchemasToExport;
  }
  
  public void setTotalSchemasToExport(int totalSchemaExports)
  {
    _TotalSchemasToExport = totalSchemaExports;
  }

  //to prevent the diff eclassifiers from being exported multiple times
  protected boolean _DiffExported = false;

  /**
   * @deprecated
   * Returns the diffExported.
   * @return boolean
   */
  public boolean isDiffExported()
  {
    return _DiffExported;
  }

  /**
   * @deprecated
   * Sets the diffExported.
   * @param diffExported The diffExported to set
   */
  public void setDiffExported(boolean diffExported)
  {
    this._DiffExported = diffExported;
  }

 }
