/////////////////////////////////////////////////////////////////////////
//  IBM Confidential
//  
//  OCO Source Materials
//  
//  PID xxxx-xxx
//  
//   Copyright IBM Corp.  2003
//  
//  The source code for this program is not published or other-
//  wise divested of its trade secrets, irrespective of what has
//  been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////


/*
 * ObjectCompiler.java
 *
 * Created on November 20, 2000, 10:20 PM
 */

package com.ibm.odcb.jrender.mediators.gen;

import java.io.*;
import java.util.*;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;

import com.ibm.odcb.jrender.mediators.*;
import com.ibm.odcb.jrender.mediators.gen.ecore.EClassMap;
import com.ibm.odcb.jrender.mediators.gen.ecore.EPackageMap;
import com.ibm.odcb.jrender.mediators.gen.generators.*;
import com.ibm.odcb.jrender.misc.CompilerUtil;
import com.ibm.odcb.jrender.misc.ODCClassLoader;
import com.ibm.odcb.jrender.misc.Streamer;
import com.ibm.odcb.jrender.misc.EMFHelper;
import com.ibm.odcb.jrender.misc.Config;

public class MappingsCompiler extends java.lang.Object
 {
   public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;

   public boolean cacheClientEcores()
    {
      Streamer.status.println().Header().println("Caching Client Ecore Resources");
      try
       {
         Iterator pkgIt = _Universe.getEPackageMapsIterator();
         while (pkgIt.hasNext())
          {
            EPackageMap EPM = (EPackageMap) pkgIt.next();
            Iterator it = _Universe.getEClassMapsForEPackageIterator(EPM.getName());
            String clientEcoreName = null;
            InputStream is = null;
            if (it.hasNext())
             {
               EClassMap M = (EClassMap) it.next();
               clientEcoreName = M.getClientEcoreName();
               is = ODCClassLoader.getResourceAsStream(clientEcoreName);
               MappingsCompiler.cacheClientEcore(EPM, clientEcoreName, is, _Universe);
             }      
          }
       }
      catch (Exception e)
       {
         Streamer.error.Header().println("Caching of Some Client Ecore Resources Failed");
         Streamer.error.Header().printStackTrace(e); 
         return false;         
       }
      
      Streamer.status.println().Header().println("Finished Caching Client Ecore Resources");
      return true;
    }

   public boolean createClientEcores()
   {
     boolean OK = true;
     Streamer.status.println().Header().println("Generating Client Ecore Files");
     Iterator I = _Universe.getEPackageMapsIterator();
     while (I.hasNext() == true)
      {
        PrintWriter out = null;
        EPackageMap EPM = (EPackageMap) I.next();
        String fileName = null;
        try
         {
           Iterator it = _Universe.getEClassMapsForEPackageIterator(EPM.getName());
           //get the fileName, then reuse static createClientEcore() method
           if (it.hasNext())
            {
              EClassMap M = (EClassMap) it.next();
              fileName = M.getClientEcoreName();
              String fullFileName = MakeFullPathForResource(_BaseSourcePath, fileName);
              File F = new File(fullFileName);
              new File(F.getParent()).mkdir();
              out = new PrintWriter(new FileOutputStream(F));
              Streamer.debug.Header().println("Generating file "+fullFileName);
                //ClientECoreGenerator.GenIntro(out, M, "", "\n", "\"");
             //output Date EDataType as a workaround for WDODate not loading
             //ClientECoreGenerator.genDateEDataType(out,"","\n","\"","WDODate");
              //ClientECoreGenerator.GenEClassifier(out, M, "", "\n", "\"");
              String s = MappingsCompiler.createClientEcore(EPM);
              out.write(s);
            }
           //it = map.values().iterator();
         //ClientECoreGenerator.GenOutro(out, it, "", "\n", "\"");
           if (out != null)
            { 
              out.close();
            }
           //copy generated file to build path
           OK = copyClientEcoreToBuildPath(fileName);
         }
        catch (Exception E)
         {
           if (out != null)
            { 
              out.close();
            }
           OK = false;
           Streamer.error.Header().println("Some client Ecore File generation failed");
           Streamer.error.Header().printStackTrace(E);
         }
      }
     Streamer.status.Header().println("Finished Generating Client Ecore Files"); 
     return OK;
   }

   protected MappingGen     _G  ;   
   protected WDO4JSMappings _Universe = new WDO4JSMappings();
   protected EcoreLoader    _ELoader;
   protected MappingsParser _Xml;
   protected boolean        _VerificationStatus = false;
   protected boolean        _GenerationStatus   = false;
   protected boolean        _ComplilationStatus = false;
   protected String         _BaseSourcePath;
   protected String         _BaseBuildPath ;
   protected String         _CompilerCommandLine;
      
   public MappingsCompiler(String BaseSourcePath, String BaseBuildPath, String CompilerCommandLine)
    {
      _BaseSourcePath = BaseSourcePath;
      _BaseBuildPath  = BaseBuildPath ;
      _CompilerCommandLine = CompilerCommandLine;
    }
   
   public boolean init()
   {
      _G   = new MappingGen();
      _Xml = new MappingsParser(_Universe);
      _ELoader = new EcoreLoader( _Universe );
      return true;
   }

   public WDO4JSMappings getUniverse() { return _Universe; }

   protected static class Toto implements FileFilter
    {
      public boolean accept(File TheFile)
       {
         return  TheFile.getPath().endsWith(".emap");
       }
    }
    
   protected static class EcoreFileFilter implements FileFilter
    {
      public boolean accept(File TheFile)
       {
         return  TheFile.getPath().endsWith(".ecore");
       }
    }
    
   protected static class DiffFileFilter implements FileFilter
    {
      public boolean accept(File TheFile)
       {
         return  TheFile.getPath().endsWith(".diff");
       }
    }

   public boolean PrepareTheUniverse()
   throws Exception
    {
       boolean OK = true;
       OK = loadEcores();
       OK = parseMappings();
       return OK;
    }
  
    public boolean loadEcores() throws Exception
    {
      boolean OK = true;

      String resNames = Config.getString(Config.PROP_ECORE_FILES);
      if (resNames == null || resNames.length() == 0)
       {
         Streamer.warning.Header().println("The config property: ECORE_FILES_PROPERTY is either not set or does not contain any ecore file names");
         return true;
       }

      final String ECORE_EXT=".ecore";
      StringTokenizer tok = new StringTokenizer(resNames, " ");
      while (tok.hasMoreTokens())
      {
        String resName = tok.nextToken().replace('\\', '/');
        if (resName.endsWith(ECORE_EXT) ==false)
         {
           String Msg = "The resource '"+resName+"' listed in the ECORE_FILES property is not a proper ecore file. It does not end in '.ecore'.";
           Streamer.error.Header().println(Msg);
           throw new Exception(Msg);
         }
      OK = _ELoader.load(resName);
      }
      Streamer.status.Header().println("All .ecore files were processed.");
      if (OK == false)
       {
         Streamer.error.Header().println("Some .ecore files failed.");     
       }
       
      return OK;
    }

   public boolean parseMappings(/*boolean buildTime*/)
   throws Exception
    {
      boolean OK = true;

      String resNames = Config.getString(Config.PROP_EMAP_FILES);
      if (resNames == null || resNames.length() == 0)
       {
         Streamer.warning.Header().println("The config property: EMAP_FILES_PROPERTY is either not set or does not contain any emap file names");
         return true;
       }

      final String EMAP_EXT=".emap";
      StringTokenizer tok = new StringTokenizer(resNames, " ");
      while (tok.hasMoreTokens())
      {
        String resName = tok.nextToken().replace('\\', '/');
        if (resName.endsWith(EMAP_EXT) ==false)
         {
           String Msg = "The resource '"+resName+"' listed in the EMAP_FILES property is not a proper emap file. It does not end in '.emap'.";
           Streamer.error.Header().println(Msg);
           throw new Exception(Msg);
         }
        InputStream is = ODCClassLoader.getResourceAsStream(resName);
        if (is != null)
        {
          Streamer.status.Header().println("Parsing EMap file: " + resName);
          _Xml.setXmlFileSource(MakeFullPathForResource(_BaseSourcePath, resName));
          _Xml.setSourceEmapName(resName);
          OK = _Xml.parse(is);
          Streamer.status.Header().println("Finished Parsing EMap file: " + resName + (OK ? " without any errors" : ".  There were some errors.") );
          is.close();
        }
        else
        {
          Streamer.error.Header().println("Error while getting InputStream for '" + resName + "'.  Please make sure that the file name and location is correct and the file exists.  The filename is case sensitive.");
        }
      }
      Streamer.status.Header().println("All .emap files were processed.");
      if (OK == false)
       {
         Streamer.error.Header().println("Some .emap files failed.");     
       }
       
      return OK;
    }   
     
   public boolean VerifyTheUniverse()
   throws Exception
    {
      _VerificationStatus = false;
      if (_Universe.Verify() == false)
       {
         Streamer.error.println().Header().println("Some Ecore/Mappings failed verification.");
       }
      else
       {
         _VerificationStatus = true;
       }
      return _VerificationStatus;
    }
   public boolean getVerificationStatus()
    {
      return _VerificationStatus;
    }

   public boolean CompileTheUniverse(boolean Compile, String EPackageName, boolean ForceRegeneration)
   throws Exception
    {
      _GenerationStatus   = false;
      _ComplilationStatus = false;

      Streamer.status.println().Header().println("Generating mediators now.");      
      if (_G.GenMediators(_Universe, EPackageName, ForceRegeneration) == false)
       {
         Streamer.error.Header().println("Some generations failed.");
         return false;
       }
      _GenerationStatus = true;
      if (Compile == false)
       {
         Streamer.status.println().Header().println("Skipping the compilation phase. Mediators have been generated, but not compiled.");      
         return true;
       }
       
      Iterator I = _Universe.getEClassMapsIterator();
      HashMap Dirs = new HashMap();
      Streamer.status.println().Header().println("Compiling the mediators now.");      
      while (I.hasNext() == true)
       {
         EClassMap M = (EClassMap) I.next();
         String SubDir = M.getXMLFileSource();
         if (SubDir == null || SubDir.trim().length() == 0)
          {
            Streamer.error.Header().println("Could not obtain the XMLFileSource of EClassMap '" + M.getExport() + "'");
            return false;
          }
         SubDir = SubDir.substring( 0, SubDir.lastIndexOf(".emap") );
         if (Dirs.put(SubDir, SubDir) == null)
          {
            final String[] Files = { "*.java" };
            if (CompilerUtil.Compile(SubDir, _BaseBuildPath, Files, _CompilerCommandLine) == false)
             {
               Streamer.error.Header().println("Could not compile the generated mediators in "+Files[0])
                             .Header().println("This is a severe error!");
               return false;
             }
          }
       }
      _ComplilationStatus = true;
      return true;
    }
   public boolean getGenerationStatus()
    {
      return _GenerationStatus;
    }
   public boolean getCompilationStatus()
    {
      return _ComplilationStatus;
    }
    
   public HashMap GetMediators()
    {
      Streamer.status.println().Header().println("Instanciating the WDO4JS mediators");      
      HashMap Mediators = new HashMap(57);
      Iterator I = _Universe.getEClassMapsIterator();
      while (I.hasNext() == true)
       {
         EClassMap M = (EClassMap) I.next();
         String ClassName= null;
         Mediator A;
         String Key = null;
         try
          {
            for (int i = 0; i < Generator.MEDIATOR_KEYS.length; ++i)
             {
               Key = MediatorFactory.GenTypeKey(M.getJavaComplexName(), Generator.GENS[i].getIdentifier());
               String pkg = M.getClientEcoreName();
               pkg = (pkg.substring(0, pkg.lastIndexOf("/"))).replace('/','.');
               ClassName = pkg+"."+M.getEPackageName()+"_"+M.getExport()+Generator.GENS[i].getIdentifier();
               Object Obj = ODCClassLoader.getClassInstance(ClassName);
               Streamer.debug.Header().println(Obj.getClass().getName());
               A = (Mediator) ODCClassLoader.getClassInstance(ClassName);
               Streamer.debug.Header().println("Adding a mediator '"+ClassName+"' for Key '"+Key+"'.");
               Mediators.put(Key, A);
             }
          }
       catch (ClassNotFoundException E)
        {
         Streamer.warning.Header().println("Cannot create an instance of the WDO4JS Mediator '"+ClassName+"' for Key '"+Key+"'.");
        }
         catch (Exception E)
          {
            Streamer.error.Header().println("Cannot create an instance of the WDO4JS Mediator for '"+ClassName+"'.");
            Streamer.error.Header().printStackTrace(E);
          }
       }
      return Mediators;
    }

   public static String createClientEcore(EPackageMap EPM)
    {
      StringBuffer sb = new StringBuffer();
      Streamer.status.println().Header().println("Generating Client Ecore String");
      String fileName = null;
      try
       {
         boolean first = true;
         Iterator it = EPM.getEClassMapsByJavaComplexName().values().iterator();
         while (it.hasNext())
          {
            EClassMap m = (EClassMap) it.next();
            if(first)
             {
               fileName = m.getClientEcoreName();
               Streamer.debug.Header().println("Generating Client Ecore: "+fileName);
               ClientECoreGenerator.genIntro(sb, m, "", "\n", "\"");
               //output Date EDataType as a workaround for WDODate not loading
               ClientECoreGenerator.genDateEDataType(sb,"","\n","\"","WDODate");
               first = false;
             }
            ClientECoreGenerator.genEClassifier(sb, m, "", "\n", "\"");
          }
         it = EPM.getEClassMapsByJavaComplexName().values().iterator();
         ClientECoreGenerator.genOutro(sb, it, "", "\n", "\"");
       }
      catch (Exception e)
       {
         Streamer.error.Header().println("Client Ecore String generation failed");
         Streamer.error.Header().printStackTrace(e);
       }

      Streamer.status.Header().println("Finished Generating Client Ecore String"); 
      return sb.toString();
    }
     
   public boolean copyClientEcoreToBuildPath(String ecoreResourceName)
    {
      if (ecoreResourceName != null)
       {
         String sourceFile = MakeFullPathForResource(_BaseSourcePath, ecoreResourceName);
         String targetFile = MakeFullPathForResource(_BaseBuildPath , ecoreResourceName);
         //copy only if source location and build location are different
         if (!sourceFile.equals(targetFile))
          {
            Streamer.debug.Header().println("Copying client ecore file '"+sourceFile+"' to build path '"+targetFile+"'");
            String content = EMFHelper.ReadFileIntoString(sourceFile);
            FileWriter writer = null;
            try
             {
               writer = new FileWriter(targetFile);
               writer.write(content);
               writer.close();
             }
            catch (IOException e)
             {
               try
                {
                  if (writer != null)
                   writer.close();
                }
               catch (IOException ex){}
             }
            finally
             {
               try
                {
                  if (writer != null)
                   writer.close();
                }
               catch (IOException ex){}
             }
          }
       }
      return true;
    }
    
   public static boolean cacheClientEcore(EPackageMap EPM, String ClientECoreName, InputStream is, WDO4JSMappings universe)
    {
      Streamer.status.println().Header().println("Caching Client Ecore Resource: " + ClientECoreName);
      try
       {
         if (is == null)
          {
            Streamer.error.Header().println("Input Stream for resource '"+ClientECoreName+"' is null");
            return false;
          }
         Resource res = EMFHelper.getResource(is, URI.createURI(ClientECoreName));
         Streamer.status.println().Header().println("Got Resource for Client Ecore '" + ClientECoreName + "': " + res);
         if (res == null)
          {
            Streamer.error.Header().println("Loading of Client Ecore Resource'" + ClientECoreName + "' Failed.  The Resource is NULL.");
            return false;
          }
         else
          {
            Streamer.status.println().Header().println("Caching Client Ecore Resource '" + ClientECoreName + "'");
            EMFHelper.initPackagesForDataObject(res);
            EPM.setClientECoreResource(res);
            universe.addClientEcoreResource(res);
          }
       }
      catch (Exception e)
       {
         Streamer.error.Header().println("Caching of Client Ecore Resource for '"+ ClientECoreName +"' Failed");
         Streamer.error.Header().printStackTrace(e);   
         return false;       
       }

      Streamer.status.println().Header().println("Finished Caching Client Ecore Resources");
      return true;
    }

    public static String MakeFullPathForResource(String BasePath, String ResourcePath)
     {
       if (BasePath == null)
        {
          BasePath=File.separator;
        }
       else
        {
          BasePath = BasePath.replace(File.separatorChar=='\\'?'/':'\\', File.separatorChar);
          if (BasePath.endsWith(File.separator) == false)
           {
             BasePath+= File.separatorChar;
           }
        }
       return BasePath+ResourcePath.replace(File.separatorChar=='\\'?'/':'\\', File.separatorChar);
     }
 }
