/////////////////////////////////////////////////////////////////////////
//  IBM Confidential
//  
//  OCO Source Materials
//  
//  PID xxxx-xxx
//  
//   Copyright IBM Corp.  2003
//  
//  The source code for this program is not published or other-
//  wise divested of its trade secrets, irrespective of what has
//  been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////


package com.ibm.odcb.jrender.mediators.gen;


import java.io.*;

import com.ibm.odcb.jrender.mediators.gen.ecore.*;
import com.ibm.odcb.jrender.misc.SaxBase;
import com.ibm.odcb.jrender.misc.Streamer;
import com.ibm.odcb.jrender.misc.StringUtil;


public class MappingsParser extends SaxBase
 {
   public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;
   
   public MappingsParser(WDO4JSMappings universe)
    {
      _Universe = universe;
    }

   public boolean parse(File f)
   throws Exception
    {
       super.Parse(f.getPath());

      if (getError() > 0)
       {
         Streamer.error.Header().println(getError() + " error(s) were found parsing '"+f.getPath()+"'.");
         return false;
       }
      return true;
   }
   
   public boolean parse(InputStream is)
   throws Exception
    {
       super.Parse(is);

      if (getError() > 0)
       {
         Streamer.error.Header().println(getError() + " error(s) were found parsing the XML InputStream.");
         return false;
       }
      return true;
   }

   public static final String TAG_MAPPINGS          = "mappings"            ;
   public static final String TAG_EPACKAGE          = "EPackage"            ;
   public static final String TAG_ECLASSMAP         = "EClassMap"           ;
   public static final String TAG_EFEATUREMAP       = "EFeatureMap"         ;
   public static final String TAG_CALCULATE         = "CalculateAttribute"  ;
    
   public static final String ATTR_NAME             = "name"                ;
   public static final String ATTR_ROOT_ECLASSNAME  = "root_eclass_name"    ;
   public static final String ATTR_ON_UPDATE        = "on_update"           ;
   public static final String ATTR_ON_CREATE        = "on_create"           ;
   public static final String ATTR_ON_DELETE        = "on_delete"           ;
   public static final String ATTR_JS_URI	        = "js_URI"              ;
   public static final String ATTR_APPCLASS         = "app_class"           ;
   public static final String ATTR_ECLASS_NAME      = "eclass_name"         ;
   public static final String ATTR_EXPORT           = "export"              ;
   public static final String ATTR_EFEATURE_NAME    = "efeature_name"       ;
   public static final String ATTR_GET              = "get"                 ;
   public static final String ATTR_SET              = "set"                 ;
   public static final String ATTR_ID               = "iD"                  ;
   public static final String ATTR_REFERENCE        = "reference"           ; 
   public static final String ATTR_DIFF_REFRESH = "diff_on_refresh" ; 
   public static final String ATTR_EXPRESSION       = "expression"          ;  
   public static final String ATTR_ATTR_NAME        = "attribute_name"      ; 
   public static final String ATTR_TYPE             = "type"                ; 
   
   protected String          _SourceEmapName;
   protected WDO4JSMappings  _Universe;

//   protected String _EPackageName   = null;
//   protected String       _RootEClassName = null;
   protected EPackageMap  _EPM = null;
   protected EClassMap    _ECM = null;

   public WDO4JSMappings getMappings()
    { 
      return _Universe;
    }

   public void setSourceEmapName(String SourceEmapName)
    {
      _SourceEmapName = SourceEmapName;
    }

   
   public void startElement (String name, org.xml.sax.AttributeList attributes)
   throws org.xml.sax.SAXException
    {
      try{
      Streamer.trace.Header().print("Line ").print(_Line).print(", Tag '").print(name).println("' found");
      if (name.equalsIgnoreCase(TAG_MAPPINGS))
       {
       }
      else if (name.equalsIgnoreCase(TAG_EPACKAGE))
       {
           if (_ECM != null)
            {
            Streamer.error.Header().println("Line " + _Line + ", '"+TAG_EPACKAGE+"' tag found inside another '"+TAG_EPACKAGE+"'."); 
            return;
            }
           doEPackageTag(attributes);
       }
      else if (name.equalsIgnoreCase(TAG_ECLASSMAP))
       {
           if (_ECM != null)
            {
            Streamer.error.Header().println("Line " + _Line + ", '"+TAG_ECLASSMAP+"' tag found inside another '"+TAG_ECLASSMAP+"'."); 
            return;
            }
         doEClassMapTag(attributes);
       }
      else if (name.equalsIgnoreCase(TAG_EFEATUREMAP))
       {
           if (_ECM == null)
            {
            Streamer.error.Header().println("Line " + _Line + ", '"+TAG_EFEATUREMAP+"' tag found outside of a '"+TAG_ECLASSMAP+"'.");
            return;
            }
         doEFeatureMapTag(attributes);
       }
      else if (name.equalsIgnoreCase(TAG_CALCULATE))
      {
        if (_ECM == null)
         {
            Streamer.error.Header().println("Line " + _Line + ", '"+TAG_CALCULATE+"' tag found outside of a '"+TAG_ECLASSMAP+"'.");
            return;
         }
         doCalculateAttributeTag(attributes);
      }
      else
       {
         Streamer.warning.Header().println("Line " + _Line + ", Unknown tag '" + name + "'.");
         ++_Warning;
       }
      }   catch (Throwable T)
        {
          Streamer.error.Header().printStackTrace(T);
        }
    }   
    
    protected void doEPackageTag(org.xml.sax.AttributeList attributes)
    {
      String EPackageName   = CheckAttribute(attributes, ATTR_NAME           , null, true);
      String RootEClassName = CheckAttribute(attributes, ATTR_ROOT_ECLASSNAME, null, false);
      String OnUpdate       = CheckAttribute(attributes, ATTR_ON_UPDATE      , null, false);
      String OnCreate       = CheckAttribute(attributes, ATTR_ON_CREATE      , null, false);
      String OnDelete       = CheckAttribute(attributes, ATTR_ON_DELETE      , null, false);
      String JsURI         = CheckAttribute(attributes, ATTR_JS_URI        , null, false);
	  
      _EPM = new EPackageMap(EPackageName);
      _EPM.setOnUpdate(OnUpdate);
      _EPM.setOnCreate(OnCreate);
      _EPM.setOnDelete(OnDelete);
      _EPM.setJSURI(JsURI);
      _EPM.setRootEClassMapName(RootEClassName);
      _EPM.setEMapResourceName (_SourceEmapName);
      _EPM.setEMapXMLFileName  (_XmlFileSource);
      _Universe.addEPackageMap(_EPM, false);
      _EPM.setUniverse(_Universe);
    }
    
    protected void doEClassMapTag(org.xml.sax.AttributeList attributes)
    {
      String app_class = CheckAttribute( attributes, ATTR_APPCLASS , null , true );
      String eclass_name = CheckAttribute( attributes, ATTR_ECLASS_NAME , null , true );
      String export = CheckAttribute( attributes, ATTR_EXPORT , null, true );
      boolean diff = CheckAttributeBool( attributes, ATTR_DIFF_REFRESH, true, false );
      if (EClassMap.ParseComplexName(app_class) == null)
       {
         Streamer.error.Header().println("Line " + _Line + ", The 'app_class' attribute for EClassMap '"+eclass_name
                                        +"' has an invalid syntax. We received '"+app_class
                                        +"' when we expected something of the form 'BaseType(ExtraName)' or 'BaseType'");
         ++_Error;         
       }
      else
       {
         _ECM = new EClassMap(_EPM.getName(), eclass_name, app_class, export, 
                            getXmlFileSource(), _SourceEmapName, diff );
       }
    }
   
   protected void doEFeatureMapTag(org.xml.sax.AttributeList attributes)
    {
      String  efeature_name = CheckAttribute (attributes, ATTR_EFEATURE_NAME, null, true);
      boolean  iD = CheckAttributeBool (attributes, ATTR_ID, false, false);
      String get = CheckAttribute (attributes, ATTR_GET, null, true);
      String set = CheckAttribute (attributes, ATTR_SET, null, false);

      String export = CheckAttribute(attributes, ATTR_EXPORT , null, false);
      if (export != null)
       {
         Streamer.warning.Header().println("The export attribute for EFeatureMaps is deprecated. The value will be ignored and the EFeature will be exported using its ogirinal name.");
       }

//      Streamer.debug.Header().println(efeature_name);
      if (EClassMap.ParseComplexName(efeature_name) == null)
       {
         Streamer.error.Header().println("Line " + _Line + ", The 'efeature_name' attribute '"+efeature_name
                                        +"' in EClassMap '"+_ECM.getJavaComplexName()
                                        +"'has an invalid syntax. We expected something of the form 'BaseType(ExtraName)' or 'BaseType'");
         ++_Error;         
       }
      else
       {
         EFeatureMap EFM = new EFeatureMap(efeature_name, iD, get, set, false);
         _ECM.addEFeatureMap(EFM);
         if (iD)
          _ECM.addEIdFeatureMap(EFM);
       }
    }
    
    protected void doCalculateAttributeTag(org.xml.sax.AttributeList attributes)
    {
      String  attr_name = CheckAttribute (attributes, ATTR_ATTR_NAME, null, true);
      boolean  iD = CheckAttributeBool (attributes, ATTR_ID, false, false);
      String expression = CheckAttribute(attributes, ATTR_EXPRESSION, null, true);
      expression = StringUtil.convertXMLEscapingtoJS(expression);
      String type = CheckAttribute(attributes, ATTR_TYPE, null, false);
      
      String export = CheckAttribute(attributes, ATTR_EXPORT , null, false);
      if (export != null)
       {
         Streamer.warning.Header().println("The export attribute for CalculateAttribute is deprecated. The value will be ignored and the EFeature will be exported using its ogirinal name.");
       }
      
      EFeatureMap am = new CalculateAttribute(attr_name, iD, expression, type);
      _ECM.addEFeatureMap(am);
      if (iD)
        _ECM.addEIdFeatureMap(am);    
    }
    
   public void endElement (String name)
   throws org.xml.sax.SAXException
    {
      try
       {
         Streamer.trace.Header().print("Line ").print(_Line).print(", Tag '").print(name).println("' found");
         if (name.equalsIgnoreCase(TAG_MAPPINGS))
          {
          }
         else if (name.equalsIgnoreCase(TAG_EPACKAGE))
          {
            if (_EPM == null)
             {
               Streamer.error.Header().println("Line " + _Line + ", closing '"+TAG_EPACKAGE+"' tag found outside a '"+TAG_EPACKAGE+"' block.");
               ++_Error;
               return;
             }
            _EPM = null;
          }
         else if (name.equalsIgnoreCase(TAG_ECLASSMAP))
          {
            if (_ECM == null)
             {
               Streamer.error.Header().println("Line " + _Line + ", closing '"+TAG_ECLASSMAP+"' tag found outside a '"+TAG_ECLASSMAP+"' block.");
               ++_Error;
               return;
             }
            if (_Universe.addEClassMap(_EPM, _ECM, false) == false)
             {
               _Universe.PrintEClassMapByEcoreName();
               Streamer.error.Header().println("Line " + _Line + ", "+TAG_ECLASSMAP+" '"+_ECM.getEcoreName() + "' is defined in duplicate. Note that both table and object definitions share the same namespace.");
               ++_Error;
               return;
             }
            _ECM = null;
          }
         else if (name.equalsIgnoreCase(TAG_EFEATUREMAP))
          {
          }
         else if (name.equalsIgnoreCase(TAG_CALCULATE))
          {
          }
         else
          {
            Streamer.warning.Header().println("Line " + _Line + ", Unknown tag '" + name + "'.");
            ++_Warning;
          }
       }
      catch (Throwable T)
       {
         Streamer.error.Header().printStackTrace(T);
       }
    }
 
   public static void main(String[] args)
    {
      try
       {
         String path = args.length > 0 ? args[0]
                                       : "F:\\eclipse\\workspace\\WDOSource\\src\\com\\ibm\\odcb\\test\\jrender\\jsdo\\sources\\imc\\portfolio.emap";
         File f = new File(path);
         WDO4JSMappings maps = new WDO4JSMappings();
         boolean OK = new MappingsParser(maps).parse(f);
         System.out.println("Finished parsing mappings");
         System.out.println("The following mappings were added:");
         maps.PrintEClassMapByEcoreName();
       }
      catch (Exception e)
       {
         System.err.println("Error parsing mappings:");
         e.printStackTrace();
       }
    }

  /**
   * Returns the _SourceEmapName.
   * @return String
   */
//  public String getSourceEmapName()
//  {
//    return _SourceEmapName;
//  }

  /**
   * Sets the _SourceEmapName.
   * @param theSourceEmapName The _SourceEmapName to set
   */
//  public void setSourceEmapName(String theSourceEmapName)
//  {
//    this._SourceEmapName = theSourceEmapName;
//  }
 }
