/////////////////////////////////////////////////////////////////////////
//  IBM Confidential
//  
//  OCO Source Materials
//  
//  PID xxxx-xxx
//  
//  Copyright IBM Corp.  2003
//  
//  The source code for this program is not published or other-
//  wise divested of its trade secrets, irrespective of what has
//  been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////


package com.ibm.odcb.jrender.mediators.gen.ecore;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EDataType;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EStructuralFeature;

import com.ibm.odcb.jrender.mediators.ExportException;
import com.ibm.odcb.jrender.mediators.Mediator;
import com.ibm.odcb.jrender.mediators.gen.WDO4JSMappings;
import com.ibm.odcb.jrender.misc.IntWrapper;
import com.ibm.odcb.jrender.misc.ODCClassLoader;
import com.ibm.odcb.jrender.misc.Streamer;
import com.ibm.odcb.jrender.misc.StringUtil;
import com.ibm.odcb.jrender.misc.TypesUtil;

////#IF# SDO
import commonj.sdo.DataObject;
////#END#

////#IF# WDO
//import com.ibm.etools.wdo.DataObject;
////#END#

/**
 * @author tgupta
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class EClassMap
 {
   public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;

   public EClassMap(String ePackageName, String eClassName, String JavaComplexName,
                    String Export, String eMapXMLFileName, String theEmapResourceName, boolean diffOnRefresh)
    {
      _EPackageName = ePackageName;
      _EClassName = eClassName;

      _JavaComplexName = JavaComplexName;
      String[] A = ParseComplexName(_JavaComplexName);
      if (A != null)
       {
         _BaseTypeName      = A[0];
         _ExtraName     = A[1];
         
         //_BaseTypeClass is used before verify() is called, so, we need to instantiate it in the constructor
         try
	       {
	         _BaseTypeClass = ODCClassLoader.getClass(_BaseTypeName);
	       }
	      catch (Exception E)
	       {
	         Streamer.error.Header().println("Cannot locate class "+_BaseTypeName);
	         Streamer.error.Header().printStackTrace(E);
	       }
       }
      _ECoreComplexName = MakeComplexName(_EClassName,_ExtraName);

      _Export = Export;
      _XMLFileSource = eMapXMLFileName;
      _SourceEmapName = theEmapResourceName;
      _DiffOnRefresh = diffOnRefresh;
    }
    
   protected String  _EPackageName     ;  // The EPackage name for this mappping
   protected String  _EClassName       ;  // The EClass name for this maping
   protected EClass  _EClass           ;  // The EClass corresponding to the EClass name
   protected String  _BaseTypeName     ;  // The base Application type for the mapping (i.e. com.ibm.wdo.DataObject)
   protected Class   _BaseTypeClass    ;  // The Java class corresponding to this EMapClass
   protected String  _ExtraName        ;  // The extra name qualifier to separate multiple mappings for the same app class
   protected String  _JavaComplexName  ;  // _BaseType+"("+_ExtraName+")"
   protected String  _ECoreComplexName ;  // _EClassName+"("+_ExtraName+")"
   protected String  _Export           ;  // The Export name
   protected String  _XMLFileSource    ;  
   protected String  _SourceEmapName   ;
   protected boolean _DiffOnRefresh    ;
   protected WDO4JSMappings _Universe  ;
   protected ArrayList _EFeatureMaps   = new ArrayList(7); // Collection of EFeatureMaps corresponding to all of the EClass's EAttributes.
   protected ArrayList _EIdFeatureMaps = new ArrayList(7); //Collection of EFeatureMaps that correspond to the id EAttributes or EReferences of the EClass.

   public String getEPackageName()              { return _EPackageName ; }
   public String getEClassName()                { return _EClassName   ; }   
   public String getEcoreName()                 { return _EPackageName + Mediator._MEDIATOR_SEPERATOR + _EClassName; }
   public String getBaseType()                  { return _BaseTypeName    ; }
   public Class  getBaseTypeClass()             { return _BaseTypeClass   ; }
   public String getJavaComplexName()           { return _JavaComplexName ; }
   public String getECoreComplexName()          { return _ECoreComplexName; }
   public String getExtraName()                 { return _ExtraName       ; }
   public String getXMLFileSource()             { return _XMLFileSource   ; }
   public String getExport()                    { return _Export          ; } 
   public EClass getEClass()                    { return _EClass          ; }
   public EPackageMap getEPackageMap()			{ return _Universe.getEPackageMap(_EPackageName); } // adam note: added for client data event support
   																									// previously only getter for EPackage name	
   public WDO4JSMappings getUniverse()			{ return _Universe; }
   public Iterator getEAttributeIterator()      { return _EClass.getEAttributes().iterator() ; }
   public Iterator getEReferenceIterator()      { return _EClass.getEReferences().iterator() ; }
   public Iterator getEFeatureMapIterator()     { return _EFeatureMaps.iterator()          ; } 
   public Iterator getEIdFeatureMapIterator()   { return _EIdFeatureMaps.iterator()        ; }

   public boolean isDiffOnRefresh ()                             { return _DiffOnRefresh; }
   public void    setDiffOnRefresh(boolean isDiffOnRefresh)  {  _DiffOnRefresh = isDiffOnRefresh; }


   public void           setUniverse(WDO4JSMappings Universe) { _Universe = Universe; }


   public static String MakeComplexName(String BaseType, String ExtraName)
    {
      return ExtraName == null ? BaseType
                               : BaseType+"("+ExtraName+")";
    }
    
   public static String[] ParseComplexName(String ComplexName)
    {
      if (ComplexName.length() == 0)
       {
         String[] A = new String[2];
         A[0]="";
         A[1]=null;
         return A;
       }

      String BaseType  = null;
      String ExtraName = null;
      
      int i = ComplexName.indexOf('(');
      if (i == -1)
       {
         BaseType = ComplexName;
         ExtraName= null;
       }
      else
       {
         BaseType = ComplexName.substring(0, i);
         int j = ComplexName.lastIndexOf(')');
         if (j != -1)
          {
            ExtraName = (i+1 == j) ? null : ComplexName.substring(i+1, j);
          }
         else
          {
            Streamer.error.Header().println("Mapping '"+ComplexName+"' has an incorrect syntax for attribute 'class'. Expected 'full_class_path{(extra_name)}'.");
            return null;
          }
       } 
      String[] A = new String[2];
      A[0]=BaseType;
      A[1]=ExtraName;
      if (isCorrectSyntax(BaseType, false) == false || isCorrectSyntax(ExtraName, true) == false)
       {
         Streamer.error.Header().println("Mapping '"+ComplexName+"' has an incorrect syntax for attribute 'class'. BaseType and ExtraName parts are expected to be valid Java identifiers.");
         return null;
       }
      return A;
    }
   public static boolean isCorrectSyntax(String Identifier, boolean Extra)
    {
      if (Extra == true && (Identifier==null || Identifier.trim().length() == 0))
       return true;
      int i = 0;
      char c = Identifier.charAt(i);
      if (Character.isLetter(c) == false && c!='_' && c!='$')
       return false;
      ++i;
      while (i < Identifier.length())
       {
         c = Identifier.charAt(i);
         if (Character.isLetterOrDigit(c) == false && c != '.' && c != '_' && c!='$' && (Extra==false || c!= '`') )
          {
            return false;
          }
         ++i;
       }
      return true;
    }

   /**
    * @deprecated
    */    
   public EFeatureMap getEFeatureMapByName(String export)
    {
      for (int i = 0; i < _EFeatureMaps.size(); ++i)
       {
           EFeatureMap ME = (EFeatureMap) _EFeatureMaps.get(i);
           if (ME.getExport().equals(export) == true)
            return ME;
       }
      return null;
    }
    
   public Iterator getClientAttributesIterator()
   {
      ArrayList ca = new ArrayList();
      for (int i = 0; i < _EFeatureMaps.size(); ++i)
      {
        EFeatureMap f = (EFeatureMap) _EFeatureMaps.get(i);
        if (f.isCalculateAttribute())
        {
           ca.add(f);
        }
      }
        return ca.iterator();
   }
   
   public void addClientAttributes(Iterator clientit)
   {
     if (clientit != null)
      while (clientit.hasNext())
      {
        CalculateAttribute ca = (CalculateAttribute) clientit.next();
        _EFeatureMaps.add(ca);
      }
   }
   
   public void addClientAttribute(CalculateAttribute ca)
   {
     if (ca != null)
      _EFeatureMaps.add(ca);
   }
      
   public EFeatureMap getEFeatureMapByExport(String export)
   {
      return getEFeatureMapByName(export);
   }
    
   public EFeatureMap getEFeatureMapByEFeatureName(String eFeatureName)
    {
      for (int i = 0; i < _EFeatureMaps.size(); ++i)
       {
         EFeatureMap ME = (EFeatureMap) _EFeatureMaps.get(i);
         if (ME.getEFeatureName() != null)
         {
           if (ME.getEFeatureName().equals(eFeatureName) == true)
           return ME;
         }
       }
       return null;
    }
    
   public ArrayList getEFeatureMaps(String eAttributeMapExportNames, IntWrapper Error, boolean SubsetOK)
    {
      if (eAttributeMapExportNames == null || eAttributeMapExportNames.length() == 0)
       {
          Error.increment();
            Streamer.error.Header().println("The comma seperated list of EAttributeMap export names to retrieve for mapping '"+_JavaComplexName+"' is empty.");
           return null;
       }
      boolean OK = true;
      ArrayList A = new ArrayList(3);
      for (int j = 0; j < eAttributeMapExportNames.length(); ++j)
       {
           int i = j;
           j = eAttributeMapExportNames.indexOf(",", j);
           if (j == -1)
            j = eAttributeMapExportNames.length();
           String E = eAttributeMapExportNames.substring(i, j).trim();
           EFeatureMap ME = getEFeatureMapByName(E);
           if (ME != null)
            A.add(ME);
           else
            {
               Error.increment();
               OK = false;
               Streamer.error.Header().println("Cannot find the EAttributeMap '"+E+"' for mapping '"+_JavaComplexName+"'.");
            }
       }
      return (SubsetOK == true || OK == true) ? A : null;
    }  
       
   public void addEFeatureMap(EFeatureMap ME) { _EFeatureMaps  .add(ME); }
   public void addEIdFeatureMap(EFeatureMap EIdAttribute) { _EIdFeatureMaps.add(EIdAttribute); }
   
   public boolean Verify(IntWrapper Error)
    {
      boolean OK = true;

      try
       {
         _BaseTypeClass = ODCClassLoader.getClass(_BaseTypeName);
       }
      catch (Exception E)
       {
         Streamer.error.Header().println("Cannot locate class "+_BaseTypeName);
         Streamer.error.Header().printStackTrace(E);
         Error.increment();
         OK = false;
       }
      
      //check if the EClass is valid, make sure it's in the universe
      EClassifier ec = _Universe.getEClassifier(_EPackageName, _EClassName);
      if (ec instanceof EClass)
        _EClass = (EClass) ec;
      if (_EClass == null)
       {
         Streamer.error.Header().println("Cannot locate Eclass "+ _EPackageName + "`" + _EClassName);
         Streamer.error.Header().println("The eClassifier for "+ _EPackageName + "`" + _EClassName + " is " + ec);
         if (ec instanceof EDataType)
          {
            Streamer.error.Header().println("EReferences of EDataTypes are not supported.  There is only limited support of EDataType eAttributes.  Currently only \"java.util.Date\" is the supported EDataType.  Remove your EClassMap entry for \"" + _EClassName + "\" from your \""+ _EPackageName + "\" emap and try again.  Please refer to the browser framework user guide for more information on correct usage of EDataTypes.");
          }
         Error.increment();
         OK = false;       
       }      

     if (OK && _EFeatureMaps.size() > 0)
      {
        for (int i = 0; i < _EFeatureMaps.size(); ++i)
         {
           EFeatureMap E1 = (EFeatureMap) _EFeatureMaps.get(i);
           E1._ParentMapping = this;
           for (int j = i+1; j < _EFeatureMaps.size(); ++j)
            {
              EFeatureMap E2 = (EFeatureMap) _EFeatureMaps.get(j);
              if (E1.getEFeatureName().equals(E2.getEFeatureName()) == true)
               {
                 Error.increment();
                 OK = false;
                 Streamer.error.Header().println("Mapping '"+_JavaComplexName+"' has a duplicate definition for EFeature '"+E1.getEFeatureType()+"'.");
               }
               if (!E1.isCalculateAttribute())
                {
                 if (E1.getGetter().equals(E2.getGetter()) == true)
                  {
                    Error.increment();
                    OK = false;
                    Streamer.error.Header().println("Mapping '"+_JavaComplexName+"' has a duplicate definition for element's get '"+E1.getGetter()+"'.");
                  }
                }
              }
         }
        if (OK)
        {
           for (int i = 0; i < _EFeatureMaps.size(); ++i)
            {
             EFeatureMap ME = (EFeatureMap) _EFeatureMaps.get(i);
              if (ME.Verify(this, _Universe, Error) == false)
              OK = false;
           }
       }
      }
     else
      {
          if (OK)
          {
          Error.increment();
          OK = false;
          Streamer.error.Header().println("Mapping EClass '"+_JavaComplexName+"' has no EFeatures.");
       }    
      }
       
      return OK;
    }  
    
    
    public EAttribute getEAttributeByName(String eAttributeName){
       Iterator it = _EClass.getEAttributes().iterator();
       while (it.hasNext()){
          EAttribute attr = (EAttribute) it.next();
          if (attr.getName().equals(eAttributeName))
          return attr;
       }
       return null;
    }
    
    public EReference getEReferenceByName(String eReferenceName){
       EList list = _EClass.getEReferences();
       Iterator it = list.iterator();
       while (it.hasNext()){
          EReference ref = (EReference) it.next();
          if (ref.getName().equals(eReferenceName))
          return ref;
       }
       return null;
    }
    
    
    public EStructuralFeature getEStructuralFeatureByName(String eFeatureName){
       if (_EClass == null) 
         return null;
       EList list = _EClass.getEAllStructuralFeatures();
       Iterator it = list.iterator();
       while (it.hasNext()){
          EStructuralFeature feature = (EStructuralFeature) it.next();
          if (feature.getName().equals(eFeatureName))
          return feature;
       }
       return null;
    }
    
    public void getKeySignature(StringBuffer sigStr, 
            DataObject o, 
            int complexPropertyCount, String containerClassName)
    {
    	getKeySignature(sigStr,o,complexPropertyCount,containerClassName,new HashMap());
    }

   protected void getKeySignature(StringBuffer sigStr, 
                               DataObject o, 
                               int complexPropertyCount, String containerClassName,
							   HashMap env)
    {
   	//check if this object was already processed to avoid infinite loop
   	if (env.put(o,o) != null)
   		return;
   	
      for (int i = 0; i < _EIdFeatureMaps.size(); ++i)
       {
         EFeatureMap EFM = (EFeatureMap) _EIdFeatureMaps.get(i);
         if (EFM.isReference())
          {
            EClassMap ECM = EFM.getReferenceMap();
            if (ECM!=null)
             {
               ++complexPropertyCount;
               DataObject fo = o.getDataObject(EFM.getEFeatureName());
          
               //checking for circular reference here

// LDH-NOTE: It really doesn't matter if it is a circular reference or not. What matters is if
//          the child reference is equal to the parent object (an object pointing back to itself). 
//          The code dealing with circular reference has been removed.

               if (containerClassName == null)
                containerClassName = this._JavaComplexName;

               if (fo == null)
                {
                  sigStr.append("`").append(complexPropertyCount+1).append("\\u00A4");
                }
               else if (fo != o)
                {
                  ECM.getKeySignature(sigStr, fo, complexPropertyCount, containerClassName, env);
                }
             }
          }
         else
          {
            if (EFM.getGetterType().equals("java.util.Date"))
             {
               sigStr.append("`").append( ( o.getDate(EFM.getEFeatureName()) == null)? -1 : o.getDate(EFM.getEFeatureName()).getTime() );
             }
            else
            {
              Object fo = o.get(EFM.getEFeatureName());
              if (fo == null)
                sigStr.append("`").append(fo);
              else if (TypesUtil.isDecimalType(EFM.getGetterType()))
              {
              	sigStr.append("`").append(StringUtil.removeTrailingDecimalZeros(fo.toString()));
              }
              else
              	sigStr.append("`").append(StringUtil.trailTrim(fo.toString()));
            }
          }
       }
    }

   public String getKeySignature(String root)
   throws ExportException
    {
       StringBuffer s = new StringBuffer(256);
       getKeySignature(s, root, new HashMap(29), 0, 0, null);
       return s.toString();
    }

   protected void getKeySignature(StringBuffer S, String BaseGet, HashMap Path, int Level, 
                                  int ComplexPropertyCount, String containerClassName)
   throws ExportException
    {
      // Here, by creating signatures based on the class name, we allow a full aliasing across complex names. For instance,
      //   - MyApp.test.User(Portfolios) and MyApp.test.User(Addresses) are different views on User, but basically, the same user.
      //This may be a recursive call if the iD field is a reference.  We only need to
      //append full classname for the first call.
//      Streamer.debug.Header().println("Creating Key for "+_ComplexName);
//      Streamer.debug.Header().println("Key String is "+ S);
      if (Path.put(_JavaComplexName, _JavaComplexName) == null)
       {
         for (int i = 0; i < _EIdFeatureMaps.size(); ++i)
          {
            EFeatureMap EFM = (EFeatureMap) _EIdFeatureMaps.get(i);
            String GetterStr = EFeatureMap.CleanupGetterSetter(EFM.getGetter());        
            //if primary key is a reference, append it's short signature
            if (EFM.isReference())
             {
               EClassMap m = EFM.getReferenceMap();
               if (m!=null)
                {
                  // This is very important. Let's assume that we have the following classes:
                  // class A { public getB1(); public getB2(); }
                  // class B { public getString1(); public getString2(); }
                  // Now, let's assume that you output an instance of A where B1 is null, and B2's 2 strings are null
                  // the  signature would be `null`null`null
                  // If B1's 2 strings were null and B2 were null, you'd get the same signature `null`null`null
                  // In order to avoid null pointers, we cannot go into complex structures and therefore keep
                  // a uniform signature for all instances, no matter what. We therefore need some extra information to
                  // keep track of the complex property count, so that the signatures would be different:
                  // `0null`null`null Vs. `null`null`1null
                  // The index is important for more complex entities where you could have multiple nesting.
                  ++ComplexPropertyCount;                   
                  S.append(StringUtil.getPad(Level)).append("         if (").append(BaseGet).append(".").append(GetterStr).append(" == null)\n")
                   .append(StringUtil.getPad(Level)).append("          {\n")
                   .append(StringUtil.getPad(Level)).append("            SigStr.append(\"`\").append("+ComplexPropertyCount+"+ComplexPropertyCount).append(\"\\u00A4\");\n")
                   .append(StringUtil.getPad(Level)).append("          }\n");
                   
               //checking for circular reference here
               boolean circularRef = false;
               if (containerClassName == null)
                {
                  containerClassName = this._JavaComplexName;
                }
               circularRef = m._JavaComplexName.equals(containerClassName); 
                
               if (circularRef) 
                {
                  S.append(StringUtil.getPad(Level)).append("         else if (").append(BaseGet).append(".").append(GetterStr).append(" != Obj)\n");
                }
               else
                {
                  S.append(StringUtil.getPad(Level)).append("         else \n"); 
                }
                  
               S.append(StringUtil.getPad(Level)).append("          {\n");
               if (EFM.getGetterType().equals(Object.class.getName()) == true)  // need some casting, otherwise, we are trying to call methods on Object
                {
                  m.getKeySignature(S, "(("+EFM.getReferenceMap().getBaseType().replace('$','.')+")"+BaseGet+"."+GetterStr+")", Path, Level+3, ComplexPropertyCount, containerClassName);
                }
               else
                {
                  m.getKeySignature(S, BaseGet+"."+GetterStr, Path, Level+3, ComplexPropertyCount, containerClassName);
                }
               S.append(StringUtil.getPad(Level)).append("          }\n");
             }
             }
            else
             {
               if (EFM.getGetterType().equals("java.util.Date"))
                {
                  S.append(StringUtil.getPad(Level)).append("         SigStr.append(\"`\").append((").append(BaseGet+"."+GetterStr).append(")==null?-1:(").append(BaseGet+"."+GetterStr).append(").getTime());\n");
                }
               else if (TypesUtil.getOutputTypeFromJavaType(EFM.getGetterType()) == TypesUtil.TypeInfo._STRING_OUT)
               {
                 S.append(StringUtil.getPad(Level)).append("         SigStr.append(\"`\").append(com.ibm.odcb.jrender.misc.StringUtil.trailTrim(").append(BaseGet+"."+GetterStr).append("));\n");
               }
               else if (TypesUtil.isDecimalType(EFM.getGetterType()))
               {
               	 //you need to check for primitive types here.  Blind toString() would cause compile errors for non Object types.
               	 if (EFM.getGetterType().equals("float"))
               	   S.append(StringUtil.getPad(Level)).append("         SigStr.append(\"`\").append(com.ibm.odcb.jrender.misc.StringUtil.removeTrailingDecimalZeros(Float.toString(").append(BaseGet+"."+GetterStr).append(")));\n");
               	 else if (EFM.getGetterType().equals("double"))
               	 S.append(StringUtil.getPad(Level)).append("         SigStr.append(\"`\").append(com.ibm.odcb.jrender.misc.StringUtil.removeTrailingDecimalZeros(Double.toString(").append(BaseGet+"."+GetterStr).append(")));\n");
               	 else
               	   S.append(StringUtil.getPad(Level)).append("         SigStr.append(\"`\").append(com.ibm.odcb.jrender.misc.StringUtil.removeTrailingDecimalZeros((").append(BaseGet+"."+GetterStr).append(").toString()));\n");
               }
               else
                {
                  S.append(StringUtil.getPad(Level)).append("         SigStr.append(\"`\").append(").append(BaseGet+"."+GetterStr).append(");\n");
                }
             }
          }
       }
      else // We are encountering a cycle in the path, so we have to delegate the creation of the Key.
       {
         S.append(StringUtil.getPad(Level)).append("         SigStr.append(com.ibm.odcb.jrender.mediators.MediatorFactory.getMediator_byTypeKey(\""+_JavaComplexName+"\", \"_wdo4js_js\").getSignature("+BaseGet+", modelName, false, ComplexPropertyCount));\n");
         // We do not need to increment ComplexPropertyCount here, as this is called recursively from the
         // if (ME.getGetType().equals(Object.class.getName()) == true) section, where it has already been incremented
       }
      // LDH-NOTE: I have no idea why i added this call here. This has for effect to create an infinite loop for cycles in class
      //     definitions. for instance, an EPackage has a list of EClasses, and each EClass has a parent EPackage, which has
      //     a list of EClasses etc.... the else clause above takes care of this at the instance level. So i am commenting this now.
//      Path.remove(_ComplexName);
    }

  /**
   * Returns the _SourceResourceName.
   * @return String
   */
  public String getSourceEmapName()
  {
    return _SourceEmapName;
  }

  /**
   * Sets the _SourceResourceName.
   * @param _SourceResourceName The _SourceResourceName to set
   */
  public void setSourceEmapName(String theSourceEmapName)
  {
    this._SourceEmapName = theSourceEmapName;
  }

  public String getClientEcoreName()
  {
    String name = null;
    if ( _SourceEmapName == null || _SourceEmapName.trim().length() == 0 )
      return null;
    else
    {
      name = _SourceEmapName.substring(0, _SourceEmapName.lastIndexOf(".emap")) + "/" + _SourceEmapName.substring( _SourceEmapName.lastIndexOf("/")+1, _SourceEmapName.lastIndexOf(".emap") ) + "_client.ecore";
    }
    return name;
  }

  public void setExport(String string) {
   _Export = string;
  }

/**
 * @param string
 */
public void setExtraName(String string) {
   _ExtraName = string;
}
/**
 * @param c
 */
public void setEClass(EClass c) {
   _EClass = c;   
}

  public void setEClassName(String s)
  {
   _EClassName = s;
  }
  
  public void removeEFeatureMap(EFeatureMap f)
  {
     _EFeatureMaps.remove(f);
     _EIdFeatureMaps.remove(f);
  }
  
}
