/////////////////////////////////////////////////////////////////////////
//   IBM Confidential
//  
//   OCO Source Materials
//  
//   PID xxxx-xxx
//  
//    Copyright IBM Corp.  2003
//  
//   The source code for this program is not published or other-
//   wise divested of its trade secrets, irrespective of what has
//   been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////


package com.ibm.odcb.jrender.mediators.gen.ecore;

import java.io.PrintWriter;
import java.lang.reflect.Field;
import java.lang.reflect.Method;

import com.ibm.odcb.jrender.misc.EMFHelper;
import com.ibm.odcb.jrender.misc.IntWrapper;
import com.ibm.odcb.jrender.misc.TypesUtil;
import com.ibm.odcb.jrender.misc.Streamer;
import com.ibm.odcb.jrender.mediators.gen.WDO4JSMappings;

import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EStructuralFeature;

/**
 * @author tgupta
 *
 * To change this generated comment edit the template Export "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class EFeatureMap
 {
   public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;
   public static final String SET_TOKEN = "X";
   public static final String XMI_ID_FEATURE = "xmi:id";
   public static final String ODYSSEY_STATUS_FEATURE = "ODCstatus";
   public static final String ODYSSEY_ID_FEATURE = "ODCid";
   public static final String[] OBF_RESERVED_NAMES = {ODYSSEY_ID_FEATURE,ODYSSEY_STATUS_FEATURE};
   
   // Now, with the new setter string, we have 2 constructors wiyth exactly the same type. We could have created
   // more classes and differentiat ethe types, but that's a lot of work right now. Instead, we just have the same
   // constructor signature, but with a dummy boolean to differentiate the 2.
   public EFeatureMap(String eAttributeName, boolean isID, String Getter, String Setter, boolean PassInAnyValue)
   {
     String[] A = EClassMap.ParseComplexName(eAttributeName);
     _EFeatureName    = A[0];
     _ExtraTargetType = A[1];
     _ID              = isID;
     _Getter          = Getter;
     _Setter          = Setter;
   }

   protected EFeatureMap(String eAttributeName, boolean isID, String calculateExpression, String calculatedType )
   {
     _EFeatureName = eAttributeName;
     _ID = isID;
     _CalculateExpression = calculateExpression;
     _CalculatedType = calculatedType;
   }    
    
   protected EStructuralFeature _EFeature        = null ;
   protected String             _EFeatureName           ;
   protected String             _ExtraTargetType        ;
   protected boolean            _ID              = false;
   protected boolean            _Reference       = false;
   protected String             _Getter          = null ;
   protected String             _GetterType      = null ;
   protected Class              _GetterTypeClass = null ;
   protected String             _Setter          = null ;
   protected EClassMap          _ReferenceMap    = null ;
   protected boolean            _Containment     = false;
   
   protected EClassMap          _ParentMapping;
   
   protected String             _CalculateExpression;
   protected String             _CalculatedType;

   public static final String __SETTER_PARAM_MARKER = "__SET_OBJ__";

   public String  getEFeatureName    () { return _EFeatureName     ; }
   public String  getExport          () { return _EFeatureName     ; }
   public String  getGetter          () { return _Getter           ; }
   public String  getGetterType      () { return _GetterType       ; }   
   public Class   getGettterTypeClass() { return _GetterTypeClass  ; }   
   public String  getSetter          () { return _Setter           ; }
   public boolean isMany             () { return _EFeature == null ? false : _EFeature.isMany(); }

   public EStructuralFeature getEFeature(                    ) { return _EFeature; }
   public void               setEFeature(EStructuralFeature f) { _EFeature = f; }
   
   public EClassMap getParentMapping(           ) { return _ParentMapping; }
   public void      setParentMapping(EClassMap m) { _ParentMapping = m   ; }
   
   public String getExtraTargetType()             { return _ExtraTargetType; }
   public void   setExtraTargetType(String Extra) { _ExtraTargetType= Extra; }

   public String    getEFeatureType() { return _EFeature!=null ? TypesUtil.getRealETypeName(_EFeature.getEType()) : null; }
   public EClassMap getReferenceMap() { return _ReferenceMap; }

   public boolean isCalculateAttribute  ()                            { return _CalculateExpression != null; }
   public String  getCalculateExpression()                            { return _CalculateExpression; }
   public void    setCalculateExpression(String calculateExpression)  {  _CalculateExpression = calculateExpression; }
   public String  getCalculatedType     ()                            { return _CalculatedType; }
   public void    setCalculatedType     (String calculatedType)       {  _CalculatedType = calculatedType; }
   
   public boolean isID()                           { return _ID; }
   public boolean isReference()                    { return _Reference; }
   public void setID( boolean isID )               { _ID = isID; }
   public void setReference( boolean isReference ) { _Reference = isReference; }
   
   public void setGet   (String string) { _Getter = string; }
   public void setSetter(String string) { _Setter = string; }


   // this is a complex method... It must basically resolve the field of methof for Get and Set.
   // The complexity is that the string passed could be something like " [SomeMethod() | SomeField] [ '.' [SomeMethod() | SomeField]]* "
   // So we have to follow the references to 1/ figure out the final type and 2/ check it's OK
   // This adds recursion in the mix to do elegantly, but involves references as well as elements, so there is a possibility of infinite loop
   public boolean Verify(EClassMap ECM, WDO4JSMappings Universe, IntWrapper Error)
    {
      boolean OK = true;
      if ( _EFeatureName == null || _EFeatureName.trim().length() == 0 )
       {
         Error.increment();
         Streamer.error.Header().println("An EFeatureMap was defined with an empty EAttributeName in EClassMap '"+ECM.getExport()+"'. This is not supported.");
         return false;                    
       }
      
      OK = !isOBFReservedName(_EFeatureName);
      if (!OK)
       {
         Error.increment();
         Streamer.error.Header().println("EFeatureMap name '" + _EFeatureName + "' is an OBF Reserved name.  It cannot be used as feature names in OBF systems.  Please rename your feature and try again.  Following are OBF Reserved Names: " + getOBFReservedNames());
         return false;
       }

      // establish some core properties such as the EFeature is valid, if it is a reference of an attribute, id and cardinality.
      _EFeature = ECM.getEStructuralFeatureByName(_EFeatureName);
      if (_EFeature == null)
       {
         Error.increment();
         Streamer.error.Header().println("The efeature_name '"+_EFeatureName+"' defined for EClassMap with export '"+ECM.getExport()+"', was not found in the source EClass '" + ECM._EClassName +"'.");
         return false;                    
       }
      else
       {
         //if multiplicity is >1, then this cannot be a primary key
         if (isMany() && isID())
          {
            _ID = false;
            Streamer.warning.Header().println("EAttribute '"+_EFeatureName+"'' has multiplicity greater than 1, and the iD attribute is set to true.  Attributes with mutliplicity greater than 1 cannot be a primary key.  The iD attribute will be set to FALSE.");
            //remove this from the parent's _EIdFeatureMaps
            this.getParentMapping()._EIdFeatureMaps.remove(this);            
          }
         
         //set iD = false if EType is WDODate - need not do this for SDO
         
         
////#IF# WDO
//         String EFeatureEDataTypeName = TypesUtil.getRealETypeName(_EFeature.getEType());
//         if (EFeatureEDataTypeName != null)
//           if (EFeatureEDataTypeName.equals(TypesUtil.DATE_ETYPE) && isID())
//           {
//         	  _ID = false;
//              Streamer.warning.Header().println("EAttribute '"+_EFeatureName+"'' is of EType WDODate, and the iD attribute is set to true.  Attributes of EType WDODate cannot be a primary key.  The iD attribute will be set to FALSE.");
//              this.getParentMapping()._EIdFeatureMaps.remove(this);            
//           }
////#END#        
         
         EReference ref = ECM.getEReferenceByName(_EFeatureName);
         if (ref == null)
          {
            _Reference = false;
            _Containment = false;
          }
         else 
          {
            _Reference = true;
            _Containment = ref.isContainment();
          } 
       }

      // Resolve the Java type of the Jave expression received for the Getter         
      _GetterTypeClass = ResolveGetterExpressionType(_Getter, ECM._BaseTypeClass, Error, null);      
      if (_GetterTypeClass == null) // No luck
       {
         Error.increment();
         Streamer.error.Header().println("The java Class for '"+_Getter+"' was not found.");
         return false;
       }
      _GetterType = _GetterTypeClass.getName();


      // Checking that the setter is of the proper syntax, and the proper type
      if (_Setter != null)
       {
         int i = _Setter.indexOf(EFeatureMap.__SETTER_PARAM_MARKER);
         if (i == -1)
          {
            Streamer.error.Header().println("Cannot find marker '"+EFeatureMap.__SETTER_PARAM_MARKER+"' in setter '"+_Setter+"'.");
            Error.increment();
            return false;
          }
         i = _Setter.indexOf("(");
         if (i == -1)
          {
            Streamer.error.Header().println("Cannot find '(' in setter '"+_Setter+"'.");
            Error.increment();
            return false;
          }
         if (_ParentMapping.getBaseType().equals(EMFHelper.DataObject_CLASS.getName()) == false)
          {
            try
             {
               Class C = ECM._BaseTypeClass;
               Class[] Params = { _GetterTypeClass };
               Method M = C.getMethod(_Setter.substring(0, i), Params);
             }
            catch (Exception E)
             {
               Streamer.warning.Header().println("Setter '"+_Setter+"' on '"+ECM._BaseTypeClass.getName()
                                                +"' could not be verified. If this is a setter with more than one argument, then this may be OK.");
             }
          }
         
       }

      // Make sure that cardinality and type agree
      try
       {
         boolean IsList = TypesUtil.isGeneralList(_GetterType);
         if (isMany() == true &&  IsList == false)
          {
            OK = false;
            Error.increment();
            Streamer.error.Header().println("The type for '"+_Getter+"' should be List, Collection, Iterator, or an array because it was defined in the model as a multi-cardinality feature.");            
          }  
         if (isMany() == false && IsList == true)
          {
            OK = false;
            Error.increment();
            Streamer.error.Header().println("The type for '"+_Getter+"' is List, Collection, Iterator, or an array, but it was defined in the model as a single cardinality feature.");            
          }
       }
      catch (ClassNotFoundException E)
       {
         OK = false;
         Error.increment();
         Streamer.error.Header().printStackTrace(E);
       }

      // Check the EReference or the EAttribute
      if (isReference() == true)
       {
         if (VerifyEReference(ECM, Universe, Error) == false) // It better map to a known complex type then
          OK = false;    
       }
      else 
       {
         if (VerifyEAttribute(ECM, Universe, Error) == false) // It better map to a known complex type then
          OK = false;    
       }
      return OK;
    }


   /**
 * @param eFeatureName
 * @return true if eFeatureName matches an OBF Reserved Name
 *         false otherwise
 */
public static boolean isOBFReservedName(String eFeatureName) {
   for (int i=0; i<OBF_RESERVED_NAMES.length; i++)
   {
      if ( OBF_RESERVED_NAMES[i].equals(eFeatureName) )
      {
       return true;
      }
   }

   return false;
}

/**
 * @return
 */
public static String getOBFReservedNames() {
   String s = "";
   for (int i=0; i<OBF_RESERVED_NAMES.length; i++)
   {
      s += ", " + OBF_RESERVED_NAMES[i];
   }
   if (s.length() > 2)
     s = s.substring(2);
   return s;
}

public static Class ResolveGetterExpressionType(String CallSegment, Class C, IntWrapper Error, String SetToken)
   {
//     Streamer.debug.Header().println("Resolving '"+CallSegment+"' for class '"+C.getName()+"'.");
     while (CallSegment != null)
      {
       int Len = CallSegment.length();
       int j = CallSegment.indexOf('.');
       if (j == -1)
        j = Len;
       String SubCallSegment = CallSegment.substring(0, j);
       String MemberName = ExtractMethodName(SubCallSegment);
       Method M = null;
       Field  F = null;
       if (MemberName != null)
        M = getClassMethod(C, MemberName);
       else
        F = getClassField(C, SubCallSegment);
       if (M == null && F == null)
        {
         Error.increment();
         return null;
        }
       if (M == null)
        C = F.getType();
       else if (SetToken == null)
        C = M.getReturnType();
       else // This is a method. Let's see if the token is there, if specified, and then let's get THAT type.
        {
         int ArgNum = ExtractMethodArg(SubCallSegment, SetToken);
         if (ArgNum != -1)
          C = M.getParameterTypes()[ArgNum];
         else
          {
            Streamer.error.Header().println("A Setter '"+CallSegment+"' for class '"+C.getName()+"' does not have a set token '"+SetToken+"'.");
            Error.increment();
            return null;
          }
        }
       CallSegment = j == Len ? null : CallSegment.substring(j+1, Len);
      }
     return C;
   }

   public static Method getClassMethod(Class C, String MethodName)
   {
     Method[] M = C.getMethods();
     for (int i = 0; i < M.length; ++i)
      {
       if (MethodName.equals(M[i].getName()) == true)
        {
         return M[i];
        }
      }
     Streamer.error.Header().println("Cannot resolve method "+MethodName+" for class "+C.getName());
     return null;
   }

   public static Field getClassField(Class C, String FieldName)
   {
     Field[] M = C.getFields();
     for (int i = 0; i < M.length; ++i)
      {
       if (FieldName.equals(M[i].getName()) == true)
        {
         return M[i];
        }
      }
     Streamer.error.Header().println("Cannot resolve field "+FieldName+" for class "+C.getName());
     return null;
   }


    
   public static String ExtractMethodName(String FullName)
   {
     int i = FullName.indexOf('(');
     int j = FullName.indexOf(')');
     return (i != -1 && j != -1) ? FullName.substring(0, i) : null;
   }
    
   public static int ExtractMethodArg(String FullName, String SetToken)
   {
     int i = FullName.indexOf('(');
     int j = FullName.indexOf(')');
     if (i >= j)
      return -1;
       
     String Args = FullName.substring(i+1, j);
     int Index = Args.indexOf(SetToken);
     if (Index == -1)
      return -1;
     int VarCount = 0;
     boolean InString = false;
     for (i = 0; i < Args.length(); ++i)
      {
       char CurrentChar = Args.charAt(i);
       if (CurrentChar == '"') // We are either opening or closing a String
        {
         InString = !InString;
         continue;
        }
       if (InString == true)
        continue;
       if (CurrentChar == ',')
        {
         ++VarCount;
         continue;
        }
       if (i == Index)
        {
         char c1 = (i == 0                              ) ? ' ' : Args.charAt(i-1);
         char c2 = (i+SetToken.length() == Args.length()) ? ' ' : Args.charAt(i+SetToken.length());
         boolean c1OK = c1==',' || c1=='(' || Character.isSpaceChar(c1);
         boolean c2OK = c2==',' || c2==')' || Character.isSpaceChar(c2);
         if (c1OK && c2OK)
          {
            // BINGO !!!!   We isolated the token sequence as a parameter in a parameter list. 
            return VarCount;
          }
         Index = Args.indexOf(SetToken, Index+1);
         if (Index == -1)
          break;
        }
      }
     return -1;
   }

   public static String CleanupGetterSetter(String S)
   {
     if (S == null)
      return null;
     StringBuffer B = new StringBuffer(S);
     for (int i = 0; i < B.length(); ++i)
      {
       if (B.charAt(i) == '\'')
        {
         if (i < B.length()-1 && B.charAt(i+1) == '\'')
          {
            B.deleteCharAt(i+1);
          }
         else
         {
           B.setCharAt(i, '"');
         }
        }
      }
     return B.toString();
   }

   // Call to verify an EReference
   protected boolean VerifyEReference(EClassMap M, WDO4JSMappings Universe, IntWrapper Error) 
    {
      // Better have been called for an EReference
      if (isReference() == false)
       {
         Error.increment();
         Streamer.error.Header().println("EFeature '"+_EFeatureName+"' in EClass '"+M._EClassName+"' is supposed to be an EReference, but it is actually an EAttribute.");
         return false;
       }
      
      // EReferences cannot be of basic types
      if (TypesUtil.isBasicJavaType(_GetterType) == true || TypesUtil.isArrayOfBasicJavaType(_GetterType) == true)
       {
         Error.increment();
         Streamer.error.Header().println("EReference '"+_EFeatureName+"' in EClass '"+M._EClassName+"' has a basic type. EReferences cannot be of basic types.");
         return false;
       }

      // We check first that the _EFeatureName is not empty. An EMap can add attributes that are not in the ECore
      if (_EFeatureName == null || _EFeatureName.trim().length()==0)
      {
         Error.increment();
         Streamer.error.Header().println("An EFeatureMap was defined with an empty EAttributeName in EClassMap '"+M.getExport()+"'.");
         return false;                    
      }
     
      // EReferences cannot be of basic types
      String EFeatureEDataTypeName = TypesUtil.getRealETypeName(_EFeature.getEType());
      if (TypesUtil.isBasicEDataType(EFeatureEDataTypeName) == true)
       {
         Streamer.error.Header().println("EReference '"+_EFeatureName+"' in EClass '"+M._EClassName+"' is of type '"+EFeatureEDataTypeName+"'.  EReferences of basic types are not supported.");
         Error.increment();
         return false;
       }

      boolean OK = true;
      _ReferenceMap = Universe.getEClassMapByEPackageAndECoreComplexName(M.getEPackageName(), EFeatureEDataTypeName, _ExtraTargetType);
      if (_ReferenceMap == null) 
       {
         Streamer.error.Header().println("EReference '"+_EFeatureName+"' with EType '" + M.getEPackageName()+"`"+EFeatureEDataTypeName + "' was not found in the parent's Ecore or the EType is not mapped in the EMap.");
         Universe.PrintEClassMapByEcoreName();
         OK = false;
         Error.increment();   
       }
      return OK;
    }  
  
   // Call to verify an EAttribute
   protected boolean VerifyEAttribute(EClassMap M, WDO4JSMappings Universe, IntWrapper Error)
    {
      if (isReference() == true)
       {
         Error.increment();
         Streamer.error.Header().println("EFeature '"+_EFeatureName+"' in EClass '"+M._EClassName+"' is supposed to be an EAttribute, but it is actually an EReference.");
         return false;
       }

      // We check first that the _EFeatureName is not empty. An EMap can add attributes that are not in the ECore
      if (_EFeatureName == null || _EFeatureName.trim().length()==0)
       {
         Error.increment();
         Streamer.error.Header().println("An EFeatureMap was defined with an empty EAttributeName in EClassMap '"+M.getExport()+"'.");
         return false;                    
       }

      // EAttributes must be of basic types. If no feature is found, this is an EFeatureMap with an empty EFeatureName.
      // This happens when adding new features over what existed in the EClass, or with CalculateAttributes
      String EFeatureEDataTypeName = TypesUtil.getRealETypeName(_EFeature.getEType());
      if (TypesUtil.isBasicEDataType(EFeatureEDataTypeName) == false)
       {
         Streamer.error.Header().println("EFeature '"+_EFeatureName+"' in EClass '"+M._EClassName+"' is of type '"+EFeatureEDataTypeName+"'.  EAttributes must be of basic types.");
         Error.increment();
         return false;
       }
      
      boolean GetterList;
      try
       {
         GetterList = TypesUtil.isGeneralListNoArrays (_GetterType);
       }
      catch (ClassNotFoundException E)
       {
         Error.increment();
         Streamer.error.Header().println("Cannot verify that the class '"+_GetterType+"' exists");
         Streamer.error.Header().printStackTrace(E);
         return false;
       }

      // EAttributes must be of basic types, or array of basic type, or of a list type that is not an array
      // We have a relaxed rule for java.lang.Object or [Ljava.lang.Object; types. We trust that what will be
      // return will be correct.
      if (   TypesUtil.isBasicJavaType(_GetterType) == false
          && TypesUtil.isArrayOfBasicJavaType(_GetterType) == false 
          && GetterList == false
          && TypesUtil.isJavaObjectType(_GetterType, true) == false
         )
       {
         Error.increment();
         Streamer.error.Header().println("EAttribute '"+_EFeatureName+"' in EClass '"+M._EClassName+"' is of type '"+_GetterType+"'. EAttribute must be of basic types.");
         return false;
       }

      if (_EFeature != null)
       {
         EClassifier ec = _EFeature.getEType();
         //If feature is available, but, the EClassifier is not, then it is unsupported type
         if (ec == null)
          {
            Error.increment();
            Streamer.error.Header().println("UNSUPPORTED EFEATURE: " + _EFeature.getName()+".  The EClassifier associated with this feature could not be obtained.");                    
            return false;
          }
       }
      
      if (GetterList == false) // if we have a general list, we can't check anything, so we trust that it is OK. Otherwise, an error will happen at runtime
       {
         // We have to make sure that the tyoe of the getter, and the typs of the ECore match
         String EFeatureJavaTypeName = TypesUtil.getJavaTypeFromEDataType(EFeatureEDataTypeName, isMany());
         if (EFeatureJavaTypeName != null && _GetterType.equals(EFeatureJavaTypeName) == false )
          {
            Streamer.warning.Header().println("EAttribute '"+_EFeatureName+"' in EClass '"+M._EClassName+"' is a '"+_GetterType+"' that is different from the expected ecore type '"+EFeatureJavaTypeName+"'. If this is really an error, runtime cast errors will occur.");
          }
       }
      else
       {
         Streamer.warning.Header().println("EAttribute '"+_EFeatureName+"' in EClass '"+M._EClassName+"' is a '"+_GetterType+"'. We cannot validate the type of a container. If invalid, runtime cast errors will occur.");
       }

      return true;
    } 



  public boolean isContainment()
  {
   return _Containment;     
  }
  
  
  public EReference getEOpposite()
  {
   if ( isReference() && _EFeature != null )
   {
     return ((EReference) _EFeature).getEOpposite();
   }
   return null;
  }    

  /**
   * @deprecated - this method will be removed in Version Version 5.1.2
   */
  public PrintWriter printXML(PrintWriter out)
  {
   out.println("      <EFeatureMap efeature_name=\""+_EFeatureName+"\" iD=\""+_ID+"\" get=\""+_Getter+"\" />");
   return out;
  }
  
  public String MergeSetterCode(String SetObject)
   {
     String Setter = EFeatureMap.CleanupGetterSetter(_Setter);
     int i = Setter.indexOf(EFeatureMap.__SETTER_PARAM_MARKER);
     if (i == -1)
      {
        Streamer.defect.Header().println("Cannot find marker '"+EFeatureMap.__SETTER_PARAM_MARKER+"' in setter '"+_Setter+"'. This should have been verified.");
        throw new Error("Cannot find marker '"+EFeatureMap.__SETTER_PARAM_MARKER+"' in setter '"+_Setter+"'.");
      }
     return Setter.substring(0, i)+SetObject+Setter.substring(i+EFeatureMap.__SETTER_PARAM_MARKER.length());
   }
 }
