/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//
//OCO Source Materials
//
//PID xxxx-xxx
//
// Copyright IBM Corp.  2003
//
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////

package com.ibm.odcb.jrender.mediators.gen.ecore;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.Resource;

import com.ibm.odcb.jrender.mediators.gen.WDO4JSMappings;
import com.ibm.odcb.jrender.misc.EMFHelper;
import com.ibm.odcb.jrender.misc.IntWrapper;
import com.ibm.odcb.jrender.misc.Streamer;


public class EPackageMap
 {
   public EPackageMap(String Name)
    {
      _Name = Name;
    }
        
   protected String          _Name    ;
   protected EPackage        _EPackage;
   protected Resource        _ClientECoreResource;
   protected Map             _EClassMapsByJavaComplexName = new HashMap(7);
   protected String          _EMapResourceName ;
   protected String          _EMapXMLFileName  ;
   protected String          _RootEClassMapName;
   protected EClassMap       _RootEClassMap    ;
   protected WDO4JSMappings  _Universe         ;
   protected String          _OnUpdate ;
   protected String          _OnCreate ;
   protected String          _OnDelete ;
   protected String          _JSURI   ;

   
   public String          getName                       () { return _Name                       ; }
   public EPackage        getEPackage                   () { return _EPackage                   ; }
   public Resource        getClientECoreResource        () { return _ClientECoreResource        ; }
   public Map             getEClassMapsByJavaComplexName() { return _EClassMapsByJavaComplexName; }
   public String          getEMapXMLFileName            () { return _EMapXMLFileName            ; }
   public String          getEMapResourceName           () { return _EMapResourceName           ; }
   public EClassMap       getRootEClassMap              () { return _RootEClassMap              ; }
   public String          getRootEClassMapName          () { return _RootEClassMapName          ; }
   public WDO4JSMappings  getUniverse                   () { return _Universe                   ; }
   public String          getOnUpdate                   () { return _OnUpdate                   ; }
   public String		  getOnCreate					() { return _OnCreate					; }
   public String		  getOnDelete					() { return _OnDelete					; }
   public String          getJSURI                     () { return _JSURI                     ; }

   public void setName               (String         v) { _Name               = v; }
   public void setEPackage           (EPackage       v) { _EPackage           = v; }
   public void setClientECoreResource(Resource       v) { _ClientECoreResource= v; }
   public void setEMapXMLFileName    (String         v) { _EMapXMLFileName    = v; }
   public void setEMapResourceName   (String         v) { _EMapResourceName   = v; }
   public void setRootEClassMap      (EClassMap      v) { _RootEClassMap      = v; }
   public void setRootEClassMapName  (String         v) { _RootEClassMapName  = v; }
   public void setUniverse           (WDO4JSMappings v) { _Universe           = v; }
   public void setOnUpdate           (String         v) { _OnUpdate           = v; }
   public void setOnCreate           (String         v) { _OnCreate           = v; }
   public void setOnDelete           (String         v) { _OnDelete           = v; }
   public void setJSURI				 (String         v) { _JSURI             = v; }

   public boolean Verify(IntWrapper Error)
    {
      boolean OK = true;
      Iterator I = _EClassMapsByJavaComplexName.values().iterator();
      while (I.hasNext() == true)
       {
         EClassMap ECM = (EClassMap) I.next();
         Streamer.status.println()
                        .Header().println("Verifying Mapping for "+ECM.getEcoreName()+" with export name: " + ECM.getExport() +".");
         if (ECM.Verify(Error) == false)
          {
            Streamer.error.Header().println("Mapping for "+ECM.getEcoreName()+" with export name: " + ECM.getExport() + " failed verification.");
            OK = false;
          }
         if (_RootEClassMapName != null)
           if (_RootEClassMapName.equals(ECM.getEClassName()) == true)
            _RootEClassMap = ECM;
       }
      if (_RootEClassMap == null)
       {
      	Streamer.warning.Header().println("Could not find Root EClass '"+_RootEClassMapName+"'.  DiffHandlers will not be generated.");
      	//for backward compatibility, we can't force error here.
         //Error.increment();
         //OK = false;
       }
      return OK;
     }
   
   /**
    * Given an EPackage with a root EClass name X we check all other EClasses in that same EPackage that have the EClass X as a reference in
    * a multicardinality feature. If so (one and only class that references X, count, and with a multicardinality reference) then we return that new class as
    * the root of the package. Else we just return the root as originally specified.
    * @return
    */
   public EClassMap getRootEClassMapTransformed()
   {
     EClassMap RootECM = getRootEClassMap();
     if (RootECM.getBaseType().equals(EMFHelper.DataObject_CLASS.getName()) == false)
      return RootECM;
     Iterator I = _EClassMapsByJavaComplexName.values().iterator();
     int count = 0;
     EClassMap newRootECM = null;
     while (I.hasNext() == true)
      {
        EClassMap ECM = (EClassMap) I.next();
        if(ECM == RootECM)
         {
           continue;
         }
        Iterator iterator = ECM.getEFeatureMapIterator();
        while(iterator.hasNext())
         {
           EFeatureMap EFM = (EFeatureMap) iterator.next();
           if(   EFM.isReference() == true                                                                // Must be a reference
              && EFM.isContainment() == true && EFM.isMany() == true                                      // Must be multicardinality and containment
              && EFM.getReferenceMap().getJavaComplexName().equals(RootECM.getJavaComplexName()) == true  // Names must match
	       )
            {
              ++count;
              newRootECM = ECM;
            }
         }
      }
     if(count == 1 && newRootECM != null)
      {
        return newRootECM;
      }
     Streamer.warning.Header().println("Could not determine a transformed root EClass as we found multiple parent EClasses for '"+RootECM.getECoreComplexName()
                                      +"' with references where isContainment==true and isMany==true");
     return RootECM;
   }
 }
