/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//
//OCO Source Materials
//
//PID xxxx-xxx
//
// Copyright IBM Corp.  2003
//
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////

package com.ibm.odcb.jrender.mediators.gen.generators;

import java.util.HashMap;
import java.util.Iterator;

import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EDataType;

import com.ibm.odcb.jrender.mediators.gen.ecore.EClassMap;
import com.ibm.odcb.jrender.mediators.gen.ecore.EFeatureMap;
import com.ibm.odcb.jrender.misc.Config;
import com.ibm.odcb.jrender.misc.Streamer;
import com.ibm.odcb.jrender.misc.StringUtil;
import com.ibm.odcb.jrender.misc.TypesUtil;


public class FeatureExporter {
	
	protected EClassMap eClassMap;
	protected Object data;
	protected String eAttributesSchema = "";
	protected String eReferencesSchema = "";
	protected String eAttributesData;
	protected String eReferencesData;
	protected HashMap referenceTypes;
	protected final int     MEDIATOR_DEBUG_MODE = Config.getInteger(Config.PROP_MEDIATOR_DEBUG_MODE);
	protected final String  HEADER= MEDIATOR_DEBUG_MODE == 1 ? "\n                +\"\\n    ," : "\n                +\",";

	public FeatureExporter(EClassMap map)
	{
		this.eClassMap = map;
	}

	public FeatureExporter(EClassMap emap, Object data)
	{
		this.eClassMap = emap;
		this.data = data;
	}

    public void createSchemaString()
    {
		Iterator I = eClassMap.getEFeatureMapIterator();
		StringBuffer EAttributesParams = new StringBuffer(128);
		//the third parameter is to indicate iD attribute 1:true; 0:false
		EAttributesParams.append("[[\"xmi:id\",\"id\",0]");
		StringBuffer EReferencesParams = new StringBuffer(128);
		referenceTypes = new HashMap(17);
		while (I.hasNext() == true)
		 {
		  EFeatureMap ME = (EFeatureMap) I.next();
		  if ( ME.isCalculateAttribute() )
		  {
			//EAttributesParams.append(",[\\\""+ME.getExport()+"\\\",\\\""+ME.getCalculatedType()+"\\\",1,\\\""+ME.getCalculateExpression()+"\\\","+((int)(ME.isID()?1:0))+"]");
			EAttributesParams.append(",[\""+ME.getExport()+"\",\""+ME.getCalculatedType()+"\",1,"+StringUtil.QuoteDoubleAndEscape(ME.getCalculateExpression(),true)+","+((int)(ME.isID()?1:0))+"]");
		  }
		  else
		  {
		   String Type = TypesUtil.getSimplifiedTypeFromJavaType(ME.getGetterType());
		   Streamer.debug.Header().println("Found type '"+Type+"' for '"+ME.getGetter()+"' from '"+eClassMap.getExport()+"'.");
		   if (Type != null)
			{
			  EAttributesParams.append(",[\""+ME.getExport()+"\",\""+Type+"\",0,"+(ME.getEFeature()==null?0:ME.getEFeature().getLowerBound())+","+(ME.getEFeature()==null?1:ME.getEFeature().getUpperBound())+","+((int)(ME.isID()?1:0))+"]");
			}
		   // If this is not a common type, then it must be a custom type, and therefore, there must be a reference
		   else 
			{
			  EClassMap MR = ME.getReferenceMap();
			  if (MR != null)
			  {
				referenceTypes.put(MR, ME);
				/*if (referenceTypes.put(ME.getEFeatureType(), MR) == null)
				 {
				   Out.write("       super.ExportSchema(Out, Ctx, \""+MediatorFactory.GenTypeKey(MR.getComplexName(),_KEY)+"\");\n");
				 }*/
				EReferencesParams.append(",[\""+ME.getExport()+"\","+MR.getExport()+"Class,"+(ME.getEFeature()==null?0:ME.getEFeature().getLowerBound())+","+(ME.getEFeature()==null?1:ME.getEFeature().getUpperBound())+","+((int)(ME.isContainment()?1:0))+","+((int)(ME.isID()?1:0))+"]");
			  }
			  else
			  {
				boolean typeAsString = true;
				//Check if it's an EDataType of java.util.Date
				EClassifier ec = ME.getEFeature().getEType() ;
				if (ec instanceof EDataType)
				 {
					 Type = TypesUtil.getSimplifiedTypeFromJavaType(ME.getGetterType());
					 if (Type != null)
					 if (Type.equals("date"))
					 {
					   EAttributesParams.append(",[\""+ME.getExport()+"\",\""+Type+"\",0,"+(ME.getEFeature()==null?0:ME.getEFeature().getLowerBound())+","+(ME.getEFeature()==null?1:ME.getEFeature().getUpperBound())+","+((int)(ME.isID()?1:0))+"]");
					   typeAsString = false;
					 }
					
				 }
				 if (typeAsString)
				 {               
				  //TGupta Note: This is a reference, but the referenced object is not in the universe.  
				  //We will treat this as a String by calling a toString() on the object during data export.
				  //so this will be treated as an attribute of type String and not a reference.
				  EAttributesParams.append(",[\""+ME.getExport()+"\",\""+"string"+"\",0,"+(ME.getEFeature()==null?0:ME.getEFeature().getLowerBound())+","+(ME.getEFeature()==null?1:ME.getEFeature().getUpperBound())+","+((int)(ME.isID()?1:0))+"]");
				 }
			  }
			}
		   }
		 }
		 
		 eAttributesSchema = "A("+eClassMap.getExport()+"Class," + EAttributesParams.toString()+"]);\n";
		 if (EReferencesParams.length() != 0)
		  {
			EReferencesParams.setCharAt(0, '[');
			eReferencesSchema = "R("+eClassMap.getExport()+"Class," + EReferencesParams.toString()+"]);\n";
		  }
    }
    
	public static void main(String[] args) {
	}
	/**
	 * @return
	 */
	public String getEAttributesSchema() {
		return eAttributesSchema;
	}

	/**
	 * @return
	 */
	public String getEReferencesSchema() {
		return eReferencesSchema;
	}

	/**
	 * @return
	 */
	public HashMap getReferenceTypes() {
		return referenceTypes;
	}

	/**
	 * @param string
	 */
	public void setEAttributesSchema(String string) {
		eAttributesSchema = string;
	}

	/**
	 * @param string
	 */
	public void setEReferencesSchema(String string) {
		eReferencesSchema = string;
	}

	/**
	 * @param map
	 */
	public void setReferenceTypes(HashMap map) {
		referenceTypes = map;
	}

	/**
	 * @return
	 */
	public Object getData() {
		return data;
	}

	/**
	 * @return
	 */
	public EClassMap getEClassMap() {
		return eClassMap;
	}

	/**
	 * @param object
	 */
	public void setData(Object object) {
		data = object;
	}

	/**
	 * @param map
	 */
	public void setEClassMap(EClassMap map) {
		eClassMap = map;
	}

}
