/////////////////////////////////////////////////////////////////////////
//  IBM Confidential
//  
//  OCO Source Materials
//  
//  PID xxxx-xxx
//  
//   Copyright IBM Corp.  2003
//  
//  The source code for this program is not published or other-
//  wise divested of its trade secrets, irrespective of what has
//  been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////


package com.ibm.odcb.jrender.mediators.gen.generators;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Writer;
import java.util.HashMap;

import com.ibm.odcb.jrender.mediators.ExportException;
import com.ibm.odcb.jrender.mediators.MediatorFactory;
import com.ibm.odcb.jrender.mediators.gen.ecore.EClassMap;
import com.ibm.odcb.jrender.misc.FileUtil;
import com.ibm.odcb.jrender.misc.Streamer;

/**
 * @author ldh
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public abstract class Generator
 {
   public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;
   
   protected static final void GenWarning(Writer Out)
    {
      try
      {
        Out.write("//     *********************************************************\n");
        Out.write("//     ** THIS CODE IS GENERATED AND **MUST NOT** BE MODIFIED **\n");
        Out.write("//     *********************************************************\n");
      }
      catch (IOException e)
      {
        e.printStackTrace();
      }
    }

   protected final void GenVersionHistory(Writer Out)
    {
      try
      {
        Out.write("\n");
        Out.write("\n");
        Out.write("////////////////////////////////////////////////////////////////////////////////////////////////////\n");
        Out.write("////////////////////////////////////////////////////////////////////////////////////////////////////\n");
        Out.write("//\n");
        GenWarning(Out);
        Out.write("//\n");

        //////// -- ABSTACT -- ////////////////////
        PrintVersionHistory(Out);
        //////// -- ABSTACT -- ////////////////////

        Out.write("//\n");
        Out.write("////////////////////////////////////////////////////////////////////////////////////////////////////\n");
        Out.write("////////////////////////////////////////////////////////////////////////////////////////////////////\n");
        Out.write("\n");
        Out.write("\n");
        GenWarning(Out);
        Out.write("\n");
        Out.write("\n");
      }
      catch (IOException e)
      {
        e.printStackTrace();
      }    
   }
    
   public void genMediator(EClassMap ECM, String Dir, String Package, boolean ForceRegeneration) 
   throws FileNotFoundException, ExportException
    {
       // The extra name could have our special separator character. In case it does, we replace it with a '_'
       String ClassName = ECM.getEPackageName()+"_"+ECM.getExport()+getIdentifier();
       String FileName  = Dir+File.separator+ClassName+".java";
       Streamer.debug.Header().println("FileName ="+FileName+", and ClassName ="+ClassName);
       //TGupta Note (defect# 274304): delete the existing file.  This is to fix a Windows file system bug:
       //if a file exists with the same name but different case,
       //ex: Stock.java vs STOCK.java, they are considered the same file.
       //but, in Java, they are different files and you get compile errors.
       //so, delete the existing file so that a new one created.
       File file = new File(FileName);
       
       if (FileUtil.isOlder(file, ECM.getSourceEmapName()) == false && ForceRegeneration == false)
        {
          return;
        }
       if (file.exists() == true)
        {
          file.delete();
        }
       FileOutputStream fos = new FileOutputStream(FileName);
       PrintWriter Out = new PrintWriter(fos);
       GenVersionHistory(Out);
       Out.write("package "+Package+";\n");
       Out.write("\n");
       Out.write("import java.io.Writer;\n");
       //Out.write("import com.ibm.odcb.jrender.mediators.MediatorFactory;\n");
       Out.write("import com.ibm.odcb.jrender.mediators.Mediator;\n");
       Out.write("import com.ibm.odcb.jrender.mediators.PageContext;\n");
       Out.write("import com.ibm.odcb.jrender.mediators.ExportException;\n");
       Out.write("import com.ibm.odcb.jrender.misc.StringUtil;\n");
       Out.write("import com.ibm.odcb.jrender.misc.URLRewriter;");
       Out.write("import com.ibm.odcb.jrender.emitters.InitializationEmitter;");
       
    //////// -- ABSTACT -- ////////////////////
       AddOtherImports(Out);
    //////// -- ABSTACT -- ////////////////////
       
       Out.write("\n");
       GenWarning(Out);

       Out.write("\n");
       Out.write("public class "+ClassName+" extends Mediator\n");
       Out.write(" {\n");
       // LDH-NOTE: Gen code does not need the copyright
//       Out.println("   public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;");

    //////// -- ABSTACT -- ////////////////////
       String Delimiter = getArrayVarDelimiter();
    //////// -- ABSTACT -- ////////////////////

       Out.write("   public "+ClassName+"() { super(\""+ECM.getExport()+"\", \""+Delimiter+"\"); }");//this was a print(), so no \n


       ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
       // getSignature method
       Out.write("\n");
       Out.write("   public String getSignature(Object O, String modelName, boolean First, int ComplexPropertyCount)\n");
       Out.write("    {\n");
       Out.write("      if (O == null) return \"\";\n");
       Out.write("      try\n");
       Out.write("       {\n");
       // We need to do some transformation in case the type pointed to is for a nested class. A '$' will apear in the name, but 
       // when generating code, we want to output a '.'.
       String BaseType = ECM.getBaseType().replace('$','.');
       Out.write("         "+BaseType+" Obj = ("+BaseType+") O;\n");
       //append modelName here
       Out.write("         StringBuffer SigStr = new StringBuffer(128);\n");
       String S = ECM.getKeySignature("Obj");
       Out.write(S);
       Out.write("\n");
       Out.write("         return First == true ? modelName+\"`"+ECM.getJavaComplexName()+"\"+SigStr.toString() : SigStr.toString();\n");
//       Streamer.debug.Header().println("Root Signature key = " + M.getComplexName()+"`"+S.toString());

       Out.write("       }\n");
       Out.write("      catch (Exception e)\n");
       Out.write("       {\n");
       Out.write("         if (e instanceof java.lang.ClassCastException)\n");
       Out.write("          {\n");
       Out.write("            System.out.println(\"ClassCastException: Expecting type '"+ECM.getBaseType()+"' but found object of type '\"+O.getClass().getName()+\"'\"); \n");
       Out.write("            System.out.println(\"Please make sure that the ecore and emap are defined correctly.\"); \n");
       Out.write("          }\n");
       Out.write("         e.printStackTrace();\n");
       Out.write("         return null; \n");
       Out.write("       }\n");           
       Out.write("    }\n");
       
       ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
       // Export (Schema) method
       Out.write("\n");
       GenWarning(Out);
       Out.write("\n");
       Out.write("   public void Export(Writer Out, PageContext Ctx, Object O) throws ExportException\n");
       Out.write("     {\n");
       //add try catch block here for the Exception thrown by writer.write()
       Out.write("      try\n");
       Out.write("      {\n");
    //////// -- ABSTACT -- ////////////////////
       GenSchema(Out, ECM);
    //////// -- ABSTACT -- ////////////////////
       Out.write("      }\n");
       Out.write("      catch (Exception e)\n");
       Out.write("      {\n");
       Out.write("        e.printStackTrace();\n");
       Out.write("      }\n");
       Out.write("     }\n");
       ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
       // XMILoaderExport method
      Out.write("\n");
      GenWarning(Out);
      Out.write("\n");
      Out.write("   public void ExportXMILoader(Writer Out, PageContext Ctx) throws ExportException");
      Out.write("     {\n");
      Out.write("		boolean isFaces = StringUtil.checkEncoding();\n");
      Out.write("		String export_var= StringUtil.encodeString(\""+ECM.getExport()+"\",isFaces);\n");
      Out.write("		String schema_var=StringUtil.encodeString(\"WDO4JSModel_\"+Ctx.getSchemaExportName(),isFaces);\n");
      Out.write("		String root_var= StringUtil.encodeString(\"WDO4JSModelRoot_\"+Ctx.getLastModelName(),isFaces);\n");
      Out.write("		String model_var = StringUtil.encodeString(\"WDO4JSMR_\"+Ctx.getLastModelName(),isFaces); \n");
      Out.write("		String class_var = StringUtil.encodeString(\""+ECM.getExport()+"Class\",isFaces);\n");
      Out.write("      ");
      Out.write("      ");
      //add try catch block here for the Exception thrown by writer.write()
      Out.write("      try\n");
      Out.write("      {\n");
      Out.write("        boolean FirstTime = Ctx.isFirstSchemaExport();\n");
      Out.write("        if (FirstTime == true)\n");
      Out.write("        {\n");
      Out.write("          Out.write(\"<script type=\\\"text/javascript\\\">\");\n");
      Out.write("          Out.write(\"var \"+schema_var); Out.write(\"=new EClass(\\\"\");\n" );
      Out.write("		   Out.write(model_var); Out.write(\"\\\");\\n\");\n");
      Out.write("          Out.write(\"var \"+ root_var); Out.write(\"=new XMILoader(\"+schema_var); Out.write(\", '\"+export_var+\"', '"+ECM.getClientEcoreName()+"');\\n\");\n");
      Out.write("          Out.write(\"</script>\\n\");\n");
      Out.write("         }\n");
      Out.write("         Out.write(\"var \"+class_var+\"=new EClass(\\\"\"+export_var+\"\\\","+(ECM.isDiffOnRefresh()?1:0)+");\"); \n");
      Out.write("      }\n");
      Out.write("      catch (Exception e)\n");
      Out.write("      {\n");
      Out.write("        e.printStackTrace();\n");
      Out.write("      }\n");
      Out.write("     }\n");
       ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
       // Export (Data) method
       Out.write("\n");
       GenWarning(Out);
       Out.write("\n");
       Out.write("   public void Export(Writer Out, PageContext Ctx, Object O, String Var, boolean DebugMode) throws ExportException\n");
       Out.write("     {\n");
       //add try catch block here for the Exception thrown by writer.write()
       Out.write("      try\n");
       Out.write("        {\n");
       Out.write("       "+BaseType+" Obj = ("+BaseType+") O;\n");
       Out.write("\n");

    //////// -- ABSTACT -- ////////////////////
       GenData(Out, ECM);
    //////// -- ABSTACT -- ////////////////////
       Out.write("      }\n");
       Out.write("      catch (Exception e)\n");
       Out.write("      {\n");
       Out.write("        e.printStackTrace();\n");
       Out.write("      }\n");
       Out.write("     }\n");       
       Out.write(" }\n"); // end class

       Out.close();
   }


   public          void   AddOtherImports     (Writer Out           )  {              }
   public          String getArrayVarDelimiter(                          )  { return ",";  }

   public abstract String getIdentifier       (                          );
   public abstract void   PrintVersionHistory (Writer Out           );
   public abstract void   GenSchema           (Writer Out, EClassMap M);
   public abstract void   GenData             (Writer Out, EClassMap M);
   
 
   public static Generator getGenerator(String TypeKey)
    {
      return (Generator) _Generators.get(TypeKey);
    }
    
   public static final String[] MEDIATOR_KEYS = { MediatorFactory.GenTypeKey("Object", Object_to_WDO4JS_as_JS_compact._KEY )
                                                , MediatorFactory.GenTypeKey("Object", Object_to_WDO4JS_as_XMI       ._KEY )
                                                };
   public static final Generator[] GENS = { new Object_to_WDO4JS_as_JS_compact()
                                          , new Object_to_WDO4JS_as_XMI       ()
                                          };

   protected static HashMap _Generators = new HashMap(57);
   public static void Init()
    {
      for (int i = 0; i < MEDIATOR_KEYS.length; ++i)
       {
         _Generators.put(MEDIATOR_KEYS[i], GENS[i]);
       }
    }
    
/*
   public static char resolveTypeInfo(EFeatureMap map)
    {
      if (map == null) 
       return TypesUtil.TypeInfo._DEFAULT_OUT;
     
      // LDH-NOTE: Here, output needs to be based on type... String types with quotes, and non string types without.... 
     //and then DATE !!!!
     //TGupta NOTE: Priority is given to the ecore type, if ecore is null, only then _GetType is used
     //String Type = (ME._EFeature==null?ME._GetType:EFeatureMap.getJavaTypeNameForEDataType(TypesUtil.getRealEType(ME._EFeature.getEType()));
             
     //TGupta 10/30/03: Priority cannot be given to ecore type; if there's a mismatch between ecore type and get type,
     //the generated code will not compile!  We must base Type on getType.  Additonal check can be done for
     //ecore type of String, but, other get types
     char getType = TypesUtil.getOutputTypeFromJavaType(map.getGetType());
     //TGupta: check for a mismatch between ecore and getType
     if (map.getEFeature()!=null)
      {
       if (!map.getGetType().equals(TypesUtil.getJavaTypeFromEDataType(map.getEFeatureType(), map.isMany())))
        {
          char ecoreType = TypesUtil.getOutputTypeFromJavaType(TypesUtil.getJavaTypeFromEDataType(map.getEFeatureType(), map.isMany()));
          //if ecore defines it as String ('S'), but get is some object (' '), a toString() must be attempted
          if (ecoreType == TypesUtil.TypeInfo._STRING_OUT && getType != TypesUtil.TypeInfo._STRING_OUT)
           {
            //toString() will fail for primitive types, so, dont call StringUtil.QuoteDoubleAndEscape for primitives
               getType = (TypesUtil.isBasicJavaType(map.getGetType()) == true) ? TypesUtil.TypeInfo._NUMBER_OUT : TypesUtil.TypeInfo._DEFAULT_OUT;
           }
        }
      }
      return getType;
    }
*/
 }
