/////////////////////////////////////////////////////////////////////////
//  IBM Confidential
//  
//  OCO Source Materials
//  
//  PID xxxx-xxx
//  
//   Copyright IBM Corp.  2003
//  
//  The source code for this program is not published or other-
//  wise divested of its trade secrets, irrespective of what has
//  been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////

package com.ibm.odcb.jrender.misc;

import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * @author ldh
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class ClassGraphWalker
 {
   public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;
   public static interface Walker
    {
      public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;
      public boolean Begin                           (Class C) throws Exception;
      public boolean ProcessClassBegin               (String Path, Class C, int Level) throws Exception;
      public boolean ProcessBasicProperty            (String Path, Class C, String PropertyName, String PropertyType, boolean Many, boolean ReadOnly, int Level) throws Exception;
      public boolean ProcessComplexProperty          (String Path, Class C, String PropertyName, String PropertyType, boolean Many, boolean ReadOnly, boolean PropertyFound, int Level) throws Exception;
      public boolean ProcessClassEnd                 (String Path, Class C, boolean PropertyFound, int Level) throws Exception;
      public boolean End                             (Class C) throws Exception;
    }

   public ClassGraphWalker(String RootClassName, Map Mappings, boolean Include, Map IncludesExcludes)
    {
      _RootClassName    = RootClassName;
      _Mappings         = Mappings;
      _Include          = Include;
      _IncludesExcludes = IncludesExcludes;
      // If we are passed an empty list of IncludesExcludes, and the flag say it's Includes, then it's
      // obviously wrong, so we switch the flag to Exclude (Include==false).
      if (_IncludesExcludes ==  null)
      {
		_IncludesExcludes = new HashMap();
		_Include = false;
      }
      else if (_IncludesExcludes.size() == 0 && _Include == true)
       _Include = false;
    }
   public String  _RootClassName;
   public Map     _Mappings;
   public Map     _IncludesExcludes;
   public boolean _Include;
   public Map     _VisitedClasses = new HashMap(17);
   public Walker  _W;
   public int     _Level;
   
   public void Walk(Walker W)
   throws Exception
    {
      _W = W;
      Class Root = ODCClassLoader.getClass(_RootClassName);
      _W.Begin(Root);
      WalkInternal(_RootClassName.substring(_RootClassName.lastIndexOf('.')+1), Root, 0);
      _W.End(Root);
    }

   public static final String CLASS_FLAG     = "C";
   public static final String PROPERTY_FLAG  = "P";
   
   public final boolean isIn(String Name, String ClassOrPropertyFlag)
    {
      // If we were given an Include list, and the property or class can tbe found there, then
      // the property or class is in, else it's out. If we were given an Exclude list, it's the 
      // opposite: if we found it, then it's out, otherwise, it's in.
      String Val = (String) _IncludesExcludes.get(Name);
      // True if the property or class was found in the hashmap, and it matches the flag expected as
      // we could have the "com.ibm.A.B" where B is a class, or B is a property of class A
      boolean InHashMap = Val != null && Val.equals(ClassOrPropertyFlag) == true;
      return _Include == true ? InHashMap : !InHashMap;
    } 
   
   public boolean WalkInternal(String Path, Class C, int Level)
   throws Exception
    {
      if (isIn(C.getName(), CLASS_FLAG) == false) // Class is to be exluded
       {
         Streamer.debug.Header().println("Class "+C.getName()+" is excluded from the ECore.");
         return false;
       }

      if (_VisitedClasses.put(C.getName(), C) != null)  // Check for loopy references of real classes
       return true;
      if (_VisitedClasses.put("!!!"+C.getName(), C) != null)  // Check for loopy references of empty classes
       return false;
       
      _W.ProcessClassBegin(Path, C, Level);
      
      PropertyDescriptor[] D = Introspector.getBeanInfo(C).getPropertyDescriptors();
      boolean PropertyFound = false;
      for (int i = 0; i < D.length; i++)
       {
         PropertyDescriptor PD = D[i];
         if (PD.getPropertyType() != null)
          {
            PropertyFound = ProcessProperty(PD.getName(), PD.getPropertyType().getName(), 
                                            PD.getWriteMethod() == null, PropertyFound, Path, Level, C);
          } 
       }

      if (_W.ProcessClassEnd(Path, C, PropertyFound, Level) == false) // The class was deemed empty
       {
         _VisitedClasses.remove(C.getName());        // remove the class in the hashmap
         _VisitedClasses.put("!!!"+C.getName(), C);  // and mark it as an empty class
         return false;
       }
      return true;
    }
    
   public boolean ProcessProperty(String Name, String Type, boolean ReadOnly, boolean PropertyFound, String Path, int Level, Class C)
   throws Exception
    {
      // The rule is that PropertyFound should only be overwritten if we do find a property. The value starts
      // off as false, and should only be changed if a property is found.
      if (TypesUtil.isValidProperty(Name) == false)
       return PropertyFound;
      if (isIn(Type, CLASS_FLAG) == false)
       {
         Streamer.debug.Header().println("Class "+Type+" is excluded from the ECore.");
         return PropertyFound;
       }
      String PropertyPath = C.getName()+"."+Name; 
      if (isIn(PropertyPath, PROPERTY_FLAG) == false)
      {
        Streamer.debug.Header().println("Property "+PropertyPath+" is excluded from the ECore.");
        return PropertyFound;
      }
      

      if (TypesUtil.isBasicJavaType(Type) == true)  // simple property of basic type
       {
         if (_W.ProcessBasicProperty(Path+'.'+Name, C, Name, Type, false, ReadOnly, Level) == true)
          PropertyFound = true;
       }
      else if (TypesUtil.isArrayOfBasicJavaType(Type) == true) // Array of basic type
       {
         Type = TypesUtil.getJavaTypeFromArrayType(Type);
         if (Type != null)
          {
            if (_W.ProcessBasicProperty(Path+'.'+Name, C, Name, Type, true, ReadOnly, Level) == true)
             PropertyFound = true;
          }
       }
      else if (TypesUtil.isArrayOfNonPrimitiveJavaType(Type) == true) // Array some classes
       {
         Type = Type.substring(2,Type.length()-1);
         boolean ret = WalkInternal(Path+'.'+Name, ODCClassLoader.getClass(Type), Level+1);
         if (_W.ProcessComplexProperty(Path+'.'+Name, C, Name, Type, true, ReadOnly, ret, Level) == true)
          PropertyFound = true;
       }
      else if (TypesUtil.isJavaObjectType(Type, true) == true)  // Object or array of Object. What's the real type?
       {
         String MappedType = (String) _Mappings.get(PropertyPath);  // check for a mapping to find the type
         if (MappedType != null && TypesUtil.isJavaObjectType(MappedType, true) == false)
          {
            PropertyFound = ProcessProperty(Name, Type.equals(Object.class.getName()) == true ? MappedType:"[L"+MappedType+";",
                                            ReadOnly, PropertyFound, Path, Level, C);
          }
         else
          {
            Streamer.warning.Header().println("Property '"+Name+"' of type '"+Type+"' doesn't have an explicit type mapping. We will treat it as a basic String. If this is not a basic type or a complex type with an appropriate toString() method, then the internal Java object signature will be output.");
            // LDH-NOTE: We need to support basic types presented as Object. for instance, in JSF, there is an object
            //      that returns a value of type Object. Underneath the covers, it's really a basic Java type. If the property
            //      is in fact of a complex type, then it better have a toString method, otherwise, the internal Java object 
            //      signature will be output as a String. 
			   PropertyFound = ProcessProperty(Name, Type.equals(Object.class.getName()) == true ? "java.lang.String":"[Ljava.lang.String;",
										              ReadOnly ,PropertyFound, Path, Level, C);
          }
       }
      else if (TypesUtil.isGeneralListNoArrays(Type) == true)  // Collection
       {
         String MappedType = (String) _Mappings.get(PropertyPath);  // check for a mapping to find the type
         if (MappedType != null)
          {
            if (TypesUtil.isBasicJavaType(MappedType) == true)  // collection of simple type
             {
               if (_W.ProcessBasicProperty(Path+'.'+Name, C, Name, MappedType, true, ReadOnly, Level) == true)
                PropertyFound = true;
             }
            else  // collection of complex type
             {
               boolean ret = WalkInternal(Path+'.'+Name, ODCClassLoader.getClass(MappedType), Level+1);
               if (_W.ProcessComplexProperty(Path+'.'+Name, C, Name, MappedType, true, ReadOnly, ret, Level) == true)
                PropertyFound = true;
             }
          }
         else
          {
            Streamer.warning.Header().println("Property '"+Name+"' of type '"+Type+"' doesn't have an explicit type mapping. This property will be skipped as a result because we do not know what type it really is.");
          }
       }
      else // Complex Type
       {
         boolean ret = WalkInternal(Path+'.'+Name, ODCClassLoader.getClass(Type), Level+1);
         if (_W.ProcessComplexProperty(Path+'.'+Name, C, Name, Type, false, ReadOnly, ret, Level) == true)
          PropertyFound = true;
       }
      return PropertyFound;
    }
     
   public static class TestWalker implements ClassGraphWalker.Walker
    {
      public boolean Begin(Class C)
       {
         return true;
       }

      public boolean ProcessClassBegin(String Path, Class C, int Level)
       {
         Streamer.debug.println(C.getName());
         Method[] M = C.getMethods();
         for (int i = 0; i < M.length; ++i)
          {
            String Name = M[i].getName();
            Streamer.debug.print("|_ "+Name+"(");
            Class[] Params = M[i].getParameterTypes();
            for (int j = 0; j < Params.length; ++j)
             {
               if (j != 0)
                Streamer.debug.print(", ");
               Streamer.debug.print(Params[j].getName());
             }
            Streamer.debug.println(")");
          }
         return true;
       }

      public boolean ProcessBasicProperty(String Path, Class C, String PropertyName, String PropertyType, boolean Many, boolean ReadOnly, int Level)
       {
         Streamer.debug.Header().println(StringUtil._PADS[Level] + StringUtil._PADS[Level] + " > " + Path +" ("+PropertyName+"#"+PropertyType+(Many == true ? "[])" : ")") );
         return true;
       }

      public boolean ProcessComplexProperty(String Path, Class C, String PropertyName, String PropertyType, boolean Many, boolean ReadOnly, boolean PropertyFound, int Level)
       {
         Streamer.debug.Header().println(StringUtil._PADS[Level] + StringUtil._PADS[Level] + " > " + Path +" ("+PropertyName+"#"+PropertyType+(Many == true ? "[])" : ")") );
         return true;
       }

      public boolean ProcessClassEnd(String Path, Class C, boolean PropertyFound, int Level)
       {
//         Streamer.debug.Header().println(StringUtil._PADS[Level] + StringUtil._PADS[Level] + Path +" ("+C.getName()+")");
         return true;
       }

      public boolean End(Class C)
       {
//         Streamer.debug.Header().println(StringUtil._PADS[Level] + StringUtil._PADS[Level] + Path +" ("+C.getName()+")");
         return true;
       }
    }

   public static class Test
    {
      public char   [] getCharArray      () { return new char   [10]; }
      public boolean[] getBooleanArray   () { return new boolean[10]; }
      public double [] getDoubleArray    () { return new double [10]; }
      public float  [] getFloatArray     () { return new float  [10]; }
      public int    [] getIntArray       () { return new int    [10]; }
      public long   [] getLongArray      () { return new long   [10]; }
      public short  [] getShortArray     () { return new short  [10]; }
      public byte   [] getByteArray      () { return new byte   [10]; }
      public Byte   [] getByteObjectArray() { return new Byte   [10]; }
      public String [] getStringArray    () { return new String [10]; }
      public Iterator   getStringIterator  () { return new ArrayList().iterator(); }
      public ArrayList  getStringArrayList () { return new ArrayList(); }
      public List       getStringList      () { return new ArrayList(); }
      public Collection getStringCollection() { return new ArrayList(); }
    }

   public static void main(String[] args)
    {
      Streamer.setLogLevel(3);
      try
       {
         ClassGraphWalker.Walker W = new TestWalker();
         ClassGraphWalker CGW = new ClassGraphWalker("com.ibm.odcb.jrender.misc.ClassGraphWalker$Test", 
                                                     new HashMap(), false, new HashMap());
         CGW.Walk(W);
         Streamer.Separator();
         CGW = new ClassGraphWalker("java.lang.Class", new HashMap(), false, new HashMap());
         CGW.Walk(W);             
       }
      catch (Exception E)
       {
         Streamer.error.Header().printStackTrace(E);
       }

      Streamer.status.println().Header().println("test done");
      Streamer.EndThread();
    }
 }
