/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//
//OCO Source Materials
//
//PID xxxx-xxx
//
// Copyright IBM Corp.  2003
//
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////

package com.ibm.odcb.jrender.misc;

import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.util.ArrayList;
import java.util.HashMap;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EReference;
////#IF# SDO
//import org.eclipse.emf.ecore.sdo.impl.DynamicEDataObjectImpl;
////#END#
////#IF# WDO
import com.ibm.etools.wdo.datagraph.impl.DynamicDataObjectImpl;
////#END#

public class ClassIntrospector {

	
	HashMap _visitedClasses;
	ArrayList _matchedClasses;
	
	public ClassIntrospector()
	{
		_visitedClasses = new HashMap();
		_matchedClasses = new ArrayList();
	}
	
	/*
	 * This method introspects the beanClass, and recursively goes through it
	 * looking for the attribute passed in attributeName.
	 * When it finds it, it returns the Class of the attribute.
	 */
	public String getAttributeClassName(Class beanClass, String attributeName)
	{
		// Stop inspecting if this class has been inspected before./
		// This is to prevent circular references causing an
		// infinite loop.
		if (_visitedClasses.put(beanClass, new Boolean(true)) != null)
		{
			return null;
		}
		
		String theClassName = null;
		try {
			PropertyDescriptor[] pd;
			if (beanClass.getComponentType() != null)
			{
				pd = Introspector.getBeanInfo(beanClass.getComponentType()).getPropertyDescriptors();
			}
			else
			{	
				pd = Introspector.getBeanInfo(beanClass).getPropertyDescriptors();
			}
		
			for (int j=0; j<pd.length; j++)
			{
				if (pd[j].getName().equals(attributeName))
				{
					String fullClassName = pd[j].getPropertyType().getName();
					theClassName = fullClassName.charAt(fullClassName.length()-1) == ';'?fullClassName.substring(fullClassName.lastIndexOf(".")+1,fullClassName.length()-1):fullClassName.substring(fullClassName.lastIndexOf(".")+1);
					return theClassName;
				}
			}
			pd = null;
			if (beanClass.getComponentType() != null)
			{
				pd = Introspector.getBeanInfo(beanClass.getComponentType(),beanClass.getComponentType().getClass().getSuperclass()).getPropertyDescriptors();
			}
			else if(!beanClass.getClass().isPrimitive())
			{	
				pd = Introspector.getBeanInfo(beanClass,beanClass.getClass().getSuperclass()).getPropertyDescriptors();
			} 
			if (pd !=null) {
				for (int j=0; j<pd.length; j++)
				{
					Class theClass = pd[j].getPropertyType();
					theClassName = getAttributeClassName(theClass,attributeName);
					
					if (theClassName != null)
					{
						return theClassName;
					}
				} 
			
			}
		}
		catch (Exception ex)
		{
			//System.out.println(ex.toString());
		}
		
		return theClassName;
	}
	
	////#IF# SDO
//	public String getEClassNamefromObject(DynamicEDataObjectImpl eObj, String refName)
	////#END#
	////#IF# WDO
        public String getEClassNamefromObject(DynamicDataObjectImpl eObj, String refName)
        ////#END#
	{
		
		EClass eClass = eObj.eClass();
		String name = eClass.getName().toString();
		
		if (name.equals(refName))
		{
			return name;
		}
		
		
		
		
		if (name != refName)
		{
			EReference nextClass = ((EReference)eClass.getEReferences().get(0));
			
			String nextName = getEClassNamefromReference(nextClass,refName);
			return nextName;
		}
		
		return null;
	}
	
	
	private String getEClassNamefromReference(EReference eRef, String refName) 
	{
		String referenceName = eRef.getName();
		
		if (_visitedClasses.put(referenceName, new Boolean(true)) != null)
		{
			return null;
		}
		
		
		if (referenceName.equals(refName))
		{
			String eClassName = eRef.getEType().getName();
			
			
			
			return eClassName;
		}
	
		
		EClass eClass = (EClass)eRef.getEType();
			
	//	EReference nextClass = ((EReference)eClass.getEReferences().get(0));
		
		
		
		for (int i=0; i<eClass.getEReferences().size();i++)
		{	
			EReference nextRef = (EReference)eClass.getEReferences().get(i);
			
			String nextName = getEClassNamefromReference(nextRef,refName);
			if (nextName != null)
			{
				return nextName;
			}
		}	
		
		
		return null;
	}
	
	
	
	
}
