/////////////////////////////////////////////////////////////////////////
//  IBM Confidential
//  
//  OCO Source Materials
//  
//  PID xxxx-xxx
//  
//   Copyright IBM Corp.  2003
//  
//  The source code for this program is not published or other-
//  wise divested of its trade secrets, irrespective of what has
//  been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////


/*
 * Compiler.java
 *
 * Created on November 19, 2000, 1:24 PM
 */

package com.ibm.odcb.jrender.misc;


import java.io.File;
import java.io.FileNotFoundException;

public class CompilerUtil extends Object
 {
   public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;
   
   public CompilerUtil() 
    {
    }

   protected static String CLASSPATH = null;
   
   public static boolean Compile(String[] Files, String CompilerCommandLine)
    {
      if (CLASSPATH == null)
       CLASSPATH = System.getProperty("java.class.path");
        
      StringBuffer CommandLine = new StringBuffer(1024);
      CommandLine.append(CompilerCommandLine);
      CommandLine.append(" -nowarn -classpath \"").append(CLASSPATH).append("\"");
      for (int i = 0; i < Files.length; ++i)
       {
         CommandLine.append(' ');
         CommandLine.append(Files[i]);
       }

      Streamer.status.Header().println("Executing the program: "+CommandLine);

      int i = ProcessExecution.Exec(CommandLine.toString(), "Compiler errors", 50, 1024, 2048);
      if (i != 0)
       {
         Streamer.error.Header().println("Cannot execute the program '"+CommandLine.toString()+"'.")
                       .Header().println("Please check your path/configuration.");
       }
      return i==0;
    }
    
   public static boolean Compile(String sourcePath, String destinationPath, String[] Files, String CompilerCommandLine)
    {
      boolean nullSourcePath      = sourcePath      == null || sourcePath     .trim().equals("");
      boolean nullDestinationPath = destinationPath == null || destinationPath.trim().equals("");
      
      if (nullSourcePath && nullDestinationPath)
        return Compile(Files, CompilerCommandLine);
     
      if (CLASSPATH == null)
       CLASSPATH = System.getProperty("java.class.path");
        
      StringBuffer CommandLine = new StringBuffer(1024);
      CommandLine.append(CompilerCommandLine);
      CommandLine.append(" -nowarn -classpath \"").append(CLASSPATH).append("\"");
      if (destinationPath.endsWith("\\"))
        destinationPath = destinationPath.substring(0,destinationPath.lastIndexOf('\\'));
      CommandLine.append(" -d \"" + destinationPath + "\"");
      for (int i = 0; i < Files.length; ++i)
      {
       CommandLine.append(' ');
       CommandLine.append(Files[i]);
      }

      Streamer.status.Header().println("Executing the program: "+CommandLine);
        
      if (nullSourcePath)
      {
        return ProcessExecution.Exec(CommandLine.toString(), "Compiler errors", 50, 1024, 2048) == 0;
      }
      else
      {
        File file = new File(sourcePath);
        int i = ProcessExecution.Exec(CommandLine.toString(), null, file, "Compiler errors", 50, 1024, 2048);
        if (i == SystemValues.EVIL_VALUE)
         {
           Streamer.error.Header().println("Cannot execute the program '"+CommandLine.toString()+"'.")
                         .Header().println("Please check your path/configuration.");
           throw new Error("Cannot execute the program '"+CommandLine.toString()+"'.");
         }
        return i==0;
      }
    }
    
   public static void CreatePath(String Path)
    {
       File F = new File(Path);
       F.mkdirs();
    }
    
   public static boolean CheckUpToDate(String Base, File Check)
   throws Exception
    {
      File B = new File(Base);
      return CheckUpToDate(B, Check);
    }

   public static boolean CheckUpToDate(File Base, String Check)
   throws Exception
    {
      File C = new File(Check);
      return CheckUpToDate(Base, C);
    }

   public static boolean CheckUpToDate(String Base, String Check)
   throws Exception
    {
      File B = new File(Base);
      File C = new File(Check);
      return CheckUpToDate(B, C);
    }

   public static boolean CheckUpToDate(File Base, File Check)
   throws Exception
    {
      long BaseTime  = Base .lastModified();
      long CheckTime = Check.lastModified();
   
      if (BaseTime == 0)
       {
         Streamer.status.Header().println("File " + Base.getAbsolutePath() + " cannot be found");
         throw new FileNotFoundException(Base.getAbsolutePath());
       }
      if (BaseTime > CheckTime)
       { 
         Streamer.debug.Header().println("The base file " + Base.getAbsolutePath() 
                                          + " has changed after the check file "
                                          + Check.getAbsolutePath());
         return false;
       }
      Streamer.debug.Header().println("The check file " + Check.getAbsolutePath() + " is up to date");
      return true;
    }
    
 }