/////////////////////////////////////////////////////////////////////////
//  IBM Confidential
//  
//  OCO Source Materials
//  
//  PID xxxx-xxx
//  
//   Copyright IBM Corp.  2003
//  
//  The source code for this program is not published or other-
//  wise divested of its trade secrets, irrespective of what has
//  been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////


/*
 * Config.java
 *
 * Created on May 14, 2000, 2:43 AM
 */
 
package com.ibm.odcb.jrender.misc;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.MissingResourceException;
import java.util.Properties;

import javax.faces.context.FacesContext;


import com.ibm.faces.util.ContextParams;
import com.ibm.faces.util.LifecycleUtil;


/** 
 *
 * @author  ldh
 * @version 
 */

public class Config
 {
	public static final String VERSION_NUMBER = "v3_0_4";
	
	public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;

   public static class PROPERTY
    {
      PROPERTY(String Name)
       {
       	 _N = Name;
       }
      protected String _N;
      public String toString() { return _N; } 
    }
   public static final PROPERTY PROP_ECORE_FILES         = new PROPERTY("ECORE_FILES"         );
   public static final PROPERTY PROP_EMAP_FILES          = new PROPERTY("EMAP_FILES"          );
   public static final PROPERTY PROP_CLIENT_LOCALE       = new PROPERTY("CLIENT_LOCALE"       );
   //public static final PROPERTY PROP_CLIENT_STYLESHEETDIR= new PROPERTY("CLIENT_STYLESHEETDIR");
   public static final PROPERTY PROP_CLIENT_DEBUG_MODE   = new PROPERTY("CLIENT_DEBUG_MODE"   );
   public static final PROPERTY PROP_CLIENT_LOG_LEVEL    = new PROPERTY("CLIENT_LOG_LEVEL"    );
   public static final PROPERTY PROP_SERVER_LOG_LEVEL    = new PROPERTY("SERVER_LOG_LEVEL"    );
   public static final PROPERTY PROP_MEDIATOR_DEBUG_MODE = new PROPERTY("MEDIATOR_DEBUG_MODE" );
   public static final PROPERTY PROP_SERVER_LOG_FILENAME = new PROPERTY("SERVER_LOG_FILENAME" );
   public static final PROPERTY PROP_REQUEST_CLASS       = new PROPERTY("REQUEST_CLASS"       );
   public static final PROPERTY PROP_PROGRESSBAR         = new PROPERTY("PROGRESSBAR"         );
   public static final PROPERTY PROP_PATH_PREFIX         = new PROPERTY("PATH_PREFIX"         );

   public static final PROPERTY PROP_SYSTEM_ECORE_FILES   = new PROPERTY("SYSTEM_ECORE_FILES");


   public Config()
    {
    }

   protected static synchronized Properties InternalInit()
    {
      try
       {
         InputStream Input;
         if (IS_RESOURCE == true)
          Input = ODCClassLoader.getResourceAsStream(PROPERTIES_FILE_NAME);
         else
          Input = new FileInputStream(PROPERTIES_FILE_NAME);
         
         //Streamer.debug.Header().println("Loading properties from file: "+PROPERTIES_FILE_NAME);
         Properties Props = new Properties();
         Props.load(Input);
         Input.close();
         return Props;
       }
      catch (Exception e)
       {
		   //Streamer.error.Header().println("ERROR  : Properties file '"+PROPERTIES_FILE_NAME+"' is missing or cannot be read properly");
		//Streamer.error.Header().printStackTrace(e);
         return null;
       }
    }
   
   public static void ChangePropertiesFileAndReInit(String PropertiesFile, boolean isResource)
    {
	  PROPERTIES_FILE_NAME = PropertiesFile;
	  IS_RESOURCE = isResource;
	  ReInit();
    }
   public static boolean ReInit()
    {
      _Props = InternalInit();
      return _Props != null;
    }

   protected static String  PROPERTIES_FILE_NAME = "OdysseyBrowserFramework.properties";
   protected static boolean IS_RESOURCE          = true;
   protected static Properties _Props = InternalInit();

   // Get a MANDATORY Integer from the configuration
   public static final int getInteger(PROPERTY Prop)
   throws MissingResourceException, NumberFormatException
    {
      try
       { 
         String Str = _Props.getProperty(Prop.toString());
         if (Str == null)
          throw new MissingResourceException("Missing property", PROPERTIES_FILE_NAME, Prop.toString());
         return Integer.parseInt(Str);
       }
      catch (MissingResourceException e)  // Missing configuration parameter
       {
         Streamer.error.Header().print("Missing mandatory resource ").println(Prop).printStackTrace(e);
         throw e;
       }
      catch (NumberFormatException e) // Not an integer
       {
         Streamer.error.Header().print("Resource ").print(Prop).println(" does not seem to be an Integer").printStackTrace(e);
         throw e;
       }
    }

   // Get an OPTIONAL Integer from the configuration. If the value is not found, return the default supplied
   public static final int getInteger(PROPERTY Prop, int Default)
   throws MissingResourceException, NumberFormatException
    {
      try
       { 
         String Str = _Props.getProperty(Prop.toString());
         if (Str == null)
          throw new MissingResourceException("Missing property", PROPERTIES_FILE_NAME, Prop.toString());
         return Integer.parseInt(Str);
       }
      catch (MissingResourceException e)
       {
         Streamer.debug.Header().print("Missing optional resource ").print(Prop).print(". Returning supplied default value ").println(Default);
         return Default;
       }
      catch (NumberFormatException e)
       {
         Streamer.error.Header().print("Resource ").print(Prop).println(" does not seem to be an Integer").printStackTrace(e);
         throw e;
       }
    }

   // Get a MANDATORY String from the configuration
   public static final String getString(PROPERTY Prop)
   throws MissingResourceException
    {
   		//RATLC01083006
   		if(Prop.equals(PROP_PATH_PREFIX))
   		{
   			//Check for encoding
   			FacesContext context = FacesContext.getCurrentInstance();
   			String resUrl = null;
   			if(context != null)
   				resUrl = LifecycleUtil.getJSResourceURLPattern(context.getExternalContext().getInitParameter(ContextParams.JS_RESOURCE_SERVLET_URL_PATTERN));
   			if(resUrl != null)
   				return resUrl;
   		}
   		
      try
       {
         String Str = _Props.getProperty(Prop.toString());
         if (Str == null)
          throw new MissingResourceException("Missing property", PROPERTIES_FILE_NAME, Prop.toString());
         return Str;
       }
      catch (MissingResourceException e)
       {
         Streamer.error.Header().print("Missing mandatory resource ").println(Prop).printStackTrace(e);
         throw e;
       }
    }

   // Get an OPTIONAL String from the configuration. If the value is not found, return the default supplied
   public static final String getString(PROPERTY Prop, String Default)
   throws MissingResourceException
    {
      try
       {
         String Str = _Props.getProperty(Prop.toString());
         if (Str == null)
          throw new MissingResourceException("Missing property", PROPERTIES_FILE_NAME, Prop.toString());
         return Str;
       }
      catch (MissingResourceException e)
       {
         Streamer.debug.Header().print("Missing optional resource ").print(Prop).print(". Returning supplied default value ").println(Default);
         return Default;
       }
    }
    
   // Get a MANDATORY Integer from the configuration
   public static final String getStringDecrypted(PROPERTY Prop, String Key)
   throws MissingResourceException//, BadEncryption
    {
//      try
//       {
         return /*Decryption.decrypt(*/getString(Prop)/*, Key)*/;
//       }
//      catch (BadEncryption e)
//       {
//         Streamer.error.Header().print("Cannot decrypt resource ").println(Var).printStackTrace(e);
//         throw e;
//       }
    }   
    
   // A funny thing happens when streamers try to initialize themselves and ask for configuration data. 
   // A circular dependency happens, so we have specialized methods for Streamers that do not output any message
   protected static final int getIntegerSilent(PROPERTY Prop, int Default)
    {
      try
       { 
         String Str = _Props.getProperty(Prop.toString());
         if (Str == null)
          throw new MissingResourceException("Missing property", PROPERTIES_FILE_NAME, Prop.toString());
         return Integer.parseInt(Str);
       }
      catch (Exception e)
       {
       }
      return Default;
    }

   public static final String getStringSilent(PROPERTY Prop)
    {
      try
       {
         String Str = _Props.getProperty(Prop.toString());
         if (Str == null)
          throw new MissingResourceException("Missing property", PROPERTIES_FILE_NAME, Prop.toString());
         return Str;
       }
      catch (Exception e)
       {
       }
      return null;
    }
   
	public static String versionizeFilename(String filename){
		if(filename == null){
			return filename;
		}
		if(filename.lastIndexOf(".") == -1){
			return filename + "_" + VERSION_NUMBER;
		}
		
		StringBuffer newname = new StringBuffer(filename.substring(0,filename.lastIndexOf(".")))
		.append("_")
		.append(VERSION_NUMBER)
		.append(filename.substring(filename.lastIndexOf(".")));
		
		return newname.toString();
	}
    
 }