/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//
//OCO Source Materials
//
//PID xxxx-xxx
//
// Copyright IBM Corp.  2003
//
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////
package com.ibm.odcb.jrender.misc;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.lang.String;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.MissingResourceException;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.EcoreFactory;
import org.eclipse.emf.ecore.EcorePackage;
import org.eclipse.emf.ecore.xmi.XMIResource;
import org.eclipse.emf.ecore.xmi.XMLResource;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceImpl;
import org.eclipse.emf.ecore.xmi.impl.XMLResourceFactoryImpl;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;
import org.eclipse.emf.ecore.util.EcoreUtil;

////#IF# SDO
import commonj.sdo.DataGraph;
import commonj.sdo.DataObject;
import commonj.sdo.Property;
import commonj.sdo.Type;

import org.eclipse.emf.ecore.sdo.EDataGraph;
import org.eclipse.emf.ecore.sdo.SDOFactory;
import org.eclipse.emf.ecore.sdo.impl.DynamicEDataObjectImpl;
import org.eclipse.emf.ecore.sdo.impl.SDOFactoryImpl;

import java.io.FileOutputStream;
import java.io.PrintWriter;
////#END#

////#IF# WDO
//import com.ibm.etools.wdo.DataObject;
//import com.ibm.etools.wdo.datagraph.DataGraph;
//import com.ibm.etools.wdo.datagraph.EDataGraph;
//import com.ibm.etools.wdo.datagraph.WdoFactory;
//import com.ibm.etools.wdo.datagraph.impl.WdoFactoryImpl;
//import org.eclipse.emf.ecore.xmi.impl.XMLResourceImpl;
////#END#


/**
 * @author ldh (Laurent Hasson)
 *
 *  This is a helper class for working with EMF/WDO objects. It provides various methods to
 *   - generically print an EPackage
 *   - generically print an EObject
 *   - generically save an EPackage to file
 *   - generially load an EPackage from file
 *   - create a DataGraph for an EPackage assuming it follows certain conventions
 *   - save a DataGraph to file
 *   - get the value of an EStructuralFeature (an EAttribute or an EReference) from an EObject
 */
public class EMFHelper
 {
	   /** Saves an EPackage to a .ecore file whose full path is supplied
	    */
	   public static void SaveEPackageToECore(EPackage Package, String Path, String ECoreFileName)
	    {      
	      Streamer.status.Header().println();
	      Streamer.status.Header().println();
	      Streamer.status.Header().println("---- SAVING AN EPACKAGE TO "+Path+ECoreFileName+" --------------------------------------");
	
	      Streamer.status.Header().println("Creating an XMI Resource");
	      XMIResource R = new XMIResourceImpl(URI.createFileURI(Path+ECoreFileName));
	      Streamer.status.Header().println("Adding the EPackage '"+Package.getName()+"' to the resource's contents");
	      R.getContents().add(Package);
	      try
	       {
	         Streamer.status.Header().println("Setting the XMI encoding style to 'compact'.");
	         Map M = R.getDefaultSaveOptions();
	         M.put(XMLResource.OPTION_USE_ENCODED_ATTRIBUTE_STYLE, Boolean.TRUE);
			 M.put(XMLResource.OPTION_ENCODING, "UTF-8");
	         Streamer.status.Header().println("Saving");
	         R.save(M);
	       }
	      catch (Exception E)
	       {
	         E.printStackTrace(System.err);
	       }
	      Streamer.status.Header().println("...D.O.N.E..........................................................................");
	    }
   public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;

////#IF# SDO
   public static final String VERSION = "JWL 1.1.1, SDO";
////#END#

////#IF# WDO
//   public static final String VERSION = "JWL 1.1.1, WDO"; 
////#END#

   public static final Class DataObject_CLASS = DataObject.class;

	public static Resource load(ResourceSet resourceSet, URI uri)
	{
		Resource resource = resourceSet.getResource(uri, true);
		EMFHelper.initPackages(resource);
		return resource;
	}
	public static EPackage getEPackageFromStream(String resourceURI)
	{
		Resource res = EMFHelper.getResourceFromStream(resourceURI);
		EPackage pkg = (EPackage) res.getContents().get(0);
		return pkg;
	}
	  public static Resource getResource(InputStream is, URI resourceURI)
	  {
	  	Resource res = null;
	    try 
	    {   
	      EMFHelper.initSDO();
	      ResourceSet resSet = new ResourceSetImpl();
	      res = resSet.createResource(resourceURI);
	      res.load(is, null);
	  }
	  catch (Exception e) {
	    Streamer.error.Header().println("ecore loading failed");
	    Streamer.error.Header().printStackTrace(e);
	  }
	  return res;
	}
	public static Resource getResourceFromStream(String resourceURI)
	{
	  Resource res = null;
	  try
	  {
		InputStream is = ODCClassLoader.getResourceAsStream(resourceURI);
		if (is != null)
		{
		 Streamer.status.Header().println("Loading resource: " + resourceURI);
		 res = EMFHelper.getResource(is, URI.createURI(resourceURI));
		 is.close();
		 Streamer.status.Header().println("Finished Loading resource: " + resourceURI + " without any errors.");
		}
		else
		{
		  Streamer.error.Header().println("Could not get InputStream for '" + resourceURI + "'.  Please make sure that the resource exists.");
		}
	  }
	  catch (IOException e)
	  {
		Streamer.error.Header().println("Error Loading resource: " + resourceURI);
	  	Streamer.error.Header().printStackTrace(e);
	  }
	  return res;
	}

   //This method retrieves the names of the resource files
   // that need to be registered in our factory
   public static Vector propertyResourceName()
   {
   	 Vector fname = new Vector();
   	 
   	 try {
   	 
   	     String systemEcoreFiles = Config.getString(Config.PROP_SYSTEM_ECORE_FILES,null);
   	     if (systemEcoreFiles != null)
   	     {
	         StringTokenizer tok = new StringTokenizer(systemEcoreFiles, " ");
	         String ecoreResourceName = null;
	         int i = 0;
	         while (tok.hasMoreTokens())
	        {
	           fname.addElement(tok.nextToken());
	        }
   	     }
   	 }
   	 catch (MissingResourceException e) {
   	 	Streamer.error.Header().print("Missing Resource Exception: SYSTEM_ECORE_FILES");
   	 }
     
     return fname;  // should return null at this point
   }

   //This code loads external files into a given resourceSet 
   public static void initLoadExternalEcoreFile()
   {
      try 
	  {
      	  Vector fileList = propertyResourceName();
      	  
      	  if ((fileList==null) || (fileList.isEmpty())) return; //there are no resources to be loaded
      	  
      	  while(!fileList.isEmpty()) {
 
      	  	 String resName = fileList.firstElement().toString();
  	  	 
      	     //load the required resource 
     	     InputStream is = ODCClassLoader.getResourceAsStream(resName);
    	     if (is != null)
		     {
    	     	// Load the external ecore file in the resource set
    	     	// The external ecore file is expected to resolve the data types
    	     	//This is for debug ... the uri should be resURI
    	     	ResourceSet resSet = new ResourceSetImpl();
    	     	Resource res = resSet.createResource(URI.createURI(resName));
    	     	res.load(is,null);

	    	    EPackage pkg = (EPackage) res.getContents().get(0);
	    	    //Register the Epackage globally 
	    	    EPackage.Registry.INSTANCE.put(pkg.getNsURI(), pkg);
			
                is.close();
		     }
    	     
    	     fileList.removeElement(resName);
		}
	  }
	  catch (IOException e)
	  {
	  	Streamer.error.Header().printStackTrace(e);
	  } 	
	  return;
   }

 	
	/**
	 * @deprecated - use initSDO() instead
	 */
	public static void InitWDO()
     {
		initSDO();
     }
	
	public static void initSDO()
    {
		Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put("xml", new XMLResourceFactoryImpl());
		Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put("xmi", new XMIResourceFactoryImpl());
		Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put("ecore", new XMIResourceFactoryImpl());
	   EcoreFactory ECFac = EcoreFactory.eINSTANCE;       // Initializes the core ECore infrastructure
	   EcorePackage pkg   = EcorePackage.eINSTANCE;       // Initializes the core EPackage infrastructure
	   
	    initLoadExternalEcoreFile();
////#IF# SDO	   
	   SDOFactory SDOFac = SDOFactory.eINSTANCE;
////#END#

    }
	
	public static void initPackage(EPackage pkg)
	  {
	  	
////#IF# SDO
	    pkg.setEFactoryInstance(new SDOFactoryImpl());
////#END#

////#IF# WDO
//  		pkg.setEFactoryInstance(new WdoFactoryImpl());
////#END#

	  }
	  
	  public static void initPackageForDataObject(EPackage pkg)
	  {
////#IF# SDO
  		pkg.setEFactoryInstance(new DynamicEDataObjectImpl.FactoryImpl());
////#END#

////#IF# WDO
//  		pkg.setEFactoryInstance(new WdoFactoryImpl());
////#END#	  
	    }

	
	public static void initPackages(Resource resource)
	  {
		Object pkg = EcoreUtil.getObjectByType(resource.getContents(), EcorePackage.eINSTANCE.getEPackage());
		if (pkg != null)
		{
		  initPackage((EPackage) pkg);
		}
	  }
	
	public static void initPackagesForDataObject(Resource resource)
	  {
		Object pkg = EcoreUtil.getObjectByType(resource.getContents(), EcorePackage.eINSTANCE.getEPackage());
		if (pkg != null)
		{
		  initPackage((EPackage) pkg);
		}
	  }

    /**
     * This method returns the String representation of the contents of a file 
     * with the given pathname.  This method is useful for reading xml files.
     * @param  fullPathFileName The full path name of the file.
     * @return The contents of the file as a String.
     */
    public static String ReadFileIntoString(String fullPathFileName)
    {
      String Str = null;
      try
       {
         BufferedReader F = new BufferedReader(new FileReader(fullPathFileName));
         StringBuffer XXX = new StringBuffer(512);
         while (true)
          {
            String Line = F.readLine();
            if (Line == null)
             break;
            XXX.append(Line).append("\n");
          }
         Str = XXX.toString();
         F.close();
       }
      catch (Exception E)
       {
         E.printStackTrace(System.err);
       }
      
      return Str;
    }

   /** 
    * @deprecated
    * Use Package.eResource().save(System.out,java.util.Collections.EMPTY_MAP);
    * Iterates through all the EClasses of an EPackage, and then, for each, itereates through all their EAttributes and EReferences
    * Prints to System.out
    */
   public static void PrintPackage(EPackage Package)
    {
      System.out.println();
      System.out.println();
      System.out.println("---- PRINTING OUT AN EPACKAGE --------------------------------------");

      System.out.println("EPackage: Name="+Package.getName()+";");
      System.out.println();
      List L = Package.getEClassifiers();
      for (int i = 0; i < L.size(); ++i)
       {
         EClass C = (EClass) L.get(i);
         System.out.println("EClass: Name="+C.getName()+";");
         for (int j = 0; j < C.getEAllAttributes().size(); ++j)
          {
            EAttribute A = (EAttribute) C.getEAllAttributes().get(j);
            System.out.println("  EAttribute: Name="+A.getName()
                                          +"; LowerBound="+A.getLowerBound()
                                          +"; UpperBound="+A.getUpperBound()+";");
          }
         for (int j = 0; j < C.getEAllReferences().size(); ++j)
          {
            EReference R = (EReference) C.getEAllReferences().get(j);
            System.out.println("  EReference: Name="+R.getName()
                                          +"; Type="+TypesUtil.getRealETypeName(R.getEType())
                                          +"; LowerBound="+R.getLowerBound()
                                          +"; UpperBound="+R.getUpperBound()
                                          +"; Containment="+R.isContainment()+";");
          }
         System.out.println();    
       }
      System.out.println("...D.O.N.E..........................................................................");
    }

   public static final String SDO_DATAGRAPH_ROOTNAME = "DataGraphRoot";
   public static final String WDO_DATAGRAPH_ROOTNAME = "RootDataObject";

   /** Saves an EPackage to a .ecore file whose full path is supplied
    */
   public static void saveEPackageToECore(EPackage pkg, String path, String ecoreFileName, boolean useWdo)
    { 

////#IF# SDO
      if (!useWdo)
	    SaveEPackageToECore(pkg, path, ecoreFileName);
	  else
	  {  
	   PrintWriter out = null;
	   try
	   {
		Streamer.status.Header().println("---- SAVING WDO EPACKAGE TO "+path+ecoreFileName+" --------------------------------------");
		StringBuffer sb = new StringBuffer("");
		sb.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n")
		  .append("<ecore:EPackage\n")
		  .append("  xmi:version=\"2.0\"\n")
		  .append("  xmlns:xmi=\"http://www.omg.org/XMI\"\n")
		  .append("  xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n")
		  .append("  xmlns:ecore=\"http://www.eclipse.org/emf/2002/Ecore\"\n")
		  .append("  name=\""+pkg.getName()+"\"\n")
		  .append("  nsURI=\""+pkg.getNsURI()+"\"\n")
		  .append("  nsPrefix=\""+pkg.getNsPrefix()+"\" >\n");
		Iterator it = pkg.getEClassifiers().iterator();
		Streamer.status.Header().println("Iterating through EClasses");
		while (it.hasNext())
		{
			EClass ec = (EClass) it.next();
			if (ec.getName().equals(SDO_DATAGRAPH_ROOTNAME))
			  sb.append("  <eClassifiers xsi:type=\"ecore:EClass\" name=\""+WDO_DATAGRAPH_ROOTNAME+"\" >\n");
			else
			  sb.append("  <eClassifiers xsi:type=\"ecore:EClass\" name=\""+ec.getName()+"\" >\n");
			Iterator eat = ec.getEAttributes().iterator();
			Streamer.status.Header().println("Iterating through EAttributes");
			while (eat.hasNext())
			{
				EAttribute ea = (EAttribute) eat.next();
				String eType = TypesUtil.getFullEType(ea.getEType().getName(), true);
				sb.append("    <eAttributes name=\""+ea.getName()+"\" eType=\""+eType+"\" lowerBound=\""+ea.getLowerBound()+"\" upperBound=\""+ea.getUpperBound()+"\" changeable=\""+ea.isChangeable()+"\" unique=\""+ea.isUnique()+"\"  />\n");
			}
			Iterator refit = ec.getEReferences().iterator();
			Streamer.status.Header().println("Iterating through EReferences");
			while (refit.hasNext())
			{
				EReference er = (EReference) refit.next();
				String eType = TypesUtil.getFullEType(er.getEType().getName(), true);
				sb.append("    <eReferences name=\""+er.getName()+"\" eType=\""+eType+"\" lowerBound=\""+er.getLowerBound()+"\" upperBound=\""+er.getUpperBound()+"\" containment=\""+er.isContainment()+"\" changeable=\""+er.isChangeable()+"\" unique=\""+er.isUnique()+"\" />\n");
			}
			sb.append("  </eClassifiers>\n");
		}
		/*sb.append("  <eClassifiers xsi:type=\"ecore:EClass\" name=\"DataGraphSchema\">\n");
		sb.append("    <eReferences name=\"root\" eType=\"#//RootDataObject\" containment=\"true\" resolveProxies=\"false\"/>\n");
	    sb.append("  </eClassifiers>");*/
		sb.append("</ecore:EPackage>\n");
		Streamer.status.Header().println("Writing the ecore to file"+path+ecoreFileName);
		out = new PrintWriter(new FileOutputStream(path+ecoreFileName));
		out.write(sb.toString());
		out.flush();
		out.close();		  
		Streamer.status.Header().println("---DONE---");
	   }
	   catch (Exception e)
	   {
	   	if (out != null)
	   	  out.close();
	   	Streamer.error.Header().println("An Error occured when saving the wdo package to ecore file.");
	   	Streamer.error.Header().printStackTrace(e);
	   }
		
	  }
////#END#

////#IF# WDO
//		SaveEPackageToECore(pkg, path, ecoreFileName);
////#END#

    }
    
   /** Loads an EPackage from an .ecore file whose full path is supplied
    */
   public static EPackage LoadEPackageFromECore(String Path, String ECoreFileName)
    {      
      Streamer.status.Header().println();
      Streamer.status.Header().println();
      Streamer.status.Header().println("---- LOADING AN EPACKAGE FROM "+Path+ECoreFileName+" --------------------------------------");
   
      Streamer.status.Header().println("Creating an XMI Resource");
      XMIResource R = new XMIResourceImpl(URI.createFileURI(Path+ECoreFileName));
      try
       {
         Streamer.status.Header().println("Setting the XMI encoding style to 'compact'.");
         Map M = R.getDefaultLoadOptions();
         M.put(XMLResource.OPTION_USE_ENCODED_ATTRIBUTE_STYLE, Boolean.TRUE);
		 M.put(XMLResource.OPTION_ENCODING, "UTF-8");
         Streamer.status.Header().println("Loading");
         R.load(M);
         Streamer.status.Header().println("Getting the EPackage from the resource's contents");
         EPackage P = (EPackage) R.getContents().get(0);
         Streamer.status.Header().println("Initializing the EPackage '"+P.getName()+"' for WDO features");
         EMFHelper.initPackage(P);
         Streamer.status.Header().println("...D.O.N.E..........................................................................");
         return P;
       }
      catch (Exception E)
       {
         E.printStackTrace(System.err);
       }
      Streamer.error.Header().println("...E.R.R.O.R.........................................................................");
      return null;
    }

  /**
   * Prints the contents of the EObject in XML format to System.out.
   * @param  eobj        The EObject to print.
   * @param  eobjName    The name of the EObject.
   * @param  padding     The number of spaces for indentation
   * @param  printedList The list of EObjects already printed.
   *                      This is empty at the first iteration, and
   *                      the same list is passed to subsequent calls to this method
   *                      when it is called recursively or in a loop.
   *                      This is required to avoid infinite loops caused by two way references.
   */
  public static void printEObject(EObject eobj, String eobjName, String padding, ArrayList printedList)
  {
    try 
    {
      if (printedList == null)
      {
        printedList = new ArrayList();
      }
       
      //if this EObject has already been printed then skip.  This is needed to avoid infinite loops.
      boolean printed = printedList.contains(eobj);
      if (!printed)  
      {         
        printedList.add(eobj);
        EClass c = eobj.eClass();
                
        System.out.print(padding+"<"+eobjName);
        
        //xmi:id cannot be obtained from EObject for some reason.       
        Iterator eats = c.getEAttributes().iterator();
        Iterator refs = c.getEReferences().iterator();
        while (eats.hasNext())
        {
          try
          {
            EAttribute eat = (EAttribute) eats.next();
            Object eatobj = eobj.eGet(eat, true);
            if (eatobj != null)
            {
              System.out.print(" "+ eat.getName() + "=\"" + eatobj + "\"");
            }
          }
          catch (Exception e) {}
        }
        System.out.println(">");
        while (refs.hasNext())
        {
          try
          {
            EReference ref = (EReference) refs.next();
            Object refobj = eobj.eGet(ref, true);
            if (refobj != null)
            {
              if (refobj instanceof org.eclipse.emf.ecore.util.EcoreEList)
              {
                printEObjects((org.eclipse.emf.ecore.util.EcoreEList) refobj, ref.getName(), padding+"  ", printedList); 
              }
              else if (refobj instanceof EObject)
              {
                printEObject((EObject)refobj, ref.getName(), padding+"  ", printedList);
              }
              else
              {
                throw new Exception("Printing the object '"+eobj.getClass().getName()+"' is not supported!  The supported types are org.eclipse.emf.ecore.util.EcoreDynamicEList and org.eclipse.emf.ecore.EObject!");
              }
            }
          }
          catch (Exception e) 
          {
          	throw e;
          }
        }
        System.out.println(padding+"</"+eobjName+">");
      }
    }
    catch (Exception e) {Streamer.error.Header().printStackTrace(e);}
  }

  /**
   * Prints the contents of the given list in XML format to System.out.  This method is called
   * to print EReference objects of multiplicity >1, usually from within the printEObject() method.
   * @param  list        The list to print.  This is usually an EReference object of multiplicity >1.
   * @param  listName    The name of the EReference object this list represents.
   * @param  padding     The number of spaces for indentation.
   * @param  printedList The list of EObjects already printed.
   *                      This is empty at the first iteration, and
   *                      the same list is passed to subsequent calls to this method
   *                      when it is called recursively or in a loop.
   *                      This is required to avoid infinite loops caused by two way references.
   */ 
  public static void printEObjects(org.eclipse.emf.ecore.util.EcoreEList list, String listName, String padding, ArrayList printedList) throws Exception
  {
    Iterator it = list.iterator();
    while (it.hasNext())
    {
      Object eobj = it.next();
      if (eobj instanceof org.eclipse.emf.ecore.util.EcoreEList)
      {
        printEObjects((org.eclipse.emf.ecore.util.EcoreEList) eobj, listName, padding, printedList); 
      }
      else if (eobj instanceof EObject)
      {
        printEObject((EObject)eobj, listName, padding, printedList);
      }
      else
      {
        throw new Exception("Printing the object '"+eobj.getClass().getName()+"' is not supported!  The supported types are org.eclipse.emf.ecore.util.EcoreDynamicEList and org.eclipse.emf.ecore.EObject!");
      }
    }
  }

  /**
   * Gets the value of the given EStructuralFeature (an EAttribute or an EReference) from the EObject.
   * @param  eobj        The EObject
   * @param  featureName The name of the EStructuralFeature (an EAttribute or an EReference).
   * @return the value of the EStructuralFeature.
   */
  public static Object getEFeatureValue(EObject eobj, String featureName)
   {
     EStructuralFeature feature = eobj.eClass().getEStructuralFeature(featureName);
     return eobj.eGet(feature, false);
   }
   
   /** 
    * Compares 2 ECores together. Returns true if both ECores are identical (same name, nsURI, 
    * have the same EClasses, and each EClass has the same EAttributes and EReferences). Returns
    * false if they are different.
    */
   public static boolean CompareECores(String ECoreFile1, String ECoreFile2)
	{
	   EPackage ECore1 = EMFHelper.LoadEPackageFromECore(ECoreFile1, "");
	   EPackage ECore2 = EMFHelper.LoadEPackageFromECore(ECoreFile2, "");

	   if (ECore1 == null || ECore2 == null)
		{
		  return false;         
		}
	   return CompareECores(ECore1, ECore2);
	}
	/** 
	 * Compares 2 ECores together. Returns true if both ECores are identical (same name, nsURI, 
	 * have the same EClasses, and each EClass has the same EAttributes and EReferences). Returns
	 * false if they are different.
	 */
   public static boolean CompareECores(EPackage P1, EPackage P2)
	{
	  if (P1.getName().equals(P2.getName()) == false)
	   {
		 Streamer.debug.Header().println("The 2 ecores have different names");
		 return false;         
	   }
	  if (P1.getNsURI().equals(P2.getNsURI()) == false)
	   {
		 Streamer.debug.Header().println("The 2 ecores have different NsURI");
		 return false;         
	   }
	  if (P1.getNsPrefix().equals(P2.getNsPrefix()) == false)
	   {
		 Streamer.debug.Header().println("The 2 ecores have different NsPrefix");
		 return false;         
	   }
	  List L1 = P1.getEClassifiers();
	  List L2 = P2.getEClassifiers();
	  if (L1.size() != L2.size())
	   {
		 Streamer.debug.Header().println("The 2 ecores have a different number of EClasses");
		 return false;
	   }
	  for (int i = 0; i < L1.size(); ++i)
	   {
		 EClass C1 = (EClass) L1.get(i);
		 EClass C2 = (EClass) P2.getEClassifier(C1.getName());
		 if (C2 == null)
		  {
			Streamer.debug.Header().println("The EClassifier '"+C1.getName()+"' cannot be found in the second ECore");
			return false;
		  }
		 if (CompareEClasses(C1, C2) == false)
		  return false;
	   }
	  return true;
	}
    
	/** 
	 * Compares 2 EClasses together. Returns true if both EClasses are identical (same name, 
	 * same EAttributes and EReferences). Returns false if they are different.
	 */    
   public static boolean CompareEClasses(EClass C1, EClass C2)
	{
	  List L1 = C1.getEAllAttributes();
	  List L2 = C2.getEAllAttributes();
	  if (L1.size() != L2.size())
	   {
		 Streamer.debug.Header().println("The 2 EClasses '"+C1.getName()+"' in the 2 ECores have a different number of EAttributes");
		 return false;
	   }
	  for (int j = 0; j < L1.size(); ++j)
	   {
		 EAttribute A1 = (EAttribute)L1.get(j);
		 EAttribute A2 = (EAttribute)C2.getEStructuralFeature(A1.getName());
		 if (A2 == null)
		  {
			Streamer.debug.Header().println("The EAttribute '"+A1.getName()+"' cannot be found in the EClass '"+C1.getName()+"' in the second ECore");
			return false;
		  }
		 if (CompareEAttributes(C1, A1, A2) == false)
		  return false;            
	   }

	  L1 = C1.getEAllReferences();
	  L2 = C2.getEAllReferences();
	  if (L1.size() != L2.size())
	   {
		 Streamer.debug.Header().println("The 2 EClasses '"+C1.getName()+"' in the 2 ECores have a different number of EReferences");
		 return false;
	   }
	  for (int j = 0; j < L1.size(); ++j)
	   {
		 EReference R1 = (EReference)L1.get(j);
		 EReference R2 = (EReference)C2.getEStructuralFeature(R1.getName());
		 if (R2 == null)
		  {
			Streamer.debug.Header().println("The EReference '"+R1.getName()+"' cannot be found in the EClass '"+C1.getName()+"' in the second ECore");
			return false;
		  }
		 if (CompareEReferences(C1, R1, R2) == false)
		  return false;            
	   }
	  return true;
	}

	/** 
	 * Compares 2 EAttributes together. Returns true they are identical, or false otherwise.
	 */    
   public static boolean CompareEAttributes(EClass C, EAttribute A1, EAttribute A2)
	 {
	   if (CompareEFeatures(C, A1, A2) == false)
		return false;
       
	   if (A1.isID() != A2.isID())
		{
		  Streamer.debug.Header().println("The EAttribute '"+A1.getName()+"' in EClass '"+C.getName()+"' has a different 'ID' in the second ECore");
		  return false;
		}
        
	   return true;
	 }
    
	/** 
	 * Compares 2 EReferences together. Returns true they are identical, or false otherwise.
	 */    
   public static boolean CompareEReferences(EClass C, EReference R1, EReference R2)
	 {
	   if (CompareEFeatures(C, R1, R2) == false)
		return false;
        
	   EReference EOpp1 = R1.getEOpposite();
	   EReference EOpp2 = R2.getEOpposite();
	   if (EOpp1 != null && EOpp2 != null) // Both are non null, so compare them
		{                                  // Make sure the Opposite point to the same reference in the same eclass off the same epackage
		   String EOppName1         = EOpp1.getName();           
		   String EOppName2         = EOpp2.getName();
		   if (EOppName1.equals(EOppName2) == false)
			{
			  Streamer.debug.Header().println("The EReference '"+R1.getName()+"' in EClass '"+C.getName()+"' has a different 'EOpposite' in the second ECore");
			  return false;
			}
		   String EOppEClassName2   = EOpp2.eClass().getName();
		   String EOppEClassName1   = EOpp1.eClass().getName();
		   if (EOppEClassName2.equals(EOppEClassName1) == false)
			{
			  Streamer.debug.Header().println("The EReference '"+R1.getName()+"' in EClass '"+C.getName()+"' has a different 'EOpposite' in the second ECore");
			  return false;
			}
		   String EOppEPackageName1 = EOpp1.eClass().getEPackage().getName();
		   String EOppEPackageName2 = EOpp2.eClass().getEPackage().getName();
		   if (EOppEPackageName1.equals(EOppEPackageName2) == false)
			{
			  Streamer.debug.Header().println("The EReference '"+R1.getName()+"' in EClass '"+C.getName()+"' has a different 'EOpposite' in the second ECore");
			  return false;
			}
		}
	   else if (EOpp1 != null || EOpp2 != null) // one of them is null, and the other is not null. 
		{                                       // This is like !(EOpp1 == null && EOpp2 == null)
		  Streamer.debug.Header().println("The EReference '"+R1.getName()+"' in EClass '"+C.getName()+"' has a different 'EOpposite' in the second ECore");
		  return false;
		}

	   if (R1.isContainer() != R2.isContainer())
		{
		  Streamer.debug.Header().println("The EReference '"+R1.getName()+"' in EClass '"+C.getName()+"' has a different 'Container' in the second ECore");
		  return false;          
		}
        
		if (R1.isContainment() != R2.isContainment())
		 {
		   Streamer.debug.Header().println("The EReference '"+R1.getName()+"' in EClass '"+C.getName()+"' has a different 'Containment' in the second ECore");
		   return false;          
		 }
         
		if (R1.isResolveProxies() != R2.isResolveProxies())
		 {
		   Streamer.debug.Header().println("The EReference '"+R1.getName()+"' in EClass '"+C.getName()+"' has a different 'ResolveProxies' in the second ECore");
		   return false;          
		 }

	   return true;
	 }

	/** 
	 * Compares 2 EFeatures together. Returns true they are identical, or false otherwise.
	 */    
   public static boolean CompareEFeatures(EClass C, EStructuralFeature F1, EStructuralFeature F2)
	{
	  if (F1.getDefaultValueLiteral() != null && F1.getDefaultValueLiteral().equals(F2.getDefaultValueLiteral()) == false)
       {
	     Streamer.debug.Header().println("The EFeature '"+F1.getName()+"' in EClass '"+C.getName()+"' has a different 'DefaultValueLitteral' in the second ECore");
	     return false;
       }

      String F1TypeName = TypesUtil.getRealETypeName(F1.getEType());
      String F2TypeName = TypesUtil.getRealETypeName(F2.getEType());
	  if (F1TypeName == null || F1TypeName.equals(F2TypeName) == false)
       {
         Streamer.debug.Header().println("The EFeature '"+F1.getName()+"' in EClass '"+C.getName()+"' has a different 'EType' in the second ECore");
         Streamer.debug.Header().println("ETypes are '"+F1TypeName+" and '"+F2TypeName+"'");
	     return false;
       }    
      
	  if (F1.getLowerBound() != F2.getLowerBound())
	   {
	     Streamer.debug.Header().println("The EFeature '"+F1.getName()+"' in EClass '"+C.getName()+"' has a different 'LowerBound' in the second ECore");
	     return false;
       }
      
	  if (F1.getUpperBound() != F2.getUpperBound())
       {
	     Streamer.debug.Header().println("The EFeature '"+F1.getName()+"' in EClass '"+C.getName()+"' has a different 'UpperBound' in the second ECore");
	     return false;
	   }
        
      if (F1.isChangeable() != F2.isChangeable())
	   {
	     Streamer.debug.Header().println("The EFeature '"+F1.getName()+"' in EClass '"+C.getName()+"' has a different 'Changeable' in the second ECore");
	     return false;
	   }

	  if (F1.isTransient() != F2.isTransient())
	   {
	     Streamer.debug.Header().println("The EFeature '"+F1.getName()+"' in EClass '"+C.getName()+"' has a different 'Transient' in the second ECore");
	     return false;
	   }

	  if (F1.isUnique() != F2.isUnique())
	   {
	     Streamer.debug.Header().println("The EFeature '"+F1.getName()+"' in EClass '"+C.getName()+"' has a different 'Unique' in the second ECore");
	     return false;
	   }

	  if (F1.isUnsettable() != F2.isUnsettable())
	   {
	     Streamer.debug.Header().println("The EFeature '"+F1.getName()+"' in EClass '"+C.getName()+"' has a different 'Unsettable' in the second ECore");
	     return false;
	   }

	  if (F1.isVolatile() != F2.isVolatile())
	   {
	     Streamer.debug.Header().println("The EFeature '"+F1.getName()+"' in EClass '"+C.getName()+"' has a different 'Volatile' in the second ECore");
	     return false;
	   }
         
	  return true;
    }

	public static DataObject loadDataObject(InputStream dataIs, EPackage pkg)
	{
	  DataObject o = null;
	  try
	  {    
	  	     
		if (pkg == null)
		{
		  Streamer.error.Header().println("Can not load DataObject.  EPackage is null!");
		  return null;
		}
		else
		{
			EMFHelper.initSDO();
			EMFHelper.initPackageForDataObject(pkg);

/*
 * The WDO part below 
 * pkg.getEClassifier("root") -> "root" is not available.  Need this as a variable
 * (XMLResourceImpl) resourceSet.createResource(URI.createURI("data")) -> "data" is ficticious
 */
			 
////#IF# SDO
		  EDataGraph dg = SDOFactory.eINSTANCE.createEDataGraph();
		  ResourceSet resourceSet = dg.getResourceSet();
		  resourceSet.getResources().add(pkg.eResource());
////#END#

////#IF# WDO
//		  Resource pkgRes = pkg.eResource();
//		  ResourceSet resourceSet = null;
//		  if (pkgRes != null)
//			resourceSet = pkgRes.getResourceSet();
//		  else 
//		  {
//			resourceSet = new ResourceSetImpl();
//		  }	    
////#END#
	  
		  Resource resource = resourceSet.createResource(URI.createURI("data.xmi"));
		  Map options = resourceSet.getLoadOptions();
          options.put(XMLResource.OPTION_USE_ENCODED_ATTRIBUTE_STYLE, Boolean.TRUE);
          options.put(XMLResource.OPTION_ENCODING, "UTF-8");
		  resource.load(dataIs, options);
		  o = (DataObject) resource.getContents().get(0);
		}
	  }
	  catch (Exception e)
	  {
		Streamer.error.Header().printStackTrace(e);
		return null;
	  }       
	   return o;
	 }

   public static DataObject getDataObject(Object o, boolean internalList)
   {
   	  DataObject d = null;
		  if (o instanceof DataGraph)
		   {
             	
////#IF# SDO
			 d = ((DataGraph) o).getRootObject();
////#END#

////#IF# WDO
//			if (internalList)
//			 d = ((DataGraph) o).getDataObject().getDataObject("root");
//			else
//			d = ((DataGraph) o).getDataObject();
////#END#

		   }
	  return d;
   }
   
   public static DataGraph createDataGraph(EPackage pkg)
    {
    	
////#IF# SDO
		pkg.setEFactoryInstance(new DynamicEDataObjectImpl.FactoryImpl());
		EDataGraph dataGraph = SDOFactory.eINSTANCE.createEDataGraph();
		ResourceSet resourceSet = dataGraph.getResourceSet();
		URI EPackageURI = URI.createURI(pkg.getNsURI());
		resourceSet.createResource(EPackageURI).getContents().add(pkg);
		return dataGraph;
////#END#  

////#IF# WDO 
//		return null;
////#END#  
 	
	}

   /**
    * @deprecated
    */
   public static DataGraph createDataGraph(EPackage pkg, String rootName)
	{
		
////#IF# SDO
		DataGraph dataGraph = createDataGraph(pkg);
////#END#   

////#IF# WDO
//	  WdoFactory.eINSTANCE.setEPackage(pkg);
//	  Streamer.status.Header().println("Creating the root class called DataGraphModel by convention here");
//	  EClass dataGraphModel = (EClass) pkg.getEClassifier(rootName);
//	  Streamer.status.Header().println("Creating the DataGraph object");
//	  EDataGraph dataGraph = WdoFactory.eINSTANCE.createEDataGraph(dataGraphModel);
////#END#

	  return dataGraph;
	}
	
   public static String getEClassName(DataObject d)
   {
   	
////#IF# SDO 
	  return d.getType().getName();
////#END# 	

////#IF# WDO 
//    return ((EObject)d).eClass().getName();
////#END#
   }

   public static List getFeatureNames(DataObject d)
    {
////#IF# SDO 
    List L = new ArrayList();
    Type T = d.getType();
    List Properties = T.getProperties();
    for (int i = 0; i < Properties.size(); ++i)
     {
       Property P = (Property) Properties.get(i);
       L.add(P.getName());
     }
    return L;
////#END#   

////#IF# WDO 
//    List L = new ArrayList();
//    EClass EC = ((EObject)d).eClass();    
//    List Features = EC.getEAllStructuralFeatures();
//    for (int i = 0; i < Features.size(); ++i)
//     {
//       EStructuralFeature ESF = (EStructuralFeature) Features.get(i);
//       L.add(ESF.getName());
//     }
//    return L;
////#END#      
    }

   public static void copy(DataObject Source, DataObject Destination)
    {
      List L = EMFHelper.getFeatureNames(Source);
      for (int i = 0; i < L.size(); ++i)
       {
         String FeatureName = (String) L.get(i);
         if (Source.isSet(FeatureName) == true)  // gotta update it
          Destination.set(FeatureName, Source.get(FeatureName));
       }
    }
   
   /**
    * Validator for Diff Handlers. Filters out "<" and ">" if found in any
    *  Strings replacing with "&lt;" and "&gt;" respectively. Adam
    * @param d	DataObject to validate.
    */
   public static void validateDataObject(DataObject d)
   {
////#IF# SDO 
   		Type T = d.getType();
   		List Properties = T.getProperties();
   		for (int i = 0; i < Properties.size(); ++i)
   		{
   			Property P = (Property) Properties.get(i);
   			if (d.isSet(P)) 
   			{
   	   			Object o = d.get(P);
   	   			if (String.class==o.getClass())
				{
   					d.setString(P,d.getString(P).replaceAll("<","&lt;").replaceAll(">","&gt;"));
   				}
   			}
   		}
////#END# 
   		
////#IF# WDO
//		EObject eo = (EObject)d;
//      EClass EC = eo.eClass();    
//      List Features = EC.getEAllStructuralFeatures();
//      for (int i = 0; i < Features.size(); ++i)
//       {
//         EStructuralFeature ESF = (EStructuralFeature) Features.get(i);
//   		if (eo.eIsSet(ESF))
//			{
//				Object o = eo.eGet(ESF);
//				if(String.class==o.getClass())
//				{  
//					eo.eSet(ESF,((String)o).replaceAll("<","&lt;").replaceAll(">","&gt;"));
//				}
//			}
//       }
////#END# 
   }
   
   /**
    * Reverse operation to validateDataObject
    * @param d	DataObject for which we want to undo validation.
    */
   public static void undoValidateDataObject(DataObject d)
   {
////#IF# SDO 
   		Type T = d.getType();
   		List Properties = T.getProperties();
   		for (int i = 0; i < Properties.size(); ++i)
   		{
   			Property P = (Property) Properties.get(i);
   			Object O = d.get(P);
   			if ((d.isSet(P)) && (String.class==O.getClass()))
   			{
   				d.setString(P,d.getString(P).replaceAll("&lt;","<").replaceAll("&gt;",">"));
   			}
   		}
////#END# 
   		
////#IF# WDO
//		EObject eo = (EObject)d;
//      EClass EC = eo.eClass();    
//      List Features = EC.getEAllStructuralFeatures();
//      for (int i = 0; i < Features.size(); ++i)
//       {
//         EStructuralFeature ESF = (EStructuralFeature) Features.get(i);
//   		if (eo.eIsSet(ESF))
//			{
//				Object o = eo.eGet(ESF);
//				if(String.class==o.getClass())
//				{  
//					eo.eSet(ESF,((String)o).replaceAll("&lt;","<").replaceAll("&gt;",">"));
//				}
//			}
//       }
////#END# 
   }



   public static void main(String[] args)
	{
	  initSDO();
	  if (CompareECores("t:\\portfolio_sample1.ecore", "t:\\portfolio_sample2.ecore") == false)
       {
	     Streamer.error.Header().println("The ECores are different");
       }
	  else
       {
		 Streamer.status.Header().println("The ECores are the same");
       }
	}
 }
