/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2004 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.odcb.jrender.misc;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;

/**
 * @author ldh
 */
public class FileUtil
 {
   public static String NEWLINE = System.getProperty("line.separator");

   public static boolean FileCopy(String Src, String Dest, boolean CreateDestPath)
    {
      return FileCopy(new File(Src), new File(Dest), CreateDestPath);
    }
   public static boolean FileCopy(File Src, String Dest, boolean CreateDestPath)
    {
      return FileCopy(Src, new File(Dest), CreateDestPath);
    }
   public static boolean FileCopy(File Src, File Dest, boolean CreateDestPath)
    {
      BufferedInputStream In = null;
      FileOutputStream    Out= null;
      try
       {
         In = new BufferedInputStream(new FileInputStream(Src));
         if (CreateDestPath == true)
          {
            File Dir = new File(Dest.getParent());
            if (Dir.exists() == false && Dir.mkdirs() == false)
             {
               throw new IOException("Cannot create path '"+Dest.getParent()+"'.");
             }
          }
         Out= new FileOutputStream(Dest);
         byte[] Buffer = new byte[4048];
         int count;
         while ( (count = In.read(Buffer)) != -1)
          {
            Out.write(Buffer, 0, count);
          }
         return true;
       }
      catch (IOException E)
       {
         Streamer.error.Header().println("Cannot copy file '"+Src.getAbsolutePath()+"' to file '"+Dest.getAbsolutePath()+"'.");
         Streamer.error.Header().printStackTrace(E);
         return false;
       }
      finally
       {
         try { if (In  != null) In .close(); } catch (Exception E) { }
         try { if (Out != null) Out.close(); } catch (Exception E) { }
       }
    }

   public static boolean FileDelete(String FileName)
    {
      return FileDelete(new File(FileName));
    }
   public static boolean FileDelete(File F)
    {    
      if (F.exists() == false)
       {
         Streamer.warning.Header().println("The file '"+F.getPath()+"' does not exist. It't already deleted.");
         return true;
       }
      if (F.delete() == false)
       {
         Streamer.error.Header().println("The file '"+F.getPath()+"' Could not be deleted.");
         return false;
       }
      return true;
    }


   public static interface FileProcessor
    {
      public boolean Process(File F) throws Exception;
    }
   public static boolean FileSystemTraverser(String RootDir, FileFilter FF, FileProcessor P)
    {
      return FileSystemTraverser(new File(RootDir), FF, P);
    }
   public static boolean FileSystemTraverser(File RootDir, FileFilter FF, FileProcessor P)
    {
      try
       {
         if (RootDir.exists() == false)
          {
            Streamer.error.Header().println("The directory '"+RootDir.getPath()+"' does not exist.");
            return false;
          }
         if (RootDir.isDirectory() == false)
          {
            Streamer.error.Header().println("The name '"+RootDir.getPath()+"' does note denote a directory.");
            return false;
          }
      
         File[] SubFiles = RootDir.listFiles(FF);
         for (int i = 0; i < SubFiles.length; ++i)
          {
            File SF = SubFiles[i];
            if (SF.isDirectory() == true)
             {
               if (FileSystemTraverser(SF, FF, P) == false)
                {
                  return false;
                }
             }
            else if (P.Process(SF) == false)
             {
               return false;
             }
          }
         return P.Process(RootDir);
       }
      catch (Exception E)
       {
         Streamer.error.Header().printStackTrace(E);
         return false;
       }
    }


   public static class FileDeleter implements FileProcessor
    {
      public boolean Process(File F) throws Exception
       {
         if (F.isDirectory() == true)
          {
            if(F.delete() == false)
             {
               Streamer.error.Header().println("The directory '"+F.getName()+"' Could not be deleted.");
               return false;
             }
          }
         else if (FileDelete(F) == false)
          {
            Streamer.error.Header().println("The file '"+F.getName()+"' Could not be deleted.");
            return false;
          }
         return true;
       }
    }
   public static boolean DirDelete(String FileName)
    {
      return DirDelete(new File(FileName));
    }
   public static boolean DirDelete(File D)
    {
      return FileSystemTraverser(D, null, new FileDeleter()); 
    }

   // A FileFilter to include or exclude files based on a list of extensions such as [ ".java", ".js", ".xml" ]
   public static class ExtensionFilter implements FileFilter
    {
      public ExtensionFilter(ArrayList Ext, boolean Include)
       {
         _Ext = Ext;
         _Include = Include;
       }
      protected ArrayList _Ext;
      protected boolean   _Include;

      public boolean accept(File F)
       {
         if (_Ext.size() == 0 || F.isDirectory() == true)
          return true;
         String FileName = F.getName();
         for (int i = 0; i < _Ext.size(); ++i)
          {
            String Ext = (String) _Ext.get(i);
            if (FileName.endsWith(Ext) == true)
             return _Include;
          }
         return !_Include;
       }
    }

   public static boolean isOlder(String FileName1, String FileName2)
    {
      return isOlder(new File(FileName1), new File(FileName2));
    }
   public static boolean isOlder(File File1, String FileName2)
    {
      return isOlder(File1, new File(FileName2));
    }
   // This method returns true if File1 is older than File2, or if File1 does not exist.
   public static boolean isOlder(File File1, File File2)
    {
      return File1.exists() == false || File1.lastModified() > File2.lastModified();
    }

   public static void main(String[] args)
    {
      String X = "abc def";
      String Sub = X;
      int i = X.indexOf(Sub);
      X = X.substring(0,i)+X.substring(i+Sub.length());
      System.out.println("X= '"+X+"'");
      DirDelete("D:\\projects\\Testing\\Copy of src");
    }
 }
