/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//
//OCO Source Materials
//
//PID xxxx-xxx
//
// Copyright IBM Corp.  2003
//
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////

package com.ibm.odcb.jrender.misc;

import java.io.Writer;
import java.io.IOException;
import java.util.Locale;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.text.DecimalFormatSymbols;
import java.util.HashMap;


public class NumberFormattingInfo 
{
	protected static HashMap _numberFormatHash = new HashMap(2);
	protected static final String DEFAULT_LOCALE = "en_US";
	
	// Maintain a static hash table here with key = locale ("en_US")
	// The value stored is the JavaScript array string for number format for that locale.
	
	public static void printNumberFormattingInfo(Writer out, String locale)
	throws IOException
	//public static void printNumberFormattingInfo(String locale)
	{
		if(locale == null)
			locale = DEFAULT_LOCALE;
			
		String formattedString = null;
		String defaultFormatString = null;
		//System.out.println("<SCRIPT>");
		out.write("<script type=\"text/javascript\">");

		out.write("var LocaleFormats = new Array();");
		defaultFormatString = (String)_numberFormatHash.get("default");

		if(defaultFormatString == null)
		{
			defaultFormatString = "LocaleFormats["+"\"default\""+"] = new Array("
												+"\",\"," 
												+"\",\","
												+"\".\","
												+"3);";
			_numberFormatHash.put("default",defaultFormatString);
		}
		//System.out.println(defaultFormatString);
		out.write(defaultFormatString);

		// NOTE: We check if there is a formatted string already in the hash for
		// this locale. If so, print out the formatted String and close out /SCRIPT
		// and RETURN. If not found in hash, go ahead and construct it, put it in hash.
		formattedString = (String)_numberFormatHash.get(locale);
		if(formattedString != null)
		{
			//System.out.println(formattedString);
			//System.out.println("</SCRIPT>");
			out.write(formattedString);
			out.write("</script>");
			return;
		}

		// Extract locale language (en) and country (US) from locale string = "en_US"
		// If locale string is like "fr", loacale object is initialized with 'null' for country (2nd arg in constructor)
		String localeLanguage = locale.substring(0,2);
		String localeCountry = "";
		Locale localeObject;
		if(locale.length()==5)
		{
			localeCountry  = locale.substring(3,5);
			localeObject = new Locale(localeLanguage, localeCountry);
		}
		else
		{
			localeObject = new Locale(localeLanguage, "");
		}
		
		
		// get currency format for this locale object
		NumberFormat curInstance = NumberFormat.getCurrencyInstance(localeObject);
		NumberFormat numInstance = NumberFormat.getNumberInstance(localeObject);
		double myNum = 123456789.00;
		String myNumString = numInstance.format(myNum);
		String myNumCurrencyString = curInstance.format(myNum);

		DecimalFormatSymbols decimalFormatSymbols = new DecimalFormatSymbols(localeObject);
		//DecimalFormat decimalFormat = new DecimalFormat(myNumString,decimalFormatSymbols);
		//DecimalFormat decimalFormat = (DecimalFormat)curInstance;
		DecimalFormat decimalFormat = new DecimalFormat();
		decimalFormat.setDecimalFormatSymbols(decimalFormatSymbols);
		
		char decimalSep  = decimalFormatSymbols.getDecimalSeparator();
		char groupingSep = decimalFormatSymbols.getGroupingSeparator();
		char monetaryDecimalSep = decimalFormatSymbols.getMonetaryDecimalSeparator();
		
		int  groupingSize;

		groupingSize = decimalFormat.getGroupingSize();
		
		String groupingSizeString;
		if(localeCountry.equalsIgnoreCase("IN"))
			groupingSizeString = "3,2";
		else
			groupingSizeString = String.valueOf(groupingSize);
							
		// Print out the JavaScript (as in FormatInfo.js)
		// Target javascript: 
		//		var LocaleFormats = new Array();
		// 		LocaleFormats["en_US"] = new Array("a","b","c",d);
		//		a -> Thousands separator for money
		//		b -> Thousands separator for number (a = b for most countries except sv_SE -> Sweden)
		//		c -> decimal separator
		//		d -> Grouping size (usually = 3; for Arabic language, equals 1, meaning no grouping; India special case: d = 3,2
		
		if(formattedString == null)
		{
			if(locale.equals("sv_SE"))
			{
				formattedString = "LocaleFormats["+"\""+locale+"\""+"] = new Array("
											+"\".\"," 
											+"\" \","
											+"\",\","
											+"3);";
			}
			else if(locale.equals("es_BO") ||
					locale.equals("es_CO") ||
					locale.equals("es_CR") ||
					locale.equals("es_DO") ||
					locale.equals("es_EC") ||
					locale.equals("cs_CZ") ||
					locale.equals("fr_LU") ||
					locale.equals("no_NO") ||
					locale.equals("pl_PL")
					)
			{
				formattedString = "LocaleFormats["+"\""+locale+"\""+"] = new Array("
											+"\".\"," 
											+"\".\","
											+"\",\","
											+"3);";
			}
			else if(locale.equals("fr_CH") ||
					locale.equals("it_CH")
					)
			{
				formattedString = "LocaleFormats["+"\""+locale+"\""+"] = new Array("
											+"\" \"," 
											+"\" \","
											+"\".\","
											+"3);";
			}
			else if(locale.equals("nl_BE"))
			{
				formattedString = "LocaleFormats["+"\""+locale+"\""+"] = new Array("
											+"\" \"," 
											+"\" \","
											+"\",\","
											+"3);";
			}
			else if(locale.equals("pt_PT"))
			{
				formattedString = "LocaleFormats["+"\""+locale+"\""+"] = new Array("
											+"\".\"," 
											+"\".\","
											+"\"$\","
											+"3);";
			}
			else if(locale.equals("es_PE"))
			{
				formattedString = "LocaleFormats["+"\""+locale+"\""+"] = new Array("
											+"\",\"," 
											+"\",\","
											+"\".\","
											+"3);";
			}
			else if(localeLanguage.equalsIgnoreCase("ar"))
			{
				formattedString = "LocaleFormats["+"\""+locale+"\""+"] = new Array("
												+"\"\"," 
												+"\"\","
												+"\",\","
												+"1);";
			}
			else
			{
				formattedString = "LocaleFormats["+"\""+locale+"\""+"] = new Array("
												+"\""+groupingSep+"\"," 
												+"\""+groupingSep+"\","
												+"\""+decimalSep+"\","
												+groupingSizeString+");";
			}
			_numberFormatHash.put(locale,formattedString);
			//System.out.println(formattedString);
			//System.out.println("</SCRIPT>");
			out.write(formattedString);
			out.write("</script>");
		}
	}
	
	public static HashMap getNumberFormatHash()
	{
		return _numberFormatHash;
	}
	
	// driver to test the NumberFormatter methods.
	public static void main(String[] args)
	{
		//NumberFormattingInfo.printNumberFormattingInfo("cs_CZ");		
	}
}
