/////////////////////////////////////////////////////////////////////////
//  IBM Confidential
//  
//  OCO Source Materials
//  
//  PID xxxx-xxx
//  
//   Copyright IBM Corp.  2003
//  
//  The source code for this program is not published or other-
//  wise divested of its trade secrets, irrespective of what has
//  been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////


/*
 * ProcessExecution.java
 *
 * Created on December 28, 2000, 12:03 AM
 */

package com.ibm.odcb.jrender.misc;


import java.io.*;


/**
 *
 * @author  ldh
 * @version 
 */
public class ProcessExecution extends Object 
 {
   public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;
   
   public ProcessExecution() 
    {
    }

   public static final int Exec(String CommandLine, String ErrorMessage, 
                                int AsynchBufferWait, int AsynchBufferInitSize, int ErrBufferInitSize)
    {
      Process                   P                 = null;
      BufferedInputStream       TempCompilerError = null;
      AsynchBufferedInputStream CompilerError     = new AsynchBufferedInputStream(AsynchBufferWait, AsynchBufferInitSize);
      ByteArrayOutputStream     ErrStream         = new ByteArrayOutputStream(ErrBufferInitSize);
      int Err = SystemValues.EVIL_VALUE;
      try
       {
         P = Runtime.getRuntime().exec(CommandLine);
         TempCompilerError = new BufferedInputStream(P.getErrorStream());
         CompilerError.Init(TempCompilerError, ErrStream);
         CompilerError.start();
         P.waitFor();
         Err = P.exitValue();
         CompilerError.join();
       } 
      catch (IOException e)
       {
         Streamer.error.Header().printStackTrace(e);
         return SystemValues.EVIL_VALUE;
       }
      catch (InterruptedException e)
       {
         Streamer.error.Header().printStackTrace(e);
         return SystemValues.EVIL_VALUE;
       }
      finally
       {
         try
          {
            if (P != null)
             P.destroy();
            if (TempCompilerError != null)
             TempCompilerError.close();
            if (ErrStream != null)
             ErrStream.close();
          }
         catch (IOException e)
          {
            Streamer.error.Header().printStackTrace(e);
            return SystemValues.EVIL_VALUE;
          }
       }
       
      if (ErrStream.size() != 0)
       {
         // Write the compiler error messages, to the real stream 
         Streamer.error.Header().println(ErrorMessage).println(ErrStream.toByteArray());
         return -1;
       }

      return Err;
    }
    
   public static final int Exec(String commandLine, String[] environment, File workingDirectory, String errorMessage, 
                                int asynchBufferWait, int asynchBufferInitSize, int errBufferInitSize)
    {
      Process                   p                 = null;
      BufferedInputStream       tempCompilerError = null;
      AsynchBufferedInputStream compilerError     = new AsynchBufferedInputStream(asynchBufferWait, asynchBufferInitSize);
      ByteArrayOutputStream     errStream         = new ByteArrayOutputStream(errBufferInitSize);
      int err = SystemValues.EVIL_VALUE;
      try
       {
         p = Runtime.getRuntime().exec(commandLine, environment, workingDirectory);
         tempCompilerError = new BufferedInputStream(p.getErrorStream());
         compilerError.Init(tempCompilerError, errStream);
         compilerError.start();
         p.waitFor();
         err = p.exitValue();
         compilerError.join();
       } 
      catch (IOException e)
       {
         Streamer.error.Header().printStackTrace(e);
         return SystemValues.EVIL_VALUE;
       }
      catch (InterruptedException e)
       {
         Streamer.error.Header().printStackTrace(e);
         return SystemValues.EVIL_VALUE;
       }
      finally
       {
         try
          {
            if (p != null)
             p.destroy();
            if (tempCompilerError != null)
             tempCompilerError.close();
            if (errStream != null)
             errStream.close();
          }
         catch (IOException e)
          {
            Streamer.error.Header().printStackTrace(e);
            return SystemValues.EVIL_VALUE;
          }
       }
       
      if (errStream.size() != 0)
       {
         // Write the compiler error messages, to the real stream 
         Streamer.error.Header().println(errorMessage).println(errStream.toByteArray());
         return -1;
       }

      return err;
    }
 }
