/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2000, 2002 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.odcb.jrender.misc;

import java.text.MessageFormat;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

public class ResourceHandler {

	private static ResourceBundle fgResourceBundle;
	public static final String RESOURCE_NAME = "com/ibm/odcb/odcb_messages";

	/**
	 * Returns the resource bundle used by all classes in this Project
	 */
	public static ResourceBundle getResourceBundle() {
		try {
			return ResourceBundle.getBundle(RESOURCE_NAME); //$NON-NLS-1$
		} catch (MissingResourceException e) {
			// does nothing - this method will return null and
			// getString(String) will return the key
			// it was called with
		}
		return null;
	}
	public static String getString(String key) {
		if (fgResourceBundle == null) {
			fgResourceBundle = getResourceBundle();
		}

		if (fgResourceBundle != null) {
			try {
				return fgResourceBundle.getString(key);
			} catch (MissingResourceException e) {
				return "!" + key + "!"; //$NON-NLS-2$//$NON-NLS-1$
			}
		} else {
			return "!" + key + "!"; //$NON-NLS-2$//$NON-NLS-1$
		}
	}
	public static String getString(String key, Object[] args) {

		try {
			return MessageFormat.format(getString(key), args);
		} catch (IllegalArgumentException e) {
			return getString(key);
		}
	}
/*	public static void main(String[] args)
	{
		String msg = ResourceHandler.getString("ECLASSMAP_NOT_FOUND_MESSAGE", new String[]{"classExportName"});
		System.out.println("ECLASSMAP_NOT_FOUND_MESSAGE: " + msg);
		msg = ResourceHandler.getString("GETTER_NOT_FOUND_MESSAGE", new String[]{"featureName"});
		System.out.println("GETTER_NOT_FOUND_MESSAGE: " + msg);
		msg = ResourceHandler.getString("EFEATUREMAP_NOT_FOUND_MESSAGE", new String[]{"featureExportName", "classExportName"});
		System.out.println("EFEATUREMAP_NOT_FOUND_MESSAGE: " + msg);
		msg = ResourceHandler.getString("FILE_NOT_FOUND_MESSAGE",new String[]{"eMapFilePath"});
		System.out.println("FILE_NOT_FOUND_MESSAGE: " + msg);
	}*/
}
