/////////////////////////////////////////////////////////////////////////
//  IBM Confidential
//  
//  OCO Source Materials
//  
//  PID xxxx-xxx
//  
//   Copyright IBM Corp.  2003
//  
//  The source code for this program is not published or other-
//  wise divested of its trade secrets, irrespective of what has
//  been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////


/*
 * SaxUtilities.java
 *
 * Created on March 1, 2002, 3:07 PM
 */

package com.ibm.odcb.jrender.misc;


import java.io.File;
import java.io.IOException;
import java.io.InputStream;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.HandlerBase;
import org.xml.sax.SAXException;


public class SaxBase extends HandlerBase
 {
   public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;
   
   public SaxBase()
    {
    }

   protected int    _Line;
   protected int    _Warning;
   protected int    _Error  ;
   protected String _XmlFileSource;

   public int    getWarning      () { return _Warning      ; }
   public int    getError        () { return _Error        ; }
   public String getXmlFileSource() { return _XmlFileSource; }
   
  /**
   * Sets the XMLFileSource.
   * @param theXMLFileSource The theXMLFileSource to set
   */
  public void setXmlFileSource(String theXMLFileSource)
  {
    this._XmlFileSource = theXMLFileSource;
  }

   protected final void Parse(String XmlFileSource)
   throws Exception
    {
      try
       {
         _XmlFileSource = XmlFileSource;
         SAXParser Sax = null;
         SAXParserFactory Factory = SAXParserFactory.newInstance();
         Sax = Factory.newSAXParser();
         _Error   = 0   ;
         _Warning = 0   ;
         _Line    = 1   ;
         Streamer.status.Header().println("Starting  parsing XML file "+_XmlFileSource);
         Sax.parse(new File(_XmlFileSource), this);
         Streamer.status.Header().println("Ending parsing XML file "+_XmlFileSource);
         if (_Error != 0)
          {
          	Streamer.error.Header().println("There were "+_Error+" Errors.");
          }
         if (_Warning != 0)
          {
          	Streamer.error.Header().println("There were "+_Warning+" Warnings.");
          }
       }
      catch (ParserConfigurationException X)
       {
         Streamer.error.Header().println("XML Parser Configuration Exception");
         Streamer.error.Header().printStackTrace(X);
         throw X;
       }
      catch (SAXException X)
       {
         Streamer.error.Header().println("Parsing Exception on file");
         throw X;
       }
      catch (IOException X)
       {
         Streamer.error.Header().println("File I/O Exception");
         throw X;
       }
    }
    
   protected final void Parse(InputStream is)
   throws Exception
    {
      try
       {
         SAXParserFactory Factory = SAXParserFactory.newInstance();
         SAXParser Sax = Factory.newSAXParser();
         _Error   = 0   ;
         _Warning = 0   ;
         _Line    = 1   ;
         Streamer.status.Header().println("Starting  parsing XML InputStream");
         Sax.parse(is, this);
         Streamer.status.Header().println("Ending parsing XML InputStream");
         if (_Error != 0)
          {
          Streamer.error.Header().println("There were "+_Error+" Errors.");
          }
         if (_Warning != 0)
          {
          Streamer.error.Header().println("There were "+_Warning+" Warnings.");
          }
       }
      catch (ParserConfigurationException X)
       {
         Streamer.error.Header().println("XML Parser Configuration Exception");
         Streamer.error.Header().printStackTrace(X);
         throw X;
       }
      catch (SAXException X)
       {
         Streamer.error.Header().println("Parsing Exception on InputStream");
         throw X;
       }
    }

   public String CheckAttribute(org.xml.sax.AttributeList Attributes, String Name, String Default, boolean Mandatory)
    {
      String XXX = Attributes.getValue(Name);
      if (XXX == null && Mandatory==true)
       {
         Streamer.error.Header().println("Line " + _Line + ", missing attribute '"+Name+"'.");
         ++_Error;
       }
      if (XXX == null)
       XXX = Default;
      return XXX;
    }
    
   public int CheckAttributeInt(org.xml.sax.AttributeList Attributes, String Name, int Default, boolean Mandatory)
    {
      String XXX = CheckAttribute(Attributes, Name, null, Mandatory);
      int YYY = Default;
      if (XXX != null)
       try
        {
          YYY = Integer.parseInt(XXX);
        }
       catch (NumberFormatException X)
        {
          Streamer.error.Header().println("Line " + _Line + ", invalid value type for '"+Name+"'. A positive integer expected.");
          ++_Error;
        }
      return YYY;
    }
    
   public boolean CheckAttributeBool(org.xml.sax.AttributeList Attributes, String Name, boolean Default, boolean Mandatory)
    {
      String XXX = CheckAttribute(Attributes, Name, null, Mandatory);
      boolean YYY = Default;
      if (XXX != null)
       {
         Boolean Res = EnumBoolean.getType(XXX);
         if (Res != null)
          YYY = Res.booleanValue();
         else
          {
            Streamer.error.Header().println("Line " + _Line + ", invalid value type for '"+Name+"'. A boolean 'Y', '1', 'N' or '0' expected.");
            ++_Error;
          }
       }
      return YYY;
    }
 }
