/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//
//OCO Source Materials
//
//PID xxxx-xxx
//
// Copyright IBM Corp.  2003
//
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////

package com.ibm.odcb.jrender.misc;

/**
 * $Id:
 *
 * (C) Copyright International Business Machines Corp., 2001,2002
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U. S. Copyright Office.
 */

//import com.ibm.mps.utl.StringBuffer;


public class SpecialSymbolHandler {
/**
 * SpecialSymbolHandler constructor comment.
 */
public SpecialSymbolHandler() {
	super();
}
/**
 * Replace character '<' and '>' in input String with &#unicode;
 * @return java.lang.String
 * @param input java.lang.String
 */
public static String replaceLessThanGreaterThan(String input) {
	if (input == null || input.equals(""))
	{
		return "";
	}

	StringBuffer outputBuffer = new StringBuffer();

    char c;

    for (int i = 0; i < input.length(); i++) {

        c = input.charAt(i);

        if (c == '\u003C' || c == '\u003E') {
            outputBuffer.append("&#" + (int) c + ";");
        }
        else {
            outputBuffer.append(c);
        }
    }
    return outputBuffer.toString();
}
/**
 * Replace character '>' (greater than), '<' (less than),
 * '''(apos) and '"'(double quote) with &#unicode;.
 * This method is for display <input type="text" value="input"> properly on html browser
 * @return java.lang.String
 * @param input java.lang.String
 */
public static String replaceLtGtAposQuot(String input) {
    if (input == null || input.equals("")) {
        return "";
    }

    StringBuffer outputBuffer = new StringBuffer();

    char c;

    for (int i = 0; i < input.length(); i++) {

        c = input.charAt(i);

        if (c == '\u003C' || c == '\u003E') {
            outputBuffer.append("&#" + (int) c + ";");
        }
        else if (c == '\'') {
            outputBuffer.append("&#39;");
        }
        else if (c == '"'){
	        outputBuffer.append("&#34;");
        }
        else {
            outputBuffer.append(c);
        }
    }
    return outputBuffer.toString();
}
/**
 * Replace any special character, whose unicode is
 * greater than 127, in input String with &#unicode;
 * @return java.lang.String
 * @param input java.lang.String
 */
public static String replaceSpecialSymbol(String input) {
	if (input == null || input.equals(""))
	{
		return "";
	}

    StringBuffer outputBuffer = new StringBuffer();

    char c;

    for (int i = 0; i < input.length(); i++) {

        c = input.charAt(i);

        if (c > '\u007F') {
            outputBuffer.append("&#" + (int) c + ";");
        }
        else {
            outputBuffer.append(c);
        }
    }
    return outputBuffer.toString();
}
/**
 * Replace pattern &#unicode; in input String back to special symbol
 * @return java.lang.String
 * @param input java.lang.String
 */
public static String revertBackToSpecialSymbol(String input) {

    if (input == null || input.equals("")) {
        return "";
    }

    StringBuffer outputBuffer = new StringBuffer(); // store output string
    StringBuffer tempBuffer = new StringBuffer(); // store found pattern

    int state = 0;
    char c;

    for (int i = 0; i < input.length(); i++) {

        c = input.charAt(i);

        switch (state) {
            case 0 :
                if (c == '&') {
                    state = 1;
                    tempBuffer.append(c);
                }
                else {
                    outputBuffer.append(c);
                }
                break;
            case 1 :
                if (c == '#') {
                    state = 2;
                    tempBuffer.append(c);
                }
                else {
                    state = 0;
                    outputBuffer.append(tempBuffer);
                    outputBuffer.append(c);
                    tempBuffer.delete(0, tempBuffer.length());	//clear up tempBuffer
                }
                break;
            case 2 :
                if (c >= '0' && c <= '9') {
                    state = 3;
                    tempBuffer.append(c);
                }
                else {
                    state = 0;
                    outputBuffer.append(tempBuffer);
                    outputBuffer.append(c);
                    tempBuffer.delete(0, tempBuffer.length());
                }
                break;
            case 3 :
                if (c >= '0' && c <= '9') {
                    tempBuffer.append(c); // state is still 3
                }
                else if (c == ';') {
                    //found pattern &#unicode;
                    state = 0;
                    tempBuffer.append(c);
                    String unicodeStr = tempBuffer.substring(2, tempBuffer.length() - 1);
                    int unicode = Integer.parseInt(unicodeStr);
                    outputBuffer.append((char) unicode);
                    tempBuffer.delete(0, tempBuffer.length());
                }
                else {
                    state = 0;
                    outputBuffer.append(tempBuffer);
                    outputBuffer.append(c);
                    tempBuffer.delete(0, tempBuffer.length());
                }
                break;
            default :
                throw new RuntimeException("Invalid State");
        }
    }
    return outputBuffer.toString();
}


/**
 * Replace character '''(apos), '"'(double quote) and '\' (back slash) with \char.
 * This method is for display alert message properly on html browser
 * @return java.lang.String
 * @param input java.lang.String
 */
public static String replaceAposQuotBackSlash(String input) {
    if (input == null || input.equals("")) {
        return "";
    }

    StringBuffer outputBuffer = new StringBuffer();

    char c;

    for (int i = 0; i < input.length(); i++) {

        c = input.charAt(i);

        if (c == '\'' || c == '"' || c == '\\') {
            outputBuffer.append("\\" + c);
        }
        else {
            outputBuffer.append(c);
        }
    }
    return outputBuffer.toString();
}

/**
 * Replace character '"'(double quote) with &#34;.
 * This method is for display <a href="javascript:f()" title="input"> properly on html browser
 * @return java.lang.String
 * @param input java.lang.String
 */
public static String replaceQuot(String input) {
    if (input == null || input.equals("")) {
        return "";
    }

    StringBuffer outputBuffer = new StringBuffer();

    char c;

    for (int i = 0; i < input.length(); i++) {

        c = input.charAt(i);

        if (c == '"'){
	        outputBuffer.append("&#34;");
        }
        else {
            outputBuffer.append(c);
        }
    }
    return outputBuffer.toString();
}
}
