/////////////////////////////////////////////////////////////////////////
//  IBM Confidential
//  
//  OCO Source Materials
//  
//  PID xxxx-xxx
//  
//   Copyright IBM Corp.  2003
//  
//  The source code for this program is not published or other-
//  wise divested of its trade secrets, irrespective of what has
//  been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////


package com.ibm.odcb.jrender.misc;

import java.io.*;
import java.util.*;


//import jscg.utils.exceptions.*;

public final class Streamer extends Object
 {
   public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;
   
   protected final static String NULL       = "null";

   protected Streamer(int StreamLevel, String Header)
    {
      _StreamLevel = StreamLevel;
      _Header      = Header     ;
      if (_Stream == null)
       synchronized(_Mutex)
        {
          if (_Stream == null)
           Init();
        }
    }
   
   public static void ForceStdErr()
    {
      if (_Stream != System.err)
       {
         _Stream.close();
         _Stream = System.err;
       }
    }

   protected static final Calendar _START_TIME    = Calendar.getInstance();

   public static Calendar getStartTime() { return _START_TIME   ; }
    
   protected static Object _Mutex = new Object();
        
   protected static final boolean OVERRIDE_STDERR = false;
   protected static final synchronized void Init()
    {
       String LogFileName = Config.getStringSilent(Config.PROP_SERVER_LOG_FILENAME);
       if (LogFileName == null || LogFileName.trim().length() == 0)
        {
          _Stream = System.err;
          if (OVERRIDE_STDERR == true)
           {
             try
              {
                _Stream = new PrintStream(new FileOutputStream("c:\\logs.txt"));
              }
              catch (FileNotFoundException E)
               {
                 _Stream = System.err;
                 System.err.println("Could not open file 'c:\\logs.txt' for logging. Using System.err.");
               }
           }
        }
       else
        {
          try
           {
             _Stream = new PrintStream(new FileOutputStream(LogFileName));
           }
          catch (FileNotFoundException E)
           {
             _Stream = System.err;
             System.err.println("Could not open file '"+LogFileName+"' for logging. Using System.err.");
           }
        }
    }
    

   protected static ThreadLocal _S = new ThreadLocal();
   
   public static StringBuffer StartThread()
    {
      StringBuffer S = (StringBuffer) _S.get();
      if (S == null)
       {
         S = new StringBuffer(4096);
         _S.set(S);
       }
      return S;
    }
    
   static final int MAX_LOG_SIZE = 1024*1024*8;
   static       int _LogSize = 0;

   // This class separates the operations to _Stream so that the minimum of execution time needs to be synchronized
   static class MoreConcurrency
    {
      public static synchronized void Write(String S, int StreamLevel)
       {
//         _Stream.println("\n\n- - - - - - - - - - - - - - - - - T H R E A D - L O G - F L U S H - - - - - - - - - - - - - - - - - - - - - - - - - -");
//         _Stream.println("SIZE IS: "+S.length());
         _LogSize+=S.length();
         if (_Stream == System.err && StreamLevel > 1)
          {
          	System.out.print(S);
          }
         else
          {
            _Stream.print(S);
          }
       }
      public static synchronized void Write(String S, Throwable T)
       {
         Write(S, 0);
         T.printStackTrace(_Stream);
       }
    }

    
   public static final String NEWLINE = System.getProperty("line.separator"); 
   
   public static void FlushThreadInternal(StringBuffer S, int StreamLevel)
    {
      FlushThread(S, StreamLevel);
    }

	public static void FlushThread()
	 {
      StringBuffer S = (StringBuffer)_S.get();
      FlushThread(S, 1);
    }
	public static void FlushThread(StringBuffer S, int StreamLevel)
	 {
		MoreConcurrency.Write(S.toString(), StreamLevel);
		S.setLength(0);
	 }

   protected static void FlushThread(Throwable T)
    {
      StringBuffer S = (StringBuffer)_S.get();
      MoreConcurrency.Write(S.toString(), T);
      S.setLength(0);
    }
    
   public static void EndThread()
	{
	  EndThread(1);
	}

   public static void EndThread(int StreamLevel)
    {
      StringBuffer S = (StringBuffer)_S.get();
      MoreConcurrency.Write(S.toString(), StreamLevel);
      _S.set(null);
      S=null;
    }
   
   protected static PrintStream  _Stream       = null;
   protected static int          _LogLevel     = Config.getIntegerSilent(Config.PROP_SERVER_LOG_LEVEL, 1);
   protected static int          _SavedLogLevel= SystemValues.EVIL_VALUE;
   
   protected static int          _LookAhead     = 256;
   protected static int          _SavedLookAhead=SystemValues.EVIL_VALUE;

   protected final  int          _StreamLevel    ;
   protected final  String       _Header         ;
  
   public static int  getLogLevel (            ) { return _LogLevel    ; }
   public static void setLogLevel (int LogLevel) { _LogLevel = LogLevel; }
   public static void pushLogLevel(int LogLevel) { if (_SavedLogLevel == SystemValues.EVIL_VALUE) _SavedLogLevel = _LogLevel; _LogLevel = LogLevel; }
   public static void popLogLevel (            ) { _LogLevel = _SavedLogLevel; _SavedLogLevel = SystemValues.EVIL_VALUE; }

   public static int  getLookAhead (             ) { return _LookAhead    ; }
   public static void setLookAhead (int LookAhead) { _LookAhead = LookAhead; }
   public static void pushLookAhead(int LookAhead) { if (_SavedLookAhead == SystemValues.EVIL_VALUE) _SavedLookAhead = _LookAhead; _LookAhead = LookAhead; }
   public static void popLookAhead (             ) { _LookAhead = _SavedLookAhead; _SavedLookAhead = SystemValues.EVIL_VALUE; }
   
   public static PrintStream getStream() { return _Stream; }
   
   public final boolean isActivated() { return _StreamLevel <= _LogLevel; }

   
   public final Streamer Header () { if (_StreamLevel <= _LogLevel) StartThread().append(_Header); return this; }

   public final Streamer println(         ) { if (isActivated() == true) FlushThreadInternal(StartThread().append(NEWLINE), _StreamLevel); return this; }

   public final Streamer print  (boolean x) { if (_StreamLevel <= _LogLevel) StartThread().append(x); return this; }
   public final Streamer println(boolean x) { if (_StreamLevel <= _LogLevel) FlushThreadInternal(StartThread().append(x).append(NEWLINE), _StreamLevel); return this; }

   public final Streamer print  (long    x) { if (_StreamLevel <= _LogLevel) StartThread().append(x); return this; }
   public final Streamer println(long    x) { if (_StreamLevel <= _LogLevel) FlushThreadInternal(StartThread().append(x).append(NEWLINE), _StreamLevel); return this; }

   public final Streamer print  (double  x) { if (_StreamLevel <= _LogLevel) StartThread().append(x); return this; }
   public final Streamer println(double  x) { if (_StreamLevel <= _LogLevel) FlushThreadInternal(StartThread().append(x).append(NEWLINE), _StreamLevel); return this; }

   public final Streamer print  (char[]  x) { if (isActivated() == true) StartThread().append(new String(x)); return this; }
   public final Streamer println(char[]  x) { if (isActivated() == true) FlushThreadInternal(StartThread().append(new String(x)).append(NEWLINE), _StreamLevel); return this; }

   public final Streamer print  (byte[]  x) { if (isActivated() == true) StartThread().append(new String(x)); return this; }
   public final Streamer println(byte[]  x) { if (isActivated() == true) FlushThreadInternal(StartThread().append(new String(x)).append(NEWLINE), _StreamLevel); return this; }

   public final Streamer print  (String  x) { if (isActivated() == true) StartThread().append(x); return this; }
   public final Streamer println(String  x) { if (isActivated() == true) FlushThreadInternal(StartThread().append(x).append(NEWLINE), _StreamLevel); return this; }

   public final Streamer print  (Object  x) { if (isActivated() == true) StartThread().append((x!=null)?x.toString():NULL); return this; }
   public final Streamer println(Object  x) { if (isActivated() == true) FlushThreadInternal(StartThread().append((x!=null)?x.toString():NULL).append(NEWLINE), _StreamLevel);return this; }

   public final Streamer printSubstring(String  x, int i       ) { if (isActivated() == true) FlushThreadInternal(StartThread().append(x.substring(i, (i+_LookAhead > x.length() ? x.length()-i : _LookAhead))).append(NEWLINE), _StreamLevel); return this; }
   public final Streamer printSubstring(String  x, int i, int j) { if (isActivated() == true) FlushThreadInternal(StartThread().append(x.substring(i, j-i+1                                                  )).append(NEWLINE), _StreamLevel); return this; }
   
   public final Streamer printSubstring(char[]  x, int i       ) { if (isActivated() == true) FlushThreadInternal(StartThread().append(new String(x, i, (i+_LookAhead > x.length ? x.length-i : _LookAhead))).append(NEWLINE), _StreamLevel); return this; }
   public final Streamer printSubstring(char[]  x, int i, int j) { if (isActivated() == true) FlushThreadInternal(StartThread().append(new String(x, i, j-i+1                                              )).append(NEWLINE), _StreamLevel); return this; }
 
   public final Streamer printSubstring(byte[]  x, int i       ) { if (isActivated() == true) FlushThreadInternal(StartThread().append(new String(x, i, (i+_LookAhead > x.length ? x.length-i : _LookAhead))).append(NEWLINE), _StreamLevel); return this; }
   public final Streamer printSubstring(byte[]  x, int i, int j) { if (isActivated() == true) FlushThreadInternal(StartThread().append(new String(x, i, j-i+1                                              )).append(NEWLINE), _StreamLevel); return this; }

   public final Streamer printStackTrace(Throwable T) { if (isActivated() == true) { FlushThread(T); } return this; }
   
   public final static Streamer defect = new Streamer(-1, "DEFECT!: T-H-I-S...S-H-O-U-L-D...N-E-V-E-R...H-A-P-P-E-N.!!!!! ");
   public final static Streamer error  = new Streamer( 0, "ERROR  : ");
   public final static Streamer warning= new Streamer( 1, "WARNING: ");
   public final static Streamer status = new Streamer( 2, "STATUS : ");
   public final static Streamer debug  = new Streamer( 3, "DEBUG  : ");
   public final static Streamer trace  = new Streamer( 4, "TRACE  : ");
   
   public final static void Separator()
    {
       FlushThreadInternal(StartThread().append(NEWLINE)
                                        .append(NEWLINE)
                                        .append(NEWLINE)
                                        .append("------------------------------------------------------------------------------------------").append(NEWLINE)
                                        .append(NEWLINE), 2);
    }
    
   public static void main (String args[])
    {
//      Streamer.ForceStdErr();
      Streamer.StartThread();

         try
          {
            PropertiesUtil Prop = new PropertiesUtil();
            Prop.Open("D:\\projects\\OBF\\odc-jsf\\OdysseyBrowserFramework.properties");
            Prop.AddProperty(Config.PROP_ECORE_FILES.toString(), "toto.ecore");
            Prop.AddProperty(Config.PROP_EMAP_FILES .toString(), "toto.emap" );
            Prop.Save();
            Config.ReInit();
          }
         catch (Exception E)
          {
            Streamer.error.Header().println("Cannot update the properties file resources");
            Streamer.error.Header().printStackTrace(E);
          }
      
      Streamer.Init();
      Object O = new Object();
      try
       {
         for (int i = 0; i < 10; ++i)
          {
          	Streamer.error.Header().println("Hello...");
			Streamer.warning.Header().println("Hello...");
			Streamer.debug.Header().println("Hello...");
            if (i % 10 == 0)
             Streamer.FlushThread();
            int Total = 0;
            for (int j = 0; j < 1000; ++j)
             {
               ++Total;
             }
          }
       }
      catch (Exception E)
       {
         Streamer.error.Header().printStackTrace(E);
       }
      Streamer.EndThread();
    }
 }
