/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//
//OCO Source Materials
//
//5724-E76 and 5724-E77
//
// Copyright IBM Corp.  2003
//
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////
package com.ibm.odcb.jrender.misc;

import java.lang.reflect.Array;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.ArrayList;


/**
 *  This Class traverses down through a bean like value structure
 *  it returns whether not a beans attributes contain values.
 *  It is a utility class that helps the tree renderer deal
 *  with page data binding.
 */
public class ValueInspector {
	
	
	Object rootValue;
	Object currentValue;
	
	boolean first = true;
	
	/* this map represents one level of the data tree accross all nodes
	 *                  o
	 *                 / \           
	 *                o   o
	 *               /   / \ 
	 *              *   *   *
	 *             /   / \ / \
	 *            o   o  oo   o 
	 *  so for the above one it would represent all the '*' nodes
	 */
	HashMap classValues = new HashMap();
	
/**
 *  Constructor
 *  @param value is the root of the bean or data object. 
 */
	public ValueInspector(Object value)
	{
		this.rootValue = value;
		this.currentValue = rootValue;
	}

/**
 * This method gets the current level of the data tree
 * checks if an attribute of a class has a value in the 
 * tree. 
 * The object  
 * @param attributeName
 *  Name of the attribute to inspect
 * @param className
 *  The class this attribute is in 
 * @return
 */	
	public boolean hasValue(String attributeName, String className)
	{
	
		Object tmpValue;
		boolean isRoot = true;
		if  (first)
		{	
			tmpValue = rootValue;
			first = false;
		}
		else {
		isRoot = false;
			if (classValues.containsKey(className) != false)
			{	
				tmpValue = classValues.get(className);
				
				
				if (tmpValue == null)
				{
					return false;
				}
	
			}
			else
			{
				return false;
			}
		}
	
		if (isRoot)
		{
			return inspectValue(attributeName,tmpValue);
		}
		else 
		{
			ArrayList theArray = (ArrayList)tmpValue;
			for (int i=0; i<theArray.size(); i++)
			{
				
				boolean hasValue = inspectValue(attributeName,theArray.get(i));
			
				
				if (hasValue)	
				{
					return true;
				}
					
			}
			
		}
			
		
		return false;
		

	}
	
	/**
	 * This method checks if an attribute of a class has a value in the 
	 * object passed. 
	 * The object  
	 * @param attributeName
	 *  Name of the attribute to inspect
	 * @param obj
	 *  The object to check for the attributes value in 
	 * @return
	 */	
	private boolean inspectValue(String attributeName, Object currentObj)
	{
		try {
			Class c = currentObj.getClass();
			
			String methodName = "get"+attributeName.substring(0,1).toUpperCase()+attributeName.substring(1);
			
			
			
			if (c.isArray())
			{
				c = c.getComponentType();
				
				Method m = c.getMethod(methodName,null);
				if (m != null)
				{	
					Object tst = null;
					
					for (int i=0;i<Array.getLength(currentObj);i++)
					{	
						Object tmpValue2 = Array.get(currentObj,0);
						
						tst = tst!=null?tst:m.invoke(tmpValue2,null);
						
					}
					
					if ( tst == null)
					{
						return false;
						
					}
					
				}
				
				
			} else 
			{	
				
				Method m = c.getMethod(methodName,null);
				
				if ( m == null)
				{
					return false;
				}
				
				Object aa = m.invoke(currentObj,null);
				
				if ( aa == null)
				{
					return false;
					
				}
			}
			
		}
		catch (Exception ex)
		{
			return false;
			
		}
		
		return true;
		
	}
	
	
	/**
	 * This method gets the current level of the data tree
	 * checks if an attribute of a class has a value in the 
	 * tree and then sets this as the current level in the tree.
	 * The object  
	 * @param attributeName
	 *  Name of the attribute to inspect
	 * @param className
	 *  The class this attribute is in 
	 * @return
	 */	
	public void setClassObject(String parentClassName, String className, String referenceName)
	{
		
		Object tmpValue = rootValue;
		
		
		// if in here it is not the the root.
		if (classValues.containsKey(parentClassName) != false)
		{	
			tmpValue = classValues.get(parentClassName);
			
			
			
			if (tmpValue == null)
			{
				classValues.put(className,null);
				
			}
			
			
			try {
				
				String methodName = "get"+referenceName.substring(0,1).toUpperCase()+referenceName.substring(1);
				
				ArrayList anArray = new ArrayList();
				
				ArrayList objectArray = (ArrayList)tmpValue;
				
				for (int j=0; j<objectArray.size(); j++) {
					
				
					Object objectValue = ((ArrayList)tmpValue).get(j);
					
					Class c = objectValue.getClass();
					
					// method name is in bean format: ie get<attribute> 
					// with the starting letter capitalised
					
					
					
					if (c.isArray())
					{
						c = c.getComponentType();
						
						Method m = c.getMethod(methodName,null);
						if (m != null)
						{	
							if (Array.getLength(objectValue) == 0)
							{
								
								classValues.put(className,null);
							}
							
							
							
							
							
							for (int i=0; i<Array.getLength(objectValue);i++)
							{	
								Object tmpValue2 = Array.get(objectValue,i);
								
													
								Object nodeValue = m.invoke(tmpValue2,null);
								
								anArray.add(nodeValue);
							
							}
							
				
						}
					}
					else {
						Method m = c.getMethod(methodName,null);
						if (m != null)
						{	
							
							Object nodeValue = m.invoke(objectValue,null);
														
							anArray.add(nodeValue);
				
						}
					}
				}	
				
			
				
				classValues.put(className,anArray);
			}
			catch (Exception ex)
			{
				Object c = ex;
				
			
			}
			
			
		
		} 
		else {		
			try {
				Class c = tmpValue.getClass();
				
				// method name is in bean format: ie get<attribute> 
				// with the starting letter capitalised
				String methodName = "get"+referenceName.substring(0,1).toUpperCase()+referenceName.substring(1);
				
				ArrayList anArray = new ArrayList();
				
				
				if (c.isArray())
				{
					c = c.getComponentType();
					
					Method m = c.getMethod(methodName,null);
					if (m != null)
					{	
						if (Array.getLength(tmpValue) == 0)
						{
							
							classValues.put(className,null);
						}
						
						
						
						
						
						for (int i=0; i<Array.getLength(tmpValue);i++)
						{	
							Object tmpValue2 = Array.get(tmpValue,0);
							
												
							Object aa = m.invoke(tmpValue2,null);
							
							anArray.add(aa);
					//		classValues.put(className,tmpValue);
					//		
						}
						
						classValues.put(className,anArray);
					}
				}
				else {
					Method m = c.getMethod(methodName,null);
					if (m != null)
					{	
						
						Object aa = m.invoke(tmpValue,null);
						
						
						
						anArray.add(aa);
						classValues.put(className,anArray);
						
					//	classValues.put(className,aa);
					}
				}
					}
			catch (Exception ex)
			{
				Object c = ex;
			}
			
		}
		
		
	}
	
	

}
