/////////////////////////////////////////////////////////////////////////
//  IBM Confidential
//  
//  OCO Source Materials
//  
//  PID xxxx-xxx
//  
//   Copyright IBM Corp.  2003
//  
//  The source code for this program is not published or other-
//  wise divested of its trade secrets, irrespective of what has
//  been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////

package com.ibm.odcb.jrender.utilities;

import java.io.File;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;

import com.ibm.odcb.jrender.misc.ClassGraphWalker;
import com.ibm.odcb.jrender.misc.Streamer;
import com.ibm.odcb.jrender.misc.TypesUtil;

/**
 * @author ldh
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class Bean2ECoreGen
 {
   public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;
   public static class Params
    {
      public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;
      public static final String USAGE="USAGE: Bean2ECoreGen <project root path> <relative ecore file path>\n"
                                      +"                     <root java class>\n"
                                      +"                     ( <path expression> <java class> )*\n"
                                      +"                     ( -include|-exclude ( <path expression> P|C )* )\n"
                                      +"   You must specify the root of your project path (such as c:\\projects\\MyApp\\src) \n"
                                      +" and the relative path from there to where you want the ecore file to be output (such \n"
                                      +" as com/mycompany/myapp/myecores/abc.ecore).\n"
                                      +"   Then, based on the root Java class specified, the utility recurses through all its\n"
                                      +" members and writes out a matching ECore file. For generic types Collection and \n"
                                      +" Iterator, the utility looks up the supplied list of path expressions to find a matching\n"
                                      +" Java type. If not found, the property is discarded. For instance, a Java bean A may have\n"
                                      +" a property x of type List, which contains instances of Java bean com.ibm.B. The utility \n"
                                      +" would expect a pair 'com.ibm.A.x  com.ibm.B', where 'com.ibm.A.x' is the \"path\" to the \n"
                                      +" property to match, and 'com.ibm.B' is the type of the contained Java bean B.\n"
                                      +"   Then, you can specify classes ('C') and properties ('P') that you want to specifically\n"
                                      +" include or exclude. Like the previous list, you specify a \"path\" starting with the fully\n"
                                      +" qualified class name, and add information whether the path specifies a class or a property.\n"
                                      +" If the list is included, nothing else but what is specified on the list will be in the ECore.\n"
                                      +" If the list is excluded, then everything but what is in the list will be in the ECore. If the\n"
                                      +" list is not supplied, then everything in the classes found will be processed.\n\n";
      public Params()
       {
       }

      public File    _BasePath;
      public File    _ECoreFile;
      public String  _RootJavaClassName;
      public Map     _Mappings = new HashMap(17);
      public boolean _Include = false; 
      public Map     _IncludesExcludes = new HashMap(17);

      public boolean Parse(String[] args)
       {
         if (args.length < 3)
          {
            Streamer.error.Header().println("The utility needs to be invoked with at least 3 parameters.");
            return false;
          }

         _BasePath = new File(args[0]);
         if (_BasePath.isDirectory() == false)
          {
            Streamer.error.Header().println("The first parameter '"+args[0]+"' is not a valid path.");
            return false;
          }

         if (args[1].endsWith(".ecore") == false)
          {
            Streamer.error.Header().println("The second parameter '"+args[1]+"' needs to end in '.ecore'.");
            return false;
          }
          
         _ECoreFile = new File(args[0].replace('/',File.separatorChar)+File.separator+args[1].replace('/',File.separatorChar));
         File Dir = new File(_ECoreFile.getParent());
         if (Dir.isDirectory() == false && Dir.mkdirs() == false)
          {
            Streamer.error.Header().println("The second parameter '"+_ECoreFile+"' does not point to a valid path, or a path which cannot be created off the project root path");
            return false;
          }
          
         _RootJavaClassName = args[2];

         // If there are no more parameters, then we are finished.          
         if (args.length == 3)
          {
            return true;
          }

         // Let's get the mappings list
         int i = 3;
         String Val=null;
         while (i < args.length)
          {
            Val = args[i].toLowerCase();
            if (Val.equals("-include") || Val.equals("-exclude"))  // encountering the delimiter, break the loop.
             {
               break;
             }
            if (i+1 < args.length)
             {
               Streamer.error.Header().println("There is an incomplete pair of arguments in the mappings list starting with "+ args[i]);
               return false;
             }
            _Mappings.put(args[i], args[i+1]);
            i+=2;
          }

         // If there are no more parameters, then we are finished.          
         if (i == args.length)
          {
            return true;
          }
          
         // Let's do the Include/Exclude list now
         _Include = Val.equals("-include");
         ++i;
         while (i < args.length)
          {
            if (i+1 < args.length)
             {
               Streamer.error.Header().println("There is an incomplete pair of arguments in the include/exclude list starting with "+ args[i]);
               return false;
             }
            String ClassOfPropertyFlag = args[i+1];
            if (   ClassOfPropertyFlag.equals(ClassGraphWalker.CLASS_FLAG) == false
                || ClassOfPropertyFlag.equals(ClassGraphWalker.PROPERTY_FLAG) == false
               )
              {
                Streamer.error.Header().println("The pair of arguments in the include/exclude list starting with "+ args[i]+" does nor specify 'P' or 'C' as a ClassOrProperty flag.");
                return false;
              }
             _IncludesExcludes.put(args[i], args[i+1]);
            i+=2;
          }
          
         return true;
       }
    }

   public static boolean _RETURN = true;
   public static void main(String[] args)
    {
      Streamer.ForceStdErr();
      Streamer.StartThread();
      Streamer.status.println().Header().println(Params.USAGE);
        
      Params P = new Params();
      if (P.Parse(args) == true)
       {
         try
          {
            Process(P);
          }
         catch (Exception E)
          {
            Streamer.error.Header().printStackTrace(E);
            _RETURN = false;
          }
       }
      else
       {
         _RETURN = false;
       }

      Streamer.status.Header().println("Bean2ECoreGen ended.");
      Streamer.EndThread();
    }

   public static void Process(Params P)
   throws Exception
    {
      ECoreWalker W = new ECoreWalker(P._BasePath, P._ECoreFile);
      ClassGraphWalker CGW = new ClassGraphWalker(P._RootJavaClassName, P._Mappings, P._Include, P._IncludesExcludes);
      CGW.Walk(W);
      _KnownTypes = W._Types;
    }

   static protected HashMap _KnownTypes;
   public static HashMap GetKnownTypesAndClear() { HashMap X = _KnownTypes; _KnownTypes = null; return X; }
   
   public static class ECoreWalker implements ClassGraphWalker.Walker
    {
      public ECoreWalker(File BasePath, File ECoreFile) throws Exception
       {
         _BasePath  = BasePath;
         _ECoreFile = ECoreFile;
         _Out = new PrintWriter(new FileOutputStream(_ECoreFile));
         Streamer.status.Header().println("Generating file '"+_ECoreFile.getPath()+"'");
       }

      protected File        _BasePath;
      protected File        _ECoreFile;
      protected String      _PackageName;
      protected String      _JavaClassName;
      protected PrintWriter _Out;
      protected HashMap     _StringBuffers = new HashMap(17);
      protected HashMap     _Types = new HashMap(17);

      public boolean Begin(Class C)
       {
         _Out.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
         _Out.println("<ecore:EPackage xmi:version=\"2.0\" xmlns:xmi=\"http://www.omg.org/XMI\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:ecore=\"http://www.eclipse.org/emf/2002/Ecore\"");

         // extract the file nanme - ".ecore" extension
         String S = _ECoreFile.getName();
         S = S.substring(0,S.indexOf(".ecore"));
         _Out.println("                name=\""+S+"\"");
         _PackageName = S;
         
         // generate URI based on the file name path - the project root path
         S = _ECoreFile.getPath().substring(_BasePath.getPath().length()+1).replace('\\','/');
         _Out.println("                nsURI=\"http:///"+S+"\"");
         
         // generate prefix based on the file name path with slashes as periods and ".ecore" removed
         S = S.substring(0, S.lastIndexOf(".ecore")).replace('/', '.');
         _Out.println("                nsPrefix=\""+S+"\">");

         return true;
       }

      public boolean ProcessClassBegin(String Path, Class C, int Level)
       {
         StringBuffer Str = new StringBuffer(256);
         Str.append("   <eClassifiers xsi:type=\"ecore:EClass\" name=\""+MakeEClassifier(C.getName())+"\">");
         _StringBuffers.put(C.getName(), Str);
         return true;
       }

      public boolean ProcessBasicProperty(String Path, Class C, String PropertyName, String PropertyType, boolean Many, boolean ReadOnly, int Level)
       {
         StringBuffer Str = (StringBuffer) _StringBuffers.get(C.getName());
         String eType = TypesUtil.getEDataTypeFromJavaType(PropertyType);
         if (eType.equals("WDODate"))
          eType="ecore:EDataType http://ibm.com/wdo#//WDODate";
         else if (eType.equals("WDOBigDecimal"))
          eType="ecore:EDataType http://ibm.com/wdo#//WDOBigDecimal";
         else if (eType.equals("EJavaObject"))
          eType = "ecore:EDataType http://www.eclipse.org/emf/2002/Ecore#//EString";
         else
          eType = "ecore:EDataType http://www.eclipse.org/emf/2002/Ecore#//" + eType;
         Str.append(Streamer.NEWLINE)
            .append("      <eAttributes name=\"").append(PropertyName)
          //.append("\" eType=\"ecore:EDataType http://www.eclipse.org/emf/2002/Ecore#//").append(eType)
            .append("\" eType=\"").append(eType)
            .append("\" lowerBound=\"").append((Many==true?0:1))
            .append("\" upperBound=\"").append((Many==true?-1:1));
         if (ReadOnly == true)
          {
            Str.append("\" changeable=\"false");
          }
         Str.append("\"/>");
         return true;
       }

      public boolean ProcessComplexProperty(String Path, Class C, String PropertyName, String PropertyType, boolean Many, boolean ReadOnly, boolean PropertyFound, int Level)
       {
         if (PropertyFound == false)
          return false;
         StringBuffer Str = (StringBuffer) _StringBuffers.get(C.getName());
         Str.append(Streamer.NEWLINE)
            .append("      <eReferences name=\"").append(PropertyName)
            .append("\" eType=\"#//").append(MakeEClassifier(PropertyType))
            .append("\" lowerBound=\"0\" upperBound=\"").append(Many==true?-1:1)
            .append("\" containment=\"false");
         if (ReadOnly == true)
          {
            Str.append("\" changeable=\"false");
          }
         Str.append("\"/>");

         // If the class has a property to itself, it doesn't count. For example, let's assume that a class has no bean property 
         // per se, exscept for proprtties to itself: we get effectively a class with no accessible real properties. so the class
         // is a phantom. For example:
         // class XXX
         //  {
         //     public XXX getParent();
         //  }
         // For this type of class, there is nothing we can get out of it 
         return PropertyType.equals(C.getName()) == false;
       }

      public boolean ProcessClassEnd(String Path, Class C, boolean PropertyFound, int Level)
       {
         StringBuffer Str = (StringBuffer) _StringBuffers.remove(C.getName());
         if (PropertyFound == true) // That class had at least one property, so we push it out
          {
            _Out.println();
            _Out.println(Str);
            _Out.println("   </eClassifiers>");
          }
         else // That class did not have at least one property, so we push it commented out
          {
            _Out.println();
            _Out.print("<!--");
            _Out.print(Str);
            _Out.println("</eClassifiers>   -->");
            RemoveEClassifier(C.getName());
          }
         return PropertyFound;
       }

      public boolean End(Class C)
       {
         _Out.println();
         _Out.println("</ecore:EPackage>");
         _Out.close();
         return true;
       }
       
      public final void RemoveEClassifier(String Type)
       {
         Streamer.debug.Header().println("Removing "+Type);
         Iterator I = _Types.entrySet().iterator();
         while (I.hasNext() == true)
          {
            Map.Entry E = (Map.Entry) I.next();
            String S = (String) E.getValue();
            if (S.equals(Type) == true)
             {
               _Types.remove((String) E.getKey());
               Streamer.debug.Header().println("    -->> Removed "+S+":"+(String) E.getKey());
               break;
             }
          }
       }

      public final String MakeEClassifier(String Type)
       {
         // LDH-NOTE: try to make this simple in the base case, where there isn't any name collision. Keep
         //   a hashmap os known ClassNames (without full path), and only push full path with '_' if there
         //   is a collision. For nested classes, do the same thing, but without the owning class.

         int i = Type.lastIndexOf('$');
         boolean NestedClass;
         if (i == -1)
          {
            i = Type.lastIndexOf('.');
            NestedClass = false;
          }
         else
          {
            NestedClass = true;
          }
         String ClassName = Type.substring(i+1);

         String KnownType = (String) _Types.get(ClassName);
         if (KnownType == null) // First time we hear of this class, so register it, and then return the class name
          {
            Streamer.debug.Header().println("Adding "+ClassName+":"+Type);
            _Types.put(ClassName, Type);
            return ClassName;  // Take care of nested classes
          }
         // Name collision.
         if (KnownType.equals(Type) == true) // Hello friend. We already heard of that class, so it's all fine.
          return ClassName;
          
         // Huh, huh... Same class name, but different type. We need to create the alias
         if (NestedClass == true)  // If nested class, let's try using the enclosing name, and play the same game
          {
            ClassName = Type.substring(Type.lastIndexOf('.')+1);
            KnownType = (String) _Types.get(ClassName);
            if (KnownType == null) // First time we hear of this class, so register it, and then return the class name
             {
               Streamer.debug.Header().println("Adding "+ClassName+":"+Type);
               ClassName = ClassName.replace('$', '_');
               _Types.put(ClassName, Type);
               return ClassName;  // Take care of nested classes
             }
            // Name collision.
            if (KnownType.equals(Type) == true) // Hello friend. We already heard of that class, so it's all fine.
             return ClassName.replace('$', '_');
          }

         // We get lofs of collisions here. the class name is already used, and so is the nested class name +
         // the owning class name. So all we have left is the full class package.
         String EClassType = Type.replace('.','_').replace('$','_');
         Streamer.debug.Header().println("Adding "+EClassType+":"+Type);
         _Types.put(EClassType, Type);
         return Type;
       }
    }
 }
