/////////////////////////////////////////////////////////////////////////
//  IBM Confidential
//  
//  OCO Source Materials
//  
//  PID xxxx-xxx
//  
//   Copyright IBM Corp.  2003
//  
//  The source code for this program is not published or other-
//  wise divested of its trade secrets, irrespective of what has
//  been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////


package com.ibm.odcb.jrender.utilities;

import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.io.File;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.EMap;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EStructuralFeature;

import com.ibm.odcb.jrender.mediators.DynamicMediator;
import com.ibm.odcb.jrender.mediators.ExportException;
import com.ibm.odcb.jrender.mediators.MappingsMerger;
import com.ibm.odcb.jrender.mediators.MediatorFactory;
import com.ibm.odcb.jrender.mediators.gen.EcoreLoader;
import com.ibm.odcb.jrender.mediators.gen.MappingsParser;
import com.ibm.odcb.jrender.mediators.gen.WDO4JSMappings;
import com.ibm.odcb.jrender.mediators.gen.ecore.EClassMap;
import com.ibm.odcb.jrender.mediators.gen.ecore.EFeatureMap;
import com.ibm.odcb.jrender.mediators.gen.ecore.EPackageMap;
import com.ibm.odcb.jrender.misc.EMFHelper;
import com.ibm.odcb.jrender.misc.ODCClassLoader;
import com.ibm.odcb.jrender.misc.Streamer;
import com.ibm.odcb.jrender.misc.StringUtil;
import com.ibm.odcb.jrender.misc.TypesUtil;


/**
 * @author ldh
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class EMapGen
 {
   public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;
   public static final String MERGE_ARG = "-merge";
   public static final String ROOT_ARG  = "-root" ;

	public static class Params
 	 {
      public Params()
       {
       }
      
      public String    _SourcePath          = null ;
      public String    _BuildPath           = null ;
      public String    _EcoreFileName       = null ;
      public String    _RootEClassName      = null ;
      public Map       _Mappings            = null ;
      public String    _EmapFileNameToMerge = null ;
       
      public boolean Parse(String[] args)
       {
         if (args.length < 3)
          {
            Streamer.error.Header().println("This program was called with an incorrect number of parameters.");
            return false;
          }
         _SourcePath = args[0];
         _BuildPath  = args[1];

         int ECoreindex = args[2].lastIndexOf(".ecore");
         if (ECoreindex == -1 || ECoreindex != args[2].length()-6)
          {
            Streamer.error.Header().println("The filename supplied '"+args[2]+"' is incorrect. Expected a filename ending in '.ecore'.");
            return false;
          }
         _EcoreFileName = args[2];

         int i = 3;
         if (i < args.length && args[i].equalsIgnoreCase(EMapGen.ROOT_ARG) == true)
          {
            ++i;
            if (i < args.length)
             {
               _RootEClassName = args[i];
               ++i;
             }
            else
             {
               Streamer.error.Header().println("We found a -root without a follow-up parameter");
               return false;
             }
          }
         else
          {
            Streamer.error.Header().println("The -root parameter is mandatory, but could not be found.");
            return false;
          }
          
         _Mappings = new HashMap(7);
         while (i < args.length)
          {
            //if you see "-merge" in the argument list, break
            if (args[i].equalsIgnoreCase(EMapGen.MERGE_ARG) == true)
             {
               break;
             }
            if (i >= args.length-1) // Need room for 2 parameters
             {
               Streamer.error.Header().println("The EClassName parameter '"+args[i]+"' is the last parameter. It should be followed by an JavaClassName paramater.");
               return false;
             }
            _Mappings.put(args[i].replace('$','_'), args[i+1]);
            i+=2;
          }
               
         if (i < args.length)
          {
            if (args[i].equalsIgnoreCase(EMapGen.MERGE_ARG) == true)
             {
               ++i;
               if (i < args.length)
                {
                  _EmapFileNameToMerge = args[i];
                }
               else
                {
                  MakeDefaultEMapMerge();
                }
             }
            else
             {
               Streamer.error.Header().println("The parameter '"+args[i]+"' is unexpected. We should either have -merge, or the end of the parameter list.");
               return false;
             }
          }

         return true;
       }
       
      public void MakeDefaultEMapMerge()
       {
         int emapindex = _EcoreFileName.lastIndexOf(".ecore");
         _EmapFileNameToMerge = _EcoreFileName.substring(0, emapindex)+".emap";
       }
    }

   public static void main(String[] args)
    {
      Streamer.ForceStdErr();
      Streamer.StartThread();
      Streamer.status.Header().println("USAGE: EMapGen <source path> <build path> <full file name for a .ecore file> -root <root_eclass_name> { <eclass_name> <java_class_name> )*  {-merge <merge_EMap_file>}")
                     .Header().println("EMapGen utility:"
           + "\n     If no list of <eclass name> <java class name> pairs is provided, the Java bean is assumed to be a WDO and the entire ECore will be mapped."
           + "\n     \"-root\" indicates the name of the EClass that acts as the root class for the ECore. That class must be mapped thereafter."
           + "\n     If the optional \"-merge\" feature is used, the new EMap generated will retain any previous customizations to the EMap."
           + "\n     If an EMap file name is provided, the custom mappings from this file will be used. "
           + "\n     If the EMap file name is omitted, the utility will look for an emap file with the same name as the target EMap."
           + "\n     If the EMap file to merge is not found, a new EMap will be created without merging."
           + "\n     The -merge feature is false by default.");

      Params P = new Params();
      if (P.Parse(args) == true)
       {
         try
          {
            Process(P);
          }
         catch (Exception E)
          {
            Streamer.error.Header().printStackTrace(E);
          }
       }

      Streamer.status.Header().println("WDO4JSEMapGen ended.");
      Streamer.EndThread();
    }

   public static void Process(Params P) throws Exception
    {
   	  WDO4JSMappings universe = loadEcore(P._EcoreFileName);
      EPackage EP = (EPackage) universe.getEPackagesIterator().next();
      P._Mappings = initParams(P._Mappings, EP);
      String eMapFileName = P._EcoreFileName.substring(0,P._EcoreFileName.length() - 6)+".emap";
      
      Streamer.status.Header().println("EMapGen: Creating the matching EMap file '"+eMapFileName+"' and merging with mappings from EMap file '."+P._EmapFileNameToMerge+"'");   
      
      Streamer.debug.Header().println("EMapGen: Creating EPackageMap"); 
      EPackageMap epMap = createEPackageMap(EP);
      Streamer.debug.Header().println("EMapGen: Creating EClassMaps");
      HashMap ecMaps = createEClassMaps(EP, P._Mappings, eMapFileName, P._SourcePath);

      if (P._EmapFileNameToMerge != null)
       {
      		
         Streamer.debug.Header().println("EMapGen: Trying to merge with the existing EMap file '"+P._EmapFileNameToMerge+"'");
         mergeWithExistingEMapFile(ecMaps, epMap, P._EmapFileNameToMerge);
       }
       
      Streamer.debug.Header().println("EMapGen: Making backups of any existing emaps");
      MakeBackups(eMapFileName);
      Streamer.status.Header().println("EMapGen: Writing EMap to file");
      PrintWriter Out = new PrintWriter(new FileOutputStream(eMapFileName));
      //ecMaps = currentUniverse.getEClassMapsForEPackage(pkg.getName());
      Out = writeEMap(Out, epMap, P._RootEClassName, ecMaps.values().iterator());
      Out.close();
      Streamer.status.Header().println("EMapGen: EMap generation successfully completed for: '"+eMapFileName+"'.");
    }

   public static final String ACCESS_METHOD_NEEDED = "CUSTOM_JAVA_CODE_FRAGMENT_NEEDED";


   private static WDO4JSMappings loadEcore(String ecoreFileName) 
    {
      Streamer.debug.Header().println("EMapGen: Looking at '"+ecoreFileName+"' and attempting to create a matching EMap file");
      Streamer.status.Header().println("EMapGen: Reading the EPackage in file '"+ecoreFileName+"'.");
      URI ecoreURI = URI.createFileURI(ecoreFileName);
      Streamer.debug.Header().println("EMapGen: Creating local universe");
      WDO4JSMappings universe = new WDO4JSMappings();
      EcoreLoader el = new EcoreLoader(universe);
      boolean OK = el.load(ecoreURI);
      if (OK == true)
       {
         Streamer.status.Header().println("EMapGen: Successfully loaded the EPackage from file '"+ecoreFileName+"'.");
       }
      else
       {
         Streamer.error.Header().println("EMapGen: Cannot load the EPackage from file '"+ecoreFileName+"'.");
         return null;
       }
      return universe;
    }


   public static Map initParams(Map Mappings, EPackage EP)
    {
      ////////////////////////////////////////////////////
      //LDH-NOTE: Changes here to handle dynamic WDOs
      ////////////////////////////////////////////////////
      // If Params is null, we need to map all EClasses defined, and we map to WDOs. So let's iterate through the ECore and fill in
      // the params automatically
      if (Mappings == null || Mappings.size() == 0)
      {
         Mappings = new HashMap(11);
         Iterator I = EP.getEClassifiers().iterator();
         while (I.hasNext() == true)
          {
            EClass EC = (EClass) I.next();
            Mappings.put(EC.getName(), DynamicMediator.MakeComplexName(EP.getName(), EC.getName()));
          }
       }
      return Mappings;
    }
   
   public static EPackageMap createEPackageMap(EPackage EP)
   {
   		EPackageMap EPM = new EPackageMap(EP.getName());
   		EPM.setEPackage(EP);
   		return EPM;
   }
   
   public static HashMap createEClassMaps(EPackage EP, Map Mappings, String eMapFileName, String BaseSourcePath)
    {
      Streamer.debug.Header().println("EMapGen: Parsing the resource name of the EMap file '"+eMapFileName+"'");
      String emapResourceName = parseEMapResourceName(eMapFileName, BaseSourcePath);
      Streamer.debug.Header().println("EMapGen: The resource name of the EMap file is '"+emapResourceName+"'");

      HashMap ecMaps = new HashMap();
      List L = EP.getEClassifiers();
      for (int i = 0; i < L.size(); ++i)
      {
         EClass C = (EClass) L.get(i);
         String JavaClassName = (String) Mappings.get(C.getName());
         Streamer.debug.Header().println("Mapping for EClass '"+C.getName()+"' is '"+JavaClassName+"'.");
         if (JavaClassName != null)
          {
            EClassMap ECM = createEClassMap(JavaClassName, C, eMapFileName, emapResourceName, Mappings);
            if (ECM != null)
             {
               ecMaps.put(ECM.getExport(), ECM);
             }
          }
       } 
      return ecMaps;
    }


    private static EClassMap createEClassMap(String JavaComplexName, EClass C, String eMapFileName, String emapResourceName, Map Mappings)
    {
      boolean DynamicWDO = DynamicMediator.isComplexNameDynamic(JavaComplexName);
      Streamer.debug.Header().println("EMapGen: Creating EClassMap for '"+C.getName()+"'");
      EClassMap ECM = new EClassMap(C.getEPackage().getName(), C.getName(), JavaComplexName
                                   ,C.getEPackage().getName()+"_"+C.getName(), eMapFileName
                                   ,emapResourceName, true);
      ECM.setEClass(C);
      
      // Let's get the Class maping to the parent Java class
      Class JavaClass;
      PropertyDescriptor[] PDs;
      try
       {
         JavaClass = ODCClassLoader.getClass(ECM.getBaseType());
         PDs = Introspector.getBeanInfo(JavaClass).getPropertyDescriptors();
       }
      catch (Exception E)
       {
         Streamer.error.Header().println("Cannot locate the class '"+ECM.getBaseType()+"' to create a mapping from.");
         Streamer.error.Header().printStackTrace(E);
         return null;
       }

      List L = C.getEAllStructuralFeatures();
      Streamer.debug.Header().println("EMapGen: Creating EFeatureMaps for EClassMap '"+C.getName()+"'");
      for (int i = 0; i < L.size(); ++i)
       {
         EStructuralFeature F = (EStructuralFeature) L.get(i);
         EList eanns = F.getEAnnotations();	// EAnnotation support by Adam in 6.0.1
         									// RATLC1085997
         boolean ro = false; // default read-only to false
         boolean iD = true;	 // default part of primary key to false
         
         // Only set iD to true if cardinality is 1.
         // if Domino, Siebel, SAP then set primary keys' iDto True
         if( (eanns!=null) && (eanns.size()>0) ){
         	EAnnotation ea = (EAnnotation)eanns.get(0); // assumes only one EAnnotation child tag
         												// if >1 then takes first only.
         	EMap em = ea.getDetails();
         	ro = ( ((String)em.get("readOnly")).equals("true") ) ? true : false; // check Domino(/SAP/Siebel) read only property
         	// unless explicitly not part of primary key, assume it is.
         	iD = ( ((String)em.get("primaryKey")).equals("true") ) ? true : false; // check Domino(/SAP/Siebel) primary key property
         }

         //set iD to false if cardinality is >1   
         iD = ( !F.isMany() && iD );

         //set iD = false if EType is WDODate - need not do this for SDO     
         
////#IF# WDO
//         String EFeatureEDataTypeName = TypesUtil.getRealETypeName(F.getEType());
//         if (EFeatureEDataTypeName != null)
//           if (EFeatureEDataTypeName.equals(TypesUtil.DATE_ETYPE))
//           {
//            iD = false;
//            Streamer.warning.Header().println("EAttribute '"+F.getName()+"'' is of EType WDODate.  Attributes of EType WDODate cannot be a primary key.  The iD attribute will be set to FALSE.");          
//           }
////#END#        

         String Getter = FindProperty(F, JavaClass, PDs, true);
         String Setter;
         
         if ( (F.isChangeable() == false) || (ro == true) ){
         	Setter = null;
         }else{
         	Setter = FindProperty(F, JavaClass, PDs, false);  
         }
         
         String FeatureName = DynamicWDO ? getWDOFeatureName(F,C.getEPackage().getName())
                                         : F.getName();

         Streamer.debug.Header().println("EMapGen: Mapped EFeature '"+FeatureName+"' to getter '"+Getter+"', with iD='"+iD+"'.");
         EFeatureMap f = new EFeatureMap(FeatureName,iD,Getter, Setter, true);
         f.setEFeature(F);
         f.setParentMapping(ECM);
         ECM.addEFeatureMap(f);
         
         if ( iD ){
         	ECM.addEIdFeatureMap(f);
         }
       }
       return ECM;
    }


      public static String getWDOFeatureName(EStructuralFeature f, String parentClassEPackageName)
      {
         boolean reference = f instanceof EReference;
         String featureName = f.getName();
         // If this is a reference, for a dynamic WDO, pointing to a class in the same package, then we assume the other class is also
         // an automatic WDO, and therefore, we can generate the extra information.
         if (reference == true)
          {
            String featureEPackage = f.getEType().getEPackage().getName();
            if (featureEPackage.equals(parentClassEPackageName) == true)
             featureName = featureName+"("+DynamicMediator.MakeExtraName(featureEPackage, TypesUtil.getRealETypeName(f.getEType()))+")";
          }
         return featureName;
      }

   
   // This method merges the EPackageMap as well as the EClassMaps with those from the existing EMap - Adam
   private static void mergeWithExistingEMapFile(HashMap ecMaps, EPackageMap epMap, String eMapFileNameToMergeWith) throws ExportException
    {
   	
      WDO4JSMappings oldMaps = getEClassMapsforEMapFile(eMapFileNameToMergeWith);
      WDO4JSMappings current = MediatorFactory.getInstance().getUniverse();
      
      EPackageMap oldEPMap = current.getEPackageMap(epMap.getName());
      
      boolean OK = true;
      if (oldEPMap != null)
      {
      	MappingsMerger.mergeEPackageMaps(epMap, oldEPMap);
      }
      else
      {
      	OK = false;
      	Streamer.warning.Header().println("EMapGen: No EpackageMap to merge.");
      }
      
      if (oldMaps != null)
      {
        
      	Iterator newit = ecMaps.values().iterator();
         while (newit.hasNext())
         {
            EClassMap newmap = (EClassMap) newit.next();
            EClassMap oldmap = oldMaps.getEClassMapByJavaComplexName(newmap.getJavaComplexName());
            MappingsMerger.mergeEClassMaps(newmap, oldmap);
         }
      }
      else
      {
      	 OK = false;
      	 Streamer.warning.Header().println("EMapGen: No EClassMaps to Merge.");
      }
      
      if(OK == false)
      {
        Streamer.warning.Header().println("EMapGen: Errors merging with the EMap file '"+eMapFileNameToMergeWith+"'.");
      }
      else
      {
      	Streamer.status.Header().println("EMapGen: Merge with '"+eMapFileNameToMergeWith+"' successful.");
      }
   }

   private static WDO4JSMappings getEClassMapsforEMapFile(String eMapFileName)
   {
   WDO4JSMappings universe = null;
   Streamer.status.Header().println("EMapGen: Trying to load EMap file '"+eMapFileName+"'");
    File file = new File(eMapFileName);
    if (file.exists())
    {
    universe = new WDO4JSMappings();
     MappingsParser parser = new MappingsParser(universe);
    boolean ok = false;
     try
     {
      ok = parser.parse(file);
    }
    catch (Exception e){}
    if (ok)
    {
      Streamer.status.Header().println("EMapGen: Successfully loaded the EMap from file '"+eMapFileName+"'.");
    }
    else
    {
      Streamer.error.Header().println("EMapGen: Cannot load the EMap from file '"+eMapFileName+"'.");
      return null;
    }
   }
   else
   {
    Streamer.warning.Header().println("EMapGen: The EMap file '"+eMapFileName+"' does not exist.");
   }
      return universe;
   }


   public static void MakeBackups(String EMapFileName)
    {
      for (int i = 8; i > 0; --i)
       {
         // In windows, and most IDEs, there is often an association between extensions and editors
         // By keeping the extension constant here, we make things easier for developers.
         File F = new File(EMapFileName+"."+i+".saved");
         if (F.exists() == true)
          {
            File FileToDelete = new File(EMapFileName+"."+(i+1)+".saved");
            FileToDelete.delete();
            F.renameTo(FileToDelete);
          }
       }
      File F = new File(EMapFileName);
      if (F.exists() == true)
       {
         F.renameTo(new File(EMapFileName+".1.saved"));
       }
    }

    public static PrintWriter writeEMap(PrintWriter out, EPackageMap ePackageMap, String RootEClassName, Iterator eClassMaps)
    {      
       try
      {
        out.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
        out.println("<mappings>");
        //out.println("  <EPackage name=\""+pkgName+"\" root_eclass_name=\""+RootEClassName+"\">");
        Streamer.debug.Header().println("EMapGen: Writing mapping for EPackageMap '"+ePackageMap.getName()+"'");
        writeEPackageMapTag(out, ePackageMap, RootEClassName);
        
        while (eClassMaps.hasNext())
         {
          EClassMap em = (EClassMap) eClassMaps.next();
          Streamer.debug.Header().println("EMapGen: Writing mapping for EClassMap '"+em.getEClassName()+"' and it's children");
          writeEClassMap(out,em);
         }
        out.println("  </EPackage>");
        out.println("</mappings>");
      }
      catch (Exception E)
      {
        Streamer.error.printStackTrace(E);
      }
      return out;
    }
    
    public static PrintWriter writeEPackageMapTag(PrintWriter out, EPackageMap epm, String RootEClassName )
    {
    	out.println("  <EPackage name=\""+epm.getName()+"\" root_eclass_name=\""+RootEClassName+"\""
				+ ( epm.getJSURI() == null ? "" : (" js_URI=\"" + epm.getJSURI() + "\"") )
				+ ( epm.getOnUpdate() == null ? "" : (" on_update=\"" + epm.getOnUpdate() + "\"") )
				+ ( epm.getOnCreate() == null ? "" : (" on_create=\"" + epm.getOnCreate() + "\"") )
				+ ( epm.getOnDelete() == null ? "" : (" on_delete=\"" + epm.getOnDelete() + "\"") )
				+ ">");
    	
    	return out;
    }

   public static String FindProperty(EStructuralFeature F, Class JavaClass,
                                     PropertyDescriptor[] PDs, boolean Getter)
    {
      // Let's resolve whether this is a field with multiple instance or single instance (cardinality > 1 or not).
      boolean Single = (F.getLowerBound() == 0 || F.getLowerBound() == 1) && F.getUpperBound() == 1;

      ////////////////////////////////////////////////////////////////////////////////////////////////////
      ////////////////////////////////////////////////////////////////////////////////////////////////////
      // Check for SDO
      ////////////////////////////////////////////////////////////////////////////////////////////////////
      if (EMFHelper.DataObject_CLASS.isAssignableFrom(JavaClass) == true)
       {
         if (Single == false)
          return Getter ? "getList('"+F.getName()+"')" 
                        : "setList('"+F.getName()+"', "+EFeatureMap.__SETTER_PARAM_MARKER+")";
          
         if (F instanceof EAttribute)
          {
            return Getter ? TypesUtil.getEDataGet(TypesUtil.getRealETypeName(F.getEType()))+"('"+F.getName()+"')"
                          : TypesUtil.getEDataSet(TypesUtil.getRealETypeName(F.getEType()))+"('"+F.getName()+"', "+EFeatureMap.__SETTER_PARAM_MARKER+")";
          }
         else if (F instanceof EReference)
          {
            return Getter ? "get('"+F.getName()+"')"
                          : "set('"+F.getName()+"', "+EFeatureMap.__SETTER_PARAM_MARKER+")";
          }
         else
          {
            Streamer.error.Header().println("The EFeature "+F.getName()+" for class "+JavaClass.getClass().getName()+" must be either an EReference or an EAttribute");
            return ACCESS_METHOD_NEEDED;
          }
       }

      ////////////////////////////////////////////////////////////////////////////////////////////////////
      ////////////////////////////////////////////////////////////////////////////////////////////////////
      // GETTERS / SETTERS
      ////////////////////////////////////////////////////////////////////////////////////////////////////
      String FeatureName = F.getName();   // Default feature name
      String FeatureName2= FeatureName;   // If default feature name starts with a '_', feature name without '_'
      String FeatureName3= FeatureName;   // If feature name is "name", becomes "Name. If "Name", becomes "name"
      {
        int i = 0;
        if (FeatureName.charAt(i) == '_')
         {
           ++i;
           FeatureName2 = FeatureName.substring(i);
         }
       char firstChar = FeatureName.charAt(i);
       firstChar = Character.isLowerCase(firstChar) ? Character.toUpperCase(firstChar) : Character.toLowerCase(firstChar);
       FeatureName3 = firstChar + FeatureName.substring(i+1);
      }
      
      String RealEDataType = TypesUtil.getRealETypeName(F.getEType());
      String PropertyTypeName  = TypesUtil.getJavaTypeFromEDataType(RealEDataType, F.isMany());
//      Class  PropertyTypeClass = TypesUtil.getPrimitiveTypeClass(PropertyTypeName);
//      if (PropertyTypeClass == null)  // At this point, we know we are not looking at a primitive type
//       {
         if (PropertyTypeName == null)  // If this is null, then we don't have a Basic Java type either
          {
            for (int i = 0; i < PDs.length; ++i)
             {
               PropertyDescriptor PD = PDs[i];
               if (   PD.getName().equals(FeatureName ) 
                   || PD.getName().equals(FeatureName2) 
                   || PD.getName().equals(FeatureName3)
                  )
                {
                  if (PD.getPropertyType() != null)
                   {
                     PropertyTypeName = PD.getPropertyType().getName();
                   }
                  else
                   {
                     PropertyTypeName = F.isMany() ? Object[].class.getName() : Object.class.getName();
                   }
                }
             }
//          }
//         try
//          { 
//            PropertyTypeClass = ODCClassLoader.getClass(PropertyTypeName);
//          }
//         catch (Exception E)
//          {
//            Streamer.error.Header().println("Cannot locate the class '"+PropertyTypeName+"' to create a mapping from.");
//            Streamer.error.Header().printStackTrace(E);
//            return ACCESS_METHOD_NEEDED;
//          }
       }

      if (Getter == true)
       {
         // Let's look for getters "get" and "Get", and "is" and "Is"
         if (TestMethod(JavaClass, "get"+FeatureName , PropertyTypeName, Getter) == true) { return "get"+FeatureName +"()"; }
         if (TestMethod(JavaClass, "get"+FeatureName2, PropertyTypeName, Getter) == true) { return "get"+FeatureName2+"()"; }
         if (TestMethod(JavaClass, "get"+FeatureName3, PropertyTypeName, Getter) == true) { return "get"+FeatureName3+"()"; }
         if (TestMethod(JavaClass, "Get"+FeatureName , PropertyTypeName, Getter) == true) { return "Get"+FeatureName +"()"; }
         if (TestMethod(JavaClass, "Get"+FeatureName2, PropertyTypeName, Getter) == true) { return "Get"+FeatureName2+"()"; }
         if (TestMethod(JavaClass, "Get"+FeatureName3, PropertyTypeName, Getter) == true) { return "Get"+FeatureName3+"()"; }
         if (TestMethod(JavaClass, "is" +FeatureName , PropertyTypeName, Getter) == true) { return "is" +FeatureName +"()"; }
         if (TestMethod(JavaClass, "is" +FeatureName2, PropertyTypeName, Getter) == true) { return "is" +FeatureName2+"()"; }
         if (TestMethod(JavaClass, "is" +FeatureName3, PropertyTypeName, Getter) == true) { return "is" +FeatureName3+"()"; }
         if (TestMethod(JavaClass, "Is" +FeatureName , PropertyTypeName, Getter) == true) { return "Is" +FeatureName +"()"; }
         if (TestMethod(JavaClass, "Is" +FeatureName2, PropertyTypeName, Getter) == true) { return "Is" +FeatureName2+"()"; }
         if (TestMethod(JavaClass, "Is" +FeatureName3, PropertyTypeName, Getter) == true) { return "Is" +FeatureName3+"()"; }
       }
      else
       {
         // Let's look for setters "set" and "Set"
         if (TestMethod(JavaClass, "set"+FeatureName , PropertyTypeName, Getter) == true) { return "set"+FeatureName +"("+EFeatureMap.__SETTER_PARAM_MARKER+")"; }
         if (TestMethod(JavaClass, "set"+FeatureName2, PropertyTypeName, Getter) == true) { return "set"+FeatureName2+"("+EFeatureMap.__SETTER_PARAM_MARKER+")"; }
         if (TestMethod(JavaClass, "set"+FeatureName3, PropertyTypeName, Getter) == true) { return "set"+FeatureName3+"("+EFeatureMap.__SETTER_PARAM_MARKER+")"; }
         if (TestMethod(JavaClass, "Set"+FeatureName , PropertyTypeName, Getter) == true) { return "Set"+FeatureName +"("+EFeatureMap.__SETTER_PARAM_MARKER+")"; }
         if (TestMethod(JavaClass, "Set"+FeatureName2, PropertyTypeName, Getter) == true) { return "Set"+FeatureName2+"("+EFeatureMap.__SETTER_PARAM_MARKER+")"; }
         if (TestMethod(JavaClass, "Set"+FeatureName3, PropertyTypeName, Getter) == true) { return "Set"+FeatureName3+"("+EFeatureMap.__SETTER_PARAM_MARKER+")"; }
       }
 
      ////////////////////////////////////////////////////////////////////////////////////////////////////
      ////////////////////////////////////////////////////////////////////////////////////////////////////
      // Public properties
      ////////////////////////////////////////////////////////////////////////////////////////////////////
      // Let's look for a public property
      if (TestField(JavaClass, FeatureName , PropertyTypeName) == true) { return Getter ? FeatureName  
                                                                                        : FeatureName +"="+EFeatureMap.__SETTER_PARAM_MARKER; }
      if (TestField(JavaClass, FeatureName2, PropertyTypeName) == true) { return Getter ? FeatureName2
                                                                                        : FeatureName2+"="+EFeatureMap.__SETTER_PARAM_MARKER; }
      if (TestField(JavaClass, FeatureName3, PropertyTypeName) == true) { return Getter ? FeatureName3 
                                                                                        : FeatureName3+"="+EFeatureMap.__SETTER_PARAM_MARKER; }

      ////////////////////////////////////////////////////////////////////////////////////////////////////
      ////////////////////////////////////////////////////////////////////////////////////////////////////
      // Nothing found. Bad. We return the marker.
      ////////////////////////////////////////////////////////////////////////////////////////////////////
      Streamer.warning.Header().println("Could not find an appropriate "+(Getter?"getter":"setter")+" in class '"+JavaClass.getName()
                                       +"' for property '"+FeatureName+"', '"+FeatureName2+"' or '"+FeatureName3+"' of type '"+PropertyTypeName+"'");
      return ACCESS_METHOD_NEEDED;
    }


   public static boolean TestMethod(Class C, String MethodName, String TypeName, boolean Getter)
    {
      try
       {
         Method[] Methods = C.getMethods();
         for (int i = 0; i < Methods.length; ++i)
          {
            Method M = Methods[i];
            if (M.getName().equals(MethodName) == true)  // Found a method with the right name.
             {
               int Mod = M.getModifiers();
               if (Getter == true)
                { 
                  return M.getParameterTypes().length == 0
                      && TestMethodOrField(M.getReturnType().getName(), TypeName, Mod);
                }
               else
                {
                  return M.getParameterTypes().length == 1
                      && TestMethodOrField(M.getParameterTypes()[0].getName(), TypeName, Mod);
                }
             }
          }
       }
      catch (ClassNotFoundException E)
       {
         Streamer.error.Header().printStackTrace(E);
       }
      return false;
    }

   public static boolean TestField(Class C, String FieldName, String FieldTypeName)
    {
      try
       { 
         Field F = C.getField(FieldName);
         int Mod = F.getModifiers();
         return TestMethodOrField(F.getType().getName(), FieldTypeName, Mod);
       }
      catch (Exception E)
       {
         return false;
       }
    }
    
   public static boolean TestMethodOrField(String FoundTypeName, String RequestedTypeName, int Mod)
   throws ClassNotFoundException
    {
         // If we are here, the property was mapped, and the real type was found, so if we do not get 
         // a match, we have to be lenient and check for Object or containers. The code later will
         // do the proper casting.
//      Streamer.debug.Header().println(FoundType.getName()+" --> "+RequestedType.getName());
      if (   FoundTypeName.equals(RequestedTypeName) == false 
          && FoundTypeName.equals(Object.class.getName()) == false
          && TypesUtil.isGeneralList(FoundTypeName) == false
         )
       {
         return false;
       }
      //Fenil Note: The isAbstract check is not needed because at runtime the method cannot be abstract. The impl class
      //have a non abstract method else the code will not compile. We can safely ignore the isAbstract check here because of this.
      /*return Modifier.isStatic  (Mod) == false 
          && Modifier.isPublic  (Mod) == true
          && Modifier.isAbstract(Mod) == false;*/
      return Modifier.isStatic  (Mod) == false 
      && Modifier.isPublic  (Mod) == true;
    }

   /**
    * 
    * @param fullEMapFilePath - Full File Path to the EMap file.  Example: "C:\\WSAD_512_040118\\workspace\\BF\\JavaSource\\com\\ibm\\odcb\\simple\\wdo4jsmediators\\Hello.emap"
    * @return the resource name of the EMap file relative to the project source folder.  Example: "com\\ibm\\odcb\\simple\\wdo4jsmediators\\Hello.emap"
    */
   public static String parseEMapResourceName(String fullEMapFilePath, String BaseSourcePath)
   {
     String emapResourceName = fullEMapFilePath;
     try
     {
      if (BaseSourcePath != null)
      {
         if (fullEMapFilePath.startsWith(BaseSourcePath))
         {
            emapResourceName = fullEMapFilePath.substring(BaseSourcePath.length());
            if (emapResourceName.startsWith("\\") || emapResourceName.startsWith("/"))
              emapResourceName = emapResourceName.substring(1);
            if (emapResourceName.indexOf('\\') > -1)
              emapResourceName = emapResourceName.replace('\\', '/');
         }
         
      }
     }
     catch (Exception e)
     {
      Streamer.error.Header().printStackTrace(e);
     }
     Streamer.debug.Header().println("ResourceURI of EMap file '" + fullEMapFilePath + "' is '" + emapResourceName + "'");
     return emapResourceName;
   }


   private static PrintWriter writeEClassMap(PrintWriter out, EClassMap em)
   {
     //out.println("    <EClassMap app_class=\""+em.getJavaComplexName()+"\" eclass_name=\""+em.getEClassName()+"\" export=\""+em.getExport()+"\">");
     out.println("    <EClassMap app_class=\""+em.getJavaComplexName()+"\" eclass_name=\""+em.getEClassName()+"\" export=\""+em.getExport()+"\" "+MappingsParser.ATTR_DIFF_REFRESH+"=\""+em.isDiffOnRefresh()+"\">");
     
     //iterate through child attributes and print the efeatures
     Iterator fit = em.getEFeatureMapIterator();
     while (fit.hasNext())
     {
        EFeatureMap f = (EFeatureMap) fit.next();
        if (f.isCalculateAttribute())
          writeClientFeatureMap(out, f);
        else
          writeEFeatureMap(out, f);
     }
     out.println("    </EClassMap>");
     return out;
   }


   public static PrintWriter writeEFeatureMap(PrintWriter out, EFeatureMap f)
   {
     String featureName = f.getEFeatureName();
     // If this is a reference, for a dynamic WDO, pointing to a class in the same package, then we assume the other class is also
     // an automatic WDO, and therefore, we can generate the extra information.
     if (f.getExtraTargetType() != null)
      {
       featureName = featureName+"("+f.getExtraTargetType()+")";
      }
       
     boolean Unsupported = TypesUtil.isUnsupportedType(f.getGetter());
     out.println((Unsupported == true ? "<!--  " : "      ")
                 +"<EFeatureMap efeature_name=\""+featureName+"\" iD=\""+f.isID()+"\" get=\""+f.getGetter()
                 +(f.getSetter() == null ? "" : ("\" set=\""+f.getSetter())) 
                 +(Unsupported == true ? "\" />   -->" : "\" />"));
     return out;
   }

   public static PrintWriter writeClientFeatureMap(PrintWriter out, EFeatureMap f)
   {
     String expression = StringUtil.QuoteDoubleAndEscapeForXML(f.getCalculateExpression(),false, true);
     out.println("      <CalculateAttribute attribute_name=\""+f.getEFeatureName()+"\" expression="+expression+" type=\""+f.getCalculatedType()+"\" iD=\"false\" />");
     return out;
   }

/*    
    
    public static void OutputEClassMap(PrintWriter Out, EClass C, ParameterSet P)
    {
      Out.println();
////////////////////////////////////////////////////
// LDH-NOTE: Changes here to handle dynamic WDOs
////////////////////////////////////////////////////
      boolean DynamicWDO = DynamicMediator.isComplexNameDynamic(P._FullJavaClassName);

      Out.println("    <EClassMap app_class=\""+P._FullJavaClassName+"\" eclass_name=\""+P._EClassName+"\" export=\""+C.getEPackage().getName()+"_"+P._EClassName+"\">");
      List L = C.getEAllStructuralFeatures();
      int MaxNameSize = 0;
      for (int i = 0; i < L.size(); ++i)
       {
         EStructuralFeature F = (EStructuralFeature) L.get(i);
         String RealETypeName = TypesUtil.getRealETypeName(F.getEType());
         if (RealETypeName.length() > MaxNameSize)
          MaxNameSize = RealETypeName.length();
       }
      for (int i = 0; i < L.size(); ++i)
       {
         EStructuralFeature F = (EStructuralFeature) L.get(i);
         // Only set iD to true if cardinality is 0 or 1.
         boolean iD = !F.isMany();
         boolean Reference = F instanceof EReference;
         String FeatureName = F.getName();
         String FeatureEPackage = F.getEType().getEPackage().getName();
         // If this is a reference, for a dynamic WDO, pointing to a class in the same package, then we assume the other class is also
         // an automatic WDO, and therefore, we can generate the extra information.
         if (Reference == true && DynamicWDO == true && FeatureEPackage.equals(C.getEPackage().getName()) == true)
          {
            FeatureName = FeatureName+"("+DynamicMediator.MakeExtraName(FeatureEPackage, TypesUtil.getRealETypeName(F.getEType()))+")";
          }
         String Pad = StringUtil.getPad(MaxNameSize-F.getName().length());
         String Getter = FindTheGetter(F, P._FullJavaClassName);
         Out.println("      <EFeatureMap efeature_name=\""+FeatureName+"\""+Pad+" iD=\""+iD+"\" get=\""+Getter+"\" />");
         Streamer.debug.Header().println("   Mapped EFeature '"+FeatureName+"' to getter '"+Getter+"', with iD='"+iD+"'.");
       }
      Out.println("    </EClassMap>");
    }
*/
 }
 
 