/////////////////////////////////////////////////////////////////////////
//IBM Confidential
//
//OCO Source Materials
//
//5724-E76 and 5724-E77
//
// Copyright IBM Corp.  2004
//
//The source code for this program is not published or other-
//wise divested of its trade secrets, irrespective of what has
//been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////

package com.ibm.odcb.jrender.utilities;

import java.io.IOException;
import java.io.Writer;

/**
 * @author ieu93149 Thomas Delahunty
 * 
 * ODCClientDataMarkupAggregator is used to aggregate content before it is
 * written by the ResponseWriter in the ClientDataRenderer. For data sets with a
 * large number of records using this aggregator solves the following problem.
 * 
 * The basic problem was that the writer used by WAS was making calls to
 * reAllocBuff a method of org.apache.jasper.runtime.BodyContentImpl. This was
 * supposed to allocate (buffer) memory for content being written however
 * although it was allocating enough memory for individual writes the rate at
 * which memory was being added (to the buffer) was not sufficient and so more
 * calls to reAllocBuff were required on future writes. Each call to reAllocBuff
 * consumed memory due to construction of temporary internal arrays. So 288
 * calls to reAllocBuff resulted in 40Mb of memory being consumed. This solution
 * was to write a ODCClientDataMarkupAggregator class the implements the writer
 * interface but in fact doesn't write at all. It does the job that
 * BodyContentImpl should be doing, it buffers the content of multiple writes.
 * When the writing is finished the content is extracted from the
 * ODCClientDataMarkupAggregator and a single write is done using the origional
 * (probelematic) writer from WAS. However because we are only doing one write
 * there will only be one call to reAllocBuff which only has to allocate enough
 * memory for this single write.
 *  
 */

public class ODCClientDataMarkupAggregator extends Writer {

	StringBuffer sb;

	public ODCClientDataMarkupAggregator() {
		sb = new StringBuffer();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.io.Writer#close()
	 */
	public void close() throws IOException {
		// We will not implement this method

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.io.Writer#flush()
	 */
	public void flush() throws IOException {
		// We will not implement this method

	}

	/**
	 * 
	 * @return the length of the string of characters represented by this
	 *         Aggregator
	 */
	public int length() {
		return sb.length();
	}

	/**
	 * Set the capacity of the aggregaor
	 * 
	 * @param x
	 *            the new Capacity of the Aggretator
	 *  
	 */
	public void setLength(int x) {
		sb.ensureCapacity(x);
	}

	public String toString() {
		return sb.toString();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.io.Writer#write(char[])
	 */
	public void write(char[] cbuf) throws IOException {
		// TODO Auto-generated method stub
		//super.write(cbuf);
		sb.append(cbuf);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.io.Writer#write(char[], int, int)
	 */
	public void write(char[] cbuf, int off, int len) throws IOException {
		sb.append(cbuf, off, len);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.io.Writer#write(int)
	 */
	public void write(int c) throws IOException {
		// TODO Auto-generated method stub
		//super.write(c);
		sb.append(c);
	}

	public void write(String s) {
		sb.append(s);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.io.Writer#write(java.lang.String, int, int)
	 */
	public void write(String str, int off, int len) throws IOException {
		// TODO Auto-generated method stub
		super.write(str, off, len);

	}
}
