/////////////////////////////////////////////////////////////////////////
//  IBM Confidential
//  
//  OCO Source Materials
//  
//  PID 5724-E76 and 5724-E77
//  
//   Copyright IBM Corp.  2003
//  
//  The source code for this program is not published or other-
//  wise divested of its trade secrets, irrespective of what has
//  been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////


package com.ibm.odcb.utilities;

import java.io.Writer;

/**
 * Contains the IBM Copyright information for WebSphere Odyssey.
 */
public class IBMCopyright
 {
 /**
  * <p>The full version of the WebSphere Odyssey Copyright statement.</p>
  * <pre>
  * Licensed Materials - Property of IBM
  * PID 5724-E76 and 5724-E77
  * (c) Copyright International Business Machines Corporation 2003.
  * All rights reserved
  * US Government Users Restricted Rights - Use, duplication or
  * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
  * </pre>
  */
  public static final String LONG_COPYRIGHT = "Licensed Materials - Property of IBM\n"
                                             +"PID 5724-E76 and 5724-E77\n"
                                             +"(c) Copyright International Business Machines Corporation 2003.\n"
                                             +"All Rights Reserved.\n"
                                             +"US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp."
                                             ;
 
 /**
  * <p>The short version of the WebSphere Odyssey Copyright statement, which happens to be the same as the full version.</p>
  * <pre>
  * Licensed Materials - Property of IBM
  * PID 5724-E76 and 5724-E77
  * (c) Copyright International Business Machines Corporation 2003.
  * All rights reserved
  * US Government Users Restricted Rights - Use, duplication or
  * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
  * </pre>
  */
 public static final String SHORT_COPYRIGHT = "Licensed Materials - Property of IBM\n"
                                             +"PID 5724-E76 and 5724-E77\n"
                                             +"(c) Copyright International Business Machines Corporation 2003.\n"
                                             +"All Rights Reserved.\n"
                                             +"US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp."
                                             ;

 /**
  * <p>The mini version of the WebSphere Odyssey copyright statement.</p>
  * <code>
  * (c) Copyright International Business Machines Corporation 2003
  * </code>
  */
 public static final String MINI_COPYRIGHT = "(c) Copyright International Business Machines Corporation 2003";
 
/**
 * Creates the IBM Copyright object.
 */
public IBMCopyright() {
 super();
}

/**
 * Returns the long version of the WebSphere Odyssey Copyright statement.
 * @return The full version of the WebSphere Odyssey Copyright statement.
 */
public static String getLongCopyright() {
 return LONG_COPYRIGHT;
}

/**
 * Returns the short version of the WebSphere Odyssey Copyright statement.
 * @return The short version of the WebSphere Odyssey Copyright statement.
 */
public static String getShortCopyright() {
 return SHORT_COPYRIGHT;
}

/**
 * Returns the mini version of the WebSphere Odyssey Copyright statement.
 * @return The mini  version of the WebSphere Odyssey Copyright statement.
 */
public static String getMiniCopyright() {
 return MINI_COPYRIGHT;
}


   public static void printLongCopyright_JAVA(Writer Out)
    {
     try
     {
      Out.write(" /************************************************************************\n");
      Out.write("  * Licensed Materials - Property of IBM\n");
      Out.write("  * (c) Copyright IBM Corp. 2003.  All rights reserved.\n");
      Out.write("  *\n");
      Out.write("  * US Government Users Restricted Rights - Use, duplication or\n");
      Out.write("  * disclosure restricted by GSA ADP Schedule Contract with IBM\n");
      Out.write("  * Corp.\n");
      Out.write("  *\n");
      Out.write("  * DISCLAIMER OF WARRANTIES.  The following [enclosed] code is\n");
      Out.write("  * sample code created by IBM Corporation.  This sample code is\n");
      Out.write("  * not part of any standard or IBM product and is provided to you\n");
      Out.write("  * solely for the purpose of assisting you in the development of\n");
      Out.write("  * your applications.  The code is provided \"AS IS\", without\n");
      Out.write("  * warranty of any kind.  IBM shall not be liable for any damages\n");
      Out.write("  * arising out of your use of the sample code, even if they have\n");
      Out.write("  * been advised of the possibility of such damages.\n");
      Out.write("  *************************************************************************/\n");
      Out.write("\n");
     }
     catch (java.io.IOException e)
     {
       e.printStackTrace();
     }
    }
   public static void printLongCopyright_XML(Writer Out)
    {
     try
     {
      Out.write("<!-- /************************************************************************\n");
      Out.write("      * Licensed Materials - Property of IBM\n");
      Out.write("      * (c) Copyright IBM Corp. 2003.  All rights reserved.\n");
      Out.write("      *\n");
      Out.write("      * US Government Users Restricted Rights - Use, duplication or\n");
      Out.write("      * disclosure restricted by GSA ADP Schedule Contract with IBM\n");
      Out.write("      * Corp.\n");
      Out.write("      *\n");
      Out.write("      * DISCLAIMER OF WARRANTIES.  The following [enclosed] code is\n");
      Out.write("      * sample code created by IBM Corporation.  This sample code is\n");
      Out.write("      * not part of any standard or IBM product and is provided to you\n");
      Out.write("      * solely for the purpose of assisting you in the development of\n");
      Out.write("      * your applications.  The code is provided \"AS IS\", without\n");
      Out.write("      * warranty of any kind.  IBM shall not be liable for any damages\n");
      Out.write("      * arising out of your use of the sample code, even if they have\n");
      Out.write("      * been advised of the possibility of such damages.\n");
      Out.write("      *************************************************************************/  -->\n");
      Out.write("\n");
     }
     catch (java.io.IOException e)
     {
       e.printStackTrace();
     }
    }
   public static void printLongCopyright_SQL(Writer Out)
    {
     try
     {
      Out.write("-- /************************************************************************\n");
      Out.write("--  * Licensed Materials - Property of IBM\n");
      Out.write("--  * (c) Copyright IBM Corp. 2003.  All rights reserved.\n");
      Out.write("--  *\n");
      Out.write("--  * US Government Users Restricted Rights - Use, duplication or\n");
      Out.write("--  * disclosure restricted by GSA ADP Schedule Contract with IBM\n");
      Out.write("--  * Corp.\n");
      Out.write("--  *\n");
      Out.write("--  * DISCLAIMER OF WARRANTIES.  The following [enclosed] code is\n");
      Out.write("--  * sample code created by IBM Corporation.  This sample code is\n");
      Out.write("--  * not part of any standard or IBM product and is provided to you\n");
      Out.write("--  * solely for the purpose of assisting you in the development of\n");
      Out.write("--  * your applications.  The code is provided \"AS IS\", without\n");
      Out.write("--  * warranty of any kind.  IBM shall not be liable for any damages\n");
      Out.write("--  * arising out of your use of the sample code, even if they have\n");
      Out.write("--  * been advised of the possibility of such damages.\n");
      Out.write("--  *************************************************************************/\n");
      Out.write("\n");
     }
     catch (java.io.IOException e)
     {
       e.printStackTrace();
     }
    }

}