// ///////////////////////////////////////////////////////////////////////
//  IBM Confidential
//  
//  OCO Source Materials
//  
//  PID xxxx-xxx
//  
//   Copyright IBM Corp. 2003
//  
//  The source code for this program is not published or other-
//  wise divested of its trade secrets, irrespective of what has
//  been deposited with the U.S. Copyright Office.
/////////////////////////////////////////////////////////////////////////

package com.ibm.odcb.utilities;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import com.ibm.odcb.utilities.jsdoc.JSDocGen;

/*
 * @author ldh
 * 
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates. To enable and disable the creation of type
 * comments go to Window>Preferences>Java>Code Generation.
 */
public class JSCompressor {
	public static class ByteCounter {
		protected int _TotalCharsIn = 0;

		protected int _TotalCharsOut = 0;

		protected int _FileCount = 0;

		public int _CurrentCharsIn;

		public int _CurrentCharsOut;

		public void StartFile(String Name) {
			System.out.println("STATUS: Compressing " + Name + ".");
			_CurrentCharsIn = _CurrentCharsOut = 0;
			++_FileCount;
		}

		public void EndFile(String Name) {
			System.out.println("STATUS:   --> "
					+ _CurrentCharsIn
					+ " to "
					+ _CurrentCharsOut
					+ " or "
					+ (Math.round(10000 - 10000.0 * _CurrentCharsOut
							/ _CurrentCharsIn) / 100.0) + "%.");
			_TotalCharsIn += _CurrentCharsIn;
			_TotalCharsOut += _CurrentCharsOut;
		}

		public void EndBatch(String Name) {
			System.out.println("STATUS: Compressed "
					+ _FileCount
					+ " files into "
					+ Name
					+ ", from "
					+ _TotalCharsIn
					+ " to "
					+ _TotalCharsOut
					+ " or "
					+ (Math.round(10000 - 10000.0 * _TotalCharsOut
							/ _TotalCharsIn) / 100.0) + "%.");
		}
	}

	private static boolean noCompression = false;

	
	// Looking for pattens like "\X" which is X escaped, but not "\\X", which is
	// '\' escaped followed by X.
	// Looking for an odd number of preceeding '\\' chars.
	public static boolean IsEscaped(String L, int i) {
		int Backslashes = 0;
		while (--i >= 0) {
			if (L.charAt(i) == '\\')
				++Backslashes;
			else
				break;
		}
		return Backslashes % 2 == 1;
	}

	public static boolean IsIn(char[] X, char c) {
		for (int i = 0; i < X.length; ++i)
			if (X[i] == c)
				return true;
		return false;
	}

	protected static char[] _INTERESTING_CHARS = { '=', '[', ']', ',', ';',
			'{', '}', '(', ')', '!', '&', '|', '+', '\'', '"', '?', '.', '<',
			'>', ':' };

	protected static final String[] REMOVABLE_CALLS = { "Log." };

	public static boolean IsRemovableCall(String L, int i) {
		for (int t = 0; t < REMOVABLE_CALLS.length; ++t) {
			if (L.indexOf(REMOVABLE_CALLS[t]) == i) {
				return true;
			}
		}
		return false;
	}

	public static void CleanFile(BufferedWriter Out, String Name, ByteCounter BC, ArrayList JSDComments) throws Exception {
		int CommentNesting = 0;
		int BlockNesting = 0;
		int CallNesting = 0;
		char StringDelim = ' ';
		int LineCount = 0;
		boolean RemovableCall = false;
		BufferedReader In = new BufferedReader(new FileReader(Name));
		String L;
		BC.StartFile(Name);
		char LastChar = ' ';
		//      boolean FunctionDeclare = false;

		StringBuffer JSDoc = new StringBuffer(512); // Pre-allocate a buffer to
													// hold comments.
		while ((L = In.readLine()) != null) {
			++LineCount;
			if (Name.equals("D:\\projects\\tutorial\\src\\html\\jsl\\jslUtil.js")
					&& LineCount > 250) {
				int i = 0;
				++i;
			}
			BC._CurrentCharsIn += L.length() + 1; // Plus one because of the
												  // newline char which is not
												  // kept in the returned
												  // String. We could have a
												  // \r\n, but ....
			boolean EmptyLine = true;
			if (noCompression == true) {
				Out.write(L);
				BC._CurrentCharsOut += L.length() + 1;
				EmptyLine = false;
			} else {
				if (JSDoc.length() != 0) // If there is a comment ongoing, just
				// append the whole line to it
				{
					JSDoc.append(L);
				}
				for (int i = 0; i < L.length(); ++i) {
					char c = L.charAt(i);
					if ((c == '\"' || c == '\'') && IsEscaped(L, i) == false
							&& CommentNesting == 0) // Are we encountering a
													// single
					// our double quote?
					{
						if (StringDelim == ' ') // Are we starting a String?
							StringDelim = c; // Save the String start marker,
											 // which
						// in JavaScript could be a ' or a "
						else if (c == StringDelim) // Are we closing a String?
							StringDelim = ' ';
					} else if (c == '*' && i < L.length() - 1
							&& L.charAt(i + 1) == '/') // We are encountering a
													   // "*/"
					{
						if (i >= 1 && L.charAt(i - 1) == '*') // We are
															  // encountering
						// a "**/" JSDoc
						// comment end
						{
							// Cut off the part of the line after the "**/" bit.
							// There is no guarantee this
							// is at the end of the line.
							JSDoc
									.setLength(JSDoc.length() - L.length() + i
											- 1);
							JSDComments.add(new JavaScriptDocComment(Name,
									BlockNesting, JSDoc.toString())); // Create
																	  // new
							// JavaScriptDoc
							// comment
							JSDoc.setLength(0); // Clear up the StringBuffer.
						}
						if (--CommentNesting < 0)
							System.err
									.println("ERROR: Comment nesting gets negative on line "
											+ LineCount + "for file " + Name);
						i += 2;
						continue;
					} else if (StringDelim == ' ' && CommentNesting == 0)
					// We are not in a string or a comment, so check for special
					// sequences
					{
						if (c == '/' && i < L.length() - 1
								&& L.charAt(i + 1) == '/') // We've encountered
														   // a "//"
							break; // Abort the loop and get the next line.
						if (c == '/' && i < L.length() - 1
								&& L.charAt(i + 1) == '*') // We've encountered
														   // a "/*"
						{
							if (i < L.length() - 2 && L.charAt(i + 2) == '*') // We've
																			  // encountered
																			  // a
																			  // "/**"
																			  // JSDoc
																			  // comment
							{
								// Let's append the rest of the line into the
								// JSDoc
								// buffer. Since this might be the end
								// of the line already, add an extra ' ' to make
								// length != 0, so the next loop around
								// the line copying logic will kick in
								JSDoc.append(" ").append(L.substring(i));
							}
							++CommentNesting;
							i += 2;
							continue;
						} else if (c == '{') // We are starting a block
						{
							//FunctionDeclare=false;
							++BlockNesting;
						} else if (c == '}') {
							if (--BlockNesting < 0)
								System.err
										.println("ERROR: block nesting gets negative on line "
												+ LineCount
												+ "for file "
												+ Name);
						} else if (c == '(') // We are starting a call
						{
							++CallNesting;
						} else if (c == ')') {
							if (--CallNesting < 0)
								System.err
										.println("ERROR: call nesting gets negative on line "
												+ LineCount
												+ "for file "
												+ Name);
						} else if ((c == ';' || c == '\n') && CallNesting == 0
								&& RemovableCall == true) {
							RemovableCall = false;
							continue;
						} else if (c == ';' && LastChar == ';'
								&& CallNesting == 0) {
							continue;
						}

						if (Character.isWhitespace(c) == true)
							c = ' ';
						if (c == ' ') {
							boolean MoreTest = true;
							if (i < L.length() - 2) // check for weird "+ ++Var"
													// or
							// "- --Var" patterns
							{
								char Next = L.charAt(i + 1);
								char NextNext = L.charAt(i + 2);
								if (LastChar == '+' && Next == '+'
										&& NextNext == '+' || LastChar == '-'
										&& Next == '-' && NextNext == '-')
									MoreTest = false;
							}
							if (MoreTest == true) {
								if (i < L.length() - 1) {
									char Next = L.charAt(i + 1);
									if (Character.isWhitespace(Next) == true
											|| IsIn(_INTERESTING_CHARS, Next)
											|| i == 0)
										continue;
								}
								if (i > 0 && EmptyLine == true) {
									char Prev = L.charAt(i - 1);
									if (Character.isWhitespace(Prev) == true
											|| IsIn(_INTERESTING_CHARS, Prev))
										continue;
								}
							}
						}
					}
					// Let's do patterns now if we are not in a comment, and the
					// char before us was really a delimiter
					if (CommentNesting == 0 && RemovableCall == false) {
						char Prev = i > 0 ? L.charAt(i - 1) : ' ';
						boolean Skip = StringDelim != ' ' || Prev == '.'
								|| Character.isJavaIdentifierPart(Prev) == true;
						if (Skip == false
								&& L.indexOf("document.images", i) == i) {
							Out.write("I");
							EmptyLine = false;
							BC._CurrentCharsOut += 1;
							i += 14;
							LastChar = 's';
						} else if (Skip == false
								&& L.indexOf("document.writeln", i) == i) {
							Out.write("WL");
							EmptyLine = false;
							BC._CurrentCharsOut += 2;
							i += 15;
							LastChar = 'n';
						} else if (Skip == false
								&& L.indexOf("document.write", i) == i) {
							Out.write("W");
							EmptyLine = false;
							BC._CurrentCharsOut += 1;
							i += 13;
							LastChar = 'e';
						} else if (Skip == false && CallNesting == 0
								&& IsRemovableCall(L, i) == true) {
							RemovableCall = true;
							continue;
						}
						/*
						 * else if (StringDelim == ' ' && L.indexOf("document.",
						 * i) == i) { Out.write("D."); BC._CurrentCharsOut+=9;
						 * i+=8; LastChar='.'; }
						 */
						else {
							Out.write(c);
							LastChar = c;
							++BC._CurrentCharsOut;
							EmptyLine = false;
						}

						//              if (Skip == false && L.indexOf("function", i) == i)
						//               {
						//                 FunctionDeclare = true;
						//               }

					}
				}
			}
			if (EmptyLine == false) {
				//           if (LastChar=='}')
				if (LastChar != ';' && LastChar != '{') {
					Out.write('\n');
					LastChar = '\n';
					++BC._CurrentCharsOut;
				}
				//           else if (LastChar!='{' && LastChar!=';' && FunctionDeclare ==
				// false) // Damn JavaScript with optional ';' sequence points.
				//            {
				//              Out.write(';');
				//              LastChar=';';
				//              ++BC._CurrentCharsOut;
				//            }
			}
			if (StringDelim != ' ') {
				System.err.println("ERROR: Unterminated String on line " + LineCount + "for file " + Name);
				StringDelim = ' ';
			}
			++BC._CurrentCharsOut;
		}
		if (BlockNesting != 0)
			System.err.println("ERROR: " + BlockNesting
					+ " unterminated block(s) at the end of file " + Name);
		if (CommentNesting != 0)
			System.err.println("ERROR: " + CommentNesting
					+ " unterminated comment(s) at the end of file " + Name);
		BC.EndFile(Name);
	}

	public static void main(String[] args) {
		List L = ProcessCommandLine(args);
		if (L == null)
			return;

		try {
			ArrayList JSDComments = new ArrayList(57);
			ByteCounter BC = new ByteCounter();
			String OutputLibName = (String) L.get(0);
			BufferedWriter Out = new BufferedWriter(new FileWriter(OutputLibName));
			Out.write(/* "var D=document; */"var I=document.images;\nfunction W(s){document.write(s)}\nfunction WL(s){document.writeln(s)}\n");
			for (int i = 1; i < L.size(); ++i) {
				CleanFile(Out, (String) L.get(i), BC, JSDComments);
			}
			Out.close();
			System.out.println();
			System.out.println();
			BC.EndBatch(args[0]);

			System.out.println();
			System.out.println();
			System.out.println("STATUS: Generating the JavaScript Docs");

			int LastDot = OutputLibName.lastIndexOf('.');
			if (LastDot != -1)
				OutputLibName = OutputLibName.substring(0, LastDot);
			GenJavaScriptDocs(JSDComments, OutputLibName);
			System.out.println();
			System.out.println();
			System.out.println("STATUS: Finished");
		} catch (Exception e) {
			e.printStackTrace(System.err);
		}
	}

	/*
	 * Sample inputs D:\projects\Tutorial\src\html\jsl\jsl.lib +r D:\projects\tutorial\src\html\jsl js
	 */
	public static List ProcessCommandLine(String[] args) {
		System.out.println("Utility to be called with a choice of 3 possible command line patterns: ");
		System.out.println("  [-c] <source file>");
		System.out.println("     Passing only a single source file will ouput the result in <source file>.lib");
		System.out.println("  [-c] <destination file> <source file>+");
		System.out.println("     Will concatenate the result of compressing all the source files, into the destination file");
		System.out.println("  [-c] <destination file> (+r|-r) <directory_path> <extension>");
		System.out.println("     Will concatenate the result of compressing all the source files found in the named directory ");
		System.out.println("     with the extension specified (c:\\abc\\*.js), into the destination file. If -r is specified, will");
		System.out.println("     only look in that directory. if +r is supplied, will resurce down children directories. *** Make sure the");
		System.out.println("     directory path doesn't with a '/' or '\\' ***");
		System.out.println("  if the first parameter -c is supplied, the files are passed through uncompressed.");
		if (args.length == 0) {
			System.err.println("ERROR: Invalid number of parameters passed for this utility");
			return null;
		}

		// See if the files are copied straight through
		if (args.length > 0)
			noCompression = args[0].equals("-c");

		int Ix = (noCompression == true) ? 1 : 0;
		
		// Only one parameter with optional compression flag
		if (args.length == 1 || (args.length == 2 && noCompression == true)) {
			ArrayList L = new ArrayList(2);
			int i = args[Ix].lastIndexOf('.');
			L.add(i == -1 ? args[Ix] + ".lib" : args[Ix].substring(0, i) + ".lib");
			L.add(args[Ix]);
			return L;
		}

		// Only list of files
		if (args[1].equals("-r") == false && args[1].equals("+r") == false && noCompression == false) {
			return Arrays.asList(args);
		}
		
		// List of files with compression flag
		if (args[2].equals("-r") == false && args[2].equals("+r") == false && noCompression == true) {
			ArrayList L = new ArrayList(args.length - 1);
			for (Ix=1; Ix<args.length; Ix++) {
				L.add(args[Ix]);
			}
			return L;
		}
		
		if ((args.length != 4 && noCompression == false) || (args.length != 5 && noCompression == true)) {
			System.err.println("ERROR: Invalid or unrecognized parameters passed for this utility");
			return null;
		}

		// expects something like "d:\toto\blah" or "d:\toto\blah\" or
		// "d:\toto\blah\*.ext"
		ArrayList L = new ArrayList(57);
		L.add(args[Ix]);
		File D = new File(args[Ix+2]);
		ListFiles(L, D, "." + args[Ix+3], args[Ix+1].equals("+r"));
		return L;
	}

	public static void ListFiles(ArrayList L, File Dir, String Ext, boolean Recurse) {
		// Something smarter could be done here... Right now, the entire content
		// of all directories in all complete branches
		// are all in memory... On a deep and populated tree, this could be an
		// issue. In most cases though, we don't expect
		// to look at thousands of files, so this should be OK for now.
		File[] Files = Dir.listFiles();
		for (int i = 0; i < Files.length; ++i) {
			File F = Files[i];
			// this has to belike that because we could get a directory with a name like
			// xyz.ext, and we don't want those to be
			// confused with a file that matches the extension.
			if (F.isDirectory() == true && F.isHidden() == false) {
				if (Recurse == true) {
					ListFiles(L, F, Ext, Recurse);
				}
			} else if (F.getPath().endsWith(Ext) == true
					&& F.isHidden() == false) {
				L.add(F.getPath());
			}
			Files[i] = null;
		}
	}

	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// Parsing JSDoc comments and generating HTML files woth docs in them.

	//Class comment
	//    /**
	//      * @class (private|public) <class name> \n <class description> \n
	//      * <comment> \n
	//     **/
	//
	//Method comment
	//    /**
	//      * @method (private|public) <method name> \n <method description> \n
	//      * <comment> \n
	//      * ( @param <Type Name, i.e. class name> <param name> \n <param
	// description> \n)*
	//      * @return <TypeName> <return description> \n
	//      * ( @exception <TypeName> <exception description> \n )*
	//     **/
	//
	// <comment> : multi line comment with HTML tags if necessarry (for bold,
	// italic etc...)
	// <class description> : <comment>
	// <param description>: <comment>
	// <return description>: <comment>
	// <exception description> : <comment>

	public static class JavaScriptDocComment {
		public JavaScriptDocComment(String FileName, int NestingLevel,
				String Comment) {
			_FileName = FileName;
			_NestingLevel = NestingLevel;
			_Comment = Comment;
		}

		public String _FileName;

		public int _NestingLevel;

		public String _Comment;
	}

	protected static void GenJavaScriptDocs(ArrayList Comments,
			String OutputFileName) throws Exception {

		JSDocGen jsdgen = new JSDocGen(Comments, OutputFileName);
		jsdgen.execute();
	}
}